#ifdef USE_PRAGMA_IDENT_HDR
#pragma ident "@(#)osThread_solaris.hpp	1.48 03/04/02 16:41:58 JVM"
#endif
/*
 * Copyright 2003 Sun Microsystems, Inc.  All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL.  Use is subject to license terms.
 */

// This is embedded via include into the class OSThread

 private:

  thread_t _thread_id;      // Solaris thread id
  unsigned int  _lwp_id;    // lwp ID, only used with bound threads

 public:

  thread_t thread_id() const      { return _thread_id; }

  unsigned int lwp_id() const     { return _lwp_id; }
#ifndef PRODUCT
  // Used for debugging, return a unique integer for each thread.
  int thread_identifier() const   { return _thread_id; }
#endif
#ifdef ASSERT
  // On solaris reposition can fail in two ways:
  // 1: a mismatched pc, because signal is delivered too late, target thread
  //    is resumed.
  // 2: on a timeout where signal is lost, target thread is resumed.
  bool valid_reposition_failure() {
    // only 1 and 2 can happen and we can handle both of them
    return true;
  }
#endif
  void set_thread_id(thread_t id) { _thread_id = id;   }
  void set_lwp_id(unsigned int id){ _lwp_id = id;   }

 // ***************************************************************
 // interrupt support.  interrupts (using signals) are used to get
 // the thread context (get_thread_pc), to set the thread context
 // (set_thread_pc), and to implement java.lang.Thread.interrupt.
 // ***************************************************************

 public:

  class InterruptArguments : StackObj {
   private:
    Thread*     _thread;   // the thread to signal was dispatched to
    ucontext_t* _ucontext; // the machine context at the time of the signal

   public:
    InterruptArguments(Thread* thread, ucontext_t* ucontext) {
      _thread   = thread;
      _ucontext = ucontext;
    }

    Thread*     thread()   const { return _thread;   }
    ucontext_t* ucontext() const { return _ucontext; }
  };

  // There are currently no asynchronous callbacks - and we'd better not 
  // support them in the future either, as they need to be deallocated from
  // the interrupt handler, which is not safe; they also require locks to 
  // protect the callback queue.
  
  class Sync_Interrupt_Callback : private StackObj {
   protected:
    volatile bool _is_done;
    Monitor*      _sync;
    Thread*       _target;
   public:
    Sync_Interrupt_Callback(Monitor * sync) { 
      _is_done = false;  _target = NULL;  _sync = sync;
    }

    bool is_done() const               { return _is_done; }
    Thread* target() const             { return _target;  }

    int interrupt(Thread * target, bool target_is_suspended, int timeout);

    // override to implement the callback. 
    virtual void execute(InterruptArguments *args) = 0;

    void leave_callback();
  };

 private:

  Sync_Interrupt_Callback * volatile _current_callback;
  enum {
    callback_in_progress = 1
  };
  Mutex * _current_callback_lock;       // only used on v8

 public:

  int set_interrupt_callback    (Sync_Interrupt_Callback * cb);
  void remove_interrupt_callback(Sync_Interrupt_Callback * cb);
  void OSThread::do_interrupt_callbacks_at_interrupt(InterruptArguments *args);

 // ***************************************************************
 // java.lang.Thread.interrupt state.
 // ***************************************************************

 private:

  os::Solaris::Event*  _interrupt_event;
  JavaThreadState      _saved_interrupt_thread_state;       // the thread state before a system call -- restored afterward
  JavaFrameAnchor      _saved_interrupt_anchor;             // the frame anchor state before a system call -- restored afterward

 public:

  os::Solaris::Event* interrupt_event() const                                   { return _interrupt_event; }
  void set_interrupt_event(os::Solaris::Event* ptr)                             { _interrupt_event = ptr;  }

  JavaThreadState   saved_interrupt_thread_state()                              { return _saved_interrupt_thread_state; }
  void              set_saved_interrupt_thread_state(JavaThreadState state)     { _saved_interrupt_thread_state = state; }
  JavaFrameAnchor*  saved_interrupt_anchor()                                    { return &_saved_interrupt_anchor; }

  static void       handle_spinlock_contention(int tries);                      // Used for thread local eden locking


  // ***************************************************************
  // Platform dependent initialization and cleanup
  // ***************************************************************

private:

  void pd_initialize();
  void pd_destroy();
