#ifdef USE_PRAGMA_IDENT_SRC
#pragma ident "@(#)stackValueCollection.cpp	1.9 03/01/23 12:25:21 JVM"
#endif
/*
 * Copyright 2003 Sun Microsystems, Inc.  All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL.  Use is subject to license terms.
 */

# include "incls/_precompiled.incl"
# include "incls/_stackValueCollection.cpp.incl"

jint StackValueCollection::int_at(int slot) const {
  intptr_t val =  at(slot)->get_int();
  jint ival = *((jint*) (&val));
  return ival;
}

jlong StackValueCollection::long_at(int slot) const {
#ifdef _LP64
  return at(slot+1)->get_int();
#else
  union {
    jlong jl;
    jint  array[2]; 
  } value;
  // Interpreter stack is reversed in memory:
  // low memory location is in higher java local slot.
  value.array[0] = at(slot+1)->get_int();
  value.array[1] = at(slot  )->get_int();
  return value.jl;
#endif
}

Handle StackValueCollection::obj_at(int slot) const {
  return at(slot)->get_obj();
}

jfloat StackValueCollection::float_at(int slot) const {
  intptr_t res = at(slot)->get_int();
  return *((jfloat*) (&res));
}

jdouble StackValueCollection::double_at(int slot) const {
#ifdef _LP64
  intptr_t res = at(slot+1)->get_int();
  return *((jdouble*) (&res));
#else
  union {
    jdouble jd;
    jint    array[2]; 
  } value;
  // Interpreter stack is reversed in memory:
  // low memory location is in higher java local slot.
  value.array[0] = at(slot+1)->get_int();
  value.array[1] = at(slot  )->get_int();
  return value.jd;
#endif
}

void StackValueCollection::set_int_at(int slot, jint value) {
  intptr_t val;
  *((jint*) (&val)) = value;
  at(slot)->set_int(val);
}

void StackValueCollection::set_long_at(int slot, jlong value) {
#ifdef _LP64
  at(slot+1)->set_int(value);
#else
  union {
    jlong jl;
    jint  array[2]; 
  } x;
  // Interpreter stack is reversed in memory:
  // low memory location is in higher java local slot.
  x.jl = value;
  at(slot+1)->set_int(x.array[0]);
  at(slot+0)->set_int(x.array[1]);
#endif
}

void StackValueCollection::set_obj_at(int slot, Handle value) {
  at(slot)->set_obj(value);
}

void StackValueCollection::set_float_at(int slot, jfloat value) {
  at(slot)->set_int(*(intptr_t*)(&value));
}

void StackValueCollection::set_double_at(int slot, jdouble value) {
#ifdef _LP64
  at(slot+1)->set_int(*(intptr_t*)(&value));
#else
  union {
    jdouble jd;
    jint    array[2]; 
  } x;
  // Interpreter stack is reversed in memory:
  // low memory location is in higher java local slot.
  x.jd = value;
  at(slot+1)->set_int(x.array[0]);
  at(slot+0)->set_int(x.array[1]);
#endif
}

#ifndef PRODUCT
void StackValueCollection::print() {
  for(int index = 0; index < size(); index++) {
    tty->print("\t  %2d ", index);
    at(index)->print_on(tty);
    if( at(index  )->type() == T_INT &&
        index+1 < size() &&
        at(index+1)->type() == T_INT ) {
      tty->print("  %lld (long)", long_at(index));
    }
    tty->cr();
  }
}
#endif
