/*
 * @(#)eventHandler.h	1.36 03/01/23
 *
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#ifndef _EVENT_HANDLER_H
#define _EVENT_HANDLER_H

#include <jni.h>
#include <jvmdi.h>
#include "bag.h"

typedef jint HandlerID;

/* structure is read-only for users */
typedef struct HandlerNode_ {    
    HandlerID handlerID;
    jbyte kind;
    jbyte suspendPolicy;
    /* private members follow */
} HandlerNode;

typedef void (*HandlerFunction)(JNIEnv *env,
                                JVMDI_Event *event, 
                                HandlerNode *node, 
                                struct bag *eventBag);

/***** HandlerNode create = alloc + install *****/

HandlerNode *eventHandler_alloc(jint filterCount, jbyte kind, 
                                jbyte suspendPolicy);
jint eventHandler_installExternal(HandlerNode *node);
HandlerNode *eventHandler_createInternal(jbyte kind, HandlerFunction func);
HandlerNode *eventHandler_createInternalThreadOnly(jbyte kind, 
                                                   HandlerFunction func,
                                                   jthread thread);
HandlerNode *eventHandler_createInternalBreakpoint(HandlerFunction func,
                                                   jthread thread,
                                                   jclass clazz, 
                                                   jmethodID method, 
                                                   jlocation location);

/***** HandlerNode free *****/

jint eventHandler_freeAll(jint kind);
jint eventHandler_freeByID(jint kind, HandlerID handlerID);
jint eventHandler_free(HandlerNode *node);
void eventHandler_freeClassBreakpoints(jclass clazz);

/***** HandlerNode manipulation *****/

void eventHandler_initialize(jbyte sessionID);
void eventHandler_reset(jbyte sessionID);

void eventHandler_lock(void);
void eventHandler_unlock(void);

/***** event redirection *****/
typedef jboolean (*EventRedirectHook)(JVMDI_Event *event, 
                                      jthread thread);
extern jthread redirectedEventThread;
extern EventRedirectHook redirectedEventFunction;

#endif /* _EVENT_HANDLER_H */
