/*
 * @(#)MainDoclet.java	1.8 03/01/23
 *
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


package com.sun.tools.javah;

import com.sun.javadoc.*;
import java.io.*;

/**
 * A doclet to parse and execute commandline options.
 *
 * @author Sucheta Dambalkar(using code from old javap)
 */
public class MainDoclet{
    
    public static  String  odir        = null;
    public static  String  ofile       = null;
    public static  boolean stubs       = false;
    public static  boolean jni         = false;
    public static  boolean old         = false;
    public static  boolean llni        = false;
    public static  boolean doubleAlign = false;
    public static  boolean force       = false;
    public static  String  genclass    = null;
    
    /**
     * Entry point.
     */
    public static boolean start(RootDoc root) {
	
	int j = 0;
	int k = 0;
	/**
	 * Command line options.
	 */
	String [][] cmdoptions = root.options();
	/**
	 * Classes specified on command line.
	 */
	ClassDoc[] classes = root.classes();
	/**
	 * Generator used by javah.  Default is JNI.
	 */
	Gen g = new JNI(root);
	
	validateOptions(cmdoptions);
	
	/*
	 * Select native interface.
	 */
	if (old && jni)   Util.error("old.jni.mixed");
	if (old && llni)  Util.error("old.llni.mixed");
	if (jni && llni)  Util.error("jni.llni.mixed");
	
	if (old && stubs) {
	    g = new OldStubs(root);
	}
	else if (old) {
	    g = new OldHeaders(root);
	}
	
	if (llni)
	    g = new LLNI(doubleAlign, root);
	
	if (g instanceof JNI && stubs)  Util.error("jni.no.stubs");
	
	/*
	 * Arrange for output destination.
	 */
	if (odir != null && ofile != null)
	    Util.error("dir.file.mixed");
	
	if (odir != null)
	    g.setOutDir(odir);
	
	if (ofile != null)
	    g.setOutFile(ofile);
	
	/*
	 * Force set to false will turn off smarts about checking file
	 * content before writing.
	 */
	g.setForce(force);
	
	/* 
	 * Grab the rest of argv[] ... this must be the classes.
	 */
	if (classes.length == 0){
	    Util.error("no.classes.specified");
	}
	/*
	 * Set classes.
	 */
	g.setClasses(classes);
	
	try {
	    g.run();
	} catch (ClassNotFoundException cnfe) {
	    Util.error("class.not.found", cnfe.getMessage());
	} catch (IOException ioe) {
	    Util.error("io.exception", ioe.getMessage());
	}
	
	return true;
    }
    
    /**
     * Required doclet method.
     */
    public static int optionLength(String option) {
	if (option.equals("-o")) {
	    return 2;
	} else if(option.equals("-d")){
	    return 2;
	} else if (option.equals("-td")) {
	    return 1;
	} else if (option.equals("-stubs")) {
	    return 1; 
	} else if(option.equals("-help")){
	    return 1;
	} else if(option.equals("--help")){
	    return 1;
	} else if(option.equals("-?")){
	    return 1;
	} else if(option.equals("-h")){
	    return 1;
	} else if(option.equals("-trace")){
	    return 1;
	} else if(option.equals("-version")) {
	    return 1;
	} else if(option.equals("-jni")){
	    return 1;
	} else if(option.equals("-force")){
	    return 1;
	} else if(option.equals("-old")){
	    return 1;
	} else if(option.equals("-Xllni")){
	    return 1;
	} else if(option.equals("-llni")){
	    return 1;
	} else if(option.equals("-llniDouble")){
	    return 1;
	} else return 0;
    }
    
    /**
     * Parse the command line options.
     */
    public static void validateOptions(String cmdoptions[][]) { 
	/* Default values for options, overridden by user options. */
	for(int p = 0; p < cmdoptions.length; p++){
	    
	    if (cmdoptions[p][0].equals("-o")) {
		ofile = cmdoptions[p][1];
	    } else if(cmdoptions[p][0].equals("-d")){
		odir = cmdoptions[p][1];
	    } else if (cmdoptions[p][0].equals("-td")) {
		if (p ==cmdoptions.length)
		    Util.usage(1);
	    } else if (cmdoptions[p][0].equals("-stubs")) {
		stubs = true;
	    } else if ((cmdoptions[p][0].equals("-v")) 
		       || (cmdoptions[p][0].equals("-verbose"))) {
	    } else if((cmdoptions[p][0].equals("-help")) 
		      || (cmdoptions[p][0].equals("--help")) 
		      || (cmdoptions[p][0].equals("-?")) 
		      || (cmdoptions[p][0].equals("-h"))) {
		Util.usage(0);
	    } else if (cmdoptions[p][0].equals("-trace")) {
		System.err.println(Util.getText("tracing.not.supported"));
	    } else if (cmdoptions[p][0].equals("-version")) {
		Util.version();
	    } else if (cmdoptions[p][0].equals("-jni")) {
		jni = true;
	    } else if (cmdoptions[p][0].equals("-force")) {
		force = true;
	    } else if (cmdoptions[p][0].equals("-old")) {
		old = true;
	    } else if (cmdoptions[p][0].equals("-Xllni")) {
		llni = true;
	    } else if (cmdoptions[p][0].equals("-llni")) {
		llni = true;
	    } else if (cmdoptions[p][0].equals("-llniDouble")) {
		llni = true; doubleAlign = true;
	    } else if (cmdoptions[p][0].equals("-classpath")) {
	    } else if (cmdoptions[p][0].equals("-bootclasspath")) {
	    } else if((cmdoptions[p][0].charAt(0) == '-')
		      && (!cmdoptions[p][0].equals("-private"))){
		Util.error("unknown.option", cmdoptions[p][0], null, true);
	    } else {
		break; /* The rest must be classes. */
	    }
	}
    }
}


  
  

    
  
 
  



    
   
