/*
 * @(#)DateFormatZoneData_sv.java	1.33 03/05/09
 */

/*
 * Portions Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * (C) Copyright Taligent, Inc. 1996, 1997 - All Rights Reserved
 * (C) Copyright IBM Corp. 1996 - 1998 - All Rights Reserved
 *
 * The original version of this source code and documentation
 * is copyrighted and owned by Taligent, Inc., a wholly-owned
 * subsidiary of IBM. These materials are provided under terms
 * of a License Agreement between Taligent and Sun. This technology
 * is protected by multiple US and International patents.
 *
 * This notice and attribution to Taligent may not be removed.
 * Taligent is a registered trademark of Taligent, Inc.
 *
 */

package sun.text.resources;

/**
 * Supplement package private date-time formatting zone data for DateFormat.
 * DateFormatData used in DateFormat will be initialized by loading the data
 * from LocaleElements and DateFormatZoneData resources. The zone data are
 * represented with the following form:
 * {ID, new String[] {ID, long zone string, short zone string, long daylight
 * string, short daylight string, representative city of zone}}, where ID is
 * NOT localized, but is used to look up the localized timezone data
 * internally. Localizers can localize any zone strings except
 * for the ID of the timezone.
 * Also, localizer should not touch "localPatternChars" entry.
 *
 * @see          Format
 * @see          DateFormatData
 * @see          LocaleElements
 * @see          SimpleDateFormat
 * @see          TimeZone
 * @version      1.23 08/05/01
 * @author       Chen-Lieh Huang
 */
//  Swedish DateFormatZoneData
//
public final class DateFormatZoneData_sv extends DateFormatZoneData
{
    /**
     * Overrides DateFormatZoneData
     */
    public Object[][] getContents() {
	String ACT[] = new String[] {"Acre-tid", "ACT",
				     "Acre-sommartid", "ACST"};
	String AGT[] = new String[] {"Argentinsk tid", "ART",
				     "Argentinsk sommartid", "ARST"};
	String AKST[] = new String[] {"Alaska-normaltid", "AKST",
				      "Alaska-dagtid", "AKDT"};
	String AMT[] = new String[] {"Amazonas-normaltid", "AMT",
				     "Amazonas-sommartid", "AMST"};
	String ARAST[] = new String[] {"Arabisk normaltid", "AST",
				       "Arabisk sommartid", "ADT"};
	String ARMT[] = new String[] {"Armenisk tid", "AMT",
				      "Armenisk sommartid", "AMST"};
	String AST[] = new String[] {"Atlantisk normaltid", "AST",
				     "Atlantisk sommartid", "ADT"};
	String BDT[] = new String[] {"Bangladeshnormaltid", "BDT",
				     "Bangladesh-sommartid", "BDST"};
	String BRT[] = new String[] {"Brasiliansk tid", "BRT",
				     "Brasiliansk sommartid", "BRST"};
	String BTT[] = new String[] {"Bhutansk tid", "BTT",
				     "Bhutansk sommartid", "BTST"};
	String CAT[] = new String[] {"Centralafrikansk tid", "CAT",
				     "Centralafrikansk sommartid", "CAST"};
	String CET[] = new String[] {"Centraleuropeisk tid", "CET",
				     "Centraleuropeisk sommartid", "CEST"};
	String CIT[] = new String[] {"Centralindonesisk Tid", "CIT",
				     "Centralindonesisk sommartid", "CIST"};
	String CLT[] = new String[] {"Chilensk tid", "CLT",
				     "Chilensk sommartid", "CLST"};
	String CST[] = new String[] {"Central Standard Time", "CST",
				     "Central Daylight Time", "CDT"};
	String CTT[] = new String[] {"Kinesisk normaltid", "CST",
				     "Kinesisk sommartid", "CDT"};
	String EAT[] = new String[] {"\u00d6stafrikansk tid", "EAT",
				     "\u00d6stafrikansk sommartid", "EAST"};
	String EET[] = new String[] {"\u00d6steuropeisk tid", "EET",
				     "\u00d6steuropeisk sommartid", "EEST"};
	String EGT[] = new String[] {"Ostgr\u00f6nl\u00e4ndsk tid", "EGT",
				     "Ostgr\u00f6nl\u00e4ndsk sommartid", "EGST"};
	String EST[] = new String[] {"Eastern Standard Time", "EST",
				     "Eastern Daylight Time", "EDT"};
	String EST_NSW[] = new String[] {"Eastern Standard Time (New South Wales)", "EST",
					 "Eastern Summer Time (New South Wales)", "EST"};
	String GMT[] = new String[] {"Greenwich Mean Time", "GMT",
				   "Greenwich Mean Time", "GMT"};
	String GST[] = new String[] {"Gulf Standard Time", "GST",
				     "Gulf Daylight Time", "GDT"};
	String HAST[] = new String[] {"Hawaii-Aleutisk normaltid", "HAST",
				      "Hawaii-Aleutisk sommartid", "HADT"};
	String HST[] = new String[] {"Hawaii-normaltid", "HST",
				     "Hawaii-sommartid", "HDT"};
	String ICT[] = new String[] {"Indokinesisk tid", "ICT",
				     "Indokinesisk sommartid", "ICST"};
	String IRT[] = new String[] {"Iransk normaltid", "IRST",
				     "Iransk sommartid", "IRDT"};
	String IST[] = new String[] {"Indisk normaltid", "IST",
				     "Indisk sommartid", "IDT"};
	String JST[] = new String[] {"Japansk normaltid", "JST",
				     "Japansk sommartid", "JDT"};
	String KST[] = new String[] {"Koreansk normaltid", "KST",
				     "Koreansk sommartid", "KDT"};
	String MST[] = new String[] {"Mountain Standard Time", "MST",
				     "Mountain Daylight Time", "MDT"};
	String NST[] = new String[] {"Newfoundland-normaltid", "NST",
				     "Newfoundland-sommartid", "NDT"};
	String NZST[] = new String[] {"Nyzeel\u00e4ndsk normaltid", "NZST",
				      "Nyzeel\u00e4ndsk sommartid", "NZDT"};
	String PKT[] = new String[] {"Pakistansk tid", "PKT",
				     "Pakistansk sommartid", "PKST"};
	String PST[] = new String[] {"Pacific Standard Time", "PST",
				     "Pacific Daylight Time", "PDT"};
	String SAST[] = new String[] {"Sydafrikansk normaltid", "SAST",
				      "Sydafrikansk sommartid", "SAST"};
	String SBT[] = new String[] {"Salomon\u00f6isk tid", "SBT",
				     "Salomon\u00f6isk sommartid", "SBST"};
	String TMT[] = new String[] {"Turkmenistansk tid", "TMT",
				     "Turkmenistansk sommartid", "TMST"};
	String ULAT[]= new String[] {"Ulaanbaatar-tid", "ULAT",
				     "Ulaanbaatar-sommartid", "ULAST"};
	String WAT[] = new String[] {"V\u00e4stafrikansk tid", "WAT",
				     "V\u00e4stafrikansk sommartid", "WAST"};
	String WET[] = new String[] {"V\u00e4steuropeisk tid", "WET",
				     "V\u00e4steuropeisk sommartid", "WEST"};
	String WST_AUS[] = new String[] {"Western Standard Time (Australien)", "WST",
					 "Western Summer Time (Australien)", "WST"};
	String WST_SAMOA[] = new String[] {"V\u00e4stsamoansk tid", "WST",
					   "V\u00e4stsamoansk sommartid", "WSST"};
	String ChST[] = new String[] {"Chamorro Standard Time", "ChST",
	                      "Chamorro Daylight Time", "ChDT"};

	return new Object[][] {
	    {"PST", PST},
	    {"America/Los_Angeles", PST},
	    {"MST", MST},
	    {"America/Denver", MST},
	    {"PNT", MST},
	    {"America/Phoenix", MST},
	    {"CST", CST},
	    {"America/Chicago", CST},
	    {"EST", EST},
	    {"America/New_York", EST},
	    {"IET", EST},
	    {"America/Indianapolis", EST},
	    {"HST", HST},
	    {"Pacific/Honolulu", HST},
	    {"AST", AKST},
	    {"America/Anchorage", AKST},
	    {"America/Halifax", AST},
	    {"CNT", NST},
	    {"America/St_Johns", NST},
	    {"ECT", CET},
	    {"Europe/Paris", CET},
	    {"GMT", GMT},
	    {"Africa/Casablanca", WET},
	    {"Asia/Jerusalem", new String[] {"Israelisk normaltid", "IST",
					     "Israelisk sommartid", "IDT"}},
	    {"JST", JST},
	    {"Asia/Tokyo", JST},
	    {"Europe/Bucharest", EET},
	    {"CTT", CTT},
	    {"Asia/Shanghai", CTT},
	    /* Don't change the order of the above zones
	     * to keep compatibility with the previous version.
	     */

	    {"ACT", new String[] {"Central Standard Time (Northern Territory)", "CST",
				  "Central Daylight Time (Northern Territory)", "CDT"}},
	    {"AET", EST_NSW},
	    {"AGT", AGT},
	    {"ART", EET},
	    {"Africa/Abidjan", GMT},
	    {"Africa/Accra", GMT},
	    {"Africa/Addis_Ababa", EAT},
	    {"Africa/Algiers", CET},
	    {"Africa/Asmera", EAT},
	    {"Africa/Bangui", WAT},
	    {"Africa/Banjul", GMT},
	    {"Africa/Bissau", GMT},
	    {"Africa/Blantyre", CAT},
	    {"Africa/Bujumbura", CAT},
	    {"Africa/Cairo", EET},
	    {"Africa/Conakry", GMT},
	    {"Africa/Dakar", GMT},
	    {"Africa/Dar_es_Salaam", EAT},
	    {"Africa/Djibouti", EAT},
	    {"Africa/Douala", WAT},
	    {"Africa/Freetown", GMT},
	    {"Africa/Gaborone", CAT},
	    {"Africa/Harare", CAT},
	    {"Africa/Johannesburg", SAST},
	    {"Africa/Kampala", EAT},
	    {"Africa/Khartoum", EAT},
	    {"Africa/Kigali", CAT},
	    {"Africa/Kinshasa", WAT},
	    {"Africa/Lagos", WAT},
	    {"Africa/Libreville", WAT},
	    {"Africa/Lome", GMT},
	    {"Africa/Luanda", WAT},
	    {"Africa/Lubumbashi", CAT},
	    {"Africa/Lusaka", CAT},
	    {"Africa/Malabo", WAT},
	    {"Africa/Maputo", CAT},
	    {"Africa/Maseru", SAST},
	    {"Africa/Mbabane", SAST},
	    {"Africa/Mogadishu", EAT},
	    {"Africa/Monrovia", GMT},
	    {"Africa/Nairobi", EAT},
	    {"Africa/Ndjamena", WAT},
	    {"Africa/Niamey", WAT},
	    {"Africa/Nouakchott", GMT},
	    {"Africa/Ouagadougou", GMT},
	    {"Africa/Porto-Novo", WAT},
	    {"Africa/Sao_Tome", GMT},
	    {"Africa/Timbuktu", GMT},
	    {"Africa/Tripoli", EET},
	    {"Africa/Tunis", CET},
	    {"Africa/Windhoek", WAT},
	    {"America/Adak", HAST},
	    {"America/Anguilla", AST},
	    {"America/Antigua", AST},
	    {"America/Aruba", AST},
	    {"America/Asuncion", new String[] {"Paraguaisk tid", "PYT",
					       "Paraguaisk sommartid", "PYST"}},
	    {"America/Barbados", AST},
	    {"America/Belize", CST},
	    {"America/Bogota", new String[] {"Colombiansk tid", "COT",
					     "Colombiansk sommartid", "COST"}},
	    {"America/Buenos_Aires", AGT},
	    {"America/Caracas", new String[] {"Venezuelansk tid", "VET",
					      "Venezuelansk sommartid", "VEST"}},
	    {"America/Cayenne", new String[] {"Franskguyansk tid", "GFT",
					      "Franskguyansk sommartid", "GFST"}},
	    {"America/Cayman", EST},
	    {"America/Costa_Rica", CST},
	    {"America/Cuiaba", AMT},
	    {"America/Curacao", AST},
	    {"America/Dawson_Creek", MST},
	    {"America/Dominica", AST},
	    {"America/Edmonton", MST},
	    {"America/El_Salvador", CST},
	    {"America/Fortaleza", BRT},
	    {"America/Godthab", new String[] {"V\u00e4stgr\u00f6nl\u00e4ndsk tid", "WGT",
					      "V\u00e4stgr\u00f6nl\u00e4ndsk sommartid", "WGST"}},
	    {"America/Grand_Turk", EST},
	    {"America/Grenada", AST},
	    {"America/Guadeloupe", AST},
	    {"America/Guatemala", CST},
	    {"America/Guayaquil", new String[] {"Ecuadoriansk tid", "ECT",
						"Ecuadoriansk sommartid", "ECST"}},
	    {"America/Guyana", new String[] {"Guyansk tid", "GYT",
					     "Guyansk sommartid", "GYST"}},
	    {"America/Havana", CST},
	    {"America/Jamaica", EST},
	    {"America/La_Paz", new String[] {"Boliviansk tid", "BOT",
					     "Boliviansk sommartid", "BOST"}},
	    {"America/Lima", new String[] {"Peruansk tid", "PET",
					   "Peruansk sommartid", "PEST"}},
	    {"America/Managua", CST},
	    {"America/Manaus", AMT},
	    {"America/Martinique", AST},
	    {"America/Mazatlan", MST},
	    {"America/Mexico_City", CST},
	    {"America/Miquelon", new String[] {"Pierre och Miquelon, normaltid", "PMST",
					       "Pierre och Miquelon, sommartid", "PMDT"}},
	    {"America/Montevideo", new String[] {"Uruguaysk tid", "UYT",
						 "Uruguaysk sommartid", "UYST"}},
	    {"America/Montreal", EST},
	    {"America/Montserrat", AST},
	    {"America/Nassau", EST},
	    {"America/Noronha", new String[] {"Fernando de Noronha-tid", "FNT",
					      "Fernando de Noronha-sommartid", "FNST"}},
	    {"America/Panama", EST},
	    {"America/Paramaribo", new String[] {"Surinamsk tid", "SRT",
						 "Surinamsk sommartid", "SRST"}},
	    {"America/Port-au-Prince", EST},
	    {"America/Port_of_Spain", AST},
	    {"America/Porto_Acre", ACT},
	    {"America/Puerto_Rico", AST},
	    {"America/Regina", CST},
	    {"America/Rio_Branco", ACT},
	    {"America/Santiago", CLT},
	    {"America/Santo_Domingo", AST},
	    {"America/Sao_Paulo", BRT},
	    {"America/Scoresbysund", EGT},
	    {"America/St_Kitts", AST},
	    {"America/St_Lucia", AST},
	    {"America/St_Thomas", AST},
	    {"America/St_Vincent", AST},
	    {"America/Tegucigalpa", CST},
	    {"America/Thule", AST},
	    {"America/Tijuana", PST},
	    {"America/Tortola", AST},
	    {"America/Vancouver", PST},
	    {"America/Winnipeg", CST},
	    {"Antarctica/Casey", WST_AUS},
	    {"Antarctica/DumontDUrville", new String[] {"Dumont-d'Urville-tid", "DDUT",
							"Dumont-d'Urville-sommartid", "DDUST"}},
	    {"Antarctica/Mawson", new String[] {"Mawson-tid", "MAWT",
						"Mawson-sommartid", "MAWST"}},
	    {"Antarctica/McMurdo", NZST},
	    {"Antarctica/Palmer", CLT},
	    {"Antarctica/Rothera", new String[] {"Rothera tid", "ROTT",
                                           "Rothera sommartid", "ROTST"}},
	    {"Asia/Aden", ARAST},
	    {"Asia/Almaty", new String[] {"Alma-Ata-tid", "ALMT",
					  "Alma-Ata-sommartid", "ALMST"}},
	    {"Asia/Amman", EET},
	    {"Asia/Anadyr", new String[] {"Anadyr-tid", "ANAT",
					  "Anadyr-sommartid", "ANAST"}},
	    {"Asia/Aqtau", new String[] {"Aqtau-tid", "AQTT",
					 "Aqtau-sommartid", "AQTST"}},
	    {"Asia/Aqtobe", new String[] {"Aqtobe-tid", "AQTT",
					  "Aqtobe-sommartid", "AQTST"}},
	    {"Asia/Ashgabat", TMT},
	    {"Asia/Ashkhabad", TMT},
	    {"Asia/Baghdad", ARAST},
	    {"Asia/Bahrain", ARAST},
	    {"Asia/Baku", new String[] {"Azerbajdzjansk tid", "AZT",
					"Azerbajdzjansk sommartid", "AZST"}},
	    {"Asia/Bangkok", ICT},
	    {"Asia/Beirut", EET},
	    {"Asia/Bishkek", new String[] {"Kirgizstan-tid", "KGT",
					   "Kirgizstansk sommartid", "KGST"}},
	    {"Asia/Brunei", new String[] {"Brunei-tid", "BNT",
					  "Brunei-sommartid", "BNST"}},
	    {"Asia/Calcutta", IST},
	    {"Asia/Colombo", new String[] {"Sri Lanka-tid", "LKT",
					   "Sri Lanka-sommartid", "LKST"}},
	    {"Asia/Dacca", BDT},
	    {"Asia/Dhaka", BDT},
	    {"Asia/Damascus", EET},
	    {"Asia/Dubai", GST},
	    {"Asia/Dushanbe", new String[] {"Tadzjikistan-tid", "TJT",
					    "Tadzjikistan-sommartid", "TJST"}},
	    {"Asia/Hong_Kong", new String[] {"Hong Kong-tid", "HKT",
					     "Hong Kong-sommartid", "HKST"}},
	    {"Asia/Irkutsk", new String[] {"Irkutsk-tid", "IRKT",
					   "Irkutsk-sommartid", "IRKST"}},
        {"Asia/Jakarta", new String[] {"West Indonesia Time", "WIT",
                       "West Indonesia Summer Time", "WIST"}},
        {"Asia/Jayapura", new String[] {"East Indonesia Time", "EIT",
                        "East Indonesia Summer Time", "EIST"}},
	    {"Asia/Kabul", new String[] {"Afghanistan-tid", "AFT",
					 "Afghanistan-sommartid", "AFST"}},
	    {"Asia/Kamchatka", new String[] {"Petropavlovsk-Kamchatski-tid", "PETT",
					     "Petropavlovsk-Kamchatski-sommartid", "PETST"}},
	    {"Asia/Karachi", PKT},
	    {"Asia/Katmandu", new String[] {"Nepal-tid", "NPT",
					    "Nepal-sommartid", "NPST"}},
	    {"Asia/Krasnoyarsk", new String[] {"Krasnoyarsk-tid", "KRAT",
					       "Krasnoyarsk-sommartid", "KRAST"}},
	    {"Asia/Kuala_Lumpur", new String[] {"Malaysisk tid", "MYT",
						"Malaysisk sommartid", "MYST"}},
	    {"Asia/Kuwait", ARAST},
	    {"Asia/Macao", CTT},
	    {"Asia/Macau", CTT},
	    {"Asia/Magadan", new String[] {"Magadan-tid", "MAGT",
					   "Magadan-sommartid", "MAGST"}},
	    {"Asia/Makassar", CIT},
	    {"Asia/Manila", new String[] {"Filippinsk tid", "PHT",
					  "Filippinsk sommartid", "PHST"}},
	    {"Asia/Muscat", GST},
	    {"Asia/Nicosia", EET},
	    {"Asia/Novosibirsk", new String[] {"Novosibirsk-tid", "NOVT",
					       "Novosibirsk-sommartid", "NOVST"}},
	    {"Asia/Oral", new String[] {"Oral-tid", "ORAT",
					"Oral sommartid", "ORAST"}},
	    {"Asia/Phnom_Penh", ICT},
	    {"Asia/Pyongyang", KST},
	    {"Asia/Qatar", ARAST},
	    {"Asia/Qyzylorda", new String[] {"Qyzylorda-tid", "QYZT",
					     "Qyzylorda sommartid", "QYZST"}},
	    {"Asia/Rangoon", new String[] {"Myanmar-tid", "MMT",
					   "Myanmar-sommartid", "MMST"}},
	    {"Asia/Riyadh", ARAST},
	    {"Asia/Saigon", ICT},
	    {"Asia/Seoul", KST},
	    {"Asia/Singapore", new String[] {"Singapore-tid", "SGT",
					     "Singapore-sommartid", "SGST"}},
	    {"Asia/Taipei", CTT},
	    {"Asia/Tashkent", new String[] {"Uzbekistan-tid", "UZT",
					    "Uzbekistan-sommartid", "UZST"}},
	    {"Asia/Tbilisi", new String[] {"Georgisk tid", "GET",
					   "Georgisk sommartid", "GEST"}},
	    {"Asia/Tehran", IRT},
	    {"Asia/Thimbu", BTT},
	    {"Asia/Thimphu", BTT},
	    {"Asia/Ujung_Pandang", CIT},
	    {"Asia/Ulaanbaatar", ULAT},
	    {"Asia/Ulan_Bator", ULAT},
	    {"Asia/Vientiane", ICT},
	    {"Asia/Vladivostok", new String[] {"Vladivostok-tid", "VLAT",
					       "Vladivostok-sommartid", "VLAST"}},
	    {"Asia/Yakutsk", new String[] {"Jakutsk-tid", "YAKT",
					   "Jakutsk-sommartid", "YAKST"}},
	    {"Asia/Yekaterinburg", new String[] {"Jekaterinburg-tid", "YEKT",
						 "Jekaterinburg-sommartid", "YEKST"}},
	    {"Asia/Yerevan", ARMT},
	    {"Atlantic/Azores", new String[] {"Azorisk tid", "AZOT",
					      "Azorisk sommartid", "AZOST"}},
	    {"Atlantic/Bermuda", AST},
	    {"Atlantic/Canary", WET},
	    {"Atlantic/Cape_Verde", new String[] {"Cape Verde-tid", "CVT",
						  "Cape Verde-sommartid", "CVST"}},
	    {"Atlantic/Faeroe", WET},
	    {"Atlantic/Jan_Mayen", EGT},
	    {"Atlantic/Reykjavik", GMT},
	    {"Atlantic/South_Georgia", new String[] {"South Georgia-normaltid", "GST",
						     "South Georgia-sommartid", "GDT"}},
	    {"Atlantic/St_Helena", GMT},
	    {"Atlantic/Stanley", new String[] {"Falklands\u00f6arna, tid", "FKT",
					       "Falklands\u00f6arna, sommartid", "FKST"}},
	    {"Australia/Adelaide", new String[] {"Central Standard Time (South Australia)", "CST",
						 "Central Summer Time (South Australia)", "CST"}},
	    {"Australia/Brisbane", new String[] {"Eastern Standard Time (Queensland)", "EST",
						 "Eastern Summer Time (Queensland)", "EST"}},
	    {"Australia/Broken_Hill", new String[] {"Central Standard Time (South Australia/New South Wales)", "CST",
						    "Central Summer Time (South Australia/New South Wales)", "CST"}},
	    {"Australia/Darwin", new String[] {"Central Standard Time (Northern Territory)", "CST",
					       "Central Summer Time (Northern Territory)", "CST"}},
	    {"Australia/Hobart", new String[] {"Eastern Standard Time (Tasmanien)", "EST",
					       "Eastern Summer Time (Tasmanien)", "EST"}},
	    {"Australia/Lord_Howe", new String[] {"Load Howe-normaltid", "LHST",
						  "Load Howe-sommartid", "LHST"}},
	    {"Australia/Perth", WST_AUS},
	    {"Australia/Sydney", EST_NSW},
	    {"BET", BRT},
	    {"BST", BDT},
	    {"CAT", CAT},
	    {"EAT", EAT},
	    {"EET", EET},
	    {"Europe/Amsterdam", CET},
	    {"Europe/Andorra", CET},
	    {"Europe/Athens", EET},
	    {"Europe/Belgrade", CET},
	    {"Europe/Berlin", CET},
	    {"Europe/Brussels", CET},
	    {"Europe/Budapest", CET},
	    {"Europe/Chisinau", EET},
	    {"Europe/Copenhagen", CET},
	    {"Europe/Dublin", new String[] {"Greenwich Mean Time", "GMT",
					    "Irl\u00e4ndsk sommartid", "IST"}},
	    {"Europe/Gibraltar", CET},
	    {"Europe/Helsinki", EET},
	    {"Europe/Istanbul", EET},
	    {"Europe/Kaliningrad", EET},
	    {"Europe/Kiev", EET},
	    {"Europe/Lisbon", WET},
	    {"Europe/London", new String[] {"Greenwich Mean Time", "GMT",
					    "Brittisk sommartid", "BST"}},
	    {"Europe/Luxembourg", CET},
	    {"Europe/Madrid", CET},
	    {"Europe/Malta", CET},
	    {"Europe/Minsk", EET},
	    {"Europe/Monaco", CET},
	    {"Europe/Moscow", new String[] {"Moskva, normaltid", "MSK",
					    "Moskva-sommartid", "MSD"}},
	    {"Europe/Oslo", CET},
	    {"Europe/Prague", CET},
	    {"Europe/Riga", EET},
	    {"Europe/Rome", CET},
	    {"Europe/Samara", new String[] {"Samara-tid", "SAMT",
					    "Samara-sommartid", "SAMST"}},
	    {"Europe/Simferopol", EET},
	    {"Europe/Sofia", EET},
	    {"Europe/Stockholm", CET},
	    {"Europe/Tallinn", EET},
	    {"Europe/Tirane", CET},
	    {"Europe/Vaduz", CET},
	    {"Europe/Vienna", CET},
	    {"Europe/Vilnius", EET},
	    {"Europe/Warsaw", CET},
	    {"Europe/Zurich", CET},
	    {"IST", IST},
	    {"Indian/Antananarivo", EAT},
	    {"Indian/Chagos", new String[] {"Indiska oceanen-tid", "IOT",
					    "Indiska oceanen-sommartid", "IOST"}},
	    {"Indian/Christmas", new String[] {"Christmas Island-tid", "CXT",
					       "Christmas Island-sommartid", "CXST"}},
	    {"Indian/Cocos", new String[] {"Cocos\u00f6isk tid", "CCT",
					   "Cocos\u00f6isk sommartid", "CCST"}},
	    {"Indian/Comoro", EAT},
	    {"Indian/Kerguelen", new String[] {"French Southern & Antarctic Lands Time", "TFT",
					       "French Southern & Antarctic Lands Summer Time", "TFST"}},
	    {"Indian/Mahe", new String[] {"Seychellisk tid", "SCT",
					  "Seychellisk sommartid", "SCST"}},
	    {"Indian/Maldives", new String[] {"Maldiverna-tid", "MVT",
					      "Maldiverna-sommartid", "MVST"}},
	    {"Indian/Mauritius", new String[] {"Mauritius-tid", "MUT",
					       "Mauritius-sommartid", "MUST"}},
	    {"Indian/Mayotte", EAT},
	    {"Indian/Reunion", new String[] {"Reunion-tid", "RET",
					     "Reunion-sommartid", "REST"}},
	    {"MET", IRT},
	    {"MIT", WST_SAMOA},
	    {"NET", ARMT},
	    {"NST", NZST},
	    {"PLT", PKT},
	    {"PRT", AST},
	    {"Pacific/Apia", WST_SAMOA},
	    {"Pacific/Auckland", NZST},
	    {"Pacific/Chatham", new String[] {"Chatham-normaltid", "CHAST",
					      "Chatham-sommartid", "CHADT"}},
	    {"Pacific/Easter", new String[] {"P\u00e5sk\u00f6arna, tid", "EAST",
					     "P\u00e5sk\u00f6arna, sommartid", "EASST"}},
	    {"Pacific/Efate", new String[] {"Vanuatu-tid", "VUT",
					    "Vanuatu-sommartid", "VUST"}},
	    {"Pacific/Enderbury", new String[] {"Phoenix\u00f6isk tid", "PHOT",
						"Phoenix\u00f6isk sommartid", "PHOST"}},
	    {"Pacific/Fakaofo", new String[] {"Tokelau-tid", "TKT",
					      "Tokelau-sommartid", "TKST"}},
	    {"Pacific/Fiji", new String[] {"Fiji-tid", "FJT",
					   "Fiji-sommartid", "FJST"}},
	    {"Pacific/Funafuti", new String[] {"Tuvalu-tid", "TVT",
					       "Tuvalu-sommartid", "TVST"}},
	    {"Pacific/Galapagos", new String[] {"Galapagos-tid", "GALT",
						"Galapagos-sommartid", "GALST"}},
	    {"Pacific/Gambier", new String[] {"Gambier-tid", "GAMT",
					      "Gambier-sommartid", "GAMST"}},
	    {"Pacific/Guadalcanal", SBT},
	    {"Pacific/Guam", ChST},
	    {"Pacific/Kiritimati", new String[] {"Line\u00f6isk tid", "LINT",
						 "Line\u00f6isk sommartid", "LINST"}},
	    {"Pacific/Kosrae", new String[] {"Kosrae-tid", "KOST",
					     "Kosrae-sommartid", "KOSST"}},
	    {"Pacific/Majuro", new String[] {"Marshall\u00f6isk tid", "MHT",
					     "Marshall\u00f6isk sommartid", "MHST"}},
	    {"Pacific/Marquesas", new String[] {"Marquesas-tid", "MART",
						"Marquesas-sommartid", "MARST"}},
	    {"Pacific/Nauru", new String[] {"Nauru-tid", "NRT",
					    "Nauru-sommartid", "NRST"}},
	    {"Pacific/Niue", new String[] {"Niue-tid", "NUT",
					   "Niue-sommartid", "NUST"}},
	    {"Pacific/Norfolk", new String[] {"Norfolk-tid", "NFT",
					      "Norfolk-sommartid", "NFST"}},
	    {"Pacific/Noumea", new String[] {"Nya Caledonien-tid", "NCT",
					     "Nya Caledonien-sommartid", "NCST"}},
	    {"Pacific/Pago_Pago", new String[] {"Samoa-normaltid", "SST",
						"Samoa-sommartid", "SDT"}},
	    {"Pacific/Palau", new String[] {"Palau-tid", "PWT",
					    "Palau-sommartid", "PWST"}},
	    {"Pacific/Pitcairn", new String[] {"Pitcairn-normaltid", "PST",
					       "Pitcairn-sommartid", "PDT"}},
	    {"Pacific/Ponape", new String[] {"Ponape-tid", "PONT",
					     "Ponape-sommartid", "PONST"}},
	    {"Pacific/Port_Moresby", new String[] {"Papua Nya Guinea-tid", "PGT",
						   "Papua Nya Guinea-sommartid", "PGST"}},
	    {"Pacific/Rarotonga", new String[] {"Cook\u00f6isk tid", "CKT",
						"Cook\u00f6isk sommartid", "CKST"}},
	    {"Pacific/Saipan", ChST},
	    {"Pacific/Tahiti", new String[] {"Tahiti-tid", "TAHT",
					     "Tahiti-sommartid", "TAHST"}},
	    {"Pacific/Tarawa", new String[] {"Gilbert\u00f6isk tid", "GILT",
					     "Gilbert\u00f6isk sommartid", "GILST"}},
	    {"Pacific/Tongatapu", new String[] {"Tonga-tid", "TOT",
						"Tonga-sommartid", "TOST"}},
	    {"Pacific/Truk", new String[] {"Truk-tid", "TRUT",
					   "Truk-sommartid", "TRUST"}},
	    {"Pacific/Wake", new String[] {"Wake-tid", "WAKT",
					   "Wake-sommartid", "WAKST"}},
	    {"Pacific/Wallis", new String[] {"Wallis och Futuna-tid", "WFT",
					     "Wallis och Futuna-sommartid", "WFST"}},
	    {"SST", SBT},
	    {"UTC", new String[] {"Coordinated Universal Time", "UTC",
			 	  "Coordinated Universal Time", "UTC"}},
	    {"VST", ICT},
	    {"WET", WET},

            {"localPatternChars", "GyMdkHmsSEDFwWahKzZ"},
        };
    }
}



