/*
 * @(#)Ports.h	1.4 03/01/23
 *
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * This file contains header information for platform port support.
 * This does not interface with the Beatnik engine at all.
 */
#ifndef PORTS_INCLUDED
#define PORTS_INCLUDED

// includes for types
#include "engine/X_API.h"
// for memset
#include <string.h>

#include "Configure.h"  // put flags for debug msgs etc. here
#include "Utilities.h"

/* *********************** PORT TYPES (for all platforms) ******************************* */

#define PORT_SRC_UNKNOWN      0x01
#define PORT_SRC_MICROPHONE   0x02
#define PORT_SRC_LINE_IN      0x03
#define PORT_SRC_COMPACT_DISC 0x04
#define PORT_SRC_MASK         0xFF
#define PORT_DST_UNKNOWN      0x0100
#define PORT_DST_SPEAKER      0x0200
#define PORT_DST_HEADPHONE    0x0300
#define PORT_DST_LINE_OUT     0x0400
#define PORT_DST_MASK         0xFF00

#define PORT_STRING_LENGTH 200

typedef struct tag_PortMixerDescription {
    char name[PORT_STRING_LENGTH];
    char vendor[PORT_STRING_LENGTH];
    char description[PORT_STRING_LENGTH];
    char version[PORT_STRING_LENGTH];
} PortMixerDescription;


// for BooleanControl.Type
#define CONTROL_TYPE_MUTE        ((char*) 1)
#define CONTROL_TYPE_SELECT      ((char*) 2)

// for FloatControl.Type
#define CONTROL_TYPE_BALANCE     ((char*) 1)
#define CONTROL_TYPE_MASTER_GAIN ((char*) 2)
#define CONTROL_TYPE_PAN         ((char*) 3)
#define CONTROL_TYPE_VOLUME      ((char*) 4)
#define CONTROL_TYPE_MAX         4

// method definitions

/* controlID: unique ID for this control
 * type: string that is used to construct the BooleanControl.Type, or CONTROL_TYPE_MUTE
 * creator: pointer to the creator struct provided by PORT_GetControls
 * returns an opaque pointer to the created control
 */
typedef void* (*PORT_NewBooleanControlPtr)(void* creator, void* controlID, char* type);

/* type: string that is used to construct the CompoundControl.Type
 * controls: an array of opaque controls returned by the CreateXXXControlPtr functions
 * controlCount: number of elements in controls
 * creator: pointer to the creator struct provided by PORT_GetControls
 * returns an opaque pointer to the created control
 */
typedef void* (*PORT_NewCompoundControlPtr)(void* creator, char* type, void** controls, int controlCount);

/* controlID: unique ID for this control
 * type: string that is used to construct the FloatControl.Type, or one of
 *       CONTROL_TYPE_BALANCE, CONTROL_TYPE_MASTER_GAIN, CONTROL_TYPE_PAN, CONTROL_TYPE_VOLUME
 * creator: pointer to the creator struct provided by PORT_GetControls
 * returns an opaque pointer to the created control
 */
typedef void* (*PORT_NewFloatControlPtr)(void* creator, void* controlID, char* type,
              float min, float max, float precision, char* units);

/* control: The control to add to current port
 * creator: pointer to the creator struct provided by PORT_GetControls
 * returns TRUE or FALSE
 */
typedef int (*PORT_AddControlPtr)(void* creator, void* control);

// struct for dynamically instantiating the controls from platform dependent code
// without creating a dependency from the platform code to JNI

typedef struct tag_PortControlCreator {
    PORT_NewBooleanControlPtr newBooleanControl;
    PORT_NewCompoundControlPtr newCompoundControl;
    PORT_NewFloatControlPtr newFloatControl;
    PORT_AddControlPtr addControl;
} PortControlCreator;

#if (USE_PORTS == TRUE)

// the following methods need to be implemented by the platform dependent code
INT32 PORT_GetPortMixerCount();
INT32 PORT_GetPortMixerDescription(INT32 mixerIndex, PortMixerDescription* description);
void* PORT_Open(INT32 mixerIndex);
void  PORT_Close(void* id);

INT32 PORT_GetPortCount(void* id);
INT32 PORT_GetPortType(void* id, INT32 portIndex);
INT32 PORT_GetPortName(void* id, INT32 portIndex, char* name, INT32 len);
void  PORT_GetControls(void* id, INT32 portIndex, PortControlCreator* creator);
float PORT_GetFloatValue(void* controlID);
INT32 PORT_GetIntValue(void* controlIDV);
void  PORT_SetFloatValue(void* controlID, float value);
void  PORT_SetIntValue(void* controlIDV, INT32 value);

#endif // USE_PORTS

#endif // PORTS_INCLUDED
