/*
 * @(#)CStrike.h	1.4 03/01/23
 *
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#ifndef cstrike_header
#define cstrike_header

#ifdef __solaris__

#include "FontGlue.h"
#include "hsGGlyphCache.h"
#include "hsGFontScaler.h"
#include "hsFixedTypes.h"

class fontObject;
class CharToGlyphMapper;

/* This is a value which is an impossible valid address */
#define UNALLOCATEDSTRIKE ((Strike*)(-1L))

class CStrike : public Strike {
 private:
    // choosing to not re-use "compositeStrike" from parent mostly because of
    // its name, but also as a level of precaution against superclass code
    // that may interpret a non-null value to mean this is a composite strike.
    // Such code expects the length of the "compositeStrike" array to
    // be based on the number of slots in a composite font.
    Strike **nrStrikes; // native rasteriser strikes.
 private:
    CStrike();
    CStrike(Strike &);
    CStrike& operator =(const CStrike&);
 public:
    CStrike(fontObject& fo, FontTransform& tx,
	    jboolean isAntiAliased, jboolean usesFractionalMetrics);
    virtual ~CStrike() {
	if (nrStrikes != NULL) {
	    int numStrikes = fFont.GetNumberOfNativeNames();
	    for (int i=0; i < numStrikes; i++) {
		if (nrStrikes[i] != NULL &&
		    nrStrikes[i] != UNALLOCATEDSTRIKE) {
		    delete nrStrikes[i];
		    nrStrikes[i] = NULL;
		}
	    }
	    if (nrStrikes != defaultStrikes) {
		delete [] nrStrikes;
	    }
	    nrStrikes = NULL;
	}
    }

    virtual Boolean getMetricsWithImage(int glyphCode, hsGGlyph& glyphRef,
				hsFixedPoint2& advXY, Boolean* refcheck = 0);
    virtual const void *getImage(int glyphCode, Boolean* refcheck = 0);
    virtual const void *refImage(int glyphCode);
    virtual hsGGlyphStrike* getStrikeForGlyph(int& glyphCode);
};

#endif /* __solaris__ */
#endif /* cstrike_header */
