/*
 * @(#)FontGlue.h	1.28 03/01/23
 *
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * @author Charlton Innovations, Inc.
 */
#ifndef fontGlue_header
#define fontGlue_header

#include <jni.h>
#include <jni_util.h>
#include <jlong_md.h>
#include <sun_awt_font_NativeFontWrapper.h>

#include <memory.h>

#include "fontObject.h"
#include "hsGGlyphCache.h"
#include "hsGFontScaler.h"
#include "hsFixedTypes.h"
#include "ImageData.h"

extern fontObject *getFontPtr(JNIEnv *env, jobject font);

// FontTransform --- constructed from primitiveArray of float
class FontTransform {
public:
    jdouble fMatrix[4];
private:
// make sure that caller must place these objects on the stack
// since they have internal state which is stack dependent.
// (actually this is a gentle hint to other programmers, since
// they can get around this if they want to)
//    FontTransform(FontTransform&);  // no implementation
    FontTransform& operator =(const FontTransform&);
//    static void *operator new(size_t size);
//    static void operator delete(void *ptr);
public:
    FontTransform() {
        fMatrix[0] = 1;
        fMatrix[1] = 0;
        fMatrix[2] = 0;
        fMatrix[3] = 1;
    }
    FontTransform(jdouble m00, jdouble m10, jdouble m01, jdouble m11) {
	fMatrix[0] = m00;
	fMatrix[1] = m10;
	fMatrix[2] = m01;
	fMatrix[3] = m11;
    }
    FontTransform(float *matrix) {
        if (matrix) {
            fMatrix[0] = matrix[0];
            fMatrix[1] = matrix[1];
            fMatrix[2] = matrix[2];
            fMatrix[3] = matrix[3];
        }
    }
    FontTransform(JNIEnv *env, jdoubleArray matRef) {
        jint matSize = env->GetArrayLength(matRef);
        if (matSize >= 4) {
            jdouble matrix[4];
            env->GetDoubleArrayRegion(matRef, 0, 4, matrix);
            fMatrix[0] = matrix[0];
            fMatrix[1] = matrix[1];
            fMatrix[2] = matrix[2];
            fMatrix[3] = matrix[3];
        }
    }
    ~FontTransform() {
    }
    void getMatrixInto(float *dest, unsigned int destSize) {
        if (fMatrix && (destSize >= 4)) {
            dest[0] = fMatrix[0];
            dest[1] = fMatrix[1];
            dest[2] = fMatrix[2];
            dest[3] = fMatrix[3];
        } else {
            memset(dest, 0, destSize * sizeof(*dest));
        }
    }
    int equals(const FontTransform& cTX) const {
        return fMatrix
            && cTX.fMatrix
            && fMatrix[0] == cTX.fMatrix[0]
            && fMatrix[1] == cTX.fMatrix[1]
            && fMatrix[2] == cTX.fMatrix[2]
            && fMatrix[3] == cTX.fMatrix[3];
    }
    FontTransform(FontTransform& cTX) {
        fMatrix[0] = cTX.fMatrix[0];
        fMatrix[1] = cTX.fMatrix[1];
        fMatrix[2] = cTX.fMatrix[2];
        fMatrix[3] = cTX.fMatrix[3];
    }
};

// UInt32Buffer --- constructed from primitiveArray of float
class UInt32Buffer {
private:
    const jintArray arrayRef;
    JNIEnv * const fEnv;
    UInt32 *fStorage;
    UInt32 *fBuffer;
    UInt32 fArraySize;
private:
// make sure that caller must place these objects on the stack
// since they have internal state which is stack dependent.
// (actually this is a gentle hint to other programmers, since
// they can get around this if they want to)
    UInt32Buffer();               // no implementation
    UInt32Buffer(UInt32Buffer&);    // no implementation
    UInt32Buffer& operator =(const UInt32Buffer&);
    static void *operator new(size_t size);
    static void operator delete(void *ptr);
public:
    UInt32Buffer(JNIEnv *env, jintArray intArray, UInt32 offset = 0)
            : arrayRef(intArray), fEnv(env),
                fStorage(NULL), fBuffer(NULL), fArraySize(0){
        if (!JNU_IsNull (env, arrayRef)) {
            fArraySize = env->GetArrayLength(arrayRef);
            fStorage = (UInt32 *)env->GetPrimitiveArrayCritical(arrayRef, NULL);
            if (fStorage) {
                fBuffer = fStorage + offset;
            }
        }
    }
    ~UInt32Buffer() {
        if (fStorage) {
            fEnv->ReleasePrimitiveArrayCritical(
                                            arrayRef, fStorage, JNI_ABORT);
        }
    }
    UInt32 getNumElements(void) const { return fArraySize;}
    UInt32 *buffer(void) const { return fBuffer; }
    Boolean invalid(void) const {   return (fBuffer == NULL); }
};


// JFloatBuffer --- constructed from primitiveArray of float
class JFloatBuffer {
private:
    const jfloatArray arrayRef;
    JNIEnv * const fEnv;
    jfloat *fStorage;
    jfloat *fBuffer;
    UInt32 fArraySize;
private:
// make sure that caller must place these objects on the stack
// since they have internal state which is stack dependent.
// (actually this is a gentle hint to other programmers, since
// they can get around this if they want to)
    JFloatBuffer();               // no implementation
    JFloatBuffer(JFloatBuffer&);    // no implementation
    JFloatBuffer& operator =(const JFloatBuffer&);
    static void *operator new(size_t size);
    static void operator delete(void *ptr);
public:
    JFloatBuffer(JNIEnv *env, jfloatArray floatArray, jint offset = 0)
            : arrayRef(floatArray), fEnv(env),
                fStorage(NULL), fBuffer(NULL), fArraySize(0){
        if (!JNU_IsNull (env, arrayRef)) {
            fArraySize = env->GetArrayLength(arrayRef);
            fStorage = (jfloat *)env->GetPrimitiveArrayCritical(arrayRef, NULL);
            if (fStorage) {
                fBuffer = fStorage + offset;
            }
        }
    }
    ~JFloatBuffer() {
        if (fStorage) {
            fEnv->ReleasePrimitiveArrayCritical(
                                            arrayRef, fStorage, JNI_ABORT);
        }
    }
    UInt32 getNumElements(void) const { return fArraySize;}
    jfloat *buffer(void) const { return fBuffer; }
    Boolean invalid(void) const {   return (fBuffer == NULL); }
};

// JStringBuffer --- constructed from primitiveArray of String
class JStringBuffer {
private:
    const jstring stringRef;
    JNIEnv * const fEnv;
    const jchar *fStorage;
    jsize fLength;
private:
// make sure that caller must place these objects on the stack
// since they have internal state which is stack dependent.
// (actually this is a gentle hint to other programmers, since
// they can get around this if they want to)
    JStringBuffer();               // no implementation
    JStringBuffer(JStringBuffer&);    // no implementation
    JStringBuffer& operator =(const JStringBuffer&);
    static void *operator new(size_t size);
    static void operator delete(void *ptr);
public:
    JStringBuffer(JNIEnv *env, jstring theString)
            : stringRef(theString), fEnv(env),
                fStorage(NULL) {
        if (!JNU_IsNull (env, theString)) {
            fLength = env->GetStringLength(stringRef);
            fStorage = env->GetStringCritical(stringRef, JNI_FALSE);
        }
    }
    ~JStringBuffer() {
        if (fStorage) {
            fEnv->ReleaseStringCritical(stringRef, fStorage);
        }
    }
    UInt32 getLength(void) const { return fLength;}
    const jchar *buffer(void) const { return fStorage; }
    Boolean invalid(void) const {   return (fStorage == NULL); }
};

// JStringBufferCopy --- constructed from array of String
class JStringBufferCopy {
private:
    const jstring stringRef;
    JNIEnv * const fEnv;
    const jchar *fStorage;
    jsize fLength;
private:
    JStringBufferCopy();               // no implementation
    JStringBufferCopy(JStringBufferCopy&);    // no implementation
    JStringBufferCopy& operator =(const JStringBufferCopy&);
    static void *operator new(size_t size);
    static void operator delete(void *ptr);
public:
    JStringBufferCopy(JNIEnv *env, jstring theString)
            : stringRef(theString), fEnv(env),
                fStorage(NULL) {
        if (!JNU_IsNull (env, theString)) {
            fLength = env->GetStringLength(stringRef);
            fStorage = env->GetStringChars(stringRef, JNI_FALSE);
        }
    }
    ~JStringBufferCopy() {
        if (fStorage) {
            fEnv->ReleaseStringChars(stringRef, fStorage);
        }
    }
    UInt32 getLength(void) const { return fLength;}
    const jchar *buffer(void) const { return fStorage; }
    Boolean invalid(void) const {   return (fStorage == NULL); }
};

#endif // fontGlue_header
