/*
 * @(#)Strike.h	1.32 03/01/23
 *
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * @author Charlton Innovations, Inc.
 */
#ifndef strike_header
#define strike_header

#include "FontGlue.h"
#include "hsGGlyphCache.h"
#include "hsGFontScaler.h"
#include "hsFixedTypes.h"

class fontObject;           // forward reference
class CompositeFont;        // forward reference
class CharToGlyphMapper;    // forward reference

// Strike --- encapsulate "strike" instance for font as determined
// by transform and rendering hints.
class Strike {
 protected:
    const fontObject& fFont;
    hsGGlyphStrike  *strike;
    // cached state
    CharToGlyphMapper *fMapper;
    // cached state to speed up line metrics repeat calls
    int           fCachedLineMetrics;
    hsFixedPoint2 fAscent;
    hsFixedPoint2 fDescent;
    hsFixedPoint2 fBaseline;
    hsFixedPoint2 fLeading;
    hsFixedPoint2 fMaxAdvance;

    // Composite strike variables not used by CStrike subclass.
 private:
    Strike          **compositeStrike;
    int             isComposite;
    int totalCompositeGlyphs;
    // int baseGlyphs;

    // cached state needed for efficient implementation of compositeStrike
    // Also used by subclass.
 protected:
    FontStrikeDesc *fStrikeDescs;
    enum { DEFAULT_STRIKE_SLOTS = 6};
    // default buffers to avoid mallocs in the common cases
    FontStrikeDesc templateDesc;
    FontStrikeDesc defaultStrikeDescs[DEFAULT_STRIKE_SLOTS];
    Strike *defaultStrikes[DEFAULT_STRIKE_SLOTS];

#ifdef __solaris__
 public:
    UInt32 *latinCharsToGlyphs;
#endif
    // methods for compositeStrike
 private:
    void allocateStrikes();
    void allocateDescs();
    inline hsGGlyphStrike *compositeStrikeForGlyph(int& glyphCode);
    inline hsGGlyphStrike *compositeStrikeForGlyphImage(int& glyphCode);
    hsGGlyphStrike *compositeStrikeForSlotInt(int slot, int& glyphCode);
    int compositeCanRotate ();
 public:
    inline hsGGlyphStrike *compositeStrikeForSlot(int slot);
    static int algorithmicStyle(const CompositeFont& cf,
                                const fontObject& fo,
                                int slot);
    static void needsAlgorithmicStyle (const CompositeFont& cf, 
				       int slot, Boolean *algorithmic );

    hsGGlyphStrike *getHSStrike() {
	return strike;
    };
				 
// make sure that caller must place these objects on the stack
// since they have internal state which is stack dependent.
// (actually this is a gentle hint to other programmers, since
// they can get around this if they want to)
 protected:
    Strike(fontObject& fo) : 
	fFont(fo),
	strike(NULL),
        fMapper(NULL),
        fCachedLineMetrics(JNI_FALSE),
        compositeStrike(NULL),
        isComposite(JNI_FALSE),
        totalCompositeGlyphs(0),
#ifdef __solaris__
        latinCharsToGlyphs(NULL),
#endif
        fStrikeDescs(NULL) {};

    virtual CharToGlyphMapper *getMapper() const;

 private:
    Strike();           // no implementatiom
    Strike(Strike &);   // no implementation
    Strike& operator =(const Strike&);
//    static void *operator new(size_t size);
//    static void operator delete(void *ptr);
 public:
    Strike( fontObject& fo, FontTransform& tx,
            jboolean isAntiAliased, jboolean usesFractionalMetrics);
    Strike( fontObject& fo, enum FontFormats scalerID, FontTransform& tx,
            jboolean isAntiAliased, jboolean usesFractionalMetrics,
	    int encodingIndex = 0);
    virtual ~Strike();

    virtual Boolean getOutlineBounds(int glyphCode, hsRect& rect);
    virtual Boolean getMetrics(int glyphCode, hsGGlyph& glyphRef,
                                                hsFixedPoint2& advXY);
    virtual Boolean getMetricsWithImage(int glyphCode, hsGGlyph& glyphRef,
				hsFixedPoint2& advXY, Boolean* refcheck = 0);
    virtual const void *getImage(int glyphCode, Boolean* refcheck = 0);
    virtual const void *refImage(int glyphCode);
    virtual hsGGlyphStrike* getStrikeForGlyph(int& glyphCode) {
	return strike;
    }
    virtual void prepareCache(int numGlyphs, Boolean* refcheck = 0);
    virtual void GetLineHeight(hsFixedPoint2& ascent,
                      hsFixedPoint2& descent,
                      hsFixedPoint2& baseline,
                      hsFixedPoint2& leading,
                      hsFixedPoint2& maxAdvance);
    virtual void GetItalicAngle(hsFixedPoint2& italicAngle);
    virtual void GetGlyphOutline(int glyphCode, hsPathSpline& path);
    virtual Boolean GetGlyphPoint(int glyphCode, int pointNumber,
				  hsFixedPoint2 &point);
    virtual void TransformFunits(int glyphCode, short xFunits, short yFunits,
				 hsFixedPoint2 &pixels);
    virtual int  GetNumGlyphs();

    virtual Boolean isStyleSupported(int);
    virtual Boolean canDisplay(Unicode ch);
    virtual int canRotate ();
    virtual Boolean canRotateString (int fNumGlyphs, UInt32 *glyphs);
    virtual UInt32 CharToGlyph(Unicode32 unicode);
    virtual int getMissingGlyphCode() const;
    virtual Boolean doesGlyphShaping(void) const { return false; }
    virtual hsPathSpline::Type GetPathType (int glyphCode);
};

#endif // strike_header
