/*
 * @(#)hsGGlyphCache.h	1.22 03/01/23
 *
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * Copyright (C) 1996-1997 all rights reserved by HeadSpin Technology Inc. Chapel Hill, NC USA
 *
 * This software is the property of HeadSpin Technology Inc. and it is furnished
 * under a license and may be used and copied only in accordance with the
 * terms of such license and with the inclusion of the above copyright notice.
 * This software or any other copies thereof may not be provided or otherwise
 * made available to any other person or entity except as allowed under license.
 * No title to and ownership of the software or intellectual property
 * therewithin is hereby transferred.
 *
 * HEADSPIN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * This information in this software is subject to change without notice
*/

#ifndef hsGGlyphCacheDefined
#define hsGGlyphCacheDefined

#include "fontObject.h"
#include "hsGText.h"
#include "hsDescriptor.h"
#include "hsGeometry.h"

////////////////////////////////////////////////////////////////////////////////

class hsGGlyphStrike : public hsRefCnt {
private:
	class hsGScalerContext*	fScalerContext;		// created by the scaler
	// cached from the scaler context
	UInt32				fGlyphCount;
	hsFixedPoint2			fAscent, fDescent, fBaseline;
        hsFixedPoint2                   fLeading, fMaxAdvance;

	struct hsGGlyphStrikeEntry*	fEntries;
	struct hsGGlyphStrikeEntry**    fExtendedEntries;
	UInt32                          fID;
	UInt32                          fSizeHint;
        static UInt32                   gID;

	void                    getEntryImage(hsGGlyphStrikeEntry* entry, UInt16 glyphIndex, Boolean* purged = 0);
	void                    refEntryImage(hsGGlyphStrikeEntry* entry, UInt16 glyphIndex);
	void                    assignEntryImage(hsGGlyphStrikeEntry* entry, UInt16 glyphIndex, Boolean* purged = 0);
	
	struct hsGGlyphStrikeEntry* getEntry(int index);
        struct hsGGlyphStrikeEntry* getExtendedEntry(int index);
	void                    initEntries(struct hsGGlyphStrikeEntry* entries, int count);

protected:
	hsDescriptor		fDesc;
public:
				hsGGlyphStrike(hsConstDescriptor desc);
	virtual			~hsGGlyphStrike();

	hsConstDescriptor	GetDesc() const { return fDesc; }
	hsPathSpline::Type	GetPathType() const;

	UInt32			CountGlyphs() const { return fGlyphCount; }
	void			GetOutlineBounds(UInt16 index, hsRect* bounds);
	void			GetMetrics(UInt16 index, hsGGlyph* glyph, hsFixedPoint2* advance);
	void			GetMetricsWithImage(UInt16 index, hsGGlyph* glyph, hsFixedPoint2* advance,
						    Boolean* refcheck = 0);
	const void*		GetImage(UInt16 index, Boolean* refcheck = 0);
	const void*             RefImage(UInt16 index);
        void                    PrepareCache(UInt32 numGlyphs, Boolean* refcheck = 0);
	void			GetPath(UInt16 index, struct hsPathSpline* path, hsFixedPoint2* advance);
    Boolean         GetGlyphPoint(UInt16 index, Int32 pointNumber, hsFixedPoint2 *point);
    void            TransformFunits(Int16 xFunits, Int16 yFunits, hsFixedPoint2 *pixels);
        void                    GetLineHeight(hsFixedPoint2* ascent,
                                        hsFixedPoint2* descent,
                                        hsFixedPoint2* baseline,
                                        hsFixedPoint2* leading,
                                        hsFixedPoint2* maxAdvance);

        void                    GetItalicAngle(hsFixedPoint2* italicAngle);
        CharToGlyphMapper       *getMapper();
	Boolean			canRotate ();
	void                    resetID();

	// Call this to find a strike. Call strike->UnRef() when you are through  with it
	static hsGGlyphStrike*	RefStrike(hsConstDescriptor desc);

	// Call this to clean-up
	static void		KillGlyphCache();

	virtual Boolean isStyleSupported(int);
};

#endif
