/*
 * @(#)MetricsInfo.h	1.8 03/01/23 
 *
 * (C) Copyright IBM Corp. 1999 - All Rights Reserved
 *
 * Portions Copyright 2003 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 *
 * The original version of this source code and documentation is
 * copyrighted and owned by IBM. These materials are provided
 * under terms of a License Agreement between IBM and Sun.
 * This technology is protected by multiple US and International
 * patents. This notice and attribution to IBM may not be removed.
 */

#ifndef __METRICSINFO_H
#define __METRICSINFO_H

#include "dtypes.h"
#include "HeadSpin.h"
#include "hsFixedTypes.h"
#include "FontGlue.h"
#include "fontObject.h"
#include "Strike.h"

class MetricsInfo
{
private:
    Strike *strike;
    tt_int32 slot;

    tt_uint16 xppem;
    tt_uint16 yppem;

    float xScaleUnitsToPoints;
    float yScaleUnitsToPoints;

    hsFixed xScalePixelsToUnits;
    hsFixed yScalePixelsToUnits;

    float euclidianDistance(float a, float b);


public:
    MetricsInfo(fontObject *fo, FontTransform *tx,
            Strike *theStrike, tt_int32 theSlot, tt_int32 xRes, tt_int32 yRes);

    ~MetricsInfo();

    tt_uint16 getXPixelsPerEm();
    tt_uint16 getYPixelsPerEm();

    float xUnitsToPoints(hsFixed units);
    float yUnitsToPoints(hsFixed units);

    hsFixed xPixelsToUnits(tt_int16 pixels);
    hsFixed yPixelsToUnits(tt_int16 pixels);

    hsFixed xPixelsToUnits(hsFixed pixels);
    hsFixed yPixelsToUnits(hsFixed pixels);

    Boolean getGlyphPoint(tt_uint32 glyphID, tt_uint32 point, hsFixedPoint2 *anchor);
    void getGlyphAdvance(tt_uint32 glyphID, hsFixedPoint2 *advance);
    void transformFunits(tt_int16 xFunits, tt_int16 yFunits, hsFixedPoint2 *pixels);
};

inline float MetricsInfo::xUnitsToPoints(hsFixed units)
{
    return hsFixedToFloat(units) * xScaleUnitsToPoints;
}

inline float MetricsInfo::yUnitsToPoints(hsFixed units)
{
    return hsFixedToFloat(units) * yScaleUnitsToPoints;
}

inline hsFixed MetricsInfo::xPixelsToUnits(tt_int16 pixels)
{
    return pixels * xScalePixelsToUnits;
}

inline hsFixed MetricsInfo::yPixelsToUnits(tt_int16 pixels)
{
    return pixels * yScalePixelsToUnits;
}

inline hsFixed MetricsInfo::xPixelsToUnits(hsFixed pixels)
{
    return hsFixMul(pixels, xScalePixelsToUnits);
}

inline hsFixed MetricsInfo::yPixelsToUnits(hsFixed pixels)
{
    return hsFixMul(pixels, yScalePixelsToUnits);
}

inline tt_uint16 MetricsInfo::getXPixelsPerEm()
{
    return xppem;
}

inline tt_uint16 MetricsInfo::getYPixelsPerEm()
{
    return yppem;
}


#endif


