/*
 * @(#)jcov_hash.h	1.10 03/01/23
 *
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#ifndef _JCOV_HASH_H
#define _JCOV_HASH_H

/* Generic support for hash tables */
/* based on src/share/tools/hprof/hprof_hash.h */
#include "jcov_types.h"

typedef struct jcov_bucket_t {
    struct jcov_bucket_t *next;
    void *content;
} jcov_bucket_t;

typedef struct {
    SIZE_T        n_entries;          /* total buckets in the table  */
    SIZE_T        size;               /* size of the top-level table */
    jcov_bucket_t **entries;
    UINTPTR_T     (*hash_f)(void *);  /* hash function       */
    SIZE_T        (*size_f)(void *);  /* content size        */
    int (*compare_f)(void *, void *); /* comparison function */
} jcov_hash_t;

jcov_hash_t *jcov_hash_new(SIZE_T    size,
                           UINTPTR_T (*hash_f)(void *),
                           SIZE_T    (*size_f)(void *),
                           int       (*compare_f)(void *, void *));
void *jcov_hash_iterate(jcov_hash_t *table,
                        void * (*f)(void *, void *),
                        void *arg);
void jcov_hash_remove(jcov_hash_t *table,
                      int (*f)(void *, void *),
                      void *arg);
void  *jcov_hash_lookup(jcov_hash_t *table, void *new);
void  *jcov_hash_put(jcov_hash_t *table, void *new);
void **jcov_hash_to_array(jcov_hash_t *table);

#endif /* _JCOV_HASH_H */
