/*
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * @(#)ByteToCharGB18030.java	1.8 03/01/23
 */
 
package sun.io;

public class ByteToCharGB18030 extends ByteToCharGB18030DB {
    
    private static final int GB18030_SINGLE_BYTE = 1;
    private static final int GB18030_DOUBLE_BYTE = 2;
    private static final int GB18030_FOUR_BYTE = 3;
    
    private int currentState;
    private int savedSize;
    private byte[] savedBytes;
    
    public ByteToCharGB18030() {
	super();
	savedBytes = new byte[3];
	currentState = GB18030_DOUBLE_BYTE;
	savedSize = 0;
    }

    public short[] getOuter() {
	return(index1);
    }

    public String[] getInner() {
	return(index2);
    }

    public short[] getDBIndex1() {
	return(super.index1);
    }

    public String[] getDBIndex2() {
	return(super.index2);
    }

    public int flush(char [] output, int outStart, int outEnd)
	throws MalformedInputException
    {
	if (savedSize != 0) {
	    savedSize = 0;
	    currentState = GB18030_DOUBLE_BYTE;
	    badInputLength = 0;
	    throw new MalformedInputException();
	}
	byteOff = charOff = 0;
	return 0;
    }


    /**
     * Character conversion
     */
    public int convert(byte[] input, int inOff, int inEnd,
		       char[] output, int outOff, int outEnd)
        throws UnknownCharacterException, MalformedInputException,
	       ConversionBufferFullException
    {
	int inputSize = 0;
    	char outputChar = '\uFFFD';
	// readOff keeps the actual buffer's pointer.
	// byteOff keeps original buffer's pointer.
	int readOff = byteOff = inOff;

	if (savedSize != 0) {
	    // Filter illegal bytes when they are detected in saved
	    // partial input from a previous conversion attempt.
	    if (((savedBytes[0] & 0xFF) < 0x81 || savedBytes[0] > 0xFE) ||
		 (savedSize > 1 && 
		 (savedBytes[1] & 0xFF) < 0x30 ) ||
		 (savedSize > 2 &&
		 ((savedBytes[2] & 0xFF) < 0x81 ||
		 (savedBytes[2] & 0xFF) > 0xFE ))) {
		    badInputLength = 0;
		    throw new MalformedInputException();
	    }

	    byte[] newBuf = new byte[inEnd - inOff + savedSize];
	    for (int i = 0; i < savedSize; i++) {
		newBuf[i] = savedBytes[i];
	    }
	    System.arraycopy(input, inOff, newBuf, savedSize, inEnd - inOff);
	    byteOff -= savedSize;
	    input = newBuf;
	    inOff = 0;
	    inEnd = newBuf.length;
	    savedSize = 0;
	} 

	charOff = outOff;
	readOff = inOff;

	while(readOff < inEnd) {
	    int byte1 = 0 , byte2 = 0, byte3 = 0, byte4 = 0;
	    
	    // Is there room in the output buffer for the result?
	    if (charOff >= outEnd) {
		throw new ConversionBufferFullException();
	    }

	    // Get the input byte
	    byte1 = input[readOff++] & 0xFF;
	    inputSize = 1;

	    if ((byte1 & (byte)0x80) == 0){ // US-ASCII range
		outputChar = (char)byte1;
		currentState = GB18030_SINGLE_BYTE;
	    }

	    else if (byte1 < 0x81 || byte1 > 0xfe) {
		if (subMode)
                    outputChar = subChars[0];
                else {
                    badInputLength = 1;
                    throw new UnknownCharacterException();
                }
	    }
	    else {
		// Either 2 or 4 byte sequence follows
		// If an underrun is detected save for later
		// replay.

		if (readOff + inputSize > inEnd) {
		    savedBytes[0]=(byte)byte1;
		    savedSize = 1;
		    break;
		}

	        byte2 = input[readOff++] & 0xFF;
		inputSize = 2;

		if (byte2 < 0x30) {
		    badInputLength = 1;
		    throw new MalformedInputException();
		} 
		else if (byte2 >= 0x30 && byte2 <= 0x39) {
		    currentState = GB18030_FOUR_BYTE;
		    inputSize = 4;

		    if (readOff + 2 > inEnd) {
			if (readOff + 1 > inEnd) {
			    savedBytes[0] = (byte)byte1;	
			    savedBytes[1] = (byte)byte2;	
			    savedSize = 2;
			}
			else {
			    savedBytes[0] = (byte)byte1;	
			    savedBytes[1] = (byte)byte2;	
			    savedBytes[2] = input[readOff++];
			    savedSize = 3;
			}
			break;
		    }
		    byte3 = input[readOff++] & 0xFF;
		    if (byte3 < 0x81 || byte3 > 0xfe) {
			badInputLength = 3;
			throw new MalformedInputException();
		    }

		    byte4 = input[readOff++] & 0xFF; 

		    if (byte4 < 0x30 || byte4 > 0x39) {
			badInputLength = 4;
			throw new MalformedInputException();
		    }
		}
		else if (byte2 == 0x7f || byte2 == 0xff ||
			(byte2 < 0x40 )) {
		   badInputLength = 2;
		   throw new MalformedInputException();
		}
		else 
		    currentState = GB18030_DOUBLE_BYTE;
	    }

	    switch (currentState){
		case GB18030_SINGLE_BYTE:
		    output[charOff++] = (char)(byte1);
		    break;
		case GB18030_DOUBLE_BYTE:
		    output[charOff++] = super.getUnicode(byte1, byte2);
		    break;
		case GB18030_FOUR_BYTE:
		    int offset = (((byte1 - 0x81) * 10 +
				   (byte2 - 0x30)) * 126 +
				    byte3 - 0x81) * 10 + byte4 - 0x30;		
		    int hiByte = (offset >>8) & 0xFF;
		    int lowByte = (offset & 0xFF);

		// Mixture of table lookups and algorithmic calculation
		// of character values.

		// BMP Ranges

		if (offset <= 0x4A62)
		    output[charOff++] = getChar(offset);
		else if (offset > 0x4A62 && offset <= 0x82BC)
		    output[charOff++] = (char) (offset + 0x5543); 
		else if (offset >= 0x82BD && offset <= 0x830D)
		    output[charOff++] = getChar(offset);
		else if (offset >= 0x830D && offset <= 0x93A8)
		    output[charOff++] = (char) (offset + 0x6557);
		else if (offset >= 0x93A9 && offset <= 0x99FB)
		    output[charOff++] = getChar(offset);
		// Supplemental UCS planes handled via surrogates
		else if (offset >= 0x2E248 && offset < 0x12E248) {
		    if (offset >= 0x12E248) {
			if (subMode)
			   return subChars[0];
			else {
			   badInputLength = 4;
			   throw new UnknownCharacterException();
			}
		    }

		    if (charOff +2 > outEnd) {
			throw new ConversionBufferFullException();
		    }
		    offset -= 0x1e248;
	            char highSurr = (char) ((offset - 0x10000) / 0x400 + 0xD800);
	            char lowSurr = (char) ((offset - 0x10000) % 0x400 + 0xDC00);
		    output[charOff++] = highSurr;
		    output[charOff++] = lowSurr;
		}
		else {
		    badInputLength = 4;
		    throw new MalformedInputException();
		    }
		break;
	      }
	byteOff += inputSize;
	}

	byteOff += savedSize;
	return charOff - outOff;
    }

    public void reset() {
	byteOff = charOff = 0;
	currentState = GB18030_DOUBLE_BYTE;
	savedSize = 0;
    }

    public String getCharacterEncoding() {
	return "GB18030";
    }

    private char getChar(int offset) throws UnknownCharacterException {
	int byte1 = (offset >>8) & 0xFF;
	int byte2 = (offset & 0xFF);
	int start = 0, end = 0xFF;

	if (((byte1 < 0) || (byte1 > index1.length))
	     || ((byte2 < start) || (byte2 > end))) {
                if (subMode)
                   return subChars[0];
                else {
                   badInputLength = 1;
                   throw new UnknownCharacterException();
                }
	}

	int n = (index1[byte1] & 0xf) * (end - start + 1) + (byte2 - start);
	return index2[index1[byte1] >> 4].charAt(n);
    }

    private final static String innerIndex0= 
	"\u0080"+
        "\u0081\u0082\u0083\u0084\u0085\u0086\u0087\u0088"+
        "\u0089\u008A\u008B\u008C\u008D\u008E\u008F\u0090"+
        "\u0091\u0092\u0093\u0094\u0095\u0096\u0097\u0098"+
        "\u0099\u009A\u009B\u009C\u009D\u009E\u009F\u00A0"+
        "\u00A1\u00A2\u00A3\u00A5\u00A6\u00A9\u00AA\u00AB"+
        "\u00AC\u00AD\u00AE\u00AF\u00B2\u00B3\u00B4\u00B5"+
        "\u00B6\u00B8\u00B9\u00BA\u00BB\u00BC\u00BD\u00BE"+
        "\u00BF\u00C0\u00C1\u00C2\u00C3\u00C4\u00C5\u00C6"+
        "\u00C7\u00C8\u00C9\u00CA\u00CB\u00CC\u00CD\u00CE"+
        "\u00CF\u00D0\u00D1\u00D2\u00D3\u00D4\u00D5\u00D6"+
        "\u00D8\u00D9\u00DA\u00DB\u00DC\u00DD\u00DE\u00DF"+
        "\u00E2\u00E3\u00E4\u00E5\u00E6\u00E7\u00EB\u00EE"+
        "\u00EF\u00F0\u00F1\u00F4\u00F5\u00F6\u00F8\u00FB"+
        "\u00FD\u00FE\u00FF\u0100\u0102\u0103\u0104\u0105"+
        "\u0106\u0107\u0108\u0109\u010A\u010B\u010C\u010D"+
        "\u010E\u010F\u0110\u0111\u0112\u0114\u0115\u0116"+
        "\u0117\u0118\u0119\u011A\u011C\u011D\u011E\u011F"+
        "\u0120\u0121\u0122\u0123\u0124\u0125\u0126\u0127"+
        "\u0128\u0129\u012A\u012C\u012D\u012E\u012F\u0130"+
        "\u0131\u0132\u0133\u0134\u0135\u0136\u0137\u0138"+
        "\u0139\u013A\u013B\u013C\u013D\u013E\u013F\u0140"+
        "\u0141\u0142\u0143\u0145\u0146\u0147\u0149\u014A"+
        "\u014B\u014C\u014E\u014F\u0150\u0151\u0152\u0153"+
        "\u0154\u0155\u0156\u0157\u0158\u0159\u015A\u015B"+
        "\u015C\u015D\u015E\u015F\u0160\u0161\u0162\u0163"+
        "\u0164\u0165\u0166\u0167\u0168\u0169\u016A\u016C"+
        "\u016D\u016E\u016F\u0170\u0171\u0172\u0173\u0174"+
        "\u0175\u0176\u0177\u0178\u0179\u017A\u017B\u017C"+
        "\u017D\u017E\u017F\u0180\u0181\u0182\u0183\u0184"+
        "\u0185\u0186\u0187\u0188\u0189\u018A\u018B\u018C"+
        "\u018D\u018E\u018F\u0190\u0191\u0192\u0193\u0194"+
        "\u0195\u0196\u0197\u0198\u0199\u019A\u019B"+
	"\u019C\u019D\u019E\u019F\u01A0\u01A1\u01A2\u01A3"+
	"\u01A4\u01A5\u01A6\u01A7\u01A8\u01A9\u01AA\u01AB"+
	"\u01AC\u01AD\u01AE\u01AF\u01B0\u01B1\u01B2\u01B3"+
	"\u01B4\u01B5\u01B6\u01B7\u01B8\u01B9\u01BA\u01BB"+
	"\u01BC\u01BD\u01BE\u01BF\u01C0\u01C1\u01C2\u01C3"+
	"\u01C4\u01C5\u01C6\u01C7\u01C8\u01C9\u01CA\u01CB"+
	"\u01CC\u01CD\u01CF\u01D1\u01D3\u01D5\u01D7\u01D9"+
	"\u01DB\u01DD\u01DE\u01DF\u01E0\u01E1\u01E2\u01E3"+
	"\u01E4\u01E5\u01E6\u01E7\u01E8\u01E9\u01EA\u01EB"+
	"\u01EC\u01ED\u01EE\u01EF\u01F0\u01F1\u01F2\u01F3"+
	"\u01F4\u01F5\u01F6\u01F7\u01F8\u01FA\u01FB\u01FC"+
	"\u01FD\u01FE\u01FF\u0200\u0201\u0202\u0203\u0204"+
	"\u0205\u0206\u0207\u0208\u0209\u020A\u020B\u020C"+
	"\u020D\u020E\u020F\u0210\u0211\u0212\u0213\u0214"+
	"\u0215\u0216\u0217\u0218\u0219\u021A\u021B\u021C"+
	"\u021D\u021E\u021F\u0220\u0221\u0222\u0223\u0224"+
	"\u0225\u0226\u0227\u0228\u0229\u022A\u022B\u022C"+
	"\u022D\u022E\u022F\u0230\u0231\u0232\u0233\u0234"+
	"\u0235\u0236\u0237\u0238\u0239\u023A\u023B\u023C"+
	"\u023D\u023E\u023F\u0240\u0241\u0242\u0243\u0244"+
	"\u0245\u0246\u0247\u0248\u0249\u024A\u024B\u024C"+
	"\u024D\u024E\u024F\u0250\u0252\u0253\u0254\u0255"+
	"\u0256\u0257\u0258\u0259\u025A\u025B\u025C\u025D"+
	"\u025E\u025F\u0260\u0262\u0263\u0264\u0265\u0266"+
	"\u0267\u0268\u0269\u026A\u026B\u026C\u026D\u026E"+
	"\u026F\u0270\u0271\u0272\u0273\u0274\u0275\u0276"+
	"\u0277\u0278\u0279\u027A\u027B\u027C\u027D\u027E"+
	"\u027F\u0280\u0281\u0282\u0283\u0284\u0285\u0286"+
	"\u0287\u0288\u0289\u028A\u028B\u028C\u028D\u028E"+
	"\u028F\u0290\u0291\u0292\u0293\u0294\u0295\u0296"+
	"\u0297\u0298\u0299\u029A\u029B\u029C\u029D\u029E"+
	"\u029F\u02A0\u02A1\u02A2\u02A3\u02A4\u02A5\u02A6"+
	"\u02A7\u02A8\u02A9\u02AA\u02AB\u02AC\u02AD\u02AE"+
	"\u02AF\u02B0\u02B1\u02B2\u02B3\u02B4\u02B5\u02B6"+
	"\u02B7\u02B8\u02B9\u02BA\u02BB\u02BC\u02BD\u02BE"+
	"\u02BF\u02C0\u02C1\u02C2\u02C3\u02C4\u02C5\u02C6"+
	"\u02C8\u02CC\u02CD\u02CE\u02CF\u02D0\u02D1\u02D2"+
	"\u02D3\u02D4\u02D5\u02D6\u02D7\u02D8\u02DA\u02DB"+
	"\u02DC\u02DD\u02DE\u02DF\u02E0\u02E1\u02E2\u02E3"+
	"\u02E4\u02E5\u02E6\u02E7\u02E8\u02E9\u02EA\u02EB"+
	"\u02EC\u02ED\u02EE\u02EF\u02F0\u02F1\u02F2\u02F3"+
	"\u02F4\u02F5\u02F6\u02F7\u02F8\u02F9\u02FA\u02FB"+
	"\u02FC\u02FD\u02FE\u02FF\u0300\u0301\u0302\u0303"+
	"\u0304\u0305\u0306\u0307\u0308\u0309\u030A\u030B"+
	"\u030C\u030D\u030E\u030F\u0310\u0311\u0312\u0313"+
	"\u0314\u0315\u0316\u0317\u0318\u0319\u031A\u031B"+
	"\u031C\u031D\u031E\u031F\u0320\u0321\u0322\u0323"+
	"\u0324\u0325\u0326\u0327\u0328\u0329\u032A\u032B"+
	"\u032C\u032D\u032E\u032F\u0330\u0331\u0332\u0333"+
	"\u0334\u0335\u0336\u0337\u0338\u0339\u033A\u033B"+
	"\u033C\u033D\u033E\u033F\u0340\u0341\u0342\u0343"+
	"\u0344\u0345\u0346\u0347\u0348\u0349\u034A\u034B"+
	"\u034C\u034D\u034E\u034F\u0350\u0351\u0352\u0353"+
	"\u0354\u0355\u0356\u0357\u0358\u0359\u035A\u035B"+
	"\u035C\u035D\u035E\u035F\u0360\u0361\u0362\u0363"+
	"\u0364\u0365\u0366\u0367\u0368\u0369\u036A\u036B"+
	"\u036C\u036D\u036E\u036F\u0370\u0371\u0372\u0373"+
	"\u0374\u0375\u0376\u0377\u0378\u0379\u037A\u037B"+
	"\u037C\u037D\u037E\u037F\u0380\u0381\u0382\u0383"+
	"\u0384\u0385\u0386\u0387\u0388\u0389\u038A\u038B"+
	"\u038C\u038D\u038E\u038F\u0390\u03A2\u03AA\u03AB"+
	"\u03AC\u03AD\u03AE\u03AF\u03B0\u03C2\u03CA\u03CB"+
	"\u03CC\u03CD\u03CE\u03CF\u03D0\u03D1\u03D2\u03D3"+
	"\u03D4\u03D5\u03D6\u03D7\u03D8\u03D9\u03DA\u03DB"+
	"\u03DC\u03DD\u03DE\u03DF\u03E0\u03E1\u03E2\u03E3"+
	"\u03E4\u03E5\u03E6\u03E7\u03E8\u03E9\u03EA\u03EB"+
	"\u03EC\u03ED\u03EE\u03EF\u03F0\u03F1\u03F2\u03F3"+
	"\u03F4\u03F5\u03F6\u03F7\u03F8\u03F9\u03FA\u03FB"+
	"\u03FC\u03FD\u03FE\u03FF\u0400\u0402\u0403\u0404"+
	"\u0405\u0406\u0407\u0408\u0409\u040A\u040B\u040C"+
	"\u040D\u040E\u040F\u0450\u0452\u0453\u0454\u0455"+
	"\u0456\u0457\u0458\u0459\u045A\u045B\u045C\u045D"+
	"\u045E\u045F\u0460\u0461\u0462\u0463\u0464\u0465"+
	"\u0466\u0467\u0468\u0469\u046A\u046B\u046C\u046D"+
	"\u046E\u046F\u0470\u0471\u0472\u0473\u0474\u0475"+
	"\u0476\u0477\u0478\u0479\u047A\u047B\u047C\u047D"+
	"\u047E\u047F\u0480\u0481\u0482\u0483\u0484\u0485"+
	"\u0486\u0487\u0488\u0489\u048A\u048B\u048C\u048D"+
	"\u048E\u048F\u0490\u0491\u0492\u0493\u0494\u0495"+
	"\u0496\u0497\u0498\u0499\u049A\u049B\u049C\u049D"+
	"\u049E\u049F\u04A0\u04A1\u04A2\u04A3\u04A4\u04A5"+
	"\u04A6\u04A7\u04A8\u04A9\u04AA\u04AB\u04AC\u04AD"+
	"\u04AE\u04AF\u04B0\u04B1\u04B2\u04B3\u04B4\u04B5"+
	"\u04B6\u04B7\u04B8\u04B9\u04BA\u04BB\u04BC\u04BD"+
	"\u04BE\u04BF\u04C0\u04C1\u04C2\u04C3\u04C4\u04C5"+
	"\u04C6\u04C7\u04C8\u04C9\u04CA\u04CB\u04CC\u04CD"+
	"\u04CE\u04CF\u04D0\u04D1\u04D2\u04D3\u04D4\u04D5"+
	"\u04D6\u04D7\u04D8\u04D9\u04DA\u04DB\u04DC\u04DD"+
	"\u04DE\u04DF\u04E0\u04E1\u04E2\u04E3\u04E4\u04E5"+
	"\u04E6\u04E7\u04E8\u04E9\u04EA\u04EB\u04EC\u04ED"+
	"\u04EE\u04EF\u04F0\u04F1\u04F2\u04F3\u04F4\u04F5"+
	"\u04F6\u04F7\u04F8\u04F9\u04FA\u04FB\u04FC\u04FD"+
	"\u04FE\u04FF\u0500\u0501\u0502\u0503\u0504\u0505"+
	"\u0506\u0507\u0508\u0509\u050A\u050B\u050C\u050D"+
	"\u050E\u050F\u0510\u0511\u0512\u0513\u0514\u0515"+
	"\u0516\u0517\u0518\u0519\u051A\u051B\u051C\u051D"+
	"\u051E\u051F\u0520\u0521\u0522\u0523\u0524\u0525"+
	"\u0526\u0527\u0528\u0529\u052A\u052B\u052C\u052D"+
	"\u052E\u052F\u0530\u0531\u0532\u0533\u0534\u0535"+
	"\u0536\u0537\u0538\u0539\u053A\u053B\u053C\u053D"+
	"\u053E\u053F\u0540\u0541\u0542\u0543\u0544\u0545"+
	"\u0546\u0547\u0548\u0549\u054A\u054B\u054C\u054D"+
	"\u054E\u054F\u0550\u0551\u0552\u0553\u0554\u0555"+
	"\u0556\u0557\u0558\u0559\u055A\u055B\u055C\u055D"+
	"\u055E\u055F\u0560\u0561\u0562\u0563\u0564\u0565"+
	"\u0566\u0567\u0568\u0569\u056A\u056B\u056C\u056D"+
	"\u056E\u056F\u0570\u0571\u0572\u0573\u0574\u0575"+
	"\u0576\u0577\u0578\u0579\u057A\u057B\u057C\u057D"+
	"\u057E\u057F\u0580\u0581\u0582\u0583\u0584\u0585"+
	"\u0586\u0587\u0588\u0589\u058A\u058B\u058C\u058D"+
	"\u058E\u058F\u0590\u0591\u0592\u0593\u0594\u0595"+
	"\u0596\u0597\u0598\u0599\u059A\u059B\u059C\u059D"+
	"\u059E\u059F\u05A0\u05A1\u05A2\u05A3\u05A4\u05A5"+
	"\u05A6\u05A7\u05A8\u05A9\u05AA\u05AB\u05AC\u05AD"+
	"\u05AE\u05AF\u05B0\u05B1\u05B2\u05B3\u05B4\u05B5"+
	"\u05B6\u05B7\u05B8\u05B9\u05BA\u05BB\u05BC\u05BD"+
	"\u05BE\u05BF\u05C0\u05C1\u05C2\u05C3\u05C4\u05C5"+
	"\u05C6\u05C7\u05C8\u05C9\u05CA\u05CB\u05CC\u05CD"+
	"\u05CE\u05CF\u05D0\u05D1\u05D2\u05D3\u05D4\u05D5"+
	"\u05D6\u05D7\u05D8\u05D9\u05DA\u05DB\u05DC\u05DD"+
	"\u05DE\u05DF\u05E0\u05E1\u05E2\u05E3\u05E4\u05E5"+
	"\u05E6\u05E7\u05E8\u05E9\u05EA\u05EB\u05EC\u05ED"+
	"\u05EE\u05EF\u05F0\u05F1\u05F2\u05F3\u05F4\u05F5"+
	"\u05F6\u05F7\u05F8\u05F9\u05FA\u05FB\u05FC\u05FD"+
	"\u05FE\u05FF\u0600\u0601\u0602\u0603\u0604\u0605"+
	"\u0606\u0607\u0608\u0609\u060A\u060B\u060C\u060D"+
	"\u060E\u060F\u0610\u0611\u0612\u0613\u0614\u0615"+
	"\u0616\u0617\u0618\u0619\u061A\u061B\u061C\u061D"+
	"\u061E\u061F\u0620\u0621\u0622\u0623\u0624\u0625"+
	"\u0626\u0627\u0628\u0629\u062A\u062B\u062C\u062D"+
	"\u062E\u062F\u0630\u0631\u0632\u0633\u0634\u0635"+
	"\u0636\u0637\u0638\u0639\u063A\u063B\u063C\u063D"+
	"\u063E\u063F\u0640\u0641\u0642\u0643\u0644\u0645"+
	"\u0646\u0647\u0648\u0649\u064A\u064B\u064C\u064D"+
	"\u064E\u064F\u0650\u0651\u0652\u0653\u0654\u0655"+
	"\u0656\u0657\u0658\u0659\u065A\u065B\u065C\u065D"+
	"\u065E\u065F\u0660\u0661\u0662\u0663\u0664\u0665"+
	"\u0666\u0667\u0668\u0669\u066A\u066B\u066C\u066D"+
	"\u066E\u066F\u0670\u0671\u0672\u0673\u0674\u0675"+
	"\u0676\u0677\u0678\u0679\u067A\u067B\u067C\u067D"+
	"\u067E\u067F\u0680\u0681\u0682\u0683\u0684\u0685"+
	"\u0686\u0687\u0688\u0689\u068A\u068B\u068C\u068D"+
	"\u068E\u068F\u0690\u0691\u0692\u0693\u0694\u0695"+
	"\u0696\u0697\u0698\u0699\u069A\u069B\u069C\u069D"+
	"\u069E\u069F\u06A0\u06A1\u06A2\u06A3\u06A4\u06A5"+
	"\u06A6\u06A7\u06A8\u06A9\u06AA\u06AB\u06AC\u06AD"+
	"\u06AE\u06AF\u06B0\u06B1\u06B2\u06B3\u06B4\u06B5"+
	"\u06B6\u06B7\u06B8\u06B9\u06BA\u06BB\u06BC\u06BD"+
	"\u06BE\u06BF\u06C0\u06C1\u06C2\u06C3\u06C4\u06C5"+
	"\u06C6\u06C7\u06C8\u06C9\u06CA\u06CB\u06CC\u06CD"+
	"\u06CE\u06CF\u06D0\u06D1\u06D2\u06D3\u06D4\u06D5"+
	"\u06D6\u06D7\u06D8\u06D9\u06DA\u06DB\u06DC\u06DD"+
	"\u06DE\u06DF\u06E0\u06E1\u06E2\u06E3\u06E4\u06E5"+
	"\u06E6\u06E7\u06E8\u06E9\u06EA\u06EB\u06EC\u06ED"+
	"\u06EE\u06EF\u06F0\u06F1\u06F2\u06F3\u06F4\u06F5"+
	"\u06F6\u06F7\u06F8\u06F9\u06FA\u06FB\u06FC\u06FD"+
	"\u06FE\u06FF\u0700\u0701\u0702\u0703\u0704\u0705"+
	"\u0706\u0707\u0708\u0709\u070A\u070B\u070C\u070D"+
	"\u070E\u070F\u0710\u0711\u0712\u0713\u0714\u0715"+
	"\u0716\u0717\u0718\u0719\u071A\u071B\u071C\u071D"+
	"\u071E\u071F\u0720\u0721\u0722\u0723\u0724\u0725"+
	"\u0726\u0727\u0728\u0729\u072A\u072B\u072C\u072D"+
	"\u072E\u072F\u0730\u0731\u0732\u0733\u0734\u0735"+
	"\u0736\u0737\u0738\u0739\u073A\u073B\u073C\u073D"+
	"\u073E\u073F\u0740\u0741\u0742\u0743\u0744\u0745"+
	"\u0746\u0747\u0748\u0749\u074A\u074B\u074C\u074D"+
	"\u074E\u074F\u0750\u0751\u0752\u0753\u0754\u0755"+
	"\u0756\u0757\u0758\u0759\u075A\u075B\u075C\u075D"+
	"\u075E\u075F\u0760\u0761\u0762\u0763\u0764\u0765"+
	"\u0766\u0767\u0768\u0769\u076A\u076B\u076C\u076D"+
	"\u076E\u076F\u0770\u0771\u0772\u0773\u0774\u0775"+
	"\u0776\u0777\u0778\u0779\u077A\u077B\u077C\u077D"+
	"\u077E\u077F\u0780\u0781\u0782\u0783\u0784\u0785"+
	"\u0786\u0787\u0788\u0789\u078A\u078B\u078C\u078D"+
	"\u078E\u078F\u0790\u0791\u0792\u0793\u0794\u0795"+
	"\u0796\u0797\u0798\u0799\u079A\u079B\u079C\u079D"+
	"\u079E\u079F\u07A0\u07A1\u07A2\u07A3\u07A4\u07A5"+
	"\u07A6\u07A7\u07A8\u07A9\u07AA\u07AB\u07AC\u07AD"+
	"\u07AE\u07AF\u07B0\u07B1\u07B2\u07B3\u07B4\u07B5"+
	"\u07B6\u07B7\u07B8\u07B9\u07BA\u07BB\u07BC\u07BD"+
	"\u07BE\u07BF\u07C0\u07C1\u07C2\u07C3\u07C4\u07C5"+
	"\u07C6\u07C7\u07C8\u07C9\u07CA\u07CB\u07CC\u07CD"+
	"\u07CE\u07CF\u07D0\u07D1\u07D2\u07D3\u07D4\u07D5"+
	"\u07D6\u07D7\u07D8\u07D9\u07DA\u07DB\u07DC\u07DD"+
	"\u07DE\u07DF\u07E0\u07E1\u07E2\u07E3\u07E4\u07E5"+
	"\u07E6\u07E7\u07E8\u07E9\u07EA\u07EB\u07EC\u07ED"+
	"\u07EE\u07EF\u07F0\u07F1\u07F2\u07F3\u07F4\u07F5"+
	"\u07F6\u07F7\u07F8\u07F9\u07FA\u07FB\u07FC\u07FD"+
	"\u07FE\u07FF\u0800\u0801\u0802\u0803\u0804\u0805"+
	"\u0806\u0807\u0808\u0809\u080A\u080B\u080C\u080D"+
	"\u080E\u080F\u0810\u0811\u0812\u0813\u0814\u0815"+
	"\u0816\u0817\u0818\u0819\u081A\u081B\u081C\u081D"+
	"\u081E\u081F\u0820\u0821\u0822\u0823\u0824\u0825"+
	"\u0826\u0827\u0828\u0829\u082A\u082B\u082C\u082D"+
	"\u082E\u082F\u0830\u0831\u0832\u0833\u0834\u0835"+
	"\u0836\u0837\u0838\u0839\u083A\u083B\u083C\u083D"+
	"\u083E\u083F\u0840\u0841\u0842\u0843\u0844\u0845"+
	"\u0846\u0847\u0848\u0849\u084A\u084B\u084C\u084D"+
	"\u084E\u084F\u0850\u0851\u0852\u0853\u0854\u0855"+
	"\u0856\u0857\u0858\u0859\u085A\u085B\u085C\u085D"+
	"\u085E\u085F\u0860\u0861\u0862\u0863\u0864\u0865"+
	"\u0866\u0867\u0868\u0869\u086A\u086B\u086C\u086D"+
	"\u086E\u086F\u0870\u0871\u0872\u0873\u0874\u0875"+
	"\u0876\u0877\u0878\u0879\u087A\u087B\u087C\u087D"+
	"\u087E\u087F\u0880\u0881\u0882\u0883\u0884\u0885"+
	"\u0886\u0887\u0888\u0889\u088A\u088B\u088C\u088D"+
	"\u088E\u088F\u0890\u0891\u0892\u0893\u0894\u0895"+
	"\u0896\u0897\u0898\u0899\u089A\u089B\u089C\u089D"+
	"\u089E\u089F\u08A0\u08A1\u08A2\u08A3\u08A4\u08A5"+
	"\u08A6\u08A7\u08A8\u08A9\u08AA\u08AB\u08AC\u08AD"+
	"\u08AE\u08AF\u08B0\u08B1\u08B2\u08B3\u08B4\u08B5"+
	"\u08B6\u08B7\u08B8\u08B9\u08BA\u08BB\u08BC\u08BD"+
	"\u08BE\u08BF\u08C0\u08C1\u08C2\u08C3\u08C4\u08C5"+
	"\u08C6\u08C7\u08C8\u08C9\u08CA\u08CB\u08CC\u08CD"+
	"\u08CE\u08CF\u08D0\u08D1\u08D2\u08D3\u08D4\u08D5"+
	"\u08D6\u08D7\u08D8\u08D9\u08DA\u08DB\u08DC\u08DD"+
	"\u08DE\u08DF\u08E0\u08E1\u08E2\u08E3\u08E4\u08E5"+
	"\u08E6\u08E7\u08E8\u08E9\u08EA\u08EB\u08EC\u08ED"+
	"\u08EE\u08EF\u08F0\u08F1\u08F2\u08F3\u08F4\u08F5"+
	"\u08F6\u08F7\u08F8\u08F9\u08FA\u08FB\u08FC\u08FD"+
	"\u08FE\u08FF\u0900\u0901\u0902\u0903\u0904\u0905"+
	"\u0906\u0907\u0908\u0909\u090A\u090B\u090C\u090D"+
	"\u090E\u090F\u0910\u0911\u0912\u0913\u0914\u0915"+
	"\u0916\u0917\u0918\u0919\u091A\u091B\u091C\u091D"+
	"\u091E\u091F\u0920\u0921\u0922\u0923\u0924\u0925"+
	"\u0926\u0927\u0928\u0929\u092A\u092B\u092C\u092D"+
	"\u092E\u092F\u0930\u0931\u0932\u0933\u0934\u0935"+
	"\u0936\u0937\u0938\u0939\u093A\u093B\u093C\u093D"+
	"\u093E\u093F\u0940\u0941\u0942\u0943\u0944\u0945"+
	"\u0946\u0947\u0948\u0949\u094A\u094B\u094C\u094D"+
	"\u094E\u094F\u0950\u0951\u0952\u0953\u0954\u0955"+
	"\u0956\u0957\u0958\u0959\u095A\u095B\u095C\u095D"+
	"\u095E\u095F\u0960\u0961\u0962\u0963\u0964\u0965"+
	"\u0966\u0967\u0968\u0969\u096A\u096B\u096C\u096D"+
	"\u096E\u096F\u0970\u0971\u0972\u0973\u0974\u0975"+
	"\u0976\u0977\u0978\u0979\u097A\u097B\u097C\u097D"+
	"\u097E\u097F\u0980\u0981\u0982\u0983\u0984\u0985"+
	"\u0986\u0987\u0988\u0989\u098A\u098B\u098C\u098D"+
	"\u098E\u098F\u0990\u0991\u0992\u0993\u0994\u0995"+
	"\u0996\u0997\u0998\u0999\u099A\u099B\u099C\u099D"+
	"\u099E\u099F\u09A0\u09A1\u09A2\u09A3\u09A4\u09A5"+
	"\u09A6\u09A7\u09A8\u09A9\u09AA\u09AB\u09AC\u09AD"+
	"\u09AE\u09AF\u09B0\u09B1\u09B2\u09B3\u09B4\u09B5"+
	"\u09B6\u09B7\u09B8\u09B9\u09BA\u09BB\u09BC\u09BD"+
	"\u09BE\u09BF\u09C0\u09C1\u09C2\u09C3\u09C4\u09C5"+
	"\u09C6\u09C7\u09C8\u09C9\u09CA\u09CB\u09CC\u09CD"+
	"\u09CE\u09CF\u09D0\u09D1\u09D2\u09D3\u09D4\u09D5"+
	"\u09D6\u09D7\u09D8\u09D9\u09DA\u09DB\u09DC\u09DD"+
	"\u09DE\u09DF\u09E0\u09E1\u09E2\u09E3\u09E4\u09E5"+
	"\u09E6\u09E7\u09E8\u09E9\u09EA\u09EB\u09EC\u09ED"+
	"\u09EE\u09EF\u09F0\u09F1\u09F2\u09F3\u09F4\u09F5"+
	"\u09F6\u09F7\u09F8\u09F9\u09FA\u09FB\u09FC\u09FD"+
	"\u09FE\u09FF\u0A00\u0A01\u0A02\u0A03\u0A04\u0A05"+
	"\u0A06\u0A07\u0A08\u0A09\u0A0A\u0A0B\u0A0C\u0A0D"+
	"\u0A0E\u0A0F\u0A10\u0A11\u0A12\u0A13\u0A14\u0A15"+
	"\u0A16\u0A17\u0A18\u0A19\u0A1A\u0A1B\u0A1C\u0A1D"+
	"\u0A1E\u0A1F\u0A20\u0A21\u0A22\u0A23\u0A24\u0A25"+
	"\u0A26\u0A27\u0A28\u0A29\u0A2A\u0A2B\u0A2C\u0A2D"+
	"\u0A2E\u0A2F\u0A30\u0A31\u0A32\u0A33\u0A34\u0A35"+
	"\u0A36\u0A37\u0A38\u0A39\u0A3A\u0A3B\u0A3C\u0A3D"+
	"\u0A3E\u0A3F\u0A40\u0A41\u0A42\u0A43\u0A44\u0A45"+
	"\u0A46\u0A47\u0A48\u0A49\u0A4A\u0A4B\u0A4C\u0A4D"+
	"\u0A4E\u0A4F\u0A50\u0A51\u0A52\u0A53\u0A54\u0A55"+
	"\u0A56\u0A57\u0A58\u0A59\u0A5A\u0A5B\u0A5C\u0A5D"+
	"\u0A5E\u0A5F\u0A60\u0A61\u0A62\u0A63\u0A64\u0A65"+
	"\u0A66\u0A67\u0A68\u0A69\u0A6A\u0A6B\u0A6C\u0A6D"+
	"\u0A6E\u0A6F\u0A70\u0A71\u0A72\u0A73\u0A74\u0A75"+
	"\u0A76\u0A77\u0A78\u0A79\u0A7A\u0A7B\u0A7C\u0A7D"+
	"\u0A7E\u0A7F\u0A80\u0A81\u0A82\u0A83\u0A84\u0A85"+
	"\u0A86\u0A87\u0A88\u0A89\u0A8A\u0A8B\u0A8C\u0A8D"+
	"\u0A8E\u0A8F\u0A90\u0A91\u0A92\u0A93\u0A94\u0A95"+
	"\u0A96\u0A97\u0A98\u0A99\u0A9A\u0A9B\u0A9C\u0A9D"+
	"\u0A9E\u0A9F\u0AA0\u0AA1\u0AA2\u0AA3\u0AA4\u0AA5"+
	"\u0AA6\u0AA7\u0AA8\u0AA9\u0AAA\u0AAB\u0AAC\u0AAD"+
	"\u0AAE\u0AAF\u0AB0\u0AB1\u0AB2\u0AB3\u0AB4\u0AB5"+
	"\u0AB6\u0AB7\u0AB8\u0AB9\u0ABA\u0ABB\u0ABC\u0ABD"+
	"\u0ABE\u0ABF\u0AC0\u0AC1\u0AC2\u0AC3\u0AC4\u0AC5"+
	"\u0AC6\u0AC7\u0AC8\u0AC9\u0ACA\u0ACB\u0ACC\u0ACD"+
	"\u0ACE\u0ACF\u0AD0\u0AD1\u0AD2\u0AD3\u0AD4\u0AD5"+
	"\u0AD6\u0AD7\u0AD8\u0AD9\u0ADA\u0ADB\u0ADC\u0ADD"+
	"\u0ADE\u0ADF\u0AE0\u0AE1\u0AE2\u0AE3\u0AE4\u0AE5"+
	"\u0AE6\u0AE7\u0AE8\u0AE9\u0AEA\u0AEB\u0AEC\u0AED"+
	"\u0AEE\u0AEF\u0AF0\u0AF1\u0AF2\u0AF3\u0AF4\u0AF5"+
	"\u0AF6\u0AF7\u0AF8\u0AF9\u0AFA\u0AFB\u0AFC\u0AFD"+
	"\u0AFE\u0AFF\u0B00\u0B01\u0B02\u0B03\u0B04\u0B05"+
	"\u0B06\u0B07\u0B08\u0B09\u0B0A\u0B0B\u0B0C\u0B0D"+
	"\u0B0E\u0B0F\u0B10\u0B11\u0B12\u0B13\u0B14\u0B15"+
	"\u0B16\u0B17\u0B18\u0B19\u0B1A\u0B1B\u0B1C\u0B1D"+
	"\u0B1E\u0B1F\u0B20\u0B21\u0B22\u0B23\u0B24\u0B25"+
	"\u0B26\u0B27\u0B28\u0B29\u0B2A\u0B2B\u0B2C\u0B2D"+
	"\u0B2E\u0B2F\u0B30\u0B31\u0B32\u0B33\u0B34\u0B35"+
	"\u0B36\u0B37\u0B38\u0B39\u0B3A\u0B3B\u0B3C\u0B3D"+
	"\u0B3E\u0B3F\u0B40\u0B41\u0B42\u0B43\u0B44\u0B45"+
	"\u0B46\u0B47\u0B48\u0B49\u0B4A\u0B4B\u0B4C\u0B4D"+
	"\u0B4E\u0B4F\u0B50\u0B51\u0B52\u0B53\u0B54\u0B55"+
	"\u0B56\u0B57\u0B58\u0B59\u0B5A\u0B5B\u0B5C\u0B5D"+
	"\u0B5E\u0B5F\u0B60\u0B61\u0B62\u0B63\u0B64\u0B65"+
	"\u0B66\u0B67\u0B68\u0B69\u0B6A\u0B6B\u0B6C\u0B6D"+
	"\u0B6E\u0B6F\u0B70\u0B71\u0B72\u0B73\u0B74\u0B75"+
	"\u0B76\u0B77\u0B78\u0B79\u0B7A\u0B7B\u0B7C\u0B7D"+
	"\u0B7E\u0B7F\u0B80\u0B81\u0B82\u0B83\u0B84\u0B85"+
	"\u0B86\u0B87\u0B88\u0B89\u0B8A\u0B8B\u0B8C\u0B8D"+
	"\u0B8E\u0B8F\u0B90\u0B91\u0B92\u0B93\u0B94\u0B95"+
	"\u0B96\u0B97\u0B98\u0B99\u0B9A\u0B9B\u0B9C\u0B9D"+
	"\u0B9E\u0B9F\u0BA0\u0BA1\u0BA2\u0BA3\u0BA4\u0BA5"+
	"\u0BA6\u0BA7\u0BA8\u0BA9\u0BAA\u0BAB\u0BAC\u0BAD"+
	"\u0BAE\u0BAF\u0BB0\u0BB1\u0BB2\u0BB3\u0BB4\u0BB5"+
	"\u0BB6\u0BB7\u0BB8\u0BB9\u0BBA\u0BBB\u0BBC\u0BBD"+
	"\u0BBE\u0BBF\u0BC0\u0BC1\u0BC2\u0BC3\u0BC4\u0BC5"+
	"\u0BC6\u0BC7\u0BC8\u0BC9\u0BCA\u0BCB\u0BCC\u0BCD"+
	"\u0BCE\u0BCF\u0BD0\u0BD1\u0BD2\u0BD3\u0BD4\u0BD5"+
	"\u0BD6\u0BD7\u0BD8\u0BD9\u0BDA\u0BDB\u0BDC\u0BDD"+
	"\u0BDE\u0BDF\u0BE0\u0BE1\u0BE2\u0BE3\u0BE4\u0BE5"+
	"\u0BE6\u0BE7\u0BE8\u0BE9\u0BEA\u0BEB\u0BEC\u0BED"+
	"\u0BEE\u0BEF\u0BF0\u0BF1\u0BF2\u0BF3\u0BF4\u0BF5"+
	"\u0BF6\u0BF7\u0BF8\u0BF9\u0BFA\u0BFB\u0BFC\u0BFD"+
	"\u0BFE\u0BFF\u0C00\u0C01\u0C02\u0C03\u0C04\u0C05"+
	"\u0C06\u0C07\u0C08\u0C09\u0C0A\u0C0B\u0C0C\u0C0D"+
	"\u0C0E\u0C0F\u0C10\u0C11\u0C12\u0C13\u0C14\u0C15"+
	"\u0C16\u0C17\u0C18\u0C19\u0C1A\u0C1B\u0C1C\u0C1D"+
	"\u0C1E\u0C1F\u0C20\u0C21\u0C22\u0C23\u0C24\u0C25"+
	"\u0C26\u0C27\u0C28\u0C29\u0C2A\u0C2B\u0C2C\u0C2D"+
	"\u0C2E\u0C2F\u0C30\u0C31\u0C32\u0C33\u0C34\u0C35"+
	"\u0C36\u0C37\u0C38\u0C39\u0C3A\u0C3B\u0C3C\u0C3D"+
	"\u0C3E\u0C3F\u0C40\u0C41\u0C42\u0C43\u0C44\u0C45"+
	"\u0C46\u0C47\u0C48\u0C49\u0C4A\u0C4B\u0C4C\u0C4D"+
	"\u0C4E\u0C4F\u0C50\u0C51\u0C52\u0C53\u0C54\u0C55"+
	"\u0C56\u0C57\u0C58\u0C59\u0C5A\u0C5B\u0C5C\u0C5D"+
	"\u0C5E\u0C5F\u0C60\u0C61\u0C62\u0C63\u0C64\u0C65"+
	"\u0C66\u0C67\u0C68\u0C69\u0C6A\u0C6B\u0C6C\u0C6D"+
	"\u0C6E\u0C6F\u0C70\u0C71\u0C72\u0C73\u0C74\u0C75"+
	"\u0C76\u0C77\u0C78\u0C79\u0C7A\u0C7B\u0C7C\u0C7D"+
	"\u0C7E\u0C7F\u0C80\u0C81\u0C82\u0C83\u0C84\u0C85"+
	"\u0C86\u0C87\u0C88\u0C89\u0C8A\u0C8B\u0C8C\u0C8D"+
	"\u0C8E\u0C8F\u0C90\u0C91\u0C92\u0C93\u0C94\u0C95"+
	"\u0C96\u0C97\u0C98\u0C99\u0C9A\u0C9B\u0C9C\u0C9D"+
	"\u0C9E\u0C9F\u0CA0\u0CA1\u0CA2\u0CA3\u0CA4\u0CA5"+
	"\u0CA6\u0CA7\u0CA8\u0CA9\u0CAA\u0CAB\u0CAC\u0CAD"+
	"\u0CAE\u0CAF\u0CB0\u0CB1\u0CB2\u0CB3\u0CB4\u0CB5"+
	"\u0CB6\u0CB7\u0CB8\u0CB9\u0CBA\u0CBB\u0CBC\u0CBD"+
	"\u0CBE\u0CBF\u0CC0\u0CC1\u0CC2\u0CC3\u0CC4\u0CC5"+
	"\u0CC6\u0CC7\u0CC8\u0CC9\u0CCA\u0CCB\u0CCC\u0CCD"+
	"\u0CCE\u0CCF\u0CD0\u0CD1\u0CD2\u0CD3\u0CD4\u0CD5"+
	"\u0CD6\u0CD7\u0CD8\u0CD9\u0CDA\u0CDB\u0CDC\u0CDD"+
	"\u0CDE\u0CDF\u0CE0\u0CE1\u0CE2\u0CE3\u0CE4\u0CE5"+
	"\u0CE6\u0CE7\u0CE8\u0CE9\u0CEA\u0CEB\u0CEC\u0CED"+
	"\u0CEE\u0CEF\u0CF0\u0CF1\u0CF2\u0CF3\u0CF4\u0CF5"+
	"\u0CF6\u0CF7\u0CF8\u0CF9\u0CFA\u0CFB\u0CFC\u0CFD"+
	"\u0CFE\u0CFF\u0D00\u0D01\u0D02\u0D03\u0D04\u0D05"+
	"\u0D06\u0D07\u0D08\u0D09\u0D0A\u0D0B\u0D0C\u0D0D"+
	"\u0D0E\u0D0F\u0D10\u0D11\u0D12\u0D13\u0D14\u0D15"+
	"\u0D16\u0D17\u0D18\u0D19\u0D1A\u0D1B\u0D1C\u0D1D"+
	"\u0D1E\u0D1F\u0D20\u0D21\u0D22\u0D23\u0D24\u0D25"+
	"\u0D26\u0D27\u0D28\u0D29\u0D2A\u0D2B\u0D2C\u0D2D"+
	"\u0D2E\u0D2F\u0D30\u0D31\u0D32\u0D33\u0D34\u0D35"+
	"\u0D36\u0D37\u0D38\u0D39\u0D3A\u0D3B\u0D3C\u0D3D"+
	"\u0D3E\u0D3F\u0D40\u0D41\u0D42\u0D43\u0D44\u0D45"+
	"\u0D46\u0D47\u0D48\u0D49\u0D4A\u0D4B\u0D4C\u0D4D"+
	"\u0D4E\u0D4F\u0D50\u0D51\u0D52\u0D53\u0D54\u0D55"+
	"\u0D56\u0D57\u0D58\u0D59\u0D5A\u0D5B\u0D5C\u0D5D"+
	"\u0D5E\u0D5F\u0D60\u0D61\u0D62\u0D63\u0D64\u0D65"+
	"\u0D66\u0D67\u0D68\u0D69\u0D6A\u0D6B\u0D6C\u0D6D"+
	"\u0D6E\u0D6F\u0D70\u0D71\u0D72\u0D73\u0D74\u0D75"+
	"\u0D76\u0D77\u0D78\u0D79\u0D7A\u0D7B\u0D7C\u0D7D"+
	"\u0D7E\u0D7F\u0D80\u0D81\u0D82\u0D83\u0D84\u0D85"+
	"\u0D86\u0D87\u0D88\u0D89\u0D8A\u0D8B\u0D8C\u0D8D"+
	"\u0D8E\u0D8F\u0D90\u0D91\u0D92\u0D93\u0D94\u0D95"+
	"\u0D96\u0D97\u0D98\u0D99\u0D9A\u0D9B\u0D9C\u0D9D"+
	"\u0D9E\u0D9F\u0DA0\u0DA1\u0DA2\u0DA3\u0DA4\u0DA5"+
	"\u0DA6\u0DA7\u0DA8\u0DA9\u0DAA\u0DAB\u0DAC\u0DAD"+
	"\u0DAE\u0DAF\u0DB0\u0DB1\u0DB2\u0DB3\u0DB4\u0DB5"+
	"\u0DB6\u0DB7\u0DB8\u0DB9\u0DBA\u0DBB\u0DBC\u0DBD"+
	"\u0DBE\u0DBF\u0DC0\u0DC1\u0DC2\u0DC3\u0DC4\u0DC5"+
	"\u0DC6\u0DC7\u0DC8\u0DC9\u0DCA\u0DCB\u0DCC\u0DCD"+
	"\u0DCE\u0DCF\u0DD0\u0DD1\u0DD2\u0DD3\u0DD4\u0DD5"+
	"\u0DD6\u0DD7\u0DD8\u0DD9\u0DDA\u0DDB\u0DDC\u0DDD"+
	"\u0DDE\u0DDF\u0DE0\u0DE1\u0DE2\u0DE3\u0DE4\u0DE5"+
	"\u0DE6\u0DE7\u0DE8\u0DE9\u0DEA\u0DEB\u0DEC\u0DED"+
	"\u0DEE\u0DEF\u0DF0\u0DF1\u0DF2\u0DF3\u0DF4\u0DF5"+
	"\u0DF6\u0DF7\u0DF8\u0DF9\u0DFA\u0DFB\u0DFC\u0DFD"+
	"\u0DFE\u0DFF\u0E00\u0E01\u0E02\u0E03\u0E04\u0E05"+
	"\u0E06\u0E07\u0E08\u0E09\u0E0A\u0E0B\u0E0C\u0E0D"+
	"\u0E0E\u0E0F\u0E10\u0E11\u0E12\u0E13\u0E14\u0E15"+
	"\u0E16\u0E17\u0E18\u0E19\u0E1A\u0E1B\u0E1C\u0E1D"+
	"\u0E1E\u0E1F\u0E20\u0E21\u0E22\u0E23\u0E24\u0E25"+
	"\u0E26\u0E27\u0E28\u0E29\u0E2A\u0E2B\u0E2C\u0E2D"+
	"\u0E2E\u0E2F\u0E30\u0E31\u0E32\u0E33\u0E34\u0E35"+
	"\u0E36\u0E37\u0E38\u0E39\u0E3A\u0E3B\u0E3C\u0E3D"+
	"\u0E3E\u0E3F\u0E40\u0E41\u0E42\u0E43\u0E44\u0E45"+
	"\u0E46\u0E47\u0E48\u0E49\u0E4A\u0E4B\u0E4C\u0E4D"+
	"\u0E4E\u0E4F\u0E50\u0E51\u0E52\u0E53\u0E54\u0E55"+
	"\u0E56\u0E57\u0E58\u0E59\u0E5A\u0E5B\u0E5C\u0E5D"+
	"\u0E5E\u0E5F\u0E60\u0E61\u0E62\u0E63\u0E64\u0E65"+
	"\u0E66\u0E67\u0E68\u0E69\u0E6A\u0E6B\u0E6C\u0E6D"+
	"\u0E6E\u0E6F\u0E70\u0E71\u0E72\u0E73\u0E74\u0E75"+
	"\u0E76\u0E77\u0E78\u0E79\u0E7A\u0E7B\u0E7C\u0E7D"+
	"\u0E7E\u0E7F\u0E80\u0E81\u0E82\u0E83\u0E84\u0E85"+
	"\u0E86\u0E87\u0E88\u0E89\u0E8A\u0E8B\u0E8C\u0E8D"+
	"\u0E8E\u0E8F\u0E90\u0E91\u0E92\u0E93\u0E94\u0E95"+
	"\u0E96\u0E97\u0E98\u0E99\u0E9A\u0E9B\u0E9C\u0E9D"+
	"\u0E9E\u0E9F\u0EA0\u0EA1\u0EA2\u0EA3\u0EA4\u0EA5"+
	"\u0EA6\u0EA7\u0EA8\u0EA9\u0EAA\u0EAB\u0EAC\u0EAD"+
	"\u0EAE\u0EAF\u0EB0\u0EB1\u0EB2\u0EB3\u0EB4\u0EB5"+
	"\u0EB6\u0EB7\u0EB8\u0EB9\u0EBA\u0EBB\u0EBC\u0EBD"+
	"\u0EBE\u0EBF\u0EC0\u0EC1\u0EC2\u0EC3\u0EC4\u0EC5"+
	"\u0EC6\u0EC7\u0EC8\u0EC9\u0ECA\u0ECB\u0ECC\u0ECD"+
	"\u0ECE\u0ECF\u0ED0\u0ED1\u0ED2\u0ED3\u0ED4\u0ED5"+
	"\u0ED6\u0ED7\u0ED8\u0ED9\u0EDA\u0EDB\u0EDC\u0EDD"+
	"\u0EDE\u0EDF\u0EE0\u0EE1\u0EE2\u0EE3\u0EE4\u0EE5"+
	"\u0EE6\u0EE7\u0EE8\u0EE9\u0EEA\u0EEB\u0EEC\u0EED"+
	"\u0EEE\u0EEF\u0EF0\u0EF1\u0EF2\u0EF3\u0EF4\u0EF5"+
	"\u0EF6\u0EF7\u0EF8\u0EF9\u0EFA\u0EFB\u0EFC\u0EFD"+
	"\u0EFE\u0EFF\u0F00\u0F01\u0F02\u0F03\u0F04\u0F05"+
	"\u0F06\u0F07\u0F08\u0F09\u0F0A\u0F0B\u0F0C\u0F0D"+
	"\u0F0E\u0F0F\u0F10\u0F11\u0F12\u0F13\u0F14\u0F15"+
	"\u0F16\u0F17\u0F18\u0F19\u0F1A\u0F1B\u0F1C\u0F1D"+
	"\u0F1E\u0F1F\u0F20\u0F21\u0F22\u0F23\u0F24\u0F25"+
	"\u0F26\u0F27\u0F28\u0F29\u0F2A\u0F2B\u0F2C\u0F2D"+
	"\u0F2E\u0F2F\u0F30\u0F31\u0F32\u0F33\u0F34\u0F35"+
	"\u0F36\u0F37\u0F38\u0F39\u0F3A\u0F3B\u0F3C\u0F3D"+
	"\u0F3E\u0F3F\u0F40\u0F41\u0F42\u0F43\u0F44\u0F45"+
	"\u0F46\u0F47\u0F48\u0F49\u0F4A\u0F4B\u0F4C\u0F4D"+
	"\u0F4E\u0F4F\u0F50\u0F51\u0F52\u0F53\u0F54\u0F55"+
	"\u0F56\u0F57\u0F58\u0F59\u0F5A\u0F5B\u0F5C\u0F5D"+
	"\u0F5E\u0F5F\u0F60\u0F61\u0F62\u0F63\u0F64\u0F65"+
	"\u0F66\u0F67\u0F68\u0F69\u0F6A\u0F6B\u0F6C\u0F6D"+
	"\u0F6E\u0F6F\u0F70\u0F71\u0F72\u0F73\u0F74\u0F75"+
	"\u0F76\u0F77\u0F78\u0F79\u0F7A\u0F7B\u0F7C\u0F7D"+
	"\u0F7E\u0F7F\u0F80\u0F81\u0F82\u0F83\u0F84\u0F85"+
	"\u0F86\u0F87\u0F88\u0F89\u0F8A\u0F8B\u0F8C\u0F8D"+
	"\u0F8E\u0F8F\u0F90\u0F91\u0F92\u0F93\u0F94\u0F95"+
	"\u0F96\u0F97\u0F98\u0F99\u0F9A\u0F9B\u0F9C\u0F9D"+
	"\u0F9E\u0F9F\u0FA0\u0FA1\u0FA2\u0FA3\u0FA4\u0FA5"+
	"\u0FA6\u0FA7\u0FA8\u0FA9\u0FAA\u0FAB\u0FAC\u0FAD"+
	"\u0FAE\u0FAF\u0FB0\u0FB1\u0FB2\u0FB3\u0FB4\u0FB5"+
	"\u0FB6\u0FB7\u0FB8\u0FB9\u0FBA\u0FBB\u0FBC\u0FBD"+
	"\u0FBE\u0FBF\u0FC0\u0FC1\u0FC2\u0FC3\u0FC4\u0FC5"+
	"\u0FC6\u0FC7\u0FC8\u0FC9\u0FCA\u0FCB\u0FCC\u0FCD"+
	"\u0FCE\u0FCF\u0FD0\u0FD1\u0FD2\u0FD3\u0FD4\u0FD5"+
	"\u0FD6\u0FD7\u0FD8\u0FD9\u0FDA\u0FDB\u0FDC\u0FDD"+
	"\u0FDE\u0FDF\u0FE0\u0FE1\u0FE2\u0FE3\u0FE4\u0FE5"+
	"\u0FE6\u0FE7\u0FE8\u0FE9\u0FEA\u0FEB\u0FEC\u0FED"+
	"\u0FEE\u0FEF\u0FF0\u0FF1\u0FF2\u0FF3\u0FF4\u0FF5"+
	"\u0FF6\u0FF7\u0FF8\u0FF9\u0FFA\u0FFB\u0FFC\u0FFD"+
	"\u0FFE\u0FFF\u1000\u1001\u1002\u1003\u1004\u1005"+
	"\u1006\u1007\u1008\u1009\u100A\u100B\u100C\u100D"+
	"\u100E\u100F\u1010\u1011\u1012\u1013\u1014\u1015"+
	"\u1016\u1017\u1018\u1019\u101A\u101B\u101C\u101D"+
	"\u101E\u101F\u1020\u1021\u1022\u1023\u1024\u1025"+
	"\u1026\u1027\u1028\u1029\u102A\u102B\u102C\u102D"+
	"\u102E\u102F\u1030\u1031\u1032\u1033\u1034\u1035"+
	"\u1036\u1037\u1038\u1039\u103A\u103B\u103C\u103D"+
	"\u103E\u103F\u1040\u1041\u1042\u1043\u1044\u1045"+
	"\u1046\u1047\u1048\u1049\u104A\u104B\u104C\u104D"+
	"\u104E\u104F\u1050\u1051\u1052\u1053\u1054\u1055"+
	"\u1056\u1057\u1058\u1059\u105A\u105B\u105C\u105D"+
	"\u105E\u105F\u1060\u1061\u1062\u1063\u1064\u1065"+
	"\u1066\u1067\u1068\u1069\u106A\u106B\u106C\u106D"+
	"\u106E\u106F\u1070\u1071\u1072\u1073\u1074\u1075"+
	"\u1076\u1077\u1078\u1079\u107A\u107B\u107C\u107D"+
	"\u107E\u107F\u1080\u1081\u1082\u1083\u1084\u1085"+
	"\u1086\u1087\u1088\u1089\u108A\u108B\u108C\u108D"+
	"\u108E\u108F\u1090\u1091\u1092\u1093\u1094\u1095"+
	"\u1096\u1097\u1098\u1099\u109A\u109B\u109C\u109D"+
	"\u109E\u109F\u10A0\u10A1\u10A2\u10A3\u10A4\u10A5"+
	"\u10A6\u10A7\u10A8\u10A9\u10AA\u10AB\u10AC\u10AD"+
	"\u10AE\u10AF\u10B0\u10B1\u10B2\u10B3\u10B4\u10B5"+
	"\u10B6\u10B7\u10B8\u10B9\u10BA\u10BB\u10BC\u10BD"+
	"\u10BE\u10BF\u10C0\u10C1\u10C2\u10C3\u10C4\u10C5"+
	"\u10C6\u10C7\u10C8\u10C9\u10CA\u10CB\u10CC\u10CD"+
	"\u10CE\u10CF\u10D0\u10D1\u10D2\u10D3\u10D4\u10D5"+
	"\u10D6\u10D7\u10D8\u10D9\u10DA\u10DB\u10DC\u10DD"+
	"\u10DE\u10DF\u10E0\u10E1\u10E2\u10E3\u10E4\u10E5"+
	"\u10E6\u10E7\u10E8\u10E9\u10EA\u10EB\u10EC\u10ED"+
	"\u10EE\u10EF\u10F0\u10F1\u10F2\u10F3\u10F4\u10F5"+
	"\u10F6\u10F7\u10F8\u10F9\u10FA\u10FB\u10FC\u10FD"+
	"\u10FE\u10FF\u1100\u1101\u1102\u1103\u1104\u1105"+
	"\u1106\u1107\u1108\u1109\u110A\u110B\u110C\u110D"+
	"\u110E\u110F\u1110\u1111\u1112\u1113\u1114\u1115"+
	"\u1116\u1117\u1118\u1119\u111A\u111B\u111C\u111D";

    private final static String innerIndex1= 
	"\u111E\u111F\u1120\u1121\u1122\u1123\u1124\u1125"+
	"\u1126\u1127\u1128\u1129\u112A\u112B\u112C\u112D"+
	"\u112E\u112F\u1130\u1131\u1132\u1133\u1134\u1135"+
	"\u1136\u1137\u1138\u1139\u113A\u113B\u113C\u113D"+
	"\u113E\u113F\u1140\u1141\u1142\u1143\u1144\u1145"+
	"\u1146\u1147\u1148\u1149\u114A\u114B\u114C\u114D"+
	"\u114E\u114F\u1150\u1151\u1152\u1153\u1154\u1155"+
	"\u1156\u1157\u1158\u1159\u115A\u115B\u115C\u115D"+
	"\u115E\u115F\u1160\u1161\u1162\u1163\u1164\u1165"+
	"\u1166\u1167\u1168\u1169\u116A\u116B\u116C\u116D"+
	"\u116E\u116F\u1170\u1171\u1172\u1173\u1174\u1175"+
	"\u1176\u1177\u1178\u1179\u117A\u117B\u117C\u117D"+
	"\u117E\u117F\u1180\u1181\u1182\u1183\u1184\u1185"+
	"\u1186\u1187\u1188\u1189\u118A\u118B\u118C\u118D"+
	"\u118E\u118F\u1190\u1191\u1192\u1193\u1194\u1195"+
	"\u1196\u1197\u1198\u1199\u119A\u119B\u119C\u119D"+
	"\u119E\u119F\u11A0\u11A1\u11A2\u11A3\u11A4\u11A5"+
	"\u11A6\u11A7\u11A8\u11A9\u11AA\u11AB\u11AC\u11AD"+
	"\u11AE\u11AF\u11B0\u11B1\u11B2\u11B3\u11B4\u11B5"+
	"\u11B6\u11B7\u11B8\u11B9\u11BA\u11BB\u11BC\u11BD"+
	"\u11BE\u11BF\u11C0\u11C1\u11C2\u11C3\u11C4\u11C5"+
	"\u11C6\u11C7\u11C8\u11C9\u11CA\u11CB\u11CC\u11CD"+
	"\u11CE\u11CF\u11D0\u11D1\u11D2\u11D3\u11D4\u11D5"+
	"\u11D6\u11D7\u11D8\u11D9\u11DA\u11DB\u11DC\u11DD"+
	"\u11DE\u11DF\u11E0\u11E1\u11E2\u11E3\u11E4\u11E5"+
	"\u11E6\u11E7\u11E8\u11E9\u11EA\u11EB\u11EC\u11ED"+
	"\u11EE\u11EF\u11F0\u11F1\u11F2\u11F3\u11F4\u11F5"+
	"\u11F6\u11F7\u11F8\u11F9\u11FA\u11FB\u11FC\u11FD"+
	"\u11FE\u11FF\u1200\u1201\u1202\u1203\u1204\u1205"+
	"\u1206\u1207\u1208\u1209\u120A\u120B\u120C\u120D"+
	"\u120E\u120F\u1210\u1211\u1212\u1213\u1214\u1215"+
	"\u1216\u1217\u1218\u1219\u121A\u121B\u121C\u121D"+
	"\u121E\u121F\u1220\u1221\u1222\u1223\u1224\u1225"+
	"\u1226\u1227\u1228\u1229\u122A\u122B\u122C\u122D"+
	"\u122E\u122F\u1230\u1231\u1232\u1233\u1234\u1235"+
	"\u1236\u1237\u1238\u1239\u123A\u123B\u123C\u123D"+
	"\u123E\u123F\u1240\u1241\u1242\u1243\u1244\u1245"+
	"\u1246\u1247\u1248\u1249\u124A\u124B\u124C\u124D"+
	"\u124E\u124F\u1250\u1251\u1252\u1253\u1254\u1255"+
	"\u1256\u1257\u1258\u1259\u125A\u125B\u125C\u125D"+
	"\u125E\u125F\u1260\u1261\u1262\u1263\u1264\u1265"+
	"\u1266\u1267\u1268\u1269\u126A\u126B\u126C\u126D"+
	"\u126E\u126F\u1270\u1271\u1272\u1273\u1274\u1275"+
	"\u1276\u1277\u1278\u1279\u127A\u127B\u127C\u127D"+
	"\u127E\u127F\u1280\u1281\u1282\u1283\u1284\u1285"+
	"\u1286\u1287\u1288\u1289\u128A\u128B\u128C\u128D"+
	"\u128E\u128F\u1290\u1291\u1292\u1293\u1294\u1295"+
	"\u1296\u1297\u1298\u1299\u129A\u129B\u129C\u129D"+
	"\u129E\u129F\u12A0\u12A1\u12A2\u12A3\u12A4\u12A5"+
	"\u12A6\u12A7\u12A8\u12A9\u12AA\u12AB\u12AC\u12AD"+
	"\u12AE\u12AF\u12B0\u12B1\u12B2\u12B3\u12B4\u12B5"+
	"\u12B6\u12B7\u12B8\u12B9\u12BA\u12BB\u12BC\u12BD"+
	"\u12BE\u12BF\u12C0\u12C1\u12C2\u12C3\u12C4\u12C5"+
	"\u12C6\u12C7\u12C8\u12C9\u12CA\u12CB\u12CC\u12CD"+
	"\u12CE\u12CF\u12D0\u12D1\u12D2\u12D3\u12D4\u12D5"+
	"\u12D6\u12D7\u12D8\u12D9\u12DA\u12DB\u12DC\u12DD"+
	"\u12DE\u12DF\u12E0\u12E1\u12E2\u12E3\u12E4\u12E5"+
	"\u12E6\u12E7\u12E8\u12E9\u12EA\u12EB\u12EC\u12ED"+
	"\u12EE\u12EF\u12F0\u12F1\u12F2\u12F3\u12F4\u12F5"+
	"\u12F6\u12F7\u12F8\u12F9\u12FA\u12FB\u12FC\u12FD"+
	"\u12FE\u12FF\u1300\u1301\u1302\u1303\u1304\u1305"+
	"\u1306\u1307\u1308\u1309\u130A\u130B\u130C\u130D"+
	"\u130E\u130F\u1310\u1311\u1312\u1313\u1314\u1315"+
	"\u1316\u1317\u1318\u1319\u131A\u131B\u131C\u131D"+
	"\u131E\u131F\u1320\u1321\u1322\u1323\u1324\u1325"+
	"\u1326\u1327\u1328\u1329\u132A\u132B\u132C\u132D"+
	"\u132E\u132F\u1330\u1331\u1332\u1333\u1334\u1335"+
	"\u1336\u1337\u1338\u1339\u133A\u133B\u133C\u133D"+
	"\u133E\u133F\u1340\u1341\u1342\u1343\u1344\u1345"+
	"\u1346\u1347\u1348\u1349\u134A\u134B\u134C\u134D"+
	"\u134E\u134F\u1350\u1351\u1352\u1353\u1354\u1355"+
	"\u1356\u1357\u1358\u1359\u135A\u135B\u135C\u135D"+
	"\u135E\u135F\u1360\u1361\u1362\u1363\u1364\u1365"+
	"\u1366\u1367\u1368\u1369\u136A\u136B\u136C\u136D"+
	"\u136E\u136F\u1370\u1371\u1372\u1373\u1374\u1375"+
	"\u1376\u1377\u1378\u1379\u137A\u137B\u137C\u137D"+
	"\u137E\u137F\u1380\u1381\u1382\u1383\u1384\u1385"+
	"\u1386\u1387\u1388\u1389\u138A\u138B\u138C\u138D"+
	"\u138E\u138F\u1390\u1391\u1392\u1393\u1394\u1395"+
	"\u1396\u1397\u1398\u1399\u139A\u139B\u139C\u139D"+
	"\u139E\u139F\u13A0\u13A1\u13A2\u13A3\u13A4\u13A5"+
	"\u13A6\u13A7\u13A8\u13A9\u13AA\u13AB\u13AC\u13AD"+
	"\u13AE\u13AF\u13B0\u13B1\u13B2\u13B3\u13B4\u13B5"+
	"\u13B6\u13B7\u13B8\u13B9\u13BA\u13BB\u13BC\u13BD"+
	"\u13BE\u13BF\u13C0\u13C1\u13C2\u13C3\u13C4\u13C5"+
	"\u13C6\u13C7\u13C8\u13C9\u13CA\u13CB\u13CC\u13CD"+
	"\u13CE\u13CF\u13D0\u13D1\u13D2\u13D3\u13D4\u13D5"+
	"\u13D6\u13D7\u13D8\u13D9\u13DA\u13DB\u13DC\u13DD"+
	"\u13DE\u13DF\u13E0\u13E1\u13E2\u13E3\u13E4\u13E5"+
	"\u13E6\u13E7\u13E8\u13E9\u13EA\u13EB\u13EC\u13ED"+
	"\u13EE\u13EF\u13F0\u13F1\u13F2\u13F3\u13F4\u13F5"+
	"\u13F6\u13F7\u13F8\u13F9\u13FA\u13FB\u13FC\u13FD"+
	"\u13FE\u13FF\u1400\u1401\u1402\u1403\u1404\u1405"+
	"\u1406\u1407\u1408\u1409\u140A\u140B\u140C\u140D"+
	"\u140E\u140F\u1410\u1411\u1412\u1413\u1414\u1415"+
	"\u1416\u1417\u1418\u1419\u141A\u141B\u141C\u141D"+
	"\u141E\u141F\u1420\u1421\u1422\u1423\u1424\u1425"+
	"\u1426\u1427\u1428\u1429\u142A\u142B\u142C\u142D"+
	"\u142E\u142F\u1430\u1431\u1432\u1433\u1434\u1435"+
	"\u1436\u1437\u1438\u1439\u143A\u143B\u143C\u143D"+
	"\u143E\u143F\u1440\u1441\u1442\u1443\u1444\u1445"+
	"\u1446\u1447\u1448\u1449\u144A\u144B\u144C\u144D"+
	"\u144E\u144F\u1450\u1451\u1452\u1453\u1454\u1455"+
	"\u1456\u1457\u1458\u1459\u145A\u145B\u145C\u145D"+
	"\u145E\u145F\u1460\u1461\u1462\u1463\u1464\u1465"+
	"\u1466\u1467\u1468\u1469\u146A\u146B\u146C\u146D"+
	"\u146E\u146F\u1470\u1471\u1472\u1473\u1474\u1475"+
	"\u1476\u1477\u1478\u1479\u147A\u147B\u147C\u147D"+
	"\u147E\u147F\u1480\u1481\u1482\u1483\u1484\u1485"+
	"\u1486\u1487\u1488\u1489\u148A\u148B\u148C\u148D"+
	"\u148E\u148F\u1490\u1491\u1492\u1493\u1494\u1495"+
	"\u1496\u1497\u1498\u1499\u149A\u149B\u149C\u149D"+
	"\u149E\u149F\u14A0\u14A1\u14A2\u14A3\u14A4\u14A5"+
	"\u14A6\u14A7\u14A8\u14A9\u14AA\u14AB\u14AC\u14AD"+
	"\u14AE\u14AF\u14B0\u14B1\u14B2\u14B3\u14B4\u14B5"+
	"\u14B6\u14B7\u14B8\u14B9\u14BA\u14BB\u14BC\u14BD"+
	"\u14BE\u14BF\u14C0\u14C1\u14C2\u14C3\u14C4\u14C5"+
	"\u14C6\u14C7\u14C8\u14C9\u14CA\u14CB\u14CC\u14CD"+
	"\u14CE\u14CF\u14D0\u14D1\u14D2\u14D3\u14D4\u14D5"+
	"\u14D6\u14D7\u14D8\u14D9\u14DA\u14DB\u14DC\u14DD"+
	"\u14DE\u14DF\u14E0\u14E1\u14E2\u14E3\u14E4\u14E5"+
	"\u14E6\u14E7\u14E8\u14E9\u14EA\u14EB\u14EC\u14ED"+
	"\u14EE\u14EF\u14F0\u14F1\u14F2\u14F3\u14F4\u14F5"+
	"\u14F6\u14F7\u14F8\u14F9\u14FA\u14FB\u14FC\u14FD"+
	"\u14FE\u14FF\u1500\u1501\u1502\u1503\u1504\u1505"+
	"\u1506\u1507\u1508\u1509\u150A\u150B\u150C\u150D"+
	"\u150E\u150F\u1510\u1511\u1512\u1513\u1514\u1515"+
	"\u1516\u1517\u1518\u1519\u151A\u151B\u151C\u151D"+
	"\u151E\u151F\u1520\u1521\u1522\u1523\u1524\u1525"+
	"\u1526\u1527\u1528\u1529\u152A\u152B\u152C\u152D"+
	"\u152E\u152F\u1530\u1531\u1532\u1533\u1534\u1535"+
	"\u1536\u1537\u1538\u1539\u153A\u153B\u153C\u153D"+
	"\u153E\u153F\u1540\u1541\u1542\u1543\u1544\u1545"+
	"\u1546\u1547\u1548\u1549\u154A\u154B\u154C\u154D"+
	"\u154E\u154F\u1550\u1551\u1552\u1553\u1554\u1555"+
	"\u1556\u1557\u1558\u1559\u155A\u155B\u155C\u155D"+
	"\u155E\u155F\u1560\u1561\u1562\u1563\u1564\u1565"+
	"\u1566\u1567\u1568\u1569\u156A\u156B\u156C\u156D"+
	"\u156E\u156F\u1570\u1571\u1572\u1573\u1574\u1575"+
	"\u1576\u1577\u1578\u1579\u157A\u157B\u157C\u157D"+
	"\u157E\u157F\u1580\u1581\u1582\u1583\u1584\u1585"+
	"\u1586\u1587\u1588\u1589\u158A\u158B\u158C\u158D"+
	"\u158E\u158F\u1590\u1591\u1592\u1593\u1594\u1595"+
	"\u1596\u1597\u1598\u1599\u159A\u159B\u159C\u159D"+
	"\u159E\u159F\u15A0\u15A1\u15A2\u15A3\u15A4\u15A5"+
	"\u15A6\u15A7\u15A8\u15A9\u15AA\u15AB\u15AC\u15AD"+
	"\u15AE\u15AF\u15B0\u15B1\u15B2\u15B3\u15B4\u15B5"+
	"\u15B6\u15B7\u15B8\u15B9\u15BA\u15BB\u15BC\u15BD"+
	"\u15BE\u15BF\u15C0\u15C1\u15C2\u15C3\u15C4\u15C5"+
	"\u15C6\u15C7\u15C8\u15C9\u15CA\u15CB\u15CC\u15CD"+
	"\u15CE\u15CF\u15D0\u15D1\u15D2\u15D3\u15D4\u15D5"+
	"\u15D6\u15D7\u15D8\u15D9\u15DA\u15DB\u15DC\u15DD"+
	"\u15DE\u15DF\u15E0\u15E1\u15E2\u15E3\u15E4\u15E5"+
	"\u15E6\u15E7\u15E8\u15E9\u15EA\u15EB\u15EC\u15ED"+
	"\u15EE\u15EF\u15F0\u15F1\u15F2\u15F3\u15F4\u15F5"+
	"\u15F6\u15F7\u15F8\u15F9\u15FA\u15FB\u15FC\u15FD"+
	"\u15FE\u15FF\u1600\u1601\u1602\u1603\u1604\u1605"+
	"\u1606\u1607\u1608\u1609\u160A\u160B\u160C\u160D"+
	"\u160E\u160F\u1610\u1611\u1612\u1613\u1614\u1615"+
	"\u1616\u1617\u1618\u1619\u161A\u161B\u161C\u161D"+
	"\u161E\u161F\u1620\u1621\u1622\u1623\u1624\u1625"+
	"\u1626\u1627\u1628\u1629\u162A\u162B\u162C\u162D"+
	"\u162E\u162F\u1630\u1631\u1632\u1633\u1634\u1635"+
	"\u1636\u1637\u1638\u1639\u163A\u163B\u163C\u163D"+
	"\u163E\u163F\u1640\u1641\u1642\u1643\u1644\u1645"+
	"\u1646\u1647\u1648\u1649\u164A\u164B\u164C\u164D"+
	"\u164E\u164F\u1650\u1651\u1652\u1653\u1654\u1655"+
	"\u1656\u1657\u1658\u1659\u165A\u165B\u165C\u165D"+
	"\u165E\u165F\u1660\u1661\u1662\u1663\u1664\u1665"+
	"\u1666\u1667\u1668\u1669\u166A\u166B\u166C\u166D"+
	"\u166E\u166F\u1670\u1671\u1672\u1673\u1674\u1675"+
	"\u1676\u1677\u1678\u1679\u167A\u167B\u167C\u167D"+
	"\u167E\u167F\u1680\u1681\u1682\u1683\u1684\u1685"+
	"\u1686\u1687\u1688\u1689\u168A\u168B\u168C\u168D"+
	"\u168E\u168F\u1690\u1691\u1692\u1693\u1694\u1695"+
	"\u1696\u1697\u1698\u1699\u169A\u169B\u169C\u169D"+
	"\u169E\u169F\u16A0\u16A1\u16A2\u16A3\u16A4\u16A5"+
	"\u16A6\u16A7\u16A8\u16A9\u16AA\u16AB\u16AC\u16AD"+
	"\u16AE\u16AF\u16B0\u16B1\u16B2\u16B3\u16B4\u16B5"+
	"\u16B6\u16B7\u16B8\u16B9\u16BA\u16BB\u16BC\u16BD"+
	"\u16BE\u16BF\u16C0\u16C1\u16C2\u16C3\u16C4\u16C5"+
	"\u16C6\u16C7\u16C8\u16C9\u16CA\u16CB\u16CC\u16CD"+
	"\u16CE\u16CF\u16D0\u16D1\u16D2\u16D3\u16D4\u16D5"+
	"\u16D6\u16D7\u16D8\u16D9\u16DA\u16DB\u16DC\u16DD"+
	"\u16DE\u16DF\u16E0\u16E1\u16E2\u16E3\u16E4\u16E5"+
	"\u16E6\u16E7\u16E8\u16E9\u16EA\u16EB\u16EC\u16ED"+
	"\u16EE\u16EF\u16F0\u16F1\u16F2\u16F3\u16F4\u16F5"+
	"\u16F6\u16F7\u16F8\u16F9\u16FA\u16FB\u16FC\u16FD"+
	"\u16FE\u16FF\u1700\u1701\u1702\u1703\u1704\u1705"+
	"\u1706\u1707\u1708\u1709\u170A\u170B\u170C\u170D"+
	"\u170E\u170F\u1710\u1711\u1712\u1713\u1714\u1715"+
	"\u1716\u1717\u1718\u1719\u171A\u171B\u171C\u171D"+
	"\u171E\u171F\u1720\u1721\u1722\u1723\u1724\u1725"+
	"\u1726\u1727\u1728\u1729\u172A\u172B\u172C\u172D"+
	"\u172E\u172F\u1730\u1731\u1732\u1733\u1734\u1735"+
	"\u1736\u1737\u1738\u1739\u173A\u173B\u173C\u173D"+
	"\u173E\u173F\u1740\u1741\u1742\u1743\u1744\u1745"+
	"\u1746\u1747\u1748\u1749\u174A\u174B\u174C\u174D"+
	"\u174E\u174F\u1750\u1751\u1752\u1753\u1754\u1755"+
	"\u1756\u1757\u1758\u1759\u175A\u175B\u175C\u175D"+
	"\u175E\u175F\u1760\u1761\u1762\u1763\u1764\u1765"+
	"\u1766\u1767\u1768\u1769\u176A\u176B\u176C\u176D"+
	"\u176E\u176F\u1770\u1771\u1772\u1773\u1774\u1775"+
	"\u1776\u1777\u1778\u1779\u177A\u177B\u177C\u177D"+
	"\u177E\u177F\u1780\u1781\u1782\u1783\u1784\u1785"+
	"\u1786\u1787\u1788\u1789\u178A\u178B\u178C\u178D"+
	"\u178E\u178F\u1790\u1791\u1792\u1793\u1794\u1795"+
	"\u1796\u1797\u1798\u1799\u179A\u179B\u179C\u179D"+
	"\u179E\u179F\u17A0\u17A1\u17A2\u17A3\u17A4\u17A5"+
	"\u17A6\u17A7\u17A8\u17A9\u17AA\u17AB\u17AC\u17AD"+
	"\u17AE\u17AF\u17B0\u17B1\u17B2\u17B3\u17B4\u17B5"+
	"\u17B6\u17B7\u17B8\u17B9\u17BA\u17BB\u17BC\u17BD"+
	"\u17BE\u17BF\u17C0\u17C1\u17C2\u17C3\u17C4\u17C5"+
	"\u17C6\u17C7\u17C8\u17C9\u17CA\u17CB\u17CC\u17CD"+
	"\u17CE\u17CF\u17D0\u17D1\u17D2\u17D3\u17D4\u17D5"+
	"\u17D6\u17D7\u17D8\u17D9\u17DA\u17DB\u17DC\u17DD"+
	"\u17DE\u17DF\u17E0\u17E1\u17E2\u17E3\u17E4\u17E5"+
	"\u17E6\u17E7\u17E8\u17E9\u17EA\u17EB\u17EC\u17ED"+
	"\u17EE\u17EF\u17F0\u17F1\u17F2\u17F3\u17F4\u17F5"+
	"\u17F6\u17F7\u17F8\u17F9\u17FA\u17FB\u17FC\u17FD"+
	"\u17FE\u17FF\u1800\u1801\u1802\u1803\u1804\u1805"+
	"\u1806\u1807\u1808\u1809\u180A\u180B\u180C\u180D"+
	"\u180E\u180F\u1810\u1811\u1812\u1813\u1814\u1815"+
	"\u1816\u1817\u1818\u1819\u181A\u181B\u181C\u181D"+
	"\u181E\u181F\u1820\u1821\u1822\u1823\u1824\u1825"+
	"\u1826\u1827\u1828\u1829\u182A\u182B\u182C\u182D"+
	"\u182E\u182F\u1830\u1831\u1832\u1833\u1834\u1835"+
	"\u1836\u1837\u1838\u1839\u183A\u183B\u183C\u183D"+
	"\u183E\u183F\u1840\u1841\u1842\u1843\u1844\u1845"+
	"\u1846\u1847\u1848\u1849\u184A\u184B\u184C\u184D"+
	"\u184E\u184F\u1850\u1851\u1852\u1853\u1854\u1855"+
	"\u1856\u1857\u1858\u1859\u185A\u185B\u185C\u185D"+
	"\u185E\u185F\u1860\u1861\u1862\u1863\u1864\u1865"+
	"\u1866\u1867\u1868\u1869\u186A\u186B\u186C\u186D"+
	"\u186E\u186F\u1870\u1871\u1872\u1873\u1874\u1875"+
	"\u1876\u1877\u1878\u1879\u187A\u187B\u187C\u187D"+
	"\u187E\u187F\u1880\u1881\u1882\u1883\u1884\u1885"+
	"\u1886\u1887\u1888\u1889\u188A\u188B\u188C\u188D"+
	"\u188E\u188F\u1890\u1891\u1892\u1893\u1894\u1895"+
	"\u1896\u1897\u1898\u1899\u189A\u189B\u189C\u189D"+
	"\u189E\u189F\u18A0\u18A1\u18A2\u18A3\u18A4\u18A5"+
	"\u18A6\u18A7\u18A8\u18A9\u18AA\u18AB\u18AC\u18AD"+
	"\u18AE\u18AF\u18B0\u18B1\u18B2\u18B3\u18B4\u18B5"+
	"\u18B6\u18B7\u18B8\u18B9\u18BA\u18BB\u18BC\u18BD"+
	"\u18BE\u18BF\u18C0\u18C1\u18C2\u18C3\u18C4\u18C5"+
	"\u18C6\u18C7\u18C8\u18C9\u18CA\u18CB\u18CC\u18CD"+
	"\u18CE\u18CF\u18D0\u18D1\u18D2\u18D3\u18D4\u18D5"+
	"\u18D6\u18D7\u18D8\u18D9\u18DA\u18DB\u18DC\u18DD"+
	"\u18DE\u18DF\u18E0\u18E1\u18E2\u18E3\u18E4\u18E5"+
	"\u18E6\u18E7\u18E8\u18E9\u18EA\u18EB\u18EC\u18ED"+
	"\u18EE\u18EF\u18F0\u18F1\u18F2\u18F3\u18F4\u18F5"+
	"\u18F6\u18F7\u18F8\u18F9\u18FA\u18FB\u18FC\u18FD"+
	"\u18FE\u18FF\u1900\u1901\u1902\u1903\u1904\u1905"+
	"\u1906\u1907\u1908\u1909\u190A\u190B\u190C\u190D"+
	"\u190E\u190F\u1910\u1911\u1912\u1913\u1914\u1915"+
	"\u1916\u1917\u1918\u1919\u191A\u191B\u191C\u191D"+
	"\u191E\u191F\u1920\u1921\u1922\u1923\u1924\u1925"+
	"\u1926\u1927\u1928\u1929\u192A\u192B\u192C\u192D"+
	"\u192E\u192F\u1930\u1931\u1932\u1933\u1934\u1935"+
	"\u1936\u1937\u1938\u1939\u193A\u193B\u193C\u193D"+
	"\u193E\u193F\u1940\u1941\u1942\u1943\u1944\u1945"+
	"\u1946\u1947\u1948\u1949\u194A\u194B\u194C\u194D"+
	"\u194E\u194F\u1950\u1951\u1952\u1953\u1954\u1955"+
	"\u1956\u1957\u1958\u1959\u195A\u195B\u195C\u195D"+
	"\u195E\u195F\u1960\u1961\u1962\u1963\u1964\u1965"+
	"\u1966\u1967\u1968\u1969\u196A\u196B\u196C\u196D"+
	"\u196E\u196F\u1970\u1971\u1972\u1973\u1974\u1975"+
	"\u1976\u1977\u1978\u1979\u197A\u197B\u197C\u197D"+
	"\u197E\u197F\u1980\u1981\u1982\u1983\u1984\u1985"+
	"\u1986\u1987\u1988\u1989\u198A\u198B\u198C\u198D"+
	"\u198E\u198F\u1990\u1991\u1992\u1993\u1994\u1995"+
	"\u1996\u1997\u1998\u1999\u199A\u199B\u199C\u199D"+
	"\u199E\u199F\u19A0\u19A1\u19A2\u19A3\u19A4\u19A5"+
	"\u19A6\u19A7\u19A8\u19A9\u19AA\u19AB\u19AC\u19AD"+
	"\u19AE\u19AF\u19B0\u19B1\u19B2\u19B3\u19B4\u19B5"+
	"\u19B6\u19B7\u19B8\u19B9\u19BA\u19BB\u19BC\u19BD"+
	"\u19BE\u19BF\u19C0\u19C1\u19C2\u19C3\u19C4\u19C5"+
	"\u19C6\u19C7\u19C8\u19C9\u19CA\u19CB\u19CC\u19CD"+
	"\u19CE\u19CF\u19D0\u19D1\u19D2\u19D3\u19D4\u19D5"+
	"\u19D6\u19D7\u19D8\u19D9\u19DA\u19DB\u19DC\u19DD"+
	"\u19DE\u19DF\u19E0\u19E1\u19E2\u19E3\u19E4\u19E5"+
	"\u19E6\u19E7\u19E8\u19E9\u19EA\u19EB\u19EC\u19ED"+
	"\u19EE\u19EF\u19F0\u19F1\u19F2\u19F3\u19F4\u19F5"+
	"\u19F6\u19F7\u19F8\u19F9\u19FA\u19FB\u19FC\u19FD"+
	"\u19FE\u19FF\u1A00\u1A01\u1A02\u1A03\u1A04\u1A05"+
	"\u1A06\u1A07\u1A08\u1A09\u1A0A\u1A0B\u1A0C\u1A0D"+
	"\u1A0E\u1A0F\u1A10\u1A11\u1A12\u1A13\u1A14\u1A15"+
	"\u1A16\u1A17\u1A18\u1A19\u1A1A\u1A1B\u1A1C\u1A1D"+
	"\u1A1E\u1A1F\u1A20\u1A21\u1A22\u1A23\u1A24\u1A25"+
	"\u1A26\u1A27\u1A28\u1A29\u1A2A\u1A2B\u1A2C\u1A2D"+
	"\u1A2E\u1A2F\u1A30\u1A31\u1A32\u1A33\u1A34\u1A35"+
	"\u1A36\u1A37\u1A38\u1A39\u1A3A\u1A3B\u1A3C\u1A3D"+
	"\u1A3E\u1A3F\u1A40\u1A41\u1A42\u1A43\u1A44\u1A45"+
	"\u1A46\u1A47\u1A48\u1A49\u1A4A\u1A4B\u1A4C\u1A4D"+
	"\u1A4E\u1A4F\u1A50\u1A51\u1A52\u1A53\u1A54\u1A55"+
	"\u1A56\u1A57\u1A58\u1A59\u1A5A\u1A5B\u1A5C\u1A5D"+
	"\u1A5E\u1A5F\u1A60\u1A61\u1A62\u1A63\u1A64\u1A65"+
	"\u1A66\u1A67\u1A68\u1A69\u1A6A\u1A6B\u1A6C\u1A6D"+
	"\u1A6E\u1A6F\u1A70\u1A71\u1A72\u1A73\u1A74\u1A75"+
	"\u1A76\u1A77\u1A78\u1A79\u1A7A\u1A7B\u1A7C\u1A7D"+
	"\u1A7E\u1A7F\u1A80\u1A81\u1A82\u1A83\u1A84\u1A85"+
	"\u1A86\u1A87\u1A88\u1A89\u1A8A\u1A8B\u1A8C\u1A8D"+
	"\u1A8E\u1A8F\u1A90\u1A91\u1A92\u1A93\u1A94\u1A95"+
	"\u1A96\u1A97\u1A98\u1A99\u1A9A\u1A9B\u1A9C\u1A9D"+
	"\u1A9E\u1A9F\u1AA0\u1AA1\u1AA2\u1AA3\u1AA4\u1AA5"+
	"\u1AA6\u1AA7\u1AA8\u1AA9\u1AAA\u1AAB\u1AAC\u1AAD"+
	"\u1AAE\u1AAF\u1AB0\u1AB1\u1AB2\u1AB3\u1AB4\u1AB5"+
	"\u1AB6\u1AB7\u1AB8\u1AB9\u1ABA\u1ABB\u1ABC\u1ABD"+
	"\u1ABE\u1ABF\u1AC0\u1AC1\u1AC2\u1AC3\u1AC4\u1AC5"+
	"\u1AC6\u1AC7\u1AC8\u1AC9\u1ACA\u1ACB\u1ACC\u1ACD"+
	"\u1ACE\u1ACF\u1AD0\u1AD1\u1AD2\u1AD3\u1AD4\u1AD5"+
	"\u1AD6\u1AD7\u1AD8\u1AD9\u1ADA\u1ADB\u1ADC\u1ADD"+
	"\u1ADE\u1ADF\u1AE0\u1AE1\u1AE2\u1AE3\u1AE4\u1AE5"+
	"\u1AE6\u1AE7\u1AE8\u1AE9\u1AEA\u1AEB\u1AEC\u1AED"+
	"\u1AEE\u1AEF\u1AF0\u1AF1\u1AF2\u1AF3\u1AF4\u1AF5"+
	"\u1AF6\u1AF7\u1AF8\u1AF9\u1AFA\u1AFB\u1AFC\u1AFD"+
	"\u1AFE\u1AFF\u1B00\u1B01\u1B02\u1B03\u1B04\u1B05"+
	"\u1B06\u1B07\u1B08\u1B09\u1B0A\u1B0B\u1B0C\u1B0D"+
	"\u1B0E\u1B0F\u1B10\u1B11\u1B12\u1B13\u1B14\u1B15"+
	"\u1B16\u1B17\u1B18\u1B19\u1B1A\u1B1B\u1B1C\u1B1D"+
	"\u1B1E\u1B1F\u1B20\u1B21\u1B22\u1B23\u1B24\u1B25"+
	"\u1B26\u1B27\u1B28\u1B29\u1B2A\u1B2B\u1B2C\u1B2D"+
	"\u1B2E\u1B2F\u1B30\u1B31\u1B32\u1B33\u1B34\u1B35"+
	"\u1B36\u1B37\u1B38\u1B39\u1B3A\u1B3B\u1B3C\u1B3D"+
	"\u1B3E\u1B3F\u1B40\u1B41\u1B42\u1B43\u1B44\u1B45"+
	"\u1B46\u1B47\u1B48\u1B49\u1B4A\u1B4B\u1B4C\u1B4D"+
	"\u1B4E\u1B4F\u1B50\u1B51\u1B52\u1B53\u1B54\u1B55"+
	"\u1B56\u1B57\u1B58\u1B59\u1B5A\u1B5B\u1B5C\u1B5D"+
	"\u1B5E\u1B5F\u1B60\u1B61\u1B62\u1B63\u1B64\u1B65"+
	"\u1B66\u1B67\u1B68\u1B69\u1B6A\u1B6B\u1B6C\u1B6D"+
	"\u1B6E\u1B6F\u1B70\u1B71\u1B72\u1B73\u1B74\u1B75"+
	"\u1B76\u1B77\u1B78\u1B79\u1B7A\u1B7B\u1B7C\u1B7D"+
	"\u1B7E\u1B7F\u1B80\u1B81\u1B82\u1B83\u1B84\u1B85"+
	"\u1B86\u1B87\u1B88\u1B89\u1B8A\u1B8B\u1B8C\u1B8D"+
	"\u1B8E\u1B8F\u1B90\u1B91\u1B92\u1B93\u1B94\u1B95"+
	"\u1B96\u1B97\u1B98\u1B99\u1B9A\u1B9B\u1B9C\u1B9D"+
	"\u1B9E\u1B9F\u1BA0\u1BA1\u1BA2\u1BA3\u1BA4\u1BA5"+
	"\u1BA6\u1BA7\u1BA8\u1BA9\u1BAA\u1BAB\u1BAC\u1BAD"+
	"\u1BAE\u1BAF\u1BB0\u1BB1\u1BB2\u1BB3\u1BB4\u1BB5"+
	"\u1BB6\u1BB7\u1BB8\u1BB9\u1BBA\u1BBB\u1BBC\u1BBD"+
	"\u1BBE\u1BBF\u1BC0\u1BC1\u1BC2\u1BC3\u1BC4\u1BC5"+
	"\u1BC6\u1BC7\u1BC8\u1BC9\u1BCA\u1BCB\u1BCC\u1BCD"+
	"\u1BCE\u1BCF\u1BD0\u1BD1\u1BD2\u1BD3\u1BD4\u1BD5"+
	"\u1BD6\u1BD7\u1BD8\u1BD9\u1BDA\u1BDB\u1BDC\u1BDD"+
	"\u1BDE\u1BDF\u1BE0\u1BE1\u1BE2\u1BE3\u1BE4\u1BE5"+
	"\u1BE6\u1BE7\u1BE8\u1BE9\u1BEA\u1BEB\u1BEC\u1BED"+
	"\u1BEE\u1BEF\u1BF0\u1BF1\u1BF2\u1BF3\u1BF4\u1BF5"+
	"\u1BF6\u1BF7\u1BF8\u1BF9\u1BFA\u1BFB\u1BFC\u1BFD"+
	"\u1BFE\u1BFF\u1C00\u1C01\u1C02\u1C03\u1C04\u1C05"+
	"\u1C06\u1C07\u1C08\u1C09\u1C0A\u1C0B\u1C0C\u1C0D"+
	"\u1C0E\u1C0F\u1C10\u1C11\u1C12\u1C13\u1C14\u1C15"+
	"\u1C16\u1C17\u1C18\u1C19\u1C1A\u1C1B\u1C1C\u1C1D"+
	"\u1C1E\u1C1F\u1C20\u1C21\u1C22\u1C23\u1C24\u1C25"+
	"\u1C26\u1C27\u1C28\u1C29\u1C2A\u1C2B\u1C2C\u1C2D"+
	"\u1C2E\u1C2F\u1C30\u1C31\u1C32\u1C33\u1C34\u1C35"+
	"\u1C36\u1C37\u1C38\u1C39\u1C3A\u1C3B\u1C3C\u1C3D"+
	"\u1C3E\u1C3F\u1C40\u1C41\u1C42\u1C43\u1C44\u1C45"+
	"\u1C46\u1C47\u1C48\u1C49\u1C4A\u1C4B\u1C4C\u1C4D"+
	"\u1C4E\u1C4F\u1C50\u1C51\u1C52\u1C53\u1C54\u1C55"+
	"\u1C56\u1C57\u1C58\u1C59\u1C5A\u1C5B\u1C5C\u1C5D"+
	"\u1C5E\u1C5F\u1C60\u1C61\u1C62\u1C63\u1C64\u1C65"+
	"\u1C66\u1C67\u1C68\u1C69\u1C6A\u1C6B\u1C6C\u1C6D"+
	"\u1C6E\u1C6F\u1C70\u1C71\u1C72\u1C73\u1C74\u1C75"+
	"\u1C76\u1C77\u1C78\u1C79\u1C7A\u1C7B\u1C7C\u1C7D"+
	"\u1C7E\u1C7F\u1C80\u1C81\u1C82\u1C83\u1C84\u1C85"+
	"\u1C86\u1C87\u1C88\u1C89\u1C8A\u1C8B\u1C8C\u1C8D"+
	"\u1C8E\u1C8F\u1C90\u1C91\u1C92\u1C93\u1C94\u1C95"+
	"\u1C96\u1C97\u1C98\u1C99\u1C9A\u1C9B\u1C9C\u1C9D"+
	"\u1C9E\u1C9F\u1CA0\u1CA1\u1CA2\u1CA3\u1CA4\u1CA5"+
	"\u1CA6\u1CA7\u1CA8\u1CA9\u1CAA\u1CAB\u1CAC\u1CAD"+
	"\u1CAE\u1CAF\u1CB0\u1CB1\u1CB2\u1CB3\u1CB4\u1CB5"+
	"\u1CB6\u1CB7\u1CB8\u1CB9\u1CBA\u1CBB\u1CBC\u1CBD"+
	"\u1CBE\u1CBF\u1CC0\u1CC1\u1CC2\u1CC3\u1CC4\u1CC5"+
	"\u1CC6\u1CC7\u1CC8\u1CC9\u1CCA\u1CCB\u1CCC\u1CCD"+
	"\u1CCE\u1CCF\u1CD0\u1CD1\u1CD2\u1CD3\u1CD4\u1CD5"+
	"\u1CD6\u1CD7\u1CD8\u1CD9\u1CDA\u1CDB\u1CDC\u1CDD"+
	"\u1CDE\u1CDF\u1CE0\u1CE1\u1CE2\u1CE3\u1CE4\u1CE5"+
	"\u1CE6\u1CE7\u1CE8\u1CE9\u1CEA\u1CEB\u1CEC\u1CED"+
	"\u1CEE\u1CEF\u1CF0\u1CF1\u1CF2\u1CF3\u1CF4\u1CF5"+
	"\u1CF6\u1CF7\u1CF8\u1CF9\u1CFA\u1CFB\u1CFC\u1CFD"+
	"\u1CFE\u1CFF\u1D00\u1D01\u1D02\u1D03\u1D04\u1D05"+
	"\u1D06\u1D07\u1D08\u1D09\u1D0A\u1D0B\u1D0C\u1D0D"+
	"\u1D0E\u1D0F\u1D10\u1D11\u1D12\u1D13\u1D14\u1D15"+
	"\u1D16\u1D17\u1D18\u1D19\u1D1A\u1D1B\u1D1C\u1D1D"+
	"\u1D1E\u1D1F\u1D20\u1D21\u1D22\u1D23\u1D24\u1D25"+
	"\u1D26\u1D27\u1D28\u1D29\u1D2A\u1D2B\u1D2C\u1D2D"+
	"\u1D2E\u1D2F\u1D30\u1D31\u1D32\u1D33\u1D34\u1D35"+
	"\u1D36\u1D37\u1D38\u1D39\u1D3A\u1D3B\u1D3C\u1D3D"+
	"\u1D3E\u1D3F\u1D40\u1D41\u1D42\u1D43\u1D44\u1D45"+
	"\u1D46\u1D47\u1D48\u1D49\u1D4A\u1D4B\u1D4C\u1D4D"+
	"\u1D4E\u1D4F\u1D50\u1D51\u1D52\u1D53\u1D54\u1D55"+
	"\u1D56\u1D57\u1D58\u1D59\u1D5A\u1D5B\u1D5C\u1D5D"+
	"\u1D5E\u1D5F\u1D60\u1D61\u1D62\u1D63\u1D64\u1D65"+
	"\u1D66\u1D67\u1D68\u1D69\u1D6A\u1D6B\u1D6C\u1D6D"+
	"\u1D6E\u1D6F\u1D70\u1D71\u1D72\u1D73\u1D74\u1D75"+
	"\u1D76\u1D77\u1D78\u1D79\u1D7A\u1D7B\u1D7C\u1D7D"+
	"\u1D7E\u1D7F\u1D80\u1D81\u1D82\u1D83\u1D84\u1D85"+
	"\u1D86\u1D87\u1D88\u1D89\u1D8A\u1D8B\u1D8C\u1D8D"+
	"\u1D8E\u1D8F\u1D90\u1D91\u1D92\u1D93\u1D94\u1D95"+
	"\u1D96\u1D97\u1D98\u1D99\u1D9A\u1D9B\u1D9C\u1D9D"+
	"\u1D9E\u1D9F\u1DA0\u1DA1\u1DA2\u1DA3\u1DA4\u1DA5"+
	"\u1DA6\u1DA7\u1DA8\u1DA9\u1DAA\u1DAB\u1DAC\u1DAD"+
	"\u1DAE\u1DAF\u1DB0\u1DB1\u1DB2\u1DB3\u1DB4\u1DB5"+
	"\u1DB6\u1DB7\u1DB8\u1DB9\u1DBA\u1DBB\u1DBC\u1DBD"+
	"\u1DBE\u1DBF\u1DC0\u1DC1\u1DC2\u1DC3\u1DC4\u1DC5"+
	"\u1DC6\u1DC7\u1DC8\u1DC9\u1DCA\u1DCB\u1DCC\u1DCD"+
	"\u1DCE\u1DCF\u1DD0\u1DD1\u1DD2\u1DD3\u1DD4\u1DD5"+
	"\u1DD6\u1DD7\u1DD8\u1DD9\u1DDA\u1DDB\u1DDC\u1DDD"+
	"\u1DDE\u1DDF\u1DE0\u1DE1\u1DE2\u1DE3\u1DE4\u1DE5"+
	"\u1DE6\u1DE7\u1DE8\u1DE9\u1DEA\u1DEB\u1DEC\u1DED"+
	"\u1DEE\u1DEF\u1DF0\u1DF1\u1DF2\u1DF3\u1DF4\u1DF5"+
	"\u1DF6\u1DF7\u1DF8\u1DF9\u1DFA\u1DFB\u1DFC\u1DFD"+
	"\u1DFE\u1DFF\u1E00\u1E01\u1E02\u1E03\u1E04\u1E05"+
	"\u1E06\u1E07\u1E08\u1E09\u1E0A\u1E0B\u1E0C\u1E0D"+
	"\u1E0E\u1E0F\u1E10\u1E11\u1E12\u1E13\u1E14\u1E15"+
	"\u1E16\u1E17\u1E18\u1E19\u1E1A\u1E1B\u1E1C\u1E1D"+
	"\u1E1E\u1E1F\u1E20\u1E21\u1E22\u1E23\u1E24\u1E25"+
	"\u1E26\u1E27\u1E28\u1E29\u1E2A\u1E2B\u1E2C\u1E2D"+
	"\u1E2E\u1E2F\u1E30\u1E31\u1E32\u1E33\u1E34\u1E35"+
	"\u1E36\u1E37\u1E38\u1E39\u1E3A\u1E3B\u1E3C\u1E3D"+
	"\u1E3E\u1E3F\u1E40\u1E41\u1E42\u1E43\u1E44\u1E45"+
	"\u1E46\u1E47\u1E48\u1E49\u1E4A\u1E4B\u1E4C\u1E4D"+
	"\u1E4E\u1E4F\u1E50\u1E51\u1E52\u1E53\u1E54\u1E55"+
	"\u1E56\u1E57\u1E58\u1E59\u1E5A\u1E5B\u1E5C\u1E5D"+
	"\u1E5E\u1E5F\u1E60\u1E61\u1E62\u1E63\u1E64\u1E65"+
	"\u1E66\u1E67\u1E68\u1E69\u1E6A\u1E6B\u1E6C\u1E6D"+
	"\u1E6E\u1E6F\u1E70\u1E71\u1E72\u1E73\u1E74\u1E75"+
	"\u1E76\u1E77\u1E78\u1E79\u1E7A\u1E7B\u1E7C\u1E7D"+
	"\u1E7E\u1E7F\u1E80\u1E81\u1E82\u1E83\u1E84\u1E85"+
	"\u1E86\u1E87\u1E88\u1E89\u1E8A\u1E8B\u1E8C\u1E8D"+
	"\u1E8E\u1E8F\u1E90\u1E91\u1E92\u1E93\u1E94\u1E95"+
	"\u1E96\u1E97\u1E98\u1E99\u1E9A\u1E9B\u1E9C\u1E9D"+
	"\u1E9E\u1E9F\u1EA0\u1EA1\u1EA2\u1EA3\u1EA4\u1EA5"+
	"\u1EA6\u1EA7\u1EA8\u1EA9\u1EAA\u1EAB\u1EAC\u1EAD"+
	"\u1EAE\u1EAF\u1EB0\u1EB1\u1EB2\u1EB3\u1EB4\u1EB5"+
	"\u1EB6\u1EB7\u1EB8\u1EB9\u1EBA\u1EBB\u1EBC\u1EBD"+
	"\u1EBE\u1EBF\u1EC0\u1EC1\u1EC2\u1EC3\u1EC4\u1EC5"+
	"\u1EC6\u1EC7\u1EC8\u1EC9\u1ECA\u1ECB\u1ECC\u1ECD"+
	"\u1ECE\u1ECF\u1ED0\u1ED1\u1ED2\u1ED3\u1ED4\u1ED5"+
	"\u1ED6\u1ED7\u1ED8\u1ED9\u1EDA\u1EDB\u1EDC\u1EDD"+
	"\u1EDE\u1EDF\u1EE0\u1EE1\u1EE2\u1EE3\u1EE4\u1EE5"+
	"\u1EE6\u1EE7\u1EE8\u1EE9\u1EEA\u1EEB\u1EEC\u1EED"+
	"\u1EEE\u1EEF\u1EF0\u1EF1\u1EF2\u1EF3\u1EF4\u1EF5"+
	"\u1EF6\u1EF7\u1EF8\u1EF9\u1EFA\u1EFB\u1EFC\u1EFD"+
	"\u1EFE\u1EFF\u1F00\u1F01\u1F02\u1F03\u1F04\u1F05"+
	"\u1F06\u1F07\u1F08\u1F09\u1F0A\u1F0B\u1F0C\u1F0D"+
	"\u1F0E\u1F0F\u1F10\u1F11\u1F12\u1F13\u1F14\u1F15"+
	"\u1F16\u1F17\u1F18\u1F19\u1F1A\u1F1B\u1F1C\u1F1D"+
	"\u1F1E\u1F1F\u1F20\u1F21\u1F22\u1F23\u1F24\u1F25"+
	"\u1F26\u1F27\u1F28\u1F29\u1F2A\u1F2B\u1F2C\u1F2D"+
	"\u1F2E\u1F2F\u1F30\u1F31\u1F32\u1F33\u1F34\u1F35"+
	"\u1F36\u1F37\u1F38\u1F39\u1F3A\u1F3B\u1F3C\u1F3D"+
	"\u1F3E\u1F3F\u1F40\u1F41\u1F42\u1F43\u1F44\u1F45"+
	"\u1F46\u1F47\u1F48\u1F49\u1F4A\u1F4B\u1F4C\u1F4D"+
	"\u1F4E\u1F4F\u1F50\u1F51\u1F52\u1F53\u1F54\u1F55"+
	"\u1F56\u1F57\u1F58\u1F59\u1F5A\u1F5B\u1F5C\u1F5D"+
	"\u1F5E\u1F5F\u1F60\u1F61\u1F62\u1F63\u1F64\u1F65"+
	"\u1F66\u1F67\u1F68\u1F69\u1F6A\u1F6B\u1F6C\u1F6D"+
	"\u1F6E\u1F6F\u1F70\u1F71\u1F72\u1F73\u1F74\u1F75"+
	"\u1F76\u1F77\u1F78\u1F79\u1F7A\u1F7B\u1F7C\u1F7D"+
	"\u1F7E\u1F7F\u1F80\u1F81\u1F82\u1F83\u1F84\u1F85"+
	"\u1F86\u1F87\u1F88\u1F89\u1F8A\u1F8B\u1F8C\u1F8D"+
	"\u1F8E\u1F8F\u1F90\u1F91\u1F92\u1F93\u1F94\u1F95"+
	"\u1F96\u1F97\u1F98\u1F99\u1F9A\u1F9B\u1F9C\u1F9D"+
	"\u1F9E\u1F9F\u1FA0\u1FA1\u1FA2\u1FA3\u1FA4\u1FA5"+
	"\u1FA6\u1FA7\u1FA8\u1FA9\u1FAA\u1FAB\u1FAC\u1FAD"+
	"\u1FAE\u1FAF\u1FB0\u1FB1\u1FB2\u1FB3\u1FB4\u1FB5"+
	"\u1FB6\u1FB7\u1FB8\u1FB9\u1FBA\u1FBB\u1FBC\u1FBD"+
	"\u1FBE\u1FBF\u1FC0\u1FC1\u1FC2\u1FC3\u1FC4\u1FC5"+
	"\u1FC6\u1FC7\u1FC8\u1FC9\u1FCA\u1FCB\u1FCC\u1FCD"+
	"\u1FCE\u1FCF\u1FD0\u1FD1\u1FD2\u1FD3\u1FD4\u1FD5"+
	"\u1FD6\u1FD7\u1FD8\u1FD9\u1FDA\u1FDB\u1FDC\u1FDD"+
	"\u1FDE\u1FDF\u1FE0\u1FE1\u1FE2\u1FE3\u1FE4\u1FE5"+
	"\u1FE6\u1FE7\u1FE8\u1FE9\u1FEA\u1FEB\u1FEC\u1FED"+
	"\u1FEE\u1FEF\u1FF0\u1FF1\u1FF2\u1FF3\u1FF4\u1FF5"+
	"\u1FF6\u1FF7\u1FF8\u1FF9\u1FFA\u1FFB\u1FFC\u1FFD"+
	"\u1FFE\u1FFF\u2000\u2001\u2002\u2003\u2004\u2005"+
	"\u2006\u2007\u2008\u2009\u200A\u200B\u200C\u200D"+
	"\u200E\u200F\u2011\u2012\u2017\u201A\u201B\u201E"+
	"\u201F\u2020\u2021\u2022\u2023\u2024\u2027\u2028"+
	"\u2029\u202A\u202B\u202C\u202D\u202E\u202F\u2031"+
	"\u2034\u2036\u2037\u2038\u2039\u203A\u203C\u203D"+
	"\u203E\u203F\u2040\u2041\u2042\u2043\u2044\u2045"+
	"\u2046\u2047\u2048\u2049\u204A\u204B\u204C\u204D"+
	"\u204E\u204F\u2050\u2051\u2052\u2053\u2054\u2055"+
	"\u2056\u2057\u2058\u2059\u205A\u205B\u205C\u205D"+
	"\u205E\u205F\u2060\u2061\u2062\u2063\u2064\u2065"+
	"\u2066\u2067\u2068\u2069\u206A\u206B\u206C\u206D"+
	"\u206E\u206F\u2070\u2071\u2072\u2073\u2074\u2075"+
	"\u2076\u2077\u2078\u2079\u207A\u207B\u207C\u207D"+
	"\u207E\u207F\u2080\u2081\u2082\u2083\u2084\u2085"+
	"\u2086\u2087\u2088\u2089\u208A\u208B\u208C\u208D"+
	"\u208E\u208F\u2090\u2091\u2092\u2093\u2094\u2095"+
	"\u2096\u2097\u2098\u2099\u209A\u209B\u209C\u209D"+
	"\u209E\u209F\u20A0\u20A1\u20A2\u20A3\u20A4\u20A5"+
	"\u20A6\u20A7\u20A8\u20A9\u20AA\u20AB\u20AD\u20AE"+
	"\u20AF\u20B0\u20B1\u20B2\u20B3\u20B4\u20B5\u20B6"+
	"\u20B7\u20B8\u20B9\u20BA\u20BB\u20BC\u20BD\u20BE"+
	"\u20BF\u20C0\u20C1\u20C2\u20C3\u20C4\u20C5\u20C6"+
	"\u20C7\u20C8\u20C9\u20CA\u20CB\u20CC\u20CD\u20CE"+
	"\u20CF\u20D0\u20D1\u20D2\u20D3\u20D4\u20D5\u20D6"+
	"\u20D7\u20D8\u20D9\u20DA\u20DB\u20DC\u20DD\u20DE"+
	"\u20DF\u20E0\u20E1\u20E2\u20E3\u20E4\u20E5\u20E6"+
	"\u20E7\u20E8\u20E9\u20EA\u20EB\u20EC\u20ED\u20EE"+
	"\u20EF\u20F0\u20F1\u20F2\u20F3\u20F4\u20F5\u20F6"+
	"\u20F7\u20F8\u20F9\u20FA\u20FB\u20FC\u20FD\u20FE"+
	"\u20FF\u2100\u2101\u2102\u2104\u2106\u2107\u2108"+
	"\u210A\u210B\u210C\u210D\u210E\u210F\u2110\u2111"+
	"\u2112\u2113\u2114\u2115\u2117\u2118\u2119\u211A"+
	"\u211B\u211C\u211D\u211E\u211F\u2120\u2122\u2123"+
	"\u2124\u2125\u2126\u2127\u2128\u2129\u212A\u212B"+
	"\u212C\u212D\u212E\u212F\u2130\u2131\u2132\u2133";

    private final static String innerIndex2= 
	"\u2134\u2135\u2136\u2137\u2138\u2139\u213A\u213B"+
	"\u213C\u213D\u213E\u213F\u2140\u2141\u2142\u2143"+
	"\u2144\u2145\u2146\u2147\u2148\u2149\u214A\u214B"+
	"\u214C\u214D\u214E\u214F\u2150\u2151\u2152\u2153"+
	"\u2154\u2155\u2156\u2157\u2158\u2159\u215A\u215B"+
	"\u215C\u215D\u215E\u215F\u216C\u216D\u216E\u216F"+
	"\u217A\u217B\u217C\u217D\u217E\u217F\u2180\u2181"+
	"\u2182\u2183\u2184\u2185\u2186\u2187\u2188\u2189"+
	"\u218A\u218B\u218C\u218D\u218E\u218F\u2194\u2195"+
	"\u219A\u219B\u219C\u219D\u219E\u219F\u21A0\u21A1"+
	"\u21A2\u21A3\u21A4\u21A5\u21A6\u21A7\u21A8\u21A9"+
	"\u21AA\u21AB\u21AC\u21AD\u21AE\u21AF\u21B0\u21B1"+
	"\u21B2\u21B3\u21B4\u21B5\u21B6\u21B7\u21B8\u21B9"+
	"\u21BA\u21BB\u21BC\u21BD\u21BE\u21BF\u21C0\u21C1"+
	"\u21C2\u21C3\u21C4\u21C5\u21C6\u21C7\u21C8\u21C9"+
	"\u21CA\u21CB\u21CC\u21CD\u21CE\u21CF\u21D0\u21D1"+
	"\u21D2\u21D3\u21D4\u21D5\u21D6\u21D7\u21D8\u21D9"+
	"\u21DA\u21DB\u21DC\u21DD\u21DE\u21DF\u21E0\u21E1"+
	"\u21E2\u21E3\u21E4\u21E5\u21E6\u21E7\u21E8\u21E9"+
	"\u21EA\u21EB\u21EC\u21ED\u21EE\u21EF\u21F0\u21F1"+
	"\u21F2\u21F3\u21F4\u21F5\u21F6\u21F7\u21F8\u21F9"+
	"\u21FA\u21FB\u21FC\u21FD\u21FE\u21FF\u2200\u2201"+
	"\u2202\u2203\u2204\u2205\u2206\u2207\u2209\u220A"+
	"\u220B\u220C\u220D\u220E\u2210\u2212\u2213\u2214"+
	"\u2216\u2217\u2218\u2219\u221B\u221C\u2221\u2222"+
	"\u2224\u2226\u222C\u222D\u222F\u2230\u2231\u2232"+
	"\u2233\u2238\u2239\u223A\u223B\u223C\u223E\u223F"+
	"\u2240\u2241\u2242\u2243\u2244\u2245\u2246\u2247"+
	"\u2249\u224A\u224B\u224D\u224E\u224F\u2250\u2251"+
	"\u2253\u2254\u2255\u2256\u2257\u2258\u2259\u225A"+
	"\u225B\u225C\u225D\u225E\u225F\u2262\u2263\u2268"+
	"\u2269\u226A\u226B\u226C\u226D\u2270\u2271\u2272"+
	"\u2273\u2274\u2275\u2276\u2277\u2278\u2279\u227A"+
	"\u227B\u227C\u227D\u227E\u227F\u2280\u2281\u2282"+
	"\u2283\u2284\u2285\u2286\u2287\u2288\u2289\u228A"+
	"\u228B\u228C\u228D\u228E\u228F\u2290\u2291\u2292"+
	"\u2293\u2294\u2296\u2297\u2298\u229A\u229B\u229C"+
	"\u229D\u229E\u229F\u22A0\u22A1\u22A2\u22A3\u22A4"+
	"\u22A6\u22A7\u22A8\u22A9\u22AA\u22AB\u22AC\u22AD"+
	"\u22AE\u22AF\u22B0\u22B1\u22B2\u22B3\u22B4\u22B5"+
	"\u22B6\u22B7\u22B8\u22B9\u22BA\u22BB\u22BC\u22BD"+
	"\u22BE\u22C0\u22C1\u22C2\u22C3\u22C4\u22C5\u22C6"+
	"\u22C7\u22C8\u22C9\u22CA\u22CB\u22CC\u22CD\u22CE"+
	"\u22CF\u22D0\u22D1\u22D2\u22D3\u22D4\u22D5\u22D6"+
	"\u22D7\u22D8\u22D9\u22DA\u22DB\u22DC\u22DD\u22DE"+
	"\u22DF\u22E0\u22E1\u22E2\u22E3\u22E4\u22E5\u22E6"+
	"\u22E7\u22E8\u22E9\u22EA\u22EB\u22EC\u22ED\u22EE"+
	"\u22EF\u22F0\u22F1\u22F2\u22F3\u22F4\u22F5\u22F6"+
	"\u22F7\u22F8\u22F9\u22FA\u22FB\u22FC\u22FD\u22FE"+
	"\u22FF\u2300\u2301\u2302\u2303\u2304\u2305\u2306"+
	"\u2307\u2308\u2309\u230A\u230B\u230C\u230D\u230E"+
	"\u230F\u2310\u2311\u2313\u2314\u2315\u2316\u2317"+
	"\u2318\u2319\u231A\u231B\u231C\u231D\u231E\u231F"+
	"\u2320\u2321\u2322\u2323\u2324\u2325\u2326\u2327"+
	"\u2328\u2329\u232A\u232B\u232C\u232D\u232E\u232F"+
	"\u2330\u2331\u2332\u2333\u2334\u2335\u2336\u2337"+
	"\u2338\u2339\u233A\u233B\u233C\u233D\u233E\u233F"+
	"\u2340\u2341\u2342\u2343\u2344\u2345\u2346\u2347"+
	"\u2348\u2349\u234A\u234B\u234C\u234D\u234E\u234F"+
	"\u2350\u2351\u2352\u2353\u2354\u2355\u2356\u2357"+
	"\u2358\u2359\u235A\u235B\u235C\u235D\u235E\u235F"+
	"\u2360\u2361\u2362\u2363\u2364\u2365\u2366\u2367"+
	"\u2368\u2369\u236A\u236B\u236C\u236D\u236E\u236F"+
	"\u2370\u2371\u2372\u2373\u2374\u2375\u2376\u2377"+
	"\u2378\u2379\u237A\u237B\u237C\u237D\u237E\u237F"+
	"\u2380\u2381\u2382\u2383\u2384\u2385\u2386\u2387"+
	"\u2388\u2389\u238A\u238B\u238C\u238D\u238E\u238F"+
	"\u2390\u2391\u2392\u2393\u2394\u2395\u2396\u2397"+
	"\u2398\u2399\u239A\u239B\u239C\u239D\u239E\u239F"+
	"\u23A0\u23A1\u23A2\u23A3\u23A4\u23A5\u23A6\u23A7"+
	"\u23A8\u23A9\u23AA\u23AB\u23AC\u23AD\u23AE\u23AF"+
	"\u23B0\u23B1\u23B2\u23B3\u23B4\u23B5\u23B6\u23B7"+
	"\u23B8\u23B9\u23BA\u23BB\u23BC\u23BD\u23BE\u23BF"+
	"\u23C0\u23C1\u23C2\u23C3\u23C4\u23C5\u23C6\u23C7"+
	"\u23C8\u23C9\u23CA\u23CB\u23CC\u23CD\u23CE\u23CF"+
	"\u23D0\u23D1\u23D2\u23D3\u23D4\u23D5\u23D6\u23D7"+
	"\u23D8\u23D9\u23DA\u23DB\u23DC\u23DD\u23DE\u23DF"+
	"\u23E0\u23E1\u23E2\u23E3\u23E4\u23E5\u23E6\u23E7"+
	"\u23E8\u23E9\u23EA\u23EB\u23EC\u23ED\u23EE\u23EF"+
	"\u23F0\u23F1\u23F2\u23F3\u23F4\u23F5\u23F6\u23F7"+
	"\u23F8\u23F9\u23FA\u23FB\u23FC\u23FD\u23FE\u23FF"+
	"\u2400\u2401\u2402\u2403\u2404\u2405\u2406\u2407"+
	"\u2408\u2409\u240A\u240B\u240C\u240D\u240E\u240F"+
	"\u2410\u2411\u2412\u2413\u2414\u2415\u2416\u2417"+
	"\u2418\u2419\u241A\u241B\u241C\u241D\u241E\u241F"+
	"\u2420\u2421\u2422\u2423\u2424\u2425\u2426\u2427"+
	"\u2428\u2429\u242A\u242B\u242C\u242D\u242E\u242F"+
	"\u2430\u2431\u2432\u2433\u2434\u2435\u2436\u2437"+
	"\u2438\u2439\u243A\u243B\u243C\u243D\u243E\u243F"+
	"\u2440\u2441\u2442\u2443\u2444\u2445\u2446\u2447"+
	"\u2448\u2449\u244A\u244B\u244C\u244D\u244E\u244F"+
	"\u2450\u2451\u2452\u2453\u2454\u2455\u2456\u2457"+
	"\u2458\u2459\u245A\u245B\u245C\u245D\u245E\u245F"+
	"\u246A\u246B\u246C\u246D\u246E\u246F\u2470\u2471"+
	"\u2472\u2473\u249C\u249D\u249E\u249F\u24A0\u24A1"+
	"\u24A2\u24A3\u24A4\u24A5\u24A6\u24A7\u24A8\u24A9"+
	"\u24AA\u24AB\u24AC\u24AD\u24AE\u24AF\u24B0\u24B1"+
	"\u24B2\u24B3\u24B4\u24B5\u24B6\u24B7\u24B8\u24B9"+
	"\u24BA\u24BB\u24BC\u24BD\u24BE\u24BF\u24C0\u24C1"+
	"\u24C2\u24C3\u24C4\u24C5\u24C6\u24C7\u24C8\u24C9"+
	"\u24CA\u24CB\u24CC\u24CD\u24CE\u24CF\u24D0\u24D1"+
	"\u24D2\u24D3\u24D4\u24D5\u24D6\u24D7\u24D8\u24D9"+
	"\u24DA\u24DB\u24DC\u24DD\u24DE\u24DF\u24E0\u24E1"+
	"\u24E2\u24E3\u24E4\u24E5\u24E6\u24E7\u24E8\u24E9"+
	"\u24EA\u24EB\u24EC\u24ED\u24EE\u24EF\u24F0\u24F1"+
	"\u24F2\u24F3\u24F4\u24F5\u24F6\u24F7\u24F8\u24F9"+
	"\u24FA\u24FB\u24FC\u24FD\u24FE\u24FF\u254C\u254D"+
	"\u254E\u254F\u2574\u2575\u2576\u2577\u2578\u2579"+
	"\u257A\u257B\u257C\u257D\u257E\u257F\u2580\u2590"+
	"\u2591\u2592\u2596\u2597\u2598\u2599\u259A\u259B"+
	"\u259C\u259D\u259E\u259F\u25A2\u25A3\u25A4\u25A5"+
	"\u25A6\u25A7\u25A8\u25A9\u25AA\u25AB\u25AC\u25AD"+
	"\u25AE\u25AF\u25B0\u25B1\u25B4\u25B5\u25B6\u25B7"+
	"\u25B8\u25B9\u25BA\u25BB\u25BE\u25BF\u25C0\u25C1"+
	"\u25C2\u25C3\u25C4\u25C5\u25C8\u25C9\u25CA\u25CC"+
	"\u25CD\u25D0\u25D1\u25D2\u25D3\u25D4\u25D5\u25D6"+
	"\u25D7\u25D8\u25D9\u25DA\u25DB\u25DC\u25DD\u25DE"+
	"\u25DF\u25E0\u25E1\u25E6\u25E7\u25E8\u25E9\u25EA"+
	"\u25EB\u25EC\u25ED\u25EE\u25EF\u25F0\u25F1\u25F2"+
	"\u25F3\u25F4\u25F5\u25F6\u25F7\u25F8\u25F9\u25FA"+
	"\u25FB\u25FC\u25FD\u25FE\u25FF\u2600\u2601\u2602"+
	"\u2603\u2604\u2607\u2608\u260A\u260B\u260C\u260D"+
	"\u260E\u260F\u2610\u2611\u2612\u2613\u2614\u2615"+
	"\u2616\u2617\u2618\u2619\u261A\u261B\u261C\u261D"+
	"\u261E\u261F\u2620\u2621\u2622\u2623\u2624\u2625"+
	"\u2626\u2627\u2628\u2629\u262A\u262B\u262C\u262D"+
	"\u262E\u262F\u2630\u2631\u2632\u2633\u2634\u2635"+
	"\u2636\u2637\u2638\u2639\u263A\u263B\u263C\u263D"+
	"\u263E\u263F\u2641\u2643\u2644\u2645\u2646\u2647"+
	"\u2648\u2649\u264A\u264B\u264C\u264D\u264E\u264F"+
	"\u2650\u2651\u2652\u2653\u2654\u2655\u2656\u2657"+
	"\u2658\u2659\u265A\u265B\u265C\u265D\u265E\u265F"+
	"\u2660\u2661\u2662\u2663\u2664\u2665\u2666\u2667"+
	"\u2668\u2669\u266A\u266B\u266C\u266D\u266E\u266F"+
	"\u2670\u2671\u2672\u2673\u2674\u2675\u2676\u2677"+
	"\u2678\u2679\u267A\u267B\u267C\u267D\u267E\u267F"+
	"\u2680\u2681\u2682\u2683\u2684\u2685\u2686\u2687"+
	"\u2688\u2689\u268A\u268B\u268C\u268D\u268E\u268F"+
	"\u2690\u2691\u2692\u2693\u2694\u2695\u2696\u2697"+
	"\u2698\u2699\u269A\u269B\u269C\u269D\u269E\u269F"+
	"\u26A0\u26A1\u26A2\u26A3\u26A4\u26A5\u26A6\u26A7"+
	"\u26A8\u26A9\u26AA\u26AB\u26AC\u26AD\u26AE\u26AF"+
	"\u26B0\u26B1\u26B2\u26B3\u26B4\u26B5\u26B6\u26B7"+
	"\u26B8\u26B9\u26BA\u26BB\u26BC\u26BD\u26BE\u26BF"+
	"\u26C0\u26C1\u26C2\u26C3\u26C4\u26C5\u26C6\u26C7"+
	"\u26C8\u26C9\u26CA\u26CB\u26CC\u26CD\u26CE\u26CF"+
	"\u26D0\u26D1\u26D2\u26D3\u26D4\u26D5\u26D6\u26D7"+
	"\u26D8\u26D9\u26DA\u26DB\u26DC\u26DD\u26DE\u26DF"+
	"\u26E0\u26E1\u26E2\u26E3\u26E4\u26E5\u26E6\u26E7"+
	"\u26E8\u26E9\u26EA\u26EB\u26EC\u26ED\u26EE\u26EF"+
	"\u26F0\u26F1\u26F2\u26F3\u26F4\u26F5\u26F6\u26F7"+
	"\u26F8\u26F9\u26FA\u26FB\u26FC\u26FD\u26FE\u26FF"+
	"\u2700\u2701\u2702\u2703\u2704\u2705\u2706\u2707"+
	"\u2708\u2709\u270A\u270B\u270C\u270D\u270E\u270F"+
	"\u2710\u2711\u2712\u2713\u2714\u2715\u2716\u2717"+
	"\u2718\u2719\u271A\u271B\u271C\u271D\u271E\u271F"+
	"\u2720\u2721\u2722\u2723\u2724\u2725\u2726\u2727"+
	"\u2728\u2729\u272A\u272B\u272C\u272D\u272E\u272F"+
	"\u2730\u2731\u2732\u2733\u2734\u2735\u2736\u2737"+
	"\u2738\u2739\u273A\u273B\u273C\u273D\u273E\u273F"+
	"\u2740\u2741\u2742\u2743\u2744\u2745\u2746\u2747"+
	"\u2748\u2749\u274A\u274B\u274C\u274D\u274E\u274F"+
	"\u2750\u2751\u2752\u2753\u2754\u2755\u2756\u2757"+
	"\u2758\u2759\u275A\u275B\u275C\u275D\u275E\u275F"+
	"\u2760\u2761\u2762\u2763\u2764\u2765\u2766\u2767"+
	"\u2768\u2769\u276A\u276B\u276C\u276D\u276E\u276F"+
	"\u2770\u2771\u2772\u2773\u2774\u2775\u2776\u2777"+
	"\u2778\u2779\u277A\u277B\u277C\u277D\u277E\u277F"+
	"\u2780\u2781\u2782\u2783\u2784\u2785\u2786\u2787"+
	"\u2788\u2789\u278A\u278B\u278C\u278D\u278E\u278F"+
	"\u2790\u2791\u2792\u2793\u2794\u2795\u2796\u2797"+
	"\u2798\u2799\u279A\u279B\u279C\u279D\u279E\u279F"+
	"\u27A0\u27A1\u27A2\u27A3\u27A4\u27A5\u27A6\u27A7"+
	"\u27A8\u27A9\u27AA\u27AB\u27AC\u27AD\u27AE\u27AF"+
	"\u27B0\u27B1\u27B2\u27B3\u27B4\u27B5\u27B6\u27B7"+
	"\u27B8\u27B9\u27BA\u27BB\u27BC\u27BD\u27BE\u27BF"+
	"\u27C0\u27C1\u27C2\u27C3\u27C4\u27C5\u27C6\u27C7"+
	"\u27C8\u27C9\u27CA\u27CB\u27CC\u27CD\u27CE\u27CF"+
	"\u27D0\u27D1\u27D2\u27D3\u27D4\u27D5\u27D6\u27D7"+
	"\u27D8\u27D9\u27DA\u27DB\u27DC\u27DD\u27DE\u27DF"+
	"\u27E0\u27E1\u27E2\u27E3\u27E4\u27E5\u27E6\u27E7"+
	"\u27E8\u27E9\u27EA\u27EB\u27EC\u27ED\u27EE\u27EF"+
	"\u27F0\u27F1\u27F2\u27F3\u27F4\u27F5\u27F6\u27F7"+
	"\u27F8\u27F9\u27FA\u27FB\u27FC\u27FD\u27FE\u27FF"+
	"\u2800\u2801\u2802\u2803\u2804\u2805\u2806\u2807"+
	"\u2808\u2809\u280A\u280B\u280C\u280D\u280E\u280F"+
	"\u2810\u2811\u2812\u2813\u2814\u2815\u2816\u2817"+
	"\u2818\u2819\u281A\u281B\u281C\u281D\u281E\u281F"+
	"\u2820\u2821\u2822\u2823\u2824\u2825\u2826\u2827"+
	"\u2828\u2829\u282A\u282B\u282C\u282D\u282E\u282F"+
	"\u2830\u2831\u2832\u2833\u2834\u2835\u2836\u2837"+
	"\u2838\u2839\u283A\u283B\u283C\u283D\u283E\u283F"+
	"\u2840\u2841\u2842\u2843\u2844\u2845\u2846\u2847"+
	"\u2848\u2849\u284A\u284B\u284C\u284D\u284E\u284F"+
	"\u2850\u2851\u2852\u2853\u2854\u2855\u2856\u2857"+
	"\u2858\u2859\u285A\u285B\u285C\u285D\u285E\u285F"+
	"\u2860\u2861\u2862\u2863\u2864\u2865\u2866\u2867"+
	"\u2868\u2869\u286A\u286B\u286C\u286D\u286E\u286F"+
	"\u2870\u2871\u2872\u2873\u2874\u2875\u2876\u2877"+
	"\u2878\u2879\u287A\u287B\u287C\u287D\u287E\u287F"+
	"\u2880\u2881\u2882\u2883\u2884\u2885\u2886\u2887"+
	"\u2888\u2889\u288A\u288B\u288C\u288D\u288E\u288F"+
	"\u2890\u2891\u2892\u2893\u2894\u2895\u2896\u2897"+
	"\u2898\u2899\u289A\u289B\u289C\u289D\u289E\u289F"+
	"\u28A0\u28A1\u28A2\u28A3\u28A4\u28A5\u28A6\u28A7"+
	"\u28A8\u28A9\u28AA\u28AB\u28AC\u28AD\u28AE\u28AF"+
	"\u28B0\u28B1\u28B2\u28B3\u28B4\u28B5\u28B6\u28B7"+
	"\u28B8\u28B9\u28BA\u28BB\u28BC\u28BD\u28BE\u28BF"+
	"\u28C0\u28C1\u28C2\u28C3\u28C4\u28C5\u28C6\u28C7"+
	"\u28C8\u28C9\u28CA\u28CB\u28CC\u28CD\u28CE\u28CF"+
	"\u28D0\u28D1\u28D2\u28D3\u28D4\u28D5\u28D6\u28D7"+
	"\u28D8\u28D9\u28DA\u28DB\u28DC\u28DD\u28DE\u28DF"+
	"\u28E0\u28E1\u28E2\u28E3\u28E4\u28E5\u28E6\u28E7"+
	"\u28E8\u28E9\u28EA\u28EB\u28EC\u28ED\u28EE\u28EF"+
	"\u28F0\u28F1\u28F2\u28F3\u28F4\u28F5\u28F6\u28F7"+
	"\u28F8\u28F9\u28FA\u28FB\u28FC\u28FD\u28FE\u28FF"+
	"\u2900\u2901\u2902\u2903\u2904\u2905\u2906\u2907"+
	"\u2908\u2909\u290A\u290B\u290C\u290D\u290E\u290F"+
	"\u2910\u2911\u2912\u2913\u2914\u2915\u2916\u2917"+
	"\u2918\u2919\u291A\u291B\u291C\u291D\u291E\u291F"+
	"\u2920\u2921\u2922\u2923\u2924\u2925\u2926\u2927"+
	"\u2928\u2929\u292A\u292B\u292C\u292D\u292E\u292F"+
	"\u2930\u2931\u2932\u2933\u2934\u2935\u2936\u2937"+
	"\u2938\u2939\u293A\u293B\u293C\u293D\u293E\u293F"+
	"\u2940\u2941\u2942\u2943\u2944\u2945\u2946\u2947"+
	"\u2948\u2949\u294A\u294B\u294C\u294D\u294E\u294F"+
	"\u2950\u2951\u2952\u2953\u2954\u2955\u2956\u2957"+
	"\u2958\u2959\u295A\u295B\u295C\u295D\u295E\u295F"+
	"\u2960\u2961\u2962\u2963\u2964\u2965\u2966\u2967"+
	"\u2968\u2969\u296A\u296B\u296C\u296D\u296E\u296F"+
	"\u2970\u2971\u2972\u2973\u2974\u2975\u2976\u2977"+
	"\u2978\u2979\u297A\u297B\u297C\u297D\u297E\u297F"+
	"\u2980\u2981\u2982\u2983\u2984\u2985\u2986\u2987"+
	"\u2988\u2989\u298A\u298B\u298C\u298D\u298E\u298F"+
	"\u2990\u2991\u2992\u2993\u2994\u2995\u2996\u2997"+
	"\u2998\u2999\u299A\u299B\u299C\u299D\u299E\u299F"+
	"\u29A0\u29A1\u29A2\u29A3\u29A4\u29A5\u29A6\u29A7"+
	"\u29A8\u29A9\u29AA\u29AB\u29AC\u29AD\u29AE\u29AF"+
	"\u29B0\u29B1\u29B2\u29B3\u29B4\u29B5\u29B6\u29B7"+
	"\u29B8\u29B9\u29BA\u29BB\u29BC\u29BD\u29BE\u29BF"+
	"\u29C0\u29C1\u29C2\u29C3\u29C4\u29C5\u29C6\u29C7"+
	"\u29C8\u29C9\u29CA\u29CB\u29CC\u29CD\u29CE\u29CF"+
	"\u29D0\u29D1\u29D2\u29D3\u29D4\u29D5\u29D6\u29D7"+
	"\u29D8\u29D9\u29DA\u29DB\u29DC\u29DD\u29DE\u29DF"+
	"\u29E0\u29E1\u29E2\u29E3\u29E4\u29E5\u29E6\u29E7"+
	"\u29E8\u29E9\u29EA\u29EB\u29EC\u29ED\u29EE\u29EF"+
	"\u29F0\u29F1\u29F2\u29F3\u29F4\u29F5\u29F6\u29F7"+
	"\u29F8\u29F9\u29FA\u29FB\u29FC\u29FD\u29FE\u29FF"+
	"\u2A00\u2A01\u2A02\u2A03\u2A04\u2A05\u2A06\u2A07"+
	"\u2A08\u2A09\u2A0A\u2A0B\u2A0C\u2A0D\u2A0E\u2A0F"+
	"\u2A10\u2A11\u2A12\u2A13\u2A14\u2A15\u2A16\u2A17"+
	"\u2A18\u2A19\u2A1A\u2A1B\u2A1C\u2A1D\u2A1E\u2A1F"+
	"\u2A20\u2A21\u2A22\u2A23\u2A24\u2A25\u2A26\u2A27"+
	"\u2A28\u2A29\u2A2A\u2A2B\u2A2C\u2A2D\u2A2E\u2A2F"+
	"\u2A30\u2A31\u2A32\u2A33\u2A34\u2A35\u2A36\u2A37"+
	"\u2A38\u2A39\u2A3A\u2A3B\u2A3C\u2A3D\u2A3E\u2A3F"+
	"\u2A40\u2A41\u2A42\u2A43\u2A44\u2A45\u2A46\u2A47"+
	"\u2A48\u2A49\u2A4A\u2A4B\u2A4C\u2A4D\u2A4E\u2A4F"+
	"\u2A50\u2A51\u2A52\u2A53\u2A54\u2A55\u2A56\u2A57"+
	"\u2A58\u2A59\u2A5A\u2A5B\u2A5C\u2A5D\u2A5E\u2A5F"+
	"\u2A60\u2A61\u2A62\u2A63\u2A64\u2A65\u2A66\u2A67"+
	"\u2A68\u2A69\u2A6A\u2A6B\u2A6C\u2A6D\u2A6E\u2A6F"+
	"\u2A70\u2A71\u2A72\u2A73\u2A74\u2A75\u2A76\u2A77"+
	"\u2A78\u2A79\u2A7A\u2A7B\u2A7C\u2A7D\u2A7E\u2A7F"+
	"\u2A80\u2A81\u2A82\u2A83\u2A84\u2A85\u2A86\u2A87"+
	"\u2A88\u2A89\u2A8A\u2A8B\u2A8C\u2A8D\u2A8E\u2A8F"+
	"\u2A90\u2A91\u2A92\u2A93\u2A94\u2A95\u2A96\u2A97"+
	"\u2A98\u2A99\u2A9A\u2A9B\u2A9C\u2A9D\u2A9E\u2A9F"+
	"\u2AA0\u2AA1\u2AA2\u2AA3\u2AA4\u2AA5\u2AA6\u2AA7"+
	"\u2AA8\u2AA9\u2AAA\u2AAB\u2AAC\u2AAD\u2AAE\u2AAF"+
	"\u2AB0\u2AB1\u2AB2\u2AB3\u2AB4\u2AB5\u2AB6\u2AB7"+
	"\u2AB8\u2AB9\u2ABA\u2ABB\u2ABC\u2ABD\u2ABE\u2ABF"+
	"\u2AC0\u2AC1\u2AC2\u2AC3\u2AC4\u2AC5\u2AC6\u2AC7"+
	"\u2AC8\u2AC9\u2ACA\u2ACB\u2ACC\u2ACD\u2ACE\u2ACF"+
	"\u2AD0\u2AD1\u2AD2\u2AD3\u2AD4\u2AD5\u2AD6\u2AD7"+
	"\u2AD8\u2AD9\u2ADA\u2ADB\u2ADC\u2ADD\u2ADE\u2ADF"+
	"\u2AE0\u2AE1\u2AE2\u2AE3\u2AE4\u2AE5\u2AE6\u2AE7"+
	"\u2AE8\u2AE9\u2AEA\u2AEB\u2AEC\u2AED\u2AEE\u2AEF"+
	"\u2AF0\u2AF1\u2AF2\u2AF3\u2AF4\u2AF5\u2AF6\u2AF7"+
	"\u2AF8\u2AF9\u2AFA\u2AFB\u2AFC\u2AFD\u2AFE\u2AFF"+
	"\u2B00\u2B01\u2B02\u2B03\u2B04\u2B05\u2B06\u2B07"+
	"\u2B08\u2B09\u2B0A\u2B0B\u2B0C\u2B0D\u2B0E\u2B0F"+
	"\u2B10\u2B11\u2B12\u2B13\u2B14\u2B15\u2B16\u2B17"+
	"\u2B18\u2B19\u2B1A\u2B1B\u2B1C\u2B1D\u2B1E\u2B1F"+
	"\u2B20\u2B21\u2B22\u2B23\u2B24\u2B25\u2B26\u2B27"+
	"\u2B28\u2B29\u2B2A\u2B2B\u2B2C\u2B2D\u2B2E\u2B2F"+
	"\u2B30\u2B31\u2B32\u2B33\u2B34\u2B35\u2B36\u2B37"+
	"\u2B38\u2B39\u2B3A\u2B3B\u2B3C\u2B3D\u2B3E\u2B3F"+
	"\u2B40\u2B41\u2B42\u2B43\u2B44\u2B45\u2B46\u2B47"+
	"\u2B48\u2B49\u2B4A\u2B4B\u2B4C\u2B4D\u2B4E\u2B4F"+
	"\u2B50\u2B51\u2B52\u2B53\u2B54\u2B55\u2B56\u2B57"+
	"\u2B58\u2B59\u2B5A\u2B5B\u2B5C\u2B5D\u2B5E\u2B5F"+
	"\u2B60\u2B61\u2B62\u2B63\u2B64\u2B65\u2B66\u2B67"+
	"\u2B68\u2B69\u2B6A\u2B6B\u2B6C\u2B6D\u2B6E\u2B6F"+
	"\u2B70\u2B71\u2B72\u2B73\u2B74\u2B75\u2B76\u2B77"+
	"\u2B78\u2B79\u2B7A\u2B7B\u2B7C\u2B7D\u2B7E\u2B7F"+
	"\u2B80\u2B81\u2B82\u2B83\u2B84\u2B85\u2B86\u2B87"+
	"\u2B88\u2B89\u2B8A\u2B8B\u2B8C\u2B8D\u2B8E\u2B8F"+
	"\u2B90\u2B91\u2B92\u2B93\u2B94\u2B95\u2B96\u2B97"+
	"\u2B98\u2B99\u2B9A\u2B9B\u2B9C\u2B9D\u2B9E\u2B9F"+
	"\u2BA0\u2BA1\u2BA2\u2BA3\u2BA4\u2BA5\u2BA6\u2BA7"+
	"\u2BA8\u2BA9\u2BAA\u2BAB\u2BAC\u2BAD\u2BAE\u2BAF"+
	"\u2BB0\u2BB1\u2BB2\u2BB3\u2BB4\u2BB5\u2BB6\u2BB7"+
	"\u2BB8\u2BB9\u2BBA\u2BBB\u2BBC\u2BBD\u2BBE\u2BBF"+
	"\u2BC0\u2BC1\u2BC2\u2BC3\u2BC4\u2BC5\u2BC6\u2BC7"+
	"\u2BC8\u2BC9\u2BCA\u2BCB\u2BCC\u2BCD\u2BCE\u2BCF"+
	"\u2BD0\u2BD1\u2BD2\u2BD3\u2BD4\u2BD5\u2BD6\u2BD7"+
	"\u2BD8\u2BD9\u2BDA\u2BDB\u2BDC\u2BDD\u2BDE\u2BDF"+
	"\u2BE0\u2BE1\u2BE2\u2BE3\u2BE4\u2BE5\u2BE6\u2BE7"+
	"\u2BE8\u2BE9\u2BEA\u2BEB\u2BEC\u2BED\u2BEE\u2BEF"+
	"\u2BF0\u2BF1\u2BF2\u2BF3\u2BF4\u2BF5\u2BF6\u2BF7"+
	"\u2BF8\u2BF9\u2BFA\u2BFB\u2BFC\u2BFD\u2BFE\u2BFF"+
	"\u2C00\u2C01\u2C02\u2C03\u2C04\u2C05\u2C06\u2C07"+
	"\u2C08\u2C09\u2C0A\u2C0B\u2C0C\u2C0D\u2C0E\u2C0F"+
	"\u2C10\u2C11\u2C12\u2C13\u2C14\u2C15\u2C16\u2C17"+
	"\u2C18\u2C19\u2C1A\u2C1B\u2C1C\u2C1D\u2C1E\u2C1F"+
	"\u2C20\u2C21\u2C22\u2C23\u2C24\u2C25\u2C26\u2C27"+
	"\u2C28\u2C29\u2C2A\u2C2B\u2C2C\u2C2D\u2C2E\u2C2F"+
	"\u2C30\u2C31\u2C32\u2C33\u2C34\u2C35\u2C36\u2C37"+
	"\u2C38\u2C39\u2C3A\u2C3B\u2C3C\u2C3D\u2C3E\u2C3F"+
	"\u2C40\u2C41\u2C42\u2C43\u2C44\u2C45\u2C46\u2C47"+
	"\u2C48\u2C49\u2C4A\u2C4B\u2C4C\u2C4D\u2C4E\u2C4F"+
	"\u2C50\u2C51\u2C52\u2C53\u2C54\u2C55\u2C56\u2C57"+
	"\u2C58\u2C59\u2C5A\u2C5B\u2C5C\u2C5D\u2C5E\u2C5F"+
	"\u2C60\u2C61\u2C62\u2C63\u2C64\u2C65\u2C66\u2C67"+
	"\u2C68\u2C69\u2C6A\u2C6B\u2C6C\u2C6D\u2C6E\u2C6F"+
	"\u2C70\u2C71\u2C72\u2C73\u2C74\u2C75\u2C76\u2C77"+
	"\u2C78\u2C79\u2C7A\u2C7B\u2C7C\u2C7D\u2C7E\u2C7F"+
	"\u2C80\u2C81\u2C82\u2C83\u2C84\u2C85\u2C86\u2C87"+
	"\u2C88\u2C89\u2C8A\u2C8B\u2C8C\u2C8D\u2C8E\u2C8F"+
	"\u2C90\u2C91\u2C92\u2C93\u2C94\u2C95\u2C96\u2C97"+
	"\u2C98\u2C99\u2C9A\u2C9B\u2C9C\u2C9D\u2C9E\u2C9F"+
	"\u2CA0\u2CA1\u2CA2\u2CA3\u2CA4\u2CA5\u2CA6\u2CA7"+
	"\u2CA8\u2CA9\u2CAA\u2CAB\u2CAC\u2CAD\u2CAE\u2CAF"+
	"\u2CB0\u2CB1\u2CB2\u2CB3\u2CB4\u2CB5\u2CB6\u2CB7"+
	"\u2CB8\u2CB9\u2CBA\u2CBB\u2CBC\u2CBD\u2CBE\u2CBF"+
	"\u2CC0\u2CC1\u2CC2\u2CC3\u2CC4\u2CC5\u2CC6\u2CC7"+
	"\u2CC8\u2CC9\u2CCA\u2CCB\u2CCC\u2CCD\u2CCE\u2CCF"+
	"\u2CD0\u2CD1\u2CD2\u2CD3\u2CD4\u2CD5\u2CD6\u2CD7"+
	"\u2CD8\u2CD9\u2CDA\u2CDB\u2CDC\u2CDD\u2CDE\u2CDF"+
	"\u2CE0\u2CE1\u2CE2\u2CE3\u2CE4\u2CE5\u2CE6\u2CE7"+
	"\u2CE8\u2CE9\u2CEA\u2CEB\u2CEC\u2CED\u2CEE\u2CEF"+
	"\u2CF0\u2CF1\u2CF2\u2CF3\u2CF4\u2CF5\u2CF6\u2CF7"+
	"\u2CF8\u2CF9\u2CFA\u2CFB\u2CFC\u2CFD\u2CFE\u2CFF"+
	"\u2D00\u2D01\u2D02\u2D03\u2D04\u2D05\u2D06\u2D07"+
	"\u2D08\u2D09\u2D0A\u2D0B\u2D0C\u2D0D\u2D0E\u2D0F"+
	"\u2D10\u2D11\u2D12\u2D13\u2D14\u2D15\u2D16\u2D17"+
	"\u2D18\u2D19\u2D1A\u2D1B\u2D1C\u2D1D\u2D1E\u2D1F"+
	"\u2D20\u2D21\u2D22\u2D23\u2D24\u2D25\u2D26\u2D27"+
	"\u2D28\u2D29\u2D2A\u2D2B\u2D2C\u2D2D\u2D2E\u2D2F"+
	"\u2D30\u2D31\u2D32\u2D33\u2D34\u2D35\u2D36\u2D37"+
	"\u2D38\u2D39\u2D3A\u2D3B\u2D3C\u2D3D\u2D3E\u2D3F"+
	"\u2D40\u2D41\u2D42\u2D43\u2D44\u2D45\u2D46\u2D47"+
	"\u2D48\u2D49\u2D4A\u2D4B\u2D4C\u2D4D\u2D4E\u2D4F"+
	"\u2D50\u2D51\u2D52\u2D53\u2D54\u2D55\u2D56\u2D57"+
	"\u2D58\u2D59\u2D5A\u2D5B\u2D5C\u2D5D\u2D5E\u2D5F"+
	"\u2D60\u2D61\u2D62\u2D63\u2D64\u2D65\u2D66\u2D67"+
	"\u2D68\u2D69\u2D6A\u2D6B\u2D6C\u2D6D\u2D6E\u2D6F"+
	"\u2D70\u2D71\u2D72\u2D73\u2D74\u2D75\u2D76\u2D77"+
	"\u2D78\u2D79\u2D7A\u2D7B\u2D7C\u2D7D\u2D7E\u2D7F"+
	"\u2D80\u2D81\u2D82\u2D83\u2D84\u2D85\u2D86\u2D87"+
	"\u2D88\u2D89\u2D8A\u2D8B\u2D8C\u2D8D\u2D8E\u2D8F"+
	"\u2D90\u2D91\u2D92\u2D93\u2D94\u2D95\u2D96\u2D97"+
	"\u2D98\u2D99\u2D9A\u2D9B\u2D9C\u2D9D\u2D9E\u2D9F"+
	"\u2DA0\u2DA1\u2DA2\u2DA3\u2DA4\u2DA5\u2DA6\u2DA7"+
	"\u2DA8\u2DA9\u2DAA\u2DAB\u2DAC\u2DAD\u2DAE\u2DAF"+
	"\u2DB0\u2DB1\u2DB2\u2DB3\u2DB4\u2DB5\u2DB6\u2DB7"+
	"\u2DB8\u2DB9\u2DBA\u2DBB\u2DBC\u2DBD\u2DBE\u2DBF"+
	"\u2DC0\u2DC1\u2DC2\u2DC3\u2DC4\u2DC5\u2DC6\u2DC7"+
	"\u2DC8\u2DC9\u2DCA\u2DCB\u2DCC\u2DCD\u2DCE\u2DCF"+
	"\u2DD0\u2DD1\u2DD2\u2DD3\u2DD4\u2DD5\u2DD6\u2DD7"+
	"\u2DD8\u2DD9\u2DDA\u2DDB\u2DDC\u2DDD\u2DDE\u2DDF"+
	"\u2DE0\u2DE1\u2DE2\u2DE3\u2DE4\u2DE5\u2DE6\u2DE7"+
	"\u2DE8\u2DE9\u2DEA\u2DEB\u2DEC\u2DED\u2DEE\u2DEF"+
	"\u2DF0\u2DF1\u2DF2\u2DF3\u2DF4\u2DF5\u2DF6\u2DF7"+
	"\u2DF8\u2DF9\u2DFA\u2DFB\u2DFC\u2DFD\u2DFE\u2DFF"+
	"\u2E00\u2E01\u2E02\u2E03\u2E04\u2E05\u2E06\u2E07"+
	"\u2E08\u2E09\u2E0A\u2E0B\u2E0C\u2E0D\u2E0E\u2E0F"+
	"\u2E10\u2E11\u2E12\u2E13\u2E14\u2E15\u2E16\u2E17"+
	"\u2E18\u2E19\u2E1A\u2E1B\u2E1C\u2E1D\u2E1E\u2E1F"+
	"\u2E20\u2E21\u2E22\u2E23\u2E24\u2E25\u2E26\u2E27"+
	"\u2E28\u2E29\u2E2A\u2E2B\u2E2C\u2E2D\u2E2E\u2E2F"+
	"\u2E30\u2E31\u2E32\u2E33\u2E34\u2E35\u2E36\u2E37"+
	"\u2E38\u2E39\u2E3A\u2E3B\u2E3C\u2E3D\u2E3E\u2E3F"+
	"\u2E40\u2E41\u2E42\u2E43\u2E44\u2E45\u2E46\u2E47"+
	"\u2E48\u2E49\u2E4A\u2E4B\u2E4C\u2E4D\u2E4E\u2E4F"+
	"\u2E50\u2E51\u2E52\u2E53\u2E54\u2E55\u2E56\u2E57"+
	"\u2E58\u2E59\u2E5A\u2E5B\u2E5C\u2E5D\u2E5E\u2E5F"+
	"\u2E60\u2E61\u2E62\u2E63\u2E64\u2E65\u2E66\u2E67"+
	"\u2E68\u2E69\u2E6A\u2E6B\u2E6C\u2E6D\u2E6E\u2E6F"+
	"\u2E70\u2E71\u2E72\u2E73\u2E74\u2E75\u2E76\u2E77"+
	"\u2E78\u2E79\u2E7A\u2E7B\u2E7C\u2E7D\u2E7E\u2E7F"+
	"\u2E80\u2E82\u2E83\u2E85\u2E86\u2E87\u2E89\u2E8A"+
	"\u2E8D\u2E8E\u2E8F\u2E90\u2E91\u2E92\u2E93\u2E94"+
	"\u2E95\u2E96\u2E98\u2E99\u2E9A\u2E9B\u2E9C\u2E9D"+
	"\u2E9E\u2E9F\u2EA0\u2EA1\u2EA2\u2EA3\u2EA4\u2EA5"+
	"\u2EA6\u2EA8\u2EA9\u2EAB\u2EAC\u2EAD\u2EAF\u2EB0"+
	"\u2EB1\u2EB2\u2EB4\u2EB5\u2EB8\u2EB9\u2EBA\u2EBC"+
	"\u2EBD\u2EBE\u2EBF\u2EC0\u2EC1\u2EC2\u2EC3\u2EC4"+
	"\u2EC5\u2EC6\u2EC7\u2EC8\u2EC9\u2ECB\u2ECC\u2ECD"+
	"\u2ECE\u2ECF\u2ED0\u2ED1\u2ED2\u2ED3\u2ED4\u2ED5"+
	"\u2ED6\u2ED7\u2ED8\u2ED9\u2EDA\u2EDB\u2EDC\u2EDD"+
	"\u2EDE\u2EDF\u2EE0\u2EE1\u2EE2\u2EE3\u2EE4\u2EE5"+
	"\u2EE6\u2EE7\u2EE8\u2EE9\u2EEA\u2EEB\u2EEC\u2EED"+
	"\u2EEE\u2EEF\u2EF0\u2EF1\u2EF2\u2EF3\u2EF4\u2EF5"+
	"\u2EF6\u2EF7\u2EF8\u2EF9\u2EFA\u2EFB\u2EFC\u2EFD"+
	"\u2EFE\u2EFF\u2F00\u2F01\u2F02\u2F03\u2F04\u2F05"+
	"\u2F06\u2F07\u2F08\u2F09\u2F0A\u2F0B\u2F0C\u2F0D"+
	"\u2F0E\u2F0F\u2F10\u2F11\u2F12\u2F13\u2F14\u2F15"+
	"\u2F16\u2F17\u2F18\u2F19\u2F1A\u2F1B\u2F1C\u2F1D"+
	"\u2F1E\u2F1F\u2F20\u2F21\u2F22\u2F23\u2F24\u2F25"+
	"\u2F26\u2F27\u2F28\u2F29\u2F2A\u2F2B\u2F2C\u2F2D"+
	"\u2F2E\u2F2F\u2F30\u2F31\u2F32\u2F33\u2F34\u2F35"+
	"\u2F36\u2F37\u2F38\u2F39\u2F3A\u2F3B\u2F3C\u2F3D"+
	"\u2F3E\u2F3F\u2F40\u2F41\u2F42\u2F43\u2F44\u2F45"+
	"\u2F46\u2F47\u2F48\u2F49\u2F4A\u2F4B\u2F4C\u2F4D"+
	"\u2F4E\u2F4F\u2F50\u2F51\u2F52\u2F53\u2F54\u2F55"+
	"\u2F56\u2F57\u2F58\u2F59\u2F5A\u2F5B\u2F5C\u2F5D"+
	"\u2F5E\u2F5F\u2F60\u2F61\u2F62\u2F63\u2F64\u2F65"+
	"\u2F66\u2F67\u2F68\u2F69\u2F6A\u2F6B\u2F6C\u2F6D"+
	"\u2F6E\u2F6F\u2F70\u2F71\u2F72\u2F73\u2F74\u2F75"+
	"\u2F76\u2F77\u2F78\u2F79\u2F7A\u2F7B\u2F7C\u2F7D"+
	"\u2F7E\u2F7F\u2F80\u2F81\u2F82\u2F83\u2F84\u2F85"+
	"\u2F86\u2F87\u2F88\u2F89\u2F8A\u2F8B\u2F8C\u2F8D"+
	"\u2F8E\u2F8F\u2F90\u2F91\u2F92\u2F93\u2F94\u2F95"+
	"\u2F96\u2F97\u2F98\u2F99\u2F9A\u2F9B\u2F9C\u2F9D"+
	"\u2F9E\u2F9F\u2FA0\u2FA1\u2FA2\u2FA3\u2FA4\u2FA5"+
	"\u2FA6\u2FA7\u2FA8\u2FA9\u2FAA\u2FAB\u2FAC\u2FAD"+
	"\u2FAE\u2FAF\u2FB0\u2FB1\u2FB2\u2FB3\u2FB4\u2FB5"+
	"\u2FB6\u2FB7\u2FB8\u2FB9\u2FBA\u2FBB\u2FBC\u2FBD"+
	"\u2FBE\u2FBF\u2FC0\u2FC1\u2FC2\u2FC3\u2FC4\u2FC5"+
	"\u2FC6\u2FC7\u2FC8\u2FC9\u2FCA\u2FCB\u2FCC\u2FCD"+
	"\u2FCE\u2FCF\u2FD0\u2FD1\u2FD2\u2FD3\u2FD4\u2FD5"+
	"\u2FD6\u2FD7\u2FD8\u2FD9\u2FDA\u2FDB\u2FDC\u2FDD"+
	"\u2FDE\u2FDF\u2FE0\u2FE1\u2FE2\u2FE3\u2FE4\u2FE5"+
	"\u2FE6\u2FE7\u2FE8\u2FE9\u2FEA\u2FEB\u2FEC\u2FED"+
	"\u2FEE\u2FEF\u2FFC\u2FFD\u2FFE\u2FFF\u3004\u3018"+
	"\u3019\u301A\u301B\u301C\u301F\u3020\u302A\u302B"+
	"\u302C\u302D\u302E\u302F\u3030\u3031\u3032\u3033"+
	"\u3034\u3035\u3036\u3037\u3038\u3039\u303A\u303B"+
	"\u303C\u303D\u303F\u3040\u3094\u3095\u3096\u3097"+
	"\u3098\u3099\u309A\u309F\u30A0\u30F7\u30F8\u30F9"+
	"\u30FA\u30FB\u30FF\u3100\u3101\u3102\u3103\u3104"+
	"\u312A\u312B\u312C\u312D\u312E\u312F\u3130\u3131"+
	"\u3132\u3133\u3134\u3135\u3136\u3137\u3138\u3139"+
	"\u313A\u313B\u313C\u313D\u313E\u313F\u3140\u3141"+
	"\u3142\u3143\u3144\u3145\u3146\u3147\u3148\u3149"+
	"\u314A\u314B\u314C\u314D\u314E\u314F\u3150\u3151"+
	"\u3152\u3153\u3154\u3155\u3156\u3157\u3158\u3159"+
	"\u315A\u315B\u315C\u315D\u315E\u315F\u3160\u3161"+
	"\u3162\u3163\u3164\u3165\u3166\u3167\u3168\u3169"+
	"\u316A\u316B\u316C\u316D\u316E\u316F\u3170\u3171"+
	"\u3172\u3173\u3174\u3175\u3176\u3177\u3178\u3179"+
	"\u317A\u317B\u317C\u317D\u317E\u317F\u3180\u3181"+
	"\u3182\u3183\u3184\u3185\u3186\u3187\u3188\u3189"+
	"\u318A\u318B\u318C\u318D\u318E\u318F\u3190\u3191"+
	"\u3192\u3193\u3194\u3195\u3196\u3197\u3198\u3199"+
	"\u319A\u319B\u319C\u319D\u319E\u319F\u31A0\u31A1"+
	"\u31A2\u31A3\u31A4\u31A5\u31A6\u31A7\u31A8\u31A9"+
	"\u31AA\u31AB\u31AC\u31AD\u31AE\u31AF\u31B0\u31B1"+
	"\u31B2\u31B3\u31B4\u31B5\u31B6\u31B7\u31B8\u31B9"+
	"\u31BA\u31BB\u31BC\u31BD\u31BE\u31BF\u31C0\u31C1"+
	"\u31C2\u31C3\u31C4\u31C5\u31C6\u31C7\u31C8\u31C9"+
	"\u31CA\u31CB\u31CC\u31CD\u31CE\u31CF\u31D0\u31D1"+
	"\u31D2\u31D3\u31D4\u31D5\u31D6\u31D7\u31D8\u31D9"+
	"\u31DA\u31DB\u31DC\u31DD\u31DE\u31DF\u31E0\u31E1"+
	"\u31E2\u31E3\u31E4\u31E5\u31E6\u31E7\u31E8\u31E9"+
	"\u31EA\u31EB\u31EC\u31ED\u31EE\u31EF\u31F0\u31F1"+
	"\u31F2\u31F3\u31F4\u31F5\u31F6\u31F7\u31F8\u31F9"+
	"\u31FA\u31FB\u31FC\u31FD\u31FE\u31FF\u3200\u3201"+
	"\u3202\u3203\u3204\u3205\u3206\u3207\u3208\u3209"+
	"\u320A\u320B\u320C\u320D\u320E\u320F\u3210\u3211"+
	"\u3212\u3213\u3214\u3215\u3216\u3217\u3218\u3219"+
	"\u321A\u321B\u321C\u321D\u321E\u321F\u322A\u322B"+
	"\u322C\u322D\u322E\u322F\u3230\u3232\u3233\u3234"+
	"\u3235\u3236\u3237\u3238\u3239\u323A\u323B\u323C"+
	"\u323D\u323E\u323F\u3240\u3241\u3242\u3243\u3244"+
	"\u3245\u3246\u3247\u3248\u3249\u324A\u324B\u324C"+
	"\u324D\u324E\u324F\u3250\u3251\u3252\u3253\u3254"+
	"\u3255\u3256\u3257\u3258\u3259\u325A\u325B\u325C"+
	"\u325D\u325E\u325F\u3260\u3261\u3262\u3263\u3264"+
	"\u3265\u3266\u3267\u3268\u3269\u326A\u326B\u326C"+
	"\u326D\u326E\u326F\u3270\u3271\u3272\u3273\u3274"+
	"\u3275\u3276\u3277\u3278\u3279\u327A\u327B\u327C"+
	"\u327D\u327E\u327F\u3280\u3281\u3282\u3283\u3284"+
	"\u3285\u3286\u3287\u3288\u3289\u328A\u328B\u328C"+
	"\u328D\u328E\u328F\u3290\u3291\u3292\u3293\u3294"+
	"\u3295\u3296\u3297\u3298\u3299\u329A\u329B\u329C"+
	"\u329D\u329E\u329F\u32A0\u32A1\u32A2\u32A4\u32A5"+
	"\u32A6\u32A7\u32A8\u32A9\u32AA\u32AB\u32AC\u32AD"+
	"\u32AE\u32AF\u32B0\u32B1\u32B2\u32B3\u32B4\u32B5"+
	"\u32B6\u32B7\u32B8\u32B9\u32BA\u32BB\u32BC\u32BD"+
	"\u32BE\u32BF\u32C0\u32C1\u32C2\u32C3\u32C4\u32C5"+
	"\u32C6\u32C7\u32C8\u32C9\u32CA\u32CB\u32CC\u32CD"+
	"\u32CE\u32CF\u32D0\u32D1\u32D2\u32D3\u32D4\u32D5"+
	"\u32D6\u32D7\u32D8\u32D9\u32DA\u32DB\u32DC\u32DD"+
	"\u32DE\u32DF\u32E0\u32E1\u32E2\u32E3\u32E4\u32E5"+
	"\u32E6\u32E7\u32E8\u32E9\u32EA\u32EB\u32EC\u32ED"+
	"\u32EE\u32EF\u32F0\u32F1\u32F2\u32F3\u32F4\u32F5"+
	"\u32F6\u32F7\u32F8\u32F9\u32FA\u32FB\u32FC\u32FD"+
	"\u32FE\u32FF\u3300\u3301\u3302\u3303\u3304\u3305"+
	"\u3306\u3307\u3308\u3309\u330A\u330B\u330C\u330D"+
	"\u330E\u330F\u3310\u3311\u3312\u3313\u3314\u3315"+
	"\u3316\u3317\u3318\u3319\u331A\u331B\u331C\u331D"+
	"\u331E\u331F\u3320\u3321\u3322\u3323\u3324\u3325"+
	"\u3326\u3327\u3328\u3329\u332A\u332B\u332C\u332D"+
	"\u332E\u332F\u3330\u3331\u3332\u3333\u3334\u3335"+
	"\u3336\u3337\u3338\u3339\u333A\u333B\u333C\u333D"+
	"\u333E\u333F\u3340\u3341\u3342\u3343\u3344\u3345"+
	"\u3346\u3347\u3348\u3349\u334A\u334B\u334C\u334D"+
	"\u334E\u334F\u3350\u3351\u3352\u3353\u3354\u3355"+
	"\u3356\u3357\u3358\u3359\u335A\u335B\u335C\u335D";

    private final static String innerIndex3= 
	"\u335E\u335F\u3360\u3361\u3362\u3363\u3364\u3365"+
	"\u3366\u3367\u3368\u3369\u336A\u336B\u336C\u336D"+
	"\u336E\u336F\u3370\u3371\u3372\u3373\u3374\u3375"+
	"\u3376\u3377\u3378\u3379\u337A\u337B\u337C\u337D"+
	"\u337E\u337F\u3380\u3381\u3382\u3383\u3384\u3385"+
	"\u3386\u3387\u3388\u3389\u338A\u338B\u338C\u338D"+
	"\u3390\u3391\u3392\u3393\u3394\u3395\u3396\u3397"+
	"\u3398\u3399\u339A\u339B\u339F\u33A0\u33A2\u33A3"+
	"\u33A4\u33A5\u33A6\u33A7\u33A8\u33A9\u33AA\u33AB"+
	"\u33AC\u33AD\u33AE\u33AF\u33B0\u33B1\u33B2\u33B3"+
	"\u33B4\u33B5\u33B6\u33B7\u33B8\u33B9\u33BA\u33BB"+
	"\u33BC\u33BD\u33BE\u33BF\u33C0\u33C1\u33C2\u33C3"+
	"\u33C5\u33C6\u33C7\u33C8\u33C9\u33CA\u33CB\u33CC"+
	"\u33CD\u33CF\u33D0\u33D3\u33D4\u33D6\u33D7\u33D8"+
	"\u33D9\u33DA\u33DB\u33DC\u33DD\u33DE\u33DF\u33E0"+
	"\u33E1\u33E2\u33E3\u33E4\u33E5\u33E6\u33E7\u33E8"+
	"\u33E9\u33EA\u33EB\u33EC\u33ED\u33EE\u33EF\u33F0"+
	"\u33F1\u33F2\u33F3\u33F4\u33F5\u33F6\u33F7\u33F8"+
	"\u33F9\u33FA\u33FB\u33FC\u33FD\u33FE\u33FF\u3400"+
	"\u3401\u3402\u3403\u3404\u3405\u3406\u3407\u3408"+
	"\u3409\u340A\u340B\u340C\u340D\u340E\u340F\u3410"+
	"\u3411\u3412\u3413\u3414\u3415\u3416\u3417\u3418"+
	"\u3419\u341A\u341B\u341C\u341D\u341E\u341F\u3420"+
	"\u3421\u3422\u3423\u3424\u3425\u3426\u3427\u3428"+
	"\u3429\u342A\u342B\u342C\u342D\u342E\u342F\u3430"+
	"\u3431\u3432\u3433\u3434\u3435\u3436\u3437\u3438"+
	"\u3439\u343A\u343B\u343C\u343D\u343E\u343F\u3440"+
	"\u3441\u3442\u3443\u3444\u3445\u3446\u3448\u3449"+
	"\u344A\u344B\u344C\u344D\u344E\u344F\u3450\u3451"+
	"\u3452\u3453\u3454\u3455\u3456\u3457\u3458\u3459"+
	"\u345A\u345B\u345C\u345D\u345E\u345F\u3460\u3461"+
	"\u3462\u3463\u3464\u3465\u3466\u3467\u3468\u3469"+
	"\u346A\u346B\u346C\u346D\u346E\u346F\u3470\u3471"+
	"\u3472\u3474\u3475\u3476\u3477\u3478\u3479\u347A"+
	"\u347B\u347C\u347D\u347E\u347F\u3480\u3481\u3482"+
	"\u3483\u3484\u3485\u3486\u3487\u3488\u3489\u348A"+
	"\u348B\u348C\u348D\u348E\u348F\u3490\u3491\u3492"+
	"\u3493\u3494\u3495\u3496\u3497\u3498\u3499\u349A"+
	"\u349B\u349C\u349D\u349E\u349F\u34A0\u34A1\u34A2"+
	"\u34A3\u34A4\u34A5\u34A6\u34A7\u34A8\u34A9\u34AA"+
	"\u34AB\u34AC\u34AD\u34AE\u34AF\u34B0\u34B1\u34B2"+
	"\u34B3\u34B4\u34B5\u34B6\u34B7\u34B8\u34B9\u34BA"+
	"\u34BB\u34BC\u34BD\u34BE\u34BF\u34C0\u34C1\u34C2"+
	"\u34C3\u34C4\u34C5\u34C6\u34C7\u34C8\u34C9\u34CA"+
	"\u34CB\u34CC\u34CD\u34CE\u34CF\u34D0\u34D1\u34D2"+
	"\u34D3\u34D4\u34D5\u34D6\u34D7\u34D8\u34D9\u34DA"+
	"\u34DB\u34DC\u34DD\u34DE\u34DF\u34E0\u34E1\u34E2"+
	"\u34E3\u34E4\u34E5\u34E6\u34E7\u34E8\u34E9\u34EA"+
	"\u34EB\u34EC\u34ED\u34EE\u34EF\u34F0\u34F1\u34F2"+
	"\u34F3\u34F4\u34F5\u34F6\u34F7\u34F8\u34F9\u34FA"+
	"\u34FB\u34FC\u34FD\u34FE\u34FF\u3500\u3501\u3502"+
	"\u3503\u3504\u3505\u3506\u3507\u3508\u3509\u350A"+
	"\u350B\u350C\u350D\u350E\u350F\u3510\u3511\u3512"+
	"\u3513\u3514\u3515\u3516\u3517\u3518\u3519\u351A"+
	"\u351B\u351C\u351D\u351E\u351F\u3520\u3521\u3522"+
	"\u3523\u3524\u3525\u3526\u3527\u3528\u3529\u352A"+
	"\u352B\u352C\u352D\u352E\u352F\u3530\u3531\u3532"+
	"\u3533\u3534\u3535\u3536\u3537\u3538\u3539\u353A"+
	"\u353B\u353C\u353D\u353E\u353F\u3540\u3541\u3542"+
	"\u3543\u3544\u3545\u3546\u3547\u3548\u3549\u354A"+
	"\u354B\u354C\u354D\u354E\u354F\u3550\u3551\u3552"+
	"\u3553\u3554\u3555\u3556\u3557\u3558\u3559\u355A"+
	"\u355B\u355C\u355D\u355E\u355F\u3560\u3561\u3562"+
	"\u3563\u3564\u3565\u3566\u3567\u3568\u3569\u356A"+
	"\u356B\u356C\u356D\u356E\u356F\u3570\u3571\u3572"+
	"\u3573\u3574\u3575\u3576\u3577\u3578\u3579\u357A"+
	"\u357B\u357C\u357D\u357E\u357F\u3580\u3581\u3582"+
	"\u3583\u3584\u3585\u3586\u3587\u3588\u3589\u358A"+
	"\u358B\u358C\u358D\u358E\u358F\u3590\u3591\u3592"+
	"\u3593\u3594\u3595\u3596\u3597\u3598\u3599\u359A"+
	"\u359B\u359C\u359D\u359F\u35A0\u35A1\u35A2\u35A3"+
	"\u35A4\u35A5\u35A6\u35A7\u35A8\u35A9\u35AA\u35AB"+
	"\u35AC\u35AD\u35AE\u35AF\u35B0\u35B1\u35B2\u35B3"+
	"\u35B4\u35B5\u35B6\u35B7\u35B8\u35B9\u35BA\u35BB"+
	"\u35BC\u35BD\u35BE\u35BF\u35C0\u35C1\u35C2\u35C3"+
	"\u35C4\u35C5\u35C6\u35C7\u35C8\u35C9\u35CA\u35CB"+
	"\u35CC\u35CD\u35CE\u35CF\u35D0\u35D1\u35D2\u35D3"+
	"\u35D4\u35D5\u35D6\u35D7\u35D8\u35D9\u35DA\u35DB"+
	"\u35DC\u35DD\u35DE\u35DF\u35E0\u35E1\u35E2\u35E3"+
	"\u35E4\u35E5\u35E6\u35E7\u35E8\u35E9\u35EA\u35EB"+
	"\u35EC\u35ED\u35EE\u35EF\u35F0\u35F1\u35F2\u35F3"+
	"\u35F4\u35F5\u35F6\u35F7\u35F8\u35F9\u35FA\u35FB"+
	"\u35FC\u35FD\u35FE\u35FF\u3600\u3601\u3602\u3603"+
	"\u3604\u3605\u3606\u3607\u3608\u3609\u360A\u360B"+
	"\u360C\u360D\u360F\u3610\u3611\u3612\u3613\u3614"+
	"\u3615\u3616\u3617\u3618\u3619\u361B\u361C\u361D"+
	"\u361E\u361F\u3620\u3621\u3622\u3623\u3624\u3625"+
	"\u3626\u3627\u3628\u3629\u362A\u362B\u362C\u362D"+
	"\u362E\u362F\u3630\u3631\u3632\u3633\u3634\u3635"+
	"\u3636\u3637\u3638\u3639\u363A\u363B\u363C\u363D"+
	"\u363E\u363F\u3640\u3641\u3642\u3643\u3644\u3645"+
	"\u3646\u3647\u3648\u3649\u364A\u364B\u364C\u364D"+
	"\u364E\u364F\u3650\u3651\u3652\u3653\u3654\u3655"+
	"\u3656\u3657\u3658\u3659\u365A\u365B\u365C\u365D"+
	"\u365E\u365F\u3660\u3661\u3662\u3663\u3664\u3665"+
	"\u3666\u3667\u3668\u3669\u366A\u366B\u366C\u366D"+
	"\u366E\u366F\u3670\u3671\u3672\u3673\u3674\u3675"+
	"\u3676\u3677\u3678\u3679\u367A\u367B\u367C\u367D"+
	"\u367E\u367F\u3680\u3681\u3682\u3683\u3684\u3685"+
	"\u3686\u3687\u3688\u3689\u368A\u368B\u368C\u368D"+
	"\u368E\u368F\u3690\u3691\u3692\u3693\u3694\u3695"+
	"\u3696\u3697\u3698\u3699\u369A\u369B\u369C\u369D"+
	"\u369E\u369F\u36A0\u36A1\u36A2\u36A3\u36A4\u36A5"+
	"\u36A6\u36A7\u36A8\u36A9\u36AA\u36AB\u36AC\u36AD"+
	"\u36AE\u36AF\u36B0\u36B1\u36B2\u36B3\u36B4\u36B5"+
	"\u36B6\u36B7\u36B8\u36B9\u36BA\u36BB\u36BC\u36BD"+
	"\u36BE\u36BF\u36C0\u36C1\u36C2\u36C3\u36C4\u36C5"+
	"\u36C6\u36C7\u36C8\u36C9\u36CA\u36CB\u36CC\u36CD"+
	"\u36CE\u36CF\u36D0\u36D1\u36D2\u36D3\u36D4\u36D5"+
	"\u36D6\u36D7\u36D8\u36D9\u36DA\u36DB\u36DC\u36DD"+
	"\u36DE\u36DF\u36E0\u36E1\u36E2\u36E3\u36E4\u36E5"+
	"\u36E6\u36E7\u36E8\u36E9\u36EA\u36EB\u36EC\u36ED"+
	"\u36EE\u36EF\u36F0\u36F1\u36F2\u36F3\u36F4\u36F5"+
	"\u36F6\u36F7\u36F8\u36F9\u36FA\u36FB\u36FC\u36FD"+
	"\u36FE\u36FF\u3700\u3701\u3702\u3703\u3704\u3705"+
	"\u3706\u3707\u3708\u3709\u370A\u370B\u370C\u370D"+
	"\u370E\u370F\u3710\u3711\u3712\u3713\u3714\u3715"+
	"\u3716\u3717\u3718\u3719\u371A\u371B\u371C\u371D"+
	"\u371E\u371F\u3720\u3721\u3722\u3723\u3724\u3725"+
	"\u3726\u3727\u3728\u3729\u372A\u372B\u372C\u372D"+
	"\u372E\u372F\u3730\u3731\u3732\u3733\u3734\u3735"+
	"\u3736\u3737\u3738\u3739\u373A\u373B\u373C\u373D"+
	"\u373E\u373F\u3740\u3741\u3742\u3743\u3744\u3745"+
	"\u3746\u3747\u3748\u3749\u374A\u374B\u374C\u374D"+
	"\u374E\u374F\u3750\u3751\u3752\u3753\u3754\u3755"+
	"\u3756\u3757\u3758\u3759\u375A\u375B\u375C\u375D"+
	"\u375E\u375F\u3760\u3761\u3762\u3763\u3764\u3765"+
	"\u3766\u3767\u3768\u3769\u376A\u376B\u376C\u376D"+
	"\u376E\u376F\u3770\u3771\u3772\u3773\u3774\u3775"+
	"\u3776\u3777\u3778\u3779\u377A\u377B\u377C\u377D"+
	"\u377E\u377F\u3780\u3781\u3782\u3783\u3784\u3785"+
	"\u3786\u3787\u3788\u3789\u378A\u378B\u378C\u378D"+
	"\u378E\u378F\u3790\u3791\u3792\u3793\u3794\u3795"+
	"\u3796\u3797\u3798\u3799\u379A\u379B\u379C\u379D"+
	"\u379E\u379F\u37A0\u37A1\u37A2\u37A3\u37A4\u37A5"+
	"\u37A6\u37A7\u37A8\u37A9\u37AA\u37AB\u37AC\u37AD"+
	"\u37AE\u37AF\u37B0\u37B1\u37B2\u37B3\u37B4\u37B5"+
	"\u37B6\u37B7\u37B8\u37B9\u37BA\u37BB\u37BC\u37BD"+
	"\u37BE\u37BF\u37C0\u37C1\u37C2\u37C3\u37C4\u37C5"+
	"\u37C6\u37C7\u37C8\u37C9\u37CA\u37CB\u37CC\u37CD"+
	"\u37CE\u37CF\u37D0\u37D1\u37D2\u37D3\u37D4\u37D5"+
	"\u37D6\u37D7\u37D8\u37D9\u37DA\u37DB\u37DC\u37DD"+
	"\u37DE\u37DF\u37E0\u37E1\u37E2\u37E3\u37E4\u37E5"+
	"\u37E6\u37E7\u37E8\u37E9\u37EA\u37EB\u37EC\u37ED"+
	"\u37EE\u37EF\u37F0\u37F1\u37F2\u37F3\u37F4\u37F5"+
	"\u37F6\u37F7\u37F8\u37F9\u37FA\u37FB\u37FC\u37FD"+
	"\u37FE\u37FF\u3800\u3801\u3802\u3803\u3804\u3805"+
	"\u3806\u3807\u3808\u3809\u380A\u380B\u380C\u380D"+
	"\u380E\u380F\u3810\u3811\u3812\u3813\u3814\u3815"+
	"\u3816\u3817\u3818\u3819\u381A\u381B\u381C\u381D"+
	"\u381E\u381F\u3820\u3821\u3822\u3823\u3824\u3825"+
	"\u3826\u3827\u3828\u3829\u382A\u382B\u382C\u382D"+
	"\u382E\u382F\u3830\u3831\u3832\u3833\u3834\u3835"+
	"\u3836\u3837\u3838\u3839\u383A\u383B\u383C\u383D"+
	"\u383E\u383F\u3840\u3841\u3842\u3843\u3844\u3845"+
	"\u3846\u3847\u3848\u3849\u384A\u384B\u384C\u384D"+
	"\u384E\u384F\u3850\u3851\u3852\u3853\u3854\u3855"+
	"\u3856\u3857\u3858\u3859\u385A\u385B\u385C\u385D"+
	"\u385E\u385F\u3860\u3861\u3862\u3863\u3864\u3865"+
	"\u3866\u3867\u3868\u3869\u386A\u386B\u386C\u386D"+
	"\u386E\u386F\u3870\u3871\u3872\u3873\u3874\u3875"+
	"\u3876\u3877\u3878\u3879\u387A\u387B\u387C\u387D"+
	"\u387E\u387F\u3880\u3881\u3882\u3883\u3884\u3885"+
	"\u3886\u3887\u3888\u3889\u388A\u388B\u388C\u388D"+
	"\u388E\u388F\u3890\u3891\u3892\u3893\u3894\u3895"+
	"\u3896\u3897\u3898\u3899\u389A\u389B\u389C\u389D"+
	"\u389E\u389F\u38A0\u38A1\u38A2\u38A3\u38A4\u38A5"+
	"\u38A6\u38A7\u38A8\u38A9\u38AA\u38AB\u38AC\u38AD"+
	"\u38AE\u38AF\u38B0\u38B1\u38B2\u38B3\u38B4\u38B5"+
	"\u38B6\u38B7\u38B8\u38B9\u38BA\u38BB\u38BC\u38BD"+
	"\u38BE\u38BF\u38C0\u38C1\u38C2\u38C3\u38C4\u38C5"+
	"\u38C6\u38C7\u38C8\u38C9\u38CA\u38CB\u38CC\u38CD"+
	"\u38CE\u38CF\u38D0\u38D1\u38D2\u38D3\u38D4\u38D5"+
	"\u38D6\u38D7\u38D8\u38D9\u38DA\u38DB\u38DC\u38DD"+
	"\u38DE\u38DF\u38E0\u38E1\u38E2\u38E3\u38E4\u38E5"+
	"\u38E6\u38E7\u38E8\u38E9\u38EA\u38EB\u38EC\u38ED"+
	"\u38EE\u38EF\u38F0\u38F1\u38F2\u38F3\u38F4\u38F5"+
	"\u38F6\u38F7\u38F8\u38F9\u38FA\u38FB\u38FC\u38FD"+
	"\u38FE\u38FF\u3900\u3901\u3902\u3903\u3904\u3905"+
	"\u3906\u3907\u3908\u3909\u390A\u390B\u390C\u390D"+
	"\u390E\u390F\u3910\u3911\u3912\u3913\u3914\u3915"+
	"\u3916\u3917\u3919\u391A\u391B\u391C\u391D\u391E"+
	"\u391F\u3920\u3921\u3922\u3923\u3924\u3925\u3926"+
	"\u3927\u3928\u3929\u392A\u392B\u392C\u392D\u392E"+
	"\u392F\u3930\u3931\u3932\u3933\u3934\u3935\u3936"+
	"\u3937\u3938\u3939\u393A\u393B\u393C\u393D\u393E"+
	"\u393F\u3940\u3941\u3942\u3943\u3944\u3945\u3946"+
	"\u3947\u3948\u3949\u394A\u394B\u394C\u394D\u394E"+
	"\u394F\u3950\u3951\u3952\u3953\u3954\u3955\u3956"+
	"\u3957\u3958\u3959\u395A\u395B\u395C\u395D\u395E"+
	"\u395F\u3960\u3961\u3962\u3963\u3964\u3965\u3966"+
	"\u3967\u3968\u3969\u396A\u396B\u396C\u396D\u396F"+
	"\u3970\u3971\u3972\u3973\u3974\u3975\u3976\u3977"+
	"\u3978\u3979\u397A\u397B\u397C\u397D\u397E\u397F"+
	"\u3980\u3981\u3982\u3983\u3984\u3985\u3986\u3987"+
	"\u3988\u3989\u398A\u398B\u398C\u398D\u398E\u398F"+
	"\u3990\u3991\u3992\u3993\u3994\u3995\u3996\u3997"+
	"\u3998\u3999\u399A\u399B\u399C\u399D\u399E\u399F"+
	"\u39A0\u39A1\u39A2\u39A3\u39A4\u39A5\u39A6\u39A7"+
	"\u39A8\u39A9\u39AA\u39AB\u39AC\u39AD\u39AE\u39AF"+
	"\u39B0\u39B1\u39B2\u39B3\u39B4\u39B5\u39B6\u39B7"+
	"\u39B8\u39B9\u39BA\u39BB\u39BC\u39BD\u39BE\u39BF"+
	"\u39C0\u39C1\u39C2\u39C3\u39C4\u39C5\u39C6\u39C7"+
	"\u39C8\u39C9\u39CA\u39CB\u39CC\u39CD\u39CE\u39D1"+
	"\u39D2\u39D3\u39D4\u39D5\u39D6\u39D7\u39D8\u39D9"+
	"\u39DA\u39DB\u39DC\u39DD\u39DE\u39E0\u39E1\u39E2"+
	"\u39E3\u39E4\u39E5\u39E6\u39E7\u39E8\u39E9\u39EA"+
	"\u39EB\u39EC\u39ED\u39EE\u39EF\u39F0\u39F1\u39F2"+
	"\u39F3\u39F4\u39F5\u39F6\u39F7\u39F8\u39F9\u39FA"+
	"\u39FB\u39FC\u39FD\u39FE\u39FF\u3A00\u3A01\u3A02"+
	"\u3A03\u3A04\u3A05\u3A06\u3A07\u3A08\u3A09\u3A0A"+
	"\u3A0B\u3A0C\u3A0D\u3A0E\u3A0F\u3A10\u3A11\u3A12"+
	"\u3A13\u3A14\u3A15\u3A16\u3A17\u3A18\u3A19\u3A1A"+
	"\u3A1B\u3A1C\u3A1D\u3A1E\u3A1F\u3A20\u3A21\u3A22"+
	"\u3A23\u3A24\u3A25\u3A26\u3A27\u3A28\u3A29\u3A2A"+
	"\u3A2B\u3A2C\u3A2D\u3A2E\u3A2F\u3A30\u3A31\u3A32"+
	"\u3A33\u3A34\u3A35\u3A36\u3A37\u3A38\u3A39\u3A3A"+
	"\u3A3B\u3A3C\u3A3D\u3A3E\u3A3F\u3A40\u3A41\u3A42"+
	"\u3A43\u3A44\u3A45\u3A46\u3A47\u3A48\u3A49\u3A4A"+
	"\u3A4B\u3A4C\u3A4D\u3A4E\u3A4F\u3A50\u3A51\u3A52"+
	"\u3A53\u3A54\u3A55\u3A56\u3A57\u3A58\u3A59\u3A5A"+
	"\u3A5B\u3A5C\u3A5D\u3A5E\u3A5F\u3A60\u3A61\u3A62"+
	"\u3A63\u3A64\u3A65\u3A66\u3A67\u3A68\u3A69\u3A6A"+
	"\u3A6B\u3A6C\u3A6D\u3A6E\u3A6F\u3A70\u3A71\u3A72"+
	"\u3A74\u3A75\u3A76\u3A77\u3A78\u3A79\u3A7A\u3A7B"+
	"\u3A7C\u3A7D\u3A7E\u3A7F\u3A80\u3A81\u3A82\u3A83"+
	"\u3A84\u3A85\u3A86\u3A87\u3A88\u3A89\u3A8A\u3A8B"+
	"\u3A8C\u3A8D\u3A8E\u3A8F\u3A90\u3A91\u3A92\u3A93"+
	"\u3A94\u3A95\u3A96\u3A97\u3A98\u3A99\u3A9A\u3A9B"+
	"\u3A9C\u3A9D\u3A9E\u3A9F\u3AA0\u3AA1\u3AA2\u3AA3"+
	"\u3AA4\u3AA5\u3AA6\u3AA7\u3AA8\u3AA9\u3AAA\u3AAB"+
	"\u3AAC\u3AAD\u3AAE\u3AAF\u3AB0\u3AB1\u3AB2\u3AB3"+
	"\u3AB4\u3AB5\u3AB6\u3AB7\u3AB8\u3AB9\u3ABA\u3ABB"+
	"\u3ABC\u3ABD\u3ABE\u3ABF\u3AC0\u3AC1\u3AC2\u3AC3"+
	"\u3AC4\u3AC5\u3AC6\u3AC7\u3AC8\u3AC9\u3ACA\u3ACB"+
	"\u3ACC\u3ACD\u3ACE\u3ACF\u3AD0\u3AD1\u3AD2\u3AD3"+
	"\u3AD4\u3AD5\u3AD6\u3AD7\u3AD8\u3AD9\u3ADA\u3ADB"+
	"\u3ADC\u3ADD\u3ADE\u3ADF\u3AE0\u3AE1\u3AE2\u3AE3"+
	"\u3AE4\u3AE5\u3AE6\u3AE7\u3AE8\u3AE9\u3AEA\u3AEB"+
	"\u3AEC\u3AED\u3AEE\u3AEF\u3AF0\u3AF1\u3AF2\u3AF3"+
	"\u3AF4\u3AF5\u3AF6\u3AF7\u3AF8\u3AF9\u3AFA\u3AFB"+
	"\u3AFC\u3AFD\u3AFE\u3AFF\u3B00\u3B01\u3B02\u3B03"+
	"\u3B04\u3B05\u3B06\u3B07\u3B08\u3B09\u3B0A\u3B0B"+
	"\u3B0C\u3B0D\u3B0E\u3B0F\u3B10\u3B11\u3B12\u3B13"+
	"\u3B14\u3B15\u3B16\u3B17\u3B18\u3B19\u3B1A\u3B1B"+
	"\u3B1C\u3B1D\u3B1E\u3B1F\u3B20\u3B21\u3B22\u3B23"+
	"\u3B24\u3B25\u3B26\u3B27\u3B28\u3B29\u3B2A\u3B2B"+
	"\u3B2C\u3B2D\u3B2E\u3B2F\u3B30\u3B31\u3B32\u3B33"+
	"\u3B34\u3B35\u3B36\u3B37\u3B38\u3B39\u3B3A\u3B3B"+
	"\u3B3C\u3B3D\u3B3E\u3B3F\u3B40\u3B41\u3B42\u3B43"+
	"\u3B44\u3B45\u3B46\u3B47\u3B48\u3B49\u3B4A\u3B4B"+
	"\u3B4C\u3B4D\u3B4F\u3B50\u3B51\u3B52\u3B53\u3B54"+
	"\u3B55\u3B56\u3B57\u3B58\u3B59\u3B5A\u3B5B\u3B5C"+
	"\u3B5D\u3B5E\u3B5F\u3B60\u3B61\u3B62\u3B63\u3B64"+
	"\u3B65\u3B66\u3B67\u3B68\u3B69\u3B6A\u3B6B\u3B6C"+
	"\u3B6D\u3B6E\u3B6F\u3B70\u3B71\u3B72\u3B73\u3B74"+
	"\u3B75\u3B76\u3B77\u3B78\u3B79\u3B7A\u3B7B\u3B7C"+
	"\u3B7D\u3B7E\u3B7F\u3B80\u3B81\u3B82\u3B83\u3B84"+
	"\u3B85\u3B86\u3B87\u3B88\u3B89\u3B8A\u3B8B\u3B8C"+
	"\u3B8D\u3B8E\u3B8F\u3B90\u3B91\u3B92\u3B93\u3B94"+
	"\u3B95\u3B96\u3B97\u3B98\u3B99\u3B9A\u3B9B\u3B9C"+
	"\u3B9D\u3B9E\u3B9F\u3BA0\u3BA1\u3BA2\u3BA3\u3BA4"+
	"\u3BA5\u3BA6\u3BA7\u3BA8\u3BA9\u3BAA\u3BAB\u3BAC"+
	"\u3BAD\u3BAE\u3BAF\u3BB0\u3BB1\u3BB2\u3BB3\u3BB4"+
	"\u3BB5\u3BB6\u3BB7\u3BB8\u3BB9\u3BBA\u3BBB\u3BBC"+
	"\u3BBD\u3BBE\u3BBF\u3BC0\u3BC1\u3BC2\u3BC3\u3BC4"+
	"\u3BC5\u3BC6\u3BC7\u3BC8\u3BC9\u3BCA\u3BCB\u3BCC"+
	"\u3BCD\u3BCE\u3BCF\u3BD0\u3BD1\u3BD2\u3BD3\u3BD4"+
	"\u3BD5\u3BD6\u3BD7\u3BD8\u3BD9\u3BDA\u3BDB\u3BDC"+
	"\u3BDD\u3BDE\u3BDF\u3BE0\u3BE1\u3BE2\u3BE3\u3BE4"+
	"\u3BE5\u3BE6\u3BE7\u3BE8\u3BE9\u3BEA\u3BEB\u3BEC"+
	"\u3BED\u3BEE\u3BEF\u3BF0\u3BF1\u3BF2\u3BF3\u3BF4"+
	"\u3BF5\u3BF6\u3BF7\u3BF8\u3BF9\u3BFA\u3BFB\u3BFC"+
	"\u3BFD\u3BFE\u3BFF\u3C00\u3C01\u3C02\u3C03\u3C04"+
	"\u3C05\u3C06\u3C07\u3C08\u3C09\u3C0A\u3C0B\u3C0C"+
	"\u3C0D\u3C0E\u3C0F\u3C10\u3C11\u3C12\u3C13\u3C14"+
	"\u3C15\u3C16\u3C17\u3C18\u3C19\u3C1A\u3C1B\u3C1C"+
	"\u3C1D\u3C1E\u3C1F\u3C20\u3C21\u3C22\u3C23\u3C24"+
	"\u3C25\u3C26\u3C27\u3C28\u3C29\u3C2A\u3C2B\u3C2C"+
	"\u3C2D\u3C2E\u3C2F\u3C30\u3C31\u3C32\u3C33\u3C34"+
	"\u3C35\u3C36\u3C37\u3C38\u3C39\u3C3A\u3C3B\u3C3C"+
	"\u3C3D\u3C3E\u3C3F\u3C40\u3C41\u3C42\u3C43\u3C44"+
	"\u3C45\u3C46\u3C47\u3C48\u3C49\u3C4A\u3C4B\u3C4C"+
	"\u3C4D\u3C4E\u3C4F\u3C50\u3C51\u3C52\u3C53\u3C54"+
	"\u3C55\u3C56\u3C57\u3C58\u3C59\u3C5A\u3C5B\u3C5C"+
	"\u3C5D\u3C5E\u3C5F\u3C60\u3C61\u3C62\u3C63\u3C64"+
	"\u3C65\u3C66\u3C67\u3C68\u3C69\u3C6A\u3C6B\u3C6C"+
	"\u3C6D\u3C6F\u3C70\u3C71\u3C72\u3C73\u3C74\u3C75"+
	"\u3C76\u3C77\u3C78\u3C79\u3C7A\u3C7B\u3C7C\u3C7D"+
	"\u3C7E\u3C7F\u3C80\u3C81\u3C82\u3C83\u3C84\u3C85"+
	"\u3C86\u3C87\u3C88\u3C89\u3C8A\u3C8B\u3C8C\u3C8D"+
	"\u3C8E\u3C8F\u3C90\u3C91\u3C92\u3C93\u3C94\u3C95"+
	"\u3C96\u3C97\u3C98\u3C99\u3C9A\u3C9B\u3C9C\u3C9D"+
	"\u3C9E\u3C9F\u3CA0\u3CA1\u3CA2\u3CA3\u3CA4\u3CA5"+
	"\u3CA6\u3CA7\u3CA8\u3CA9\u3CAA\u3CAB\u3CAC\u3CAD"+
	"\u3CAE\u3CAF\u3CB0\u3CB1\u3CB2\u3CB3\u3CB4\u3CB5"+
	"\u3CB6\u3CB7\u3CB8\u3CB9\u3CBA\u3CBB\u3CBC\u3CBD"+
	"\u3CBE\u3CBF\u3CC0\u3CC1\u3CC2\u3CC3\u3CC4\u3CC5"+
	"\u3CC6\u3CC7\u3CC8\u3CC9\u3CCA\u3CCB\u3CCC\u3CCD"+
	"\u3CCE\u3CCF\u3CD0\u3CD1\u3CD2\u3CD3\u3CD4\u3CD5"+
	"\u3CD6\u3CD7\u3CD8\u3CD9\u3CDA\u3CDB\u3CDC\u3CDD"+
	"\u3CDE\u3CDF\u3CE1\u3CE2\u3CE3\u3CE4\u3CE5\u3CE6"+
	"\u3CE7\u3CE8\u3CE9\u3CEA\u3CEB\u3CEC\u3CED\u3CEE"+
	"\u3CEF\u3CF0\u3CF1\u3CF2\u3CF3\u3CF4\u3CF5\u3CF6"+
	"\u3CF7\u3CF8\u3CF9\u3CFA\u3CFB\u3CFC\u3CFD\u3CFE"+
	"\u3CFF\u3D00\u3D01\u3D02\u3D03\u3D04\u3D05\u3D06"+
	"\u3D07\u3D08\u3D09\u3D0A\u3D0B\u3D0C\u3D0D\u3D0E"+
	"\u3D0F\u3D10\u3D11\u3D12\u3D13\u3D14\u3D15\u3D16"+
	"\u3D17\u3D18\u3D19\u3D1A\u3D1B\u3D1C\u3D1D\u3D1E"+
	"\u3D1F\u3D20\u3D21\u3D22\u3D23\u3D24\u3D25\u3D26"+
	"\u3D27\u3D28\u3D29\u3D2A\u3D2B\u3D2C\u3D2D\u3D2E"+
	"\u3D2F\u3D30\u3D31\u3D32\u3D33\u3D34\u3D35\u3D36"+
	"\u3D37\u3D38\u3D39\u3D3A\u3D3B\u3D3C\u3D3D\u3D3E"+
	"\u3D3F\u3D40\u3D41\u3D42\u3D43\u3D44\u3D45\u3D46"+
	"\u3D47\u3D48\u3D49\u3D4A\u3D4B\u3D4C\u3D4D\u3D4E"+
	"\u3D4F\u3D50\u3D51\u3D52\u3D53\u3D54\u3D55\u3D56"+
	"\u3D57\u3D58\u3D59\u3D5A\u3D5B\u3D5C\u3D5D\u3D5E"+
	"\u3D5F\u3D60\u3D61\u3D62\u3D63\u3D64\u3D65\u3D66"+
	"\u3D67\u3D68\u3D69\u3D6A\u3D6B\u3D6C\u3D6D\u3D6E"+
	"\u3D6F\u3D70\u3D71\u3D72\u3D73\u3D74\u3D75\u3D76"+
	"\u3D77\u3D78\u3D79\u3D7A\u3D7B\u3D7C\u3D7D\u3D7E"+
	"\u3D7F\u3D80\u3D81\u3D82\u3D83\u3D84\u3D85\u3D86"+
	"\u3D87\u3D88\u3D89\u3D8A\u3D8B\u3D8C\u3D8D\u3D8E"+
	"\u3D8F\u3D90\u3D91\u3D92\u3D93\u3D94\u3D95\u3D96"+
	"\u3D97\u3D98\u3D99\u3D9A\u3D9B\u3D9C\u3D9D\u3D9E"+
	"\u3D9F\u3DA0\u3DA1\u3DA2\u3DA3\u3DA4\u3DA5\u3DA6"+
	"\u3DA7\u3DA8\u3DA9\u3DAA\u3DAB\u3DAC\u3DAD\u3DAE"+
	"\u3DAF\u3DB0\u3DB1\u3DB2\u3DB3\u3DB4\u3DB5\u3DB6"+
	"\u3DB7\u3DB8\u3DB9\u3DBA\u3DBB\u3DBC\u3DBD\u3DBE"+
	"\u3DBF\u3DC0\u3DC1\u3DC2\u3DC3\u3DC4\u3DC5\u3DC6"+
	"\u3DC7\u3DC8\u3DC9\u3DCA\u3DCB\u3DCC\u3DCD\u3DCE"+
	"\u3DCF\u3DD0\u3DD1\u3DD2\u3DD3\u3DD4\u3DD5\u3DD6"+
	"\u3DD7\u3DD8\u3DD9\u3DDA\u3DDB\u3DDC\u3DDD\u3DDE"+
	"\u3DDF\u3DE0\u3DE1\u3DE2\u3DE3\u3DE4\u3DE5\u3DE6"+
	"\u3DE7\u3DE8\u3DE9\u3DEA\u3DEB\u3DEC\u3DED\u3DEE"+
	"\u3DEF\u3DF0\u3DF1\u3DF2\u3DF3\u3DF4\u3DF5\u3DF6"+
	"\u3DF7\u3DF8\u3DF9\u3DFA\u3DFB\u3DFC\u3DFD\u3DFE"+
	"\u3DFF\u3E00\u3E01\u3E02\u3E03\u3E04\u3E05\u3E06"+
	"\u3E07\u3E08\u3E09\u3E0A\u3E0B\u3E0C\u3E0D\u3E0E"+
	"\u3E0F\u3E10\u3E11\u3E12\u3E13\u3E14\u3E15\u3E16"+
	"\u3E17\u3E18\u3E19\u3E1A\u3E1B\u3E1C\u3E1D\u3E1E"+
	"\u3E1F\u3E20\u3E21\u3E22\u3E23\u3E24\u3E25\u3E26"+
	"\u3E27\u3E28\u3E29\u3E2A\u3E2B\u3E2C\u3E2D\u3E2E"+
	"\u3E2F\u3E30\u3E31\u3E32\u3E33\u3E34\u3E35\u3E36"+
	"\u3E37\u3E38\u3E39\u3E3A\u3E3B\u3E3C\u3E3D\u3E3E"+
	"\u3E3F\u3E40\u3E41\u3E42\u3E43\u3E44\u3E45\u3E46"+
	"\u3E47\u3E48\u3E49\u3E4A\u3E4B\u3E4C\u3E4D\u3E4E"+
	"\u3E4F\u3E50\u3E51\u3E52\u3E53\u3E54\u3E55\u3E56"+
	"\u3E57\u3E58\u3E59\u3E5A\u3E5B\u3E5C\u3E5D\u3E5E"+
	"\u3E5F\u3E60\u3E61\u3E62\u3E63\u3E64\u3E65\u3E66"+
	"\u3E67\u3E68\u3E69\u3E6A\u3E6B\u3E6C\u3E6D\u3E6E"+
	"\u3E6F\u3E70\u3E71\u3E72\u3E73\u3E74\u3E75\u3E76"+
	"\u3E77\u3E78\u3E79\u3E7A\u3E7B\u3E7C\u3E7D\u3E7E"+
	"\u3E7F\u3E80\u3E81\u3E82\u3E83\u3E84\u3E85\u3E86"+
	"\u3E87\u3E88\u3E89\u3E8A\u3E8B\u3E8C\u3E8D\u3E8E"+
	"\u3E8F\u3E90\u3E91\u3E92\u3E93\u3E94\u3E95\u3E96"+
	"\u3E97\u3E98\u3E99\u3E9A\u3E9B\u3E9C\u3E9D\u3E9E"+
	"\u3E9F\u3EA0\u3EA1\u3EA2\u3EA3\u3EA4\u3EA5\u3EA6"+
	"\u3EA7\u3EA8\u3EA9\u3EAA\u3EAB\u3EAC\u3EAD\u3EAE"+
	"\u3EAF\u3EB0\u3EB1\u3EB2\u3EB3\u3EB4\u3EB5\u3EB6"+
	"\u3EB7\u3EB8\u3EB9\u3EBA\u3EBB\u3EBC\u3EBD\u3EBE"+
	"\u3EBF\u3EC0\u3EC1\u3EC2\u3EC3\u3EC4\u3EC5\u3EC6"+
	"\u3EC7\u3EC8\u3EC9\u3ECA\u3ECB\u3ECC\u3ECD\u3ECE"+
	"\u3ECF\u3ED0\u3ED1\u3ED2\u3ED3\u3ED4\u3ED5\u3ED6"+
	"\u3ED7\u3ED8\u3ED9\u3EDA\u3EDB\u3EDC\u3EDD\u3EDE"+
	"\u3EDF\u3EE0\u3EE1\u3EE2\u3EE3\u3EE4\u3EE5\u3EE6"+
	"\u3EE7\u3EE8\u3EE9\u3EEA\u3EEB\u3EEC\u3EED\u3EEE"+
	"\u3EEF\u3EF0\u3EF1\u3EF2\u3EF3\u3EF4\u3EF5\u3EF6"+
	"\u3EF7\u3EF8\u3EF9\u3EFA\u3EFB\u3EFC\u3EFD\u3EFE"+
	"\u3EFF\u3F00\u3F01\u3F02\u3F03\u3F04\u3F05\u3F06"+
	"\u3F07\u3F08\u3F09\u3F0A\u3F0B\u3F0C\u3F0D\u3F0E"+
	"\u3F0F\u3F10\u3F11\u3F12\u3F13\u3F14\u3F15\u3F16"+
	"\u3F17\u3F18\u3F19\u3F1A\u3F1B\u3F1C\u3F1D\u3F1E"+
	"\u3F1F\u3F20\u3F21\u3F22\u3F23\u3F24\u3F25\u3F26"+
	"\u3F27\u3F28\u3F29\u3F2A\u3F2B\u3F2C\u3F2D\u3F2E"+
	"\u3F2F\u3F30\u3F31\u3F32\u3F33\u3F34\u3F35\u3F36"+
	"\u3F37\u3F38\u3F39\u3F3A\u3F3B\u3F3C\u3F3D\u3F3E"+
	"\u3F3F\u3F40\u3F41\u3F42\u3F43\u3F44\u3F45\u3F46"+
	"\u3F47\u3F48\u3F49\u3F4A\u3F4B\u3F4C\u3F4D\u3F4E"+
	"\u3F4F\u3F50\u3F51\u3F52\u3F53\u3F54\u3F55\u3F56"+
	"\u3F57\u3F58\u3F59\u3F5A\u3F5B\u3F5C\u3F5D\u3F5E"+
	"\u3F5F\u3F60\u3F61\u3F62\u3F63\u3F64\u3F65\u3F66"+
	"\u3F67\u3F68\u3F69\u3F6A\u3F6B\u3F6C\u3F6D\u3F6E"+
	"\u3F6F\u3F70\u3F71\u3F72\u3F73\u3F74\u3F75\u3F76"+
	"\u3F77\u3F78\u3F79\u3F7A\u3F7B\u3F7C\u3F7D\u3F7E"+
	"\u3F7F\u3F80\u3F81\u3F82\u3F83\u3F84\u3F85\u3F86"+
	"\u3F87\u3F88\u3F89\u3F8A\u3F8B\u3F8C\u3F8D\u3F8E"+
	"\u3F8F\u3F90\u3F91\u3F92\u3F93\u3F94\u3F95\u3F96"+
	"\u3F97\u3F98\u3F99\u3F9A\u3F9B\u3F9C\u3F9D\u3F9E"+
	"\u3F9F\u3FA0\u3FA1\u3FA2\u3FA3\u3FA4\u3FA5\u3FA6"+
	"\u3FA7\u3FA8\u3FA9\u3FAA\u3FAB\u3FAC\u3FAD\u3FAE"+
	"\u3FAF\u3FB0\u3FB1\u3FB2\u3FB3\u3FB4\u3FB5\u3FB6"+
	"\u3FB7\u3FB8\u3FB9\u3FBA\u3FBB\u3FBC\u3FBD\u3FBE"+
	"\u3FBF\u3FC0\u3FC1\u3FC2\u3FC3\u3FC4\u3FC5\u3FC6"+
	"\u3FC7\u3FC8\u3FC9\u3FCA\u3FCB\u3FCC\u3FCD\u3FCE"+
	"\u3FCF\u3FD0\u3FD1\u3FD2\u3FD3\u3FD4\u3FD5\u3FD6"+
	"\u3FD7\u3FD8\u3FD9\u3FDA\u3FDB\u3FDC\u3FDD\u3FDE"+
	"\u3FDF\u3FE0\u3FE1\u3FE2\u3FE3\u3FE4\u3FE5\u3FE6"+
	"\u3FE7\u3FE8\u3FE9\u3FEA\u3FEB\u3FEC\u3FED\u3FEE"+
	"\u3FEF\u3FF0\u3FF1\u3FF2\u3FF3\u3FF4\u3FF5\u3FF6"+
	"\u3FF7\u3FF8\u3FF9\u3FFA\u3FFB\u3FFC\u3FFD\u3FFE"+
	"\u3FFF\u4000\u4001\u4002\u4003\u4004\u4005\u4006"+
	"\u4007\u4008\u4009\u400A\u400B\u400C\u400D\u400E"+
	"\u400F\u4010\u4011\u4012\u4013\u4014\u4015\u4016"+
	"\u4017\u4018\u4019\u401A\u401B\u401C\u401D\u401E"+
	"\u401F\u4020\u4021\u4022\u4023\u4024\u4025\u4026"+
	"\u4027\u4028\u4029\u402A\u402B\u402C\u402D\u402E"+
	"\u402F\u4030\u4031\u4032\u4033\u4034\u4035\u4036"+
	"\u4037\u4038\u4039\u403A\u403B\u403C\u403D\u403E"+
	"\u403F\u4040\u4041\u4042\u4043\u4044\u4045\u4046"+
	"\u4047\u4048\u4049\u404A\u404B\u404C\u404D\u404E"+
	"\u404F\u4050\u4051\u4052\u4053\u4054\u4055\u4057"+
	"\u4058\u4059\u405A\u405B\u405C\u405D\u405E\u405F"+
	"\u4060\u4061\u4062\u4063\u4064\u4065\u4066\u4067"+
	"\u4068\u4069\u406A\u406B\u406C\u406D\u406E\u406F"+
	"\u4070\u4071\u4072\u4073\u4074\u4075\u4076\u4077"+
	"\u4078\u4079\u407A\u407B\u407C\u407D\u407E\u407F"+
	"\u4080\u4081\u4082\u4083\u4084\u4085\u4086\u4087"+
	"\u4088\u4089\u408A\u408B\u408C\u408D\u408E\u408F"+
	"\u4090\u4091\u4092\u4093\u4094\u4095\u4096\u4097"+
	"\u4098\u4099\u409A\u409B\u409C\u409D\u409E\u409F"+
	"\u40A0\u40A1\u40A2\u40A3\u40A4\u40A5\u40A6\u40A7"+
	"\u40A8\u40A9\u40AA\u40AB\u40AC\u40AD\u40AE\u40AF"+
	"\u40B0\u40B1\u40B2\u40B3\u40B4\u40B5\u40B6\u40B7"+
	"\u40B8\u40B9\u40BA\u40BB\u40BC\u40BD\u40BE\u40BF"+
	"\u40C0\u40C1\u40C2\u40C3\u40C4\u40C5\u40C6\u40C7"+
	"\u40C8\u40C9\u40CA\u40CB\u40CC\u40CD\u40CE\u40CF"+
	"\u40D0\u40D1\u40D2\u40D3\u40D4\u40D5\u40D6\u40D7"+
	"\u40D8\u40D9\u40DA\u40DB\u40DC\u40DD\u40DE\u40DF"+
	"\u40E0\u40E1\u40E2\u40E3\u40E4\u40E5\u40E6\u40E7"+
	"\u40E8\u40E9\u40EA\u40EB\u40EC\u40ED\u40EE\u40EF"+
	"\u40F0\u40F1\u40F2\u40F3\u40F4\u40F5\u40F6\u40F7"+
	"\u40F8\u40F9\u40FA\u40FB\u40FC\u40FD\u40FE\u40FF"+
	"\u4100\u4101\u4102\u4103\u4104\u4105\u4106\u4107"+
	"\u4108\u4109\u410A\u410B\u410C\u410D\u410E\u410F"+
	"\u4110\u4111\u4112\u4113\u4114\u4115\u4116\u4117"+
	"\u4118\u4119\u411A\u411B\u411C\u411D\u411E\u411F"+
	"\u4120\u4121\u4122\u4123\u4124\u4125\u4126\u4127"+
	"\u4128\u4129\u412A\u412B\u412C\u412D\u412E\u412F"+
	"\u4130\u4131\u4132\u4133\u4134\u4135\u4136\u4137"+
	"\u4138\u4139\u413A\u413B\u413C\u413D\u413E\u413F"+
	"\u4140\u4141\u4142\u4143\u4144\u4145\u4146\u4147"+
	"\u4148\u4149\u414A\u414B\u414C\u414D\u414E\u414F"+
	"\u4150\u4151\u4152\u4153\u4154\u4155\u4156\u4157"+
	"\u4158\u4159\u415A\u415B\u415C\u415D\u415E\u4160"+
	"\u4161\u4162\u4163\u4164\u4165\u4166\u4167\u4168"+
	"\u4169\u416A\u416B\u416C\u416D\u416E\u416F\u4170"+
	"\u4171\u4172\u4173\u4174\u4175\u4176\u4177\u4178"+
	"\u4179\u417A\u417B\u417C\u417D\u417E\u417F\u4180"+
	"\u4181\u4182\u4183\u4184\u4185\u4186\u4187\u4188"+
	"\u4189\u418A\u418B\u418C\u418D\u418E\u418F\u4190"+
	"\u4191\u4192\u4193\u4194\u4195\u4196\u4197\u4198"+
	"\u4199\u419A\u419B\u419C\u419D\u419E\u419F\u41A0"+
	"\u41A1\u41A2\u41A3\u41A4\u41A5\u41A6\u41A7\u41A8"+
	"\u41A9\u41AA\u41AB\u41AC\u41AD\u41AE\u41AF\u41B0"+
	"\u41B1\u41B2\u41B3\u41B4\u41B5\u41B6\u41B7\u41B8"+
	"\u41B9\u41BA\u41BB\u41BC\u41BD\u41BE\u41BF\u41C0"+
	"\u41C1\u41C2\u41C3\u41C4\u41C5\u41C6\u41C7\u41C8"+
	"\u41C9\u41CA\u41CB\u41CC\u41CD\u41CE\u41CF\u41D0"+
	"\u41D1\u41D2\u41D3\u41D4\u41D5\u41D6\u41D7\u41D8"+
	"\u41D9\u41DA\u41DB\u41DC\u41DD\u41DE\u41DF\u41E0"+
	"\u41E1\u41E2\u41E3\u41E4\u41E5\u41E6\u41E7\u41E8"+
	"\u41E9\u41EA\u41EB\u41EC\u41ED\u41EE\u41EF\u41F0"+
	"\u41F1\u41F2\u41F3\u41F4\u41F5\u41F6\u41F7\u41F8"+
	"\u41F9\u41FA\u41FB\u41FC\u41FD\u41FE\u41FF\u4200"+
	"\u4201\u4202\u4203\u4204\u4205\u4206\u4207\u4208"+
	"\u4209\u420A\u420B\u420C\u420D\u420E\u420F\u4210"+
	"\u4211\u4212\u4213\u4214\u4215\u4216\u4217\u4218"+
	"\u4219\u421A\u421B\u421C\u421D\u421E\u421F\u4220"+
	"\u4221\u4222\u4223\u4224\u4225\u4226\u4227\u4228"+
	"\u4229\u422A\u422B\u422C\u422D\u422E\u422F\u4230"+
	"\u4231\u4232\u4233\u4234\u4235\u4236\u4237\u4238"+
	"\u4239\u423A\u423B\u423C\u423D\u423E\u423F\u4240"+
	"\u4241\u4242\u4243\u4244\u4245\u4246\u4247\u4248"+
	"\u4249\u424A\u424B\u424C\u424D\u424E\u424F\u4250"+
	"\u4251\u4252\u4253\u4254\u4255\u4256\u4257\u4258"+
	"\u4259\u425A\u425B\u425C\u425D\u425E\u425F\u4260"+
	"\u4261\u4262\u4263\u4264\u4265\u4266\u4267\u4268"+
	"\u4269\u426A\u426B\u426C\u426D\u426E\u426F\u4270"+
	"\u4271\u4272\u4273\u4274\u4275\u4276\u4277\u4278"+
	"\u4279\u427A\u427B\u427C\u427D\u427E\u427F\u4280"+
	"\u4281\u4282\u4283\u4284\u4285\u4286\u4287\u4288"+
	"\u4289\u428A\u428B\u428C\u428D\u428E\u428F\u4290"+
	"\u4291\u4292\u4293\u4294\u4295\u4296\u4297\u4298"+
	"\u4299\u429A\u429B\u429C\u429D\u429E\u429F\u42A0"+
	"\u42A1\u42A2\u42A3\u42A4\u42A5\u42A6\u42A7\u42A8"+
	"\u42A9\u42AA\u42AB\u42AC\u42AD\u42AE\u42AF\u42B0"+
	"\u42B1\u42B2\u42B3\u42B4\u42B5\u42B6\u42B7\u42B8"+
	"\u42B9\u42BA\u42BB\u42BC\u42BD\u42BE\u42BF\u42C0"+
	"\u42C1\u42C2\u42C3\u42C4\u42C5\u42C6\u42C7\u42C8"+
	"\u42C9\u42CA\u42CB\u42CC\u42CD\u42CE\u42CF\u42D0"+
	"\u42D1\u42D2\u42D3\u42D4\u42D5\u42D6\u42D7\u42D8"+
	"\u42D9\u42DA\u42DB\u42DC\u42DD\u42DE\u42DF\u42E0"+
	"\u42E1\u42E2\u42E3\u42E4\u42E5\u42E6\u42E7\u42E8"+
	"\u42E9\u42EA\u42EB\u42EC\u42ED\u42EE\u42EF\u42F0"+
	"\u42F1\u42F2\u42F3\u42F4\u42F5\u42F6\u42F7\u42F8"+
	"\u42F9\u42FA\u42FB\u42FC\u42FD\u42FE\u42FF\u4300"+
	"\u4301\u4302\u4303\u4304\u4305\u4306\u4307\u4308"+
	"\u4309\u430A\u430B\u430C\u430D\u430E\u430F\u4310"+
	"\u4311\u4312\u4313\u4314\u4315\u4316\u4317\u4318"+
	"\u4319\u431A\u431B\u431C\u431D\u431E\u431F\u4320"+
	"\u4321\u4322\u4323\u4324\u4325\u4326\u4327\u4328"+
	"\u4329\u432A\u432B\u432C\u432D\u432E\u432F\u4330"+
	"\u4331\u4332\u4333\u4334\u4335\u4336\u4338\u4339"+
	"\u433A\u433B\u433C\u433D\u433E\u433F\u4340\u4341"+
	"\u4342\u4343\u4344\u4345\u4346\u4347\u4348\u4349"+
	"\u434A\u434B\u434C\u434D\u434E\u434F\u4350\u4351"+
	"\u4352\u4353\u4354\u4355\u4356\u4357\u4358\u4359"+
	"\u435A\u435B\u435C\u435D\u435E\u435F\u4360\u4361"+
	"\u4362\u4363\u4364\u4365\u4366\u4367\u4368\u4369"+
	"\u436A\u436B\u436C\u436D\u436E\u436F\u4370\u4371"+
	"\u4372\u4373\u4374\u4375\u4376\u4377\u4378\u4379";

    private final static String innerIndex4= 
	"\u437A\u437B\u437C\u437D\u437E\u437F\u4380\u4381"+
	"\u4382\u4383\u4384\u4385\u4386\u4387\u4388\u4389"+
	"\u438A\u438B\u438C\u438D\u438E\u438F\u4390\u4391"+
	"\u4392\u4393\u4394\u4395\u4396\u4397\u4398\u4399"+
	"\u439A\u439B\u439C\u439D\u439E\u439F\u43A0\u43A1"+
	"\u43A2\u43A3\u43A4\u43A5\u43A6\u43A7\u43A8\u43A9"+
	"\u43AA\u43AB\u43AD\u43AE\u43AF\u43B0\u43B2\u43B3"+
	"\u43B4\u43B5\u43B6\u43B7\u43B8\u43B9\u43BA\u43BB"+
	"\u43BC\u43BD\u43BE\u43BF\u43C0\u43C1\u43C2\u43C3"+
	"\u43C4\u43C5\u43C6\u43C7\u43C8\u43C9\u43CA\u43CB"+
	"\u43CC\u43CD\u43CE\u43CF\u43D0\u43D1\u43D2\u43D3"+
	"\u43D4\u43D5\u43D6\u43D7\u43D8\u43D9\u43DA\u43DB"+
	"\u43DC\u43DE\u43DF\u43E0\u43E1\u43E2\u43E3\u43E4"+
	"\u43E5\u43E6\u43E7\u43E8\u43E9\u43EA\u43EB\u43EC"+
	"\u43ED\u43EE\u43EF\u43F0\u43F1\u43F2\u43F3\u43F4"+
	"\u43F5\u43F6\u43F7\u43F8\u43F9\u43FA\u43FB\u43FC"+
	"\u43FD\u43FE\u43FF\u4400\u4401\u4402\u4403\u4404"+
	"\u4405\u4406\u4407\u4408\u4409\u440A\u440B\u440C"+
	"\u440D\u440E\u440F\u4410\u4411\u4412\u4413\u4414"+
	"\u4415\u4416\u4417\u4418\u4419\u441A\u441B\u441C"+
	"\u441D\u441E\u441F\u4420\u4421\u4422\u4423\u4424"+
	"\u4425\u4426\u4427\u4428\u4429\u442A\u442B\u442C"+
	"\u442D\u442E\u442F\u4430\u4431\u4432\u4433\u4434"+
	"\u4435\u4436\u4437\u4438\u4439\u443A\u443B\u443C"+
	"\u443D\u443E\u443F\u4440\u4441\u4442\u4443\u4444"+
	"\u4445\u4446\u4447\u4448\u4449\u444A\u444B\u444C"+
	"\u444D\u444E\u444F\u4450\u4451\u4452\u4453\u4454"+
	"\u4455\u4456\u4457\u4458\u4459\u445A\u445B\u445C"+
	"\u445D\u445E\u445F\u4460\u4461\u4462\u4463\u4464"+
	"\u4465\u4466\u4467\u4468\u4469\u446A\u446B\u446C"+
	"\u446D\u446E\u446F\u4470\u4471\u4472\u4473\u4474"+
	"\u4475\u4476\u4477\u4478\u4479\u447A\u447B\u447C"+
	"\u447D\u447E\u447F\u4480\u4481\u4482\u4483\u4484"+
	"\u4485\u4486\u4487\u4488\u4489\u448A\u448B\u448C"+
	"\u448D\u448E\u448F\u4490\u4491\u4492\u4493\u4494"+
	"\u4495\u4496\u4497\u4498\u4499\u449A\u449B\u449C"+
	"\u449D\u449E\u449F\u44A0\u44A1\u44A2\u44A3\u44A4"+
	"\u44A5\u44A6\u44A7\u44A8\u44A9\u44AA\u44AB\u44AC"+
	"\u44AD\u44AE\u44AF\u44B0\u44B1\u44B2\u44B3\u44B4"+
	"\u44B5\u44B6\u44B7\u44B8\u44B9\u44BA\u44BB\u44BC"+
	"\u44BD\u44BE\u44BF\u44C0\u44C1\u44C2\u44C3\u44C4"+
	"\u44C5\u44C6\u44C7\u44C8\u44C9\u44CA\u44CB\u44CC"+
	"\u44CD\u44CE\u44CF\u44D0\u44D1\u44D2\u44D3\u44D4"+
	"\u44D5\u44D7\u44D8\u44D9\u44DA\u44DB\u44DC\u44DD"+
	"\u44DE\u44DF\u44E0\u44E1\u44E2\u44E3\u44E4\u44E5"+
	"\u44E6\u44E7\u44E8\u44E9\u44EA\u44EB\u44EC\u44ED"+
	"\u44EE\u44EF\u44F0\u44F1\u44F2\u44F3\u44F4\u44F5"+
	"\u44F6\u44F7\u44F8\u44F9\u44FA\u44FB\u44FC\u44FD"+
	"\u44FE\u44FF\u4500\u4501\u4502\u4503\u4504\u4505"+
	"\u4506\u4507\u4508\u4509\u450A\u450B\u450C\u450D"+
	"\u450E\u450F\u4510\u4511\u4512\u4513\u4514\u4515"+
	"\u4516\u4517\u4518\u4519\u451A\u451B\u451C\u451D"+
	"\u451E\u451F\u4520\u4521\u4522\u4523\u4524\u4525"+
	"\u4526\u4527\u4528\u4529\u452A\u452B\u452C\u452D"+
	"\u452E\u452F\u4530\u4531\u4532\u4533\u4534\u4535"+
	"\u4536\u4537\u4538\u4539\u453A\u453B\u453C\u453D"+
	"\u453E\u453F\u4540\u4541\u4542\u4543\u4544\u4545"+
	"\u4546\u4547\u4548\u4549\u454A\u454B\u454C\u454D"+
	"\u454E\u454F\u4550\u4551\u4552\u4553\u4554\u4555"+
	"\u4556\u4557\u4558\u4559\u455A\u455B\u455C\u455D"+
	"\u455E\u455F\u4560\u4561\u4562\u4563\u4564\u4565"+
	"\u4566\u4567\u4568\u4569\u456A\u456B\u456C\u456D"+
	"\u456E\u456F\u4570\u4571\u4572\u4573\u4574\u4575"+
	"\u4576\u4577\u4578\u4579\u457A\u457B\u457C\u457D"+
	"\u457E\u457F\u4580\u4581\u4582\u4583\u4584\u4585"+
	"\u4586\u4587\u4588\u4589\u458A\u458B\u458C\u458D"+
	"\u458E\u458F\u4590\u4591\u4592\u4593\u4594\u4595"+
	"\u4596\u4597\u4598\u4599\u459A\u459B\u459C\u459D"+
	"\u459E\u459F\u45A0\u45A1\u45A2\u45A3\u45A4\u45A5"+
	"\u45A6\u45A7\u45A8\u45A9\u45AA\u45AB\u45AC\u45AD"+
	"\u45AE\u45AF\u45B0\u45B1\u45B2\u45B3\u45B4\u45B5"+
	"\u45B6\u45B7\u45B8\u45B9\u45BA\u45BB\u45BC\u45BD"+
	"\u45BE\u45BF\u45C0\u45C1\u45C2\u45C3\u45C4\u45C5"+
	"\u45C6\u45C7\u45C8\u45C9\u45CA\u45CB\u45CC\u45CD"+
	"\u45CE\u45CF\u45D0\u45D1\u45D2\u45D3\u45D4\u45D5"+
	"\u45D6\u45D7\u45D8\u45D9\u45DA\u45DB\u45DC\u45DD"+
	"\u45DE\u45DF\u45E0\u45E1\u45E2\u45E3\u45E4\u45E5"+
	"\u45E6\u45E7\u45E8\u45E9\u45EA\u45EB\u45EC\u45ED"+
	"\u45EE\u45EF\u45F0\u45F1\u45F2\u45F3\u45F4\u45F5"+
	"\u45F6\u45F7\u45F8\u45F9\u45FA\u45FB\u45FC\u45FD"+
	"\u45FE\u45FF\u4600\u4601\u4602\u4603\u4604\u4605"+
	"\u4606\u4607\u4608\u4609\u460A\u460B\u460C\u460D"+
	"\u460E\u460F\u4610\u4611\u4612\u4613\u4614\u4615"+
	"\u4616\u4617\u4618\u4619\u461A\u461B\u461C\u461D"+
	"\u461E\u461F\u4620\u4621\u4622\u4623\u4624\u4625"+
	"\u4626\u4627\u4628\u4629\u462A\u462B\u462C\u462D"+
	"\u462E\u462F\u4630\u4631\u4632\u4633\u4634\u4635"+
	"\u4636\u4637\u4638\u4639\u463A\u463B\u463C\u463D"+
	"\u463E\u463F\u4640\u4641\u4642\u4643\u4644\u4645"+
	"\u4646\u4647\u4648\u4649\u464A\u464B\u464D\u464E"+
	"\u464F\u4650\u4651\u4652\u4653\u4654\u4655\u4656"+
	"\u4657\u4658\u4659\u465A\u465B\u465C\u465D\u465E"+
	"\u465F\u4660\u4662\u4663\u4664\u4665\u4666\u4667"+
	"\u4668\u4669\u466A\u466B\u466C\u466D\u466E\u466F"+
	"\u4670\u4671\u4672\u4673\u4674\u4675\u4676\u4677"+
	"\u4678\u4679\u467A\u467B\u467C\u467D\u467E\u467F"+
	"\u4680\u4681\u4682\u4683\u4684\u4685\u4686\u4687"+
	"\u4688\u4689\u468A\u468B\u468C\u468D\u468E\u468F"+
	"\u4690\u4691\u4692\u4693\u4694\u4695\u4696\u4697"+
	"\u4698\u4699\u469A\u469B\u469C\u469D\u469E\u469F"+
	"\u46A0\u46A1\u46A2\u46A3\u46A4\u46A5\u46A6\u46A7"+
	"\u46A8\u46A9\u46AA\u46AB\u46AC\u46AD\u46AE\u46AF"+
	"\u46B0\u46B1\u46B2\u46B3\u46B4\u46B5\u46B6\u46B7"+
	"\u46B8\u46B9\u46BA\u46BB\u46BC\u46BD\u46BE\u46BF"+
	"\u46C0\u46C1\u46C2\u46C3\u46C4\u46C5\u46C6\u46C7"+
	"\u46C8\u46C9\u46CA\u46CB\u46CC\u46CD\u46CE\u46CF"+
	"\u46D0\u46D1\u46D2\u46D3\u46D4\u46D5\u46D6\u46D7"+
	"\u46D8\u46D9\u46DA\u46DB\u46DC\u46DD\u46DE\u46DF"+
	"\u46E0\u46E1\u46E2\u46E3\u46E4\u46E5\u46E6\u46E7"+
	"\u46E8\u46E9\u46EA\u46EB\u46EC\u46ED\u46EE\u46EF"+
	"\u46F0\u46F1\u46F2\u46F3\u46F4\u46F5\u46F6\u46F7"+
	"\u46F8\u46F9\u46FA\u46FB\u46FC\u46FD\u46FE\u46FF"+
	"\u4700\u4701\u4702\u4703\u4704\u4705\u4706\u4707"+
	"\u4708\u4709\u470A\u470B\u470C\u470D\u470E\u470F"+
	"\u4710\u4711\u4712\u4713\u4714\u4715\u4716\u4717"+
	"\u4718\u4719\u471A\u471B\u471C\u471D\u471E\u471F"+
	"\u4720\u4721\u4722\u4724\u4725\u4726\u4727\u4728"+
	"\u472A\u472B\u472C\u472D\u472E\u472F\u4730\u4731"+
	"\u4732\u4733\u4734\u4735\u4736\u4737\u4738\u4739"+
	"\u473A\u473B\u473C\u473D\u473E\u473F\u4740\u4741"+
	"\u4742\u4743\u4744\u4745\u4746\u4747\u4748\u4749"+
	"\u474A\u474B\u474C\u474D\u474E\u474F\u4750\u4751"+
	"\u4752\u4753\u4754\u4755\u4756\u4757\u4758\u4759"+
	"\u475A\u475B\u475C\u475D\u475E\u475F\u4760\u4761"+
	"\u4762\u4763\u4764\u4765\u4766\u4767\u4768\u4769"+
	"\u476A\u476B\u476C\u476D\u476E\u476F\u4770\u4771"+
	"\u4772\u4773\u4774\u4775\u4776\u4777\u4778\u4779"+
	"\u477A\u477B\u477D\u477E\u477F\u4780\u4781\u4782"+
	"\u4783\u4784\u4785\u4786\u4787\u4788\u4789\u478A"+
	"\u478B\u478C\u478E\u478F\u4790\u4791\u4792\u4793"+
	"\u4794\u4795\u4796\u4797\u4798\u4799\u479A\u479B"+
	"\u479C\u479D\u479E\u479F\u47A0\u47A1\u47A2\u47A3"+
	"\u47A4\u47A5\u47A6\u47A7\u47A8\u47A9\u47AA\u47AB"+
	"\u47AC\u47AD\u47AE\u47AF\u47B0\u47B1\u47B2\u47B3"+
	"\u47B4\u47B5\u47B6\u47B7\u47B8\u47B9\u47BA\u47BB"+
	"\u47BC\u47BD\u47BE\u47BF\u47C0\u47C1\u47C2\u47C3"+
	"\u47C4\u47C5\u47C6\u47C7\u47C8\u47C9\u47CA\u47CB"+
	"\u47CC\u47CD\u47CE\u47CF\u47D0\u47D1\u47D2\u47D3"+
	"\u47D4\u47D5\u47D6\u47D7\u47D8\u47D9\u47DA\u47DB"+
	"\u47DC\u47DD\u47DE\u47DF\u47E0\u47E1\u47E2\u47E3"+
	"\u47E4\u47E5\u47E6\u47E7\u47E8\u47E9\u47EA\u47EB"+
	"\u47EC\u47ED\u47EE\u47EF\u47F0\u47F1\u47F2\u47F3"+
	"\u47F4\u47F5\u47F6\u47F7\u47F8\u47F9\u47FA\u47FB"+
	"\u47FC\u47FD\u47FE\u47FF\u4800\u4801\u4802\u4803"+
	"\u4804\u4805\u4806\u4807\u4808\u4809\u480A\u480B"+
	"\u480C\u480D\u480E\u480F\u4810\u4811\u4812\u4813"+
	"\u4814\u4815\u4816\u4817\u4818\u4819\u481A\u481B"+
	"\u481C\u481D\u481E\u481F\u4820\u4821\u4822\u4823"+
	"\u4824\u4825\u4826\u4827\u4828\u4829\u482A\u482B"+
	"\u482C\u482D\u482E\u482F\u4830\u4831\u4832\u4833"+
	"\u4834\u4835\u4836\u4837\u4838\u4839\u483A\u483B"+
	"\u483C\u483D\u483E\u483F\u4840\u4841\u4842\u4843"+
	"\u4844\u4845\u4846\u4847\u4848\u4849\u484A\u484B"+
	"\u484C\u484D\u484E\u484F\u4850\u4851\u4852\u4853"+
	"\u4854\u4855\u4856\u4857\u4858\u4859\u485A\u485B"+
	"\u485C\u485D\u485E\u485F\u4860\u4861\u4862\u4863"+
	"\u4864\u4865\u4866\u4867\u4868\u4869\u486A\u486B"+
	"\u486C\u486D\u486E\u486F\u4870\u4871\u4872\u4873"+
	"\u4874\u4875\u4876\u4877\u4878\u4879\u487A\u487B"+
	"\u487C\u487D\u487E\u487F\u4880\u4881\u4882\u4883"+
	"\u4884\u4885\u4886\u4887\u4888\u4889\u488A\u488B"+
	"\u488C\u488D\u488E\u488F\u4890\u4891\u4892\u4893"+
	"\u4894\u4895\u4896\u4897\u4898\u4899\u489A\u489B"+
	"\u489C\u489D\u489E\u489F\u48A0\u48A1\u48A2\u48A3"+
	"\u48A4\u48A5\u48A6\u48A7\u48A8\u48A9\u48AA\u48AB"+
	"\u48AC\u48AD\u48AE\u48AF\u48B0\u48B1\u48B2\u48B3"+
	"\u48B4\u48B5\u48B6\u48B7\u48B8\u48B9\u48BA\u48BB"+
	"\u48BC\u48BD\u48BE\u48BF\u48C0\u48C1\u48C2\u48C3"+
	"\u48C4\u48C5\u48C6\u48C7\u48C8\u48C9\u48CA\u48CB"+
	"\u48CC\u48CD\u48CE\u48CF\u48D0\u48D1\u48D2\u48D3"+
	"\u48D4\u48D5\u48D6\u48D7\u48D8\u48D9\u48DA\u48DB"+
	"\u48DC\u48DD\u48DE\u48DF\u48E0\u48E1\u48E2\u48E3"+
	"\u48E4\u48E5\u48E6\u48E7\u48E8\u48E9\u48EA\u48EB"+
	"\u48EC\u48ED\u48EE\u48EF\u48F0\u48F1\u48F2\u48F3"+
	"\u48F4\u48F5\u48F6\u48F7\u48F8\u48F9\u48FA\u48FB"+
	"\u48FC\u48FD\u48FE\u48FF\u4900\u4901\u4902\u4903"+
	"\u4904\u4905\u4906\u4907\u4908\u4909\u490A\u490B"+
	"\u490C\u490D\u490E\u490F\u4910\u4911\u4912\u4913"+
	"\u4914\u4915\u4916\u4917\u4918\u4919\u491A\u491B"+
	"\u491C\u491D\u491E\u491F\u4920\u4921\u4922\u4923"+
	"\u4924\u4925\u4926\u4927\u4928\u4929\u492A\u492B"+
	"\u492C\u492D\u492E\u492F\u4930\u4931\u4932\u4933"+
	"\u4934\u4935\u4936\u4937\u4938\u4939\u493A\u493B"+
	"\u493C\u493D\u493E\u493F\u4940\u4941\u4942\u4943"+
	"\u4944\u4945\u4946\u4948\u4949\u494A\u494B\u494C"+
	"\u494D\u494E\u494F\u4950\u4951\u4952\u4953\u4954"+
	"\u4955\u4956\u4957\u4958\u4959\u495A\u495B\u495C"+
	"\u495D\u495E\u495F\u4960\u4961\u4962\u4963\u4964"+
	"\u4965\u4966\u4967\u4968\u4969\u496A\u496B\u496C"+
	"\u496D\u496E\u496F\u4970\u4971\u4972\u4973\u4974"+
	"\u4975\u4976\u4977\u4978\u4979\u497B\u497C\u497E"+
	"\u497F\u4980\u4981\u4984\u4987\u4988\u4989\u498A"+
	"\u498B\u498C\u498D\u498E\u498F\u4990\u4991\u4992"+
	"\u4993\u4994\u4995\u4996\u4997\u4998\u4999\u499A"+
	"\u499C\u499D\u499E\u49A0\u49A1\u49A2\u49A3\u49A4"+
	"\u49A5\u49A6\u49A7\u49A8\u49A9\u49AA\u49AB\u49AC"+
	"\u49AD\u49AE\u49AF\u49B0\u49B1\u49B2\u49B3\u49B4"+
	"\u49B5\u49B8\u49B9\u49BA\u49BB\u49BC\u49BD\u49BE"+
	"\u49BF\u49C0\u49C1\u49C2\u49C3\u49C4\u49C5\u49C6"+
	"\u49C7\u49C8\u49C9\u49CA\u49CB\u49CC\u49CD\u49CE"+
	"\u49CF\u49D0\u49D1\u49D2\u49D3\u49D4\u49D5\u49D6"+
	"\u49D7\u49D8\u49D9\u49DA\u49DB\u49DC\u49DD\u49DE"+
	"\u49DF\u49E0\u49E1\u49E2\u49E3\u49E4\u49E5\u49E6"+
	"\u49E7\u49E8\u49E9\u49EA\u49EB\u49EC\u49ED\u49EE"+
	"\u49EF\u49F0\u49F1\u49F2\u49F3\u49F4\u49F5\u49F6"+
	"\u49F7\u49F8\u49F9\u49FA\u49FB\u49FC\u49FD\u49FE"+
	"\u49FF\u4A00\u4A01\u4A02\u4A03\u4A04\u4A05\u4A06"+
	"\u4A07\u4A08\u4A09\u4A0A\u4A0B\u4A0C\u4A0D\u4A0E"+
	"\u4A0F\u4A10\u4A11\u4A12\u4A13\u4A14\u4A15\u4A16"+
	"\u4A17\u4A18\u4A19\u4A1A\u4A1B\u4A1C\u4A1D\u4A1E"+
	"\u4A1F\u4A20\u4A21\u4A22\u4A23\u4A24\u4A25\u4A26"+
	"\u4A27\u4A28\u4A29\u4A2A\u4A2B\u4A2C\u4A2D\u4A2E"+
	"\u4A2F\u4A30\u4A31\u4A32\u4A33\u4A34\u4A35\u4A36"+
	"\u4A37\u4A38\u4A39\u4A3A\u4A3B\u4A3C\u4A3D\u4A3E"+
	"\u4A3F\u4A40\u4A41\u4A42\u4A43\u4A44\u4A45\u4A46"+
	"\u4A47\u4A48\u4A49\u4A4A\u4A4B\u4A4C\u4A4D\u4A4E"+
	"\u4A4F\u4A50\u4A51\u4A52\u4A53\u4A54\u4A55\u4A56"+
	"\u4A57\u4A58\u4A59\u4A5A\u4A5B\u4A5C\u4A5D\u4A5E"+
	"\u4A5F\u4A60\u4A61\u4A62\u4A63\u4A64\u4A65\u4A66"+
	"\u4A67\u4A68\u4A69\u4A6A\u4A6B\u4A6C\u4A6D\u4A6E"+
	"\u4A6F\u4A70\u4A71\u4A72\u4A73\u4A74\u4A75\u4A76"+
	"\u4A77\u4A78\u4A79\u4A7A\u4A7B\u4A7C\u4A7D\u4A7E"+
	"\u4A7F\u4A80\u4A81\u4A82\u4A83\u4A84\u4A85\u4A86"+
	"\u4A87\u4A88\u4A89\u4A8A\u4A8B\u4A8C\u4A8D\u4A8E"+
	"\u4A8F\u4A90\u4A91\u4A92\u4A93\u4A94\u4A95\u4A96"+
	"\u4A97\u4A98\u4A99\u4A9A\u4A9B\u4A9C\u4A9D\u4A9E"+
	"\u4A9F\u4AA0\u4AA1\u4AA2\u4AA3\u4AA4\u4AA5\u4AA6"+
	"\u4AA7\u4AA8\u4AA9\u4AAA\u4AAB\u4AAC\u4AAD\u4AAE"+
	"\u4AAF\u4AB0\u4AB1\u4AB2\u4AB3\u4AB4\u4AB5\u4AB6"+
	"\u4AB7\u4AB8\u4AB9\u4ABA\u4ABB\u4ABC\u4ABD\u4ABE"+
	"\u4ABF\u4AC0\u4AC1\u4AC2\u4AC3\u4AC4\u4AC5\u4AC6"+
	"\u4AC7\u4AC8\u4AC9\u4ACA\u4ACB\u4ACC\u4ACD\u4ACE"+
	"\u4ACF\u4AD0\u4AD1\u4AD2\u4AD3\u4AD4\u4AD5\u4AD6"+
	"\u4AD7\u4AD8\u4AD9\u4ADA\u4ADB\u4ADC\u4ADD\u4ADE"+
	"\u4ADF\u4AE0\u4AE1\u4AE2\u4AE3\u4AE4\u4AE5\u4AE6"+
	"\u4AE7\u4AE8\u4AE9\u4AEA\u4AEB\u4AEC\u4AED\u4AEE"+
	"\u4AEF\u4AF0\u4AF1\u4AF2\u4AF3\u4AF4\u4AF5\u4AF6"+
	"\u4AF7\u4AF8\u4AF9\u4AFA\u4AFB\u4AFC\u4AFD\u4AFE"+
	"\u4AFF\u4B00\u4B01\u4B02\u4B03\u4B04\u4B05\u4B06"+
	"\u4B07\u4B08\u4B09\u4B0A\u4B0B\u4B0C\u4B0D\u4B0E"+
	"\u4B0F\u4B10\u4B11\u4B12\u4B13\u4B14\u4B15\u4B16"+
	"\u4B17\u4B18\u4B19\u4B1A\u4B1B\u4B1C\u4B1D\u4B1E"+
	"\u4B1F\u4B20\u4B21\u4B22\u4B23\u4B24\u4B25\u4B26"+
	"\u4B27\u4B28\u4B29\u4B2A\u4B2B\u4B2C\u4B2D\u4B2E"+
	"\u4B2F\u4B30\u4B31\u4B32\u4B33\u4B34\u4B35\u4B36"+
	"\u4B37\u4B38\u4B39\u4B3A\u4B3B\u4B3C\u4B3D\u4B3E"+
	"\u4B3F\u4B40\u4B41\u4B42\u4B43\u4B44\u4B45\u4B46"+
	"\u4B47\u4B48\u4B49\u4B4A\u4B4B\u4B4C\u4B4D\u4B4E"+
	"\u4B4F\u4B50\u4B51\u4B52\u4B53\u4B54\u4B55\u4B56"+
	"\u4B57\u4B58\u4B59\u4B5A\u4B5B\u4B5C\u4B5D\u4B5E"+
	"\u4B5F\u4B60\u4B61\u4B62\u4B63\u4B64\u4B65\u4B66"+
	"\u4B67\u4B68\u4B69\u4B6A\u4B6B\u4B6C\u4B6D\u4B6E"+
	"\u4B6F\u4B70\u4B71\u4B72\u4B73\u4B74\u4B75\u4B76"+
	"\u4B77\u4B78\u4B79\u4B7A\u4B7B\u4B7C\u4B7D\u4B7E"+
	"\u4B7F\u4B80\u4B81\u4B82\u4B83\u4B84\u4B85\u4B86"+
	"\u4B87\u4B88\u4B89\u4B8A\u4B8B\u4B8C\u4B8D\u4B8E"+
	"\u4B8F\u4B90\u4B91\u4B92\u4B93\u4B94\u4B95\u4B96"+
	"\u4B97\u4B98\u4B99\u4B9A\u4B9B\u4B9C\u4B9D\u4B9E"+
	"\u4B9F\u4BA0\u4BA1\u4BA2\u4BA3\u4BA4\u4BA5\u4BA6"+
	"\u4BA7\u4BA8\u4BA9\u4BAA\u4BAB\u4BAC\u4BAD\u4BAE"+
	"\u4BAF\u4BB0\u4BB1\u4BB2\u4BB3\u4BB4\u4BB5\u4BB6"+
	"\u4BB7\u4BB8\u4BB9\u4BBA\u4BBB\u4BBC\u4BBD\u4BBE"+
	"\u4BBF\u4BC0\u4BC1\u4BC2\u4BC3\u4BC4\u4BC5\u4BC6"+
	"\u4BC7\u4BC8\u4BC9\u4BCA\u4BCB\u4BCC\u4BCD\u4BCE"+
	"\u4BCF\u4BD0\u4BD1\u4BD2\u4BD3\u4BD4\u4BD5\u4BD6"+
	"\u4BD7\u4BD8\u4BD9\u4BDA\u4BDB\u4BDC\u4BDD\u4BDE"+
	"\u4BDF\u4BE0\u4BE1\u4BE2\u4BE3\u4BE4\u4BE5\u4BE6"+
	"\u4BE7\u4BE8\u4BE9\u4BEA\u4BEB\u4BEC\u4BED\u4BEE"+
	"\u4BEF\u4BF0\u4BF1\u4BF2\u4BF3\u4BF4\u4BF5\u4BF6"+
	"\u4BF7\u4BF8\u4BF9\u4BFA\u4BFB\u4BFC\u4BFD\u4BFE"+
	"\u4BFF\u4C00\u4C01\u4C02\u4C03\u4C04\u4C05\u4C06"+
	"\u4C07\u4C08\u4C09\u4C0A\u4C0B\u4C0C\u4C0D\u4C0E"+
	"\u4C0F\u4C10\u4C11\u4C12\u4C13\u4C14\u4C15\u4C16"+
	"\u4C17\u4C18\u4C19\u4C1A\u4C1B\u4C1C\u4C1D\u4C1E"+
	"\u4C1F\u4C20\u4C21\u4C22\u4C23\u4C24\u4C25\u4C26"+
	"\u4C27\u4C28\u4C29\u4C2A\u4C2B\u4C2C\u4C2D\u4C2E"+
	"\u4C2F\u4C30\u4C31\u4C32\u4C33\u4C34\u4C35\u4C36"+
	"\u4C37\u4C38\u4C39\u4C3A\u4C3B\u4C3C\u4C3D\u4C3E"+
	"\u4C3F\u4C40\u4C41\u4C42\u4C43\u4C44\u4C45\u4C46"+
	"\u4C47\u4C48\u4C49\u4C4A\u4C4B\u4C4C\u4C4D\u4C4E"+
	"\u4C4F\u4C50\u4C51\u4C52\u4C53\u4C54\u4C55\u4C56"+
	"\u4C57\u4C58\u4C59\u4C5A\u4C5B\u4C5C\u4C5D\u4C5E"+
	"\u4C5F\u4C60\u4C61\u4C62\u4C63\u4C64\u4C65\u4C66"+
	"\u4C67\u4C68\u4C69\u4C6A\u4C6B\u4C6C\u4C6D\u4C6E"+
	"\u4C6F\u4C70\u4C71\u4C72\u4C73\u4C74\u4C75\u4C76"+
	"\u4C78\u4C79\u4C7A\u4C7B\u4C7C\u4C7D\u4C7E\u4C7F"+
	"\u4C80\u4C81\u4C82\u4C83\u4C84\u4C85\u4C86\u4C87"+
	"\u4C88\u4C89\u4C8A\u4C8B\u4C8C\u4C8D\u4C8E\u4C8F"+
	"\u4C90\u4C91\u4C92\u4C93\u4C94\u4C95\u4C96\u4C97"+
	"\u4C98\u4C99\u4C9A\u4C9B\u4C9C\u4C9D\u4C9E\u4CA4"+
	"\u4CA5\u4CA6\u4CA7\u4CA8\u4CA9\u4CAA\u4CAB\u4CAC"+
	"\u4CAD\u4CAE\u4CAF\u4CB0\u4CB1\u4CB2\u4CB3\u4CB4"+
	"\u4CB5\u4CB6\u4CB7\u4CB8\u4CB9\u4CBA\u4CBB\u4CBC"+
	"\u4CBD\u4CBE\u4CBF\u4CC0\u4CC1\u4CC2\u4CC3\u4CC4"+
	"\u4CC5\u4CC6\u4CC7\u4CC8\u4CC9\u4CCA\u4CCB\u4CCC"+
	"\u4CCD\u4CCE\u4CCF\u4CD0\u4CD1\u4CD2\u4CD3\u4CD4"+
	"\u4CD5\u4CD6\u4CD7\u4CD8\u4CD9\u4CDA\u4CDB\u4CDC"+
	"\u4CDD\u4CDE\u4CDF\u4CE0\u4CE1\u4CE2\u4CE3\u4CE4"+
	"\u4CE5\u4CE6\u4CE7\u4CE8\u4CE9\u4CEA\u4CEB\u4CEC"+
	"\u4CED\u4CEE\u4CEF\u4CF0\u4CF1\u4CF2\u4CF3\u4CF4"+
	"\u4CF5\u4CF6\u4CF7\u4CF8\u4CF9\u4CFA\u4CFB\u4CFC"+
	"\u4CFD\u4CFE\u4CFF\u4D00\u4D01\u4D02\u4D03\u4D04"+
	"\u4D05\u4D06\u4D07\u4D08\u4D09\u4D0A\u4D0B\u4D0C"+
	"\u4D0D\u4D0E\u4D0F\u4D10\u4D11\u4D12\u4D1A\u4D1B"+
	"\u4D1C\u4D1D\u4D1E\u4D1F\u4D20\u4D21\u4D22\u4D23"+
	"\u4D24\u4D25\u4D26\u4D27\u4D28\u4D29\u4D2A\u4D2B"+
	"\u4D2C\u4D2D\u4D2E\u4D2F\u4D30\u4D31\u4D32\u4D33"+
	"\u4D34\u4D35\u4D36\u4D37\u4D38\u4D39\u4D3A\u4D3B"+
	"\u4D3C\u4D3D\u4D3E\u4D3F\u4D40\u4D41\u4D42\u4D43"+
	"\u4D44\u4D45\u4D46\u4D47\u4D48\u4D49\u4D4A\u4D4B"+
	"\u4D4C\u4D4D\u4D4E\u4D4F\u4D50\u4D51\u4D52\u4D53"+
	"\u4D54\u4D55\u4D56\u4D57\u4D58\u4D59\u4D5A\u4D5B"+
	"\u4D5C\u4D5D\u4D5E\u4D5F\u4D60\u4D61\u4D62\u4D63"+
	"\u4D64\u4D65\u4D66\u4D67\u4D68\u4D69\u4D6A\u4D6B"+
	"\u4D6C\u4D6D\u4D6E\u4D6F\u4D70\u4D71\u4D72\u4D73"+
	"\u4D74\u4D75\u4D76\u4D77\u4D78\u4D79\u4D7A\u4D7B"+
	"\u4D7C\u4D7D\u4D7E\u4D7F\u4D80\u4D81\u4D82\u4D83"+
	"\u4D84\u4D85\u4D86\u4D87\u4D88\u4D89\u4D8A\u4D8B"+
	"\u4D8C\u4D8D\u4D8E\u4D8F\u4D90\u4D91\u4D92\u4D93"+
	"\u4D94\u4D95\u4D96\u4D97\u4D98\u4D99\u4D9A\u4D9B"+
	"\u4D9C\u4D9D\u4D9E\u4D9F\u4DA0\u4DA1\u4DA2\u4DA3"+
	"\u4DA4\u4DA5\u4DA6\u4DA7\u4DA8\u4DA9\u4DAA\u4DAB"+
	"\u4DAC\u4DAD\u4DAF\u4DB0\u4DB1\u4DB2\u4DB3\u4DB4"+
	"\u4DB5\u4DB6\u4DB7\u4DB8\u4DB9\u4DBA\u4DBB\u4DBC"+
	"\u4DBD\u4DBE\u4DBF\u4DC0\u4DC1\u4DC2\u4DC3\u4DC4"+
	"\u4DC5\u4DC6\u4DC7\u4DC8\u4DC9\u4DCA\u4DCB\u4DCC"+
	"\u4DCD\u4DCE\u4DCF\u4DD0\u4DD1\u4DD2\u4DD3\u4DD4"+
	"\u4DD5\u4DD6\u4DD7\u4DD8\u4DD9\u4DDA\u4DDB\u4DDC"+
	"\u4DDD\u4DDE\u4DDF\u4DE0\u4DE1\u4DE2\u4DE3\u4DE4"+
	"\u4DE5\u4DE6\u4DE7\u4DE8\u4DE9\u4DEA\u4DEB\u4DEC"+
	"\u4DED\u4DEE\u4DEF\u4DF0\u4DF1\u4DF2\u4DF3\u4DF4"+
	"\u4DF5\u4DF6\u4DF7\u4DF8\u4DF9\u4DFA\u4DFB\u4DFC"+
	"\u4DFD\u4DFE\u4DFF\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uE76C\uE7C8\uE7E7"+
	"\uE7E8\uE7E9\uE7EA\uE7EB\uE7EC\uE7ED\uE7EE\uE7EF"+
	"\uE7F0\uE7F1\uE7F2\uE7F3\uE815\uE819\uE81A\uE81B"+
	"\uE81C\uE81D\uE81F\uE820\uE821\uE822\uE823\uE824"+
	"\uE825\uE827\uE828\uE829\uE82A\uE82D\uE82E\uE82F"+
	"\uE830\uE833\uE834\uE835\uE836\uE837\uE838\uE839"+
	"\uE83A\uE83C\uE83D\uE83E\uE83F\uE840\uE841\uE842"+
	"\uE844\uE845\uE846\uE847\uE848\uE849\uE84A\uE84B"+
	"\uE84C\uE84D\uE84E\uE84F\uE850\uE851\uE852\uE853"+
	"\uE856\uE857\uE858\uE859\uE85A\uE85B\uE85C\uE85D"+
	"\uE85E\uE85F\uE860\uE861\uE862\uE863\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uF900\uF901\uF902\uF903\uF904\uF905\uF906"+
	"\uF907\uF908\uF909\uF90A\uF90B\uF90C\uF90D\uF90E"+
	"\uF90F\uF910\uF911\uF912\uF913\uF914\uF915\uF916"+
	"\uF917\uF918\uF919\uF91A\uF91B\uF91C\uF91D\uF91E"+
	"\uF91F\uF920\uF921\uF922\uF923\uF924\uF925\uF926"+
	"\uF927\uF928\uF929\uF92A\uF92B\uF92D\uF92E\uF92F"+
	"\uF930\uF931\uF932\uF933\uF934\uF935\uF936\uF937"+
	"\uF938\uF939\uF93A\uF93B\uF93C\uF93D\uF93E\uF93F"+
	"\uF940\uF941\uF942\uF943\uF944\uF945\uF946\uF947"+
	"\uF948\uF949\uF94A\uF94B\uF94C\uF94D\uF94E\uF94F"+
	"\uF950\uF951\uF952\uF953\uF954\uF955\uF956\uF957"+
	"\uF958\uF959\uF95A\uF95B\uF95C\uF95D\uF95E\uF95F"+
	"\uF960\uF961\uF962\uF963\uF964\uF965\uF966\uF967"+
	"\uF968\uF969\uF96A\uF96B\uF96C\uF96D\uF96E\uF96F"+
	"\uF970\uF971\uF972\uF973\uF974\uF975\uF976\uF977"+
	"\uF978\uF97A\uF97B\uF97C\uF97D\uF97E\uF97F\uF980"+
	"\uF981\uF982\uF983\uF984\uF985\uF986\uF987\uF988"+
	"\uF989\uF98A\uF98B\uF98C\uF98D\uF98E\uF98F\uF990"+
	"\uF991\uF992\uF993\uF994\uF996\uF997\uF998\uF999"+
	"\uF99A\uF99B\uF99C\uF99D\uF99E\uF99F\uF9A0\uF9A1"+
	"\uF9A2\uF9A3\uF9A4\uF9A5\uF9A6\uF9A7\uF9A8\uF9A9"+
	"\uF9AA\uF9AB\uF9AC\uF9AD\uF9AE\uF9AF\uF9B0\uF9B1"+
	"\uF9B2\uF9B3\uF9B4\uF9B5\uF9B6\uF9B7\uF9B8\uF9B9"+
	"\uF9BA\uF9BB\uF9BC\uF9BD\uF9BE\uF9BF\uF9C0\uF9C1"+
	"\uF9C2\uF9C3\uF9C4\uF9C5\uF9C6\uF9C7\uF9C8\uF9C9"+
	"\uF9CA\uF9CB\uF9CC\uF9CD\uF9CE\uF9CF\uF9D0\uF9D1"+
	"\uF9D2\uF9D3\uF9D4\uF9D5\uF9D6\uF9D7\uF9D8\uF9D9"+
	"\uF9DA\uF9DB\uF9DC\uF9DD\uF9DE\uF9DF\uF9E0\uF9E1"+
	"\uF9E2\uF9E3\uF9E4\uF9E5\uF9E6\uF9E8\uF9E9\uF9EA"+
	"\uF9EB\uF9EC\uF9ED\uF9EE\uF9EF\uF9F0\uF9F2\uF9F3"+
	"\uF9F4\uF9F5\uF9F6\uF9F7\uF9F8\uF9F9\uF9FA\uF9FB"+
	"\uF9FC\uF9FD\uF9FE\uF9FF\uFA00\uFA01\uFA02\uFA03"+
	"\uFA04\uFA05\uFA06\uFA07\uFA08\uFA09\uFA0A\uFA0B"+
	"\uFA10\uFA12\uFA15\uFA16\uFA17\uFA19\uFA1A\uFA1B"+
	"\uFA1C\uFA1D\uFA1E\uFA22\uFA25\uFA26\uFA2A\uFA2B"+
	"\uFA2C\uFA2D\uFA2E\uFA2F\uFA30\uFA31\uFA32\uFA33"+
	"\uFA34\uFA35\uFA36\uFA37\uFA38\uFA39\uFA3A\uFA3B"+
	"\uFA3C\uFA3D\uFA3E\uFA3F\uFA40\uFA41\uFA42\uFA43"+
	"\uFA44\uFA45\uFA46\uFA47\uFA48\uFA49\uFA4A\uFA4B"+
	"\uFA4C\uFA4D\uFA4E\uFA4F\uFA50\uFA51\uFA52\uFA53"+
	"\uFA54\uFA55\uFA56\uFA57\uFA58\uFA59\uFA5A\uFA5B"+
	"\uFA5C\uFA5D\uFA5E\uFA5F\uFA60\uFA61\uFA62\uFA63"+
	"\uFA64\uFA65\uFA66\uFA67\uFA68\uFA69\uFA6A\uFA6B"+
	"\uFA6C\uFA6D\uFA6E\uFA6F\uFA70\uFA71\uFA72\uFA73"+
	"\uFA74\uFA75\uFA76\uFA77\uFA78\uFA79\uFA7A\uFA7B"+
	"\uFA7C\uFA7D\uFA7E\uFA7F\uFA80\uFA81\uFA82\uFA83"+
	"\uFA84\uFA85\uFA86\uFA87\uFA88\uFA89\uFA8A\uFA8B"+
	"\uFA8C\uFA8D\uFA8E\uFA8F\uFA90\uFA91\uFA92\uFA93"+
	"\uFA94\uFA95\uFA96\uFA97\uFA98\uFA99\uFA9A\uFA9B"+
	"\uFA9C\uFA9D\uFA9E\uFA9F\uFAA0\uFAA1\uFAA2\uFAA3"+
	"\uFAA4\uFAA5\uFAA6\uFAA7\uFAA8\uFAA9\uFAAA\uFAAB"+
	"\uFAAC\uFAAD\uFAAE\uFAAF\uFAB0\uFAB1\uFAB2\uFAB3"+
	"\uFAB4\uFAB5\uFAB6\uFAB7\uFAB8\uFAB9\uFABA\uFABB"+
	"\uFABC\uFABD\uFABE\uFABF\uFAC0\uFAC1\uFAC2\uFAC3"+
	"\uFAC4\uFAC5\uFAC6\uFAC7\uFAC8\uFAC9\uFACA\uFACB"+
	"\uFACC\uFACD\uFACE\uFACF\uFAD0\uFAD1\uFAD2\uFAD3"+
	"\uFAD4\uFAD5\uFAD6\uFAD7\uFAD8\uFAD9\uFADA\uFADB"+
	"\uFADC\uFADD\uFADE\uFADF\uFAE0\uFAE1\uFAE2\uFAE3"+
	"\uFAE4\uFAE5\uFAE6\uFAE7\uFAE8\uFAE9\uFAEA\uFAEB"+
	"\uFAEC\uFAED\uFAEE\uFAEF\uFAF0\uFAF1\uFAF2\uFAF3"+
	"\uFAF4\uFAF5\uFAF6\uFAF7\uFAF8\uFAF9\uFAFA\uFAFB"+
	"\uFAFC\uFAFD\uFAFE\uFAFF\uFB00\uFB01\uFB02\uFB03"+
	"\uFB04\uFB05\uFB06\uFB07\uFB08\uFB09\uFB0A\uFB0B"+
	"\uFB0C\uFB0D\uFB0E\uFB0F\uFB10\uFB11\uFB12\uFB13"+
	"\uFB14\uFB15\uFB16\uFB17\uFB18\uFB19\uFB1A\uFB1B"+
	"\uFB1C\uFB1D\uFB1E\uFB1F\uFB20\uFB21\uFB22\uFB23"+
	"\uFB24\uFB25\uFB26\uFB27\uFB28\uFB29\uFB2A\uFB2B"+
	"\uFB2C\uFB2D\uFB2E\uFB2F\uFB30\uFB31\uFB32\uFB33"+
	"\uFB34\uFB35\uFB36\uFB37\uFB38\uFB39\uFB3A\uFB3B"+
	"\uFB3C\uFB3D\uFB3E\uFB3F\uFB40\uFB41\uFB42\uFB43"+
	"\uFB44\uFB45\uFB46\uFB47\uFB48\uFB49\uFB4A\uFB4B"+
	"\uFB4C\uFB4D\uFB4E\uFB4F\uFB50\uFB51\uFB52\uFB53"+
	"\uFB54\uFB55\uFB56\uFB57\uFB58\uFB59\uFB5A\uFB5B"+
	"\uFB5C\uFB5D\uFB5E\uFB5F\uFB60\uFB61\uFB62\uFB63"+
	"\uFB64\uFB65\uFB66\uFB67\uFB68\uFB69\uFB6A\uFB6B";

    private final static String innerIndex5= 
	"\uFB6C\uFB6D\uFB6E\uFB6F\uFB70\uFB71\uFB72\uFB73"+
	"\uFB74\uFB75\uFB76\uFB77\uFB78\uFB79\uFB7A\uFB7B"+
	"\uFB7C\uFB7D\uFB7E\uFB7F\uFB80\uFB81\uFB82\uFB83"+
	"\uFB84\uFB85\uFB86\uFB87\uFB88\uFB89\uFB8A\uFB8B"+
	"\uFB8C\uFB8D\uFB8E\uFB8F\uFB90\uFB91\uFB92\uFB93"+
	"\uFB94\uFB95\uFB96\uFB97\uFB98\uFB99\uFB9A\uFB9B"+
	"\uFB9C\uFB9D\uFB9E\uFB9F\uFBA0\uFBA1\uFBA2\uFBA3"+
	"\uFBA4\uFBA5\uFBA6\uFBA7\uFBA8\uFBA9\uFBAA\uFBAB"+
	"\uFBAC\uFBAD\uFBAE\uFBAF\uFBB0\uFBB1\uFBB2\uFBB3"+
	"\uFBB4\uFBB5\uFBB6\uFBB7\uFBB8\uFBB9\uFBBA\uFBBB"+
	"\uFBBC\uFBBD\uFBBE\uFBBF\uFBC0\uFBC1\uFBC2\uFBC3"+
	"\uFBC4\uFBC5\uFBC6\uFBC7\uFBC8\uFBC9\uFBCA\uFBCB"+
	"\uFBCC\uFBCD\uFBCE\uFBCF\uFBD0\uFBD1\uFBD2\uFBD3"+
	"\uFBD4\uFBD5\uFBD6\uFBD7\uFBD8\uFBD9\uFBDA\uFBDB"+
	"\uFBDC\uFBDD\uFBDE\uFBDF\uFBE0\uFBE1\uFBE2\uFBE3"+
	"\uFBE4\uFBE5\uFBE6\uFBE7\uFBE8\uFBE9\uFBEA\uFBEB"+
	"\uFBEC\uFBED\uFBEE\uFBEF\uFBF0\uFBF1\uFBF2\uFBF3"+
	"\uFBF4\uFBF5\uFBF6\uFBF7\uFBF8\uFBF9\uFBFA\uFBFB"+
	"\uFBFC\uFBFD\uFBFE\uFBFF\uFC00\uFC01\uFC02\uFC03"+
	"\uFC04\uFC05\uFC06\uFC07\uFC08\uFC09\uFC0A\uFC0B"+
	"\uFC0C\uFC0D\uFC0E\uFC0F\uFC10\uFC11\uFC12\uFC13"+
	"\uFC14\uFC15\uFC16\uFC17\uFC18\uFC19\uFC1A\uFC1B"+
	"\uFC1C\uFC1D\uFC1E\uFC1F\uFC20\uFC21\uFC22\uFC23"+
	"\uFC24\uFC25\uFC26\uFC27\uFC28\uFC29\uFC2A\uFC2B"+
	"\uFC2C\uFC2D\uFC2E\uFC2F\uFC30\uFC31\uFC32\uFC33"+
	"\uFC34\uFC35\uFC36\uFC37\uFC38\uFC39\uFC3A\uFC3B"+
	"\uFC3C\uFC3D\uFC3E\uFC3F\uFC40\uFC41\uFC42\uFC43"+
	"\uFC44\uFC45\uFC46\uFC47\uFC48\uFC49\uFC4A\uFC4B"+
	"\uFC4C\uFC4D\uFC4E\uFC4F\uFC50\uFC51\uFC52\uFC53"+
	"\uFC54\uFC55\uFC56\uFC57\uFC58\uFC59\uFC5A\uFC5B"+
	"\uFC5C\uFC5D\uFC5E\uFC5F\uFC60\uFC61\uFC62\uFC63"+
	"\uFC64\uFC65\uFC66\uFC67\uFC68\uFC69\uFC6A\uFC6B"+
	"\uFC6C\uFC6D\uFC6E\uFC6F\uFC70\uFC71\uFC72\uFC73"+
	"\uFC74\uFC75\uFC76\uFC77\uFC78\uFC79\uFC7A\uFC7B"+
	"\uFC7C\uFC7D\uFC7E\uFC7F\uFC80\uFC81\uFC82\uFC83"+
	"\uFC84\uFC85\uFC86\uFC87\uFC88\uFC89\uFC8A\uFC8B"+
	"\uFC8C\uFC8D\uFC8E\uFC8F\uFC90\uFC91\uFC92\uFC93"+
	"\uFC94\uFC95\uFC96\uFC97\uFC98\uFC99\uFC9A\uFC9B"+
	"\uFC9C\uFC9D\uFC9E\uFC9F\uFCA0\uFCA1\uFCA2\uFCA3"+
	"\uFCA4\uFCA5\uFCA6\uFCA7\uFCA8\uFCA9\uFCAA\uFCAB"+
	"\uFCAC\uFCAD\uFCAE\uFCAF\uFCB0\uFCB1\uFCB2\uFCB3"+
	"\uFCB4\uFCB5\uFCB6\uFCB7\uFCB8\uFCB9\uFCBA\uFCBB"+
	"\uFCBC\uFCBD\uFCBE\uFCBF\uFCC0\uFCC1\uFCC2\uFCC3"+
	"\uFCC4\uFCC5\uFCC6\uFCC7\uFCC8\uFCC9\uFCCA\uFCCB"+
	"\uFCCC\uFCCD\uFCCE\uFCCF\uFCD0\uFCD1\uFCD2\uFCD3"+
	"\uFCD4\uFCD5\uFCD6\uFCD7\uFCD8\uFCD9\uFCDA\uFCDB"+
	"\uFCDC\uFCDD\uFCDE\uFCDF\uFCE0\uFCE1\uFCE2\uFCE3"+
	"\uFCE4\uFCE5\uFCE6\uFCE7\uFCE8\uFCE9\uFCEA\uFCEB"+
	"\uFCEC\uFCED\uFCEE\uFCEF\uFCF0\uFCF1\uFCF2\uFCF3"+
	"\uFCF4\uFCF5\uFCF6\uFCF7\uFCF8\uFCF9\uFCFA\uFCFB"+
	"\uFCFC\uFCFD\uFCFE\uFCFF\uFD00\uFD01\uFD02\uFD03"+
	"\uFD04\uFD05\uFD06\uFD07\uFD08\uFD09\uFD0A\uFD0B"+
	"\uFD0C\uFD0D\uFD0E\uFD0F\uFD10\uFD11\uFD12\uFD13"+
	"\uFD14\uFD15\uFD16\uFD17\uFD18\uFD19\uFD1A\uFD1B"+
	"\uFD1C\uFD1D\uFD1E\uFD1F\uFD20\uFD21\uFD22\uFD23"+
	"\uFD24\uFD25\uFD26\uFD27\uFD28\uFD29\uFD2A\uFD2B"+
	"\uFD2C\uFD2D\uFD2E\uFD2F\uFD30\uFD31\uFD32\uFD33"+
	"\uFD34\uFD35\uFD36\uFD37\uFD38\uFD39\uFD3A\uFD3B"+
	"\uFD3C\uFD3D\uFD3E\uFD3F\uFD40\uFD41\uFD42\uFD43"+
	"\uFD44\uFD45\uFD46\uFD47\uFD48\uFD49\uFD4A\uFD4B"+
	"\uFD4C\uFD4D\uFD4E\uFD4F\uFD50\uFD51\uFD52\uFD53"+
	"\uFD54\uFD55\uFD56\uFD57\uFD58\uFD59\uFD5A\uFD5B"+
	"\uFD5C\uFD5D\uFD5E\uFD5F\uFD60\uFD61\uFD62\uFD63"+
	"\uFD64\uFD65\uFD66\uFD67\uFD68\uFD69\uFD6A\uFD6B"+
	"\uFD6C\uFD6D\uFD6E\uFD6F\uFD70\uFD71\uFD72\uFD73"+
	"\uFD74\uFD75\uFD76\uFD77\uFD78\uFD79\uFD7A\uFD7B"+
	"\uFD7C\uFD7D\uFD7E\uFD7F\uFD80\uFD81\uFD82\uFD83"+
	"\uFD84\uFD85\uFD86\uFD87\uFD88\uFD89\uFD8A\uFD8B"+
	"\uFD8C\uFD8D\uFD8E\uFD8F\uFD90\uFD91\uFD92\uFD93"+
	"\uFD94\uFD95\uFD96\uFD97\uFD98\uFD99\uFD9A\uFD9B"+
	"\uFD9C\uFD9D\uFD9E\uFD9F\uFDA0\uFDA1\uFDA2\uFDA3"+
	"\uFDA4\uFDA5\uFDA6\uFDA7\uFDA8\uFDA9\uFDAA\uFDAB"+
	"\uFDAC\uFDAD\uFDAE\uFDAF\uFDB0\uFDB1\uFDB2\uFDB3"+
	"\uFDB4\uFDB5\uFDB6\uFDB7\uFDB8\uFDB9\uFDBA\uFDBB"+
	"\uFDBC\uFDBD\uFDBE\uFDBF\uFDC0\uFDC1\uFDC2\uFDC3"+
	"\uFDC4\uFDC5\uFDC6\uFDC7\uFDC8\uFDC9\uFDCA\uFDCB"+
	"\uFDCC\uFDCD\uFDCE\uFDCF\uFDD0\uFDD1\uFDD2\uFDD3"+
	"\uFDD4\uFDD5\uFDD6\uFDD7\uFDD8\uFDD9\uFDDA\uFDDB"+
	"\uFDDC\uFDDD\uFDDE\uFDDF\uFDE0\uFDE1\uFDE2\uFDE3"+
	"\uFDE4\uFDE5\uFDE6\uFDE7\uFDE8\uFDE9\uFDEA\uFDEB"+
	"\uFDEC\uFDED\uFDEE\uFDEF\uFDF0\uFDF1\uFDF2\uFDF3"+
	"\uFDF4\uFDF5\uFDF6\uFDF7\uFDF8\uFDF9\uFDFA\uFDFB"+
	"\uFDFC\uFDFD\uFDFE\uFDFF\uFE00\uFE01\uFE02\uFE03"+
	"\uFE04\uFE05\uFE06\uFE07\uFE08\uFE09\uFE0A\uFE0B"+
	"\uFE0C\uFE0D\uFE0E\uFE0F\uFE10\uFE11\uFE12\uFE13"+
	"\uFE14\uFE15\uFE16\uFE17\uFE18\uFE19\uFE1A\uFE1B"+
	"\uFE1C\uFE1D\uFE1E\uFE1F\uFE20\uFE21\uFE22\uFE23"+
	"\uFE24\uFE25\uFE26\uFE27\uFE28\uFE29\uFE2A\uFE2B"+
	"\uFE2C\uFE2D\uFE2E\uFE2F\uFE32\uFE45\uFE46\uFE47"+
	"\uFE48\uFE53\uFE58\uFE67\uFE6C\uFE6D\uFE6E\uFE6F"+
	"\uFE70\uFE71\uFE72\uFE73\uFE74\uFE75\uFE76\uFE77"+
	"\uFE78\uFE79\uFE7A\uFE7B\uFE7C\uFE7D\uFE7E\uFE7F"+
	"\uFE80\uFE81\uFE82\uFE83\uFE84\uFE85\uFE86\uFE87"+
	"\uFE88\uFE89\uFE8A\uFE8B\uFE8C\uFE8D\uFE8E\uFE8F"+
	"\uFE90\uFE91\uFE92\uFE93\uFE94\uFE95\uFE96\uFE97"+
	"\uFE98\uFE99\uFE9A\uFE9B\uFE9C\uFE9D\uFE9E\uFE9F"+
	"\uFEA0\uFEA1\uFEA2\uFEA3\uFEA4\uFEA5\uFEA6\uFEA7"+
	"\uFEA8\uFEA9\uFEAA\uFEAB\uFEAC\uFEAD\uFEAE\uFEAF"+
	"\uFEB0\uFEB1\uFEB2\uFEB3\uFEB4\uFEB5\uFEB6\uFEB7"+
	"\uFEB8\uFEB9\uFEBA\uFEBB\uFEBC\uFEBD\uFEBE\uFEBF"+
	"\uFEC0\uFEC1\uFEC2\uFEC3\uFEC4\uFEC5\uFEC6\uFEC7"+
	"\uFEC8\uFEC9\uFECA\uFECB\uFECC\uFECD\uFECE\uFECF"+
	"\uFED0\uFED1\uFED2\uFED3\uFED4\uFED5\uFED6\uFED7"+
	"\uFED8\uFED9\uFEDA\uFEDB\uFEDC\uFEDD\uFEDE\uFEDF"+
	"\uFEE0\uFEE1\uFEE2\uFEE3\uFEE4\uFEE5\uFEE6\uFEE7"+
	"\uFEE8\uFEE9\uFEEA\uFEEB\uFEEC\uFEED\uFEEE\uFEEF"+
	"\uFEF0\uFEF1\uFEF2\uFEF3\uFEF4\uFEF5\uFEF6\uFEF7"+
	"\uFEF8\uFEF9\uFEFA\uFEFB\uFEFC\uFEFD\uFEFE\uFEFF"+
	"\uFF00\uFF5F\uFF60\uFF61\uFF62\uFF63\uFF64\uFF65"+
	"\uFF66\uFF67\uFF68\uFF69\uFF6A\uFF6B\uFF6C\uFF6D"+
	"\uFF6E\uFF6F\uFF70\uFF71\uFF72\uFF73\uFF74\uFF75"+
	"\uFF76\uFF77\uFF78\uFF79\uFF7A\uFF7B\uFF7C\uFF7D"+
	"\uFF7E\uFF7F\uFF80\uFF81\uFF82\uFF83\uFF84\uFF85"+
	"\uFF86\uFF87\uFF88\uFF89\uFF8A\uFF8B\uFF8C\uFF8D"+
	"\uFF8E\uFF8F\uFF90\uFF91\uFF92\uFF93\uFF94\uFF95"+
	"\uFF96\uFF97\uFF98\uFF99\uFF9A\uFF9B\uFF9C\uFF9D"+
	"\uFF9E\uFF9F\uFFA0\uFFA1\uFFA2\uFFA3\uFFA4\uFFA5"+
	"\uFFA6\uFFA7\uFFA8\uFFA9\uFFAA\uFFAB\uFFAC\uFFAD"+
	"\uFFAE\uFFAF\uFFB0\uFFB1\uFFB2\uFFB3\uFFB4\uFFB5"+
	"\uFFB6\uFFB7\uFFB8\uFFB9\uFFBA\uFFBB\uFFBC\uFFBD"+
	"\uFFBE\uFFBF\uFFC0\uFFC1\uFFC2\uFFC3\uFFC4\uFFC5"+
	"\uFFC6\uFFC7\uFFC8\uFFC9\uFFCA\uFFCB\uFFCC\uFFCD"+
	"\uFFCE\uFFCF\uFFD0\uFFD1\uFFD2\uFFD3\uFFD4\uFFD5"+
	"\uFFD6\uFFD7\uFFD8\uFFD9\uFFDA\uFFDB\uFFDC\uFFDD"+
	"\uFFDE\uFFDF\uFFE6\uFFE7\uFFE8\uFFE9\uFFEA\uFFEB"+
	"\uFFEC\uFFED\uFFEE\uFFEF\uFFF0\uFFF1\uFFF2\uFFF3"+
	"\uFFF4\uFFF5\uFFF6\uFFF7\uFFF8\uFFF9\uFFFA\uFFFB"+
	"\uFFFC\uFFFD\uFFFE\uFFFF\uFFFD\uFFFD\uFFFD\uFFFD";

    private final static short index1[] = {
        0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 
        16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 
        32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 
        48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 
        64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 75, 76, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 77, 78, 79, 80, 81, 82, 83, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    };

    String index2[] = {
        innerIndex0,
        innerIndex1,
        innerIndex2,
        innerIndex3,
        innerIndex4,
        innerIndex5
    };
}
