/*
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * @(#)CharToByteGB18030.java	1.7 03/01/23
 */


package sun.io;

public class CharToByteGB18030 extends CharToByteConverter
{

    private char highHalfZoneCode;
    boolean flushed = true;

    private final static int GB18030_SINGLE_BYTE = 1;
    private final static int GB18030_DOUBLE_BYTE = 2;
    private final static int GB18030_FOUR_BYTE = 3;
    private int currentState;

    public CharToByteGB18030() {
	currentState = GB18030_DOUBLE_BYTE;
         subBytes = new byte[1];
         subBytes[0] = (byte)'?';
    }

    public short[] getIndex1() {
	return(index1);
    }

    public String[] getIndex2() {
	return(index2);
    }

    public int flush(byte[] output, int outStart, int outEnd)
	throws MalformedInputException
    {
	if (highHalfZoneCode != 0) {
	    highHalfZoneCode = 0;
	    badInputLength = 0;
	    throw new MalformedInputException();
	}
	reset();
	flushed = true;
	return 0;
    }

    public void reset() {
	byteOff = charOff = 0;
	currentState = GB18030_DOUBLE_BYTE;
    }

    public boolean canConvert(char c) {
	// converts all but unpaired surrogates
	// and illegal chars, U+FFFE & U+FFFF
  
	if ((c >= 0xd800 && c <=0xdfff) || (c >= 0xfffe))
	    return false;
	else
	    return true;
    }

    /**
     * Character conversion
     */
    public int convert(char[] input, int inOff, int inEnd,
                       byte[] output, int outOff, int outEnd)
        throws UnknownCharacterException, MalformedInputException,
               ConversionBufferFullException
    {
	int linearDiffValue = 0;
	int hiByte = 0 , loByte = 0;  // low and high order bytes
	char inputChar;  // Input character to be converted
	charOff = inOff;
	byteOff = outOff;
	int inputSize;  // Size of the input
	int outputSize; // Size of the output

	flushed = false;

	if (highHalfZoneCode != 0) {
	    if (input[inOff] >= 0xDC00 && input[inOff] <= 0xDFFF) {

		// This is legal UTF16 sequence, so shunt in the high
		// surrogate for conversion by convert() loop.

	        char[] newBuf = new char[inEnd - inOff + 1];
		newBuf[0] = highHalfZoneCode;
	        System.arraycopy(input, inOff, newBuf, 1, inEnd - inOff);
		charOff -= 1;
		input = newBuf;
		inOff = 0;
		inEnd = newBuf.length;
	        highHalfZoneCode = 0;
	    } else {
		// This is illegal UTF16 sequence.
		badInputLength = 0;
		throw new MalformedInputException();
	    }
	}

	// Main encode loop

	while (charOff < inEnd) {
	    inputChar = input[charOff++];

            if(inputChar >= '\uD800' && inputChar <= '\uDBFF') {
                // Is this the last character of the input?
                if (charOff + 1 > inEnd) {
		    highHalfZoneCode = inputChar;
		    break;
                }

                char previousChar = inputChar;
	        inputChar = input[charOff];

                // Is there a low surrogate following?
                if (inputChar >= '\uDC00' && inputChar <= '\uDFFF') {
		    inputSize = 2;
		    charOff++;
		    linearDiffValue = ( previousChar - 0xD800) * 0x400 +
				( inputChar - 0xDC00) + 0x2E248;

		    currentState = GB18030_FOUR_BYTE;
                } else {
                    // We have a malformed surrogate pair
                    badInputLength = 1;
                    throw new MalformedInputException();
                }
            }
            // Is this an unaccompanied low surrogate?
	    else if (inputChar >= '\uDC00' && inputChar <= '\uDFFF') {
		badInputLength = 1;
		throw new MalformedInputException();
	    }
	    
	    // Not part of a surrogate
	    else if (inputChar >= 0x0000 && inputChar <= 0x007F) {
		if (byteOff >= outEnd) {
		   throw new ConversionBufferFullException();
		}
		currentState = GB18030_SINGLE_BYTE;
		output[byteOff++] = (byte) inputChar;
	    }
	    else if (inputChar <= 0xA4C6 || inputChar >= 0xE000) {
		int outByteVal = getGB18030(index1, index2, inputChar);

		if (outByteVal == 0xFFFD ) {
		    if (subMode) {
			if (byteOff >= outEnd) {
		           throw new ConversionBufferFullException();
			} else {
			    output[byteOff++] = subBytes[0];
			    continue;
			}
		    } else {
			badInputLength = 1;
			throw new UnknownCharacterException();
		    }
		}

		hiByte = (outByteVal & 0xFF00) >> 8;
		loByte = (outByteVal & 0xFF);

	        linearDiffValue = (hiByte - 0x20) * 256 + loByte;

		if (inputChar >= 0xE000 && inputChar < 0xF900)
			linearDiffValue += 0x82BD;
		else if (inputChar >= 0xF900)
			linearDiffValue += 0x93A9;

		if (hiByte > 0x80)
		     currentState = GB18030_DOUBLE_BYTE;
		else
		     currentState = GB18030_FOUR_BYTE;
	    }
	    else if (inputChar >= 0xA4C7 && inputChar <= 0xD7FF) {
		linearDiffValue = inputChar - 0x5543;
		currentState = GB18030_FOUR_BYTE;
	    }
	    else {
		badInputLength = 1;
		throw new MalformedInputException();
	    }

	    if (currentState == GB18030_SINGLE_BYTE)
		continue;

	    if (currentState == GB18030_DOUBLE_BYTE) {
		if (byteOff + 2 > outEnd) {
		    throw new ConversionBufferFullException();
		}
		output[byteOff++] = (byte)hiByte;
		output[byteOff++] = (byte)loByte;
	    }
	    else { // Four Byte encoding
		if (byteOff + 4 > outEnd) {
		    throw new ConversionBufferFullException();
		}

	        byte b1, b2, b3, b4;

		b4 = (byte)((linearDiffValue % 10) + 0x30);
		linearDiffValue /= 10;
		b3 = (byte)((linearDiffValue % 126) + 0x81);
		linearDiffValue /= 126;
		b2 = (byte)((linearDiffValue % 10) + 0x30);
		b1 = (byte)((linearDiffValue / 10) + 0x81);
		output[byteOff++] = b1;
		output[byteOff++] = b2;
		output[byteOff++] = b3;
		output[byteOff++] = b4;
	    }
	}
	// Return number of bytes written to the output buffer. 
	return byteOff - outOff;
    }


    /**
     * returns the maximum number of bytes needed to convert a char
     */
    public int getMaxBytesPerChar() {
        return 4;
    }


    /**
     * Return the character set ID
     */
    public String getCharacterEncoding() {
        return "GB18030";
    }


    /**
     * Unicode table
     */

    private final static String innerIndex0= 
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
        "\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007"+
        "\u2008\u2009\u200A\u200B\u200C\u200D\u200E\u200F"+
        "\u2010\u2011\u2012\u2013\u2014\u2015\u2016\u2017"+
        "\u2018\u2019\u201A\u201B\u201C\u201D\u201E\u201F"+
        "\u2020\u2021\u2022\u2023\uA1E8\u2024\u2025\uA1EC"+
        "\uA1A7\u2026\u2027\u2028\u2029\u202A\u202B\u202C"+
        "\uA1E3\uA1C0\u202D\u202E\u202F\u2030\u2031\uA1A4"+
        "\u2032\u2033\u2034\u2035\u2036\u2037\u2038\u2039"+
        "\u203A\u203B\u203C\u203D\u203E\u203F\u2040\u2041"+
        "\u2042\u2043\u2044\u2045\u2046\u2047\u2048\u2049"+
        "\u204A\u204B\u204C\u204D\u204E\u204F\u2050\uA1C1"+
        "\u2051\u2052\u2053\u2054\u2055\u2056\u2057\u2058"+
        "\uA8A4\uA8A2\u2059\u205A\u205B\u205C\u205D\u205E"+
        "\uA8A8\uA8A6\uA8BA\u205F\uA8AC\uA8AA\u2060\u2061"+
        "\u2062\u2063\uA8B0\uA8AE\u2064\u2065\u2066\uA1C2"+
        "\u2067\uA8B4\uA8B2\u2068\uA8B9\u2069\u206A\u206B"+
	"\u206C\uA8A1\u206D\u206E\u206F\u2070\u2071\u2072"+
	"\u2073\u2074\u2075\u2076\u2077\u2078\u2079\u207A"+
	"\u207B\u207C\u207D\uA8A5\u207E\u207F\u2080\u2081"+
	"\u2082\u2083\u2084\uA8A7\u2085\u2086\u2087\u2088"+
	"\u2089\u208A\u208B\u208C\u208D\u208E\u208F\u2090"+
	"\u2091\u2092\u2093\uA8A9\u2094\u2095\u2096\u2097"+
	"\u2098\u2099\u209A\u209B\u209C\u209D\u209E\u209F"+
	"\u20A0\u20A1\u20A2\u20A3\u20A4\u20A5\u20A6\u20A7"+
	"\u20A8\u20A9\u20AA\u20AB\uA8BD\u20AC\u20AD\u20AE"+
	"\uA8BE\u20AF\u20B0\u20B1\u20B2\uA8AD\u20B3\u20B4"+
	"\u20B5\u20B6\u20B7\u20B8\u20B9\u20BA\u20BB\u20BC"+
	"\u20BD\u20BE\u20BF\u20C0\u20C1\u20C2\u20C3\u20C4"+
	"\u20C5\u20C6\u20C7\u20C8\u20C9\u20CA\u20CB\u20CC"+
	"\u20CD\u20CE\u20CF\uA8B1\u20D0\u20D1\u20D2\u20D3"+
	"\u20D4\u20D5\u20D6\u20D7\u20D8\u20D9\u20DA\u20DB"+
	"\u20DC\u20DD\u20DE\u20DF\u20E0\u20E1\u20E2\u20E3"+
	"\u20E4\u20E5\u20E6\u20E7\u20E8\u20E9\u20EA\u20EB"+
	"\u20EC\u20ED\u20EE\u20EF\u20F0\u20F1\u20F2\u20F3"+
	"\u20F4\u20F5\u20F6\u20F7\u20F8\u20F9\u20FA\u20FB"+
	"\u20FC\u20FD\u20FE\u20FF\u2100\u2101\u2102\u2103"+
	"\u2104\u2105\u2106\u2107\u2108\u2109\u210A\u210B"+
	"\u210C\u210D\u210E\u210F\u2110\u2111\u2112\u2113"+
	"\u2114\u2115\u2116\u2117\u2118\u2119\u211A\u211B"+
	"\u211C\u211D\u211E\u211F\u2120\u2121\u2122\u2123"+
	"\u2124\u2125\u2126\u2127\u2128\u2129\u212A\u212B"+
	"\u212C\u212D\u212E\u212F\u2130\u2131\uA8A3\u2132"+
	"\uA8AB\u2133\uA8AF\u2134\uA8B3\u2135\uA8B5\u2136"+
	"\uA8B6\u2137\uA8B7\u2138\uA8B8\u2139\u213A\u213B"+
	"\u213C\u213D\u213E\u213F\u2140\u2141\u2142\u2143"+
	"\u2144\u2145\u2146\u2147\u2148\u2149\u214A\u214B"+
	"\u214C\u214D\u214E\u214F\u2150\u2151\u2152\u2153"+
	"\u2154\uA8BF\u2155\u2156\u2157\u2158\u2159\u215A"+
	"\u215B\u215C\u215D\u215E\u215F\u2160\u2161\u2162"+
	"\u2163\u2164\u2165\u2166\u2167\u2168\u2169\u216A"+
	"\u216B\u216C\u216D\u216E\u216F\u2170\u2171\u2172"+
	"\u2173\u2174\u2175\u2176\u2177\u2178\u2179\u217A"+
	"\u217B\u217C\u217D\u217E\u217F\u2180\u2181\u2182"+
	"\u2183\u2184\u2185\u2186\u2187\u2188\u2189\u218A"+
	"\u218B\u218C\u218D\u218E\u218F\u2190\u2191\u2192"+
	"\u2193\u2194\u2195\u2196\u2197\u2198\u2199\u219A"+
	"\u219B\u219C\u219D\u219E\u219F\u21A0\u21A1\u21A2"+
	"\u21A3\u21A4\u21A5\u21A6\u21A7\u21A8\u21A9\u21AA"+
	"\u21AB\uA8BB\u21AC\u21AD\u21AE\u21AF\u21B0\u21B1"+
	"\u21B2\u21B3\u21B4\u21B5\u21B6\u21B7\u21B8\u21B9"+
	"\u21BA\uA8C0\u21BB\u21BC\u21BD\u21BE\u21BF\u21C0"+
	"\u21C1\u21C2\u21C3\u21C4\u21C5\u21C6\u21C7\u21C8"+
	"\u21C9\u21CA\u21CB\u21CC\u21CD\u21CE\u21CF\u21D0"+
	"\u21D1\u21D2\u21D3\u21D4\u21D5\u21D6\u21D7\u21D8"+
	"\u21D9\u21DA\u21DB\u21DC\u21DD\u21DE\u21DF\u21E0"+
	"\u21E1\u21E2\u21E3\u21E4\u21E5\u21E6\u21E7\u21E8"+
	"\u21E9\u21EA\u21EB\u21EC\u21ED\u21EE\u21EF\u21F0"+
	"\u21F1\u21F2\u21F3\u21F4\u21F5\u21F6\u21F7\u21F8"+
	"\u21F9\u21FA\u21FB\u21FC\u21FD\u21FE\u21FF\u2200"+
	"\u2201\u2202\u2203\u2204\u2205\u2206\u2207\u2208"+
	"\u2209\u220A\u220B\u220C\u220D\u220E\u220F\u2210"+
	"\u2211\u2212\u2213\u2214\u2215\u2216\u2217\u2218"+
	"\u2219\u221A\u221B\u221C\u221D\u221E\u221F\uA1A6"+
	"\u2220\uA1A5\uA840\uA841\u2221\u2222\u2223\u2224"+
	"\u2225\u2226\u2227\u2228\u2229\u222A\u222B\u222C"+
	"\u222D\uA842\u222E\u222F\u2230\u2231\u2232\u2233"+
	"\u2234\u2235\u2236\u2237\u2238\u2239\u223A\u223B"+
	"\u223C\u223D\u223E\u223F\u2240\u2241\u2242\u2243"+
	"\u2244\u2245\u2246\u2247\u2248\u2249\u224A\u224B"+
	"\u224C\u224D\u224E\u224F\u2250\u2251\u2252\u2253"+
	"\u2254\u2255\u2256\u2257\u2258\u2259\u225A\u225B"+
	"\u225C\u225D\u225E\u225F\u2260\u2261\u2262\u2263"+
	"\u2264\u2265\u2266\u2267\u2268\u2269\u226A\u226B"+
	"\u226C\u226D\u226E\u226F\u2270\u2271\u2272\u2273"+
	"\u2274\u2275\u2276\u2277\u2278\u2279\u227A\u227B"+
	"\u227C\u227D\u227E\u227F\u2280\u2281\u2282\u2283"+
	"\u2284\u2285\u2286\u2287\u2288\u2289\u228A\u228B"+
	"\u228C\u228D\u228E\u228F\u2290\u2291\u2292\u2293"+
	"\u2294\u2295\u2296\u2297\u2298\u2299\u229A\u229B"+
	"\u229C\u229D\u229E\u229F\u22A0\u22A1\u22A2\u22A3"+
	"\u22A4\u22A5\u22A6\u22A7\u22A8\u22A9\u22AA\u22AB"+
	"\u22AC\u22AD\u22AE\u22AF\u22B0\u22B1\u22B2\u22B3"+
	"\u22B4\u22B5\u22B6\u22B7\u22B8\u22B9\u22BA\u22BB"+
	"\u22BC\u22BD\u22BE\u22BF\u22C0\u22C1\u22C2\u22C3"+
	"\u22C4\u22C5\u22C6\u22C7\u22C8\u22C9\u22CA\u22CB"+
	"\u22CC\u22CD\u22CE\u22CF\u22D0\u22D1\u22D2\u22D3"+
	"\u22D4\u22D5\u22D6\u22D7\u22D8\u22D9\u22DA\u22DB"+
	"\u22DC\u22DD\u22DE\u22DF\u22E0\u22E1\u22E2\u22E3"+
	"\u22E4\uA6A1\uA6A2\uA6A3\uA6A4\uA6A5\uA6A6\uA6A7"+
	"\uA6A8\uA6A9\uA6AA\uA6AB\uA6AC\uA6AD\uA6AE\uA6AF"+
	"\uA6B0\uA6B1\u22E5\uA6B2\uA6B3\uA6B4\uA6B5\uA6B6"+
	"\uA6B7\uA6B8\u22E6\u22E7\u22E8\u22E9\u22EA\u22EB"+
	"\u22EC\uA6C1\uA6C2\uA6C3\uA6C4\uA6C5\uA6C6\uA6C7"+
	"\uA6C8\uA6C9\uA6CA\uA6CB\uA6CC\uA6CD\uA6CE\uA6CF"+
	"\uA6D0\uA6D1\u22ED\uA6D2\uA6D3\uA6D4\uA6D5\uA6D6"+
	"\uA6D7\uA6D8\u22EE\u22EF\u22F0\u22F1\u22F2\u22F3"+
	"\u22F4\u22F5\u22F6\u22F7\u22F8\u22F9\u22FA\u22FB"+
	"\u22FC\u22FD\u22FE\u22FF\u2300\u2301\u2302\u2303"+
	"\u2304\u2305\u2306\u2307\u2308\u2309\u230A\u230B"+
	"\u230C\u230D\u230E\u230F\u2310\u2311\u2312\u2313"+
	"\u2314\u2315\u2316\u2317\u2318\u2319\u231A\u231B"+
	"\u231C\u231D\u231E\u231F\u2320\u2321\u2322\u2323"+
	"\u2324\uA7A7\u2325\u2326\u2327\u2328\u2329\u232A"+
	"\u232B\u232C\u232D\u232E\u232F\u2330\u2331\u2332"+
	"\uA7A1\uA7A2\uA7A3\uA7A4\uA7A5\uA7A6\uA7A8\uA7A9"+
	"\uA7AA\uA7AB\uA7AC\uA7AD\uA7AE\uA7AF\uA7B0\uA7B1"+
	"\uA7B2\uA7B3\uA7B4\uA7B5\uA7B6\uA7B7\uA7B8\uA7B9"+
	"\uA7BA\uA7BB\uA7BC\uA7BD\uA7BE\uA7BF\uA7C0\uA7C1"+
	"\uA7D1\uA7D2\uA7D3\uA7D4\uA7D5\uA7D6\uA7D8\uA7D9"+
	"\uA7DA\uA7DB\uA7DC\uA7DD\uA7DE\uA7DF\uA7E0\uA7E1"+
	"\uA7E2\uA7E3\uA7E4\uA7E5\uA7E6\uA7E7\uA7E8\uA7E9"+
	"\uA7EA\uA7EB\uA7EC\uA7ED\uA7EE\uA7EF\uA7F0\uA7F1"+
	"\u2333\uA7D7\u2334\u2335\u2336\u2337\u2338\u2339"+
	"\u233A\u233B\u233C\u233D\u233E\u233F\u2340\u2341"+
	"\u2342\u2343\u2344\u2345\u2346\u2347\u2348\u2349"+
	"\u234A\u234B\u234C\u234D\u234E\u234F\u2350\u2351"+
	"\u2352\u2353\u2354\u2355\u2356\u2357\u2358\u2359"+
	"\u235A\u235B\u235C\u235D\u235E\u235F\u2360\u2361"+
	"\u2362\u2363\u2364\u2365\u2366\u2367\u2368\u2369"+
	"\u236A\u236B\u236C\u236D\u236E\u236F\u2370\u2371"+
	"\u2372\u2373\u2374\u2375\u2376\u2377\u2378\u2379"+
	"\u237A\u237B\u237C\u237D\u237E\u237F\u2380\u2381"+
	"\u2382\u2383\u2384\u2385\u2386\u2387\u2388\u2389"+
	"\u238A\u238B\u238C\u238D\u238E\u238F\u2390\u2391"+
	"\u2392\u2393\u2394\u2395\u2396\u2397\u2398\u2399"+
	"\u239A\u239B\u239C\u239D\u239E\u239F\u23A0\u23A1"+
	"\u23A2\u23A3\u23A4\u23A5\u23A6\u23A7\u23A8\u23A9"+
	"\u23AA\u23AB\u23AC\u23AD\u23AE\u23AF\u23B0\u23B1"+
	"\u23B2\u23B3\u23B4\u23B5\u23B6\u23B7\u23B8\u23B9"+
	"\u23BA\u23BB\u23BC\u23BD\u23BE\u23BF\u23C0\u23C1"+
	"\u23C2\u23C3\u23C4\u23C5\u23C6\u23C7\u23C8\u23C9"+
	"\u23CA\u23CB\u23CC\u23CD\u23CE\u23CF\u23D0\u23D1"+
	"\u23D2\u23D3\u23D4\u23D5\u23D6\u23D7\u23D8\u23D9"+
	"\u23DA\u23DB\u23DC\u23DD\u23DE\u23DF\u23E0\u23E1"+
	"\u23E2\u23E3\u23E4\u23E5\u23E6\u23E7\u23E8\u23E9"+
	"\u23EA\u23EB\u23EC\u23ED\u23EE\u23EF\u23F0\u23F1"+
	"\u23F2\u23F3\u23F4\u23F5\u23F6\u23F7\u23F8\u23F9"+
	"\u23FA\u23FB\u23FC\u23FD\u23FE\u23FF\u2400\u2401"+
	"\u2402\u2403\u2404\u2405\u2406\u2407\u2408\u2409"+
	"\u240A\u240B\u240C\u240D\u240E\u240F\u2410\u2411"+
	"\u2412\u2413\u2414\u2415\u2416\u2417\u2418\u2419"+
	"\u241A\u241B\u241C\u241D\u241E\u241F\u2420\u2421"+
	"\u2422\u2423\u2424\u2425\u2426\u2427\u2428\u2429"+
	"\u242A\u242B\u242C\u242D\u242E\u242F\u2430\u2431"+
	"\u2432\u2433\u2434\u2435\u2436\u2437\u2438\u2439"+
	"\u243A\u243B\u243C\u243D\u243E\u243F\u2440\u2441"+
	"\u2442\u2443\u2444\u2445\u2446\u2447\u2448\u2449"+
	"\u244A\u244B\u244C\u244D\u244E\u244F\u2450\u2451"+
	"\u2452\u2453\u2454\u2455\u2456\u2457\u2458\u2459"+
	"\u245A\u245B\u245C\u245D\u245E\u245F\u2460\u2461"+
	"\u2462\u2463\u2464\u2465\u2466\u2467\u2468\u2469"+
	"\u246A\u246B\u246C\u246D\u246E\u246F\u2470\u2471"+
	"\u2472\u2473\u2474\u2475\u2476\u2477\u2478\u2479"+
	"\u247A\u247B\u247C\u247D\u247E\u247F\u2480\u2481"+
	"\u2482\u2483\u2484\u2485\u2486\u2487\u2488\u2489"+
	"\u248A\u248B\u248C\u248D\u248E\u248F\u2490\u2491"+
	"\u2492\u2493\u2494\u2495\u2496\u2497\u2498\u2499"+
	"\u249A\u249B\u249C\u249D\u249E\u249F\u24A0\u24A1"+
	"\u24A2\u24A3\u24A4\u24A5\u24A6\u24A7\u24A8\u24A9"+
	"\u24AA\u24AB\u24AC\u24AD\u24AE\u24AF\u24B0\u24B1"+
	"\u24B2\u24B3\u24B4\u24B5\u24B6\u24B7\u24B8\u24B9"+
	"\u24BA\u24BB\u24BC\u24BD\u24BE\u24BF\u24C0\u24C1"+
	"\u24C2\u24C3\u24C4\u24C5\u24C6\u24C7\u24C8\u24C9"+
	"\u24CA\u24CB\u24CC\u24CD\u24CE\u24CF\u24D0\u24D1"+
	"\u24D2\u24D3\u24D4\u24D5\u24D6\u24D7\u24D8\u24D9"+
	"\u24DA\u24DB\u24DC\u24DD\u24DE\u24DF\u24E0\u24E1"+
	"\u24E2\u24E3\u24E4\u24E5\u24E6\u24E7\u24E8\u24E9"+
	"\u24EA\u24EB\u24EC\u24ED\u24EE\u24EF\u24F0\u24F1"+
	"\u24F2\u24F3\u24F4\u24F5\u24F6\u24F7\u24F8\u24F9"+
	"\u24FA\u24FB\u24FC\u24FD\u24FE\u24FF\u2500\u2501"+
	"\u2502\u2503\u2504\u2505\u2506\u2507\u2508\u2509"+
	"\u250A\u250B\u250C\u250D\u250E\u250F\u2510\u2511"+
	"\u2512\u2513\u2514\u2515\u2516\u2517\u2518\u2519"+
	"\u251A\u251B\u251C\u251D\u251E\u251F\u2520\u2521"+
	"\u2522\u2523\u2524\u2525\u2526\u2527\u2528\u2529"+
	"\u252A\u252B\u252C\u252D\u252E\u252F\u2530\u2531"+
	"\u2532\u2533\u2534\u2535\u2536\u2537\u2538\u2539"+
	"\u253A\u253B\u253C\u253D\u253E\u253F\u2540\u2541"+
	"\u2542\u2543\u2544\u2545\u2546\u2547\u2548\u2549"+
	"\u254A\u254B\u254C\u254D\u254E\u254F\u2550\u2551"+
	"\u2552\u2553\u2554\u2555\u2556\u2557\u2558\u2559"+
	"\u255A\u255B\u255C\u255D\u255E\u255F\u2560\u2561"+
	"\u2562\u2563\u2564\u2565\u2566\u2567\u2568\u2569"+
	"\u256A\u256B\u256C\u256D\u256E\u256F\u2570\u2571"+
	"\u2572\u2573\u2574\u2575\u2576\u2577\u2578\u2579"+
	"\u257A\u257B\u257C\u257D\u257E\u257F\u2580\u2581"+
	"\u2582\u2583\u2584\u2585\u2586\u2587\u2588\u2589"+
	"\u258A\u258B\u258C\u258D\u258E\u258F\u2590\u2591"+
	"\u2592\u2593\u2594\u2595\u2596\u2597\u2598\u2599"+
	"\u259A\u259B\u259C\u259D\u259E\u259F\u25A0\u25A1"+
	"\u25A2\u25A3\u25A4\u25A5\u25A6\u25A7\u25A8\u25A9"+
	"\u25AA\u25AB\u25AC\u25AD\u25AE\u25AF\u25B0\u25B1"+
	"\u25B2\u25B3\u25B4\u25B5\u25B6\u25B7\u25B8\u25B9"+
	"\u25BA\u25BB\u25BC\u25BD\u25BE\u25BF\u25C0\u25C1"+
	"\u25C2\u25C3\u25C4\u25C5\u25C6\u25C7\u25C8\u25C9"+
	"\u25CA\u25CB\u25CC\u25CD\u25CE\u25CF\u25D0\u25D1"+
	"\u25D2\u25D3\u25D4\u25D5\u25D6\u25D7\u25D8\u25D9"+
	"\u25DA\u25DB\u25DC\u25DD\u25DE\u25DF\u25E0\u25E1"+
	"\u25E2\u25E3\u25E4\u25E5\u25E6\u25E7\u25E8\u25E9"+
	"\u25EA\u25EB\u25EC\u25ED\u25EE\u25EF\u25F0\u25F1"+
	"\u25F2\u25F3\u25F4\u25F5\u25F6\u25F7\u25F8\u25F9"+
	"\u25FA\u25FB\u25FC\u25FD\u25FE\u25FF\u2600\u2601"+
	"\u2602\u2603\u2604\u2605\u2606\u2607\u2608\u2609"+
	"\u260A\u260B\u260C\u260D\u260E\u260F\u2610\u2611"+
	"\u2612\u2613\u2614\u2615\u2616\u2617\u2618\u2619"+
	"\u261A\u261B\u261C\u261D\u261E\u261F\u2620\u2621"+
	"\u2622\u2623\u2624\u2625\u2626\u2627\u2628\u2629"+
	"\u262A\u262B\u262C\u262D\u262E\u262F\u2630\u2631"+
	"\u2632\u2633\u2634\u2635\u2636\u2637\u2638\u2639"+
	"\u263A\u263B\u263C\u263D\u263E\u263F\u2640\u2641"+
	"\u2642\u2643\u2644\u2645\u2646\u2647\u2648\u2649"+
	"\u264A\u264B\u264C\u264D\u264E\u264F\u2650\u2651"+
	"\u2652\u2653\u2654\u2655\u2656\u2657\u2658\u2659"+
	"\u265A\u265B\u265C\u265D\u265E\u265F\u2660\u2661"+
	"\u2662\u2663\u2664\u2665\u2666\u2667\u2668\u2669"+
	"\u266A\u266B\u266C\u266D\u266E\u266F\u2670\u2671"+
	"\u2672\u2673\u2674\u2675\u2676\u2677\u2678\u2679"+
	"\u267A\u267B\u267C\u267D\u267E\u267F\u2680\u2681"+
	"\u2682\u2683\u2684\u2685\u2686\u2687\u2688\u2689"+
	"\u268A\u268B\u268C\u268D\u268E\u268F\u2690\u2691"+
	"\u2692\u2693\u2694\u2695\u2696\u2697\u2698\u2699"+
	"\u269A\u269B\u269C\u269D\u269E\u269F\u26A0\u26A1"+
	"\u26A2\u26A3\u26A4\u26A5\u26A6\u26A7\u26A8\u26A9"+
	"\u26AA\u26AB\u26AC\u26AD\u26AE\u26AF\u26B0\u26B1"+
	"\u26B2\u26B3\u26B4\u26B5\u26B6\u26B7\u26B8\u26B9"+
	"\u26BA\u26BB\u26BC\u26BD\u26BE\u26BF\u26C0\u26C1"+
	"\u26C2\u26C3\u26C4\u26C5\u26C6\u26C7\u26C8\u26C9"+
	"\u26CA\u26CB\u26CC\u26CD\u26CE\u26CF\u26D0\u26D1"+
	"\u26D2\u26D3\u26D4\u26D5\u26D6\u26D7\u26D8\u26D9"+
	"\u26DA\u26DB\u26DC\u26DD\u26DE\u26DF\u26E0\u26E1"+
	"\u26E2\u26E3\u26E4\u26E5\u26E6\u26E7\u26E8\u26E9"+
	"\u26EA\u26EB\u26EC\u26ED\u26EE\u26EF\u26F0\u26F1"+
	"\u26F2\u26F3\u26F4\u26F5\u26F6\u26F7\u26F8\u26F9"+
	"\u26FA\u26FB\u26FC\u26FD\u26FE\u26FF\u2700\u2701"+
	"\u2702\u2703\u2704\u2705\u2706\u2707\u2708\u2709"+
	"\u270A\u270B\u270C\u270D\u270E\u270F\u2710\u2711"+
	"\u2712\u2713\u2714\u2715\u2716\u2717\u2718\u2719"+
	"\u271A\u271B\u271C\u271D\u271E\u271F\u2720\u2721"+
	"\u2722\u2723\u2724\u2725\u2726\u2727\u2728\u2729"+
	"\u272A\u272B\u272C\u272D\u272E\u272F\u2730\u2731"+
	"\u2732\u2733\u2734\u2735\u2736\u2737\u2738\u2739"+
	"\u273A\u273B\u273C\u273D\u273E\u273F\u2740\u2741"+
	"\u2742\u2743\u2744\u2745\u2746\u2747\u2748\u2749"+
	"\u274A\u274B\u274C\u274D\u274E\u274F\u2750\u2751"+
	"\u2752\u2753\u2754\u2755\u2756\u2757\u2758\u2759"+
	"\u275A\u275B\u275C\u275D\u275E\u275F\u2760\u2761"+
	"\u2762\u2763\u2764\u2765\u2766\u2767\u2768\u2769"+
	"\u276A\u276B\u276C\u276D\u276E\u276F\u2770\u2771"+
	"\u2772\u2773\u2774\u2775\u2776\u2777\u2778\u2779"+
	"\u277A\u277B\u277C\u277D\u277E\u277F\u2780\u2781"+
	"\u2782\u2783\u2784\u2785\u2786\u2787\u2788\u2789"+
	"\u278A\u278B\u278C\u278D\u278E\u278F\u2790\u2791"+
	"\u2792\u2793\u2794\u2795\u2796\u2797\u2798\u2799"+
	"\u279A\u279B\u279C\u279D\u279E\u279F\u27A0\u27A1"+
	"\u27A2\u27A3\u27A4\u27A5\u27A6\u27A7\u27A8\u27A9"+
	"\u27AA\u27AB\u27AC\u27AD\u27AE\u27AF\u27B0\u27B1"+
	"\u27B2\u27B3\u27B4\u27B5\u27B6\u27B7\u27B8\u27B9"+
	"\u27BA\u27BB\u27BC\u27BD\u27BE\u27BF\u27C0\u27C1"+
	"\u27C2\u27C3\u27C4\u27C5\u27C6\u27C7\u27C8\u27C9"+
	"\u27CA\u27CB\u27CC\u27CD\u27CE\u27CF\u27D0\u27D1"+
	"\u27D2\u27D3\u27D4\u27D5\u27D6\u27D7\u27D8\u27D9"+
	"\u27DA\u27DB\u27DC\u27DD\u27DE\u27DF\u27E0\u27E1"+
	"\u27E2\u27E3\u27E4\u27E5\u27E6\u27E7\u27E8\u27E9"+
	"\u27EA\u27EB\u27EC\u27ED\u27EE\u27EF\u27F0\u27F1"+
	"\u27F2\u27F3\u27F4\u27F5\u27F6\u27F7\u27F8\u27F9"+
	"\u27FA\u27FB\u27FC\u27FD\u27FE\u27FF\u2800\u2801"+
	"\u2802\u2803\u2804\u2805\u2806\u2807\u2808\u2809"+
	"\u280A\u280B\u280C\u280D\u280E\u280F\u2810\u2811"+
	"\u2812\u2813\u2814\u2815\u2816\u2817\u2818\u2819"+
	"\u281A\u281B\u281C\u281D\u281E\u281F\u2820\u2821"+
	"\u2822\u2823\u2824\u2825\u2826\u2827\u2828\u2829"+
	"\u282A\u282B\u282C\u282D\u282E\u282F\u2830\u2831"+
	"\u2832\u2833\u2834\u2835\u2836\u2837\u2838\u2839"+
	"\u283A\u283B\u283C\u283D\u283E\u283F\u2840\u2841"+
	"\u2842\u2843\u2844\u2845\u2846\u2847\u2848\u2849"+
	"\u284A\u284B\u284C\u284D\u284E\u284F\u2850\u2851"+
	"\u2852\u2853\u2854\u2855\u2856\u2857\u2858\u2859"+
	"\u285A\u285B\u285C\u285D\u285E\u285F\u2860\u2861"+
	"\u2862\u2863\u2864\u2865\u2866\u2867\u2868\u2869"+
	"\u286A\u286B\u286C\u286D\u286E\u286F\u2870\u2871"+
	"\u2872\u2873\u2874\u2875\u2876\u2877\u2878\u2879"+
	"\u287A\u287B\u287C\u287D\u287E\u287F\u2880\u2881"+
	"\u2882\u2883\u2884\u2885\u2886\u2887\u2888\u2889"+
	"\u288A\u288B\u288C\u288D\u288E\u288F\u2890\u2891"+
	"\u2892\u2893\u2894\u2895\u2896\u2897\u2898\u2899"+
	"\u289A\u289B\u289C\u289D\u289E\u289F\u28A0\u28A1"+
	"\u28A2\u28A3\u28A4\u28A5\u28A6\u28A7\u28A8\u28A9"+
	"\u28AA\u28AB\u28AC\u28AD\u28AE\u28AF\u28B0\u28B1"+
	"\u28B2\u28B3\u28B4\u28B5\u28B6\u28B7\u28B8\u28B9"+
	"\u28BA\u28BB\u28BC\u28BD\u28BE\u28BF\u28C0\u28C1"+
	"\u28C2\u28C3\u28C4\u28C5\u28C6\u28C7\u28C8\u28C9"+
	"\u28CA\u28CB\u28CC\u28CD\u28CE\u28CF\u28D0\u28D1"+
	"\u28D2\u28D3\u28D4\u28D5\u28D6\u28D7\u28D8\u28D9"+
	"\u28DA\u28DB\u28DC\u28DD\u28DE\u28DF\u28E0\u28E1"+
	"\u28E2\u28E3\u28E4\u28E5\u28E6\u28E7\u28E8\u28E9"+
	"\u28EA\u28EB\u28EC\u28ED\u28EE\u28EF\u28F0\u28F1"+
	"\u28F2\u28F3\u28F4\u28F5\u28F6\u28F7\u28F8\u28F9"+
	"\u28FA\u28FB\u28FC\u28FD\u28FE\u28FF\u2900\u2901"+
	"\u2902\u2903\u2904\u2905\u2906\u2907\u2908\u2909"+
	"\u290A\u290B\u290C\u290D\u290E\u290F\u2910\u2911"+
	"\u2912\u2913\u2914\u2915\u2916\u2917\u2918\u2919"+
	"\u291A\u291B\u291C\u291D\u291E\u291F\u2920\u2921"+
	"\u2922\u2923\u2924\u2925\u2926\u2927\u2928\u2929"+
	"\u292A\u292B\u292C\u292D\u292E\u292F\u2930\u2931"+
	"\u2932\u2933\u2934\u2935\u2936\u2937\u2938\u2939"+
	"\u293A\u293B\u293C\u293D\u293E\u293F\u2940\u2941"+
	"\u2942\u2943\u2944\u2945\u2946\u2947\u2948\u2949"+
	"\u294A\u294B\u294C\u294D\u294E\u294F\u2950\u2951"+
	"\u2952\u2953\u2954\u2955\u2956\u2957\u2958\u2959"+
	"\u295A\u295B\u295C\u295D\u295E\u295F\u2960\u2961"+
	"\u2962\u2963\u2964\u2965\u2966\u2967\u2968\u2969"+
	"\u296A\u296B\u296C\u296D\u296E\u296F\u2970\u2971"+
	"\u2972\u2973\u2974\u2975\u2976\u2977\u2978\u2979"+
	"\u297A\u297B\u297C\u297D\u297E\u297F\u2980\u2981"+
	"\u2982\u2983\u2984\u2985\u2986\u2987\u2988\u2989"+
	"\u298A\u298B\u298C\u298D\u298E\u298F\u2990\u2991"+
	"\u2992\u2993\u2994\u2995\u2996\u2997\u2998\u2999"+
	"\u299A\u299B\u299C\u299D\u299E\u299F\u29A0\u29A1"+
	"\u29A2\u29A3\u29A4\u29A5\u29A6\u29A7\u29A8\u29A9"+
	"\u29AA\u29AB\u29AC\u29AD\u29AE\u29AF\u29B0\u29B1"+
	"\u29B2\u29B3\u29B4\u29B5\u29B6\u29B7\u29B8\u29B9"+
	"\u29BA\u29BB\u29BC\u29BD\u29BE\u29BF\u29C0\u29C1"+
	"\u29C2\u29C3\u29C4\u29C5\u29C6\u29C7\u29C8\u29C9"+
	"\u29CA\u29CB\u29CC\u29CD\u29CE\u29CF\u29D0\u29D1"+
	"\u29D2\u29D3\u29D4\u29D5\u29D6\u29D7\u29D8\u29D9"+
	"\u29DA\u29DB\u29DC\u29DD\u29DE\u29DF\u29E0\u29E1"+
	"\u29E2\u29E3\u29E4\u29E5\u29E6\u29E7\u29E8\u29E9"+
	"\u29EA\u29EB\u29EC\u29ED\u29EE\u29EF\u29F0\u29F1"+
	"\u29F2\u29F3\u29F4\u29F5\u29F6\u29F7\u29F8\u29F9"+
	"\u29FA\u29FB\u29FC\u29FD\u29FE\u29FF\u2A00\u2A01"+
	"\u2A02\u2A03\u2A04\u2A05\u2A06\u2A07\u2A08\u2A09"+
	"\u2A0A\u2A0B\u2A0C\u2A0D\u2A0E\u2A0F\u2A10\u2A11"+
	"\u2A12\u2A13\u2A14\u2A15\u2A16\u2A17\u2A18\u2A19"+
	"\u2A1A\u2A1B\u2A1C\u2A1D\u2A1E\u2A1F\u2A20\u2A21"+
	"\u2A22\u2A23\u2A24\u2A25\u2A26\u2A27\u2A28\u2A29"+
	"\u2A2A\u2A2B\u2A2C\u2A2D\u2A2E\u2A2F\u2A30\u2A31"+
	"\u2A32\u2A33\u2A34\u2A35\u2A36\u2A37\u2A38\u2A39"+
	"\u2A3A\u2A3B\u2A3C\u2A3D\u2A3E\u2A3F\u2A40\u2A41"+
	"\u2A42\u2A43\u2A44\u2A45\u2A46\u2A47\u2A48\u2A49"+
	"\u2A4A\u2A4B\u2A4C\u2A4D\u2A4E\u2A4F\u2A50\u2A51"+
	"\u2A52\u2A53\u2A54\u2A55\u2A56\u2A57\u2A58\u2A59"+
	"\u2A5A\u2A5B\u2A5C\u2A5D\u2A5E\u2A5F\u2A60\u2A61"+
	"\u2A62\u2A63\u2A64\u2A65\u2A66\u2A67\u2A68\u2A69"+
	"\u2A6A\u2A6B\u2A6C\u2A6D\u2A6E\u2A6F\u2A70\u2A71"+
	"\u2A72\u2A73\u2A74\u2A75\u2A76\u2A77\u2A78\u2A79"+
	"\u2A7A\u2A7B\u2A7C\u2A7D\u2A7E\u2A7F\u2A80\u2A81"+
	"\u2A82\u2A83\u2A84\u2A85\u2A86\u2A87\u2A88\u2A89"+
	"\u2A8A\u2A8B\u2A8C\u2A8D\u2A8E\u2A8F\u2A90\u2A91"+
	"\u2A92\u2A93\u2A94\u2A95\u2A96\u2A97\u2A98\u2A99"+
	"\u2A9A\u2A9B\u2A9C\u2A9D\u2A9E\u2A9F\u2AA0\u2AA1"+
	"\u2AA2\u2AA3\u2AA4\u2AA5\u2AA6\u2AA7\u2AA8\u2AA9"+
	"\u2AAA\u2AAB\u2AAC\u2AAD\u2AAE\u2AAF\u2AB0\u2AB1"+
	"\u2AB2\u2AB3\u2AB4\u2AB5\u2AB6\u2AB7\u2AB8\u2AB9"+
	"\u2ABA\u2ABB\u2ABC\u2ABD\u2ABE\u2ABF\u2AC0\u2AC1"+
	"\u2AC2\u2AC3\u2AC4\u2AC5\u2AC6\u2AC7\u2AC8\u2AC9"+
	"\u2ACA\u2ACB\u2ACC\u2ACD\u2ACE\u2ACF\u2AD0\u2AD1"+
	"\u2AD2\u2AD3\u2AD4\u2AD5\u2AD6\u2AD7\u2AD8\u2AD9"+
	"\u2ADA\u2ADB\u2ADC\u2ADD\u2ADE\u2ADF\u2AE0\u2AE1"+
	"\u2AE2\u2AE3\u2AE4\u2AE5\u2AE6\u2AE7\u2AE8\u2AE9"+
	"\u2AEA\u2AEB\u2AEC\u2AED\u2AEE\u2AEF\u2AF0\u2AF1"+
	"\u2AF2\u2AF3\u2AF4\u2AF5\u2AF6\u2AF7\u2AF8\u2AF9"+
	"\u2AFA\u2AFB\u2AFC\u2AFD\u2AFE\u2AFF\u2B00\u2B01"+
	"\u2B02\u2B03\u2B04\u2B05\u2B06\u2B07\u2B08\u2B09"+
	"\u2B0A\u2B0B\u2B0C\u2B0D\u2B0E\u2B0F\u2B10\u2B11"+
	"\u2B12\u2B13\u2B14\u2B15\u2B16\u2B17\u2B18\u2B19"+
	"\u2B1A\u2B1B\u2B1C\u2B1D\u2B1E\u2B1F\u2B20\u2B21"+
	"\u2B22\u2B23\u2B24\u2B25\u2B26\u2B27\u2B28\u2B29"+
	"\u2B2A\u2B2B\u2B2C\u2B2D\u2B2E\u2B2F\u2B30\u2B31"+
	"\u2B32\u2B33\u2B34\u2B35\u2B36\u2B37\u2B38\u2B39"+
	"\u2B3A\u2B3B\u2B3C\u2B3D\u2B3E\u2B3F\u2B40\u2B41"+
	"\u2B42\u2B43\u2B44\u2B45\u2B46\u2B47\u2B48\u2B49"+
	"\u2B4A\u2B4B\u2B4C\u2B4D\u2B4E\u2B4F\u2B50\u2B51"+
	"\u2B52\u2B53\u2B54\u2B55\u2B56\u2B57\u2B58\u2B59"+
	"\u2B5A\u2B5B\u2B5C\u2B5D\u2B5E\u2B5F\u2B60\u2B61"+
	"\u2B62\u2B63\u2B64\u2B65\u2B66\u2B67\u2B68\u2B69"+
	"\u2B6A\u2B6B\u2B6C\u2B6D\u2B6E\u2B6F\u2B70\u2B71"+
	"\u2B72\u2B73\u2B74\u2B75\u2B76\u2B77\u2B78\u2B79"+
	"\u2B7A\u2B7B\u2B7C\u2B7D\u2B7E\u2B7F\u2B80\u2B81"+
	"\u2B82\u2B83\u2B84\u2B85\u2B86\u2B87\u2B88\u2B89"+
	"\u2B8A\u2B8B\u2B8C\u2B8D\u2B8E\u2B8F\u2B90\u2B91"+
	"\u2B92\u2B93\u2B94\u2B95\u2B96\u2B97\u2B98\u2B99"+
	"\u2B9A\u2B9B\u2B9C\u2B9D\u2B9E\u2B9F\u2BA0\u2BA1"+
	"\u2BA2\u2BA3\u2BA4\u2BA5\u2BA6\u2BA7\u2BA8\u2BA9"+
	"\u2BAA\u2BAB\u2BAC\u2BAD\u2BAE\u2BAF\u2BB0\u2BB1"+
	"\u2BB2\u2BB3\u2BB4\u2BB5\u2BB6\u2BB7\u2BB8\u2BB9"+
	"\u2BBA\u2BBB\u2BBC\u2BBD\u2BBE\u2BBF\u2BC0\u2BC1"+
	"\u2BC2\u2BC3\u2BC4\u2BC5\u2BC6\u2BC7\u2BC8\u2BC9"+
	"\u2BCA\u2BCB\u2BCC\u2BCD\u2BCE\u2BCF\u2BD0\u2BD1"+
	"\u2BD2\u2BD3\u2BD4\u2BD5\u2BD6\u2BD7\u2BD8\u2BD9"+
	"\u2BDA\u2BDB\u2BDC\u2BDD\u2BDE\u2BDF\u2BE0\u2BE1"+
	"\u2BE2\u2BE3\u2BE4\u2BE5\u2BE6\u2BE7\u2BE8\u2BE9"+
	"\u2BEA\u2BEB\u2BEC\u2BED\u2BEE\u2BEF\u2BF0\u2BF1"+
	"\u2BF2\u2BF3\u2BF4\u2BF5\u2BF6\u2BF7\u2BF8\u2BF9"+
	"\u2BFA\u2BFB\u2BFC\u2BFD\u2BFE\u2BFF\u2C00\u2C01"+
	"\u2C02\u2C03\u2C04\u2C05\u2C06\u2C07\u2C08\u2C09"+
	"\u2C0A\u2C0B\u2C0C\u2C0D\u2C0E\u2C0F\u2C10\u2C11"+
	"\u2C12\u2C13\u2C14\u2C15\u2C16\u2C17\u2C18\u2C19"+
	"\u2C1A\u2C1B\u2C1C\u2C1D\u2C1E\u2C1F\u2C20\u2C21"+
	"\u2C22\u2C23\u2C24\u2C25\u2C26\u2C27\u2C28\u2C29"+
	"\u2C2A\u2C2B\u2C2C\u2C2D\u2C2E\u2C2F\u2C30\u2C31"+
	"\u2C32\u2C33\u2C34\u2C35\u2C36\u2C37\u2C38\u2C39"+
	"\u2C3A\u2C3B\u2C3C\u2C3D\u2C3E\u2C3F\u2C40\u2C41"+
	"\u2C42\u2C43\u2C44\u2C45\u2C46\u2C47\u2C48\u2C49"+
	"\u2C4A\u2C4B\u2C4C\u2C4D\u2C4E\u2C4F\u2C50\u2C51"+
	"\u2C52\u2C53\u2C54\u2C55\u2C56\u2C57\u2C58\u2C59"+
	"\u2C5A\u2C5B\u2C5C\u2C5D\u2C5E\u2C5F\u2C60\u2C61"+
	"\u2C62\u2C63\u2C64\u2C65\u2C66\u2C67\u2C68\u2C69"+
	"\u2C6A\u2C6B\u2C6C\u2C6D\u2C6E\u2C6F\u2C70\u2C71"+
	"\u2C72\u2C73\u2C74\u2C75\u2C76\u2C77\u2C78\u2C79"+
	"\u2C7A\u2C7B\u2C7C\u2C7D\u2C7E\u2C7F\u2C80\u2C81"+
	"\u2C82\u2C83\u2C84\u2C85\u2C86\u2C87\u2C88\u2C89"+
	"\u2C8A\u2C8B\u2C8C\u2C8D\u2C8E\u2C8F\u2C90\u2C91"+
	"\u2C92\u2C93\u2C94\u2C95\u2C96\u2C97\u2C98\u2C99"+
	"\u2C9A\u2C9B\u2C9C\u2C9D\u2C9E\u2C9F\u2CA0\u2CA1"+
	"\u2CA2\u2CA3\u2CA4\u2CA5\u2CA6\u2CA7\u2CA8\u2CA9"+
	"\u2CAA\u2CAB\u2CAC\u2CAD\u2CAE\u2CAF\u2CB0\u2CB1"+
	"\u2CB2\u2CB3\u2CB4\u2CB5\u2CB6\u2CB7\u2CB8\u2CB9"+
	"\u2CBA\u2CBB\u2CBC\u2CBD\u2CBE\u2CBF\u2CC0\u2CC1"+
	"\u2CC2\u2CC3\u2CC4\u2CC5\u2CC6\u2CC7\u2CC8\u2CC9"+
	"\u2CCA\u2CCB\u2CCC\u2CCD\u2CCE\u2CCF\u2CD0\u2CD1"+
	"\u2CD2\u2CD3\u2CD4\u2CD5\u2CD6\u2CD7\u2CD8\u2CD9"+
	"\u2CDA\u2CDB\u2CDC\u2CDD\u2CDE\u2CDF\u2CE0\u2CE1"+
	"\u2CE2\u2CE3\u2CE4\u2CE5\u2CE6\u2CE7\u2CE8\u2CE9"+
	"\u2CEA\u2CEB\u2CEC\u2CED\u2CEE\u2CEF\u2CF0\u2CF1"+
	"\u2CF2\u2CF3\u2CF4\u2CF5\u2CF6\u2CF7\u2CF8\u2CF9"+
	"\u2CFA\u2CFB\u2CFC\u2CFD\u2CFE\u2CFF\u2D00\u2D01"+
	"\u2D02\u2D03\u2D04\u2D05\u2D06\u2D07\u2D08\u2D09"+
	"\u2D0A\u2D0B\u2D0C\u2D0D\u2D0E\u2D0F\u2D10\u2D11"+
	"\u2D12\u2D13\u2D14\u2D15\u2D16\u2D17\u2D18\u2D19"+
	"\u2D1A\u2D1B\u2D1C\u2D1D\u2D1E\u2D1F\u2D20\u2D21"+
	"\u2D22\u2D23\u2D24\u2D25\u2D26\u2D27\u2D28\u2D29"+
	"\u2D2A\u2D2B\u2D2C\u2D2D\u2D2E\u2D2F\u2D30\u2D31"+
	"\u2D32\u2D33\u2D34\u2D35\u2D36\u2D37\u2D38\u2D39"+
	"\u2D3A\u2D3B\u2D3C\u2D3D\u2D3E\u2D3F\u2D40\u2D41"+
	"\u2D42\u2D43\u2D44\u2D45\u2D46\u2D47\u2D48\u2D49"+
	"\u2D4A\u2D4B\u2D4C\u2D4D\u2D4E\u2D4F\u2D50\u2D51"+
	"\u2D52\u2D53\u2D54\u2D55\u2D56\u2D57\u2D58\u2D59"+
	"\u2D5A\u2D5B\u2D5C\u2D5D\u2D5E\u2D5F\u2D60\u2D61"+
	"\u2D62\u2D63\u2D64\u2D65\u2D66\u2D67\u2D68\u2D69"+
	"\u2D6A\u2D6B\u2D6C\u2D6D\u2D6E\u2D6F\u2D70\u2D71"+
	"\u2D72\u2D73\u2D74\u2D75\u2D76\u2D77\u2D78\u2D79"+
	"\u2D7A\u2D7B\u2D7C\u2D7D\u2D7E\u2D7F\u2D80\u2D81"+
	"\u2D82\u2D83\u2D84\u2D85\u2D86\u2D87\u2D88\u2D89"+
	"\u2D8A\u2D8B\u2D8C\u2D8D\u2D8E\u2D8F\u2D90\u2D91"+
	"\u2D92\u2D93\u2D94\u2D95\u2D96\u2D97\u2D98\u2D99"+
	"\u2D9A\u2D9B\u2D9C\u2D9D\u2D9E\u2D9F\u2DA0\u2DA1"+
	"\u2DA2\u2DA3\u2DA4\u2DA5\u2DA6\u2DA7\u2DA8\u2DA9"+
	"\u2DAA\u2DAB\u2DAC\u2DAD\u2DAE\u2DAF\u2DB0\u2DB1"+
	"\u2DB2\u2DB3\u2DB4\u2DB5\u2DB6\u2DB7\u2DB8\u2DB9"+
	"\u2DBA\u2DBB\u2DBC\u2DBD\u2DBE\u2DBF\u2DC0\u2DC1"+
	"\u2DC2\u2DC3\u2DC4\u2DC5\u2DC6\u2DC7\u2DC8\u2DC9"+
	"\u2DCA\u2DCB\u2DCC\u2DCD\u2DCE\u2DCF\u2DD0\u2DD1"+
	"\u2DD2\u2DD3\u2DD4\u2DD5\u2DD6\u2DD7\u2DD8\u2DD9"+
	"\u2DDA\u2DDB\u2DDC\u2DDD\u2DDE\u2DDF\u2DE0\u2DE1"+
	"\u2DE2\u2DE3\u2DE4\u2DE5\u2DE6\u2DE7\u2DE8\u2DE9"+
	"\u2DEA\u2DEB\u2DEC\u2DED\u2DEE\u2DEF\u2DF0\u2DF1"+
	"\u2DF2\u2DF3\u2DF4\u2DF5\u2DF6\u2DF7\u2DF8\u2DF9"+
	"\u2DFA\u2DFB\u2DFC\u2DFD\u2DFE\u2DFF\u2E00\u2E01"+
	"\u2E02\u2E03\u2E04\u2E05\u2E06\u2E07\u2E08\u2E09"+
	"\u2E0A\u2E0B\u2E0C\u2E0D\u2E0E\u2E0F\u2E10\u2E11"+
	"\u2E12\u2E13\u2E14\u2E15\u2E16\u2E17\u2E18\u2E19"+
	"\u2E1A\u2E1B\u2E1C\u2E1D\u2E1E\u2E1F\u2E20\u2E21"+
	"\u2E22\u2E23\u2E24\u2E25\u2E26\u2E27\u2E28\u2E29"+
	"\u2E2A\u2E2B\u2E2C\u2E2D\u2E2E\u2E2F\u2E30\u2E31"+
	"\u2E32\u2E33\u2E34\u2E35\u2E36\u2E37\u2E38\u2E39"+
	"\u2E3A\u2E3B\u2E3C\u2E3D\u2E3E\u2E3F\u2E40\u2E41"+
	"\u2E42\u2E43\u2E44\u2E45\u2E46\u2E47\u2E48\u2E49"+
	"\u2E4A\u2E4B\u2E4C\u2E4D\u2E4E\u2E4F\u2E50\u2E51"+
	"\u2E52\u2E53\u2E54\u2E55\u2E56\u2E57\u2E58\u2E59"+
	"\u2E5A\u2E5B\u2E5C\u2E5D\u2E5E\u2E5F\u2E60\u2E61"+
	"\u2E62\u2E63\u2E64\u2E65\u2E66\u2E67\u2E68\u2E69"+
	"\u2E6A\u2E6B\u2E6C\u2E6D\u2E6E\u2E6F\u2E70\u2E71"+
	"\u2E72\u2E73\u2E74\u2E75\u2E76\u2E77\u2E78\u2E79"+
	"\u2E7A\u2E7B\u2E7C\u2E7D\u2E7E\u2E7F\u2E80\u2E81"+
	"\u2E82\u2E83\u2E84\u2E85\u2E86\u2E87\u2E88\u2E89"+
	"\u2E8A\u2E8B\u2E8C\u2E8D\u2E8E\u2E8F\u2E90\u2E91"+
	"\u2E92\u2E93\u2E94\u2E95\u2E96\u2E97\u2E98\u2E99"+
	"\u2E9A\u2E9B\u2E9C\u2E9D\u2E9E\u2E9F\u2EA0\u2EA1"+
	"\u2EA2\u2EA3\u2EA4\u2EA5\u2EA6\u2EA7\u2EA8\u2EA9"+
	"\u2EAA\u2EAB\u2EAC\u2EAD\u2EAE\u2EAF\u2EB0\u2EB1"+
	"\u2EB2\u2EB3\u2EB4\u2EB5\u2EB6\u2EB7\u2EB8\u2EB9"+
	"\u2EBA\u2EBB\u2EBC\u2EBD\u2EBE\u2EBF\u2EC0\u2EC1"+
	"\u2EC2\u2EC3\u2EC4\u2EC5\u2EC6\u2EC7\u2EC8\u2EC9"+
	"\u2ECA\u2ECB\u2ECC\u2ECD\u2ECE\u2ECF\u2ED0\u2ED1"+
	"\u2ED2\u2ED3\u2ED4\u2ED5\u2ED6\u2ED7\u2ED8\u2ED9"+
	"\u2EDA\u2EDB\u2EDC\u2EDD\u2EDE\u2EDF\u2EE0\u2EE1";

    private final static String innerIndex1= 
	"\u2EE2\u2EE3\u2EE4\u2EE5\u2EE6\u2EE7\u2EE8\u2EE9"+
	"\u2EEA\u2EEB\u2EEC\u2EED\u2EEE\u2EEF\u2EF0\u2EF1"+
	"\u2EF2\u2EF3\u2EF4\u2EF5\u2EF6\u2EF7\u2EF8\u2EF9"+
	"\u2EFA\u2EFB\u2EFC\u2EFD\u2EFE\u2EFF\u2F00\u2F01"+
	"\u2F02\u2F03\u2F04\u2F05\u2F06\u2F07\u2F08\u2F09"+
	"\u2F0A\u2F0B\u2F0C\u2F0D\u2F0E\u2F0F\u2F10\u2F11"+
	"\u2F12\u2F13\u2F14\u2F15\u2F16\u2F17\u2F18\u2F19"+
	"\u2F1A\u2F1B\u2F1C\u2F1D\u2F1E\u2F1F\u2F20\u2F21"+
	"\u2F22\u2F23\u2F24\u2F25\u2F26\u2F27\u2F28\u2F29"+
	"\u2F2A\u2F2B\u2F2C\u2F2D\u2F2E\u2F2F\u2F30\u2F31"+
	"\u2F32\u2F33\u2F34\u2F35\u2F36\u2F37\u2F38\u2F39"+
	"\u2F3A\u2F3B\u2F3C\u2F3D\u2F3E\u2F3F\u2F40\u2F41"+
	"\u2F42\u2F43\u2F44\u2F45\u2F46\u2F47\u2F48\u2F49"+
	"\u2F4A\u2F4B\u2F4C\u2F4D\u2F4E\u2F4F\u2F50\u2F51"+
	"\u2F52\u2F53\u2F54\u2F55\u2F56\u2F57\u2F58\u2F59"+
	"\u2F5A\u2F5B\u2F5C\u2F5D\u2F5E\u2F5F\u2F60\u2F61"+
	"\u2F62\u2F63\u2F64\u2F65\u2F66\u2F67\u2F68\u2F69"+
	"\u2F6A\u2F6B\u2F6C\u2F6D\u2F6E\u2F6F\u2F70\u2F71"+
	"\u2F72\u2F73\u2F74\u2F75\u2F76\u2F77\u2F78\u2F79"+
	"\u2F7A\u2F7B\u2F7C\u2F7D\u2F7E\u2F7F\u2F80\u2F81"+
	"\u2F82\u2F83\u2F84\u2F85\u2F86\u2F87\u2F88\u2F89"+
	"\u2F8A\u2F8B\u2F8C\u2F8D\u2F8E\u2F8F\u2F90\u2F91"+
	"\u2F92\u2F93\u2F94\u2F95\u2F96\u2F97\u2F98\u2F99"+
	"\u2F9A\u2F9B\u2F9C\u2F9D\u2F9E\u2F9F\u2FA0\u2FA1"+
	"\u2FA2\u2FA3\u2FA4\u2FA5\u2FA6\u2FA7\u2FA8\u2FA9"+
	"\u2FAA\u2FAB\u2FAC\u2FAD\u2FAE\u2FAF\u2FB0\u2FB1"+
	"\u2FB2\u2FB3\u2FB4\u2FB5\u2FB6\u2FB7\u2FB8\u2FB9"+
	"\u2FBA\u2FBB\u2FBC\u2FBD\u2FBE\u2FBF\u2FC0\u2FC1"+
	"\u2FC2\u2FC3\u2FC4\u2FC5\u2FC6\u2FC7\u2FC8\u2FC9"+
	"\u2FCA\u2FCB\u2FCC\u2FCD\u2FCE\u2FCF\u2FD0\u2FD1"+
	"\u2FD2\u2FD3\u2FD4\u2FD5\u2FD6\u2FD7\u2FD8\u2FD9"+
	"\u2FDA\u2FDB\u2FDC\u2FDD\u2FDE\u2FDF\u2FE0\u2FE1"+
	"\u2FE2\u2FE3\u2FE4\u2FE5\u2FE6\u2FE7\u2FE8\u2FE9"+
	"\u2FEA\u2FEB\u2FEC\u2FED\u2FEE\u2FEF\u2FF0\u2FF1"+
	"\u2FF2\u2FF3\u2FF4\u2FF5\u2FF6\u2FF7\u2FF8\u2FF9"+
	"\u2FFA\u2FFB\u2FFC\u2FFD\u2FFE\u2FFF\u3000\u3001"+
	"\u3002\u3003\u3004\u3005\u3006\u3007\u3008\u3009"+
	"\u300A\u300B\u300C\u300D\u300E\u300F\u3010\u3011"+
	"\u3012\u3013\u3014\u3015\u3016\u3017\u3018\u3019"+
	"\u301A\u301B\u301C\u301D\u301E\u301F\u3020\u3021"+
	"\u3022\u3023\u3024\u3025\u3026\u3027\u3028\u3029"+
	"\u302A\u302B\u302C\u302D\u302E\u302F\u3030\u3031"+
	"\u3032\u3033\u3034\u3035\u3036\u3037\u3038\u3039"+
	"\u303A\u303B\u303C\u303D\u303E\u303F\u3040\u3041"+
	"\u3042\u3043\u3044\u3045\u3046\u3047\u3048\u3049"+
	"\u304A\u304B\u304C\u304D\u304E\u304F\u3050\u3051"+
	"\u3052\u3053\u3054\u3055\u3056\u3057\u3058\u3059"+
	"\u305A\u305B\u305C\u305D\u305E\u305F\u3060\u3061"+
	"\u3062\u3063\u3064\u3065\u3066\u3067\u3068\u3069"+
	"\u306A\u306B\u306C\u306D\u306E\u306F\u3070\u3071"+
	"\u3072\u3073\u3074\u3075\u3076\u3077\u3078\u3079"+
	"\u307A\u307B\u307C\u307D\u307E\u307F\u3080\u3081"+
	"\u3082\u3083\u3084\u3085\u3086\u3087\u3088\u3089"+
	"\u308A\u308B\u308C\u308D\u308E\u308F\u3090\u3091"+
	"\u3092\u3093\u3094\u3095\u3096\u3097\u3098\u3099"+
	"\u309A\u309B\u309C\u309D\u309E\u309F\u30A0\u30A1"+
	"\u30A2\u30A3\u30A4\u30A5\u30A6\u30A7\u30A8\u30A9"+
	"\u30AA\u30AB\u30AC\u30AD\u30AE\u30AF\u30B0\u30B1"+
	"\u30B2\u30B3\u30B4\u30B5\u30B6\u30B7\u30B8\u30B9"+
	"\u30BA\u30BB\u30BC\u30BD\u30BE\u30BF\u30C0\u30C1"+
	"\u30C2\u30C3\u30C4\u30C5\u30C6\u30C7\u30C8\u30C9"+
	"\u30CA\u30CB\u30CC\u30CD\u30CE\u30CF\u30D0\u30D1"+
	"\u30D2\u30D3\u30D4\u30D5\u30D6\u30D7\u30D8\u30D9"+
	"\u30DA\u30DB\u30DC\u30DD\u30DE\u30DF\u30E0\u30E1"+
	"\u30E2\u30E3\u30E4\u30E5\u30E6\u30E7\u30E8\u30E9"+
	"\u30EA\u30EB\u30EC\u30ED\u30EE\u30EF\u30F0\u30F1"+
	"\u30F2\u30F3\u30F4\u30F5\u30F6\u30F7\u30F8\u30F9"+
	"\u30FA\u30FB\u30FC\u30FD\u30FE\u30FF\u3100\u3101"+
	"\u3102\u3103\u3104\u3105\u3106\u3107\u3108\u3109"+
	"\u310A\u310B\u310C\u310D\u310E\u310F\u3110\u3111"+
	"\u3112\u3113\u3114\u3115\u3116\u3117\u3118\u3119"+
	"\u311A\u311B\u311C\u311D\u311E\u311F\u3120\u3121"+
	"\u3122\u3123\u3124\u3125\u3126\u3127\u3128\u3129"+
	"\u312A\u312B\u312C\u312D\u312E\u312F\u3130\u3131"+
	"\u3132\u3133\u3134\u3135\u3136\u3137\u3138\u3139"+
	"\u313A\u313B\u313C\u313D\u313E\u313F\u3140\u3141"+
	"\u3142\u3143\u3144\u3145\u3146\u3147\u3148\u3149"+
	"\u314A\u314B\u314C\u314D\u314E\u314F\u3150\u3151"+
	"\u3152\u3153\u3154\u3155\u3156\u3157\u3158\u3159"+
	"\u315A\u315B\u315C\u315D\u315E\u315F\u3160\u3161"+
	"\u3162\u3163\u3164\u3165\u3166\u3167\u3168\u3169"+
	"\u316A\u316B\u316C\u316D\u316E\u316F\u3170\u3171"+
	"\u3172\u3173\u3174\u3175\u3176\u3177\u3178\u3179"+
	"\u317A\u317B\u317C\u317D\u317E\u317F\u3180\u3181"+
	"\u3182\u3183\u3184\u3185\u3186\u3187\u3188\u3189"+
	"\u318A\u318B\u318C\u318D\u318E\u318F\u3190\u3191"+
	"\u3192\u3193\u3194\u3195\u3196\u3197\u3198\u3199"+
	"\u319A\u319B\u319C\u319D\u319E\u319F\u31A0\u31A1"+
	"\u31A2\u31A3\u31A4\u31A5\u31A6\u31A7\u31A8\u31A9"+
	"\u31AA\u31AB\u31AC\u31AD\u31AE\u31AF\u31B0\u31B1"+
	"\u31B2\u31B3\u31B4\u31B5\u31B6\u31B7\u31B8\u31B9"+
	"\u31BA\u31BB\u31BC\u31BD\u31BE\u31BF\u31C0\u31C1"+
	"\u31C2\u31C3\u31C4\u31C5\u31C6\u31C7\u31C8\u31C9"+
	"\u31CA\u31CB\u31CC\u31CD\u31CE\u31CF\u31D0\u31D1"+
	"\u31D2\u31D3\u31D4\u31D5\u31D6\u31D7\u31D8\u31D9"+
	"\u31DA\u31DB\u31DC\u31DD\u31DE\u31DF\u31E0\u31E1"+
	"\u31E2\u31E3\u31E4\u31E5\u31E6\u31E7\u31E8\u31E9"+
	"\u31EA\u31EB\u31EC\u31ED\u31EE\u31EF\u31F0\u31F1"+
	"\u31F2\u31F3\u31F4\u31F5\u31F6\u31F7\u31F8\u31F9"+
	"\u31FA\u31FB\u31FC\u31FD\u31FE\u31FF\u3200\u3201"+
	"\u3202\u3203\u3204\u3205\u3206\u3207\u3208\u3209"+
	"\u320A\u320B\u320C\u320D\u320E\u320F\u3210\u3211"+
	"\u3212\u3213\u3214\u3215\u3216\u3217\u3218\u3219"+
	"\u321A\u321B\u321C\u321D\u321E\u321F\u3220\u3221"+
	"\u3222\u3223\u3224\u3225\u3226\u3227\u3228\u3229"+
	"\u322A\u322B\u322C\u322D\u322E\u322F\u3230\u3231"+
	"\u3232\u3233\u3234\u3235\u3236\u3237\u3238\u3239"+
	"\u323A\u323B\u323C\u323D\u323E\u323F\u3240\u3241"+
	"\u3242\u3243\u3244\u3245\u3246\u3247\u3248\u3249"+
	"\u324A\u324B\u324C\u324D\u324E\u324F\u3250\u3251"+
	"\u3252\u3253\u3254\u3255\u3256\u3257\u3258\u3259"+
	"\u325A\u325B\u325C\u325D\u325E\u325F\u3260\u3261"+
	"\u3262\u3263\u3264\u3265\u3266\u3267\u3268\u3269"+
	"\u326A\u326B\u326C\u326D\u326E\u326F\u3270\u3271"+
	"\u3272\u3273\u3274\u3275\u3276\u3277\u3278\u3279"+
	"\u327A\u327B\u327C\u327D\u327E\u327F\u3280\u3281"+
	"\u3282\u3283\u3284\u3285\u3286\u3287\u3288\u3289"+
	"\u328A\u328B\u328C\u328D\u328E\u328F\u3290\u3291"+
	"\u3292\u3293\u3294\u3295\u3296\u3297\u3298\u3299"+
	"\u329A\u329B\u329C\u329D\u329E\u329F\u32A0\u32A1"+
	"\u32A2\u32A3\u32A4\u32A5\u32A6\u32A7\u32A8\u32A9"+
	"\u32AA\u32AB\u32AC\u32AD\u32AE\u32AF\u32B0\u32B1"+
	"\u32B2\u32B3\u32B4\u32B5\u32B6\u32B7\u32B8\u32B9"+
	"\u32BA\u32BB\u32BC\u32BD\u32BE\u32BF\u32C0\u32C1"+
	"\u32C2\u32C3\u32C4\u32C5\u32C6\u32C7\u32C8\u32C9"+
	"\u32CA\u32CB\u32CC\u32CD\u32CE\u32CF\u32D0\u32D1"+
	"\u32D2\u32D3\u32D4\u32D5\u32D6\u32D7\u32D8\u32D9"+
	"\u32DA\u32DB\u32DC\u32DD\u32DE\u32DF\u32E0\u32E1"+
	"\u32E2\u32E3\u32E4\u32E5\u32E6\u32E7\u32E8\u32E9"+
	"\u32EA\u32EB\u32EC\u32ED\u32EE\u32EF\u32F0\u32F1"+
	"\u32F2\u32F3\u32F4\u32F5\u32F6\u32F7\u32F8\u32F9"+
	"\u32FA\u32FB\u32FC\u32FD\u32FE\u32FF\u3300\u3301"+
	"\u3302\u3303\u3304\u3305\u3306\u3307\u3308\u3309"+
	"\u330A\u330B\u330C\u330D\u330E\u330F\u3310\u3311"+
	"\u3312\u3313\u3314\u3315\u3316\u3317\u3318\u3319"+
	"\u331A\u331B\u331C\u331D\u331E\u331F\u3320\u3321"+
	"\u3322\u3323\u3324\u3325\u3326\u3327\u3328\u3329"+
	"\u332A\u332B\u332C\u332D\u332E\u332F\u3330\u3331"+
	"\u3332\u3333\u3334\u3335\u3336\u3337\u3338\u3339"+
	"\u333A\u333B\u333C\u333D\u333E\u333F\u3340\u3341"+
	"\u3342\u3343\u3344\u3345\u3346\u3347\u3348\u3349"+
	"\u334A\u334B\u334C\u334D\u334E\u334F\u3350\u3351"+
	"\u3352\u3353\u3354\u3355\u3356\u3357\u3358\u3359"+
	"\u335A\u335B\u335C\u335D\u335E\u335F\u3360\u3361"+
	"\u3362\u3363\u3364\u3365\u3366\u3367\u3368\u3369"+
	"\u336A\u336B\u336C\u336D\u336E\u336F\u3370\u3371"+
	"\u3372\u3373\u3374\u3375\u3376\u3377\u3378\u3379"+
	"\u337A\u337B\u337C\u337D\u337E\u337F\u3380\u3381"+
	"\u3382\u3383\u3384\u3385\u3386\u3387\u3388\u3389"+
	"\u338A\u338B\u338C\u338D\u338E\u338F\u3390\u3391"+
	"\u3392\u3393\u3394\u3395\u3396\u3397\u3398\u3399"+
	"\u339A\u339B\u339C\u339D\u339E\u339F\u33A0\u33A1"+
	"\u33A2\u33A3\u33A4\u33A5\u33A6\u33A7\u33A8\u33A9"+
	"\u33AA\u33AB\u33AC\u33AD\u33AE\u33AF\u33B0\u33B1"+
	"\u33B2\u33B3\u33B4\u33B5\u33B6\u33B7\u33B8\u33B9"+
	"\u33BA\u33BB\u33BC\u33BD\u33BE\u33BF\u33C0\u33C1"+
	"\u33C2\u33C3\u33C4\u33C5\u33C6\u33C7\u33C8\u33C9"+
	"\u33CA\u33CB\u33CC\u33CD\u33CE\u33CF\u33D0\u33D1"+
	"\u33D2\u33D3\u33D4\u33D5\u33D6\u33D7\u33D8\u33D9"+
	"\u33DA\u33DB\u33DC\u33DD\u33DE\u33DF\u33E0\u33E1"+
	"\u33E2\u33E3\u33E4\u33E5\u33E6\u33E7\u33E8\u33E9"+
	"\u33EA\u33EB\u33EC\u33ED\u33EE\u33EF\u33F0\u33F1"+
	"\u33F2\u33F3\u33F4\u33F5\u33F6\u33F7\u33F8\u33F9"+
	"\u33FA\u33FB\u33FC\u33FD\u33FE\u33FF\u3400\u3401"+
	"\u3402\u3403\u3404\u3405\u3406\u3407\u3408\u3409"+
	"\u340A\u340B\u340C\u340D\u340E\u340F\u3410\u3411"+
	"\u3412\u3413\u3414\u3415\u3416\u3417\u3418\u3419"+
	"\u341A\u341B\u341C\u341D\u341E\u341F\u3420\u3421"+
	"\u3422\u3423\u3424\u3425\u3426\u3427\u3428\u3429"+
	"\u342A\u342B\u342C\u342D\u342E\u342F\u3430\u3431"+
	"\u3432\u3433\u3434\u3435\u3436\u3437\u3438\u3439"+
	"\u343A\u343B\u343C\u343D\u343E\u343F\u3440\u3441"+
	"\u3442\u3443\u3444\u3445\u3446\u3447\u3448\u3449"+
	"\u344A\u344B\u344C\u344D\u344E\u344F\u3450\u3451"+
	"\u3452\u3453\u3454\u3455\u3456\u3457\u3458\u3459"+
	"\u345A\u345B\u345C\u345D\u345E\u345F\u3460\u3461"+
	"\u3462\u3463\u3464\u3465\u3466\u3467\u3468\u3469"+
	"\u346A\u346B\u346C\u346D\u346E\u346F\u3470\u3471"+
	"\u3472\u3473\u3474\u3475\u3476\u3477\u3478\u3479"+
	"\u347A\u347B\u347C\u347D\u347E\u347F\u3480\u3481"+
	"\u3482\u3483\u3484\u3485\u3486\u3487\u3488\u3489"+
	"\u348A\u348B\u348C\u348D\u348E\u348F\u3490\u3491"+
	"\u3492\u3493\u3494\u3495\u3496\u3497\u3498\u3499"+
	"\u349A\u349B\u349C\u349D\u349E\u349F\u34A0\u34A1"+
	"\u34A2\u34A3\u34A4\u34A5\u34A6\u34A7\u34A8\u34A9"+
	"\u34AA\u34AB\u34AC\u34AD\u34AE\u34AF\u34B0\u34B1"+
	"\u34B2\u34B3\u34B4\u34B5\u34B6\u34B7\u34B8\u34B9"+
	"\u34BA\u34BB\u34BC\u34BD\u34BE\u34BF\u34C0\u34C1"+
	"\u34C2\u34C3\u34C4\u34C5\u34C6\u34C7\u34C8\u34C9"+
	"\u34CA\u34CB\u34CC\u34CD\u34CE\u34CF\u34D0\u34D1"+
	"\u34D2\u34D3\u34D4\u34D5\u34D6\u34D7\u34D8\u34D9"+
	"\u34DA\u34DB\u34DC\u34DD\u34DE\u34DF\u34E0\u34E1"+
	"\u34E2\u34E3\u34E4\u34E5\u34E6\u34E7\u34E8\u34E9"+
	"\u34EA\u34EB\u34EC\u34ED\u34EE\u34EF\u34F0\u34F1"+
	"\u34F2\u34F3\u34F4\u34F5\u34F6\u34F7\u34F8\u34F9"+
	"\u34FA\u34FB\u34FC\u34FD\u34FE\u34FF\u3500\u3501"+
	"\u3502\u3503\u3504\u3505\u3506\u3507\u3508\u3509"+
	"\u350A\u350B\u350C\u350D\u350E\u350F\u3510\u3511"+
	"\u3512\u3513\u3514\u3515\u3516\u3517\u3518\u3519"+
	"\u351A\u351B\u351C\u351D\u351E\u351F\u3520\u3521"+
	"\u3522\u3523\u3524\u3525\u3526\u3527\u3528\u3529"+
	"\u352A\u352B\u352C\u352D\u352E\u352F\u3530\u3531"+
	"\u3532\u3533\u3534\u3535\u3536\u3537\u3538\u3539"+
	"\u353A\u353B\u353C\u353D\u353E\u353F\u3540\u3541"+
	"\u3542\u3543\u3544\u3545\u3546\u3547\u3548\u3549"+
	"\u354A\u354B\u354C\u354D\u354E\u354F\u3550\u3551"+
	"\u3552\u3553\u3554\u3555\u3556\u3557\u3558\u3559"+
	"\u355A\u355B\u355C\u355D\u355E\u355F\u3560\u3561"+
	"\u3562\u3563\u3564\u3565\u3566\u3567\u3568\u3569"+
	"\u356A\u356B\u356C\u356D\u356E\u356F\u3570\u3571"+
	"\u3572\u3573\u3574\u3575\u3576\u3577\u3578\u3579"+
	"\u357A\u357B\u357C\u357D\u357E\u357F\u3580\u3581"+
	"\u3582\u3583\u3584\u3585\u3586\u3587\u3588\u3589"+
	"\u358A\u358B\u358C\u358D\u358E\u358F\u3590\u3591"+
	"\u3592\u3593\u3594\u3595\u3596\u3597\u3598\u3599"+
	"\u359A\u359B\u359C\u359D\u359E\u359F\u35A0\u35A1"+
	"\u35A2\u35A3\u35A4\u35A5\u35A6\u35A7\u35A8\u35A9"+
	"\u35AA\u35AB\u35AC\u35AD\u35AE\u35AF\u35B0\u35B1"+
	"\u35B2\u35B3\u35B4\u35B5\u35B6\u35B7\u35B8\u35B9"+
	"\u35BA\u35BB\u35BC\u35BD\u35BE\u35BF\u35C0\u35C1"+
	"\u35C2\u35C3\u35C4\u35C5\u35C6\u35C7\u35C8\u35C9"+
	"\u35CA\u35CB\u35CC\u35CD\u35CE\u35CF\u35D0\u35D1"+
	"\u35D2\u35D3\u35D4\u35D5\u35D6\u35D7\u35D8\u35D9"+
	"\u35DA\u35DB\u35DC\u35DD\u35DE\u35DF\u35E0\u35E1"+
	"\u35E2\u35E3\u35E4\u35E5\u35E6\u35E7\u35E8\u35E9"+
	"\u35EA\u35EB\u35EC\u35ED\u35EE\u35EF\u35F0\u35F1"+
	"\u35F2\u35F3\u35F4\u35F5\u35F6\u35F7\u35F8\u35F9"+
	"\u35FA\u35FB\u35FC\u35FD\u35FE\u35FF\u3600\u3601"+
	"\u3602\u3603\u3604\u3605\u3606\u3607\u3608\u3609"+
	"\u360A\u360B\u360C\u360D\u360E\u360F\u3610\u3611"+
	"\u3612\u3613\u3614\u3615\u3616\u3617\u3618\u3619"+
	"\u361A\u361B\u361C\u361D\u361E\u361F\u3620\u3621"+
	"\u3622\u3623\u3624\u3625\u3626\u3627\u3628\u3629"+
	"\u362A\u362B\u362C\u362D\u362E\u362F\u3630\u3631"+
	"\u3632\u3633\u3634\u3635\u3636\u3637\u3638\u3639"+
	"\u363A\u363B\u363C\u363D\u363E\u363F\u3640\u3641"+
	"\u3642\u3643\u3644\u3645\u3646\u3647\u3648\u3649"+
	"\u364A\u364B\u364C\u364D\u364E\u364F\u3650\u3651"+
	"\u3652\u3653\u3654\u3655\u3656\u3657\u3658\u3659"+
	"\u365A\u365B\u365C\u365D\u365E\u365F\u3660\u3661"+
	"\u3662\u3663\u3664\u3665\u3666\u3667\u3668\u3669"+
	"\u366A\u366B\u366C\u366D\u366E\u366F\u3670\u3671"+
	"\u3672\u3673\u3674\u3675\u3676\u3677\u3678\u3679"+
	"\u367A\u367B\u367C\u367D\u367E\u367F\u3680\u3681"+
	"\u3682\u3683\u3684\u3685\u3686\u3687\u3688\u3689"+
	"\u368A\u368B\u368C\u368D\u368E\u368F\u3690\u3691"+
	"\u3692\u3693\u3694\u3695\u3696\u3697\u3698\u3699"+
	"\u369A\u369B\u369C\u369D\u369E\u369F\u36A0\u36A1"+
	"\u36A2\u36A3\u36A4\u36A5\u36A6\u36A7\u36A8\u36A9"+
	"\u36AA\u36AB\u36AC\u36AD\u36AE\u36AF\u36B0\u36B1"+
	"\u36B2\u36B3\u36B4\u36B5\u36B6\u36B7\u36B8\u36B9"+
	"\u36BA\u36BB\u36BC\u36BD\u36BE\u36BF\u36C0\u36C1"+
	"\u36C2\u36C3\u36C4\u36C5\u36C6\u36C7\u36C8\u36C9"+
	"\u36CA\u36CB\u36CC\u36CD\u36CE\u36CF\u36D0\u36D1"+
	"\u36D2\u36D3\u36D4\u36D5\u36D6\u36D7\u36D8\u36D9"+
	"\u36DA\u36DB\u36DC\u36DD\u36DE\u36DF\u36E0\u36E1"+
	"\u36E2\u36E3\u36E4\u36E5\u36E6\u36E7\u36E8\u36E9"+
	"\u36EA\u36EB\u36EC\u36ED\u36EE\u36EF\u36F0\u36F1"+
	"\u36F2\u36F3\u36F4\u36F5\u36F6\u36F7\u36F8\u36F9"+
	"\u36FA\u36FB\u36FC\u36FD\u36FE\u36FF\u3700\u3701"+
	"\u3702\u3703\u3704\u3705\u3706\u3707\u3708\u3709"+
	"\u370A\u370B\u370C\u370D\u370E\u370F\u3710\u3711"+
	"\u3712\u3713\u3714\u3715\u3716\u3717\u3718\u3719"+
	"\u371A\u371B\u371C\u371D\u371E\u371F\u3720\u3721"+
	"\u3722\u3723\u3724\u3725\u3726\u3727\u3728\u3729"+
	"\u372A\u372B\u372C\u372D\u372E\u372F\u3730\u3731"+
	"\u3732\u3733\u3734\u3735\u3736\u3737\u3738\u3739"+
	"\u373A\u373B\u373C\u373D\u373E\u373F\u3740\u3741"+
	"\u3742\u3743\u3744\u3745\u3746\u3747\u3748\u3749"+
	"\u374A\u374B\u374C\u374D\u374E\u374F\u3750\u3751"+
	"\u3752\u3753\u3754\u3755\u3756\u3757\u3758\u3759"+
	"\u375A\u375B\u375C\u375D\u375E\u375F\u3760\u3761"+
	"\u3762\u3763\u3764\u3765\u3766\u3767\u3768\u3769"+
	"\u376A\u376B\u376C\u376D\u376E\u376F\u3770\u3771"+
	"\u3772\u3773\u3774\u3775\u3776\u3777\u3778\u3779"+
	"\u377A\u377B\u377C\u377D\u377E\u377F\u3780\u3781"+
	"\u3782\u3783\u3784\u3785\u3786\u3787\u3788\u3789"+
	"\u378A\u378B\u378C\u378D\u378E\u378F\u3790\u3791"+
	"\u3792\u3793\u3794\u3795\u3796\u3797\u3798\u3799"+
	"\u379A\u379B\u379C\u379D\u379E\u379F\u37A0\u37A1"+
	"\u37A2\u37A3\u37A4\u37A5\u37A6\u37A7\u37A8\u37A9"+
	"\u37AA\u37AB\u37AC\u37AD\u37AE\u37AF\u37B0\u37B1"+
	"\u37B2\u37B3\u37B4\u37B5\u37B6\u37B7\u37B8\u37B9"+
	"\u37BA\u37BB\u37BC\u37BD\u37BE\u37BF\u37C0\u37C1"+
	"\u37C2\u37C3\u37C4\u37C5\u37C6\u37C7\u37C8\u37C9"+
	"\u37CA\u37CB\u37CC\u37CD\u37CE\u37CF\u37D0\u37D1"+
	"\u37D2\u37D3\u37D4\u37D5\u37D6\u37D7\u37D8\u37D9"+
	"\u37DA\u37DB\u37DC\u37DD\u37DE\u37DF\u37E0\u37E1"+
	"\u37E2\u37E3\u37E4\u37E5\u37E6\u37E7\u37E8\u37E9"+
	"\u37EA\u37EB\u37EC\u37ED\u37EE\u37EF\u37F0\u37F1"+
	"\u37F2\u37F3\u37F4\u37F5\u37F6\u37F7\u37F8\u37F9"+
	"\u37FA\u37FB\u37FC\u37FD\u37FE\u37FF\u3800\u3801"+
	"\u3802\u3803\u3804\u3805\u3806\u3807\u3808\u3809"+
	"\u380A\u380B\u380C\u380D\u380E\u380F\u3810\u3811"+
	"\u3812\u3813\u3814\u3815\u3816\u3817\u3818\u3819"+
	"\u381A\u381B\u381C\u381D\u381E\u381F\u3820\u3821"+
	"\u3822\u3823\u3824\u3825\u3826\u3827\u3828\u3829"+
	"\u382A\u382B\u382C\u382D\u382E\u382F\u3830\u3831"+
	"\u3832\u3833\u3834\u3835\u3836\u3837\u3838\u3839"+
	"\u383A\u383B\u383C\u383D\u383E\u383F\u3840\u3841"+
	"\u3842\u3843\u3844\u3845\u3846\u3847\u3848\u3849"+
	"\u384A\u384B\u384C\u384D\u384E\u384F\u3850\u3851"+
	"\u3852\u3853\u3854\u3855\u3856\u3857\u3858\u3859"+
	"\u385A\u385B\u385C\u385D\u385E\u385F\u3860\u3861"+
	"\u3862\u3863\u3864\u3865\u3866\u3867\u3868\u3869"+
	"\u386A\u386B\u386C\u386D\u386E\u386F\u3870\u3871"+
	"\u3872\u3873\u3874\u3875\u3876\u3877\u3878\u3879"+
	"\u387A\u387B\u387C\u387D\u387E\u387F\u3880\u3881"+
	"\u3882\u3883\u3884\u3885\u3886\u3887\u3888\u3889"+
	"\u388A\u388B\u388C\u388D\u388E\u388F\u3890\u3891"+
	"\u3892\u3893\u3894\u3895\u3896\u3897\u3898\u3899"+
	"\u389A\u389B\u389C\u389D\u389E\u389F\u38A0\u38A1"+
	"\u38A2\u38A3\u38A4\u38A5\u38A6\u38A7\u38A8\u38A9"+
	"\u38AA\u38AB\u38AC\u38AD\u38AE\u38AF\u38B0\u38B1"+
	"\u38B2\u38B3\u38B4\u38B5\u38B6\u38B7\u38B8\u38B9"+
	"\u38BA\u38BB\u38BC\u38BD\u38BE\u38BF\u38C0\u38C1"+
	"\u38C2\u38C3\u38C4\u38C5\u38C6\u38C7\u38C8\u38C9"+
	"\u38CA\u38CB\u38CC\u38CD\u38CE\u38CF\u38D0\u38D1"+
	"\u38D2\u38D3\u38D4\u38D5\u38D6\u38D7\u38D8\u38D9"+
	"\u38DA\u38DB\u38DC\u38DD\u38DE\u38DF\u38E0\u38E1"+
	"\u38E2\u38E3\u38E4\u38E5\u38E6\u38E7\u38E8\u38E9"+
	"\u38EA\u38EB\u38EC\u38ED\u38EE\u38EF\u38F0\u38F1"+
	"\u38F2\u38F3\u38F4\u38F5\u38F6\u38F7\u38F8\u38F9"+
	"\u38FA\u38FB\u38FC\u38FD\u38FE\u38FF\u3900\u3901"+
	"\u3902\u3903\u3904\u3905\u3906\u3907\u3908\u3909"+
	"\u390A\u390B\u390C\u390D\u390E\u390F\u3910\u3911"+
	"\u3912\u3913\u3914\u3915\u3916\u3917\u3918\u3919"+
	"\u391A\u391B\u391C\u391D\u391E\u391F\u3920\u3921"+
	"\u3922\u3923\u3924\u3925\u3926\u3927\u3928\u3929"+
	"\u392A\u392B\u392C\u392D\u392E\u392F\u3930\u3931"+
	"\u3932\u3933\u3934\u3935\u3936\u3937\u3938\u3939"+
	"\u393A\u393B\u393C\u393D\u393E\u393F\u3940\u3941"+
	"\u3942\u3943\u3944\u3945\u3946\u3947\u3948\u3949"+
	"\u394A\u394B\u394C\u394D\u394E\u394F\u3950\u3951"+
	"\u3952\u3953\u3954\u3955\u3956\u3957\u3958\u3959"+
	"\u395A\u395B\u395C\u395D\u395E\u395F\u3960\u3961"+
	"\u3962\u3963\u3964\u3965\u3966\u3967\u3968\u3969"+
	"\u396A\u396B\u396C\u396D\u396E\u396F\u3970\u3971"+
	"\u3972\u3973\u3974\u3975\u3976\u3977\u3978\u3979"+
	"\u397A\u397B\u397C\u397D\u397E\u397F\u3980\u3981"+
	"\u3982\u3983\u3984\u3985\u3986\u3987\u3988\u3989"+
	"\u398A\u398B\u398C\u398D\u398E\u398F\u3990\u3991"+
	"\u3992\u3993\u3994\u3995\u3996\u3997\u3998\u3999"+
	"\u399A\u399B\u399C\u399D\u399E\u399F\u39A0\u39A1"+
	"\u39A2\u39A3\u39A4\u39A5\u39A6\u39A7\u39A8\u39A9"+
	"\u39AA\u39AB\u39AC\u39AD\u39AE\u39AF\u39B0\u39B1"+
	"\u39B2\u39B3\u39B4\u39B5\u39B6\u39B7\u39B8\u39B9"+
	"\u39BA\u39BB\u39BC\u39BD\u39BE\u39BF\u39C0\u39C1"+
	"\u39C2\u39C3\u39C4\u39C5\u39C6\u39C7\u39C8\u39C9"+
	"\u39CA\u39CB\u39CC\u39CD\u39CE\u39CF\u39D0\u39D1"+
	"\u39D2\u39D3\u39D4\u39D5\u39D6\u39D7\u39D8\u39D9"+
	"\u39DA\u39DB\u39DC\u39DD\u39DE\u39DF\u39E0\u39E1"+
	"\u39E2\u39E3\u39E4\u39E5\u39E6\u39E7\u39E8\u39E9"+
	"\u39EA\u39EB\u39EC\u39ED\u39EE\u39EF\u39F0\u39F1"+
	"\u39F2\u39F3\u39F4\u39F5\u39F6\u39F7\u39F8\u39F9"+
	"\u39FA\u39FB\u39FC\u39FD\u39FE\u39FF\u3A00\u3A01"+
	"\u3A02\u3A03\u3A04\u3A05\u3A06\u3A07\u3A08\u3A09"+
	"\u3A0A\u3A0B\u3A0C\u3A0D\u3A0E\u3A0F\u3A10\u3A11"+
	"\u3A12\u3A13\u3A14\u3A15\u3A16\u3A17\u3A18\u3A19"+
	"\u3A1A\u3A1B\u3A1C\u3A1D\u3A1E\u3A1F\u3A20\u3A21"+
	"\u3A22\u3A23\u3A24\u3A25\u3A26\u3A27\u3A28\u3A29"+
	"\u3A2A\u3A2B\u3A2C\u3A2D\u3A2E\u3A2F\u3A30\u3A31"+
	"\u3A32\u3A33\u3A34\u3A35\u3A36\u3A37\u3A38\u3A39"+
	"\u3A3A\u3A3B\u3A3C\u3A3D\u3A3E\u3A3F\u3A40\u3A41"+
	"\u3A42\u3A43\u3A44\u3A45\u3A46\u3A47\u3A48\u3A49"+
	"\u3A4A\u3A4B\u3A4C\u3A4D\u3A4E\u3A4F\u3A50\u3A51"+
	"\u3A52\u3A53\u3A54\u3A55\u3A56\u3A57\u3A58\u3A59"+
	"\u3A5A\u3A5B\u3A5C\u3A5D\u3A5E\u3A5F\u3A60\u3A61"+
	"\u3A62\u3A63\u3A64\u3A65\u3A66\u3A67\u3A68\u3A69"+
	"\u3A6A\u3A6B\u3A6C\u3A6D\u3A6E\u3A6F\u3A70\u3A71"+
	"\u3A72\u3A73\u3A74\u3A75\u3A76\u3A77\u3A78\u3A79"+
	"\u3A7A\u3A7B\u3A7C\u3A7D\u3A7E\u3A7F\u3A80\u3A81"+
	"\u3A82\u3A83\u3A84\u3A85\u3A86\u3A87\u3A88\u3A89"+
	"\u3A8A\u3A8B\u3A8C\u3A8D\u3A8E\u3A8F\u3A90\u3A91"+
	"\u3A92\u3A93\u3A94\u3A95\u3A96\u3A97\u3A98\u3A99"+
	"\u3A9A\u3A9B\u3A9C\u3A9D\u3A9E\u3A9F\u3AA0\u3AA1"+
	"\u3AA2\u3AA3\u3AA4\u3AA5\u3AA6\u3AA7\u3AA8\u3AA9"+
	"\u3AAA\u3AAB\u3AAC\u3AAD\u3AAE\u3AAF\u3AB0\u3AB1"+
	"\u3AB2\u3AB3\u3AB4\u3AB5\u3AB6\u3AB7\u3AB8\u3AB9"+
	"\u3ABA\u3ABB\u3ABC\u3ABD\u3ABE\u3ABF\u3AC0\u3AC1"+
	"\u3AC2\u3AC3\u3AC4\u3AC5\u3AC6\u3AC7\u3AC8\u3AC9"+
	"\u3ACA\u3ACB\u3ACC\u3ACD\u3ACE\u3ACF\u3AD0\u3AD1"+
	"\u3AD2\u3AD3\u3AD4\u3AD5\u3AD6\u3AD7\u3AD8\u3AD9"+
	"\u3ADA\u3ADB\u3ADC\u3ADD\u3ADE\u3ADF\u3AE0\u3AE1"+
	"\u3AE2\u3AE3\u3AE4\u3AE5\u3AE6\u3AE7\u3AE8\u3AE9"+
	"\u3AEA\u3AEB\u3AEC\u3AED\u3AEE\u3AEF\u3AF0\u3AF1"+
	"\u3AF2\u3AF3\u3AF4\u3AF5\u3AF6\u3AF7\u3AF8\u3AF9"+
	"\u3AFA\u3AFB\u3AFC\u3AFD\u3AFE\u3AFF\u3B00\u3B01"+
	"\u3B02\u3B03\u3B04\u3B05\u3B06\u3B07\u3B08\u3B09"+
	"\u3B0A\u3B0B\u3B0C\u3B0D\u3B0E\u3B0F\u3B10\u3B11"+
	"\u3B12\u3B13\u3B14\u3B15\u3B16\u3B17\u3B18\u3B19"+
	"\u3B1A\u3B1B\u3B1C\u3B1D\u3B1E\u3B1F\u3B20\u3B21"+
	"\u3B22\u3B23\u3B24\u3B25\u3B26\u3B27\u3B28\u3B29"+
	"\u3B2A\u3B2B\u3B2C\u3B2D\u3B2E\u3B2F\u3B30\u3B31"+
	"\u3B32\u3B33\u3B34\u3B35\u3B36\u3B37\u3B38\u3B39"+
	"\u3B3A\u3B3B\u3B3C\u3B3D\u3B3E\u3B3F\u3B40\u3B41"+
	"\u3B42\u3B43\u3B44\u3B45\u3B46\u3B47\u3B48\u3B49"+
	"\u3B4A\u3B4B\u3B4C\u3B4D\u3B4E\u3B4F\u3B50\u3B51"+
	"\u3B52\u3B53\u3B54\u3B55\u3B56\u3B57\u3B58\u3B59"+
	"\u3B5A\u3B5B\u3B5C\u3B5D\u3B5E\u3B5F\u3B60\u3B61"+
	"\u3B62\u3B63\u3B64\u3B65\u3B66\u3B67\u3B68\u3B69"+
	"\u3B6A\u3B6B\u3B6C\u3B6D\u3B6E\u3B6F\u3B70\u3B71"+
	"\u3B72\u3B73\u3B74\u3B75\u3B76\u3B77\u3B78\u3B79"+
	"\u3B7A\u3B7B\u3B7C\u3B7D\u3B7E\u3B7F\u3B80\u3B81"+
	"\u3B82\u3B83\u3B84\u3B85\u3B86\u3B87\u3B88\u3B89"+
	"\u3B8A\u3B8B\u3B8C\u3B8D\u3B8E\u3B8F\u3B90\u3B91"+
	"\u3B92\u3B93\u3B94\u3B95\u3B96\u3B97\u3B98\u3B99"+
	"\u3B9A\u3B9B\u3B9C\u3B9D\u3B9E\u3B9F\u3BA0\u3BA1"+
	"\u3BA2\u3BA3\u3BA4\u3BA5\u3BA6\u3BA7\u3BA8\u3BA9"+
	"\u3BAA\u3BAB\u3BAC\u3BAD\u3BAE\u3BAF\u3BB0\u3BB1"+
	"\u3BB2\u3BB3\u3BB4\u3BB5\u3BB6\u3BB7\u3BB8\u3BB9"+
	"\u3BBA\u3BBB\u3BBC\u3BBD\u3BBE\u3BBF\u3BC0\u3BC1"+
	"\u3BC2\u3BC3\u3BC4\u3BC5\u3BC6\u3BC7\u3BC8\u3BC9"+
	"\u3BCA\u3BCB\u3BCC\u3BCD\u3BCE\u3BCF\u3BD0\u3BD1"+
	"\u3BD2\u3BD3\u3BD4\u3BD5\u3BD6\u3BD7\u3BD8\u3BD9"+
	"\u3BDA\u3BDB\u3BDC\u3BDD\u3BDE\u3BDF\u3BE0\u3BE1"+
	"\u3BE2\u3BE3\u3BE4\u3BE5\u3BE6\u3BE7\u3BE8\u3BE9"+
	"\u3BEA\u3BEB\u3BEC\u3BED\u3BEE\u3BEF\u3BF0\u3BF1"+
	"\u3BF2\u3BF3\u3BF4\u3BF5\u3BF6\u3BF7\u3BF8\u3BF9"+
	"\u3BFA\u3BFB\u3BFC\u3BFD\u3BFE\u3BFF\u3C00\u3C01"+
	"\u3C02\u3C03\u3C04\u3C05\u3C06\u3C07\u3C08\u3C09"+
	"\u3C0A\u3C0B\u3C0C\u3C0D\u3C0E\u3C0F\u3C10\u3C11"+
	"\u3C12\u3C13\u3C14\u3C15\u3C16\u3C17\u3C18\u3C19"+
	"\u3C1A\u3C1B\u3C1C\u3C1D\u3C1E\u3C1F\u3C20\u3C21"+
	"\u3C22\u3C23\u3C24\u3C25\u3C26\u3C27\u3C28\u3C29"+
	"\u3C2A\u3C2B\u3C2C\u3C2D\u3C2E\u3C2F\u3C30\u3C31"+
	"\u3C32\u3C33\u3C34\u3C35\u3C36\u3C37\u3C38\u3C39"+
	"\u3C3A\u3C3B\u3C3C\u3C3D\u3C3E\u3C3F\u3C40\u3C41"+
	"\u3C42\u3C43\u3C44\u3C45\u3C46\u3C47\u3C48\u3C49"+
	"\u3C4A\u3C4B\u3C4C\u3C4D\u3C4E\u3C4F\u3C50\u3C51"+
	"\u3C52\u3C53\u3C54\u3C55\u3C56\u3C57\u3C58\u3C59"+
	"\u3C5A\u3C5B\u3C5C\u3C5D\u3C5E\u3C5F\u3C60\u3C61"+
	"\u3C62\u3C63\u3C64\u3C65\u3C66\u3C67\u3C68\u3C69"+
	"\u3C6A\u3C6B\u3C6C\u3C6D\u3C6E\u3C6F\u3C70\u3C71"+
	"\u3C72\u3C73\u3C74\u3C75\u3C76\u3C77\u3C78\u3C79"+
	"\u3C7A\u3C7B\u3C7C\u3C7D\u3C7E\u3C7F\u3C80\u3C81"+
	"\u3C82\u3C83\u3C84\u3C85\u3C86\u3C87\u3C88\u3C89"+
	"\u3C8A\u3C8B\u3C8C\u3C8D\u3C8E\u3C8F\u3C90\u3C91"+
	"\u3C92\u3C93\u3C94\u3C95\u3C96\u3C97\u3C98\u3C99"+
	"\u3C9A\u3C9B\u3C9C\u3C9D\u3C9E\u3C9F\u3CA0\u3CA1"+
	"\u3CA2\u3CA3\u3CA4\u3CA5\u3CA6\u3CA7\u3CA8\u3CA9"+
	"\u3CAA\u3CAB\u3CAC\u3CAD\u3CAE\u3CAF\u3CB0\u3CB1"+
	"\u3CB2\u3CB3\u3CB4\u3CB5\u3CB6\u3CB7\u3CB8\u3CB9"+
	"\u3CBA\u3CBB\u3CBC\u3CBD\u3CBE\u3CBF\u3CC0\u3CC1"+
	"\u3CC2\u3CC3\u3CC4\u3CC5\u3CC6\u3CC7\u3CC8\u3CC9"+
	"\u3CCA\u3CCB\u3CCC\u3CCD\u3CCE\u3CCF\u3CD0\u3CD1"+
	"\u3CD2\u3CD3\u3CD4\u3CD5\u3CD6\u3CD7\u3CD8\u3CD9"+
	"\u3CDA\u3CDB\u3CDC\u3CDD\u3CDE\u3CDF\u3CE0\u3CE1"+
	"\u3CE2\u3CE3\u3CE4\u3CE5\u3CE6\u3CE7\u3CE8\u3CE9"+
	"\u3CEA\u3CEB\u3CEC\u3CED\u3CEE\u3CEF\u3CF0\u3CF1"+
	"\u3CF2\u3CF3\u3CF4\u3CF5\u3CF6\u3CF7\u3CF8\u3CF9"+
	"\u3CFA\u3CFB\u3CFC\u3CFD\u3CFE\u3CFF\u3D00\u3D01"+
	"\u3D02\u3D03\u3D04\u3D05\u3D06\u3D07\u3D08\u3D09"+
	"\u3D0A\u3D0B\u3D0C\u3D0D\u3D0E\u3D0F\u3D10\u3D11"+
	"\u3D12\u3D13\u3D14\u3D15\u3D16\u3D17\u3D18\u3D19"+
	"\u3D1A\u3D1B\u3D1C\u3D1D\u3D1E\u3D1F\u3D20\u3D21"+
	"\u3D22\u3D23\u3D24\u3D25\u3D26\u3D27\u3D28\u3D29"+
	"\u3D2A\u3D2B\u3D2C\u3D2D\u3D2E\u3D2F\u3D30\u3D31"+
	"\u3D32\u3D33\u3D34\u3D35\u3D36\u3D37\u3D38\u3D39"+
	"\u3D3A\u3D3B\u3D3C\u3D3D\u3D3E\u3D3F\u3D40\u3D41"+
	"\u3D42\u3D43\u3D44\u3D45\u3D46\u3D47\u3D48\u3D49"+
	"\u3D4A\u3D4B\u3D4C\u3D4D\u3D4E\u3D4F\u3D50\u3D51"+
	"\u3D52\u3D53\u3D54\u3D55\u3D56\u3D57\u3D58\u3D59"+
	"\u3D5A\u3D5B\u3D5C\u3D5D\u3D5E\u3D5F\u3D60\u3D61"+
	"\u3D62\u3D63\u3D64\u3D65\u3D66\u3D67\u3D68\u3D69"+
	"\u3D6A\u3D6B\u3D6C\u3D6D\u3D6E\u3D6F\u3D70\u3D71"+
	"\u3D72\u3D73\u3D74\u3D75\u3D76\u3D77\u3D78\u3D79"+
	"\u3D7A\u3D7B\u3D7C\u3D7D\u3D7E\u3D7F\u3D80\u3D81"+
	"\u3D82\u3D83\u3D84\u3D85\u3D86\u3D87\u3D88\u3D89"+
	"\u3D8A\u3D8B\u3D8C\u3D8D\u3D8E\u3D8F\u3D90\u3D91"+
	"\u3D92\u3D93\u3D94\u3D95\u3D96\u3D97\u3D98\u3D99"+
	"\u3D9A\u3D9B\u3D9C\u3D9D\u3D9E\u3D9F\u3DA0\u3DA1"+
	"\u3DA2\u3DA3\u3DA4\u3DA5\u3DA6\u3DA7\u3DA8\u3DA9"+
	"\u3DAA\u3DAB\u3DAC\u3DAD\u3DAE\u3DAF\u3DB0\u3DB1"+
	"\u3DB2\u3DB3\u3DB4\u3DB5\u3DB6\u3DB7\u3DB8\u3DB9"+
	"\u3DBA\u3DBB\u3DBC\u3DBD\u3DBE\u3DBF\u3DC0\u3DC1"+
	"\u3DC2\u3DC3\u3DC4\u3DC5\u3DC6\u3DC7\u3DC8\u3DC9"+
	"\u3DCA\u3DCB\u3DCC\u3DCD\u3DCE\u3DCF\u3DD0\u3DD1"+
	"\u3DD2\u3DD3\u3DD4\u3DD5\u3DD6\u3DD7\u3DD8\u3DD9"+
	"\u3DDA\u3DDB\u3DDC\u3DDD\u3DDE\u3DDF\u3DE0\u3DE1"+
	"\u3DE2\u3DE3\u3DE4\u3DE5\u3DE6\u3DE7\u3DE8\u3DE9"+
	"\u3DEA\u3DEB\u3DEC\u3DED\u3DEE\u3DEF\u3DF0\u3DF1"+
	"\u3DF2\u3DF3\u3DF4\u3DF5\u3DF6\u3DF7\u3DF8\u3DF9"+
	"\u3DFA\u3DFB\u3DFC\u3DFD\u3DFE\u3DFF\u3E00\u3E01"+
	"\u3E02\u3E03\u3E04\u3E05\u3E06\u3E07\u3E08\u3E09"+
	"\u3E0A\u3E0B\u3E0C\u3E0D\u3E0E\u3E0F\u3E10\u3E11"+
	"\u3E12\u3E13\u3E14\u3E15\u3E16\u3E17\u3E18\u3E19"+
	"\u3E1A\u3E1B\u3E1C\u3E1D\u3E1E\u3E1F\u3E20\u3E21"+
	"\u3E22\u3E23\u3E24\u3E25\u3E26\u3E27\u3E28\u3E29"+
	"\u3E2A\u3E2B\u3E2C\u3E2D\u3E2E\u3E2F\u3E30\u3E31"+
	"\u3E32\u3E33\u3E34\u3E35\u3E36\u3E37\u3E38\u3E39"+
	"\u3E3A\u3E3B\u3E3C\u3E3D\u3E3E\u3E3F\u3E40\u3E41"+
	"\u3E42\u3E43\u3E44\u3E45\u3E46\u3E47\u3E48\u3E49"+
	"\u3E4A\u3E4B\u3E4C\u3E4D\u3E4E\u3E4F\u3E50\u3E51"+
	"\u3E52\u3E53\u3E54\u3E55\u3E56\u3E57\u3E58\u3E59"+
	"\u3E5A\u3E5B\u3E5C\u3E5D\u3E5E\u3E5F\u3E60\u3E61"+
	"\u3E62\u3E63\u3E64\u3E65\u3E66\u3E67\u3E68\u3E69"+
	"\u3E6A\u3E6B\u3E6C\u3E6D\u3E6E\u3E6F\u3E70\u3E71"+
	"\u3E72\u3E73\u3E74\u3E75\u3E76\u3E77\u3E78\u3E79"+
	"\u3E7A\u3E7B\u3E7C\u3E7D\u3E7E\u3E7F\u3E80\u3E81"+
	"\u3E82\u3E83\u3E84\u3E85\u3E86\u3E87\u3E88\u3E89"+
	"\u3E8A\u3E8B\u3E8C\u3E8D\u3E8E\u3E8F\u3E90\u3E91"+
	"\u3E92\u3E93\u3E94\u3E95\u3E96\u3E97\u3E98\u3E99"+
	"\u3E9A\u3E9B\u3E9C\u3E9D\u3E9E\u3E9F\u3EA0\u3EA1"+
	"\u3EA2\u3EA3\u3EA4\u3EA5\u3EA6\u3EA7\u3EA8\u3EA9"+
	"\u3EAA\u3EAB\u3EAC\u3EAD\u3EAE\u3EAF\u3EB0\u3EB1"+
	"\u3EB2\u3EB3\u3EB4\u3EB5\u3EB6\u3EB7\u3EB8\u3EB9"+
	"\u3EBA\u3EBB\u3EBC\u3EBD\u3EBE\u3EBF\u3EC0\u3EC1"+
	"\u3EC2\u3EC3\u3EC4\u3EC5\u3EC6\u3EC7\u3EC8\u3EC9"+
	"\u3ECA\u3ECB\u3ECC\u3ECD\u3ECE\u3ECF\u3ED0\u3ED1"+
	"\u3ED2\u3ED3\u3ED4\u3ED5\u3ED6\u3ED7\u3ED8\u3ED9"+
	"\u3EDA\u3EDB\u3EDC\u3EDD\u3EDE\u3EDF\u3EE0\u3EE1";

    private final static String innerIndex2= 
	"\u3EE2\u3EE3\u3EE4\u3EE5\u3EE6\u3EE7\u3EE8\u3EE9"+
	"\u3EEA\u3EEB\u3EEC\u3EED\u3EEE\u3EEF\u3EF0\u3EF1"+
	"\uA95C\u3EF2\u3EF3\uA843\uA1AA\uA844\uA1AC\u3EF4"+
	"\uA1AE\uA1AF\u3EF5\u3EF6\uA1B0\uA1B1\u3EF7\u3EF8"+
	"\u3EF9\u3EFA\u3EFB\u3EFC\u3EFD\uA845\uA1AD\u3EFE"+
	"\u3EFF\u3F00\u3F01\u3F02\u3F03\u3F04\u3F05\u3F06"+
	"\uA1EB\u3F07\uA1E4\uA1E5\u3F08\uA846\u3F09\u3F0A"+
	"\u3F0B\u3F0C\u3F0D\uA1F9\u3F0E\u3F0F\u3F10\u3F11"+
	"\u3F12\u3F13\u3F14\u3F15\u3F16\u3F17\u3F18\u3F19"+
	"\u3F1A\u3F1B\u3F1C\u3F1D\u3F1E\u3F1F\u3F20\u3F21"+
	"\u3F22\u3F23\u3F24\u3F25\u3F26\u3F27\u3F28\u3F29"+
	"\u3F2A\u3F2B\u3F2C\u3F2D\u3F2E\u3F2F\u3F30\u3F31"+
	"\u3F32\u3F33\u3F34\u3F35\u3F36\u3F37\u3F38\u3F39"+
	"\u3F3A\u3F3B\u3F3C\u3F3D\u3F3E\u3F3F\u3F40\u3F41"+
	"\u3F42\u3F43\u3F44\u3F45\u3F46\u3F47\u3F48\u3F49"+
	"\u3F4A\u3F4B\u3F4C\u3F4D\u3F4E\u3F4F\u3F50\u3F51"+
	"\u3F52\u3F53\u3F54\u3F55\u3F56\u3F57\u3F58\u3F59"+
	"\u3F5A\u3F5B\u3F5C\u3F5D\u3F5E\u3F5F\u3F60\u3F61"+
	"\u3F62\u3F63\u3F64\u3F65\u3F66\u3F67\u3F68\u3F69"+
	"\u3F6A\u3F6B\u3F6C\u3F6D\u3F6E\u3F6F\u3F70\u3F71"+
	"\u3F72\u3F73\u3F74\u3F75\u3F76\u3F77\u3F78\u3F79"+
	"\u3F7A\u3F7B\u3F7C\u3F7D\uA2E3\u3F7E\u3F7F\u3F80"+
	"\u3F81\u3F82\u3F83\u3F84\u3F85\u3F86\u3F87\u3F88"+
	"\u3F89\u3F8A\u3F8B\u3F8C\u3F8D\u3F8E\u3F8F\u3F90"+
	"\u3F91\u3F92\u3F93\u3F94\u3F95\u3F96\u3F97\u3F98"+
	"\u3F99\u3F9A\u3F9B\u3F9C\u3F9D\u3F9E\u3F9F\u3FA0"+
	"\u3FA1\u3FA2\u3FA3\u3FA4\u3FA5\u3FA6\u3FA7\u3FA8"+
	"\u3FA9\u3FAA\u3FAB\u3FAC\u3FAD\u3FAE\u3FAF\u3FB0"+
	"\u3FB1\u3FB2\u3FB3\u3FB4\u3FB5\u3FB6\u3FB7\u3FB8"+
	"\u3FB9\u3FBA\u3FBB\u3FBC\u3FBD\u3FBE\u3FBF\u3FC0"+
	"\u3FC1\u3FC2\u3FC3\u3FC4\u3FC5\u3FC6\u3FC7\u3FC8"+
	"\u3FC9\u3FCA\u3FCB\u3FCC\u3FCD\u3FCE\u3FCF\u3FD0"+
	"\u3FD1\u3FD2\u3FD3\uA1E6\u3FD4\uA847\u3FD5\u3FD6"+
	"\u3FD7\uA848\u3FD8\u3FD9\u3FDA\u3FDB\u3FDC\u3FDD"+
	"\u3FDE\u3FDF\u3FE0\u3FE1\u3FE2\u3FE3\uA1ED\u3FE4"+
	"\u3FE5\u3FE6\u3FE7\u3FE8\u3FE9\u3FEA\u3FEB\u3FEC"+
	"\u3FED\uA959\u3FEE\u3FEF\u3FF0\u3FF1\u3FF2\u3FF3"+
	"\u3FF4\u3FF5\u3FF6\u3FF7\u3FF8\u3FF9\u3FFA\u3FFB"+
	"\u3FFC\u3FFD\u3FFE\u3FFF\u4000\u4001\u4002\u4003"+
	"\u4004\u4005\u4006\u4007\u4008\u4009\u400A\u400B"+
	"\u400C\u400D\u400E\u400F\u4010\u4011\u4012\u4013"+
	"\u4014\u4015\u4016\u4017\u4018\u4019\u401A\u401B"+
	"\u401C\u401D\u401E\u401F\u4020\u4021\u4022\u4023"+
	"\u4024\u4025\u4026\u4027\u4028\u4029\u402A\u402B"+
	"\uA2F1\uA2F2\uA2F3\uA2F4\uA2F5\uA2F6\uA2F7\uA2F8"+
	"\uA2F9\uA2FA\uA2FB\uA2FC\u402C\u402D\u402E\u402F"+
	"\uA2A1\uA2A2\uA2A3\uA2A4\uA2A5\uA2A6\uA2A7\uA2A8"+
	"\uA2A9\uA2AA\u4030\u4031\u4032\u4033\u4034\u4035"+
	"\u4036\u4037\u4038\u4039\u403A\u403B\u403C\u403D"+
	"\u403E\u403F\u4040\u4041\u4042\u4043\u4044\u4045"+
	"\uA1FB\uA1FC\uA1FA\uA1FD\u4046\u4047\uA849\uA84A"+
	"\uA84B\uA84C\u4048\u4049\u404A\u404B\u404C\u404D"+
	"\u404E\u404F\u4050\u4051\u4052\u4053\u4054\u4055"+
	"\u4056\u4057\u4058\u4059\u405A\u405B\u405C\u405D"+
	"\u405E\u405F\u4060\u4061\u4062\u4063\u4064\u4065"+
	"\u4066\u4067\u4068\u4069\u406A\u406B\u406C\u406D"+
	"\u406E\u406F\u4070\u4071\u4072\u4073\u4074\u4075"+
	"\u4076\u4077\u4078\u4079\u407A\u407B\u407C\u407D"+
	"\u407E\u407F\u4080\u4081\u4082\u4083\u4084\u4085"+
	"\u4086\u4087\u4088\u4089\u408A\u408B\u408C\u408D"+
	"\u408E\u408F\u4090\u4091\u4092\u4093\u4094\u4095"+
	"\u4096\u4097\u4098\u4099\u409A\u409B\u409C\u409D"+
	"\u409E\u409F\u40A0\u40A1\u40A2\u40A3\u40A4\u40A5"+
	"\u40A6\u40A7\u40A8\u40A9\u40AA\u40AB\u40AC\u40AD"+
	"\u40AE\u40AF\u40B0\u40B1\u40B2\u40B3\u40B4\u40B5"+
	"\uA1CA\u40B6\u40B7\u40B8\u40B9\u40BA\u40BB\uA1C7"+
	"\u40BC\uA1C6\u40BD\u40BE\u40BF\uA84D\u40C0\u40C1"+
	"\u40C2\u40C3\uA1CC\u40C4\u40C5\uA1D8\uA1DE\uA84E"+
	"\uA1CF\u40C6\u40C7\uA84F\u40C8\uA1CE\u40C9\uA1C4"+
	"\uA1C5\uA1C9\uA1C8\uA1D2\u40CA\u40CB\uA1D3\u40CC"+
	"\u40CD\u40CE\u40CF\u40D0\uA1E0\uA1DF\uA1C3\uA1CB"+
	"\u40D1\u40D2\u40D3\u40D4\u40D5\uA1D7\u40D6\u40D7"+
	"\u40D8\u40D9\u40DA\u40DB\u40DC\u40DD\u40DE\u40DF"+
	"\uA1D6\u40E0\u40E1\u40E2\uA1D5\u40E3\u40E4\u40E5"+
	"\u40E6\u40E7\uA850\u40E8\u40E9\u40EA\u40EB\u40EC"+
	"\u40ED\u40EE\u40EF\u40F0\u40F1\u40F2\u40F3\u40F4"+
	"\uA1D9\uA1D4\u40F5\u40F6\uA1DC\uA1DD\uA851\uA852"+
	"\u40F7\u40F8\u40F9\u40FA\u40FB\u40FC\uA1DA\uA1DB"+
	"\u40FD\u40FE\u40FF\u4100\u4101\u4102\u4103\u4104"+
	"\u4105\u4106\u4107\u4108\u4109\u410A\u410B\u410C"+
	"\u410D\u410E\u410F\u4110\u4111\u4112\u4113\u4114"+
	"\u4115\u4116\u4117\u4118\u4119\u411A\u411B\u411C"+
	"\u411D\u411E\u411F\u4120\u4121\uA892\u4122\u4123"+
	"\u4124\uA1D1\u4125\u4126\u4127\u4128\u4129\u412A"+
	"\u412B\u412C\u412D\u412E\u412F\uA1CD\u4130\u4131"+
	"\u4132\u4133\u4134\u4135\u4136\u4137\u4138\u4139"+
	"\u413A\u413B\u413C\u413D\u413E\u413F\u4140\u4141"+
	"\u4142\u4143\u4144\u4145\u4146\u4147\u4148\uA853"+
	"\u4149\u414A\u414B\u414C\u414D\u414E\u414F\u4150"+
	"\u4151\u4152\u4153\u4154\u4155\u4156\u4157\u4158"+
	"\u4159\u415A\u415B\u415C\u415D\u415E\u415F\u4160"+
	"\u4161\u4162\u4163\u4164\u4165\u4166\u4167\u4168"+
	"\u4169\u416A\u416B\u416C\u416D\u416E\u416F\u4170"+
	"\u4171\u4172\u4173\u4174\u4175\u4176\u4177\u4178"+
	"\u4179\u417A\u417B\u417C\u417D\u417E\u417F\u4180"+
	"\u4181\u4182\u4183\u4184\u4185\u4186\u4187\u4188"+
	"\u4189\u418A\u418B\u418C\u418D\u418E\u418F\u4190"+
	"\u4191\u4192\u4193\u4194\u4195\u4196\u4197\u4198"+
	"\u4199\u419A\uA1D0\u419B\u419C\u419D\u419E\u419F"+
	"\u41A0\u41A1\u41A2\u41A3\u41A4\u41A5\u41A6\u41A7"+
	"\u41A8\u41A9\u41AA\u41AB\u41AC\u41AD\u41AE\u41AF"+
	"\u41B0\u41B1\u41B2\u41B3\u41B4\u41B5\u41B6\u41B7"+
	"\u41B8\u41B9\u41BA\u41BB\u41BC\u41BD\u41BE\u41BF"+
	"\u41C0\u41C1\u41C2\u41C3\u41C4\u41C5\u41C6\u41C7"+
	"\u41C8\u41C9\u41CA\u41CB\u41CC\u41CD\u41CE\u41CF"+
	"\u41D0\u41D1\u41D2\u41D3\u41D4\u41D5\u41D6\u41D7"+
	"\u41D8\u41D9\u41DA\u41DB\u41DC\u41DD\u41DE\u41DF"+
	"\u41E0\u41E1\u41E2\u41E3\u41E4\u41E5\u41E6\u41E7"+
	"\u41E8\u41E9\u41EA\u41EB\u41EC\u41ED\u41EE\u41EF"+
	"\u41F0\u41F1\u41F2\u41F3\u41F4\u41F5\u41F6\u41F7"+
	"\u41F8\u41F9\u41FA\u41FB\u41FC\u41FD\u41FE\u41FF"+
	"\u4200\u4201\u4202\u4203\u4204\u4205\u4206\u4207"+
	"\u4208\u4209\u420A\u420B\u420C\u420D\u420E\u420F"+
	"\u4210\u4211\u4212\u4213\u4214\u4215\u4216\u4217"+
	"\u4218\u4219\u421A\u421B\u421C\u421D\u421E\u421F"+
	"\u4220\u4221\u4222\u4223\u4224\u4225\u4226\u4227"+
	"\u4228\u4229\u422A\u422B\u422C\u422D\u422E\u422F"+
	"\u4230\u4231\u4232\u4233\u4234\u4235\u4236\u4237"+
	"\u4238\u4239\u423A\u423B\u423C\u423D\u423E\u423F"+
	"\u4240\u4241\u4242\u4243\u4244\u4245\u4246\u4247"+
	"\u4248\u4249\u424A\u424B\u424C\u424D\u424E\u424F"+
	"\u4250\u4251\u4252\u4253\u4254\u4255\u4256\u4257"+
	"\u4258\u4259\u425A\u425B\u425C\u425D\u425E\u425F"+
	"\u4260\u4261\u4262\u4263\u4264\u4265\u4266\u4267"+
	"\u4268\u4269\u426A\u426B\u426C\u426D\u426E\u426F"+
	"\u4270\u4271\u4272\u4273\u4274\u4275\u4276\u4277"+
	"\u4278\u4279\u427A\u427B\u427C\u427D\u427E\u427F"+
	"\u4280\u4281\u4282\u4283\u4284\u4285\u4286\u4287"+
	"\u4288\u4289\u428A\u428B\u428C\u428D\u428E\u428F"+
	"\u4290\u4291\u4292\u4293\u4294\u4295\u4296\u4297"+
	"\u4298\u4299\u429A\u429B\u429C\u429D\u429E\u429F"+
	"\u42A0\u42A1\u42A2\u42A3\u42A4\u42A5\u42A6\u42A7"+
	"\u42A8\u42A9\u42AA\u42AB\u42AC\u42AD\u42AE\u42AF"+
	"\u42B0\u42B1\u42B2\u42B3\u42B4\u42B5\u42B6\u42B7"+
	"\u42B8\u42B9\u42BA\u42BB\u42BC\u42BD\u42BE\u42BF"+
	"\u42C0\u42C1\u42C2\u42C3\u42C4\u42C5\u42C6\u42C7"+
	"\u42C8\u42C9\u42CA\u42CB\u42CC\u42CD\u42CE\u42CF"+
	"\u42D0\u42D1\u42D2\u42D3\u42D4\u42D5\u42D6\u42D7"+
	"\u42D8\u42D9\u42DA\u42DB\u42DC\u42DD\u42DE\u42DF"+
	"\u42E0\u42E1\u42E2\u42E3\u42E4\u42E5\u42E6\u42E7"+
	"\uA2D9\uA2DA\uA2DB\uA2DC\uA2DD\uA2DE\uA2DF\uA2E0"+
	"\uA2E1\uA2E2\u42E8\u42E9\u42EA\u42EB\u42EC\u42ED"+
	"\u42EE\u42EF\u42F0\u42F1\uA2C5\uA2C6\uA2C7\uA2C8"+
	"\uA2C9\uA2CA\uA2CB\uA2CC\uA2CD\uA2CE\uA2CF\uA2D0"+
	"\uA2D1\uA2D2\uA2D3\uA2D4\uA2D5\uA2D6\uA2D7\uA2D8"+
	"\uA2B1\uA2B2\uA2B3\uA2B4\uA2B5\uA2B6\uA2B7\uA2B8"+
	"\uA2B9\uA2BA\uA2BB\uA2BC\uA2BD\uA2BE\uA2BF\uA2C0"+
	"\uA2C1\uA2C2\uA2C3\uA2C4\u42F2\u42F3\u42F4\u42F5"+
	"\u42F6\u42F7\u42F8\u42F9\u42FA\u42FB\u42FC\u42FD"+
	"\u42FE\u42FF\u4300\u4301\u4302\u4303\u4304\u4305"+
	"\u4306\u4307\u4308\u4309\u430A\u430B\u430C\u430D"+
	"\u430E\u430F\u4310\u4311\u4312\u4313\u4314\u4315"+
	"\u4316\u4317\u4318\u4319\u431A\u431B\u431C\u431D"+
	"\u431E\u431F\u4320\u4321\u4322\u4323\u4324\u4325"+
	"\u4326\u4327\u4328\u4329\u432A\u432B\u432C\u432D"+
	"\u432E\u432F\u4330\u4331\u4332\u4333\u4334\u4335"+
	"\u4336\u4337\u4338\u4339\u433A\u433B\u433C\u433D"+
	"\u433E\u433F\u4340\u4341\u4342\u4343\u4344\u4345"+
	"\u4346\u4347\u4348\u4349\u434A\u434B\u434C\u434D"+
	"\u434E\u434F\u4350\u4351\u4352\u4353\u4354\u4355"+
	"\uA9A4\uA9A5\uA9A6\uA9A7\uA9A8\uA9A9\uA9AA\uA9AB"+
	"\uA9AC\uA9AD\uA9AE\uA9AF\uA9B0\uA9B1\uA9B2\uA9B3"+
	"\uA9B4\uA9B5\uA9B6\uA9B7\uA9B8\uA9B9\uA9BA\uA9BB"+
	"\uA9BC\uA9BD\uA9BE\uA9BF\uA9C0\uA9C1\uA9C2\uA9C3"+
	"\uA9C4\uA9C5\uA9C6\uA9C7\uA9C8\uA9C9\uA9CA\uA9CB"+
	"\uA9CC\uA9CD\uA9CE\uA9CF\uA9D0\uA9D1\uA9D2\uA9D3"+
	"\uA9D4\uA9D5\uA9D6\uA9D7\uA9D8\uA9D9\uA9DA\uA9DB"+
	"\uA9DC\uA9DD\uA9DE\uA9DF\uA9E0\uA9E1\uA9E2\uA9E3"+
	"\uA9E4\uA9E5\uA9E6\uA9E7\uA9E8\uA9E9\uA9EA\uA9EB"+
	"\uA9EC\uA9ED\uA9EE\uA9EF\u4356\u4357\u4358\u4359"+
	"\uA854\uA855\uA856\uA857\uA858\uA859\uA85A\uA85B"+
	"\uA85C\uA85D\uA85E\uA85F\uA860\uA861\uA862\uA863"+
	"\uA864\uA865\uA866\uA867\uA868\uA869\uA86A\uA86B"+
	"\uA86C\uA86D\uA86E\uA86F\uA870\uA871\uA872\uA873"+
	"\uA874\uA875\uA876\uA877\u435A\u435B\u435C\u435D"+
	"\u435E\u435F\u4360\u4361\u4362\u4363\u4364\u4365"+
	"\u4366\uA878\uA879\uA87A\uA87B\uA87C\uA87D\uA87E"+
	"\uA880\uA881\uA882\uA883\uA884\uA885\uA886\uA887"+
	"\u4367\u4368\u4369\uA888\uA889\uA88A\u436A\u436B"+
	"\u436C\u436D\u436E\u436F\u4370\u4371\u4372\u4373"+
	"\uA1F6\uA1F5\u4374\u4375\u4376\u4377\u4378\u4379"+
	"\u437A\u437B\u437C\u437D\u437E\u437F\u4380\u4381"+
	"\u4382\u4383\uA1F8\uA1F7\u4384\u4385\u4386\u4387"+
	"\u4388\u4389\u438A\u438B\uA88B\uA88C\u438C\u438D"+
	"\u438E\u438F\u4390\u4391\u4392\u4393\uA1F4\uA1F3"+
	"\u4394\u4395\u4396\uA1F0\u4397\u4398\uA1F2\uA1F1"+
	"\u4399\u439A\u439B\u439C\u439D\u439E\u439F\u43A0"+
	"\u43A1\u43A2\u43A3\u43A4\u43A5\u43A6\u43A7\u43A8"+
	"\u43A9\u43AA\uA88D\uA88E\uA88F\uA890\u43AB\u43AC"+
	"\u43AD\u43AE\u43AF\u43B0\u43B1\u43B2\u43B3\u43B4"+
	"\u43B5\u43B6\u43B7\u43B8\u43B9\u43BA\u43BB\u43BC"+
	"\u43BD\u43BE\u43BF\u43C0\u43C1\u43C2\u43C3\u43C4"+
	"\u43C5\u43C6\u43C7\u43C8\u43C9\uA1EF\uA1EE\u43CA"+
	"\u43CB\uA891\u43CC\u43CD\u43CE\u43CF\u43D0\u43D1"+
	"\u43D2\u43D3\u43D4\u43D5\u43D6\u43D7\u43D8\u43D9"+
	"\u43DA\u43DB\u43DC\u43DD\u43DE\u43DF\u43E0\u43E1"+
	"\u43E2\u43E3\u43E4\u43E5\u43E6\u43E7\u43E8\u43E9"+
	"\u43EA\u43EB\u43EC\u43ED\u43EE\u43EF\u43F0\u43F1"+
	"\u43F2\u43F3\u43F4\u43F5\u43F6\u43F7\u43F8\u43F9"+
	"\u43FA\u43FB\u43FC\u43FD\u43FE\u43FF\u4400\u4401"+
	"\uA1E2\u4402\uA1E1\u4403\u4404\u4405\u4406\u4407"+
	"\u4408\u4409\u440A\u440B\u440C\u440D\u440E\u440F"+
	"\u4410\u4411\u4412\u4413\u4414\u4415\u4416\u4417"+
	"\u4418\u4419\u441A\u441B\u441C\u441D\u441E\u441F"+
	"\u4420\u4421\u4422\u4423\u4424\u4425\u4426\u4427"+
	"\u4428\u4429\u442A\u442B\u442C\u442D\u442E\u442F"+
	"\u4430\u4431\u4432\u4433\u4434\u4435\u4436\u4437"+
	"\u4438\u4439\u443A\u443B\u443C\u443D\u443E\u443F"+
	"\u4440\u4441\u4442\u4443\u4444\u4445\u4446\u4447"+
	"\u4448\u4449\u444A\u444B\u444C\u444D\u444E\u444F"+
	"\u4450\u4451\u4452\u4453\u4454\u4455\u4456\u4457"+
	"\u4458\u4459\u445A\u445B\u445C\u445D\u445E\u445F"+
	"\u4460\u4461\u4462\u4463\u4464\u4465\u4466\u4467"+
	"\u4468\u4469\u446A\u446B\u446C\u446D\u446E\u446F"+
	"\u4470\u4471\u4472\u4473\u4474\u4475\u4476\u4477"+
	"\u4478\u4479\u447A\u447B\u447C\u447D\u447E\u447F"+
	"\u4480\u4481\u4482\u4483\u4484\u4485\u4486\u4487"+
	"\u4488\u4489\u448A\u448B\u448C\u448D\u448E\u448F"+
	"\u4490\u4491\u4492\u4493\u4494\u4495\u4496\u4497"+
	"\u4498\u4499\u449A\u449B\u449C\u449D\u449E\u449F"+
	"\u44A0\u44A1\u44A2\u44A3\u44A4\u44A5\u44A6\u44A7"+
	"\u44A8\u44A9\u44AA\u44AB\u44AC\u44AD\u44AE\u44AF"+
	"\u44B0\u44B1\u44B2\u44B3\u44B4\u44B5\u44B6\u44B7"+
	"\u44B8\u44B9\u44BA\u44BB\u44BC\u44BD\u44BE\u44BF"+
	"\u44C0\u44C1\u44C2\u44C3\u44C4\u44C5\u44C6\u44C7"+
	"\u44C8\u44C9\u44CA\u44CB\u44CC\u44CD\u44CE\u44CF"+
	"\u44D0\u44D1\u44D2\u44D3\u44D4\u44D5\u44D6\u44D7"+
	"\u44D8\u44D9\u44DA\u44DB\u44DC\u44DD\u44DE\u44DF"+
	"\u44E0\u44E1\u44E2\u44E3\u44E4\u44E5\u44E6\u44E7"+
	"\u44E8\u44E9\u44EA\u44EB\u44EC\u44ED\u44EE\u44EF"+
	"\u44F0\u44F1\u44F2\u44F3\u44F4\u44F5\u44F6\u44F7"+
	"\u44F8\u44F9\u44FA\u44FB\u44FC\u44FD\u44FE\u44FF"+
	"\u4500\u4501\u4502\u4503\u4504\u4505\u4506\u4507"+
	"\u4508\u4509\u450A\u450B\u450C\u450D\u450E\u450F"+
	"\u4510\u4511\u4512\u4513\u4514\u4515\u4516\u4517"+
	"\u4518\u4519\u451A\u451B\u451C\u451D\u451E\u451F"+
	"\u4520\u4521\u4522\u4523\u4524\u4525\u4526\u4527"+
	"\u4528\u4529\u452A\u452B\u452C\u452D\u452E\u452F"+
	"\u4530\u4531\u4532\u4533\u4534\u4535\u4536\u4537"+
	"\u4538\u4539\u453A\u453B\u453C\u453D\u453E\u453F"+
	"\u4540\u4541\u4542\u4543\u4544\u4545\u4546\u4547"+
	"\u4548\u4549\u454A\u454B\u454C\u454D\u454E\u454F"+
	"\u4550\u4551\u4552\u4553\u4554\u4555\u4556\u4557"+
	"\u4558\u4559\u455A\u455B\u455C\u455D\u455E\u455F"+
	"\u4560\u4561\u4562\u4563\u4564\u4565\u4566\u4567"+
	"\u4568\u4569\u456A\u456B\u456C\u456D\u456E\u456F"+
	"\u4570\u4571\u4572\u4573\u4574\u4575\u4576\u4577"+
	"\u4578\u4579\u457A\u457B\u457C\u457D\u457E\u457F"+
	"\u4580\u4581\u4582\u4583\u4584\u4585\u4586\u4587"+
	"\u4588\u4589\u458A\u458B\u458C\u458D\u458E\u458F"+
	"\u4590\u4591\u4592\u4593\u4594\u4595\u4596\u4597"+
	"\u4598\u4599\u459A\u459B\u459C\u459D\u459E\u459F"+
	"\u45A0\u45A1\u45A2\u45A3\u45A4\u45A5\u45A6\u45A7"+
	"\u45A8\u45A9\u45AA\u45AB\u45AC\u45AD\u45AE\u45AF"+
	"\u45B0\u45B1\u45B2\u45B3\u45B4\u45B5\u45B6\u45B7"+
	"\u45B8\u45B9\u45BA\u45BB\u45BC\u45BD\u45BE\u45BF"+
	"\u45C0\u45C1\u45C2\u45C3\u45C4\u45C5\u45C6\u45C7"+
	"\u45C8\u45C9\u45CA\u45CB\u45CC\u45CD\u45CE\u45CF"+
	"\u45D0\u45D1\u45D2\u45D3\u45D4\u45D5\u45D6\u45D7"+
	"\u45D8\u45D9\u45DA\u45DB\u45DC\u45DD\u45DE\u45DF"+
	"\u45E0\u45E1\u45E2\u45E3\u45E4\u45E5\u45E6\u45E7"+
	"\u45E8\u45E9\u45EA\u45EB\u45EC\u45ED\u45EE\u45EF"+
	"\u45F0\u45F1\u45F2\u45F3\u45F4\u45F5\u45F6\u45F7"+
	"\u45F8\u45F9\u45FA\u45FB\u45FC\u45FD\u45FE\u45FF"+
	"\u4600\u4601\u4602\u4603\u4604\u4605\u4606\u4607"+
	"\u4608\u4609\u460A\u460B\u460C\u460D\u460E\u460F"+
	"\u4610\u4611\u4612\u4613\u4614\u4615\u4616\u4617"+
	"\u4618\u4619\u461A\u461B\u461C\u461D\u461E\u461F"+
	"\u4620\u4621\u4622\u4623\u4624\u4625\u4626\u4627"+
	"\u4628\u4629\u462A\u462B\u462C\u462D\u462E\u462F"+
	"\u4630\u4631\u4632\u4633\u4634\u4635\u4636\u4637"+
	"\u4638\u4639\u463A\u463B\u463C\u463D\u463E\u463F"+
	"\u4640\u4641\u4642\u4643\u4644\u4645\u4646\u4647"+
	"\u4648\u4649\u464A\u464B\u464C\u464D\u464E\u464F"+
	"\u4650\u4651\u4652\u4653\u4654\u4655\u4656\u4657"+
	"\u4658\u4659\u465A\u465B\u465C\u465D\u465E\u465F"+
	"\u4660\u4661\u4662\u4663\u4664\u4665\u4666\u4667"+
	"\u4668\u4669\u466A\u466B\u466C\u466D\u466E\u466F"+
	"\u4670\u4671\u4672\u4673\u4674\u4675\u4676\u4677"+
	"\u4678\u4679\u467A\u467B\u467C\u467D\u467E\u467F"+
	"\u4680\u4681\u4682\u4683\u4684\u4685\u4686\u4687"+
	"\u4688\u4689\u468A\u468B\u468C\u468D\u468E\u468F"+
	"\u4690\u4691\u4692\u4693\u4694\u4695\u4696\u4697"+
	"\u4698\u4699\u469A\u469B\u469C\u469D\u469E\u469F"+
	"\u46A0\u46A1\u46A2\u46A3\u46A4\u46A5\u46A6\u46A7"+
	"\u46A8\u46A9\u46AA\u46AB\u46AC\u46AD\u46AE\u46AF"+
	"\u46B0\u46B1\u46B2\u46B3\u46B4\u46B5\u46B6\u46B7"+
	"\u46B8\u46B9\u46BA\u46BB\u46BC\u46BD\u46BE\u46BF"+
	"\u46C0\u46C1\u46C2\u46C3\u46C4\u46C5\u46C6\u46C7"+
	"\u46C8\u46C9\u46CA\u46CB\u46CC\u46CD\u46CE\u46CF"+
	"\u46D0\u46D1\u46D2\u46D3\u46D4\u46D5\u46D6\u46D7"+
	"\u46D8\u46D9\u46DA\u46DB\u46DC\u46DD\u46DE\u46DF"+
	"\u46E0\u46E1\u46E2\u46E3\u46E4\u46E5\u46E6\u46E7"+
	"\u46E8\u46E9\u46EA\u46EB\u46EC\u46ED\u46EE\u46EF"+
	"\u46F0\u46F1\u46F2\u46F3\u46F4\u46F5\u46F6\u46F7"+
	"\u46F8\u46F9\u46FA\u46FB\u46FC\u46FD\u46FE\u46FF"+
	"\u4700\u4701\u4702\u4703\u4704\u4705\u4706\u4707"+
	"\u4708\u4709\u470A\u470B\u470C\u470D\u470E\u470F"+
	"\u4710\u4711\u4712\u4713\u4714\u4715\u4716\u4717"+
	"\u4718\u4719\u471A\u471B\u471C\u471D\u471E\u471F"+
	"\u4720\u4721\u4722\u4723\u4724\u4725\u4726\u4727"+
	"\u4728\u4729\u472A\u472B\u472C\u472D\u472E\u472F"+
	"\u4730\u4731\u4732\u4733\u4734\u4735\u4736\u4737"+
	"\u4738\u4739\u473A\u473B\u473C\u473D\u473E\u473F"+
	"\u4740\u4741\u4742\u4743\u4744\u4745\u4746\u4747"+
	"\u4748\u4749\u474A\u474B\u474C\u474D\u474E\u474F"+
	"\u4750\u4751\u4752\u4753\u4754\u4755\u4756\u4757"+
	"\u4758\u4759\u475A\u475B\u475C\u475D\u475E\u475F"+
	"\u4760\u4761\u4762\u4763\u4764\u4765\u4766\u4767"+
	"\u4768\u4769\u476A\u476B\u476C\u476D\u476E\u476F"+
	"\u4770\u4771\u4772\u4773\u4774\u4775\u4776\u4777"+
	"\u4778\u4779\u477A\u477B\u477C\u477D\u477E\u477F"+
	"\u4780\u4781\u4782\u4783\u4784\u4785\u4786\u4787"+
	"\u4788\u4789\u478A\u478B\u478C\u478D\u478E\u478F"+
	"\u4790\u4791\u4792\u4793\u4794\u4795\u4796\u4797"+
	"\u4798\u4799\u479A\u479B\u479C\u479D\u479E\u479F"+
	"\u47A0\u47A1\u47A2\u47A3\u47A4\u47A5\u47A6\u47A7"+
	"\u47A8\u47A9\u47AA\u47AB\u47AC\u47AD\u47AE\u47AF"+
	"\u47B0\u47B1\u47B2\u47B3\u47B4\u47B5\u47B6\u47B7"+
	"\u47B8\u47B9\u47BA\u47BB\u47BC\u47BD\u47BE\u47BF"+
	"\u47C0\u47C1\u47C2\u47C3\u47C4\u47C5\u47C6\u47C7"+
	"\u47C8\u47C9\u47CA\u47CB\u47CC\u47CD\u47CE\u47CF"+
	"\u47D0\u47D1\u47D2\u47D3\u47D4\u47D5\u47D6\u47D7"+
	"\u47D8\u47D9\u47DA\u47DB\u47DC\u47DD\u47DE\u47DF"+
	"\u47E0\u47E1\u47E2\u47E3\u47E4\u47E5\u47E6\u47E7"+
	"\u47E8\u47E9\u47EA\u47EB\u47EC\u47ED\u47EE\u47EF"+
	"\u47F0\u47F1\u47F2\u47F3\u47F4\u47F5\u47F6\u47F7"+
	"\u47F8\u47F9\u47FA\u47FB\u47FC\u47FD\u47FE\u47FF"+
	"\u4800\u4801\u4802\u4803\u4804\u4805\u4806\u4807"+
	"\u4808\u4809\u480A\u480B\u480C\u480D\u480E\u480F"+
	"\u4810\u4811\u4812\u4813\u4814\u4815\u4816\u4817"+
	"\u4818\u4819\u481A\u481B\u481C\u481D\u481E\u481F"+
	"\u4820\u4821\u4822\u4823\u4824\u4825\u4826\u4827"+
	"\u4828\u4829\u482A\u482B\u482C\u482D\u482E\u482F"+
	"\u4830\u4831\u4832\u4833\u4834\u4835\u4836\u4837"+
	"\u4838\u4839\u483A\u483B\u483C\u483D\u483E\u483F"+
	"\u4840\u4841\u4842\u4843\u4844\u4845\u4846\u4847"+
	"\u4848\u4849\u484A\u484B\u484C\u484D\u484E\u484F"+
	"\u4850\u4851\u4852\u4853\u4854\u4855\u4856\u4857"+
	"\u4858\u4859\u485A\u485B\u485C\u485D\u485E\u485F"+
	"\u4860\u4861\u4862\u4863\u4864\u4865\u4866\u4867"+
	"\u4868\u4869\u486A\u486B\u486C\u486D\u486E\u486F"+
	"\u4870\u4871\u4872\u4873\u4874\u4875\u4876\u4877"+
	"\u4878\u4879\u487A\u487B\u487C\u487D\u487E\u487F"+
	"\u4880\u4881\u4882\u4883\u4884\u4885\u4886\u4887"+
	"\u4888\u4889\u488A\u488B\u488C\u488D\u488E\u488F"+
	"\u4890\u4891\u4892\u4893\u4894\u4895\u4896\u4897"+
	"\u4898\u4899\u489A\u489B\u489C\u489D\u489E\u489F"+
	"\u48A0\u48A1\u48A2\u48A3\u48A4\u48A5\u48A6\u48A7"+
	"\u48A8\u48A9\u48AA\u48AB\u48AC\u48AD\u48AE\u48AF"+
	"\u48B0\u48B1\u48B2\u48B3\u48B4\u48B5\u48B6\u48B7"+
	"\u48B8\u48B9\u48BA\u48BB\u48BC\u48BD\u48BE\u48BF"+
	"\u48C0\u48C1\u48C2\u48C3\u48C4\u48C5\u48C6\u48C7"+
	"\u48C8\u48C9\u48CA\u48CB\u48CC\u48CD\u48CE\u48CF"+
	"\u48D0\u48D1\u48D2\u48D3\u48D4\u48D5\u48D6\u48D7"+
	"\u48D8\u48D9\u48DA\u48DB\u48DC\u48DD\u48DE\u48DF"+
	"\u48E0\u48E1\u48E2\u48E3\u48E4\u48E5\u48E6\u48E7"+
	"\u48E8\u48E9\u48EA\u48EB\u48EC\u48ED\u48EE\u48EF"+
	"\u48F0\u48F1\u48F2\u48F3\u48F4\u48F5\u48F6\u48F7"+
	"\u48F8\u48F9\u48FA\u48FB\u48FC\u48FD\u48FE\u48FF"+
	"\u4900\u4901\u4902\u4903\u4904\u4905\u4906\u4907"+
	"\u4908\u4909\u490A\u490B\u490C\u490D\u490E\u490F"+
	"\u4910\u4911\u4912\u4913\u4914\u4915\u4916\u4917"+
	"\u4918\u4919\u491A\u491B\u491C\u491D\u491E\u491F"+
	"\u4920\u4921\u4922\u4923\u4924\u4925\u4926\u4927"+
	"\u4928\u4929\u492A\u492B\u492C\u492D\u492E\u492F"+
	"\u4930\u4931\u4932\u4933\u4934\u4935\u4936\u4937"+
	"\u4938\u4939\u493A\u493B\u493C\u493D\u493E\u493F"+
	"\u4940\u4941\u4942\u4943\u4944\u4945\u4946\u4947"+
	"\u4948\u4949\u494A\u494B\u494C\u494D\u494E\u494F"+
	"\u4950\u4951\u4952\u4953\u4954\u4955\u4956\u4957"+
	"\u4958\u4959\u495A\u495B\u495C\u495D\u495E\u495F"+
	"\u4960\u4961\u4962\u4963\u4964\u4965\u4966\u4967"+
	"\u4968\u4969\u496A\u496B\u496C\u496D\u496E\u496F"+
	"\u4970\u4971\u4972\u4973\u4974\u4975\u4976\u4977"+
	"\u4978\u4979\u497A\u497B\u497C\u497D\u497E\u497F"+
	"\u4980\u4981\u4982\u4983\u4984\u4985\u4986\u4987"+
	"\u4988\u4989\u498A\u498B\u498C\u498D\u498E\u498F"+
	"\u4990\u4991\u4992\u4993\u4994\u4995\u4996\u4997"+
	"\u4998\u4999\u499A\u499B\u499C\u499D\u499E\u499F"+
	"\u49A0\u49A1\u49A2\u49A3\u49A4\u49A5\u49A6\u49A7"+
	"\u49A8\u49A9\u49AA\u49AB\u49AC\u49AD\u49AE\u49AF"+
	"\u49B0\u49B1\u49B2\u49B3\u49B4\u49B5\u49B6\u49B7"+
	"\u49B8\u49B9\u49BA\u49BB\u49BC\u49BD\u49BE\u49BF"+
	"\u49C0\u49C1\u49C2\u49C3\u49C4\u49C5\u49C6\u49C7"+
	"\u49C8\u49C9\u49CA\u49CB\u49CC\u49CD\u49CE\u49CF"+
	"\u49D0\u49D1\u49D2\u49D3\u49D4\u49D5\u49D6\u49D7"+
	"\u49D8\u49D9\u49DA\u49DB\u49DC\u49DD\u49DE\u49DF"+
	"\u49E0\u49E1\u49E2\u49E3\u49E4\u49E5\u49E6\u49E7"+
	"\u49E8\u49E9\u49EA\u49EB\u49EC\u49ED\u49EE\u49EF"+
	"\u49F0\u49F1\u49F2\u49F3\u49F4\u49F5\u49F6\u49F7"+
	"\u49F8\u49F9\u49FA\u49FB\u49FC\u49FD\u49FE\u49FF"+
	"\u4A00\u4A01\u4A02\u4A03\u4A04\u4A05\u4A06\u4A07"+
	"\u4A08\u4A09\u4A0A\u4A0B\u4A0C\u4A0D\u4A0E\u4A0F"+
	"\u4A10\u4A11\u4A12\u4A13\u4A14\u4A15\u4A16\u4A17"+
	"\u4A18\u4A19\u4A1A\u4A1B\u4A1C\u4A1D\u4A1E\u4A1F"+
	"\u4A20\u4A21\u4A22\u4A23\u4A24\u4A25\u4A26\u4A27"+
	"\u4A28\u4A29\u4A2A\u4A2B\u4A2C\u4A2D\u4A2E\u4A2F"+
	"\u4A30\u4A31\u4A32\u4A33\u4A34\u4A35\u4A36\u4A37"+
	"\u4A38\u4A39\u4A3A\u4A3B\u4A3C\u4A3D\u4A3E\u4A3F"+
	"\u4A40\u4A41\u4A42\u4A43\u4A44\u4A45\u4A46\u4A47"+
	"\u4A48\u4A49\u4A4A\u4A4B\u4A4C\u4A4D\u4A4E\u4A4F"+
	"\u4A50\u4A51\u4A52\u4A53\u4A54\u4A55\u4A56\u4A57"+
	"\u4A58\u4A59\u4A5A\u4A5B\u4A5C\u4A5D\u4A5E\u4A5F"+
	"\u4A60\u4A61\u4A62\u4A63\u4A64\u4A65\u4A66\u4A67"+
	"\u4A68\u4A69\u4A6A\u4A6B\u4A6C\u4A6D\u4A6E\u4A6F"+
	"\u4A70\u4A71\u4A72\u4A73\u4A74\u4A75\u4A76\u4A77"+
	"\u4A78\u4A79\u4A7A\u4A7B\u4A7C\u4A7D\u4A7E\u4A7F"+
	"\u4A80\u4A81\u4A82\u4A83\u4A84\u4A85\u4A86\u4A87"+
	"\u4A88\u4A89\u4A8A\u4A8B\u4A8C\u4A8D\u4A8E\u4A8F"+
	"\u4A90\u4A91\u4A92\u4A93\u4A94\u4A95\u4A96\u4A97"+
	"\u4A98\u4A99\u4A9A\u4A9B\u4A9C\u4A9D\u4A9E\u4A9F"+
	"\u4AA0\u4AA1\u4AA2\u4AA3\u4AA4\u4AA5\u4AA6\u4AA7"+
	"\u4AA8\u4AA9\u4AAA\u4AAB\u4AAC\u4AAD\u4AAE\u4AAF"+
	"\u4AB0\u4AB1\u4AB2\u4AB3\u4AB4\u4AB5\u4AB6\u4AB7"+
	"\u4AB8\u4AB9\u4ABA\u4ABB\u4ABC\u4ABD\u4ABE\u4ABF"+
	"\u4AC0\u4AC1\u4AC2\u4AC3\u4AC4\u4AC5\u4AC6\u4AC7"+
	"\u4AC8\u4AC9\u4ACA\u4ACB\u4ACC\u4ACD\u4ACE\u4ACF"+
	"\u4AD0\u4AD1\u4AD2\u4AD3\u4AD4\u4AD5\u4AD6\u4AD7"+
	"\u4AD8\u4AD9\u4ADA\u4ADB\u4ADC\u4ADD\u4ADE\u4ADF"+
	"\u4AE0\u4AE1\u4AE2\u4AE3\u4AE4\u4AE5\u4AE6\u4AE7"+
	"\u4AE8\u4AE9\u4AEA\u4AEB\u4AEC\u4AED\u4AEE\u4AEF"+
	"\u4AF0\u4AF1\u4AF2\u4AF3\u4AF4\u4AF5\u4AF6\u4AF7"+
	"\u4AF8\u4AF9\u4AFA\u4AFB\u4AFC\u4AFD\u4AFE\u4AFF"+
	"\u4B00\u4B01\u4B02\u4B03\u4B04\u4B05\u4B06\u4B07"+
	"\u4B08\u4B09\u4B0A\u4B0B\u4B0C\u4B0D\u4B0E\u4B0F"+
	"\u4B10\u4B11\u4B12\u4B13\u4B14\u4B15\u4B16\u4B17"+
	"\u4B18\u4B19\u4B1A\u4B1B\u4B1C\u4B1D\u4B1E\u4B1F"+
	"\u4B20\u4B21\u4B22\u4B23\u4B24\u4B25\u4B26\u4B27"+
	"\u4B28\u4B29\u4B2A\u4B2B\u4B2C\u4B2D\u4B2E\u4B2F"+
	"\u4B30\u4B31\u4B32\u4B33\u4B34\u4B35\u4B36\u4B37"+
	"\u4B38\u4B39\u4B3A\u4B3B\u4B3C\u4B3D\u4B3E\u4B3F"+
	"\u4B40\u4B41\u4B42\u4B43\u4B44\u4B45\u4B46\u4B47"+
	"\u4B48\u4B49\u4B4A\u4B4B\u4B4C\u4B4D\u4B4E\u4B4F"+
	"\u4B50\u4B51\u4B52\u4B53\u4B54\u4B55\u4B56\u4B57"+
	"\u4B58\u4B59\u4B5A\u4B5B\u4B5C\u4B5D\u4B5E\u4B5F"+
	"\u4B60\u4B61\u4B62\u4B63\u4B64\u4B65\u4B66\u4B67"+
	"\u4B68\u4B69\u4B6A\u4B6B\u4B6C\u4B6D\u4B6E\u4B6F"+
	"\u4B70\u4B71\u4B72\u4B73\u4B74\u4B75\u4B76\u4B77"+
	"\u4B78\u4B79\u4B7A\u4B7B\u4B7C\u4B7D\u4B7E\u4B7F"+
	"\u4B80\u4B81\u4B82\u4B83\u4B84\u4B85\u4B86\u4B87"+
	"\u4B88\u4B89\u4B8A\u4B8B\u4B8C\u4B8D\u4B8E\u4B8F"+
	"\u4B90\u4B91\u4B92\u4B93\u4B94\u4B95\u4B96\u4B97"+
	"\u4B98\u4B99\u4B9A\u4B9B\u4B9C\u4B9D\u4B9E\u4B9F"+
	"\u4BA0\u4BA1\u4BA2\u4BA3\u4BA4\u4BA5\u4BA6\u4BA7"+
	"\u4BA8\u4BA9\u4BAA\u4BAB\u4BAC\u4BAD\u4BAE\u4BAF"+
	"\u4BB0\u4BB1\u4BB2\u4BB3\u4BB4\u4BB5\u4BB6\u4BB7"+
	"\u4BB8\u4BB9\u4BBA\u4BBB\u4BBC\u4BBD\u4BBE\u4BBF"+
	"\u4BC0\u4BC1\u4BC2\u4BC3\u4BC4\u4BC5\u4BC6\u4BC7"+
	"\u4BC8\u4BC9\u4BCA\u4BCB\u4BCC\u4BCD\u4BCE\u4BCF"+
	"\u4BD0\u4BD1\u4BD2\u4BD3\u4BD4\u4BD5\u4BD6\u4BD7"+
	"\u4BD8\u4BD9\u4BDA\u4BDB\u4BDC\u4BDD\u4BDE\u4BDF"+
	"\u4BE0\u4BE1\u4BE2\u4BE3\u4BE4\u4BE5\u4BE6\u4BE7"+
	"\u4BE8\u4BE9\u4BEA\u4BEB\u4BEC\u4BED\u4BEE\u4BEF"+
	"\u4BF0\u4BF1\u4BF2\u4BF3\u4BF4\u4BF5\u4BF6\u4BF7"+
	"\u4BF8\u4BF9\u4BFA\u4BFB\u4BFC\u4BFD\u4BFE\u4BFF"+
	"\u4C00\u4C01\u4C02\u4C03\u4C04\u4C05\u4C06\u4C07"+
	"\u4C08\u4C09\u4C0A\u4C0B\u4C0C\u4C0D\u4C0E\u4C0F"+
	"\u4C10\u4C11\u4C12\u4C13\u4C14\u4C15\u4C16\u4C17"+
	"\u4C18\u4C19\u4C1A\u4C1B\u4C1C\u4C1D\u4C1E\u4C1F"+
	"\u4C20\u4C21\u4C22\u4C23\u4C24\u4C25\u4C26\u4C27"+
	"\u4C28\u4C29\u4C2A\u4C2B\u4C2C\u4C2D\u4C2E\u4C2F"+
	"\u4C30\u4C31\u4C32\u4C33\u4C34\u4C35\u4C36\u4C37"+
	"\u4C38\u4C39\u4C3A\u4C3B\u4C3C\u4C3D\u4C3E\u4C3F"+
	"\u4C40\uFE50\u4C41\u4C42\uFE54\u4C43\u4C44\u4C45"+
	"\uFE57\u4C46\u4C47\uFE58\uFE5D\u4C48\u4C49\u4C4A"+
	"\u4C4B\u4C4C\u4C4D\u4C4E\u4C4F\u4C50\u4C51\uFE5E"+
	"\u4C52\u4C53\u4C54\u4C55\u4C56\u4C57\u4C58\u4C59"+
	"\u4C5A\u4C5B\u4C5C\u4C5D\u4C5E\u4C5F\u4C60\uFE6B"+
	"\u4C61\u4C62\uFE6E\u4C63\u4C64\u4C65\uFE71\u4C66"+
	"\u4C67\u4C68\u4C69\uFE73\u4C6A\u4C6B\uFE74\uFE75"+
	"\u4C6C\u4C6D\u4C6E\uFE79\u4C6F\u4C70\u4C71\u4C72"+
	"\u4C73\u4C74\u4C75\u4C76\u4C77\u4C78\u4C79\u4C7A"+
	"\u4C7B\u4C7C\uFE84\u4C7D\u4C7E\u4C7F\u4C80\u4C81"+
	"\u4C82\u4C83\u4C84\u4C85\u4C86\u4C87\u4C88\u4C89"+
	"\u4C8A\u4C8B\u4C8C\u4C8D\u4C8E\u4C8F\u4C90\u4C91"+
	"\u4C92\u4C93\u4C94\u4C95\u4C96\u4C97\u4C98\u4C99"+
	"\u4C9A\u4C9B\u4C9C\u4C9D\u4C9E\u4C9F\u4CA0\u4CA1"+
	"\u4CA2\u4CA3\u4CA4\u4CA5\u4CA6\u4CA7\u4CA8\u4CA9"+
	"\u4CAA\u4CAB\u4CAC\u4CAD\u4CAE\u4CAF\u4CB0\u4CB1"+
	"\u4CB2\u4CB3\u4CB4\u4CB5\u4CB6\u4CB7\u4CB8\u4CB9"+
	"\u4CBA\u4CBB\u4CBC\u4CBD\u4CBE\u4CBF\u4CC0\u4CC1"+
	"\u4CC2\u4CC3\u4CC4\u4CC5\u4CC6\u4CC7\u4CC8\u4CC9"+
	"\u4CCA\u4CCB\u4CCC\u4CCD\u4CCE\u4CCF\u4CD0\u4CD1"+
	"\u4CD2\u4CD3\u4CD4\u4CD5\u4CD6\u4CD7\u4CD8\u4CD9"+
	"\u4CDA\u4CDB\u4CDC\u4CDD\u4CDE\u4CDF\u4CE0\u4CE1"+
	"\u4CE2\u4CE3\u4CE4\u4CE5\u4CE6\u4CE7\u4CE8\u4CE9"+
	"\u4CEA\u4CEB\u4CEC\u4CED\u4CEE\u4CEF\u4CF0\u4CF1"+
	"\u4CF2\u4CF3\u4CF4\u4CF5\u4CF6\u4CF7\u4CF8\u4CF9"+
	"\u4CFA\u4CFB\u4CFC\u4CFD\u4CFE\u4CFF\u4D00\u4D01"+
	"\u4D02\u4D03\u4D04\u4D05\u4D06\u4D07\u4D08\u4D09"+
	"\u4D0A\u4D0B\u4D0C\u4D0D\u4D0E\u4D0F\u4D10\u4D11"+
	"\u4D12\u4D13\u4D14\u4D15\u4D16\u4D17\u4D18\u4D19"+
	"\u4D1A\u4D1B\u4D1C\u4D1D\u4D1E\u4D1F\u4D20\u4D21"+
	"\u4D22\u4D23\u4D24\u4D25\u4D26\u4D27\u4D28\u4D29"+
	"\u4D2A\u4D2B\u4D2C\u4D2D\u4D2E\u4D2F\u4D30\u4D31"+
	"\u4D32\u4D33\u4D34\u4D35\u4D36\u4D37\u4D38\u4D39"+
	"\u4D3A\u4D3B\u4D3C\u4D3D\u4D3E\u4D3F\u4D40\u4D41"+
	"\u4D42\u4D43\u4D44\u4D45\u4D46\u4D47\u4D48\u4D49"+
	"\u4D4A\u4D4B\u4D4C\u4D4D\u4D4E\u4D4F\u4D50\u4D51"+
	"\u4D52\u4D53\u4D54\u4D55\u4D56\u4D57\u4D58\u4D59"+
	"\u4D5A\u4D5B\u4D5C\u4D5D\u4D5E\u4D5F\u4D60\u4D61"+
	"\u4D62\u4D63\u4D64\u4D65\u4D66\u4D67\u4D68\u4D69"+
	"\u4D6A\u4D6B\u4D6C\u4D6D\u4D6E\u4D6F\u4D70\u4D71"+
	"\u4D72\u4D73\u4D74\u4D75\u4D76\u4D77\u4D78\u4D79"+
	"\u4D7A\u4D7B\u4D7C\u4D7D\u4D7E\u4D7F\u4D80\u4D81"+
	"\u4D82\u4D83\u4D84\u4D85\u4D86\u4D87\u4D88\u4D89"+
	"\u4D8A\u4D8B\u4D8C\u4D8D\u4D8E\u4D8F\u4D90\u4D91"+
	"\u4D92\u4D93\u4D94\u4D95\u4D96\u4D97\u4D98\u4D99"+
	"\u4D9A\u4D9B\u4D9C\u4D9D\u4D9E\u4D9F\u4DA0\u4DA1"+
	"\uA98A\uA98B\uA98C\uA98D\uA98E\uA98F\uA990\uA991"+
	"\uA992\uA993\uA994\uA995\u4DA2\u4DA3\u4DA4\u4DA5";

    private final static String innerIndex3= 
	"\uA1A1\uA1A2\uA1A3\uA1A8\u4DA6\uA1A9\uA965\uA996"+
	"\uA1B4\uA1B5\uA1B6\uA1B7\uA1B8\uA1B9\uA1BA\uA1BB"+
	"\uA1BE\uA1BF\uA893\uA1FE\uA1B2\uA1B3\uA1BC\uA1BD"+
	"\u4DA7\u4DA8\u4DA9\u4DAA\u4DAB\uA894\uA895\u4DAC"+
	"\u4DAD\uA940\uA941\uA942\uA943\uA944\uA945\uA946"+
	"\uA947\uA948\u4DAE\u4DAF\u4DB0\u4DB1\u4DB2\u4DB3"+
	"\u4DB4\u4DB5\u4DB6\u4DB7\u4DB8\u4DB9\u4DBA\u4DBB"+
	"\u4DBC\u4DBD\u4DBE\u4DBF\u4DC0\u4DC1\uA989\u4DC2"+
	"\u4DC3\uA4A1\uA4A2\uA4A3\uA4A4\uA4A5\uA4A6\uA4A7"+
	"\uA4A8\uA4A9\uA4AA\uA4AB\uA4AC\uA4AD\uA4AE\uA4AF"+
	"\uA4B0\uA4B1\uA4B2\uA4B3\uA4B4\uA4B5\uA4B6\uA4B7"+
	"\uA4B8\uA4B9\uA4BA\uA4BB\uA4BC\uA4BD\uA4BE\uA4BF"+
	"\uA4C0\uA4C1\uA4C2\uA4C3\uA4C4\uA4C5\uA4C6\uA4C7"+
	"\uA4C8\uA4C9\uA4CA\uA4CB\uA4CC\uA4CD\uA4CE\uA4CF"+
	"\uA4D0\uA4D1\uA4D2\uA4D3\uA4D4\uA4D5\uA4D6\uA4D7"+
	"\uA4D8\uA4D9\uA4DA\uA4DB\uA4DC\uA4DD\uA4DE\uA4DF"+
	"\uA4E0\uA4E1\uA4E2\uA4E3\uA4E4\uA4E5\uA4E6\uA4E7"+
	"\uA4E8\uA4E9\uA4EA\uA4EB\uA4EC\uA4ED\uA4EE\uA4EF"+
	"\uA4F0\uA4F1\uA4F2\uA4F3\u4DC4\u4DC5\u4DC6\u4DC7"+
	"\u4DC8\u4DC9\u4DCA\uA961\uA962\uA966\uA967\u4DCB"+
	"\u4DCC\uA5A1\uA5A2\uA5A3\uA5A4\uA5A5\uA5A6\uA5A7"+
	"\uA5A8\uA5A9\uA5AA\uA5AB\uA5AC\uA5AD\uA5AE\uA5AF"+
	"\uA5B0\uA5B1\uA5B2\uA5B3\uA5B4\uA5B5\uA5B6\uA5B7"+
	"\uA5B8\uA5B9\uA5BA\uA5BB\uA5BC\uA5BD\uA5BE\uA5BF"+
	"\uA5C0\uA5C1\uA5C2\uA5C3\uA5C4\uA5C5\uA5C6\uA5C7"+
	"\uA5C8\uA5C9\uA5CA\uA5CB\uA5CC\uA5CD\uA5CE\uA5CF"+
	"\uA5D0\uA5D1\uA5D2\uA5D3\uA5D4\uA5D5\uA5D6\uA5D7"+
	"\uA5D8\uA5D9\uA5DA\uA5DB\uA5DC\uA5DD\uA5DE\uA5DF"+
	"\uA5E0\uA5E1\uA5E2\uA5E3\uA5E4\uA5E5\uA5E6\uA5E7"+
	"\uA5E8\uA5E9\uA5EA\uA5EB\uA5EC\uA5ED\uA5EE\uA5EF"+
	"\uA5F0\uA5F1\uA5F2\uA5F3\uA5F4\uA5F5\uA5F6\u4DCD"+
	"\u4DCE\u4DCF\u4DD0\u4DD1\uA960\uA963\uA964\u4DD2"+
	"\u4DD3\u4DD4\u4DD5\u4DD6\u4DD7\uA8C5\uA8C6\uA8C7"+
	"\uA8C8\uA8C9\uA8CA\uA8CB\uA8CC\uA8CD\uA8CE\uA8CF"+
	"\uA8D0\uA8D1\uA8D2\uA8D3\uA8D4\uA8D5\uA8D6\uA8D7"+
	"\uA8D8\uA8D9\uA8DA\uA8DB\uA8DC\uA8DD\uA8DE\uA8DF"+
	"\uA8E0\uA8E1\uA8E2\uA8E3\uA8E4\uA8E5\uA8E6\uA8E7"+
	"\uA8E8\uA8E9\u4DD8\u4DD9\u4DDA\u4DDB\u4DDC\u4DDD"+
	"\u4DDE\u4DDF\u4DE0\u4DE1\u4DE2\u4DE3\u4DE4\u4DE5"+
	"\u4DE6\u4DE7\u4DE8\u4DE9\u4DEA\u4DEB\u4DEC\u4DED"+
	"\u4DEE\u4DEF\u4DF0\u4DF1\u4DF2\u4DF3\u4DF4\u4DF5"+
	"\u4DF6\u4DF7\u4DF8\u4DF9\u4DFA\u4DFB\u4DFC\u4DFD"+
	"\u4DFE\u4DFF\u4E00\u4E01\u4E02\u4E03\u4E04\u4E05"+
	"\u4E06\u4E07\u4E08\u4E09\u4E0A\u4E0B\u4E0C\u4E0D"+
	"\u4E0E\u4E0F\u4E10\u4E11\u4E12\u4E13\u4E14\u4E15"+
	"\u4E16\u4E17\u4E18\u4E19\u4E1A\u4E1B\u4E1C\u4E1D"+
	"\u4E1E\u4E1F\u4E20\u4E21\u4E22\u4E23\u4E24\u4E25"+
	"\u4E26\u4E27\u4E28\u4E29\u4E2A\u4E2B\u4E2C\u4E2D"+
	"\u4E2E\u4E2F\u4E30\u4E31\u4E32\u4E33\u4E34\u4E35"+
	"\u4E36\u4E37\u4E38\u4E39\u4E3A\u4E3B\u4E3C\u4E3D"+
	"\u4E3E\u4E3F\u4E40\u4E41\u4E42\u4E43\u4E44\u4E45"+
	"\u4E46\u4E47\u4E48\u4E49\u4E4A\u4E4B\u4E4C\u4E4D"+
	"\u4E4E\u4E4F\u4E50\u4E51\u4E52\u4E53\u4E54\u4E55"+
	"\u4E56\u4E57\u4E58\u4E59\u4E5A\u4E5B\u4E5C\u4E5D"+
	"\u4E5E\u4E5F\u4E60\u4E61\u4E62\u4E63\u4E64\u4E65"+
	"\u4E66\u4E67\u4E68\u4E69\u4E6A\u4E6B\u4E6C\u4E6D"+
	"\u4E6E\u4E6F\u4E70\u4E71\u4E72\u4E73\u4E74\u4E75"+
	"\u4E76\u4E77\u4E78\u4E79\u4E7A\u4E7B\u4E7C\u4E7D"+
	"\u4E7E\u4E7F\u4E80\u4E81\u4E82\u4E83\u4E84\u4E85"+
	"\u4E86\u4E87\u4E88\u4E89\u4E8A\u4E8B\u4E8C\u4E8D"+
	"\u4E8E\u4E8F\u4E90\u4E91\u4E92\u4E93\u4E94\u4E95"+
	"\u4E96\u4E97\u4E98\u4E99\u4E9A\u4E9B\u4E9C\u4E9D"+
	"\u4E9E\u4E9F\u4EA0\u4EA1\u4EA2\u4EA3\u4EA4\u4EA5"+
	"\u4EA6\u4EA7\u4EA8\u4EA9\u4EAA\u4EAB\u4EAC\u4EAD"+
	"\u4EAE\u4EAF\u4EB0\u4EB1\u4EB2\u4EB3\u4EB4\u4EB5"+
	"\u4EB6\u4EB7\u4EB8\u4EB9\u4EBA\u4EBB\u4EBC\u4EBD"+
	"\u4EBE\u4EBF\u4EC0\u4EC1\u4EC2\u4EC3\u4EC4\u4EC5"+
	"\u4EC6\u4EC7\u4EC8\u4EC9\u4ECA\u4ECB\u4ECC\u4ECD"+
	"\uA2E5\uA2E6\uA2E7\uA2E8\uA2E9\uA2EA\uA2EB\uA2EC"+
	"\uA2ED\uA2EE\u4ECE\u4ECF\u4ED0\u4ED1\u4ED2\u4ED3"+
	"\u4ED4\uA95A\u4ED5\u4ED6\u4ED7\u4ED8\u4ED9\u4EDA"+
	"\u4EDB\u4EDC\u4EDD\u4EDE\u4EDF\u4EE0\u4EE1\u4EE2"+
	"\u4EE3\u4EE4\u4EE5\u4EE6\u4EE7\u4EE8\u4EE9\u4EEA"+
	"\u4EEB\u4EEC\u4EED\u4EEE\u4EEF\u4EF0\u4EF1\u4EF2"+
	"\u4EF3\u4EF4\u4EF5\u4EF6\u4EF7\u4EF8\u4EF9\u4EFA"+
	"\u4EFB\u4EFC\u4EFD\u4EFE\u4EFF\u4F00\u4F01\u4F02"+
	"\u4F03\u4F04\u4F05\u4F06\u4F07\u4F08\u4F09\u4F0A"+
	"\u4F0B\u4F0C\u4F0D\u4F0E\u4F0F\u4F10\u4F11\u4F12"+
	"\u4F13\u4F14\u4F15\u4F16\u4F17\u4F18\u4F19\u4F1A"+
	"\u4F1B\u4F1C\u4F1D\u4F1E\u4F1F\u4F20\u4F21\u4F22"+
	"\u4F23\u4F24\u4F25\u4F26\u4F27\u4F28\u4F29\u4F2A"+
	"\u4F2B\u4F2C\u4F2D\u4F2E\u4F2F\u4F30\u4F31\u4F32"+
	"\u4F33\u4F34\u4F35\u4F36\u4F37\u4F38\u4F39\u4F3A"+
	"\u4F3B\u4F3C\u4F3D\u4F3E\u4F3F\u4F40\u4F41\u4F42"+
	"\u4F43\u4F44\u4F45\uA949\u4F46\u4F47\u4F48\u4F49"+
	"\u4F4A\u4F4B\u4F4C\u4F4D\u4F4E\u4F4F\u4F50\u4F51"+
	"\u4F52\u4F53\u4F54\u4F55\u4F56\u4F57\u4F58\u4F59"+
	"\u4F5A\u4F5B\u4F5C\u4F5D\u4F5E\u4F5F\u4F60\u4F61"+
	"\u4F62\u4F63\u4F64\u4F65\u4F66\u4F67\u4F68\u4F69"+
	"\u4F6A\u4F6B\u4F6C\u4F6D\u4F6E\u4F6F\u4F70\u4F71"+
	"\u4F72\u4F73\u4F74\u4F75\u4F76\u4F77\u4F78\u4F79"+
	"\u4F7A\u4F7B\u4F7C\u4F7D\u4F7E\u4F7F\u4F80\u4F81"+
	"\u4F82\u4F83\u4F84\u4F85\u4F86\u4F87\u4F88\u4F89"+
	"\u4F8A\u4F8B\u4F8C\u4F8D\u4F8E\u4F8F\u4F90\u4F91"+
	"\u4F92\u4F93\u4F94\u4F95\u4F96\u4F97\u4F98\u4F99"+
	"\u4F9A\u4F9B\u4F9C\u4F9D\u4F9E\u4F9F\u4FA0\u4FA1"+
	"\u4FA2\u4FA3\u4FA4\u4FA5\u4FA6\u4FA7\u4FA8\u4FA9"+
	"\u4FAA\u4FAB\u4FAC\u4FAD\u4FAE\u4FAF\u4FB0\u4FB1"+
	"\u4FB2\u4FB3\u4FB4\u4FB5\u4FB6\u4FB7\u4FB8\u4FB9"+
	"\u4FBA\u4FBB\u4FBC\u4FBD\u4FBE\u4FBF\u4FC0\u4FC1"+
	"\u4FC2\u4FC3\u4FC4\u4FC5\u4FC6\u4FC7\u4FC8\u4FC9"+
	"\u4FCA\u4FCB\u4FCC\u4FCD\u4FCE\u4FCF\u4FD0\u4FD1"+
	"\u4FD2\u4FD3\u4FD4\u4FD5\u4FD6\u4FD7\u4FD8\u4FD9"+
	"\u4FDA\u4FDB\u4FDC\u4FDD\u4FDE\u4FDF\u4FE0\u4FE1"+
	"\u4FE2\u4FE3\u4FE4\u4FE5\u4FE6\u4FE7\u4FE8\u4FE9"+
	"\u4FEA\u4FEB\u4FEC\u4FED\u4FEE\u4FEF\u4FF0\u4FF1"+
	"\u4FF2\u4FF3\u4FF4\u4FF5\u4FF6\u4FF7\u4FF8\u4FF9"+
	"\u4FFA\u4FFB\u4FFC\u4FFD\u4FFE\u4FFF\u5000\u5001"+
	"\u5002\u5003\u5004\u5005\u5006\u5007\u5008\u5009"+
	"\u500A\u500B\u500C\u500D\u500E\u500F\u5010\u5011"+
	"\u5012\u5013\u5014\u5015\u5016\u5017\u5018\u5019"+
	"\u501A\u501B\u501C\u501D\u501E\u501F\u5020\u5021"+
	"\u5022\u5023\u5024\u5025\u5026\u5027\u5028\u5029"+
	"\u502A\u502B\u502C\u502D\u502E\u502F\uA94A\uA94B"+
	"\u5030\u5031\u5032\u5033\u5034\u5035\u5036\u5037"+
	"\u5038\u5039\u503A\u503B\uA94C\uA94D\uA94E\u503C"+
	"\u503D\uA94F\u503E\u503F\u5040\u5041\u5042\u5043"+
	"\u5044\u5045\u5046\u5047\u5048\u5049\u504A\u504B"+
	"\u504C\u504D\u504E\u504F\u5050\u5051\u5052\u5053"+
	"\u5054\u5055\u5056\u5057\u5058\u5059\u505A\u505B"+
	"\u505C\u505D\u505E\u505F\uA950\u5060\u5061\u5062"+
	"\u5063\u5064\u5065\u5066\u5067\u5068\uA951\u5069"+
	"\u506A\uA952\uA953\u506B\u506C\uA954\u506D\u506E"+
	"\u506F\u5070\u5071\u5072\u5073\u5074\u5075\u5076"+
	"\u5077\u5078\u5079\u507A\u507B\u507C\u507D\u507E"+
	"\u507F\u5080\u5081\u5082\u5083\u5084\u5085\u5086"+
	"\u5087\u5088\u5089\u508A\u508B\u508C\u508D\u508E"+
	"\u508F\u5090\u5091\u5092\u5093\u5094\u5095\u5096"+
	"\u5097\u5098\u5099\u509A\u509B\u509C\u509D\u509E"+
	"\u509F\u50A0\u50A1\u50A2\u50A3\u50A4\u50A5\u50A6"+
	"\u50A7\u50A8\u50A9\u50AA\u50AB\u50AC\u50AD\u50AE"+
	"\u50AF\u50B0\u50B1\u50B2\u50B3\u50B4\u50B5\u50B6"+
	"\u50B7\u50B8\u50B9\u50BA\u50BB\u50BC\u50BD\u50BE"+
	"\u50BF\u50C0\u50C1\u50C2\u50C3\u50C4\u50C5\u50C6"+
	"\u50C7\u50C8\u50C9\u50CA\u50CB\u50CC\u50CD\u50CE"+
	"\u50CF\u50D0\u50D1\u50D2\u50D3\u50D4\u50D5\u50D6"+
	"\u50D7\u50D8\u50D9\u50DA\u50DB\u50DC\u50DD\uFE56"+
	"\u50DE\u50DF\u50E0\u50E1\u50E2\u50E3\u50E4\u50E5"+
	"\u50E6\u50E7\u50E8\u50E9\u50EA\u50EB\u50EC\u50ED"+
	"\u50EE\u50EF\u50F0\u50F1\u50F2\u50F3\u50F4\u50F5"+
	"\u50F6\u50F7\u50F8\u50F9\u50FA\u50FB\u50FC\u50FD"+
	"\u50FE\u50FF\u5100\u5101\u5102\u5103\u5104\u5105"+
	"\u5106\u5107\u5108\uFE55\u5109\u510A\u510B\u510C"+
	"\u510D\u510E\u510F\u5110\u5111\u5112\u5113\u5114"+
	"\u5115\u5116\u5117\u5118\u5119\u511A\u511B\u511C"+
	"\u511D\u511E\u511F\u5120\u5121\u5122\u5123\u5124"+
	"\u5125\u5126\u5127\u5128\u5129\u512A\u512B\u512C"+
	"\u512D\u512E\u512F\u5130\u5131\u5132\u5133\u5134"+
	"\u5135\u5136\u5137\u5138\u5139\u513A\u513B\u513C"+
	"\u513D\u513E\u513F\u5140\u5141\u5142\u5143\u5144"+
	"\u5145\u5146\u5147\u5148\u5149\u514A\u514B\u514C"+
	"\u514D\u514E\u514F\u5150\u5151\u5152\u5153\u5154"+
	"\u5155\u5156\u5157\u5158\u5159\u515A\u515B\u515C"+
	"\u515D\u515E\u515F\u5160\u5161\u5162\u5163\u5164"+
	"\u5165\u5166\u5167\u5168\u5169\u516A\u516B\u516C"+
	"\u516D\u516E\u516F\u5170\u5171\u5172\u5173\u5174"+
	"\u5175\u5176\u5177\u5178\u5179\u517A\u517B\u517C"+
	"\u517D\u517E\u517F\u5180\u5181\u5182\u5183\u5184"+
	"\u5185\u5186\u5187\u5188\u5189\u518A\u518B\u518C"+
	"\u518D\u518E\u518F\u5190\u5191\u5192\u5193\u5194"+
	"\u5195\u5196\u5197\u5198\u5199\u519A\u519B\u519C"+
	"\u519D\u519E\u519F\u51A0\u51A1\u51A2\u51A3\u51A4"+
	"\u51A5\u51A6\u51A7\u51A8\u51A9\u51AA\u51AB\u51AC"+
	"\u51AD\u51AE\u51AF\u51B0\u51B1\u51B2\u51B3\u51B4"+
	"\u51B5\u51B6\u51B7\u51B8\u51B9\u51BA\u51BB\u51BC"+
	"\u51BD\u51BE\u51BF\u51C0\u51C1\u51C2\u51C3\u51C4"+
	"\u51C5\u51C6\u51C7\u51C8\u51C9\u51CA\u51CB\u51CC"+
	"\u51CD\u51CE\u51CF\u51D0\u51D1\u51D2\u51D3\u51D4"+
	"\u51D5\u51D6\u51D7\u51D8\u51D9\u51DA\u51DB\u51DC"+
	"\u51DD\u51DE\u51DF\u51E0\u51E1\u51E2\u51E3\u51E4"+
	"\u51E5\u51E6\u51E7\u51E8\u51E9\u51EA\u51EB\u51EC"+
	"\u51ED\u51EE\u51EF\u51F0\u51F1\u51F2\u51F3\u51F4"+
	"\u51F5\u51F6\u51F7\u51F8\u51F9\u51FA\u51FB\u51FC"+
	"\u51FD\u51FE\u51FF\u5200\u5201\u5202\u5203\u5204"+
	"\u5205\u5206\u5207\u5208\u5209\u520A\u520B\u520C"+
	"\u520D\u520E\u520F\u5210\u5211\u5212\u5213\u5214"+
	"\u5215\u5216\u5217\u5218\u5219\u521A\u521B\u521C"+
	"\u521D\u521E\u521F\u5220\u5221\u5222\u5223\u5224"+
	"\u5225\u5226\u5227\u5228\u5229\u522A\u522B\u522C"+
	"\u522D\u522E\u522F\u5230\u5231\u5232\uFE5A\u5233"+
	"\u5234\u5235\u5236\u5237\u5238\u5239\u523A\u523B"+
	"\u523C\u523D\u523E\u523F\u5240\u5241\u5242\u5243"+
	"\u5244\u5245\u5246\u5247\u5248\u5249\u524A\u524B"+
	"\u524C\u524D\u524E\u524F\u5250\u5251\u5252\u5253"+
	"\u5254\u5255\u5256\u5257\u5258\u5259\u525A\u525B"+
	"\u525C\u525D\u525E\u525F\u5260\u5261\u5262\u5263"+
	"\u5264\u5265\u5266\u5267\u5268\u5269\u526A\u526B"+
	"\u526C\u526D\u526E\u526F\u5270\u5271\u5272\u5273"+
	"\u5274\u5275\u5276\u5277\u5278\u5279\u527A\u527B"+
	"\u527C\u527D\u527E\u527F\u5280\u5281\u5282\u5283"+
	"\u5284\u5285\u5286\u5287\u5288\u5289\u528A\u528B"+
	"\u528C\u528D\u528E\u528F\u5290\u5291\u5292\u5293"+
	"\u5294\u5295\u5296\u5297\u5298\u5299\u529A\u529B"+
	"\u529C\u529D\u529E\u529F\u52A0\u52A1\uFE5C\u52A2"+
	"\u52A3\u52A4\u52A5\u52A6\u52A7\u52A8\u52A9\u52AA"+
	"\u52AB\u52AC\uFE5B\u52AD\u52AE\u52AF\u52B0\u52B1"+
	"\u52B2\u52B3\u52B4\u52B5\u52B6\u52B7\u52B8\u52B9"+
	"\u52BA\u52BB\u52BC\u52BD\u52BE\u52BF\u52C0\u52C1"+
	"\u52C2\u52C3\u52C4\u52C5\u52C6\u52C7\u52C8\u52C9"+
	"\u52CA\u52CB\u52CC\u52CD\u52CE\u52CF\u52D0\u52D1"+
	"\u52D2\u52D3\u52D4\u52D5\u52D6\u52D7\u52D8\u52D9"+
	"\u52DA\u52DB\u52DC\u52DD\u52DE\u52DF\u52E0\u52E1"+
	"\u52E2\u52E3\u52E4\u52E5\u52E6\u52E7\u52E8\u52E9"+
	"\u52EA\u52EB\u52EC\u52ED\u52EE\u52EF\u52F0\u52F1"+
	"\u52F2\u52F3\u52F4\u52F5\u52F6\u52F7\u52F8\u52F9"+
	"\u52FA\u52FB\u52FC\u52FD\u52FE\u52FF\u5300\u5301"+
	"\u5302\u5303\u5304\u5305\u5306\u5307\u5308\u5309"+
	"\u530A\u530B\u530C\u530D\u530E\u530F\u5310\u5311"+
	"\u5312\u5313\u5314\u5315\u5316\u5317\u5318\u5319"+
	"\u531A\u531B\u531C\u531D\u531E\u531F\u5320\u5321"+
	"\u5322\u5323\u5324\u5325\u5326\u5327\u5328\u5329"+
	"\u532A\u532B\u532C\u532D\u532E\u532F\u5330\u5331"+
	"\u5332\u5333\u5334\u5335\u5336\u5337\u5338\u5339"+
	"\u533A\u533B\u533C\u533D\u533E\u533F\u5340\u5341"+
	"\u5342\u5343\u5344\u5345\u5346\u5347\u5348\u5349"+
	"\u534A\u534B\u534C\u534D\u534E\u534F\u5350\u5351"+
	"\u5352\u5353\u5354\u5355\u5356\u5357\u5358\u5359"+
	"\u535A\u535B\u535C\u535D\u535E\u535F\u5360\u5361"+
	"\u5362\u5363\u5364\u5365\u5366\u5367\u5368\u5369"+
	"\u536A\u536B\u536C\u536D\u536E\u536F\u5370\u5371"+
	"\u5372\u5373\u5374\u5375\u5376\u5377\u5378\u5379"+
	"\u537A\u537B\u537C\u537D\u537E\u537F\u5380\u5381"+
	"\u5382\u5383\u5384\u5385\u5386\u5387\u5388\u5389"+
	"\u538A\u538B\u538C\u538D\u538E\u538F\u5390\u5391"+
	"\u5392\u5393\u5394\u5395\u5396\u5397\u5398\u5399"+
	"\u539A\u539B\u539C\u539D\u539E\u539F\u53A0\u53A1"+
	"\u53A2\u53A3\u53A4\u53A5\u53A6\u53A7\u53A8\u53A9"+
	"\u53AA\u53AB\u53AC\u53AD\u53AE\u53AF\u53B0\u53B1"+
	"\u53B2\u53B3\u53B4\u53B5\u53B6\u53B7\u53B8\u53B9"+
	"\u53BA\u53BB\u53BC\u53BD\u53BE\u53BF\u53C0\u53C1"+
	"\u53C2\u53C3\u53C4\u53C5\u53C6\u53C7\u53C8\u53C9"+
	"\u53CA\u53CB\u53CC\u53CD\u53CE\u53CF\u53D0\u53D1"+
	"\u53D2\u53D3\u53D4\u53D5\u53D6\u53D7\u53D8\u53D9"+
	"\u53DA\u53DB\u53DC\u53DD\u53DE\u53DF\u53E0\u53E1"+
	"\u53E2\u53E3\u53E4\u53E5\u53E6\u53E7\u53E8\u53E9"+
	"\u53EA\u53EB\u53EC\u53ED\u53EE\u53EF\u53F0\u53F1"+
	"\u53F2\u53F3\u53F4\u53F5\u53F6\u53F7\u53F8\u53F9"+
	"\u53FA\u53FB\u53FC\u53FD\u53FE\u53FF\u5400\u5401"+
	"\u5402\u5403\u5404\u5405\u5406\u5407\u5408\u5409"+
	"\u540A\u540B\u540C\u540D\u540E\u540F\u5410\u5411"+
	"\u5412\u5413\u5414\u5415\u5416\u5417\u5418\u5419"+
	"\u541A\u541B\u541C\u541D\u541E\u541F\u5420\u5421"+
	"\u5422\u5423\u5424\u5425\u5426\u5427\u5428\u5429"+
	"\u542A\u542B\u542C\u542D\u542E\u542F\u5430\u5431"+
	"\u5432\u5433\u5434\u5435\u5436\u5437\u5438\u5439"+
	"\u543A\u543B\u543C\u543D\u543E\u543F\u5440\u5441"+
	"\u5442\u5443\u5444\u5445\u5446\u5447\u5448\u5449"+
	"\u544A\u544B\u544C\u544D\u544E\u544F\u5450\u5451"+
	"\u5452\u5453\u5454\u5455\u5456\u5457\u5458\u5459"+
	"\u545A\u545B\u545C\u545D\u545E\u545F\u5460\u5461"+
	"\u5462\u5463\u5464\u5465\u5466\u5467\u5468\u5469"+
	"\u546A\u546B\u546C\u546D\u546E\u546F\u5470\u5471"+
	"\u5472\u5473\u5474\u5475\u5476\u5477\u5478\u5479"+
	"\u547A\u547B\u547C\u547D\u547E\u547F\u5480\u5481"+
	"\u5482\u5483\u5484\u5485\u5486\u5487\u5488\u5489"+
	"\u548A\u548B\u548C\u548D\u548E\u548F\u5490\u5491"+
	"\u5492\u5493\u5494\u5495\u5496\u5497\u5498\u5499"+
	"\u549A\u549B\u549C\u549D\u549E\u549F\u54A0\u54A1"+
	"\u54A2\u54A3\u54A4\u54A5\u54A6\u54A7\u54A8\u54A9"+
	"\u54AA\u54AB\u54AC\u54AD\u54AE\u54AF\u54B0\u54B1"+
	"\u54B2\u54B3\u54B4\u54B5\u54B6\u54B7\u54B8\u54B9"+
	"\u54BA\u54BB\u54BC\u54BD\u54BE\u54BF\u54C0\u54C1"+
	"\u54C2\u54C3\u54C4\u54C5\u54C6\u54C7\u54C8\u54C9"+
	"\u54CA\u54CB\u54CC\u54CD\u54CE\u54CF\u54D0\u54D1"+
	"\u54D2\u54D3\u54D4\u54D5\u54D6\u54D7\u54D8\u54D9"+
	"\u54DA\u54DB\u54DC\u54DD\u54DE\u54DF\u54E0\u54E1"+
	"\u54E2\u54E3\u54E4\u54E5\u54E6\u54E7\u54E8\u54E9"+
	"\u54EA\u54EB\u54EC\u54ED\u54EE\u54EF\u54F0\u54F1"+
	"\u54F2\u54F3\u54F4\u54F5\u54F6\u54F7\u54F8\u54F9"+
	"\u54FA\u54FB\u54FC\u54FD\u54FE\u54FF\u5500\u5501"+
	"\u5502\u5503\u5504\u5505\u5506\u5507\u5508\u5509"+
	"\u550A\u550B\u550C\u550D\u550E\u550F\u5510\u5511"+
	"\u5512\u5513\u5514\u5515\u5516\u5517\u5518\u5519"+
	"\u551A\u551B\u551C\u551D\u551E\u551F\u5520\u5521"+
	"\u5522\u5523\u5524\u5525\u5526\u5527\u5528\u5529"+
	"\u552A\u552B\u552C\u552D\u552E\u552F\u5530\u5531"+
	"\u5532\u5533\u5534\u5535\u5536\u5537\u5538\u5539"+
	"\u553A\u553B\u553C\u553D\u553E\u553F\u5540\u5541"+
	"\u5542\u5543\u5544\u5545\u5546\u5547\u5548\u5549"+
	"\u554A\u554B\u554C\u554D\u554E\u554F\u5550\u5551"+
	"\u5552\u5553\u5554\u5555\u5556\u5557\u5558\u5559"+
	"\u555A\u555B\u555C\u555D\u555E\u555F\u5560\u5561"+
	"\u5562\u5563\u5564\u5565\u5566\u5567\u5568\u5569"+
	"\u556A\u556B\u556C\u556D\u556E\u556F\u5570\u5571"+
	"\u5572\u5573\u5574\u5575\u5576\u5577\u5578\u5579"+
	"\u557A\u557B\u557C\u557D\u557E\u557F\u5580\u5581"+
	"\u5582\u5583\u5584\u5585\u5586\u5587\u5588\u5589"+
	"\u558A\u558B\u558C\u558D\u558E\u558F\u5590\u5591"+
	"\u5592\u5593\u5594\u5595\u5596\u5597\u5598\u5599"+
	"\u559A\u559B\u559C\u559D\u559E\u559F\u55A0\u55A1"+
	"\u55A2\u55A3\u55A4\u55A5\u55A6\u55A7\u55A8\u55A9"+
	"\uFE60\u55AA\u55AB\u55AC\u55AD\u55AE\u55AF\u55B0"+
	"\u55B1\u55B2\u55B3\u55B4\u55B5\u55B6\u55B7\u55B8"+
	"\u55B9\u55BA\u55BB\u55BC\u55BD\u55BE\u55BF\u55C0"+
	"\u55C1\u55C2\u55C3\u55C4\u55C5\u55C6\u55C7\u55C8"+
	"\u55C9\u55CA\u55CB\u55CC\u55CD\u55CE\u55CF\u55D0"+
	"\u55D1\u55D2\u55D3\u55D4\u55D5\u55D6\u55D7\u55D8"+
	"\u55D9\u55DA\u55DB\u55DC\u55DD\u55DE\u55DF\u55E0"+
	"\u55E1\u55E2\u55E3\u55E4\u55E5\u55E6\u55E7\u55E8"+
	"\u55E9\u55EA\u55EB\u55EC\u55ED\u55EE\u55EF\u55F0"+
	"\u55F1\u55F2\u55F3\u55F4\u55F5\u55F6\u55F7\u55F8"+
	"\u55F9\u55FA\u55FB\u55FC\u55FD\u55FE\uFE5F\u55FF"+
	"\u5600\u5601\u5602\u5603\u5604\u5605\u5606\u5607"+
	"\u5608\u5609\u560A\u560B\u560C\u560D\u560E\u560F"+
	"\u5610\u5611\u5612\u5613\u5614\u5615\u5616\u5617"+
	"\u5618\u5619\u561A\u561B\u561C\u561D\u561E\u561F"+
	"\u5620\u5621\u5622\u5623\u5624\u5625\u5626\u5627"+
	"\u5628\u5629\u562A\u562B\u562C\u562D\u562E\u562F"+
	"\u5630\u5631\u5632\u5633\u5634\u5635\u5636\u5637"+
	"\u5638\u5639\u563A\u563B\u563C\u563D\u563E\u563F"+
	"\u5640\u5641\u5642\u5643\u5644\u5645\u5646\u5647"+
	"\u5648\u5649\u564A\u564B\u564C\u564D\u564E\u564F"+
	"\u5650\u5651\u5652\u5653\u5654\u5655\u5656\u5657"+
	"\u5658\u5659\u565A\u565B\u565C\u565D\u565E\uFE62"+
	"\uFE65\u565F\u5660\u5661\u5662\u5663\u5664\u5665"+
	"\u5666\u5667\u5668\u5669\u566A\u566B\u566C\uFE63"+
	"\u566D\u566E\u566F\u5670\u5671\u5672\u5673\u5674"+
	"\u5675\u5676\u5677\u5678\u5679\u567A\u567B\u567C"+
	"\u567D\u567E\u567F\u5680\u5681\u5682\u5683\u5684"+
	"\u5685\u5686\u5687\u5688\u5689\u568A\u568B\u568C"+
	"\u568D\u568E\u568F\u5690\u5691\u5692\u5693\u5694"+
	"\u5695\u5696\u5697\u5698\u5699\u569A\u569B\u569C"+
	"\u569D\u569E\u569F\u56A0\u56A1\u56A2\u56A3\u56A4"+
	"\u56A5\u56A6\u56A7\u56A8\u56A9\u56AA\u56AB\u56AC"+
	"\u56AD\u56AE\u56AF\u56B0\u56B1\u56B2\u56B3\u56B4"+
	"\u56B5\u56B6\u56B7\u56B8\u56B9\u56BA\u56BB\u56BC"+
	"\u56BD\u56BE\u56BF\u56C0\u56C1\u56C2\u56C3\u56C4"+
	"\u56C5\u56C6\u56C7\u56C8\u56C9\u56CA\u56CB\u56CC"+
	"\u56CD\u56CE\u56CF\u56D0\u56D1\u56D2\u56D3\u56D4"+
	"\u56D5\u56D6\u56D7\u56D8\u56D9\u56DA\u56DB\u56DC"+
	"\u56DD\u56DE\u56DF\u56E0\u56E1\u56E2\u56E3\u56E4"+
	"\u56E5\u56E6\u56E7\u56E8\u56E9\u56EA\u56EB\u56EC"+
	"\u56ED\u56EE\u56EF\u56F0\u56F1\u56F2\u56F3\u56F4"+
	"\u56F5\u56F6\u56F7\u56F8\u56F9\u56FA\u56FB\u56FC"+
	"\u56FD\u56FE\u56FF\uFE64\u5700\u5701\u5702\u5703"+
	"\u5704\u5705\u5706\u5707\u5708\u5709\u570A\u570B"+
	"\u570C\u570D\u570E\u570F\u5710\u5711\u5712\u5713"+
	"\u5714\u5715\u5716\u5717\u5718\u5719\u571A\u571B"+
	"\u571C\u571D\u571E\u571F\u5720\u5721\u5722\u5723"+
	"\u5724\u5725\u5726\u5727\u5728\u5729\u572A\u572B"+
	"\u572C\u572D\u572E\u572F\u5730\u5731\u5732\u5733"+
	"\u5734\u5735\u5736\u5737\u5738\u5739\u573A\u573B"+
	"\u573C\u573D\u573E\u573F\u5740\u5741\u5742\u5743"+
	"\u5744\u5745\u5746\u5747\u5748\u5749\u574A\u574B"+
	"\u574C\u574D\u574E\u574F\u5750\u5751\u5752\u5753"+
	"\u5754\u5755\u5756\u5757\u5758\u5759\u575A\u575B"+
	"\u575C\u575D\u575E\u575F\u5760\u5761\u5762\u5763"+
	"\u5764\u5765\u5766\u5767\u5768\u5769\u576A\u576B"+
	"\u576C\u576D\u576E\u576F\u5770\u5771\u5772\u5773"+
	"\u5774\u5775\u5776\u5777\u5778\u5779\u577A\u577B"+
	"\u577C\u577D\u577E\u577F\u5780\u5781\u5782\u5783"+
	"\u5784\u5785\u5786\u5787\u5788\u5789\u578A\u578B"+
	"\u578C\u578D\u578E\u578F\u5790\u5791\u5792\u5793"+
	"\u5794\u5795\u5796\u5797\u5798\u5799\u579A\u579B"+
	"\u579C\u579D\u579E\u579F\u57A0\u57A1\u57A2\u57A3"+
	"\u57A4\u57A5\u57A6\u57A7\u57A8\u57A9\u57AA\u57AB"+
	"\u57AC\u57AD\u57AE\u57AF\u57B0\u57B1\u57B2\u57B3"+
	"\u57B4\u57B5\u57B6\u57B7\u57B8\u57B9\u57BA\u57BB"+
	"\u57BC\u57BD\u57BE\u57BF\u57C0\u57C1\u57C2\u57C3"+
	"\u57C4\u57C5\u57C6\u57C7\u57C8\u57C9\u57CA\u57CB"+
	"\u57CC\u57CD\u57CE\u57CF\u57D0\u57D1\u57D2\u57D3"+
	"\u57D4\u57D5\u57D6\u57D7\u57D8\u57D9\uFE68\u57DA"+
	"\u57DB\u57DC\u57DD\u57DE\u57DF\u57E0\u57E1\u57E2"+
	"\u57E3\u57E4\u57E5\u57E6\u57E7\u57E8\u57E9\u57EA"+
	"\u57EB\u57EC\u57ED\u57EE\u57EF\u57F0\u57F1\u57F2"+
	"\u57F3\u57F4\u57F5\u57F6\u57F7\u57F8\u57F9\u57FA"+
	"\u57FB\u57FC\u57FD\u57FE\u57FF\u5800\u5801\u5802"+
	"\u5803\u5804\u5805\u5806\u5807\u5808\u5809\u580A"+
	"\u580B\u580C\u580D\u580E\u580F\u5810\u5811\u5812"+
	"\u5813\u5814\u5815\u5816\u5817\u5818\u5819\u581A"+
	"\u581B\u581C\u581D\u581E\u581F\u5820\u5821\u5822"+
	"\u5823\u5824\u5825\u5826\u5827\u5828\u5829\u582A"+
	"\u582B\u582C\u582D\u582E\u582F\u5830\u5831\u5832"+
	"\u5833\u5834\u5835\u5836\u5837\u5838\u5839\u583A"+
	"\u583B\u583C\u583D\u583E\u583F\u5840\u5841\u5842"+
	"\u5843\u5844\u5845\u5846\u5847\u5848\u5849\u584A"+
	"\u584B\u584C\u584D\u584E\u584F\u5850\u5851\u5852"+
	"\u5853\u5854\u5855\u5856\u5857\u5858\u5859\u585A"+
	"\u585B\u585C\u585D\u585E\u585F\u5860\u5861\u5862"+
	"\u5863\u5864\u5865\u5866\u5867\u5868\u5869\u586A"+
	"\u586B\u586C\u586D\u586E\u586F\u5870\u5871\u5872"+
	"\u5873\u5874\u5875\u5876\u5877\u5878\u5879\u587A"+
	"\u587B\u587C\u587D\u587E\u587F\u5880\u5881\u5882"+
	"\u5883\u5884\u5885\u5886\u5887\u5888\u5889\u588A"+
	"\u588B\u588C\u588D\u588E\u588F\u5890\u5891\u5892"+
	"\u5893\u5894\u5895\u5896\u5897\u5898\u5899\u589A"+
	"\u589B\u589C\u589D\u589E\u589F\u58A0\u58A1\u58A2"+
	"\u58A3\u58A4\u58A5\u58A6\u58A7\u58A8\u58A9\u58AA"+
	"\u58AB\u58AC\u58AD\u58AE\u58AF\u58B0\u58B1\u58B2"+
	"\u58B3\u58B4\u58B5\u58B6\u58B7\u58B8\u58B9\u58BA"+
	"\u58BB\u58BC\u58BD\u58BE\u58BF\u58C0\u58C1\u58C2"+
	"\u58C3\u58C4\u58C5\u58C6\u58C7\u58C8\u58C9\u58CA"+
	"\u58CB\u58CC\u58CD\u58CE\u58CF\u58D0\u58D1\u58D2"+
	"\u58D3\u58D4\u58D5\u58D6\u58D7\u58D8\u58D9\u58DA"+
	"\u58DB\u58DC\u58DD\u58DE\u58DF\u58E0\u58E1\u58E2"+
	"\u58E3\u58E4\u58E5\u58E6\u58E7\u58E8\u58E9\u58EA"+
	"\u58EB\u58EC\u58ED\u58EE\u58EF\u58F0\u58F1\u58F2"+
	"\u58F3\u58F4\u58F5\u58F6\u58F7\u58F8\uFE69\u58F9"+
	"\u58FA\u58FB\u58FC\u58FD\u58FE\u58FF\u5900\u5901"+
	"\u5902\u5903\u5904\u5905\u5906\u5907\u5908\u5909"+
	"\u590A\u590B\u590C\u590D\u590E\u590F\u5910\u5911"+
	"\u5912\u5913\u5914\u5915\u5916\u5917\u5918\u5919"+
	"\u591A\u591B\u591C\u591D\u591E\u591F\u5920\u5921"+
	"\u5922\u5923\u5924\u5925\u5926\u5927\u5928\u5929"+
	"\u592A\u592B\u592C\u592D\u592E\u592F\u5930\u5931"+
	"\u5932\u5933\u5934\u5935\u5936\u5937\u5938\u5939"+
	"\u593A\u593B\u593C\u593D\u593E\u593F\u5940\u5941"+
	"\u5942\u5943\u5944\u5945\u5946\u5947\u5948\u5949"+
	"\u594A\u594B\u594C\u594D\u594E\u594F\u5950\u5951"+
	"\u5952\u5953\u5954\u5955\u5956\u5957\u5958\u5959"+
	"\u595A\u595B\u595C\u595D\u595E\u595F\u5960\u5961"+
	"\u5962\u5963\u5964\u5965\u5966\u5967\u5968\u5969"+
	"\uFE6A\u596A\u596B\u596C\u596D\u596E\u596F\u5970"+
	"\u5971\u5972\u5973\u5974\u5975\u5976\u5977\u5978"+
	"\u5979\u597A\u597B\u597C\u597D\u597E\u597F\u5980"+
	"\u5981\u5982\u5983\u5984\u5985\u5986\u5987\u5988"+
	"\u5989\u598A\u598B\u598C\u598D\u598E\u598F\u5990"+
	"\u5991\u5992\u5993\u5994\u5995\u5996\u5997\u5998"+
	"\u5999\u599A\u599B\u599C\u599D\u599E\u599F\u59A0"+
	"\u59A1\u59A2\u59A3\u59A4\u59A5\u59A6\u59A7\u59A8"+
	"\u59A9\u59AA\u59AB\u59AC\u59AD\u59AE\u59AF\u59B0"+
	"\u59B1\u59B2\u59B3\u59B4\u59B5\u59B6\u59B7\u59B8"+
	"\u59B9\u59BA\u59BB\u59BC\u59BD\u59BE\u59BF\u59C0"+
	"\u59C1\u59C2\u59C3\u59C4\u59C5\u59C6\u59C7\u59C8"+
	"\u59C9\u59CA\u59CB\u59CC\u59CD\u59CE\u59CF\u59D0"+
	"\u59D1\u59D2\u59D3\u59D4\u59D5\u59D6\u59D7\u59D8"+
	"\u59D9\u59DA\u59DB\u59DC\u59DD\u59DE\u59DF\u59E0"+
	"\u59E1\u59E2\u59E3\u59E4\u59E5\u59E6\u59E7\u59E8"+
	"\u59E9\u59EA\u59EB\u59EC\u59ED\u59EE\u59EF\u59F0"+
	"\u59F1\u59F2\u59F3\u59F4\u59F5\u59F6\u59F7\u59F8"+
	"\u59F9\u59FA\u59FB\u59FC\u59FD\u59FE\u59FF\u5A00"+
	"\u5A01\u5A02\u5A03\u5A04\u5A05\u5A06\u5A07\u5A08"+
	"\u5A09\u5A0A\u5A0B\u5A0C\u5A0D\u5A0E\u5A0F\u5A10"+
	"\u5A11\u5A12\u5A13\u5A14\u5A15\u5A16\u5A17\u5A18"+
	"\u5A19\u5A1A\u5A1B\u5A1C\u5A1D\u5A1E\u5A1F\u5A20"+
	"\u5A21\u5A22\u5A23\u5A24\u5A25\u5A26\u5A27\u5A28"+
	"\u5A29\u5A2A\u5A2B\u5A2C\u5A2D\u5A2E\u5A2F\u5A30"+
	"\u5A31\u5A32\u5A33\u5A34\u5A35\u5A36\u5A37\u5A38"+
	"\u5A39\u5A3A\u5A3B\u5A3C\u5A3D\u5A3E\u5A3F\u5A40"+
	"\u5A41\u5A42\u5A43\u5A44\u5A45\u5A46\u5A47\u5A48"+
	"\u5A49\u5A4A\u5A4B\u5A4C\u5A4D\u5A4E\u5A4F\u5A50"+
	"\u5A51\u5A52\u5A53\u5A54\u5A55\u5A56\u5A57\u5A58"+
	"\u5A59\u5A5A\u5A5B\u5A5C\u5A5D\u5A5E\u5A5F\u5A60"+
	"\u5A61\u5A62\u5A63\u5A64\u5A65\u5A66\u5A67\u5A68"+
	"\u5A69\u5A6A\u5A6B\u5A6C\u5A6D\u5A6E\u5A6F\u5A70"+
	"\u5A71\u5A72\u5A73\u5A74\u5A75\u5A76\u5A77\u5A78"+
	"\u5A79\u5A7A\u5A7B\u5A7C\u5A7D\u5A7E\u5A7F\u5A80"+
	"\u5A81\u5A82\u5A83\u5A84\u5A85\u5A86\u5A87\u5A88"+
	"\u5A89\u5A8A\u5A8B\u5A8C\u5A8D\u5A8E\u5A8F\u5A90"+
	"\u5A91\u5A92\u5A93\u5A94\u5A95\u5A96\u5A97\u5A98"+
	"\u5A99\u5A9A\u5A9B\u5A9C\u5A9D\u5A9E\u5A9F\u5AA0"+
	"\u5AA1\u5AA2\u5AA3\u5AA4\u5AA5\u5AA6\u5AA7\u5AA8"+
	"\u5AA9\u5AAA\u5AAB\u5AAC\u5AAD\u5AAE\u5AAF\u5AB0"+
	"\u5AB1\u5AB2\u5AB3\u5AB4\u5AB5\u5AB6\u5AB7\u5AB8"+
	"\u5AB9\u5ABA\u5ABB\u5ABC\u5ABD\u5ABE\u5ABF\u5AC0"+
	"\u5AC1\u5AC2\u5AC3\u5AC4\u5AC5\u5AC6\u5AC7\u5AC8"+
	"\u5AC9\u5ACA\u5ACB\u5ACC\u5ACD\u5ACE\u5ACF\u5AD0"+
	"\u5AD1\u5AD2\u5AD3\u5AD4\u5AD5\u5AD6\u5AD7\u5AD8"+
	"\u5AD9\u5ADA\u5ADB\u5ADC\u5ADD\u5ADE\u5ADF\u5AE0"+
	"\u5AE1\u5AE2\u5AE3\u5AE4\u5AE5\u5AE6\u5AE7\u5AE8"+
	"\u5AE9\u5AEA\u5AEB\u5AEC\u5AED\u5AEE\u5AEF\u5AF0"+
	"\u5AF1\u5AF2\u5AF3\u5AF4\u5AF5\u5AF6\u5AF7\u5AF8"+
	"\u5AF9\u5AFA\u5AFB\u5AFC\u5AFD\u5AFE\u5AFF\u5B00"+
	"\u5B01\u5B02\u5B03\u5B04\u5B05\u5B06\u5B07\u5B08"+
	"\u5B09\u5B0A\u5B0B\u5B0C\u5B0D\u5B0E\u5B0F\u5B10"+
	"\u5B11\u5B12\u5B13\u5B14\u5B15\u5B16\u5B17\u5B18"+
	"\u5B19\u5B1A\u5B1B\u5B1C\u5B1D\u5B1E\u5B1F\u5B20"+
	"\u5B21\u5B22\u5B23\u5B24\u5B25\u5B26\u5B27\u5B28"+
	"\u5B29\u5B2A\u5B2B\u5B2C\u5B2D\u5B2E\u5B2F\u5B30"+
	"\u5B31\u5B32\u5B33\u5B34\u5B35\u5B36\u5B37\u5B38"+
	"\u5B39\u5B3A\u5B3B\u5B3C\u5B3D\u5B3E\u5B3F\u5B40"+
	"\u5B41\u5B42\u5B43\u5B44\u5B45\u5B46\u5B47\u5B48"+
	"\u5B49\u5B4A\u5B4B\u5B4C\u5B4D\u5B4E\u5B4F\u5B50"+
	"\u5B51\u5B52\u5B53\u5B54\u5B55\u5B56\u5B57\u5B58"+
	"\u5B59\u5B5A\u5B5B\u5B5C\u5B5D\u5B5E\u5B5F\u5B60"+
	"\u5B61\u5B62\u5B63\u5B64\u5B65\u5B66\u5B67\u5B68"+
	"\u5B69\u5B6A\u5B6B\u5B6C\u5B6D\u5B6E\u5B6F\u5B70"+
	"\u5B71\u5B72\u5B73\u5B74\u5B75\u5B76\u5B77\u5B78"+
	"\u5B79\u5B7A\u5B7B\u5B7C\u5B7D\u5B7E\u5B7F\u5B80"+
	"\u5B81\u5B82\u5B83\u5B84\u5B85\u5B86\u5B87\u5B88"+
	"\u5B89\u5B8A\u5B8B\u5B8C\u5B8D\u5B8E\u5B8F\u5B90"+
	"\u5B91\u5B92\u5B93\u5B94\u5B95\u5B96\u5B97\u5B98"+
	"\u5B99\u5B9A\u5B9B\u5B9C\u5B9D\u5B9E\u5B9F\u5BA0"+
	"\u5BA1\u5BA2\u5BA3\u5BA4\u5BA5\u5BA6\u5BA7\u5BA8"+
	"\u5BA9\u5BAA\u5BAB\u5BAC\u5BAD\u5BAE\u5BAF\u5BB0"+
	"\u5BB1\u5BB2\u5BB3\u5BB4\u5BB5\u5BB6\u5BB7\u5BB8"+
	"\u5BB9\u5BBA\u5BBB\u5BBC\u5BBD\u5BBE\u5BBF\u5BC0"+
	"\u5BC1\u5BC2\u5BC3\u5BC4\u5BC5\u5BC6\u5BC7\u5BC8"+
	"\u5BC9\u5BCA\u5BCB\u5BCC\u5BCD\u5BCE\u5BCF\u5BD0"+
	"\u5BD1\u5BD2\u5BD3\u5BD4\u5BD5\u5BD6\u5BD7\u5BD8"+
	"\u5BD9\u5BDA\u5BDB\u5BDC\u5BDD\u5BDE\u5BDF\u5BE0"+
	"\u5BE1\u5BE2\u5BE3\u5BE4\u5BE5\u5BE6\u5BE7\u5BE8"+
	"\u5BE9\u5BEA\u5BEB\u5BEC\u5BED\u5BEE\u5BEF\u5BF0"+
	"\u5BF1\u5BF2\u5BF3\u5BF4\u5BF5\u5BF6\u5BF7\u5BF8"+
	"\u5BF9\u5BFA\u5BFB\u5BFC\u5BFD\u5BFE\u5BFF\u5C00"+
	"\u5C01\u5C02\u5C03\u5C04\u5C05\u5C06\u5C07\u5C08"+
	"\u5C09\u5C0A\u5C0B\u5C0C\u5C0D\u5C0E\u5C0F\u5C10"+
	"\u5C11\u5C12\u5C13\u5C14\u5C15\u5C16\u5C17\u5C18"+
	"\u5C19\u5C1A\u5C1B\u5C1C\u5C1D\u5C1E\u5C1F\u5C20"+
	"\u5C21\u5C22\u5C23\u5C24\u5C25\u5C26\u5C27\u5C28"+
	"\u5C29\u5C2A\u5C2B\u5C2C\u5C2D\u5C2E\u5C2F\u5C30"+
	"\u5C31\u5C32\u5C33\u5C34\u5C35\u5C36\u5C37\u5C38"+
	"\u5C39\u5C3A\u5C3B\u5C3C\u5C3D\u5C3E\u5C3F\u5C40"+
	"\u5C41\u5C42\u5C43\u5C44\u5C45\u5C46\u5C47\u5C48"+
	"\u5C49\u5C4A\u5C4B\u5C4C\u5C4D\u5C4E\u5C4F\u5C50"+
	"\u5C51\u5C52\u5C53\u5C54\u5C55\u5C56\u5C57\u5C58"+
	"\u5C59\u5C5A\u5C5B\u5C5C\u5C5D\u5C5E\u5C5F\u5C60"+
	"\u5C61\u5C62\u5C63\u5C64\u5C65\u5C66\u5C67\u5C68"+
	"\u5C69\u5C6A\u5C6B\u5C6C\u5C6D\u5C6E\u5C6F\u5C70"+
	"\u5C71\u5C72\u5C73\u5C74\u5C75\u5C76\u5C77\u5C78"+
	"\u5C79\u5C7A\u5C7B\u5C7C\u5C7D\u5C7E\u5C7F\u5C80"+
	"\u5C81\u5C82\u5C83\u5C84\u5C85\u5C86\u5C87\u5C88";

    private final static String innerIndex4= 
	"\u5C89\u5C8A\u5C8B\u5C8C\u5C8D\u5C8E\u5C8F\u5C90"+
	"\u5C91\u5C92\u5C93\u5C94\u5C95\u5C96\u5C97\u5C98"+
	"\u5C99\u5C9A\u5C9B\u5C9C\u5C9D\u5C9E\u5C9F\u5CA0"+
	"\u5CA1\u5CA2\u5CA3\u5CA4\u5CA5\u5CA6\u5CA7\u5CA8"+
	"\u5CA9\u5CAA\u5CAB\u5CAC\u5CAD\u5CAE\u5CAF\u5CB0"+
	"\u5CB1\u5CB2\u5CB3\u5CB4\u5CB5\u5CB6\u5CB7\u5CB8"+
	"\u5CB9\u5CBA\u5CBB\u5CBC\u5CBD\u5CBE\u5CBF\u5CC0"+
	"\u5CC1\u5CC2\u5CC3\u5CC4\u5CC5\u5CC6\u5CC7\u5CC8"+
	"\u5CC9\u5CCA\u5CCB\u5CCC\u5CCD\u5CCE\u5CCF\u5CD0"+
	"\u5CD1\u5CD2\u5CD3\u5CD4\u5CD5\u5CD6\u5CD7\u5CD8"+
	"\u5CD9\u5CDA\u5CDB\u5CDC\u5CDD\u5CDE\uFE6F\u5CDF"+
	"\u5CE0\u5CE1\u5CE2\u5CE3\u5CE4\u5CE5\u5CE6\u5CE7"+
	"\u5CE8\u5CE9\u5CEA\u5CEB\u5CEC\u5CED\u5CEE\u5CEF"+
	"\u5CF0\u5CF1\u5CF2\u5CF3\u5CF4\u5CF5\u5CF6\u5CF7"+
	"\u5CF8\u5CF9\u5CFA\u5CFB\u5CFC\u5CFD\u5CFE\u5CFF"+
	"\u5D00\u5D01\u5D02\u5D03\u5D04\u5D05\u5D06\u5D07"+
	"\u5D08\u5D09\u5D0A\u5D0B\u5D0C\u5D0D\u5D0E\u5D0F"+
	"\u5D10\u5D11\u5D12\u5D13\u5D14\u5D15\u5D16\u5D17"+
	"\u5D18\u5D19\u5D1A\u5D1B\u5D1C\u5D1D\u5D1E\u5D1F"+
	"\u5D20\u5D21\u5D22\u5D23\u5D24\u5D25\u5D26\u5D27"+
	"\u5D28\u5D29\u5D2A\u5D2B\u5D2C\u5D2D\u5D2E\u5D2F"+
	"\u5D30\u5D31\u5D32\u5D33\u5D34\u5D35\u5D36\u5D37"+
	"\u5D38\u5D39\u5D3A\u5D3B\u5D3C\u5D3D\u5D3E\u5D3F"+
	"\u5D40\u5D41\u5D42\u5D43\u5D44\u5D45\u5D46\u5D47"+
	"\u5D48\u5D49\u5D4A\u5D4B\u5D4C\u5D4D\u5D4E\u5D4F"+
	"\u5D50\u5D51\u5D52\u5D53\u5D54\u5D55\u5D56\u5D57"+
	"\u5D58\u5D59\u5D5A\u5D5B\u5D5C\u5D5D\u5D5E\u5D5F"+
	"\u5D60\u5D61\u5D62\u5D63\u5D64\u5D65\u5D66\u5D67"+
	"\u5D68\u5D69\u5D6A\u5D6B\u5D6C\u5D6D\u5D6E\u5D6F"+
	"\u5D70\u5D71\u5D72\u5D73\u5D74\u5D75\u5D76\u5D77"+
	"\u5D78\u5D79\u5D7A\u5D7B\u5D7C\u5D7D\u5D7E\u5D7F"+
	"\u5D80\u5D81\u5D82\u5D83\u5D84\u5D85\u5D86\u5D87"+
	"\u5D88\u5D89\u5D8A\u5D8B\u5D8C\u5D8D\u5D8E\u5D8F"+
	"\u5D90\u5D91\u5D92\u5D93\u5D94\u5D95\u5D96\u5D97"+
	"\u5D98\u5D99\u5D9A\u5D9B\u5D9C\u5D9D\u5D9E\u5D9F"+
	"\u5DA0\u5DA1\u5DA2\u5DA3\u5DA4\u5DA5\u5DA6\u5DA7"+
	"\u5DA8\u5DA9\u5DAA\u5DAB\u5DAC\u5DAD\u5DAE\u5DAF"+
	"\u5DB0\u5DB1\u5DB2\u5DB3\u5DB4\u5DB5\u5DB6\u5DB7"+
	"\u5DB8\u5DB9\u5DBA\u5DBB\u5DBC\u5DBD\u5DBE\u5DBF"+
	"\u5DC0\u5DC1\u5DC2\u5DC3\u5DC4\u5DC5\u5DC6\u5DC7"+
	"\u5DC8\u5DC9\u5DCA\u5DCB\u5DCC\u5DCD\u5DCE\u5DCF"+
	"\u5DD0\u5DD1\u5DD2\u5DD3\u5DD4\u5DD5\u5DD6\u5DD7"+
	"\u5DD8\u5DD9\u5DDA\u5DDB\u5DDC\u5DDD\u5DDE\u5DDF"+
	"\u5DE0\u5DE1\u5DE2\u5DE3\u5DE4\u5DE5\u5DE6\uFE70"+
	"\u5DE7\u5DE8\u5DE9\u5DEA\u5DEB\u5DEC\u5DED\u5DEE"+
	"\u5DEF\u5DF0\u5DF1\u5DF2\u5DF3\u5DF4\u5DF5\u5DF6"+
	"\u5DF7\u5DF8\u5DF9\u5DFA\u5DFB\u5DFC\u5DFD\u5DFE"+
	"\u5DFF\u5E00\u5E01\u5E02\u5E03\u5E04\u5E05\u5E06"+
	"\u5E07\u5E08\u5E09\u5E0A\u5E0B\u5E0C\u5E0D\u5E0E"+
	"\u5E0F\u5E10\u5E11\u5E12\u5E13\u5E14\u5E15\u5E16"+
	"\u5E17\u5E18\u5E19\u5E1A\u5E1B\u5E1C\u5E1D\u5E1E"+
	"\u5E1F\u5E20\u5E21\u5E22\u5E23\u5E24\u5E25\u5E26"+
	"\u5E27\u5E28\u5E29\u5E2A\u5E2B\u5E2C\u5E2D\u5E2E"+
	"\u5E2F\u5E30\u5E31\u5E32\u5E33\u5E34\u5E35\u5E36"+
	"\u5E37\u5E38\u5E39\u5E3A\u5E3B\u5E3C\u5E3D\u5E3E"+
	"\u5E3F\u5E40\u5E41\u5E42\u5E43\u5E44\u5E45\u5E46"+
	"\u5E47\u5E48\u5E49\u5E4A\u5E4B\u5E4C\u5E4D\u5E4E"+
	"\u5E4F\u5E50\u5E51\u5E52\u5E53\u5E54\u5E55\u5E56"+
	"\u5E57\u5E58\u5E59\u5E5A\u5E5B\u5E5C\u5E5D\u5E5E"+
	"\u5E5F\u5E60\u5E61\u5E62\u5E63\u5E64\u5E65\u5E66"+
	"\u5E67\u5E68\u5E69\u5E6A\u5E6B\u5E6C\u5E6D\u5E6E"+
	"\u5E6F\u5E70\u5E71\u5E72\u5E73\u5E74\u5E75\u5E76"+
	"\u5E77\u5E78\u5E79\u5E7A\u5E7B\u5E7C\u5E7D\u5E7E"+
	"\u5E7F\u5E80\u5E81\u5E82\u5E83\u5E84\u5E85\u5E86"+
	"\u5E87\u5E88\u5E89\u5E8A\u5E8B\u5E8C\u5E8D\u5E8E"+
	"\u5E8F\u5E90\u5E91\u5E92\u5E93\u5E94\u5E95\u5E96"+
	"\u5E97\u5E98\u5E99\u5E9A\u5E9B\u5E9C\u5E9D\u5E9E"+
	"\u5E9F\u5EA0\u5EA1\u5EA2\u5EA3\u5EA4\u5EA5\u5EA6"+
	"\u5EA7\u5EA8\u5EA9\u5EAA\u5EAB\u5EAC\u5EAD\u5EAE"+
	"\u5EAF\u5EB0\u5EB1\u5EB2\u5EB3\u5EB4\u5EB5\u5EB6"+
	"\u5EB7\u5EB8\u5EB9\u5EBA\u5EBB\u5EBC\u5EBD\u5EBE"+
	"\u5EBF\u5EC0\u5EC1\u5EC2\u5EC3\u5EC4\u5EC5\u5EC6"+
	"\u5EC7\u5EC8\u5EC9\u5ECA\u5ECB\u5ECC\u5ECD\u5ECE"+
	"\u5ECF\u5ED0\u5ED1\u5ED2\u5ED3\u5ED4\u5ED5\u5ED6"+
	"\u5ED7\u5ED8\u5ED9\u5EDA\u5EDB\u5EDC\u5EDD\u5EDE"+
	"\u5EDF\u5EE0\u5EE1\u5EE2\u5EE3\u5EE4\u5EE5\u5EE6"+
	"\u5EE7\u5EE8\u5EE9\u5EEA\u5EEB\u5EEC\u5EED\u5EEE"+
	"\u5EEF\u5EF0\u5EF1\u5EF2\u5EF3\u5EF4\u5EF5\u5EF6"+
	"\u5EF7\u5EF8\u5EF9\u5EFA\u5EFB\u5EFC\u5EFD\u5EFE"+
	"\u5EFF\u5F00\u5F01\u5F02\u5F03\u5F04\u5F05\u5F06"+
	"\u5F07\u5F08\u5F09\u5F0A\u5F0B\u5F0C\u5F0D\u5F0E"+
	"\u5F0F\u5F10\u5F11\u5F12\u5F13\u5F14\u5F15\u5F16"+
	"\u5F17\u5F18\u5F19\u5F1A\u5F1B\u5F1C\u5F1D\u5F1E"+
	"\u5F1F\u5F20\u5F21\u5F22\u5F23\u5F24\u5F25\u5F26"+
	"\u5F27\u5F28\u5F29\u5F2A\u5F2B\u5F2C\u5F2D\u5F2E"+
	"\u5F2F\u5F30\u5F31\u5F32\u5F33\u5F34\u5F35\u5F36"+
	"\u5F37\u5F38\u5F39\u5F3A\u5F3B\u5F3C\u5F3D\u5F3E"+
	"\u5F3F\u5F40\u5F41\u5F42\u5F43\u5F44\u5F45\u5F46"+
	"\u5F47\u5F48\u5F49\u5F4A\u5F4B\u5F4C\u5F4D\u5F4E"+
	"\u5F4F\u5F50\u5F51\u5F52\u5F53\u5F54\u5F55\u5F56"+
	"\u5F57\u5F58\u5F59\u5F5A\u5F5B\u5F5C\u5F5D\u5F5E"+
	"\u5F5F\u5F60\u5F61\u5F62\u5F63\u5F64\u5F65\u5F66"+
	"\u5F67\u5F68\u5F69\u5F6A\u5F6B\u5F6C\u5F6D\u5F6E"+
	"\u5F6F\u5F70\u5F71\u5F72\u5F73\u5F74\u5F75\u5F76"+
	"\u5F77\u5F78\u5F79\u5F7A\u5F7B\u5F7C\u5F7D\u5F7E"+
	"\u5F7F\u5F80\u5F81\u5F82\u5F83\u5F84\u5F85\u5F86"+
	"\u5F87\u5F88\u5F89\u5F8A\u5F8B\u5F8C\u5F8D\u5F8E"+
	"\u5F8F\u5F90\u5F91\u5F92\u5F93\u5F94\u5F95\u5F96"+
	"\u5F97\u5F98\u5F99\u5F9A\u5F9B\u5F9C\u5F9D\u5F9E"+
	"\u5F9F\u5FA0\u5FA1\u5FA2\u5FA3\u5FA4\u5FA5\u5FA6"+
	"\u5FA7\u5FA8\u5FA9\u5FAA\u5FAB\u5FAC\u5FAD\u5FAE"+
	"\u5FAF\u5FB0\u5FB1\u5FB2\u5FB3\u5FB4\u5FB5\u5FB6"+
	"\u5FB7\u5FB8\u5FB9\u5FBA\u5FBB\u5FBC\u5FBD\uFE72"+
	"\u5FBE\u5FBF\u5FC0\u5FC1\u5FC2\u5FC3\u5FC4\u5FC5"+
	"\u5FC6\u5FC7\u5FC8\u5FC9\u5FCA\u5FCB\u5FCC\u5FCD"+
	"\u5FCE\u5FCF\u5FD0\u5FD1\u5FD2\u5FD3\u5FD4\u5FD5"+
	"\u5FD6\u5FD7\u5FD8\u5FD9\u5FDA\u5FDB\u5FDC\u5FDD"+
	"\u5FDE\u5FDF\u5FE0\u5FE1\u5FE2\u5FE3\u5FE4\u5FE5"+
	"\u5FE6\u5FE7\u5FE8\u5FE9\u5FEA\u5FEB\u5FEC\u5FED"+
	"\u5FEE\u5FEF\u5FF0\u5FF1\u5FF2\u5FF3\u5FF4\u5FF5"+
	"\u5FF6\u5FF7\u5FF8\u5FF9\u5FFA\u5FFB\u5FFC\u5FFD"+
	"\u5FFE\u5FFF\u6000\u6001\u6002\u6003\u6004\u6005"+
	"\u6006\u6007\u6008\u6009\u600A\u600B\u600C\u600D"+
	"\u600E\u600F\u6010\u6011\u6012\u6013\u6014\u6015"+
	"\u6016\u6017\u6018\u6019\u601A\u601B\u601C\u601D"+
	"\u601E\u601F\u6020\u6021\u6022\u6023\u6024\u6025"+
	"\u6026\u6027\u6028\u6029\u602A\u602B\u602C\u602D"+
	"\u602E\u602F\u6030\u6031\uFE78\u6032\u6033\u6034"+
	"\u6035\uFE77\u6036\u6037\u6038\u6039\u603A\u603B"+
	"\u603C\u603D\u603E\u603F\u6040\u6041\u6042\u6043"+
	"\u6044\u6045\u6046\u6047\u6048\u6049\u604A\u604B"+
	"\u604C\u604D\u604E\u604F\u6050\u6051\u6052\u6053"+
	"\u6054\u6055\u6056\u6057\u6058\u6059\u605A\u605B"+
	"\u605C\u605D\u605E\u605F\u6060\uFE7A\u6061\u6062"+
	"\u6063\u6064\u6065\u6066\u6067\u6068\u6069\u606A"+
	"\u606B\u606C\u606D\u606E\u606F\u6070\u6071\u6072"+
	"\u6073\u6074\u6075\u6076\u6077\u6078\u6079\u607A"+
	"\u607B\u607C\u607D\u607E\u607F\u6080\u6081\u6082"+
	"\u6083\u6084\u6085\u6086\u6087\u6088\u6089\u608A"+
	"\u608B\u608C\u608D\u608E\u608F\u6090\u6091\u6092"+
	"\u6093\u6094\u6095\u6096\u6097\u6098\u6099\u609A"+
	"\u609B\u609C\u609D\u609E\u609F\u60A0\u60A1\u60A2"+
	"\u60A3\u60A4\u60A5\u60A6\u60A7\u60A8\u60A9\u60AA"+
	"\u60AB\u60AC\u60AD\u60AE\u60AF\u60B0\u60B1\u60B2"+
	"\u60B3\u60B4\u60B5\u60B6\u60B7\u60B8\u60B9\u60BA"+
	"\u60BB\u60BC\u60BD\u60BE\u60BF\u60C0\u60C1\u60C2"+
	"\u60C3\u60C4\u60C5\u60C6\u60C7\u60C8\u60C9\u60CA"+
	"\u60CB\u60CC\u60CD\u60CE\u60CF\u60D0\u60D1\u60D2"+
	"\u60D3\u60D4\u60D5\u60D6\u60D7\u60D8\u60D9\u60DA"+
	"\u60DB\u60DC\u60DD\u60DE\u60DF\u60E0\u60E1\u60E2"+
	"\u60E3\u60E4\u60E5\u60E6\u60E7\u60E8\u60E9\u60EA"+
	"\u60EB\u60EC\u60ED\u60EE\u60EF\u60F0\u60F1\u60F2"+
	"\u60F3\u60F4\u60F5\u60F6\u60F7\u60F8\u60F9\u60FA"+
	"\u60FB\u60FC\u60FD\u60FE\u60FF\u6100\u6101\u6102"+
	"\u6103\u6104\u6105\u6106\u6107\u6108\u6109\u610A"+
	"\u610B\u610C\u610D\u610E\u610F\u6110\u6111\u6112"+
	"\u6113\u6114\u6115\u6116\u6117\u6118\u6119\u611A"+
	"\u611B\u611C\u611D\u611E\u611F\u6120\u6121\u6122"+
	"\u6123\u6124\u6125\u6126\u6127\u6128\u6129\u612A"+
	"\u612B\u612C\u612D\u612E\u612F\u6130\u6131\u6132"+
	"\u6133\u6134\u6135\u6136\u6137\u6138\u6139\u613A"+
	"\u613B\u613C\u613D\u613E\u613F\u6140\u6141\u6142"+
	"\u6143\u6144\u6145\u6146\u6147\u6148\u6149\u614A"+
	"\u614B\u614C\u614D\u614E\u614F\u6150\u6151\u6152"+
	"\u6153\u6154\u6155\u6156\u6157\u6158\uFE7B\u6159"+
	"\u615A\u615B\u615C\u615D\u615E\u615F\u6160\u6161"+
	"\u6162\u6163\u6164\u6165\u6166\u6167\u6168\u6169"+
	"\u616A\u616B\u616C\u616D\u616E\u616F\u6170\u6171"+
	"\u6172\u6173\u6174\u6175\u6176\u6177\u6178\u6179"+
	"\u617A\u617B\u617C\u617D\u617E\u617F\u6180\u6181"+
	"\u6182\u6183\u6184\u6185\u6186\u6187\u6188\u6189"+
	"\u618A\u618B\u618C\u618D\u618E\u618F\u6190\u6191"+
	"\u6192\u6193\u6194\u6195\u6196\u6197\u6198\u6199"+
	"\u619A\u619B\u619C\u619D\u619E\u619F\u61A0\u61A1"+
	"\u61A2\u61A3\u61A4\u61A5\u61A6\u61A7\u61A8\u61A9"+
	"\u61AA\u61AB\u61AC\u61AD\u61AE\u61AF\u61B0\u61B1"+
	"\u61B2\u61B3\u61B4\u61B5\u61B6\u61B7\u61B8\u61B9"+
	"\u61BA\u61BB\u61BC\u61BD\u61BE\u61BF\u61C0\u61C1"+
	"\u61C2\u61C3\u61C4\u61C5\u61C6\u61C7\u61C8\u61C9"+
	"\u61CA\u61CB\u61CC\u61CD\u61CE\u61CF\u61D0\u61D1"+
	"\u61D2\u61D3\u61D4\u61D5\u61D6\u61D7\u61D8\u61D9"+
	"\u61DA\u61DB\u61DC\u61DD\u61DE\u61DF\u61E0\u61E1"+
	"\u61E2\u61E3\u61E4\u61E5\u61E6\u61E7\u61E8\u61E9"+
	"\u61EA\u61EB\u61EC\u61ED\u61EE\u61EF\u61F0\u61F1"+
	"\u61F2\u61F3\u61F4\u61F5\u61F6\u61F7\u61F8\u61F9"+
	"\u61FA\u61FB\u61FC\u61FD\u61FE\u61FF\u6200\u6201"+
	"\u6202\u6203\u6204\u6205\u6206\u6207\u6208\u6209"+
	"\u620A\u620B\u620C\u620D\u620E\u620F\u6210\u6211"+
	"\u6212\u6213\u6214\u6215\u6216\u6217\u6218\u6219"+
	"\u621A\u621B\u621C\u621D\u621E\u621F\u6220\u6221"+
	"\u6222\u6223\u6224\u6225\u6226\u6227\u6228\u6229"+
	"\u622A\u622B\u622C\u622D\u622E\u622F\u6230\u6231"+
	"\u6232\u6233\u6234\u6235\u6236\u6237\u6238\u6239"+
	"\u623A\u623B\u623C\u623D\u623E\u623F\u6240\u6241"+
	"\u6242\u6243\u6244\u6245\u6246\u6247\u6248\u6249"+
	"\u624A\u624B\u624C\u624D\u624E\u624F\u6250\u6251"+
	"\u6252\u6253\u6254\u6255\u6256\u6257\u6258\u6259"+
	"\u625A\u625B\u625C\u625D\u625E\u625F\u6260\u6261"+
	"\u6262\u6263\u6264\u6265\u6266\u6267\u6268\u6269"+
	"\u626A\u626B\u626C\u626D\u626E\u626F\u6270\u6271"+
	"\u6272\u6273\u6274\u6275\u6276\u6277\u6278\u6279"+
	"\u627A\u627B\u627C\u627D\u627E\u627F\u6280\u6281"+
	"\u6282\u6283\u6284\u6285\u6286\u6287\u6288\u6289"+
	"\u628A\u628B\u628C\u628D\u628E\u628F\u6290\u6291"+
	"\u6292\u6293\u6294\u6295\u6296\u6297\u6298\u6299"+
	"\u629A\u629B\u629C\u629D\u629E\u629F\u62A0\u62A1"+
	"\u62A2\u62A3\u62A4\u62A5\u62A6\u62A7\u62A8\u62A9"+
	"\u62AA\u62AB\u62AC\u62AD\u62AE\u62AF\u62B0\u62B1"+
	"\u62B2\u62B3\u62B4\u62B5\u62B6\u62B7\u62B8\u62B9"+
	"\u62BA\u62BB\u62BC\u62BD\u62BE\u62BF\u62C0\u62C1"+
	"\u62C2\u62C3\u62C4\u62C5\u62C6\u62C7\u62C8\u62C9"+
	"\u62CA\u62CB\u62CC\u62CD\uFE7D\u62CE\u62CF\u62D0"+
	"\u62D1\u62D2\u62D3\u62D4\u62D5\u62D6\u62D7\u62D8"+
	"\u62D9\u62DA\u62DB\u62DC\u62DD\u62DE\u62DF\u62E0"+
	"\u62E1\uFE7C\u62E2\u62E3\u62E4\u62E5\u62E6\u62E7"+
	"\u62E8\u62E9\u62EA\u62EB\u62EC\u62ED\u62EE\u62EF"+
	"\u62F0\u62F1\u62F2\u62F3\u62F4\u62F5\u62F6\u62F7"+
	"\u62F8\u62F9\u62FA\u62FB\u62FC\u62FD\u62FE\u62FF"+
	"\u6300\u6301\u6302\u6303\u6304\u6305\u6306\u6307"+
	"\u6308\u6309\u630A\u630B\u630C\u630D\u630E\u630F"+
	"\u6310\u6311\u6312\u6313\u6314\u6315\u6316\u6317"+
	"\u6318\u6319\u631A\u631B\u631C\u631D\u631E\u631F"+
	"\u6320\u6321\u6322\u6323\u6324\u6325\u6326\u6327"+
	"\u6328\u6329\u632A\u632B\u632C\u632D\u632E\u632F"+
	"\u6330\u6331\u6332\u6333\u6334\u6335\u6336\u6337"+
	"\u6338\u6339\u633A\u633B\u633C\u633D\u633E\u633F"+
	"\u6340\u6341\u6342\u6343\u6344\u6345\u6346\u6347"+
	"\u6348\u6349\u634A\u634B\u634C\u634D\u634E\u634F"+
	"\u6350\u6351\u6352\u6353\u6354\u6355\u6356\u6357"+
	"\u6358\u6359\u635A\u635B\u635C\u635D\u635E\u635F"+
	"\u6360\u6361\u6362\u6363\u6364\u6365\u6366\u6367"+
	"\u6368\u6369\u636A\u636B\u636C\u636D\u636E\u636F"+
	"\u6370\u6371\u6372\u6373\u6374\u6375\u6376\u6377"+
	"\u6378\u6379\u637A\u637B\u637C\u637D\u637E\u637F"+
	"\u6380\u6381\u6382\u6383\u6384\u6385\u6386\u6387"+
	"\u6388\u6389\u638A\u638B\u638C\u638D\u638E\u638F"+
	"\u6390\u6391\u6392\u6393\u6394\u6395\u6396\u6397"+
	"\u6398\u6399\u639A\u639B\u639C\u639D\u639E\u639F"+
	"\u63A0\u63A1\u63A2\uFE80\u63A3\u63A4\u63A5\u63A6"+
	"\u63A7\uFE81\u63A8\u63A9\u63AA\u63AB\u63AC\u63AD"+
	"\u63AE\u63AF\u63B0\u63B1\u63B2\u63B3\u63B4\u63B5"+
	"\u63B6\u63B7\u63B8\u63B9\u63BA\u63BB\u63BC\u63BD"+
	"\u63BE\u63BF\u63C0\u63C1\u63C2\u63C3\u63C4\u63C5"+
	"\u63C6\u63C7\u63C8\u63C9\u63CA\u63CB\u63CC\u63CD"+
	"\u63CE\u63CF\u63D0\u63D1\u63D2\u63D3\u63D4\u63D5"+
	"\u63D6\u63D7\u63D8\u63D9\u63DA\u63DB\u63DC\u63DD"+
	"\u63DE\u63DF\u63E0\u63E1\u63E2\u63E3\u63E4\u63E5"+
	"\u63E6\u63E7\u63E8\u63E9\u63EA\u63EB\u63EC\u63ED"+
	"\u63EE\u63EF\u63F0\u63F1\u63F2\u63F3\u63F4\u63F5"+
	"\u63F6\u63F7\u63F8\u63F9\uFE82\u63FA\u63FB\u63FC"+
	"\u63FD\u63FE\u63FF\u6400\u6401\u6402\u6403\u6404"+
	"\u6405\u6406\u6407\u6408\u6409\uFE83\u640A\u640B"+
	"\u640C\u640D\u640E\u640F\u6410\u6411\u6412\u6413"+
	"\u6414\u6415\u6416\u6417\u6418\u6419\u641A\u641B"+
	"\u641C\u641D\u641E\u641F\u6420\u6421\u6422\u6423"+
	"\u6424\u6425\u6426\u6427\u6428\u6429\u642A\u642B"+
	"\u642C\u642D\u642E\u642F\u6430\u6431\u6432\u6433"+
	"\u6434\u6435\u6436\u6437\u6438\u6439\u643A\u643B"+
	"\u643C\u643D\u643E\u643F\u6440\u6441\u6442\u6443"+
	"\u6444\u6445\u6446\u6447\u6448\u6449\u644A\u644B"+
	"\u644C\u644D\u644E\u644F\u6450\u6451\u6452\u6453"+
	"\u6454\u6455\u6456\u6457\u6458\u6459\u645A\u645B"+
	"\u645C\u645D\u645E\u645F\u6460\u6461\u6462\u6463"+
	"\u6464\u6465\u6466\u6467\u6468\u6469\u646A\u646B"+
	"\u646C\u646D\u646E\u646F\u6470\u6471\u6472\u6473"+
	"\u6474\u6475\u6476\u6477\u6478\u6479\u647A\u647B"+
	"\u647C\u647D\u647E\u647F\u6480\u6481\u6482\u6483"+
	"\u6484\u6485\u6486\u6487\u6488\u6489\u648A\u648B"+
	"\u648C\u648D\u648E\u648F\u6490\u6491\u6492\u6493"+
	"\u6494\u6495\u6496\u6497\u6498\u6499\u649A\u649B"+
	"\u649C\u649D\u649E\u649F\u64A0\u64A1\u64A2\u64A3"+
	"\u64A4\u64A5\u64A6\u64A7\u64A8\u64A9\u64AA\u64AB"+
	"\u64AC\u64AD\u64AE\u64AF\u64B0\u64B1\u64B2\u64B3"+
	"\u64B4\u64B5\u64B6\u64B7\u64B8\u64B9\u64BA\u64BB"+
	"\u64BC\u64BD\u64BE\u64BF\u64C0\u64C1\u64C2\u64C3"+
	"\u64C4\u64C5\u64C6\u64C7\u64C8\u64C9\u64CA\u64CB"+
	"\u64CC\u64CD\u64CE\u64CF\u64D0\u64D1\u64D2\u64D3"+
	"\u64D4\u64D5\u64D6\u64D7\u64D8\u64D9\u64DA\u64DB"+
	"\u64DC\u64DD\u64DE\u64DF\u64E0\u64E1\u64E2\u64E3"+
	"\u64E4\u64E5\u64E6\u64E7\u64E8\u64E9\u64EA\u64EB"+
	"\u64EC\u64ED\u64EE\u64EF\u64F0\u64F1\u64F2\u64F3"+
	"\u64F4\u64F5\u64F6\u64F7\u64F8\u64F9\u64FA\u64FB"+
	"\u64FC\u64FD\u64FE\u64FF\u6500\u6501\u6502\u6503"+
	"\u6504\u6505\u6506\u6507\u6508\u6509\u650A\u650B"+
	"\u650C\u650D\u650E\u650F\u6510\u6511\u6512\u6513"+
	"\u6514\u6515\u6516\u6517\u6518\u6519\u651A\u651B"+
	"\u651C\u651D\u651E\u651F\u6520\u6521\u6522\u6523"+
	"\u6524\u6525\u6526\u6527\u6528\u6529\u652A\u652B"+
	"\u652C\u652D\u652E\u652F\u6530\u6531\u6532\u6533"+
	"\u6534\u6535\u6536\u6537\u6538\u6539\u653A\u653B"+
	"\u653C\u653D\u653E\u653F\u6540\u6541\u6542\u6543"+
	"\u6544\u6545\u6546\u6547\u6548\u6549\u654A\u654B"+
	"\u654C\u654D\u654E\u654F\u6550\u6551\u6552\u6553"+
	"\u6554\u6555\u6556\u6557\u6558\u6559\u655A\u655B"+
	"\u655C\u655D\u655E\u655F\u6560\u6561\u6562\u6563"+
	"\u6564\u6565\u6566\u6567\u6568\u6569\u656A\u656B"+
	"\u656C\u656D\u656E\u656F\u6570\u6571\u6572\u6573"+
	"\u6574\u6575\u6576\u6577\u6578\u6579\u657A\u657B"+
	"\u657C\u657D\u657E\u657F\u6580\u6581\u6582\u6583"+
	"\u6584\u6585\u6586\u6587\u6588\u6589\u658A\u658B"+
	"\u658C\u658D\u658E\u658F\u6590\u6591\u6592\u6593"+
	"\u6594\u6595\u6596\u6597\u6598\u6599\u659A\u659B"+
	"\u659C\u659D\u659E\u659F\u65A0\u65A1\u65A2\u65A3"+
	"\u65A4\u65A5\u65A6\u65A7\u65A8\u65A9\u65AA\u65AB"+
	"\u65AC\u65AD\u65AE\u65AF\u65B0\u65B1\u65B2\u65B3"+
	"\u65B4\u65B5\u65B6\u65B7\u65B8\u65B9\u65BA\u65BB"+
	"\u65BC\u65BD\u65BE\u65BF\u65C0\u65C1\u65C2\uFE85"+
	"\u65C3\u65C4\u65C5\u65C6\u65C7\u65C8\u65C9\u65CA"+
	"\u65CB\u65CC\u65CD\u65CE\u65CF\u65D0\u65D1\u65D2"+
	"\u65D3\u65D4\u65D5\u65D6\u65D7\u65D8\u65D9\u65DA"+
	"\u65DB\u65DC\u65DD\u65DE\u65DF\u65E0\u65E1\u65E2"+
	"\u65E3\u65E4\u65E5\u65E6\u65E7\u65E8\u65E9\u65EA"+
	"\u65EB\u65EC\u65ED\u65EE\u65EF\u65F0\u65F1\u65F2"+
	"\u65F3\u65F4\uFE86\u65F5\u65F6\uFE87\u65F7\u65F8"+
	"\u65F9\u65FA\uFE88\uFE89\u65FB\uFE8A\uFE8B\u65FC"+
	"\u65FD\u65FE\u65FF\u6600\u6601\u6602\u6603\u6604"+
	"\u6605\u6606\u6607\u6608\u6609\u660A\u660B\u660C"+
	"\u660D\u660E\u660F\uFE8D\u6610\u6611\u6612\uFE8C"+
	"\u6613\u6614\u6615\u6616\u6617\u6618\u6619\u661A"+
	"\u661B\u661C\u661D\u661E\u661F\u6620\u6621\u6622"+
	"\u6623\u6624\u6625\u6626\u6627\u6628\uFE8F\uFE8E"+
	"\u6629\u662A\u662B\u662C\u662D\u662E\u662F\u6630"+
	"\u6631\u6632\u6633\u6634\u6635\u6636\u6637\u6638"+
	"\u6639\u663A\u663B\u663C\u663D\u663E\u663F\u6640"+
	"\u6641\u6642\u6643\u6644\u6645\u6646\u6647\u6648"+
	"\u6649\u664A\u664B\u664C\u664D\u664E\u664F\u6650"+
	"\u6651\u6652\u6653\u6654\u6655\u6656\u6657\u6658"+
	"\u6659\u665A\u665B\u665C\u665D\u665E\u665F\u6660"+
	"\u6661\u6662\u6663\u6664\u6665\u6666\u6667\u6668"+
	"\u6669\u666A\u666B\u666C\u666D\u666E\u666F\u6670"+
	"\u6671\u6672\u6673\u6674\u6675\u6676\u6677\u6678"+
	"\u6679\u667A\u667B\u667C\u667D\u667E\u667F\u6680"+
	"\u6681\u6682\u6683\u6684\u6685\u6686\u6687\u6688"+
	"\u6689\u668A\u668B\u668C\u668D\u668E\u668F\u6690"+
	"\u6691\u6692\u6693\u6694\u6695\u6696\u6697\u6698"+
	"\u6699\u669A\u669B\u669C\u669D\u669E\u669F\u66A0"+
	"\u66A1\u66A2\u66A3\u66A4\u66A5\u66A6\u66A7\u66A8"+
	"\u66A9\u66AA\u66AB\u66AC\u66AD\u66AE\u66AF\u66B0"+
	"\u66B1\u66B2\u66B3\u66B4\u66B5\u66B6\u66B7\u66B8"+
	"\u66B9\u66BA\u66BB\u66BC\u66BD\u66BE\u66BF\u66C0"+
	"\u66C1\u66C2\u66C3\u66C4\u66C5\u66C6\u66C7\u66C8"+
	"\u66C9\u66CA\u66CB\u66CC\u66CD\u66CE\u66CF\u66D0"+
	"\u66D1\u66D2\u66D3\u66D4\u66D5\u66D6\u66D7\u66D8"+
	"\u66D9\u66DA\u66DB\u66DC\u66DD\u66DE\u66DF\u66E0"+
	"\u66E1\u66E2\u66E3\u66E4\u66E5\u66E6\u66E7\u66E8"+
	"\u66E9\u66EA\u66EB\u66EC\u66ED\u66EE\u66EF\u66F0"+
	"\u66F1\u66F2\u66F3\u66F4\u66F5\u66F6\u66F7\u66F8"+
	"\u66F9\u66FA\u66FB\u66FC\u66FD\u66FE\u66FF\u6700"+
	"\u6701\u6702\u6703\u6704\u6705\u6706\u6707\u6708"+
	"\u6709\u670A\u670B\u670C\u670D\u670E\u670F\u6710"+
	"\u6711\u6712\u6713\u6714\u6715\u6716\u6717\u6718"+
	"\u6719\u671A\u671B\u671C\u671D\u671E\u671F\u6720"+
	"\u6721\u6722\u6723\u6724\u6725\u6726\u6727\u6728"+
	"\u6729\u672A\u672B\u672C\u672D\u672E\u672F\u6730"+
	"\u6731\u6732\u6733\u6734\u6735\u6736\u6737\u6738"+
	"\u6739\u673A\u673B\u673C\u673D\u673E\u673F\u6740"+
	"\u6741\u6742\u6743\u6744\u6745\u6746\u6747\u6748"+
	"\u6749\u674A\u674B\u674C\u674D\u674E\u674F\u6750"+
	"\u6751\u6752\u6753\u6754\u6755\u6756\u6757\u6758"+
	"\u6759\u675A\u675B\u675C\u675D\u675E\u675F\u6760"+
	"\u6761\u6762\u6763\u6764\u6765\u6766\u6767\u6768"+
	"\u6769\u676A\u676B\u676C\u676D\u676E\u676F\u6770"+
	"\u6771\u6772\u6773\u6774\u6775\u6776\u6777\u6778"+
	"\u6779\u677A\u677B\u677C\u677D\u677E\u677F\u6780"+
	"\u6781\u6782\u6783\u6784\u6785\u6786\u6787\u6788"+
	"\u6789\u678A\u678B\u678C\u678D\u678E\u678F\u6790"+
	"\u6791\u6792\u6793\u6794\u6795\u6796\u6797\u6798"+
	"\u6799\u679A\u679B\u679C\u679D\u679E\u679F\u67A0"+
	"\u67A1\u67A2\u67A3\u67A4\u67A5\u67A6\u67A7\u67A8"+
	"\u67A9\u67AA\u67AB\u67AC\u67AD\u67AE\u67AF\u67B0"+
	"\u67B1\u67B2\u67B3\u67B4\u67B5\u67B6\u67B7\u67B8"+
	"\u67B9\u67BA\u67BB\u67BC\u67BD\u67BE\u67BF\u67C0"+
	"\u67C1\u67C2\u67C3\u67C4\u67C5\u67C6\u67C7\u67C8"+
	"\u67C9\u67CA\u67CB\u67CC\u67CD\u67CE\u67CF\u67D0"+
	"\u67D1\u67D2\u67D3\u67D4\u67D5\u67D6\u67D7\u67D8"+
	"\u67D9\u67DA\u67DB\u67DC\u67DD\u67DE\u67DF\u67E0"+
	"\u67E1\u67E2\u67E3\u67E4\u67E5\u67E6\u67E7\u67E8"+
	"\u67E9\u67EA\u67EB\u67EC\u67ED\u67EE\u67EF\u67F0"+
	"\u67F1\u67F2\u67F3\u67F4\u67F5\u67F6\u67F7\u67F8"+
	"\u67F9\u67FA\u67FB\u67FC\u67FD\u67FE\u67FF\u6800"+
	"\u6801\u6802\u6803\u6804\u6805\u6806\u6807\u6808"+
	"\u6809\u680A\u680B\u680C\u680D\u680E\u680F\u6810"+
	"\u6811\u6812\u6813\u6814\u6815\u6816\u6817\u6818"+
	"\u6819\u681A\u681B\u681C\u681D\u681E\u681F\u6820"+
	"\u6821\u6822\u6823\u6824\u6825\u6826\u6827\u6828"+
	"\u6829\u682A\u682B\u682C\u682D\u682E\u682F\u6830"+
	"\u6831\u6832\u6833\u6834\u6835\u6836\u6837\u6838"+
	"\u6839\u683A\u683B\u683C\u683D\u683E\u683F\u6840"+
	"\u6841\u6842\u6843\u6844\u6845\u6846\u6847\u6848"+
	"\u6849\u684A\u684B\u684C\u684D\u684E\u684F\u6850"+
	"\u6851\u6852\u6853\u6854\u6855\u6856\u6857\u6858"+
	"\u6859\u685A\u685B\u685C\u685D\u685E\u685F\u6860"+
	"\u6861\u6862\u6863\u6864\u6865\u6866\u6867\u6868"+
	"\u6869\u686A\u686B\u686C\u686D\u686E\u686F\u6870"+
	"\u6871\u6872\u6873\u6874\u6875\u6876\u6877\u6878"+
	"\u6879\u687A\u687B\u687C\u687D\u687E\u687F\u6880"+
	"\u6881\u6882\u6883\u6884\u6885\u6886\u6887\u6888"+
	"\u6889\u688A\u688B\u688C\u688D\u688E\u688F\u6890"+
	"\u6891\u6892\u6893\u6894\u6895\u6896\u6897\u6898"+
	"\u6899\u689A\u689B\u689C\u689D\u689E\u689F\u68A0"+
	"\u68A1\u68A2\u68A3\u68A4\u68A5\u68A6\u68A7\u68A8"+
	"\u68A9\u68AA\u68AB\u68AC\u68AD\u68AE\u68AF\u68B0"+
	"\u68B1\u68B2\u68B3\u68B4\u68B5\u68B6\u68B7\u68B8"+
	"\u68B9\u68BA\u68BB\u68BC\u68BD\u68BE\u68BF\u68C0"+
	"\u68C1\u68C2\u68C3\u68C4\u68C5\u68C6\u68C7\u68C8"+
	"\u68C9\u68CA\u68CB\u68CC\u68CD\u68CE\u68CF\u68D0"+
	"\u68D1\u68D2\u68D3\u68D4\u68D5\u68D6\u68D7\u68D8"+
	"\u68D9\u68DA\u68DB\u68DC\u68DD\u68DE\u68DF\u68E0"+
	"\u68E1\u68E2\u68E3\u68E4\u68E5\u68E6\u68E7\uFE96"+
	"\u68E8\u68E9\u68EA\u68EB\u68EC\u68ED\u68EE\u68EF"+
	"\u68F0\u68F1\u68F2\u68F3\u68F4\u68F5\u68F6\u68F7"+
	"\u68F8\u68F9\u68FA\u68FB\u68FC\u68FD\u68FE\u68FF"+
	"\u6900\u6901\u6902\u6903\u6904\u6905\u6906\u6907"+
	"\u6908\u6909\u690A\u690B\u690C\u690D\u690E\uFE93"+
	"\uFE94\uFE95\uFE97\uFE92\u690F\u6910\u6911\u6912"+
	"\u6913\u6914\u6915\u6916\u6917\u6918\u6919\u691A"+
	"\u691B\u691C\u691D\u691E\u691F\u6920\u6921\u6922"+
	"\u6923\u6924\u6925\u6926\u6927\u6928\u6929\u692A"+
	"\u692B\u692C\u692D\u692E\u692F\u6930\u6931\u6932"+
	"\u6933\u6934\u6935\u6936\u6937\u6938\u6939\u693A"+
	"\u693B\u693C\u693D\u693E\u693F\u6940\u6941\u6942"+
	"\u6943\u6944\u6945\u6946\u6947\u6948\u6949\u694A"+
	"\u694B\u694C\u694D\u694E\u694F\u6950\u6951\u6952"+
	"\u6953\u6954\u6955\u6956\u6957\u6958\u6959\u695A"+
	"\u695B\u695C\u695D\u695E\u695F\u6960\u6961\u6962"+
	"\u6963\u6964\u6965\u6966\u6967\u6968\u6969\u696A"+
	"\u696B\u696C\u696D\u696E\u696F\u6970\u6971\u6972"+
	"\u6973\u6974\u6975\u6976\u6977\u6978\u6979\u697A"+
	"\u697B\u697C\u697D\uFE98\uFE99\uFE9A\uFE9B\uFE9C"+
	"\uFE9D\uFE9E\u697E\u697F\u6980\u6981\u6982\u6983"+
	"\u6984\u6985\u6986\u6987\u6988\u6989\u698A\u698B"+
	"\u698C\u698D\u698E\u698F\u6990\u6991\u6992\u6993"+
	"\u6994\u6995\u6996\u6997\u6998\u6999\u699A\u699B"+
	"\u699C\u699D\u699E\u699F\u69A0\u69A1\u69A2\u69A3"+
	"\u69A4\u69A5\u69A6\u69A7\u69A8\u69A9\u69AA\u69AB"+
	"\u69AC\u69AD\u69AE\u69AF\u69B0\u69B1\u69B2\u69B3"+
	"\u69B4\u69B5\u69B6\u69B7\u69B8\u69B9\u69BA\u69BB"+
	"\u69BC\u69BD\u69BE\u69BF\u69C0\u69C1\u69C2\u69C3"+
	"\u69C4\u69C5\u69C6\u69C7\u69C8\u69C9\u69CA\u69CB"+
	"\u69CC\u69CD\u69CE\u69CF\u69D0\u69D1\u69D2\u69D3"+
	"\u69D4\u69D5\u69D6\u69D7\u69D8\u69D9\u69DA\u69DB"+
	"\u69DC\u69DD\u69DE\u69DF\u69E0\u69E1\u69E2\u69E3"+
	"\u69E4\u69E5\u69E6\u69E7\u69E8\u69E9\u69EA\u69EB"+
	"\u69EC\u69ED\u69EE\u69EF\u69F0\u69F1\u69F2\u69F3"+
	"\u69F4\u69F5\u69F6\u69F7\u69F8\u69F9\u69FA\u69FB"+
	"\u69FC\u69FD\u69FE\u69FF\u6A00\u6A01\u6A02\u6A03"+
	"\u6A04\u6A05\u6A06\u6A07\u6A08\u6A09\u6A0A\u6A0B"+
	"\u6A0C\u6A0D\u6A0E\u6A0F\u6A10\u6A11\uFE9F\u6A12"+
	"\u6A13\u6A14\u6A15\u6A16\u6A17\u6A18\u6A19\u6A1A"+
	"\u6A1B\u6A1C\u6A1D\u6A1E\u6A1F\u6A20\u6A21\u6A22"+
	"\u6A23\u6A24\u6A25\u6A26\u6A27\u6A28\u6A29\u6A2A"+
	"\u6A2B\u6A2C\u6A2D\u6A2E\u6A2F\u6A30\u6A31\u6A32"+
	"\u6A33\u6A34\u6A35\u6A36\u6A37\u6A38\u6A39\u6A3A"+
	"\u6A3B\u6A3C\u6A3D\u6A3E\u6A3F\u6A40\u6A41\u6A42"+
	"\u6A43\u6A44\u6A45\u6A46\u6A47\u6A48\u6A49\u6A4A"+
	"\u6A4B\u6A4C\u6A4D\u6A4E\u6A4F\u6A50\u6A51\u6A52"+
	"\u6A53\u6A54\u6A55\u6A56\u6A57\u6A58\u6A59\u6A5A"+
	"\u6A5B\u6A5C\u6A5D\u6A5E\u6A5F\u6A60\u6A61\u6A62"+
	"\uD2BB\uB6A1\u8140\uC6DF\u8141\u8142\u8143\uCDF2"+
	"\uD5C9\uC8FD\uC9CF\uCFC2\uD8A2\uB2BB\uD3EB\u8144"+
	"\uD8A4\uB3F3\u8145\uD7A8\uC7D2\uD8A7\uCAC0\u8146"+
	"\uC7F0\uB1FB\uD2B5\uB4D4\uB6AB\uCBBF\uD8A9\u8147"+
	"\u8148\u8149\uB6AA\u814A\uC1BD\uD1CF\u814B\uC9A5"+
	"\uD8AD\u814C\uB8F6\uD1BE\uE3DC\uD6D0\u814D\u814E"+
	"\uB7E1\u814F\uB4AE\u8150\uC1D9\u8151\uD8BC\u8152"+
	"\uCDE8\uB5A4\uCEAA\uD6F7\u8153\uC0F6\uBED9\uD8AF"+
	"\u8154\u8155\u8156\uC4CB\u8157\uBEC3\u8158\uD8B1"+
	"\uC3B4\uD2E5\u8159\uD6AE\uCEDA\uD5A7\uBAF5\uB7A6"+
	"\uC0D6\u815A\uC6B9\uC5D2\uC7C7\u815B\uB9D4\u815C"+
	"\uB3CB\uD2D2\u815D\u815E\uD8BF\uBEC5\uC6F2\uD2B2"+
	"\uCFB0\uCFE7\u815F\u8160\u8161\u8162\uCAE9\u8163"+
	"\u8164\uD8C0\u8165\u8166\u8167\u8168\u8169\u816A"+
	"\uC2F2\uC2D2\u816B\uC8E9\u816C\u816D\u816E\u816F"+
	"\u8170\u8171\u8172\u8173\u8174\u8175\uC7AC\u8176"+
	"\u8177\u8178\u8179\u817A\u817B\u817C\uC1CB\u817D"+
	"\uD3E8\uD5F9\u817E\uCAC2\uB6FE\uD8A1\uD3DA\uBFF7"+
	"\u8180\uD4C6\uBBA5\uD8C1\uCEE5\uBEAE\u8181\u8182"+
	"\uD8A8\u8183\uD1C7\uD0A9\u8184\u8185\u8186\uD8BD"+
	"\uD9EF\uCDF6\uBFBA\u8187\uBDBB\uBAA5\uD2E0\uB2FA"+
	"\uBAE0\uC4B6\u8188\uCFED\uBEA9\uCDA4\uC1C1\u8189"+
	"\u818A\u818B\uC7D7\uD9F1\u818C\uD9F4\u818D\u818E"+
	"\u818F\u8190\uC8CB\uD8E9\u8191\u8192\u8193\uD2DA"+
	"\uCAB2\uC8CA\uD8EC\uD8EA\uD8C6\uBDF6\uC6CD\uB3F0"+
	"\u8194\uD8EB\uBDF1\uBDE9\u8195\uC8D4\uB4D3\u8196"+
	"\u8197\uC2D8\u8198\uB2D6\uD7D0\uCACB\uCBFB\uD5CC"+
	"\uB8B6\uCFC9\u8199\u819A\u819B\uD9DA\uD8F0\uC7AA"+
	"\u819C\uD8EE\u819D\uB4FA\uC1EE\uD2D4\u819E\u819F"+
	"\uD8ED\u81A0\uD2C7\uD8EF\uC3C7\u81A1\u81A2\u81A3"+
	"\uD1F6\u81A4\uD6D9\uD8F2\u81A5\uD8F5\uBCFE\uBCDB"+
	"\u81A6\u81A7\u81A8\uC8CE\u81A9\uB7DD\u81AA\uB7C2"+
	"\u81AB\uC6F3\u81AC\u81AD\u81AE\u81AF\u81B0\u81B1"+
	"\u81B2\uD8F8\uD2C1\u81B3\u81B4\uCEE9\uBCBF\uB7FC"+
	"\uB7A5\uD0DD\u81B5\u81B6\u81B7\u81B8\u81B9\uD6DA"+
	"\uD3C5\uBBEF\uBBE1\uD8F1\u81BA\u81BB\uC9A1\uCEB0"+
	"\uB4AB\u81BC\uD8F3\u81BD\uC9CB\uD8F6\uC2D7\uD8F7"+
	"\u81BE\u81BF\uCEB1\uD8F9\u81C0\u81C1\u81C2\uB2AE"+
	"\uB9C0\u81C3\uD9A3\u81C4\uB0E9\u81C5\uC1E6\u81C6"+
	"\uC9EC\u81C7\uCBC5\u81C8\uCBC6\uD9A4\u81C9\u81CA"+
	"\u81CB\u81CC\u81CD\uB5E8\u81CE\u81CF\uB5AB\u81D0"+
	"\u81D1\u81D2\u81D3\u81D4\u81D5\uCEBB\uB5CD\uD7A1"+
	"\uD7F4\uD3D3\u81D6\uCCE5\u81D7\uBACE\u81D8\uD9A2"+
	"\uD9DC\uD3E0\uD8FD\uB7F0\uD7F7\uD8FE\uD8FA\uD9A1"+
	"\uC4E3\u81D9\u81DA\uD3B6\uD8F4\uD9DD\u81DB\uD8FB"+
	"\u81DC\uC5E5\u81DD\u81DE\uC0D0\u81DF\u81E0\uD1F0"+
	"\uB0DB\u81E1\u81E2\uBCD1\uD9A6\u81E3\uD9A5\u81E4"+
	"\u81E5\u81E6\u81E7\uD9AC\uD9AE\u81E8\uD9AB\uCAB9"+
	"\u81E9\u81EA\u81EB\uD9A9\uD6B6\u81EC\u81ED\u81EE"+
	"\uB3DE\uD9A8\u81EF\uC0FD\u81F0\uCACC\u81F1\uD9AA"+
	"\u81F2\uD9A7\u81F3\u81F4\uD9B0\u81F5\u81F6\uB6B1"+
	"\u81F7\u81F8\u81F9\uB9A9\u81FA\uD2C0\u81FB\u81FC"+
	"\uCFC0\u81FD\u81FE\uC2C2\u8240\uBDC4\uD5EC\uB2E0"+
	"\uC7C8\uBFEB\uD9AD\u8241\uD9AF\u8242\uCEEA\uBAEE"+
	"\u8243\u8244\u8245\u8246\u8247\uC7D6\u8248\u8249"+
	"\u824A\u824B\u824C\u824D\u824E\u824F\u8250\uB1E3"+
	"\u8251\u8252\u8253\uB4D9\uB6ED\uD9B4\u8254\u8255"+
	"\u8256\u8257\uBFA1\u8258\u8259\u825A\uD9DE\uC7CE"+
	"\uC0FE\uD9B8\u825B\u825C\u825D\u825E\u825F\uCBD7"+
	"\uB7FD\u8260\uD9B5\u8261\uD9B7\uB1A3\uD3E1\uD9B9"+
	"\u8262\uD0C5\u8263\uD9B6\u8264\u8265\uD9B1\u8266"+
	"\uD9B2\uC1A9\uD9B3\u8267\u8268\uBCF3\uD0DE\uB8A9"+
	"\u8269\uBEE3\u826A\uD9BD\u826B\u826C\u826D\u826E"+
	"\uD9BA\u826F\uB0B3\u8270\u8271\u8272\uD9C2\u8273";

    private final static String innerIndex5= 
	"\u8274\u8275\u8276\u8277\u8278\u8279\u827A\u827B"+
	"\u827C\u827D\u827E\u8280\uD9C4\uB1B6\u8281\uD9BF"+
	"\u8282\u8283\uB5B9\u8284\uBEF3\u8285\u8286\u8287"+
	"\uCCC8\uBAF2\uD2D0\u8288\uD9C3\u8289\u828A\uBDE8"+
	"\u828B\uB3AB\u828C\u828D\u828E\uD9C5\uBEEB\u828F"+
	"\uD9C6\uD9BB\uC4DF\u8290\uD9BE\uD9C1\uD9C0\u8291"+
	"\u8292\u8293\u8294\u8295\u8296\u8297\u8298\u8299"+
	"\u829A\u829B\uD5AE\u829C\uD6B5\u829D\uC7E3\u829E"+
	"\u829F\u82A0\u82A1\uD9C8\u82A2\u82A3\u82A4\uBCD9"+
	"\uD9CA\u82A5\u82A6\u82A7\uD9BC\u82A8\uD9CB\uC6AB"+
	"\u82A9\u82AA\u82AB\u82AC\u82AD\uD9C9\u82AE\u82AF"+
	"\u82B0\u82B1\uD7F6\u82B2\uCDA3\u82B3\u82B4\u82B5"+
	"\u82B6\u82B7\u82B8\u82B9\u82BA\uBDA1\u82BB\u82BC"+
	"\u82BD\u82BE\u82BF\u82C0\uD9CC\u82C1\u82C2\u82C3"+
	"\u82C4\u82C5\u82C6\u82C7\u82C8\u82C9\uC5BC\uCDB5"+
	"\u82CA\u82CB\u82CC\uD9CD\u82CD\u82CE\uD9C7\uB3A5"+
	"\uBFFE\u82CF\u82D0\u82D1\u82D2\uB8B5\u82D3\u82D4"+
	"\uC0FC\u82D5\u82D6\u82D7\u82D8\uB0F8\u82D9\u82DA"+
	"\u82DB\u82DC\u82DD\u82DE\u82DF\u82E0\u82E1\u82E2"+
	"\u82E3\u82E4\u82E5\u82E6\u82E7\u82E8\u82E9\u82EA"+
	"\u82EB\u82EC\u82ED\uB4F6\u82EE\uD9CE\u82EF\uD9CF"+
	"\uB4A2\uD9D0\u82F0\u82F1\uB4DF\u82F2\u82F3\u82F4"+
	"\u82F5\u82F6\uB0C1\u82F7\u82F8\u82F9\u82FA\u82FB"+
	"\u82FC\u82FD\uD9D1\uC9B5\u82FE\u8340\u8341\u8342"+
	"\u8343\u8344\u8345\u8346\u8347\u8348\u8349\u834A"+
	"\u834B\u834C\u834D\u834E\u834F\u8350\u8351\uCFF1"+
	"\u8352\u8353\u8354\u8355\u8356\u8357\uD9D2\u8358"+
	"\u8359\u835A\uC1C5\u835B\u835C\u835D\u835E\u835F"+
	"\u8360\u8361\u8362\u8363\u8364\u8365\uD9D6\uC9AE"+
	"\u8366\u8367\u8368\u8369\uD9D5\uD9D4\uD9D7\u836A"+
	"\u836B\u836C\u836D\uCBDB\u836E\uBDA9\u836F\u8370"+
	"\u8371\u8372\u8373\uC6A7\u8374\u8375\u8376\u8377"+
	"\u8378\u8379\u837A\u837B\u837C\u837D\uD9D3\uD9D8"+
	"\u837E\u8380\u8381\uD9D9\u8382\u8383\u8384\u8385"+
	"\u8386\u8387\uC8E5\u8388\u8389\u838A\u838B\u838C"+
	"\u838D\u838E\u838F\u8390\u8391\u8392\u8393\u8394"+
	"\u8395\uC0DC\u8396\u8397\u8398\u8399\u839A\u839B"+
	"\u839C\u839D\u839E\u839F\u83A0\u83A1\u83A2\u83A3"+
	"\u83A4\u83A5\u83A6\u83A7\u83A8\u83A9\u83AA\u83AB"+
	"\u83AC\u83AD\u83AE\u83AF\u83B0\u83B1\u83B2\uB6F9"+
	"\uD8A3\uD4CA\u83B3\uD4AA\uD0D6\uB3E4\uD5D7\u83B4"+
	"\uCFC8\uB9E2\u83B5\uBFCB\u83B6\uC3E2\u83B7\u83B8"+
	"\u83B9\uB6D2\u83BA\u83BB\uCDC3\uD9EE\uD9F0\u83BC"+
	"\u83BD\u83BE\uB5B3\u83BF\uB6B5\u83C0\u83C1\u83C2"+
	"\u83C3\u83C4\uBEA4\u83C5\u83C6\uC8EB\u83C7\u83C8"+
	"\uC8AB\u83C9\u83CA\uB0CB\uB9AB\uC1F9\uD9E2\u83CB"+
	"\uC0BC\uB9B2\u83CC\uB9D8\uD0CB\uB1F8\uC6E4\uBEDF"+
	"\uB5E4\uD7C8\u83CD\uD1F8\uBCE6\uCADE\u83CE\u83CF"+
	"\uBCBD\uD9E6\uD8E7\u83D0\u83D1\uC4DA\u83D2\u83D3"+
	"\uB8D4\uC8BD\u83D4\u83D5\uB2E1\uD4D9\u83D6\u83D7"+
	"\u83D8\u83D9\uC3B0\u83DA\u83DB\uC3E1\uDAA2\uC8DF"+
	"\u83DC\uD0B4\u83DD\uBEFC\uC5A9\u83DE\u83DF\u83E0"+
	"\uB9DA\u83E1\uDAA3\u83E2\uD4A9\uDAA4\u83E3\u83E4"+
	"\u83E5\u83E6\u83E7\uD9FB\uB6AC\u83E8\u83E9\uB7EB"+
	"\uB1F9\uD9FC\uB3E5\uBEF6\u83EA\uBFF6\uD2B1\uC0E4"+
	"\u83EB\u83EC\u83ED\uB6B3\uD9FE\uD9FD\u83EE\u83EF"+
	"\uBEBB\u83F0\u83F1\u83F2\uC6E0\u83F3\uD7BC\uDAA1"+
	"\u83F4\uC1B9\u83F5\uB5F2\uC1E8\u83F6\u83F7\uBCF5"+
	"\u83F8\uB4D5\u83F9\u83FA\u83FB\u83FC\u83FD\u83FE"+
	"\u8440\u8441\u8442\uC1DD\u8443\uC4FD\u8444\u8445"+
	"\uBCB8\uB7B2\u8446\u8447\uB7EF\u8448\u8449\u844A"+
	"\u844B\u844C\u844D\uD9EC\u844E\uC6BE\u844F\uBFAD"+
	"\uBBCB\u8450\u8451\uB5CA\u8452\uDBC9\uD0D7\u8453"+
	"\uCDB9\uB0BC\uB3F6\uBBF7\uDBCA\uBAAF\u8454\uD4E4"+
	"\uB5B6\uB5F3\uD8D6\uC8D0\u8455\u8456\uB7D6\uC7D0"+
	"\uD8D7\u8457\uBFAF\u8458\u8459\uDBBB\uD8D8\u845A"+
	"\u845B\uD0CC\uBBAE\u845C\u845D\u845E\uEBBE\uC1D0"+
	"\uC1F5\uD4F2\uB8D5\uB4B4\u845F\uB3F5\u8460\u8461"+
	"\uC9BE\u8462\u8463\u8464\uC5D0\u8465\u8466\u8467"+
	"\uC5D9\uC0FB\u8468\uB1F0\u8469\uD8D9\uB9CE\u846A"+
	"\uB5BD\u846B\u846C\uD8DA\u846D\u846E\uD6C6\uCBA2"+
	"\uC8AF\uC9B2\uB4CC\uBFCC\u846F\uB9F4\u8470\uD8DB"+
	"\uD8DC\uB6E7\uBCC1\uCCEA\u8471\u8472\u8473\u8474"+
	"\u8475\u8476\uCFF7\u8477\uD8DD\uC7B0\u8478\u8479"+
	"\uB9D0\uBDA3\u847A\u847B\uCCDE\u847C\uC6CA\u847D"+
	"\u847E\u8480\u8481\u8482\uD8E0\u8483\uD8DE\u8484"+
	"\u8485\uD8DF\u8486\u8487\u8488\uB0FE\u8489\uBEE7"+
	"\u848A\uCAA3\uBCF4\u848B\u848C\u848D\u848E\uB8B1"+
	"\u848F\u8490\uB8EE\u8491\u8492\u8493\u8494\u8495"+
	"\u8496\u8497\u8498\u8499\u849A\uD8E2\u849B\uBDCB"+
	"\u849C\uD8E4\uD8E3\u849D\u849E\u849F\u84A0\u84A1"+
	"\uC5FC\u84A2\u84A3\u84A4\u84A5\u84A6\u84A7\u84A8"+
	"\uD8E5\u84A9\u84AA\uD8E6\u84AB\u84AC\u84AD\u84AE"+
	"\u84AF\u84B0\u84B1\uC1A6\u84B2\uC8B0\uB0EC\uB9A6"+
	"\uBCD3\uCEF1\uDBBD\uC1D3\u84B3\u84B4\u84B5\u84B6"+
	"\uB6AF\uD6FA\uC5AC\uBDD9\uDBBE\uDBBF\u84B7\u84B8"+
	"\u84B9\uC0F8\uBEA2\uC0CD\u84BA\u84BB\u84BC\u84BD"+
	"\u84BE\u84BF\u84C0\u84C1\u84C2\u84C3\uDBC0\uCAC6"+
	"\u84C4\u84C5\u84C6\uB2AA\u84C7\u84C8\u84C9\uD3C2"+
	"\u84CA\uC3E3\u84CB\uD1AB\u84CC\u84CD\u84CE\u84CF"+
	"\uDBC2\u84D0\uC0D5\u84D1\u84D2\u84D3\uDBC3\u84D4"+
	"\uBFB1\u84D5\u84D6\u84D7\u84D8\u84D9\u84DA\uC4BC"+
	"\u84DB\u84DC\u84DD\u84DE\uC7DA\u84DF\u84E0\u84E1"+
	"\u84E2\u84E3\u84E4\u84E5\u84E6\u84E7\u84E8\u84E9"+
	"\uDBC4\u84EA\u84EB\u84EC\u84ED\u84EE\u84EF\u84F0"+
	"\u84F1\uD9E8\uC9D7\u84F2\u84F3\u84F4\uB9B4\uCEF0"+
	"\uD4C8\u84F5\u84F6\u84F7\u84F8\uB0FC\uB4D2\u84F9"+
	"\uD0D9\u84FA\u84FB\u84FC\u84FD\uD9E9\u84FE\uDECB"+
	"\uD9EB\u8540\u8541\u8542\u8543\uD8B0\uBBAF\uB1B1"+
	"\u8544\uB3D7\uD8CE\u8545\u8546\uD4D1\u8547\u8548"+
	"\uBDB3\uBFEF\u8549\uCFBB\u854A\u854B\uD8D0\u854C"+
	"\u854D\u854E\uB7CB\u854F\u8550\u8551\uD8D1\u8552"+
	"\u8553\u8554\u8555\u8556\u8557\u8558\u8559\u855A"+
	"\u855B\uC6A5\uC7F8\uD2BD\u855C\u855D\uD8D2\uC4E4"+
	"\u855E\uCAAE\u855F\uC7A7\u8560\uD8A6\u8561\uC9FD"+
	"\uCEE7\uBBDC\uB0EB\u8562\u8563\u8564\uBBAA\uD0AD"+
	"\u8565\uB1B0\uD7E4\uD7BF\u8566\uB5A5\uC2F4\uC4CF"+
	"\u8567\u8568\uB2A9\u8569\uB2B7\u856A\uB1E5\uDFB2"+
	"\uD5BC\uBFA8\uC2AC\uD8D5\uC2B1\u856B\uD8D4\uCED4"+
	"\u856C\uDAE0\u856D\uCEC0\u856E\u856F\uD8B4\uC3AE"+
	"\uD3A1\uCEA3\u8570\uBCB4\uC8B4\uC2D1\u8571\uBEED"+
	"\uD0B6\u8572\uDAE1\u8573\u8574\u8575\u8576\uC7E4"+
	"\u8577\u8578\uB3A7\u8579\uB6F2\uCCFC\uC0FA\u857A"+
	"\u857B\uC0F7\u857C\uD1B9\uD1E1\uD8C7\u857D\u857E"+
	"\u8580\u8581\u8582\u8583\u8584\uB2DE\u8585\u8586"+
	"\uC0E5\u8587\uBAF1\u8588\u8589\uD8C8\u858A\uD4AD"+
	"\u858B\u858C\uCFE1\uD8C9\u858D\uD8CA\uCFC3\u858E"+
	"\uB3F8\uBEC7\u858F\u8590\u8591\u8592\uD8CB\u8593"+
	"\u8594\u8595\u8596\u8597\u8598\u8599\uDBCC\u859A"+
	"\u859B\u859C\u859D\uC8A5\u859E\u859F\u85A0\uCFD8"+
	"\u85A1\uC8FE\uB2CE\u85A2\u85A3\u85A4\u85A5\u85A6"+
	"\uD3D6\uB2E6\uBCB0\uD3D1\uCBAB\uB7B4\u85A7\u85A8"+
	"\u85A9\uB7A2\u85AA\u85AB\uCAE5\u85AC\uC8A1\uCADC"+
	"\uB1E4\uD0F0\u85AD\uC5D1\u85AE\u85AF\u85B0\uDBC5"+
	"\uB5FE\u85B1\u85B2\uBFDA\uB9C5\uBEE4\uC1ED\u85B3"+
	"\uDFB6\uDFB5\uD6BB\uBDD0\uD5D9\uB0C8\uB6A3\uBFC9"+
	"\uCCA8\uDFB3\uCAB7\uD3D2\u85B4\uD8CF\uD2B6\uBAC5"+
	"\uCBBE\uCCBE\u85B5\uDFB7\uB5F0\uDFB4\u85B6\u85B7"+
	"\u85B8\uD3F5\u85B9\uB3D4\uB8F7\u85BA\uDFBA\u85BB"+
	"\uBACF\uBCAA\uB5F5\u85BC\uCDAC\uC3FB\uBAF3\uC0F4"+
	"\uCDC2\uCFF2\uDFB8\uCFC5\u85BD\uC2C0\uDFB9\uC2F0"+
	"\u85BE\u85BF\u85C0\uBEFD\u85C1\uC1DF\uCDCC\uD2F7"+
	"\uB7CD\uDFC1\u85C2\uDFC4\u85C3\u85C4\uB7F1\uB0C9"+
	"\uB6D6\uB7D4\u85C5\uBAAC\uCCFD\uBFD4\uCBB1\uC6F4"+
	"\u85C6\uD6A8\uDFC5\u85C7\uCEE2\uB3B3\u85C8\u85C9"+
	"\uCEFC\uB4B5\u85CA\uCEC7\uBAF0\u85CB\uCEE1\u85CC"+
	"\uD1BD\u85CD\u85CE\uDFC0\u85CF\u85D0\uB4F4\u85D1"+
	"\uB3CA\u85D2\uB8E6\uDFBB\u85D3\u85D4\u85D5\u85D6"+
	"\uC4C5\u85D7\uDFBC\uDFBD\uDFBE\uC5BB\uDFBF\uDFC2"+
	"\uD4B1\uDFC3\u85D8\uC7BA\uCED8\u85D9\u85DA\u85DB"+
	"\u85DC\u85DD\uC4D8\u85DE\uDFCA\u85DF\uDFCF\u85E0"+
	"\uD6DC\u85E1\u85E2\u85E3\u85E4\u85E5\u85E6\u85E7"+
	"\u85E8\uDFC9\uDFDA\uCEB6\u85E9\uBAC7\uDFCE\uDFC8"+
	"\uC5DE\u85EA\u85EB\uC9EB\uBAF4\uC3FC\u85EC\u85ED"+
	"\uBED7\u85EE\uDFC6\u85EF\uDFCD\u85F0\uC5D8\u85F1"+
	"\u85F2\u85F3\u85F4\uD5A6\uBACD\u85F5\uBECC\uD3BD"+
	"\uB8C0\u85F6\uD6E4\u85F7\uDFC7\uB9BE\uBFA7\u85F8"+
	"\u85F9\uC1FC\uDFCB\uDFCC\u85FA\uDFD0\u85FB\u85FC"+
	"\u85FD\u85FE\u8640\uDFDB\uDFE5\u8641\uDFD7\uDFD6"+
	"\uD7C9\uDFE3\uDFE4\uE5EB\uD2A7\uDFD2\u8642\uBFA9"+
	"\u8643\uD4DB\u8644\uBFC8\uDFD4\u8645\u8646\u8647"+
	"\uCFCC\u8648\u8649\uDFDD\u864A\uD1CA\u864B\uDFDE"+
	"\uB0A7\uC6B7\uDFD3\u864C\uBAE5\u864D\uB6DF\uCDDB"+
	"\uB9FE\uD4D5\u864E\u864F\uDFDF\uCFEC\uB0A5\uDFE7"+
	"\uDFD1\uD1C6\uDFD5\uDFD8\uDFD9\uDFDC\u8650\uBBA9"+
	"\u8651\uDFE0\uDFE1\u8652\uDFE2\uDFE6\uDFE8\uD3B4"+
	"\u8653\u8654\u8655\u8656\u8657\uB8E7\uC5B6\uDFEA"+
	"\uC9DA\uC1A8\uC4C4\u8658\u8659\uBFDE\uCFF8\u865A"+
	"\u865B\u865C\uD5DC\uDFEE\u865D\u865E\u865F\u8660"+
	"\u8661\u8662\uB2B8\u8663\uBADF\uDFEC\u8664\uDBC1"+
	"\u8665\uD1E4\u8666\u8667\u8668\u8669\uCBF4\uB4BD"+
	"\u866A\uB0A6\u866B\u866C\u866D\u866E\u866F\uDFF1"+
	"\uCCC6\uDFF2\u8670\u8671\uDFED\u8672\u8673\u8674"+
	"\u8675\u8676\u8677\uDFE9\u8678\u8679\u867A\u867B"+
	"\uDFEB\u867C\uDFEF\uDFF0\uBBBD\u867D\u867E\uDFF3"+
	"\u8680\u8681\uDFF4\u8682\uBBA3\u8683\uCADB\uCEA8"+
	"\uE0A7\uB3AA\u8684\uE0A6\u8685\u8686\u8687\uE0A1"+
	"\u8688\u8689\u868A\u868B\uDFFE\u868C\uCDD9\uDFFC"+
	"\u868D\uDFFA\u868E\uBFD0\uD7C4\u868F\uC9CC\u8690"+
	"\u8691\uDFF8\uB0A1\u8692\u8693\u8694\u8695\u8696"+
	"\uDFFD\u8697\u8698\u8699\u869A\uDFFB\uE0A2\u869B"+
	"\u869C\u869D\u869E\u869F\uE0A8\u86A0\u86A1\u86A2"+
	"\u86A3\uB7C8\u86A4\u86A5\uC6A1\uC9B6\uC0B2\uDFF5"+
	"\u86A6\u86A7\uC5BE\u86A8\uD8C4\uDFF9\uC4F6\u86A9"+
	"\u86AA\u86AB\u86AC\u86AD\u86AE\uE0A3\uE0A4\uE0A5"+
	"\uD0A5\u86AF\u86B0\uE0B4\uCCE4\u86B1\uE0B1\u86B2"+
	"\uBFA6\uE0AF\uCEB9\uE0AB\uC9C6\u86B3\u86B4\uC0AE"+
	"\uE0AE\uBAED\uBAB0\uE0A9\u86B5\u86B6\u86B7\uDFF6"+
	"\u86B8\uE0B3\u86B9\u86BA\uE0B8\u86BB\u86BC\u86BD"+
	"\uB4AD\uE0B9\u86BE\u86BF\uCFB2\uBAC8\u86C0\uE0B0"+
	"\u86C1\u86C2\u86C3\u86C4\u86C5\u86C6\u86C7\uD0FA"+
	"\u86C8\u86C9\u86CA\u86CB\u86CC\u86CD\u86CE\u86CF"+
	"\u86D0\uE0AC\u86D1\uD4FB\u86D2\uDFF7\u86D3\uC5E7"+
	"\u86D4\uE0AD\u86D5\uD3F7\u86D6\uE0B6\uE0B7\u86D7"+
	"\u86D8\u86D9\u86DA\u86DB\uE0C4\uD0E1\u86DC\u86DD"+
	"\u86DE\uE0BC\u86DF\u86E0\uE0C9\uE0CA\u86E1\u86E2"+
	"\u86E3\uE0BE\uE0AA\uC9A4\uE0C1\u86E4\uE0B2\u86E5"+
	"\u86E6\u86E7\u86E8\u86E9\uCAC8\uE0C3\u86EA\uE0B5"+
	"\u86EB\uCECB\u86EC\uCBC3\uE0CD\uE0C6\uE0C2\u86ED"+
	"\uE0CB\u86EE\uE0BA\uE0BF\uE0C0\u86EF\u86F0\uE0C5"+
	"\u86F1\u86F2\uE0C7\uE0C8\u86F3\uE0CC\u86F4\uE0BB"+
	"\u86F5\u86F6\u86F7\u86F8\u86F9\uCBD4\uE0D5\u86FA"+
	"\uE0D6\uE0D2\u86FB\u86FC\u86FD\u86FE\u8740\u8741"+
	"\uE0D0\uBCCE\u8742\u8743\uE0D1\u8744\uB8C2\uD8C5"+
	"\u8745\u8746\u8747\u8748\u8749\u874A\u874B\u874C"+
	"\uD0EA\u874D\u874E\uC2EF\u874F\u8750\uE0CF\uE0BD"+
	"\u8751\u8752\u8753\uE0D4\uE0D3\u8754\u8755\uE0D7"+
	"\u8756\u8757\u8758\u8759\uE0DC\uE0D8\u875A\u875B"+
	"\u875C\uD6F6\uB3B0\u875D\uD7EC\u875E\uCBBB\u875F"+
	"\u8760\uE0DA\u8761\uCEFB\u8762\u8763\u8764\uBAD9"+
	"\u8765\u8766\u8767\u8768\u8769\u876A\u876B\u876C"+
	"\u876D\u876E\u876F\u8770\uE0E1\uE0DD\uD2AD\u8771"+
	"\u8772\u8773\u8774\u8775\uE0E2\u8776\u8777\uE0DB"+
	"\uE0D9\uE0DF\u8778\u8779\uE0E0\u877A\u877B\u877C"+
	"\u877D\u877E\uE0DE\u8780\uE0E4\u8781\u8782\u8783"+
	"\uC6F7\uD8AC\uD4EB\uE0E6\uCAC9\u8784\u8785\u8786"+
	"\u8787\uE0E5\u8788\u8789\u878A\u878B\uB8C1\u878C"+
	"\u878D\u878E\u878F\uE0E7\uE0E8\u8790\u8791\u8792"+
	"\u8793\u8794\u8795\u8796\u8797\uE0E9\uE0E3\u8798"+
	"\u8799\u879A\u879B\u879C\u879D\u879E\uBABF\uCCE7"+
	"\u879F\u87A0\u87A1\uE0EA\u87A2\u87A3\u87A4\u87A5"+
	"\u87A6\u87A7\u87A8\u87A9\u87AA\u87AB\u87AC\u87AD"+
	"\u87AE\u87AF\u87B0\uCFF9\u87B1\u87B2\u87B3\u87B4"+
	"\u87B5\u87B6\u87B7\u87B8\u87B9\u87BA\u87BB\uE0EB"+
	"\u87BC\u87BD\u87BE\u87BF\u87C0\u87C1\u87C2\uC8C2"+
	"\u87C3\u87C4\u87C5\u87C6\uBDC0\u87C7\u87C8\u87C9"+
	"\u87CA\u87CB\u87CC\u87CD\u87CE\u87CF\u87D0\u87D1"+
	"\u87D2\u87D3\uC4D2\u87D4\u87D5\u87D6\u87D7\u87D8"+
	"\u87D9\u87DA\u87DB\u87DC\uE0EC\u87DD\u87DE\uE0ED"+
	"\u87DF\u87E0\uC7F4\uCBC4\u87E1\uE0EE\uBBD8\uD8B6"+
	"\uD2F2\uE0EF\uCDC5\u87E2\uB6DA\u87E3\u87E4\u87E5"+
	"\u87E6\u87E7\u87E8\uE0F1\u87E9\uD4B0\u87EA\u87EB"+
	"\uC0A7\uB4D1\u87EC\u87ED\uCEA7\uE0F0\u87EE\u87EF"+
	"\u87F0\uE0F2\uB9CC\u87F1\u87F2\uB9FA\uCDBC\uE0F3"+
	"\u87F3\u87F4\u87F5\uC6D4\uE0F4\u87F6\uD4B2\u87F7"+
	"\uC8A6\uE0F6\uE0F5\u87F8\u87F9\u87FA\u87FB\u87FC"+
	"\u87FD\u87FE\u8840\u8841\u8842\u8843\u8844\u8845"+
	"\u8846\u8847\u8848\u8849\uE0F7\u884A\u884B\uCDC1"+
	"\u884C\u884D\u884E\uCAA5\u884F\u8850\u8851\u8852"+
	"\uD4DA\uDBD7\uDBD9\u8853\uDBD8\uB9E7\uDBDC\uDBDD"+
	"\uB5D8\u8854\u8855\uDBDA\u8856\u8857\u8858\u8859"+
	"\u885A\uDBDB\uB3A1\uDBDF\u885B\u885C\uBBF8\u885D"+
	"\uD6B7\u885E\uDBE0\u885F\u8860\u8861\u8862\uBEF9"+
	"\u8863\u8864\uB7BB\u8865\uDBD0\uCCAE\uBFB2\uBBB5"+
	"\uD7F8\uBFD3\u8866\u8867\u8868\u8869\u886A\uBFE9"+
	"\u886B\u886C\uBCE1\uCCB3\uDBDE\uB0D3\uCEEB\uB7D8"+
	"\uD7B9\uC6C2\u886D\u886E\uC0A4\u886F\uCCB9\u8870"+
	"\uDBE7\uDBE1\uC6BA\uDBE3\u8871\uDBE8\u8872\uC5F7"+
	"\u8873\u8874\u8875\uDBEA\u8876\u8877\uDBE9\uBFC0"+
	"\u8878\u8879\u887A\uDBE6\uDBE5\u887B\u887C\u887D"+
	"\u887E\u8880\uB4B9\uC0AC\uC2A2\uDBE2\uDBE4\u8881"+
	"\u8882\u8883\u8884\uD0CD\uDBED\u8885\u8886\u8887"+
	"\u8888\u8889\uC0DD\uDBF2\u888A\u888B\u888C\u888D"+
	"\u888E\u888F\u8890\uB6E2\u8891\u8892\u8893\u8894"+
	"\uDBF3\uDBD2\uB9B8\uD4AB\uDBEC\u8895\uBFD1\uDBF0"+
	"\u8896\uDBD1\u8897\uB5E6\u8898\uDBEB\uBFE5\u8899"+
	"\u889A\u889B\uDBEE\u889C\uDBF1\u889D\u889E\u889F"+
	"\uDBF9\u88A0\u88A1\u88A2\u88A3\u88A4\u88A5\u88A6"+
	"\u88A7\u88A8\uB9A1\uB0A3\u88A9\u88AA\u88AB\u88AC"+
	"\u88AD\u88AE\u88AF\uC2F1\u88B0\u88B1\uB3C7\uDBEF"+
	"\u88B2\u88B3\uDBF8\u88B4\uC6D2\uDBF4\u88B5\u88B6"+
	"\uDBF5\uDBF7\uDBF6\u88B7\u88B8\uDBFE\u88B9\uD3F2"+
	"\uB2BA\u88BA\u88BB\u88BC\uDBFD\u88BD\u88BE\u88BF"+
	"\u88C0\u88C1\u88C2\u88C3\u88C4\uDCA4\u88C5\uDBFB"+
	"\u88C6\u88C7\u88C8\u88C9\uDBFA\u88CA\u88CB\u88CC"+
	"\uDBFC\uC5E0\uBBF9\u88CD\u88CE\uDCA3\u88CF\u88D0"+
	"\uDCA5\u88D1\uCCC3\u88D2\u88D3\u88D4\uB6D1\uDDC0"+
	"\u88D5\u88D6\u88D7\uDCA1\u88D8\uDCA2\u88D9\u88DA"+
	"\u88DB\uC7B5\u88DC\u88DD\u88DE\uB6E9\u88DF\u88E0"+
	"\u88E1\uDCA7\u88E2\u88E3\u88E4\u88E5\uDCA6\u88E6"+
	"\uDCA9\uB1A4\u88E7\u88E8\uB5CC\u88E9\u88EA\u88EB"+
	"\u88EC\u88ED\uBFB0\u88EE\u88EF\u88F0\u88F1\u88F2"+
	"\uD1DF\u88F3\u88F4\u88F5\u88F6\uB6C2\u88F7\u88F8"+
	"\u88F9\u88FA\u88FB\u88FC\u88FD\u88FE\u8940\u8941"+
	"\u8942\u8943\u8944\u8945\uDCA8\u8946\u8947\u8948"+
	"\u8949\u894A\u894B\u894C\uCBFA\uEBF3\u894D\u894E"+
	"\u894F\uCBDC\u8950\u8951\uCBFE\u8952\u8953\u8954"+
	"\uCCC1\u8955\u8956\u8957\u8958\u8959\uC8FB\u895A"+
	"\u895B\u895C\u895D\u895E\u895F\uDCAA\u8960\u8961"+
	"\u8962\u8963\u8964\uCCEE\uDCAB\u8965\u8966\u8967"+
	"\u8968\u8969\u896A\u896B\u896C\u896D\u896E\u896F"+
	"\u8970\u8971\u8972\u8973\u8974\u8975\uDBD3\u8976"+
	"\uDCAF\uDCAC\u8977\uBEB3\u8978\uCAFB\u8979\u897A"+
	"\u897B\uDCAD\u897C\u897D\u897E\u8980\u8981\u8982"+
	"\u8983\u8984\uC9CA\uC4B9\u8985\u8986\u8987\u8988"+
	"\u8989\uC7BD\uDCAE\u898A\u898B\u898C\uD4F6\uD0E6"+
	"\u898D\u898E\u898F\u8990\u8991\u8992\u8993\u8994"+
	"\uC4AB\uB6D5\u8995\u8996\u8997\u8998\u8999\u899A"+
	"\u899B\u899C\u899D\u899E\u899F\u89A0\u89A1\u89A2"+
	"\u89A3\u89A4\u89A5\u89A6\uDBD4\u89A7\u89A8\u89A9"+
	"\u89AA\uB1DA\u89AB\u89AC\u89AD\uDBD5\u89AE\u89AF"+
	"\u89B0\u89B1\u89B2\u89B3\u89B4\u89B5\u89B6\u89B7"+
	"\u89B8\uDBD6\u89B9\u89BA\u89BB\uBABE\u89BC\u89BD"+
	"\u89BE\u89BF\u89C0\u89C1\u89C2\u89C3\u89C4\u89C5"+
	"\u89C6\u89C7\u89C8\u89C9\uC8C0\u89CA\u89CB\u89CC"+
	"\u89CD\u89CE\u89CF\uCABF\uC8C9\u89D0\uD7B3\u89D1"+
	"\uC9F9\u89D2\u89D3\uBFC7\u89D4\u89D5\uBAF8\u89D6"+
	"\u89D7\uD2BC\u89D8\u89D9\u89DA\u89DB\u89DC\u89DD"+
	"\u89DE\u89DF\uE2BA\u89E0\uB4A6\u89E1\u89E2\uB1B8"+
	"\u89E3\u89E4\u89E5\u89E6\u89E7\uB8B4\u89E8\uCFC4"+
	"\u89E9\u89EA\u89EB\u89EC\uD9E7\uCFA6\uCDE2\u89ED"+
	"\u89EE\uD9ED\uB6E0\u89EF\uD2B9\u89F0\u89F1\uB9BB"+
	"\u89F2\u89F3\u89F4\u89F5\uE2B9\uE2B7\u89F6\uB4F3"+
	"\u89F7\uCCEC\uCCAB\uB7F2\u89F8\uD8B2\uD1EB\uBABB"+
	"\u89F9\uCAA7\u89FA\u89FB\uCDB7\u89FC\u89FD\uD2C4"+
	"\uBFE4\uBCD0\uB6E1\u89FE\uDEC5\u8A40\u8A41\u8A42"+
	"\u8A43\uDEC6\uDBBC\u8A44\uD1D9\u8A45\u8A46\uC6E6"+
	"\uC4CE\uB7EE\u8A47\uB7DC\u8A48\u8A49\uBFFC\uD7E0"+
	"\u8A4A\uC6F5\u8A4B\u8A4C\uB1BC\uDEC8\uBDB1\uCCD7"+
	"\uDECA\u8A4D\uDEC9\u8A4E\u8A4F\u8A50\u8A51\u8A52"+
	"\uB5EC\u8A53\uC9DD\u8A54\u8A55\uB0C2\u8A56\u8A57"+
	"\u8A58\u8A59\u8A5A\u8A5B\u8A5C\u8A5D\u8A5E\u8A5F"+
	"\u8A60\u8A61\u8A62\uC5AE\uC5AB\u8A63\uC4CC\u8A64"+
	"\uBCE9\uCBFD\u8A65\u8A66\u8A67\uBAC3\u8A68\u8A69"+
	"\u8A6A\uE5F9\uC8E7\uE5FA\uCDFD\u8A6B\uD7B1\uB8BE"+
	"\uC2E8\u8A6C\uC8D1\u8A6D\u8A6E\uE5FB\u8A6F\u8A70"+
	"\u8A71\u8A72\uB6CA\uBCCB\u8A73\u8A74\uD1FD\uE6A1"+
	"\u8A75\uC3EE\u8A76\u8A77\u8A78\u8A79\uE6A4\u8A7A"+
	"\u8A7B\u8A7C\u8A7D\uE5FE\uE6A5\uCDD7\u8A7E\u8A80"+
	"\uB7C1\uE5FC\uE5FD\uE6A3\u8A81\u8A82\uC4DD\uE6A8"+
	"\u8A83\u8A84\uE6A7\u8A85\u8A86\u8A87\u8A88\u8A89"+
	"\u8A8A\uC3C3\u8A8B\uC6DE\u8A8C\u8A8D\uE6AA\u8A8E"+
	"\u8A8F\u8A90\u8A91\u8A92\u8A93\u8A94\uC4B7\u8A95"+
	"\u8A96\u8A97\uE6A2\uCABC\u8A98\u8A99\u8A9A\u8A9B"+
	"\uBDE3\uB9C3\uE6A6\uD0D5\uCEAF\u8A9C\u8A9D\uE6A9"+
	"\uE6B0\u8A9E\uD2A6\u8A9F\uBDAA\uE6AD\u8AA0\u8AA1"+
	"\u8AA2\u8AA3\u8AA4\uE6AF\u8AA5\uC0D1\u8AA6\u8AA7"+
	"\uD2CC\u8AA8\u8AA9\u8AAA\uBCA7\u8AAB\u8AAC\u8AAD"+
	"\u8AAE\u8AAF\u8AB0\u8AB1\u8AB2\u8AB3\u8AB4\u8AB5"+
	"\u8AB6\uE6B1\u8AB7\uD2F6\u8AB8\u8AB9\u8ABA\uD7CB"+
	"\u8ABB\uCDFE\u8ABC\uCDDE\uC2A6\uE6AB\uE6AC\uBDBF"+
	"\uE6AE\uE6B3\u8ABD\u8ABE\uE6B2\u8ABF\u8AC0\u8AC1"+
	"\u8AC2\uE6B6\u8AC3\uE6B8\u8AC4\u8AC5\u8AC6\u8AC7"+
	"\uC4EF\u8AC8\u8AC9\u8ACA\uC4C8\u8ACB\u8ACC\uBEEA"+
	"\uC9EF\u8ACD\u8ACE\uE6B7\u8ACF\uB6F0\u8AD0\u8AD1"+
	"\u8AD2\uC3E4\u8AD3\u8AD4\u8AD5\u8AD6\u8AD7\u8AD8"+
	"\u8AD9\uD3E9\uE6B4\u8ADA\uE6B5\u8ADB\uC8A2\u8ADC"+
	"\u8ADD\u8ADE\u8ADF\u8AE0\uE6BD\u8AE1\u8AE2\u8AE3"+
	"\uE6B9\u8AE4\u8AE5\u8AE6\u8AE7\u8AE8\uC6C5\u8AE9"+
	"\u8AEA\uCDF1\uE6BB\u8AEB\u8AEC\u8AED\u8AEE\u8AEF"+
	"\u8AF0\u8AF1\u8AF2\u8AF3\u8AF4\uE6BC\u8AF5\u8AF6"+
	"\u8AF7\u8AF8\uBBE9\u8AF9\u8AFA\u8AFB\u8AFC\u8AFD"+
	"\u8AFE\u8B40\uE6BE\u8B41\u8B42\u8B43\u8B44\uE6BA"+
	"\u8B45\u8B46\uC0B7\u8B47\u8B48\u8B49\u8B4A\u8B4B"+
	"\u8B4C\u8B4D\u8B4E\u8B4F\uD3A4\uE6BF\uC9F4\uE6C3"+
	"\u8B50\u8B51\uE6C4\u8B52\u8B53\u8B54\u8B55\uD0F6"+
	"\u8B56\u8B57\u8B58\u8B59\u8B5A\u8B5B\u8B5C\u8B5D"+
	"\u8B5E\u8B5F\u8B60\u8B61\u8B62\u8B63\u8B64\u8B65"+
	"\u8B66\u8B67\uC3BD\u8B68\u8B69\u8B6A\u8B6B\u8B6C"+
	"\u8B6D\u8B6E\uC3C4\uE6C2\u8B6F\u8B70\u8B71\u8B72"+
	"\u8B73\u8B74\u8B75\u8B76\u8B77\u8B78\u8B79\u8B7A"+
	"\u8B7B\u8B7C\uE6C1\u8B7D\u8B7E\u8B80\u8B81\u8B82"+
	"\u8B83\u8B84\uE6C7\uCFB1\u8B85\uEBF4\u8B86\u8B87"+
	"\uE6CA\u8B88\u8B89\u8B8A\u8B8B\u8B8C\uE6C5\u8B8D"+
	"\u8B8E\uBCDE\uC9A9\u8B8F\u8B90\u8B91\u8B92\u8B93"+
	"\u8B94\uBCB5\u8B95\u8B96\uCFD3\u8B97\u8B98\u8B99"+
	"\u8B9A\u8B9B\uE6C8\u8B9C\uE6C9\u8B9D\uE6CE\u8B9E"+
	"\uE6D0\u8B9F\u8BA0\u8BA1\uE6D1\u8BA2\u8BA3\u8BA4"+
	"\uE6CB\uB5D5\u8BA5\uE6CC\u8BA6\u8BA7\uE6CF\u8BA8"+
	"\u8BA9\uC4DB\u8BAA\uE6C6\u8BAB\u8BAC\u8BAD\u8BAE"+
	"\u8BAF\uE6CD\u8BB0\u8BB1\u8BB2\u8BB3\u8BB4\u8BB5"+
	"\u8BB6\u8BB7\u8BB8\u8BB9\u8BBA\u8BBB\u8BBC\u8BBD"+
	"\u8BBE\u8BBF\u8BC0\u8BC1\u8BC2\u8BC3\u8BC4\u8BC5"+
	"\u8BC6\uE6D2\u8BC7\u8BC8\u8BC9\u8BCA\u8BCB\u8BCC"+
	"\u8BCD\u8BCE\u8BCF\u8BD0\u8BD1\u8BD2\uE6D4\uE6D3"+
	"\u8BD3\u8BD4\u8BD5\u8BD6\u8BD7\u8BD8\u8BD9\u8BDA"+
	"\u8BDB\u8BDC\u8BDD\u8BDE\u8BDF\u8BE0\u8BE1\u8BE2"+
	"\u8BE3\u8BE4\u8BE5\u8BE6\u8BE7\u8BE8\u8BE9\u8BEA"+
	"\u8BEB\u8BEC\uE6D5\u8BED\uD9F8\u8BEE\u8BEF\uE6D6"+
	"\u8BF0\u8BF1\u8BF2\u8BF3\u8BF4\u8BF5\u8BF6\u8BF7"+
	"\uE6D7\u8BF8\u8BF9\u8BFA\u8BFB\u8BFC\u8BFD\u8BFE"+
	"\u8C40\u8C41\u8C42\u8C43\u8C44\u8C45\u8C46\u8C47"+
	"\uD7D3\uE6DD\u8C48\uE6DE\uBFD7\uD4D0\u8C49\uD7D6"+
	"\uB4E6\uCBEF\uE6DA\uD8C3\uD7CE\uD0A2\u8C4A\uC3CF"+
	"\u8C4B\u8C4C\uE6DF\uBCBE\uB9C2\uE6DB\uD1A7\u8C4D"+
	"\u8C4E\uBAA2\uC2CF\u8C4F\uD8AB\u8C50\u8C51\u8C52"+
	"\uCAEB\uE5EE\u8C53\uE6DC\u8C54\uB7F5\u8C55\u8C56"+
	"\u8C57\u8C58\uC8E6\u8C59\u8C5A\uC4F5\u8C5B\u8C5C"+
	"\uE5B2\uC4FE\u8C5D\uCBFC\uE5B3\uD5AC\u8C5E\uD3EE"+
	"\uCAD8\uB0B2\u8C5F\uCBCE\uCDEA\u8C60\u8C61\uBAEA"+
	"\u8C62\u8C63\u8C64\uE5B5\u8C65\uE5B4\u8C66\uD7DA"+
	"\uB9D9\uD6E6\uB6A8\uCDF0\uD2CB\uB1A6\uCAB5\u8C67"+
	"\uB3E8\uC9F3\uBFCD\uD0FB\uCAD2\uE5B6\uBBC2\u8C68"+
	"\u8C69\u8C6A\uCFDC\uB9AC\u8C6B\u8C6C\u8C6D\u8C6E"+
	"\uD4D7\u8C6F\u8C70\uBAA6\uD1E7\uCFFC\uBCD2\u8C71"+
	"\uE5B7\uC8DD\u8C72\u8C73\u8C74\uBFED\uB1F6\uCBDE"+
	"\u8C75\u8C76\uBCC5\u8C77\uBCC4\uD2FA\uC3DC\uBFDC"+
	"\u8C78\u8C79\u8C7A\u8C7B\uB8BB\u8C7C\u8C7D\u8C7E"+
	"\uC3C2\u8C80\uBAAE\uD4A2\u8C81\u8C82\u8C83\u8C84"+
	"\u8C85\u8C86\u8C87\u8C88\u8C89\uC7DE\uC4AF\uB2EC"+
	"\u8C8A\uB9D1\u8C8B\u8C8C\uE5BB\uC1C8\u8C8D\u8C8E"+
	"\uD5AF\u8C8F\u8C90\u8C91\u8C92\u8C93\uE5BC\u8C94"+
	"\uE5BE\u8C95\u8C96\u8C97\u8C98\u8C99\u8C9A\u8C9B"+
	"\uB4E7\uB6D4\uCBC2\uD1B0\uB5BC\u8C9C\u8C9D\uCAD9"+
	"\u8C9E\uB7E2\u8C9F\u8CA0\uC9E4\u8CA1\uBDAB\u8CA2"+
	"\u8CA3\uCEBE\uD7F0\u8CA4\u8CA5\u8CA6\u8CA7\uD0A1"+
	"\u8CA8\uC9D9\u8CA9\u8CAA\uB6FB\uE6D8\uBCE2\u8CAB"+
	"\uB3BE\u8CAC\uC9D0\u8CAD\uE6D9\uB3A2\u8CAE\u8CAF"+
	"\u8CB0\u8CB1\uDECC\u8CB2\uD3C8\uDECD\u8CB3\uD2A2"+
	"\u8CB4\u8CB5\u8CB6\u8CB7\uDECE\u8CB8\u8CB9\u8CBA"+
	"\u8CBB\uBECD\u8CBC\u8CBD\uDECF\u8CBE\u8CBF\u8CC0"+
	"\uCAAC\uD2FC\uB3DF\uE5EA\uC4E1\uBEA1\uCEB2\uC4F2"+
	"\uBED6\uC6A8\uB2E3\u8CC1\u8CC2\uBED3\u8CC3\u8CC4"+
	"\uC7FC\uCCEB\uBDEC\uCEDD\u8CC5\u8CC6\uCABA\uC6C1"+
	"\uE5EC\uD0BC\u8CC7\u8CC8\u8CC9\uD5B9\u8CCA\u8CCB"+
	"\u8CCC\uE5ED\u8CCD\u8CCE\u8CCF\u8CD0\uCAF4\u8CD1"+
	"\uCDC0\uC2C5\u8CD2\uE5EF\u8CD3\uC2C4\uE5F0\u8CD4"+
	"\u8CD5\u8CD6\u8CD7\u8CD8\u8CD9\u8CDA\uE5F8\uCDCD"+
	"\u8CDB\uC9BD\u8CDC\u8CDD\u8CDE\u8CDF\u8CE0\u8CE1"+
	"\u8CE2\uD2D9\uE1A8\u8CE3\u8CE4\u8CE5\u8CE6\uD3EC"+
	"\u8CE7\uCBEA\uC6F1\u8CE8\u8CE9\u8CEA\u8CEB\u8CEC"+
	"\uE1AC\u8CED\u8CEE\u8CEF\uE1A7\uE1A9\u8CF0\u8CF1"+
	"\uE1AA\uE1AF\u8CF2\u8CF3\uB2ED\u8CF4\uE1AB\uB8DA"+
	"\uE1AD\uE1AE\uE1B0\uB5BA\uE1B1\u8CF5\u8CF6\u8CF7"+
	"\u8CF8\u8CF9\uE1B3\uE1B8\u8CFA\u8CFB\u8CFC\u8CFD"+
	"\u8CFE\uD1D2\u8D40\uE1B6\uE1B5\uC1EB\u8D41\u8D42"+
	"\u8D43\uE1B7\u8D44\uD4C0\u8D45\uE1B2\u8D46\uE1BA"+
	"\uB0B6\u8D47\u8D48\u8D49\u8D4A\uE1B4\u8D4B\uBFF9"+
	"\u8D4C\uE1B9\u8D4D\u8D4E\uE1BB\u8D4F\u8D50\u8D51"+
	"\u8D52\u8D53\u8D54\uE1BE\u8D55\u8D56\u8D57\u8D58"+
	"\u8D59\u8D5A\uE1BC\u8D5B\u8D5C\u8D5D\u8D5E\u8D5F"+
	"\u8D60\uD6C5\u8D61\u8D62\u8D63\u8D64\u8D65\u8D66"+
	"\u8D67\uCFBF\u8D68\u8D69\uE1BD\uE1BF\uC2CD\u8D6A"+
	"\uB6EB\u8D6B\uD3F8\u8D6C\u8D6D\uC7CD\u8D6E\u8D6F"+
	"\uB7E5\u8D70\u8D71\u8D72\u8D73\u8D74\u8D75\u8D76"+
	"\u8D77\u8D78\u8D79\uBEFE\u8D7A\u8D7B\u8D7C\u8D7D"+
	"\u8D7E\u8D80\uE1C0\uE1C1\u8D81\u8D82\uE1C7\uB3E7"+
	"\u8D83\u8D84\u8D85\u8D86\u8D87\u8D88\uC6E9\u8D89"+
	"\u8D8A\u8D8B\u8D8C\u8D8D\uB4DE\u8D8E\uD1C2\u8D8F"+
	"\u8D90\u8D91\u8D92\uE1C8\u8D93\u8D94\uE1C6\u8D95"+
	"\u8D96\u8D97\u8D98\u8D99\uE1C5\u8D9A\uE1C3\uE1C2"+
	"\u8D9B\uB1C0\u8D9C\u8D9D\u8D9E\uD5B8\uE1C4\u8D9F"+
	"\u8DA0\u8DA1\u8DA2\u8DA3\uE1CB\u8DA4\u8DA5\u8DA6"+
	"\u8DA7\u8DA8\u8DA9\u8DAA\u8DAB\uE1CC\uE1CA\u8DAC"+
	"\u8DAD\u8DAE\u8DAF\u8DB0\u8DB1\u8DB2\u8DB3\uEFFA"+
	"\u8DB4\u8DB5\uE1D3\uE1D2\uC7B6\u8DB6\u8DB7\u8DB8"+
	"\u8DB9\u8DBA\u8DBB\u8DBC\u8DBD\u8DBE\u8DBF\u8DC0"+
	"\uE1C9\u8DC1\u8DC2\uE1CE\u8DC3\uE1D0\u8DC4\u8DC5"+
	"\u8DC6\u8DC7\u8DC8\u8DC9\u8DCA\u8DCB\u8DCC\u8DCD"+
	"\u8DCE\uE1D4\u8DCF\uE1D1\uE1CD\u8DD0\u8DD1\uE1CF"+
	"\u8DD2\u8DD3\u8DD4\u8DD5\uE1D5\u8DD6\u8DD7\u8DD8"+
	"\u8DD9\u8DDA\u8DDB\u8DDC\u8DDD\u8DDE\u8DDF\u8DE0"+
	"\u8DE1\u8DE2\uE1D6\u8DE3\u8DE4\u8DE5\u8DE6\u8DE7"+
	"\u8DE8\u8DE9\u8DEA\u8DEB\u8DEC\u8DED\u8DEE\u8DEF"+
	"\u8DF0\u8DF1\u8DF2\u8DF3\u8DF4\u8DF5\u8DF6\u8DF7"+
	"\u8DF8\uE1D7\u8DF9\u8DFA\u8DFB\uE1D8\u8DFC\u8DFD"+
	"\u8DFE\u8E40\u8E41\u8E42\u8E43\u8E44\u8E45\u8E46"+
	"\u8E47\u8E48\u8E49\u8E4A\u8E4B\u8E4C\u8E4D\u8E4E"+
	"\u8E4F\u8E50\u8E51\u8E52\u8E53\u8E54\u8E55\uE1DA"+
	"\u8E56\u8E57\u8E58\u8E59\u8E5A\u8E5B\u8E5C\u8E5D"+
	"\u8E5E\u8E5F\u8E60\u8E61\u8E62\uE1DB\u8E63\u8E64"+
	"\u8E65\u8E66\u8E67\u8E68\u8E69\uCEA1\u8E6A\u8E6B"+
	"\u8E6C\u8E6D\u8E6E\u8E6F\u8E70\u8E71\u8E72\u8E73"+
	"\u8E74\u8E75\u8E76\uE7DD\u8E77\uB4A8\uD6DD\u8E78"+
	"\u8E79\uD1B2\uB3B2\u8E7A\u8E7B\uB9A4\uD7F3\uC7C9"+
	"\uBEDE\uB9AE\u8E7C\uCED7\u8E7D\u8E7E\uB2EE\uDBCF"+
	"\u8E80\uBCBA\uD2D1\uCBC8\uB0CD\u8E81\u8E82\uCFEF"+
	"\u8E83\u8E84\u8E85\u8E86\u8E87\uD9E3\uBDED\u8E88"+
	"\u8E89\uB1D2\uCAD0\uB2BC\u8E8A\uCBA7\uB7AB\u8E8B"+
	"\uCAA6\u8E8C\u8E8D\u8E8E\uCFA3\u8E8F\u8E90\uE0F8"+
	"\uD5CA\uE0FB\u8E91\u8E92\uE0FA\uC5C1\uCCFB\u8E93"+
	"\uC1B1\uE0F9\uD6E3\uB2AF\uD6C4\uB5DB\u8E94\u8E95"+
	"\u8E96\u8E97\u8E98\u8E99\u8E9A\u8E9B\uB4F8\uD6A1"+
	"\u8E9C\u8E9D\u8E9E\u8E9F\u8EA0\uCFAF\uB0EF\u8EA1"+
	"\u8EA2\uE0FC\u8EA3\u8EA4\u8EA5\u8EA6\u8EA7\uE1A1"+
	"\uB3A3\u8EA8\u8EA9\uE0FD\uE0FE\uC3B1\u8EAA\u8EAB"+
	"\u8EAC\u8EAD\uC3DD\u8EAE\uE1A2\uB7F9\u8EAF\u8EB0"+
	"\u8EB1\u8EB2\u8EB3\u8EB4\uBBCF\u8EB5\u8EB6\u8EB7"+
	"\u8EB8\u8EB9\u8EBA\u8EBB\uE1A3\uC4BB\u8EBC\u8EBD"+
	"\u8EBE\u8EBF\u8EC0\uE1A4\u8EC1\u8EC2\uE1A5\u8EC3"+
	"\u8EC4\uE1A6\uB4B1\u8EC5\u8EC6\u8EC7\u8EC8\u8EC9"+
	"\u8ECA\u8ECB\u8ECC\u8ECD\u8ECE\u8ECF\u8ED0\u8ED1"+
	"\u8ED2\u8ED3\uB8C9\uC6BD\uC4EA\u8ED4\uB2A2\u8ED5"+
	"\uD0D2\u8ED6\uE7DB\uBBC3\uD3D7\uD3C4\u8ED7\uB9E3"+
	"\uE2CF\u8ED8\u8ED9\u8EDA\uD7AF\u8EDB\uC7EC\uB1D3"+
	"\u8EDC\u8EDD\uB4B2\uE2D1\u8EDE\u8EDF\u8EE0\uD0F2"+
	"\uC2AE\uE2D0\u8EE1\uBFE2\uD3A6\uB5D7\uE2D2\uB5EA"+
	"\u8EE2\uC3ED\uB8FD\u8EE3\uB8AE\u8EE4\uC5D3\uB7CF"+
	"\uE2D4\u8EE5\u8EE6\u8EE7\u8EE8\uE2D3\uB6C8\uD7F9"+
	"\u8EE9\u8EEA\u8EEB\u8EEC\u8EED\uCDA5\u8EEE\u8EEF"+
	"\u8EF0\u8EF1\u8EF2\uE2D8\u8EF3\uE2D6\uCAFC\uBFB5"+
	"\uD3B9\uE2D5\u8EF4\u8EF5\u8EF6\u8EF7\uE2D7\u8EF8"+
	"\u8EF9\u8EFA\u8EFB\u8EFC\u8EFD\u8EFE\u8F40\u8F41"+
	"\u8F42\uC1AE\uC0C8\u8F43\u8F44\u8F45\u8F46\u8F47"+
	"\u8F48\uE2DB\uE2DA\uC0AA\u8F49\u8F4A\uC1CE\u8F4B"+
	"\u8F4C\u8F4D\u8F4E\uE2DC\u8F4F\u8F50\u8F51\u8F52"+
	"\u8F53\u8F54\u8F55\u8F56\u8F57\u8F58\u8F59\u8F5A"+
	"\uE2DD\u8F5B\uE2DE\u8F5C\u8F5D\u8F5E\u8F5F\u8F60"+
	"\u8F61\u8F62\u8F63\u8F64\uDBC8\u8F65\uD1D3\uCDA2"+
	"\u8F66\u8F67\uBDA8\u8F68\u8F69\u8F6A\uDEC3\uD8A5"+
	"\uBFAA\uDBCD\uD2EC\uC6FA\uC5AA\u8F6B\u8F6C\u8F6D"+
	"\uDEC4\u8F6E\uB1D7\uDFAE\u8F6F\u8F70\u8F71\uCABD"+
	"\u8F72\uDFB1\u8F73\uB9AD\u8F74\uD2FD\u8F75\uB8A5"+
	"\uBAEB\u8F76\u8F77\uB3DA\u8F78\u8F79\u8F7A\uB5DC"+
	"\uD5C5\u8F7B\u8F7C\u8F7D\u8F7E\uC3D6\uCFD2\uBBA1"+
	"\u8F80\uE5F3\uE5F2\u8F81\u8F82\uE5F4\u8F83\uCDE4"+
	"\u8F84\uC8F5\u8F85\u8F86\u8F87\u8F88\u8F89\u8F8A"+
	"\u8F8B\uB5AF\uC7BF\u8F8C\uE5F6\u8F8D\u8F8E\u8F8F"+
	"\uECB0\u8F90\u8F91\u8F92\u8F93\u8F94\u8F95\u8F96"+
	"\u8F97\u8F98\u8F99\u8F9A\u8F9B\u8F9C\u8F9D\u8F9E"+
	"\uE5E6\u8F9F\uB9E9\uB5B1\u8FA0\uC2BC\uE5E8\uE5E7"+
	"\uE5E9\u8FA1\u8FA2\u8FA3\u8FA4\uD2CD\u8FA5\u8FA6"+
	"\u8FA7\uE1EA\uD0CE\u8FA8\uCDAE\u8FA9\uD1E5\u8FAA"+
	"\u8FAB\uB2CA\uB1EB\u8FAC\uB1F2\uC5ED\u8FAD\u8FAE"+
	"\uD5C3\uD3B0\u8FAF\uE1DC\u8FB0\u8FB1\u8FB2\uE1DD"+
	"\u8FB3\uD2DB\u8FB4\uB3B9\uB1CB\u8FB5\u8FB6\u8FB7"+
	"\uCDF9\uD5F7\uE1DE\u8FB8\uBEB6\uB4FD\u8FB9\uE1DF"+
	"\uBADC\uE1E0\uBBB2\uC2C9\uE1E1\u8FBA\u8FBB\u8FBC"+
	"\uD0EC\u8FBD\uCDBD\u8FBE\u8FBF\uE1E2\u8FC0\uB5C3"+
	"\uC5C7\uE1E3\u8FC1\u8FC2\uE1E4\u8FC3\u8FC4\u8FC5"+
	"\u8FC6\uD3F9\u8FC7\u8FC8\u8FC9\u8FCA\u8FCB\u8FCC"+
	"\uE1E5\u8FCD\uD1AD\u8FCE\u8FCF\uE1E6\uCEA2\u8FD0"+
	"\u8FD1\u8FD2\u8FD3\u8FD4\u8FD5\uE1E7\u8FD6\uB5C2"+
	"\u8FD7\u8FD8\u8FD9\u8FDA\uE1E8\uBBD5\u8FDB\u8FDC"+
	"\u8FDD\u8FDE\u8FDF\uD0C4\uE2E0\uB1D8\uD2E4\u8FE0"+
	"\u8FE1\uE2E1\u8FE2\u8FE3\uBCC9\uC8CC\u8FE4\uE2E3"+
	"\uECFE\uECFD\uDFAF\u8FE5\u8FE6\u8FE7\uE2E2\uD6BE"+
	"\uCDFC\uC3A6\u8FE8\u8FE9\u8FEA\uE3C3\u8FEB\u8FEC"+
	"\uD6D2\uE2E7\u8FED\u8FEE\uE2E8\u8FEF\u8FF0\uD3C7"+
	"\u8FF1\u8FF2\uE2EC\uBFEC\u8FF3\uE2ED\uE2E5\u8FF4"+
	"\u8FF5\uB3C0\u8FF6\u8FF7\u8FF8\uC4EE\u8FF9\u8FFA"+
	"\uE2EE\u8FFB\u8FFC\uD0C3\u8FFD\uBAF6\uE2E9\uB7DE";

    private final static String innerIndex6= 
	"\uBBB3\uCCAC\uCBCB\uE2E4\uE2E6\uE2EA\uE2EB\u8FFE"+
	"\u9040\u9041\uE2F7\u9042\u9043\uE2F4\uD4F5\uE2F3"+
	"\u9044\u9045\uC5AD\u9046\uD5FA\uC5C2\uB2C0\u9047"+
	"\u9048\uE2EF\u9049\uE2F2\uC1AF\uCBBC\u904A\u904B"+
	"\uB5A1\uE2F9\u904C\u904D\u904E\uBCB1\uE2F1\uD0D4"+
	"\uD4B9\uE2F5\uB9D6\uE2F6\u904F\u9050\u9051\uC7D3"+
	"\u9052\u9053\u9054\u9055\u9056\uE2F0\u9057\u9058"+
	"\u9059\u905A\u905B\uD7DC\uEDA1\u905C\u905D\uE2F8"+
	"\u905E\uEDA5\uE2FE\uCAD1\u905F\u9060\u9061\u9062"+
	"\u9063\u9064\u9065\uC1B5\u9066\uBBD0\u9067\u9068"+
	"\uBFD6\u9069\uBAE3\u906A\u906B\uCBA1\u906C\u906D"+
	"\u906E\uEDA6\uEDA3\u906F\u9070\uEDA2\u9071\u9072"+
	"\u9073\u9074\uBBD6\uEDA7\uD0F4\u9075\u9076\uEDA4"+
	"\uBADE\uB6F7\uE3A1\uB6B2\uCCF1\uB9A7\u9077\uCFA2"+
	"\uC7A1\u9078\u9079\uBFD2\u907A\u907B\uB6F1\u907C"+
	"\uE2FA\uE2FB\uE2FD\uE2FC\uC4D5\uE3A2\u907D\uD3C1"+
	"\u907E\u9080\u9081\uE3A7\uC7C4\u9082\u9083\u9084"+
	"\u9085\uCFA4\u9086\u9087\uE3A9\uBAB7\u9088\u9089"+
	"\u908A\u908B\uE3A8\u908C\uBBDA\u908D\uE3A3\u908E"+
	"\u908F\u9090\uE3A4\uE3AA\u9091\uE3A6\u9092\uCEF2"+
	"\uD3C6\u9093\u9094\uBBBC\u9095\u9096\uD4C3\u9097"+
	"\uC4FA\u9098\u9099\uEDA8\uD0FC\uE3A5\u909A\uC3F5"+
	"\u909B\uE3AD\uB1AF\u909C\uE3B2\u909D\u909E\u909F"+
	"\uBCC2\u90A0\u90A1\uE3AC\uB5BF\u90A2\u90A3\u90A4"+
	"\u90A5\u90A6\u90A7\u90A8\u90A9\uC7E9\uE3B0\u90AA"+
	"\u90AB\u90AC\uBEAA\uCDEF\u90AD\u90AE\u90AF\u90B0"+
	"\u90B1\uBBF3\u90B2\u90B3\u90B4\uCCE8\u90B5\u90B6"+
	"\uE3AF\u90B7\uE3B1\u90B8\uCFA7\uE3AE\u90B9\uCEA9"+
	"\uBBDD\u90BA\u90BB\u90BC\u90BD\u90BE\uB5EB\uBEE5"+
	"\uB2D2\uB3CD\u90BF\uB1B9\uE3AB\uB2D1\uB5AC\uB9DF"+
	"\uB6E8\u90C0\u90C1\uCFEB\uE3B7\u90C2\uBBCC\u90C3"+
	"\u90C4\uC8C7\uD0CA\u90C5\u90C6\u90C7\u90C8\u90C9"+
	"\uE3B8\uB3EE\u90CA\u90CB\u90CC\u90CD\uEDA9\u90CE"+
	"\uD3FA\uD3E4\u90CF\u90D0\u90D1\uEDAA\uE3B9\uD2E2"+
	"\u90D2\u90D3\u90D4\u90D5\u90D6\uE3B5\u90D7\u90D8"+
	"\u90D9\u90DA\uD3DE\u90DB\u90DC\u90DD\u90DE\uB8D0"+
	"\uE3B3\u90DF\u90E0\uE3B6\uB7DF\u90E1\uE3B4\uC0A2"+
	"\u90E2\u90E3\u90E4\uE3BA\u90E5\u90E6\u90E7\u90E8"+
	"\u90E9\u90EA\u90EB\u90EC\u90ED\u90EE\u90EF\u90F0"+
	"\u90F1\u90F2\u90F3\u90F4\u90F5\u90F6\u90F7\uD4B8"+
	"\u90F8\u90F9\u90FA\u90FB\u90FC\u90FD\u90FE\u9140"+
	"\uB4C8\u9141\uE3BB\u9142\uBBC5\u9143\uC9F7\u9144"+
	"\u9145\uC9E5\u9146\u9147\u9148\uC4BD\u9149\u914A"+
	"\u914B\u914C\u914D\u914E\u914F\uEDAB\u9150\u9151"+
	"\u9152\u9153\uC2FD\u9154\u9155\u9156\u9157\uBBDB"+
	"\uBFAE\u9158\u9159\u915A\u915B\u915C\u915D\u915E"+
	"\uCEBF\u915F\u9160\u9161\u9162\uE3BC\u9163\uBFB6"+
	"\u9164\u9165\u9166\u9167\u9168\u9169\u916A\u916B"+
	"\u916C\u916D\u916E\u916F\u9170\u9171\u9172\u9173"+
	"\u9174\u9175\u9176\uB1EF\u9177\u9178\uD4F7\u9179"+
	"\u917A\u917B\u917C\u917D\uE3BE\u917E\u9180\u9181"+
	"\u9182\u9183\u9184\u9185\u9186\uEDAD\u9187\u9188"+
	"\u9189\u918A\u918B\u918C\u918D\u918E\u918F\uE3BF"+
	"\uBAA9\uEDAC\u9190\u9191\uE3BD\u9192\u9193\u9194"+
	"\u9195\u9196\u9197\u9198\u9199\u919A\u919B\uE3C0"+
	"\u919C\u919D\u919E\u919F\u91A0\u91A1\uBAB6\u91A2"+
	"\u91A3\u91A4\uB6AE\u91A5\u91A6\u91A7\u91A8\u91A9"+
	"\uD0B8\u91AA\uB0C3\uEDAE\u91AB\u91AC\u91AD\u91AE"+
	"\u91AF\uEDAF\uC0C1\u91B0\uE3C1\u91B1\u91B2\u91B3"+
	"\u91B4\u91B5\u91B6\u91B7\u91B8\u91B9\u91BA\u91BB"+
	"\u91BC\u91BD\u91BE\u91BF\u91C0\u91C1\uC5B3\u91C2"+
	"\u91C3\u91C4\u91C5\u91C6\u91C7\u91C8\u91C9\u91CA"+
	"\u91CB\u91CC\u91CD\u91CE\u91CF\uE3C2\u91D0\u91D1"+
	"\u91D2\u91D3\u91D4\u91D5\u91D6\u91D7\u91D8\uDCB2"+
	"\u91D9\u91DA\u91DB\u91DC\u91DD\u91DE\uEDB0\u91DF"+
	"\uB8EA\u91E0\uCEEC\uEAA7\uD0E7\uCAF9\uC8D6\uCFB7"+
	"\uB3C9\uCED2\uBDE4\u91E1\u91E2\uE3DE\uBBF2\uEAA8"+
	"\uD5BD\u91E3\uC6DD\uEAA9\u91E4\u91E5\u91E6\uEAAA"+
	"\u91E7\uEAAC\uEAAB\u91E8\uEAAE\uEAAD\u91E9\u91EA"+
	"\u91EB\u91EC\uBDD8\u91ED\uEAAF\u91EE\uC2BE\u91EF"+
	"\u91F0\u91F1\u91F2\uB4C1\uB4F7\u91F3\u91F4\uBBA7"+
	"\u91F5\u91F6\u91F7\u91F8\u91F9\uECE6\uECE5\uB7BF"+
	"\uCBF9\uB1E2\u91FA\uECE7\u91FB\u91FC\u91FD\uC9C8"+
	"\uECE8\uECE9\u91FE\uCAD6\uDED0\uB2C5\uD4FA\u9240"+
	"\u9241\uC6CB\uB0C7\uB4F2\uC8D3\u9242\u9243\u9244"+
	"\uCDD0\u9245\u9246\uBFB8\u9247\u9248\u9249\u924A"+
	"\u924B\u924C\u924D\uBFDB\u924E\u924F\uC7A4\uD6B4"+
	"\u9250\uC0A9\uDED1\uC9A8\uD1EF\uC5A4\uB0E7\uB3B6"+
	"\uC8C5\u9251\u9252\uB0E2\u9253\u9254\uB7F6\u9255"+
	"\u9256\uC5FA\u9257\u9258\uB6F3\u9259\uD5D2\uB3D0"+
	"\uBCBC\u925A\u925B\u925C\uB3AD\u925D\u925E\u925F"+
	"\u9260\uBEF1\uB0D1\u9261\u9262\u9263\u9264\u9265"+
	"\u9266\uD2D6\uCAE3\uD7A5\u9267\uCDB6\uB6B6\uBFB9"+
	"\uD5DB\u9268\uB8A7\uC5D7\u9269\u926A\u926B\uDED2"+
	"\uBFD9\uC2D5\uC7C0\u926C\uBBA4\uB1A8\u926D\u926E"+
	"\uC5EA\u926F\u9270\uC5FB\uCCA7\u9271\u9272\u9273"+
	"\u9274\uB1A7\u9275\u9276\u9277\uB5D6\u9278\u9279"+
	"\u927A\uC4A8\u927B\uDED3\uD1BA\uB3E9\u927C\uC3F2"+
	"\u927D\u927E\uB7F7\u9280\uD6F4\uB5A3\uB2F0\uC4B4"+
	"\uC4E9\uC0AD\uDED4\u9281\uB0E8\uC5C4\uC1E0\u9282"+
	"\uB9D5\u9283\uBEDC\uCDD8\uB0CE\u9284\uCDCF\uDED6"+
	"\uBED0\uD7BE\uDED5\uD5D0\uB0DD\u9285\u9286\uC4E2"+
	"\u9287\u9288\uC2A3\uBCF0\u9289\uD3B5\uC0B9\uC5A1"+
	"\uB2A6\uD4F1\u928A\u928B\uC0A8\uCAC3\uDED7\uD5FC"+
	"\u928C\uB9B0\u928D\uC8AD\uCBA9\u928E\uDED9\uBFBD"+
	"\u928F\u9290\u9291\u9292\uC6B4\uD7A7\uCAB0\uC4C3"+
	"\u9293\uB3D6\uB9D2\u9294\u9295\u9296\u9297\uD6B8"+
	"\uEAFC\uB0B4\u9298\u9299\u929A\u929B\uBFE6\u929C"+
	"\u929D\uCCF4\u929E\u929F\u92A0\u92A1\uCDDA\u92A2"+
	"\u92A3\u92A4\uD6BF\uC2CE\u92A5\uCECE\uCCA2\uD0AE"+
	"\uC4D3\uB5B2\uDED8\uD5F5\uBCB7\uBBD3\u92A6\u92A7"+
	"\uB0A4\u92A8\uC5B2\uB4EC\u92A9\u92AA\u92AB\uD5F1"+
	"\u92AC\u92AD\uEAFD\u92AE\u92AF\u92B0\u92B1\u92B2"+
	"\u92B3\uDEDA\uCDA6\u92B4\u92B5\uCDEC\u92B6\u92B7"+
	"\u92B8\u92B9\uCEE6\uDEDC\u92BA\uCDB1\uC0A6\u92BB"+
	"\u92BC\uD7BD\u92BD\uDEDB\uB0C6\uBAB4\uC9D3\uC4F3"+
	"\uBEE8\u92BE\u92BF\u92C0\u92C1\uB2B6\u92C2\u92C3"+
	"\u92C4\u92C5\u92C6\u92C7\u92C8\u92C9\uC0CC\uCBF0"+
	"\u92CA\uBCF1\uBBBB\uB5B7\u92CB\u92CC\u92CD\uC5F5"+
	"\u92CE\uDEE6\u92CF\u92D0\u92D1\uDEE3\uBEDD\u92D2"+
	"\u92D3\uDEDF\u92D4\u92D5\u92D6\u92D7\uB4B7\uBDDD"+
	"\u92D8\u92D9\uDEE0\uC4ED\u92DA\u92DB\u92DC\u92DD"+
	"\uCFC6\u92DE\uB5E0\u92DF\u92E0\u92E1\u92E2\uB6DE"+
	"\uCADA\uB5F4\uDEE5\u92E3\uD5C6\u92E4\uDEE1\uCCCD"+
	"\uC6FE\u92E5\uC5C5\u92E6\u92E7\u92E8\uD2B4\u92E9"+
	"\uBEF2\u92EA\u92EB\u92EC\u92ED\u92EE\u92EF\u92F0"+
	"\uC2D3\u92F1\uCCBD\uB3B8\u92F2\uBDD3\u92F3\uBFD8"+
	"\uCDC6\uD1DA\uB4EB\u92F4\uDEE4\uDEDD\uDEE7\u92F5"+
	"\uEAFE\u92F6\u92F7\uC2B0\uDEE2\u92F8\u92F9\uD6C0"+
	"\uB5A7\u92FA\uB2F4\u92FB\uDEE8\u92FC\uDEF2\u92FD"+
	"\u92FE\u9340\u9341\u9342\uDEED\u9343\uDEF1\u9344"+
	"\u9345\uC8E0\u9346\u9347\u9348\uD7E1\uDEEF\uC3E8"+
	"\uCCE1\u9349\uB2E5\u934A\u934B\u934C\uD2BE\u934D"+
	"\u934E\u934F\u9350\u9351\u9352\u9353\uDEEE\u9354"+
	"\uDEEB\uCED5\u9355\uB4A7\u9356\u9357\u9358\u9359"+
	"\u935A\uBFAB\uBEBE\u935B\u935C\uBDD2\u935D\u935E"+
	"\u935F\u9360\uDEE9\u9361\uD4AE\u9362\uDEDE\u9363"+
	"\uDEEA\u9364\u9365\u9366\u9367\uC0BF\u9368\uDEEC"+
	"\uB2F3\uB8E9\uC2A7\u9369\u936A\uBDC1\u936B\u936C"+
	"\u936D\u936E\u936F\uDEF5\uDEF8\u9370\u9371\uB2AB"+
	"\uB4A4\u9372\u9373\uB4EA\uC9A6\u9374\u9375\u9376"+
	"\u9377\u9378\u9379\uDEF6\uCBD1\u937A\uB8E3\u937B"+
	"\uDEF7\uDEFA\u937C\u937D\u937E\u9380\uDEF9\u9381"+
	"\u9382\u9383\uCCC2\u9384\uB0E1\uB4EE\u9385\u9386"+
	"\u9387\u9388\u9389\u938A\uE5BA\u938B\u938C\u938D"+
	"\u938E\u938F\uD0AF\u9390\u9391\uB2EB\u9392\uEBA1"+
	"\u9393\uDEF4\u9394\u9395\uC9E3\uDEF3\uB0DA\uD2A1"+
	"\uB1F7\u9396\uCCAF\u9397\u9398\u9399\u939A\u939B"+
	"\u939C\u939D\uDEF0\u939E\uCBA4\u939F\u93A0\u93A1"+
	"\uD5AA\u93A2\u93A3\u93A4\u93A5\u93A6\uDEFB\u93A7"+
	"\u93A8\u93A9\u93AA\u93AB\u93AC\u93AD\u93AE\uB4DD"+
	"\u93AF\uC4A6\u93B0\u93B1\u93B2\uDEFD\u93B3\u93B4"+
	"\u93B5\u93B6\u93B7\u93B8\u93B9\u93BA\u93BB\u93BC"+
	"\uC3FE\uC4A1\uDFA1\u93BD\u93BE\u93BF\u93C0\u93C1"+
	"\u93C2\u93C3\uC1CC\u93C4\uDEFC\uBEEF\u93C5\uC6B2"+
	"\u93C6\u93C7\u93C8\u93C9\u93CA\u93CB\u93CC\u93CD"+
	"\u93CE\uB3C5\uC8F6\u93CF\u93D0\uCBBA\uDEFE\u93D1"+
	"\u93D2\uDFA4\u93D3\u93D4\u93D5\u93D6\uD7B2\u93D7"+
	"\u93D8\u93D9\u93DA\u93DB\uB3B7\u93DC\u93DD\u93DE"+
	"\u93DF\uC1C3\u93E0\u93E1\uC7CB\uB2A5\uB4E9\u93E2"+
	"\uD7AB\u93E3\u93E4\u93E5\u93E6\uC4EC\u93E7\uDFA2"+
	"\uDFA3\u93E8\uDFA5\u93E9\uBAB3\u93EA\u93EB\u93EC"+
	"\uDFA6\u93ED\uC0DE\u93EE\u93EF\uC9C3\u93F0\u93F1"+
	"\u93F2\u93F3\u93F4\u93F5\u93F6\uB2D9\uC7E6\u93F7"+
	"\uDFA7\u93F8\uC7DC\u93F9\u93FA\u93FB\u93FC\uDFA8"+
	"\uEBA2\u93FD\u93FE\u9440\u9441\u9442\uCBD3\u9443"+
	"\u9444\u9445\uDFAA\u9446\uDFA9\u9447\uB2C1\u9448"+
	"\u9449\u944A\u944B\u944C\u944D\u944E\u944F\u9450"+
	"\u9451\u9452\u9453\u9454\u9455\u9456\u9457\u9458"+
	"\u9459\u945A\u945B\u945C\u945D\u945E\u945F\u9460"+
	"\uC5CA\u9461\u9462\u9463\u9464\u9465\u9466\u9467"+
	"\u9468\uDFAB\u9469\u946A\u946B\u946C\u946D\u946E"+
	"\u946F\u9470\uD4DC\u9471\u9472\u9473\u9474\u9475"+
	"\uC8C1\u9476\u9477\u9478\u9479\u947A\u947B\u947C"+
	"\u947D\u947E\u9480\u9481\u9482\uDFAC\u9483\u9484"+
	"\u9485\u9486\u9487\uBEF0\u9488\u9489\uDFAD\uD6A7"+
	"\u948A\u948B\u948C\u948D\uEAB7\uEBB6\uCAD5\u948E"+
	"\uD8FC\uB8C4\u948F\uB9A5\u9490\u9491\uB7C5\uD5FE"+
	"\u9492\u9493\u9494\u9495\u9496\uB9CA\u9497\u9498"+
	"\uD0A7\uF4CD\u9499\u949A\uB5D0\u949B\u949C\uC3F4"+
	"\u949D\uBEC8\u949E\u949F\u94A0\uEBB7\uB0BD\u94A1"+
	"\u94A2\uBDCC\u94A3\uC1B2\u94A4\uB1D6\uB3A8\u94A5"+
	"\u94A6\u94A7\uB8D2\uC9A2\u94A8\u94A9\uB6D8\u94AA"+
	"\u94AB\u94AC\u94AD\uEBB8\uBEB4\u94AE\u94AF\u94B0"+
	"\uCAFD\u94B1\uC7C3\u94B2\uD5FB\u94B3\u94B4\uB7F3"+
	"\u94B5\u94B6\u94B7\u94B8\u94B9\u94BA\u94BB\u94BC"+
	"\u94BD\u94BE\u94BF\u94C0\u94C1\u94C2\u94C3\uCEC4"+
	"\u94C4\u94C5\u94C6\uD5AB\uB1F3\u94C7\u94C8\u94C9"+
	"\uECB3\uB0DF\u94CA\uECB5\u94CB\u94CC\u94CD\uB6B7"+
	"\u94CE\uC1CF\u94CF\uF5FA\uD0B1\u94D0\u94D1\uD5E5"+
	"\u94D2\uCED3\u94D3\u94D4\uBDEF\uB3E2\u94D5\uB8AB"+
	"\u94D6\uD5B6\u94D7\uEDBD\u94D8\uB6CF\u94D9\uCBB9"+
	"\uD0C2\u94DA\u94DB\u94DC\u94DD\u94DE\u94DF\u94E0"+
	"\u94E1\uB7BD\u94E2\u94E3\uECB6\uCAA9\u94E4\u94E5"+
	"\u94E6\uC5D4\u94E7\uECB9\uECB8\uC2C3\uECB7\u94E8"+
	"\u94E9\u94EA\u94EB\uD0FD\uECBA\u94EC\uECBB\uD7E5"+
	"\u94ED\u94EE\uECBC\u94EF\u94F0\u94F1\uECBD\uC6EC"+
	"\u94F2\u94F3\u94F4\u94F5\u94F6\u94F7\u94F8\u94F9"+
	"\uCEDE\u94FA\uBCC8\u94FB\u94FC\uC8D5\uB5A9\uBEC9"+
	"\uD6BC\uD4E7\u94FD\u94FE\uD1AE\uD0F1\uEAB8\uEAB9"+
	"\uEABA\uBAB5\u9540\u9541\u9542\u9543\uCAB1\uBFF5"+
	"\u9544\u9545\uCDFA\u9546\u9547\u9548\u9549\u954A"+
	"\uEAC0\u954B\uB0BA\uEABE\u954C\u954D\uC0A5\u954E"+
	"\u954F\u9550\uEABB\u9551\uB2FD\u9552\uC3F7\uBBE8"+
	"\u9553\u9554\u9555\uD2D7\uCEF4\uEABF\u9556\u9557"+
	"\u9558\uEABC\u9559\u955A\u955B\uEAC3\u955C\uD0C7"+
	"\uD3B3\u955D\u955E\u955F\u9560\uB4BA\u9561\uC3C1"+
	"\uD7F2\u9562\u9563\u9564\u9565\uD5D1\u9566\uCAC7"+
	"\u9567\uEAC5\u9568\u9569\uEAC4\uEAC7\uEAC6\u956A"+
	"\u956B\u956C\u956D\u956E\uD6E7\u956F\uCFD4\u9570"+
	"\u9571\uEACB\u9572\uBBCE\u9573\u9574\u9575\u9576"+
	"\u9577\u9578\u9579\uBDFA\uC9CE\u957A\u957B\uEACC"+
	"\u957C\u957D\uC9B9\uCFFE\uEACA\uD4CE\uEACD\uEACF"+
	"\u957E\u9580\uCDED\u9581\u9582\u9583\u9584\uEAC9"+
	"\u9585\uEACE\u9586\u9587\uCEEE\u9588\uBBDE\u9589"+
	"\uB3BF\u958A\u958B\u958C\u958D\u958E\uC6D5\uBEB0"+
	"\uCEFA\u958F\u9590\u9591\uC7E7\u9592\uBEA7\uEAD0"+
	"\u9593\u9594\uD6C7\u9595\u9596\u9597\uC1C0\u9598"+
	"\u9599\u959A\uD4DD\u959B\uEAD1\u959C\u959D\uCFBE"+
	"\u959E\u959F\u95A0\u95A1\uEAD2\u95A2\u95A3\u95A4"+
	"\u95A5\uCAEE\u95A6\u95A7\u95A8\u95A9\uC5AF\uB0B5"+
	"\u95AA\u95AB\u95AC\u95AD\u95AE\uEAD4\u95AF\u95B0"+
	"\u95B1\u95B2\u95B3\u95B4\u95B5\u95B6\u95B7\uEAD3"+
	"\uF4DF\u95B8\u95B9\u95BA\u95BB\u95BC\uC4BA\u95BD"+
	"\u95BE\u95BF\u95C0\u95C1\uB1A9\u95C2\u95C3\u95C4"+
	"\u95C5\uE5DF\u95C6\u95C7\u95C8\u95C9\uEAD5\u95CA"+
	"\u95CB\u95CC\u95CD\u95CE\u95CF\u95D0\u95D1\u95D2"+
	"\u95D3\u95D4\u95D5\u95D6\u95D7\u95D8\u95D9\u95DA"+
	"\u95DB\u95DC\u95DD\u95DE\u95DF\u95E0\u95E1\u95E2"+
	"\u95E3\uCAEF\u95E4\uEAD6\uEAD7\uC6D8\u95E5\u95E6"+
	"\u95E7\u95E8\u95E9\u95EA\u95EB\u95EC\uEAD8\u95ED"+
	"\u95EE\uEAD9\u95EF\u95F0\u95F1\u95F2\u95F3\u95F4"+
	"\uD4BB\u95F5\uC7FA\uD2B7\uB8FC\u95F6\u95F7\uEAC2"+
	"\u95F8\uB2DC\u95F9\u95FA\uC2FC\u95FB\uD4F8\uCCE6"+
	"\uD7EE\u95FC\u95FD\u95FE\u9640\u9641\u9642\u9643"+
	"\uD4C2\uD3D0\uEBC3\uC5F3\u9644\uB7FE\u9645\u9646"+
	"\uEBD4\u9647\u9648\u9649\uCBB7\uEBDE\u964A\uC0CA"+
	"\u964B\u964C\u964D\uCDFB\u964E\uB3AF\u964F\uC6DA"+
	"\u9650\u9651\u9652\u9653\u9654\u9655\uEBFC\u9656"+
	"\uC4BE\u9657\uCEB4\uC4A9\uB1BE\uD4FD\u9658\uCAF5"+
	"\u9659\uD6EC\u965A\u965B\uC6D3\uB6E4\u965C\u965D"+
	"\u965E\u965F\uBBFA\u9660\u9661\uD0E0\u9662\u9663"+
	"\uC9B1\u9664\uD4D3\uC8A8\u9665\u9666\uB8CB\u9667"+
	"\uE8BE\uC9BC\u9668\u9669\uE8BB\u966A\uC0EE\uD0D3"+
	"\uB2C4\uB4E5\u966B\uE8BC\u966C\u966D\uD5C8\u966E"+
	"\u966F\u9670\u9671\u9672\uB6C5\u9673\uE8BD\uCAF8"+
	"\uB8DC\uCCF5\u9674\u9675\u9676\uC0B4\u9677\u9678"+
	"\uD1EE\uE8BF\uE8C2\u9679\u967A\uBABC\u967B\uB1AD"+
	"\uBDDC\u967C\uEABD\uE8C3\u967D\uE8C6\u967E\uE8CB"+
	"\u9680\u9681\u9682\u9683\uE8CC\u9684\uCBC9\uB0E5"+
	"\u9685\uBCAB\u9686\u9687\uB9B9\u9688\u9689\uE8C1"+
	"\u968A\uCDF7\u968B\uE8CA\u968C\u968D\u968E\u968F"+
	"\uCEF6\u9690\u9691\u9692\u9693\uD5ED\u9694\uC1D6"+
	"\uE8C4\u9695\uC3B6\u9696\uB9FB\uD6A6\uE8C8\u9697"+
	"\u9698\u9699\uCAE0\uD4E6\u969A\uE8C0\u969B\uE8C5"+
	"\uE8C7\u969C\uC7B9\uB7E3\u969D\uE8C9\u969E\uBFDD"+
	"\uE8D2\u969F\u96A0\uE8D7\u96A1\uE8D5\uBCDC\uBCCF"+
	"\uE8DB\u96A2\u96A3\u96A4\u96A5\u96A6\u96A7\u96A8"+
	"\u96A9\uE8DE\u96AA\uE8DA\uB1FA\u96AB\u96AC\u96AD"+
	"\u96AE\u96AF\u96B0\u96B1\u96B2\u96B3\u96B4\uB0D8"+
	"\uC4B3\uB8CC\uC6E2\uC8BE\uC8E1\u96B5\u96B6\u96B7"+
	"\uE8CF\uE8D4\uE8D6\u96B8\uB9F1\uE8D8\uD7F5\u96B9"+
	"\uC4FB\u96BA\uE8DC\u96BB\u96BC\uB2E9\u96BD\u96BE"+
	"\u96BF\uE8D1\u96C0\u96C1\uBCED\u96C2\u96C3\uBFC2"+
	"\uE8CD\uD6F9\u96C4\uC1F8\uB2F1\u96C5\u96C6\u96C7"+
	"\u96C8\u96C9\u96CA\u96CB\u96CC\uE8DF\u96CD\uCAC1"+
	"\uE8D9\u96CE\u96CF\u96D0\u96D1\uD5A4\u96D2\uB1EA"+
	"\uD5BB\uE8CE\uE8D0\uB6B0\uE8D3\u96D3\uE8DD\uC0B8"+
	"\u96D4\uCAF7\u96D5\uCBA8\u96D6\u96D7\uC6DC\uC0F5"+
	"\u96D8\u96D9\u96DA\u96DB\u96DC\uE8E9\u96DD\u96DE"+
	"\u96DF\uD0A3\u96E0\u96E1\u96E2\u96E3\u96E4\u96E5"+
	"\u96E6\uE8F2\uD6EA\u96E7\u96E8\u96E9\u96EA\u96EB"+
	"\u96EC\u96ED\uE8E0\uE8E1\u96EE\u96EF\u96F0\uD1F9"+
	"\uBACB\uB8F9\u96F1\u96F2\uB8F1\uD4D4\uE8EF\u96F3"+
	"\uE8EE\uE8EC\uB9F0\uCCD2\uE8E6\uCEA6\uBFF2\u96F4"+
	"\uB0B8\uE8F1\uE8F0\u96F5\uD7C0\u96F6\uE8E4\u96F7"+
	"\uCDA9\uC9A3\u96F8\uBBB8\uBDDB\uE8EA\u96F9\u96FA"+
	"\u96FB\u96FC\u96FD\u96FE\u9740\u9741\u9742\u9743"+
	"\uE8E2\uE8E3\uE8E5\uB5B5\uE8E7\uC7C5\uE8EB\uE8ED"+
	"\uBDB0\uD7AE\u9744\uE8F8\u9745\u9746\u9747\u9748"+
	"\u9749\u974A\u974B\u974C\uE8F5\u974D\uCDB0\uE8F6"+
	"\u974E\u974F\u9750\u9751\u9752\u9753\u9754\u9755"+
	"\u9756\uC1BA\u9757\uE8E8\u9758\uC3B7\uB0F0\u9759"+
	"\u975A\u975B\u975C\u975D\u975E\u975F\u9760\uE8F4"+
	"\u9761\u9762\u9763\uE8F7\u9764\u9765\u9766\uB9A3"+
	"\u9767\u9768\u9769\u976A\u976B\u976C\u976D\u976E"+
	"\u976F\u9770\uC9D2\u9771\u9772\u9773\uC3CE\uCEE0"+
	"\uC0E6\u9774\u9775\u9776\u9777\uCBF3\u9778\uCCDD"+
	"\uD0B5\u9779\u977A\uCAE1\u977B\uE8F3\u977C\u977D"+
	"\u977E\u9780\u9781\u9782\u9783\u9784\u9785\u9786"+
	"\uBCEC\u9787\uE8F9\u9788\u9789\u978A\u978B\u978C"+
	"\u978D\uC3DE\u978E\uC6E5\u978F\uB9F7\u9790\u9791"+
	"\u9792\u9793\uB0F4\u9794\u9795\uD7D8\u9796\u9797"+
	"\uBCAC\u9798\uC5EF\u9799\u979A\u979B\u979C\u979D"+
	"\uCCC4\u979E\u979F\uE9A6\u97A0\u97A1\u97A2\u97A3"+
	"\u97A4\u97A5\u97A6\u97A7\u97A8\u97A9\uC9AD\u97AA"+
	"\uE9A2\uC0E2\u97AB\u97AC\u97AD\uBFC3\u97AE\u97AF"+
	"\u97B0\uE8FE\uB9D7\u97B1\uE8FB\u97B2\u97B3\u97B4"+
	"\u97B5\uE9A4\u97B6\u97B7\u97B8\uD2CE\u97B9\u97BA"+
	"\u97BB\u97BC\u97BD\uE9A3\u97BE\uD6B2\uD7B5\u97BF"+
	"\uE9A7\u97C0\uBDB7\u97C1\u97C2\u97C3\u97C4\u97C5"+
	"\u97C6\u97C7\u97C8\u97C9\u97CA\u97CB\u97CC\uE8FC"+
	"\uE8FD\u97CD\u97CE\u97CF\uE9A1\u97D0\u97D1\u97D2"+
	"\u97D3\u97D4\u97D5\u97D6\u97D7\uCDD6\u97D8\u97D9"+
	"\uD2AC\u97DA\u97DB\u97DC\uE9B2\u97DD\u97DE\u97DF"+
	"\u97E0\uE9A9\u97E1\u97E2\u97E3\uB4AA\u97E4\uB4BB"+
	"\u97E5\u97E6\uE9AB\u97E7\u97E8\u97E9\u97EA\u97EB"+
	"\u97EC\u97ED\u97EE\u97EF\u97F0\u97F1\u97F2\u97F3"+
	"\u97F4\u97F5\u97F6\u97F7\uD0A8\u97F8\u97F9\uE9A5"+
	"\u97FA\u97FB\uB3FE\u97FC\u97FD\uE9AC\uC0E3\u97FE"+
	"\uE9AA\u9840\u9841\uE9B9\u9842\u9843\uE9B8\u9844"+
	"\u9845\u9846\u9847\uE9AE\u9848\u9849\uE8FA\u984A"+
	"\u984B\uE9A8\u984C\u984D\u984E\u984F\u9850\uBFAC"+
	"\uE9B1\uE9BA\u9851\u9852\uC2A5\u9853\u9854\u9855"+
	"\uE9AF\u9856\uB8C5\u9857\uE9AD\u9858\uD3DC\uE9B4"+
	"\uE9B5\uE9B7\u9859\u985A\u985B\uE9C7\u985C\u985D"+
	"\u985E\u985F\u9860\u9861\uC0C6\uE9C5\u9862\u9863"+
	"\uE9B0\u9864\u9865\uE9BB\uB0F1\u9866\u9867\u9868"+
	"\u9869\u986A\u986B\u986C\u986D\u986E\u986F\uE9BC"+
	"\uD5A5\u9870\u9871\uE9BE\u9872\uE9BF\u9873\u9874"+
	"\u9875\uE9C1\u9876\u9877\uC1F1\u9878\u9879\uC8B6"+
	"\u987A\u987B\u987C\uE9BD\u987D\u987E\u9880\u9881"+
	"\u9882\uE9C2\u9883\u9884\u9885\u9886\u9887\u9888"+
	"\u9889\u988A\uE9C3\u988B\uE9B3\u988C\uE9B6\u988D"+
	"\uBBB1\u988E\u988F\u9890\uE9C0\u9891\u9892\u9893"+
	"\u9894\u9895\u9896\uBCF7\u9897\u9898\u9899\uE9C4"+
	"\uE9C6\u989A\u989B\u989C\u989D\u989E\u989F\u98A0"+
	"\u98A1\u98A2\u98A3\u98A4\u98A5\uE9CA\u98A6\u98A7"+
	"\u98A8\u98A9\uE9CE\u98AA\u98AB\u98AC\u98AD\u98AE"+
	"\u98AF\u98B0\u98B1\u98B2\u98B3\uB2DB\u98B4\uE9C8"+
	"\u98B5\u98B6\u98B7\u98B8\u98B9\u98BA\u98BB\u98BC"+
	"\u98BD\u98BE\uB7AE\u98BF\u98C0\u98C1\u98C2\u98C3"+
	"\u98C4\u98C5\u98C6\u98C7\u98C8\u98C9\u98CA\uE9CB"+
	"\uE9CC\u98CB\u98CC\u98CD\u98CE\u98CF\u98D0\uD5C1"+
	"\u98D1\uC4A3\u98D2\u98D3\u98D4\u98D5\u98D6\u98D7"+
	"\uE9D8\u98D8\uBAE1\u98D9\u98DA\u98DB\u98DC\uE9C9"+
	"\u98DD\uD3A3\u98DE\u98DF\u98E0\uE9D4\u98E1\u98E2"+
	"\u98E3\u98E4\u98E5\u98E6\u98E7\uE9D7\uE9D0\u98E8"+
	"\u98E9\u98EA\u98EB\u98EC\uE9CF\u98ED\u98EE\uC7C1"+
	"\u98EF\u98F0\u98F1\u98F2\u98F3\u98F4\u98F5\u98F6"+
	"\uE9D2\u98F7\u98F8\u98F9\u98FA\u98FB\u98FC\u98FD"+
	"\uE9D9\uB3C8\u98FE\uE9D3\u9940\u9941\u9942\u9943"+
	"\u9944\uCFF0\u9945\u9946\u9947\uE9CD\u9948\u9949"+
	"\u994A\u994B\u994C\u994D\u994E\u994F\u9950\u9951"+
	"\u9952\uB3F7\u9953\u9954\u9955\u9956\u9957\u9958"+
	"\u9959\uE9D6\u995A\u995B\uE9DA\u995C\u995D\u995E"+
	"\uCCB4\u995F\u9960\u9961\uCFAD\u9962\u9963\u9964"+
	"\u9965\u9966\u9967\u9968\u9969\u996A\uE9D5\u996B"+
	"\uE9DC\uE9DB\u996C\u996D\u996E\u996F\u9970\uE9DE"+
	"\u9971\u9972\u9973\u9974\u9975\u9976\u9977\u9978"+
	"\uE9D1\u9979\u997A\u997B\u997C\u997D\u997E\u9980"+
	"\u9981\uE9DD\u9982\uE9DF\uC3CA\u9983\u9984\u9985"+
	"\u9986\u9987\u9988\u9989\u998A\u998B\u998C\u998D"+
	"\u998E\u998F\u9990\u9991\u9992\u9993\u9994\u9995"+
	"\u9996\u9997\u9998\u9999\u999A\u999B\u999C\u999D"+
	"\u999E\u999F\u99A0\u99A1\u99A2\u99A3\u99A4\u99A5"+
	"\u99A6\u99A7\u99A8\u99A9\u99AA\u99AB\u99AC\u99AD"+
	"\u99AE\u99AF\u99B0\u99B1\u99B2\u99B3\u99B4\u99B5"+
	"\u99B6\u99B7\u99B8\u99B9\u99BA\u99BB\u99BC\u99BD"+
	"\u99BE\u99BF\u99C0\u99C1\u99C2\u99C3\u99C4\u99C5"+
	"\u99C6\u99C7\u99C8\u99C9\u99CA\u99CB\u99CC\u99CD"+
	"\u99CE\u99CF\u99D0\u99D1\u99D2\u99D3\u99D4\u99D5"+
	"\u99D6\u99D7\u99D8\u99D9\u99DA\u99DB\u99DC\u99DD"+
	"\u99DE\u99DF\u99E0\u99E1\u99E2\u99E3\u99E4\u99E5"+
	"\u99E6\u99E7\u99E8\u99E9\u99EA\u99EB\u99EC\u99ED"+
	"\u99EE\u99EF\u99F0\u99F1\u99F2\u99F3\u99F4\u99F5"+
	"\uC7B7\uB4CE\uBBB6\uD0C0\uECA3\u99F6\u99F7\uC5B7"+
	"\u99F8\u99F9\u99FA\u99FB\u99FC\u99FD\u99FE\u9A40"+
	"\u9A41\u9A42\uD3FB\u9A43\u9A44\u9A45\u9A46\uECA4"+
	"\u9A47\uECA5\uC6DB\u9A48\u9A49\u9A4A\uBFEE\u9A4B"+
	"\u9A4C\u9A4D\u9A4E\uECA6\u9A4F\u9A50\uECA7\uD0AA"+
	"\u9A51\uC7B8\u9A52\u9A53\uB8E8\u9A54\u9A55\u9A56"+
	"\u9A57\u9A58\u9A59\u9A5A\u9A5B\u9A5C\u9A5D\u9A5E"+
	"\u9A5F\uECA8\u9A60\u9A61\u9A62\u9A63\u9A64\u9A65"+
	"\u9A66\u9A67\uD6B9\uD5FD\uB4CB\uB2BD\uCEE4\uC6E7"+
	"\u9A68\u9A69\uCDE1\u9A6A\u9A6B\u9A6C\u9A6D\u9A6E"+
	"\u9A6F\u9A70\u9A71\u9A72\u9A73\u9A74\u9A75\u9A76"+
	"\u9A77\uB4F5\u9A78\uCBC0\uBCDF\u9A79\u9A7A\u9A7B"+
	"\u9A7C\uE9E2\uE9E3\uD1EA\uE9E5\u9A7D\uB4F9\uE9E4"+
	"\u9A7E\uD1B3\uCAE2\uB2D0\u9A80\uE9E8\u9A81\u9A82"+
	"\u9A83\u9A84\uE9E6\uE9E7\u9A85\u9A86\uD6B3\u9A87"+
	"\u9A88\u9A89\uE9E9\uE9EA\u9A8A\u9A8B\u9A8C\u9A8D"+
	"\u9A8E\uE9EB\u9A8F\u9A90\u9A91\u9A92\u9A93\u9A94"+
	"\u9A95\u9A96\uE9EC\u9A97\u9A98\u9A99\u9A9A\u9A9B"+
	"\u9A9C\u9A9D\u9A9E\uECAF\uC5B9\uB6CE\u9A9F\uD2F3"+
	"\u9AA0\u9AA1\u9AA2\u9AA3\u9AA4\u9AA5\u9AA6\uB5EE"+
	"\u9AA7\uBBD9\uECB1\u9AA8\u9AA9\uD2E3\u9AAA\u9AAB"+
	"\u9AAC\u9AAD\u9AAE\uCEE3\u9AAF\uC4B8\u9AB0\uC3BF"+
	"\u9AB1\u9AB2\uB6BE\uD8B9\uB1C8\uB1CF\uB1D1\uC5FE"+
	"\u9AB3\uB1D0\u9AB4\uC3AB\u9AB5\u9AB6\u9AB7\u9AB8"+
	"\u9AB9\uD5B1\u9ABA\u9ABB\u9ABC\u9ABD\u9ABE\u9ABF"+
	"\u9AC0\u9AC1\uEBA4\uBAC1\u9AC2\u9AC3\u9AC4\uCCBA"+
	"\u9AC5\u9AC6\u9AC7\uEBA5\u9AC8\uEBA7\u9AC9\u9ACA"+
	"\u9ACB\uEBA8\u9ACC\u9ACD\u9ACE\uEBA6\u9ACF\u9AD0"+
	"\u9AD1\u9AD2\u9AD3\u9AD4\u9AD5\uEBA9\uEBAB\uEBAA"+
	"\u9AD6\u9AD7\u9AD8\u9AD9\u9ADA\uEBAC\u9ADB\uCACF"+
	"\uD8B5\uC3F1\u9ADC\uC3A5\uC6F8\uEBAD\uC4CA\u9ADD"+
	"\uEBAE\uEBAF\uEBB0\uB7D5\u9ADE\u9ADF\u9AE0\uB7FA"+
	"\u9AE1\uEBB1\uC7E2\u9AE2\uEBB3\u9AE3\uBAA4\uD1F5"+
	"\uB0B1\uEBB2\uEBB4\u9AE4\u9AE5\u9AE6\uB5AA\uC2C8"+
	"\uC7E8\u9AE7\uEBB5\u9AE8\uCBAE\uE3DF\u9AE9\u9AEA"+
	"\uD3C0\u9AEB\u9AEC\u9AED\u9AEE\uD9DB\u9AEF\u9AF0"+
	"\uCDA1\uD6AD\uC7F3\u9AF1\u9AF2\u9AF3\uD9E0\uBBE3"+
	"\u9AF4\uBABA\uE3E2\u9AF5\u9AF6\u9AF7\u9AF8\u9AF9"+
	"\uCFAB\u9AFA\u9AFB\u9AFC\uE3E0\uC9C7\u9AFD\uBAB9"+
	"\u9AFE\u9B40\u9B41\uD1B4\uE3E1\uC8EA\uB9AF\uBDAD"+
	"\uB3D8\uCEDB\u9B42\u9B43\uCCC0\u9B44\u9B45\u9B46"+
	"\uE3E8\uE3E9\uCDF4\u9B47\u9B48\u9B49\u9B4A\u9B4B"+
	"\uCCAD\u9B4C\uBCB3\u9B4D\uE3EA\u9B4E\uE3EB\u9B4F"+
	"\u9B50\uD0DA\u9B51\u9B52\u9B53\uC6FB\uB7DA\u9B54"+
	"\u9B55\uC7DF\uD2CA\uCED6\u9B56\uE3E4\uE3EC\u9B57"+
	"\uC9F2\uB3C1\u9B58\u9B59\uE3E7\u9B5A\u9B5B\uC6E3"+
	"\uE3E5\u9B5C\u9B5D\uEDB3\uE3E6\u9B5E\u9B5F\u9B60"+
	"\u9B61\uC9B3\u9B62\uC5E6\u9B63\u9B64\u9B65\uB9B5"+
	"\u9B66\uC3BB\u9B67\uE3E3\uC5BD\uC1A4\uC2D9\uB2D7"+
	"\u9B68\uE3ED\uBBA6\uC4AD\u9B69\uE3F0\uBEDA\u9B6A"+
	"\u9B6B\uE3FB\uE3F5\uBAD3\u9B6C\u9B6D\u9B6E\u9B6F"+
	"\uB7D0\uD3CD\u9B70\uD6CE\uD5D3\uB9C1\uD5B4\uD1D8"+
	"\u9B71\u9B72\u9B73\u9B74\uD0B9\uC7F6\u9B75\u9B76"+
	"\u9B77\uC8AA\uB2B4\u9B78\uC3DA\u9B79\u9B7A\u9B7B"+
	"\uE3EE\u9B7C\u9B7D\uE3FC\uE3EF\uB7A8\uE3F7\uE3F4"+
	"\u9B7E\u9B80\u9B81\uB7BA\u9B82\u9B83\uC5A2\u9B84"+
	"\uE3F6\uC5DD\uB2A8\uC6FC\u9B85\uC4E0\u9B86\u9B87"+
	"\uD7A2\u9B88\uC0E1\uE3F9\u9B89\u9B8A\uE3FA\uE3FD"+
	"\uCCA9\uE3F3\u9B8B\uD3BE\u9B8C\uB1C3\uEDB4\uE3F1"+
	"\uE3F2\u9B8D\uE3F8\uD0BA\uC6C3\uD4F3\uE3FE\u9B8E"+
	"\u9B8F\uBDE0\u9B90\u9B91\uE4A7\u9B92\u9B93\uE4A6"+
	"\u9B94\u9B95\u9B96\uD1F3\uE4A3\u9B97\uE4A9\u9B98"+
	"\u9B99\u9B9A\uC8F7\u9B9B\u9B9C\u9B9D\u9B9E\uCFB4"+
	"\u9B9F\uE4A8\uE4AE\uC2E5\u9BA0\u9BA1\uB6B4\u9BA2"+
	"\u9BA3\u9BA4\u9BA5\u9BA6\u9BA7\uBDF2\u9BA8\uE4A2"+
	"\u9BA9\u9BAA\uBAE9\uE4AA\u9BAB\u9BAC\uE4AC\u9BAD"+
	"\u9BAE\uB6FD\uD6DE\uE4B2\u9BAF\uE4AD\u9BB0\u9BB1"+
	"\u9BB2\uE4A1\u9BB3\uBBEE\uCDDD\uC7A2\uC5C9\u9BB4"+
	"\u9BB5\uC1F7\u9BB6\uE4A4\u9BB7\uC7B3\uBDAC\uBDBD"+
	"\uE4A5\u9BB8\uD7C7\uB2E2\u9BB9\uE4AB\uBCC3\uE4AF"+
	"\u9BBA\uBBEB\uE4B0\uC5A8\uE4B1\u9BBB\u9BBC\u9BBD"+
	"\u9BBE\uD5E3\uBFA3\u9BBF\uE4BA\u9BC0\uE4B7\u9BC1"+
	"\uE4BB\u9BC2\u9BC3\uE4BD\u9BC4\u9BC5\uC6D6\u9BC6"+
	"\u9BC7\uBAC6\uC0CB\u9BC8\u9BC9\u9BCA\uB8A1\uE4B4"+
	"\u9BCB\u9BCC\u9BCD\u9BCE\uD4A1\u9BCF\u9BD0\uBAA3"+
	"\uBDFE\u9BD1\u9BD2\u9BD3\uE4BC\u9BD4\u9BD5\u9BD6"+
	"\u9BD7\u9BD8\uCDBF\u9BD9\u9BDA\uC4F9\u9BDB\u9BDC"+
	"\uCFFB\uC9E6\u9BDD\u9BDE\uD3BF\u9BDF\uCFD1\u9BE0"+
	"\u9BE1\uE4B3\u9BE2\uE4B8\uE4B9\uCCE9\u9BE3\u9BE4"+
	"\u9BE5\u9BE6\u9BE7\uCCCE\u9BE8\uC0D4\uE4B5\uC1B0"+
	"\uE4B6\uCED0\u9BE9\uBBC1\uB5D3\u9BEA\uC8F3\uBDA7"+
	"\uD5C7\uC9AC\uB8A2\uE4CA\u9BEB\u9BEC\uE4CC\uD1C4"+
	"\u9BED\u9BEE\uD2BA\u9BEF\u9BF0\uBAAD\u9BF1\u9BF2"+
	"\uBAD4\u9BF3\u9BF4\u9BF5\u9BF6\u9BF7\u9BF8\uE4C3"+
	"\uB5ED\u9BF9\u9BFA\u9BFB\uD7CD\uE4C0\uCFFD\uE4BF"+
	"\u9BFC\u9BFD\u9BFE\uC1DC\uCCCA\u9C40\u9C41\u9C42"+
	"\u9C43\uCAE7\u9C44\u9C45\u9C46\u9C47\uC4D7\u9C48"+
	"\uCCD4\uE4C8\u9C49\u9C4A\u9C4B\uE4C7\uE4C1\u9C4C"+
	"\uE4C4\uB5AD\u9C4D\u9C4E\uD3D9\u9C4F\uE4C6\u9C50"+
	"\u9C51\u9C52\u9C53\uD2F9\uB4E3\u9C54\uBBB4\u9C55"+
	"\u9C56\uC9EE\u9C57\uB4BE\u9C58\u9C59\u9C5A\uBBEC"+
	"\u9C5B\uD1CD\u9C5C\uCCED\uEDB5\u9C5D\u9C5E\u9C5F"+
	"\u9C60\u9C61\u9C62\u9C63\u9C64\uC7E5\u9C65\u9C66"+
	"\u9C67\u9C68\uD4A8\u9C69\uE4CB\uD7D5\uE4C2\u9C6A"+
	"\uBDA5\uE4C5\u9C6B\u9C6C\uD3E6\u9C6D\uE4C9\uC9F8"+
	"\u9C6E\u9C6F\uE4BE\u9C70\u9C71\uD3E5\u9C72\u9C73"+
	"\uC7FE\uB6C9\u9C74\uD4FC\uB2B3\uE4D7\u9C75\u9C76"+
	"\u9C77\uCEC2\u9C78\uE4CD\u9C79\uCEBC\u9C7A\uB8DB"+
	"\u9C7B\u9C7C\uE4D6\u9C7D\uBFCA\u9C7E\u9C80\u9C81"+
	"\uD3CE\u9C82\uC3EC\u9C83\u9C84\u9C85\u9C86\u9C87"+
	"\u9C88\u9C89\u9C8A\uC5C8\uE4D8\u9C8B\u9C8C\u9C8D"+
	"\u9C8E\u9C8F\u9C90\u9C91\u9C92\uCDC4\uE4CF\u9C93"+
	"\u9C94\u9C95\u9C96\uE4D4\uE4D5\u9C97\uBAFE\u9C98"+
	"\uCFE6\u9C99\u9C9A\uD5BF\u9C9B\u9C9C\u9C9D\uE4D2"+
	"\u9C9E\u9C9F\u9CA0\u9CA1\u9CA2\u9CA3\u9CA4\u9CA5"+
	"\u9CA6\u9CA7\u9CA8\uE4D0\u9CA9\u9CAA\uE4CE\u9CAB"+
	"\u9CAC\u9CAD\u9CAE\u9CAF\u9CB0\u9CB1\u9CB2\u9CB3"+
	"\u9CB4\u9CB5\u9CB6\u9CB7\u9CB8\u9CB9\uCDE5\uCAAA"+
	"\u9CBA\u9CBB\u9CBC\uC0A3\u9CBD\uBDA6\uE4D3\u9CBE"+
	"\u9CBF\uB8C8\u9CC0\u9CC1\u9CC2\u9CC3\u9CC4\uE4E7"+
	"\uD4B4\u9CC5\u9CC6\u9CC7\u9CC8\u9CC9\u9CCA\u9CCB"+
	"\uE4DB\u9CCC\u9CCD\u9CCE\uC1EF\u9CCF\u9CD0\uE4E9"+
	"\u9CD1\u9CD2\uD2E7\u9CD3\u9CD4\uE4DF\u9CD5\uE4E0"+
	"\u9CD6\u9CD7\uCFAA\u9CD8\u9CD9\u9CDA\u9CDB\uCBDD"+
	"\u9CDC\uE4DA\uE4D1\u9CDD\uE4E5\u9CDE\uC8DC\uE4E3"+
	"\u9CDF\u9CE0\uC4E7\uE4E2\u9CE1\uE4E1\u9CE2\u9CE3"+
	"\u9CE4\uB3FC\uE4E8\u9CE5\u9CE6\u9CE7\u9CE8\uB5E1"+
	"\u9CE9\u9CEA\u9CEB\uD7CC\u9CEC\u9CED\u9CEE\uE4E6"+
	"\u9CEF\uBBAC\u9CF0\uD7D2\uCCCF\uEBF8\u9CF1\uE4E4"+
	"\u9CF2\u9CF3\uB9F6\u9CF4\u9CF5\u9CF6\uD6CD\uE4D9"+
	"\uE4DC\uC2FA\uE4DE\u9CF7\uC2CB\uC0C4\uC2D0\u9CF8"+
	"\uB1F5\uCCB2\u9CF9\u9CFA\u9CFB\u9CFC\u9CFD\u9CFE"+
	"\u9D40\u9D41\u9D42\u9D43\uB5CE\u9D44\u9D45\u9D46"+
	"\u9D47\uE4EF\u9D48\u9D49\u9D4A\u9D4B\u9D4C\u9D4D"+
	"\u9D4E\u9D4F\uC6AF\u9D50\u9D51\u9D52\uC6E1\u9D53"+
	"\u9D54\uE4F5\u9D55\u9D56\u9D57\u9D58\u9D59\uC2A9"+
	"\u9D5A\u9D5B\u9D5C\uC0EC\uD1DD\uE4EE\u9D5D\u9D5E"+
	"\u9D5F\u9D60\u9D61\u9D62\u9D63\u9D64\u9D65\u9D66"+
	"\uC4AE\u9D67\u9D68\u9D69\uE4ED\u9D6A\u9D6B\u9D6C"+
	"\u9D6D\uE4F6\uE4F4\uC2FE\u9D6E\uE4DD\u9D6F\uE4F0"+
	"\u9D70\uCAFE\u9D71\uD5C4\u9D72\u9D73\uE4F1\u9D74"+
	"\u9D75\u9D76\u9D77\u9D78\u9D79\u9D7A\uD1FA\u9D7B"+
	"\u9D7C\u9D7D\u9D7E\u9D80\u9D81\u9D82\uE4EB\uE4EC"+
	"\u9D83\u9D84\u9D85\uE4F2\u9D86\uCEAB\u9D87\u9D88"+
	"\u9D89\u9D8A\u9D8B\u9D8C\u9D8D\u9D8E\u9D8F\u9D90"+
	"\uC5CB\u9D91\u9D92\u9D93\uC7B1\u9D94\uC2BA\u9D95"+
	"\u9D96\u9D97\uE4EA\u9D98\u9D99\u9D9A\uC1CA\u9D9B"+
	"\u9D9C\u9D9D\u9D9E\u9D9F\u9DA0\uCCB6\uB3B1\u9DA1"+
	"\u9DA2\u9DA3\uE4FB\u9DA4\uE4F3\u9DA5\u9DA6\u9DA7"+
	"\uE4FA\u9DA8\uE4FD\u9DA9\uE4FC\u9DAA\u9DAB\u9DAC"+
	"\u9DAD\u9DAE\u9DAF\u9DB0\uB3CE\u9DB1\u9DB2\u9DB3"+
	"\uB3BA\uE4F7\u9DB4\u9DB5\uE4F9\uE4F8\uC5EC\u9DB6"+
	"\u9DB7\u9DB8\u9DB9\u9DBA\u9DBB\u9DBC\u9DBD\u9DBE"+
	"\u9DBF\u9DC0\u9DC1\u9DC2\uC0BD\u9DC3\u9DC4\u9DC5"+
	"\u9DC6\uD4E8\u9DC7\u9DC8\u9DC9\u9DCA\u9DCB\uE5A2"+
	"\u9DCC\u9DCD\u9DCE\u9DCF\u9DD0\u9DD1\u9DD2\u9DD3"+
	"\u9DD4\u9DD5\u9DD6\uB0C4\u9DD7\u9DD8\uE5A4\u9DD9"+
	"\u9DDA\uE5A3\u9DDB\u9DDC\u9DDD\u9DDE\u9DDF\u9DE0"+
	"\uBCA4\u9DE1\uE5A5\u9DE2\u9DE3\u9DE4\u9DE5\u9DE6"+
	"\u9DE7\uE5A1\u9DE8\u9DE9\u9DEA\u9DEB\u9DEC\u9DED"+
	"\u9DEE\uE4FE\uB1F4\u9DEF\u9DF0\u9DF1\u9DF2\u9DF3"+
	"\u9DF4\u9DF5\u9DF6\u9DF7\u9DF8\u9DF9\uE5A8\u9DFA"+
	"\uE5A9\uE5A6\u9DFB\u9DFC\u9DFD\u9DFE\u9E40\u9E41"+
	"\u9E42\u9E43\u9E44\u9E45\u9E46\u9E47\uE5A7\uE5AA"+
	"\u9E48\u9E49\u9E4A\u9E4B\u9E4C\u9E4D\u9E4E\u9E4F"+
	"\u9E50\u9E51\u9E52\u9E53\u9E54\u9E55\u9E56\u9E57";

    private final static String innerIndex7= 
	"\u9E58\u9E59\u9E5A\u9E5B\u9E5C\u9E5D\u9E5E\u9E5F"+
	"\u9E60\u9E61\u9E62\u9E63\u9E64\u9E65\u9E66\u9E67"+
	"\u9E68\uC6D9\u9E69\u9E6A\u9E6B\u9E6C\u9E6D\u9E6E"+
	"\u9E6F\u9E70\uE5AB\uE5AD\u9E71\u9E72\u9E73\u9E74"+
	"\u9E75\u9E76\u9E77\uE5AC\u9E78\u9E79\u9E7A\u9E7B"+
	"\u9E7C\u9E7D\u9E7E\u9E80\u9E81\u9E82\u9E83\u9E84"+
	"\u9E85\u9E86\u9E87\u9E88\u9E89\uE5AF\u9E8A\u9E8B"+
	"\u9E8C\uE5AE\u9E8D\u9E8E\u9E8F\u9E90\u9E91\u9E92"+
	"\u9E93\u9E94\u9E95\u9E96\u9E97\u9E98\u9E99\u9E9A"+
	"\u9E9B\u9E9C\u9E9D\u9E9E\uB9E0\u9E9F\u9EA0\uE5B0"+
	"\u9EA1\u9EA2\u9EA3\u9EA4\u9EA5\u9EA6\u9EA7\u9EA8"+
	"\u9EA9\u9EAA\u9EAB\u9EAC\u9EAD\u9EAE\uE5B1\u9EAF"+
	"\u9EB0\u9EB1\u9EB2\u9EB3\u9EB4\u9EB5\u9EB6\u9EB7"+
	"\u9EB8\u9EB9\u9EBA\uBBF0\uECE1\uC3F0\u9EBB\uB5C6"+
	"\uBBD2\u9EBC\u9EBD\u9EBE\u9EBF\uC1E9\uD4EE\u9EC0"+
	"\uBEC4\u9EC1\u9EC2\u9EC3\uD7C6\u9EC4\uD4D6\uB2D3"+
	"\uECBE\u9EC5\u9EC6\u9EC7\u9EC8\uEAC1\u9EC9\u9ECA"+
	"\u9ECB\uC2AF\uB4B6\u9ECC\u9ECD\u9ECE\uD1D7\u9ECF"+
	"\u9ED0\u9ED1\uB3B4\u9ED2\uC8B2\uBFBB\uECC0\u9ED3"+
	"\u9ED4\uD6CB\u9ED5\u9ED6\uECBF\uECC1\u9ED7\u9ED8"+
	"\u9ED9\u9EDA\u9EDB\u9EDC\u9EDD\u9EDE\u9EDF\u9EE0"+
	"\u9EE1\u9EE2\u9EE3\uECC5\uBEE6\uCCBF\uC5DA\uBEBC"+
	"\u9EE4\uECC6\u9EE5\uB1FE\u9EE6\u9EE7\u9EE8\uECC4"+
	"\uD5A8\uB5E3\u9EE9\uECC2\uC1B6\uB3E3\u9EEA\u9EEB"+
	"\uECC3\uCBB8\uC0C3\uCCFE\u9EEC\u9EED\u9EEE\u9EEF"+
	"\uC1D2\u9EF0\uECC8\u9EF1\u9EF2\u9EF3\u9EF4\u9EF5"+
	"\u9EF6\u9EF7\u9EF8\u9EF9\u9EFA\u9EFB\u9EFC\u9EFD"+
	"\uBAE6\uC0D3\u9EFE\uD6F2\u9F40\u9F41\u9F42\uD1CC"+
	"\u9F43\u9F44\u9F45\u9F46\uBFBE\u9F47\uB7B3\uC9D5"+
	"\uECC7\uBBE2\u9F48\uCCCC\uBDFD\uC8C8\u9F49\uCFA9"+
	"\u9F4A\u9F4B\u9F4C\u9F4D\u9F4E\u9F4F\u9F50\uCDE9"+
	"\u9F51\uC5EB\u9F52\u9F53\u9F54\uB7E9\u9F55\u9F56"+
	"\u9F57\u9F58\u9F59\u9F5A\u9F5B\u9F5C\u9F5D\u9F5E"+
	"\u9F5F\uD1C9\uBAB8\u9F60\u9F61\u9F62\u9F63\u9F64"+
	"\uECC9\u9F65\u9F66\uECCA\u9F67\uBBC0\uECCB\u9F68"+
	"\uECE2\uB1BA\uB7D9\u9F69\u9F6A\u9F6B\u9F6C\u9F6D"+
	"\u9F6E\u9F6F\u9F70\u9F71\u9F72\u9F73\uBDB9\u9F74"+
	"\u9F75\u9F76\u9F77\u9F78\u9F79\u9F7A\u9F7B\uECCC"+
	"\uD1E6\uECCD\u9F7C\u9F7D\u9F7E\u9F80\uC8BB\u9F81"+
	"\u9F82\u9F83\u9F84\u9F85\u9F86\u9F87\u9F88\u9F89"+
	"\u9F8A\u9F8B\u9F8C\u9F8D\u9F8E\uECD1\u9F8F\u9F90"+
	"\u9F91\u9F92\uECD3\u9F93\uBBCD\u9F94\uBCE5\u9F95"+
	"\u9F96\u9F97\u9F98\u9F99\u9F9A\u9F9B\u9F9C\u9F9D"+
	"\u9F9E\u9F9F\u9FA0\u9FA1\uECCF\u9FA2\uC9B7\u9FA3"+
	"\u9FA4\u9FA5\u9FA6\u9FA7\uC3BA\u9FA8\uECE3\uD5D5"+
	"\uECD0\u9FA9\u9FAA\u9FAB\u9FAC\u9FAD\uD6F3\u9FAE"+
	"\u9FAF\u9FB0\uECD2\uECCE\u9FB1\u9FB2\u9FB3\u9FB4"+
	"\uECD4\u9FB5\uECD5\u9FB6\u9FB7\uC9BF\u9FB8\u9FB9"+
	"\u9FBA\u9FBB\u9FBC\u9FBD\uCFA8\u9FBE\u9FBF\u9FC0"+
	"\u9FC1\u9FC2\uD0DC\u9FC3\u9FC4\u9FC5\u9FC6\uD1AC"+
	"\u9FC7\u9FC8\u9FC9\u9FCA\uC8DB\u9FCB\u9FCC\u9FCD"+
	"\uECD6\uCEF5\u9FCE\u9FCF\u9FD0\u9FD1\u9FD2\uCAEC"+
	"\uECDA\u9FD3\u9FD4\u9FD5\u9FD6\u9FD7\u9FD8\u9FD9"+
	"\uECD9\u9FDA\u9FDB\u9FDC\uB0BE\u9FDD\u9FDE\u9FDF"+
	"\u9FE0\u9FE1\u9FE2\uECD7\u9FE3\uECD8\u9FE4\u9FE5"+
	"\u9FE6\uECE4\u9FE7\u9FE8\u9FE9\u9FEA\u9FEB\u9FEC"+
	"\u9FED\u9FEE\u9FEF\uC8BC\u9FF0\u9FF1\u9FF2\u9FF3"+
	"\u9FF4\u9FF5\u9FF6\u9FF7\u9FF8\u9FF9\uC1C7\u9FFA"+
	"\u9FFB\u9FFC\u9FFD\u9FFE\uECDC\uD1E0\uA040\uA041"+
	"\uA042\uA043\uA044\uA045\uA046\uA047\uA048\uA049"+
	"\uECDB\uA04A\uA04B\uA04C\uA04D\uD4EF\uA04E\uECDD"+
	"\uA04F\uA050\uA051\uA052\uA053\uA054\uDBC6\uA055"+
	"\uA056\uA057\uA058\uA059\uA05A\uA05B\uA05C\uA05D"+
	"\uA05E\uECDE\uA05F\uA060\uA061\uA062\uA063\uA064"+
	"\uA065\uA066\uA067\uA068\uA069\uA06A\uB1AC\uA06B"+
	"\uA06C\uA06D\uA06E\uA06F\uA070\uA071\uA072\uA073"+
	"\uA074\uA075\uA076\uA077\uA078\uA079\uA07A\uA07B"+
	"\uA07C\uA07D\uA07E\uA080\uA081\uECDF\uA082\uA083"+
	"\uA084\uA085\uA086\uA087\uA088\uA089\uA08A\uA08B"+
	"\uECE0\uA08C\uD7A6\uA08D\uC5C0\uA08E\uA08F\uA090"+
	"\uEBBC\uB0AE\uA091\uA092\uA093\uBEF4\uB8B8\uD2AF"+
	"\uB0D6\uB5F9\uA094\uD8B3\uA095\uCBAC\uA096\uE3DD"+
	"\uA097\uA098\uA099\uA09A\uA09B\uA09C\uA09D\uC6AC"+
	"\uB0E6\uA09E\uA09F\uA0A0\uC5C6\uEBB9\uA0A1\uA0A2"+
	"\uA0A3\uA0A4\uEBBA\uA0A5\uA0A6\uA0A7\uEBBB\uA0A8"+
	"\uA0A9\uD1C0\uA0AA\uC5A3\uA0AB\uEAF2\uA0AC\uC4B2"+
	"\uA0AD\uC4B5\uC0CE\uA0AE\uA0AF\uA0B0\uEAF3\uC4C1"+
	"\uA0B1\uCEEF\uA0B2\uA0B3\uA0B4\uA0B5\uEAF0\uEAF4"+
	"\uA0B6\uA0B7\uC9FC\uA0B8\uA0B9\uC7A3\uA0BA\uA0BB"+
	"\uA0BC\uCCD8\uCEFE\uA0BD\uA0BE\uA0BF\uEAF5\uEAF6"+
	"\uCFAC\uC0E7\uA0C0\uA0C1\uEAF7\uA0C2\uA0C3\uA0C4"+
	"\uA0C5\uA0C6\uB6BF\uEAF8\uA0C7\uEAF9\uA0C8\uEAFA"+
	"\uA0C9\uA0CA\uEAFB\uA0CB\uA0CC\uA0CD\uA0CE\uA0CF"+
	"\uA0D0\uA0D1\uA0D2\uA0D3\uA0D4\uA0D5\uA0D6\uEAF1"+
	"\uA0D7\uA0D8\uA0D9\uA0DA\uA0DB\uA0DC\uA0DD\uA0DE"+
	"\uA0DF\uA0E0\uA0E1\uA0E2\uC8AE\uE1EB\uA0E3\uB7B8"+
	"\uE1EC\uA0E4\uA0E5\uA0E6\uE1ED\uA0E7\uD7B4\uE1EE"+
	"\uE1EF\uD3CC\uA0E8\uA0E9\uA0EA\uA0EB\uA0EC\uA0ED"+
	"\uA0EE\uE1F1\uBFF1\uE1F0\uB5D2\uA0EF\uA0F0\uA0F1"+
	"\uB1B7\uA0F2\uA0F3\uA0F4\uA0F5\uE1F3\uE1F2\uA0F6"+
	"\uBAFC\uA0F7\uE1F4\uA0F8\uA0F9\uA0FA\uA0FB\uB9B7"+
	"\uA0FC\uBED1\uA0FD\uA0FE\uAA40\uAA41\uC4FC\uAA42"+
	"\uBADD\uBDC6\uAA43\uAA44\uAA45\uAA46\uAA47\uAA48"+
	"\uE1F5\uE1F7\uAA49\uAA4A\uB6C0\uCFC1\uCAA8\uE1F6"+
	"\uD5F8\uD3FC\uE1F8\uE1FC\uE1F9\uAA4B\uAA4C\uE1FA"+
	"\uC0EA\uAA4D\uE1FE\uE2A1\uC0C7\uAA4E\uAA4F\uAA50"+
	"\uAA51\uE1FB\uAA52\uE1FD\uAA53\uAA54\uAA55\uAA56"+
	"\uAA57\uAA58\uE2A5\uAA59\uAA5A\uAA5B\uC1D4\uAA5C"+
	"\uAA5D\uAA5E\uAA5F\uE2A3\uAA60\uE2A8\uB2FE\uE2A2"+
	"\uAA61\uAA62\uAA63\uC3CD\uB2C2\uE2A7\uE2A6\uAA64"+
	"\uAA65\uE2A4\uE2A9\uAA66\uAA67\uE2AB\uAA68\uAA69"+
	"\uAA6A\uD0C9\uD6ED\uC3A8\uE2AC\uAA6B\uCFD7\uAA6C"+
	"\uAA6D\uE2AE\uAA6E\uAA6F\uBAEF\uAA70\uAA71\uE9E0"+
	"\uE2AD\uE2AA\uAA72\uAA73\uAA74\uAA75\uBBAB\uD4B3"+
	"\uAA76\uAA77\uAA78\uAA79\uAA7A\uAA7B\uAA7C\uAA7D"+
	"\uAA7E\uAA80\uAA81\uAA82\uAA83\uE2B0\uAA84\uAA85"+
	"\uE2AF\uAA86\uE9E1\uAA87\uAA88\uAA89\uAA8A\uE2B1"+
	"\uAA8B\uAA8C\uAA8D\uAA8E\uAA8F\uAA90\uAA91\uAA92"+
	"\uE2B2\uAA93\uAA94\uAA95\uAA96\uAA97\uAA98\uAA99"+
	"\uAA9A\uAA9B\uAA9C\uAA9D\uE2B3\uCCA1\uAA9E\uE2B4"+
	"\uAA9F\uAAA0\uAB40\uAB41\uAB42\uAB43\uAB44\uAB45"+
	"\uAB46\uAB47\uAB48\uAB49\uAB4A\uAB4B\uE2B5\uAB4C"+
	"\uAB4D\uAB4E\uAB4F\uAB50\uD0FE\uAB51\uAB52\uC2CA"+
	"\uAB53\uD3F1\uAB54\uCDF5\uAB55\uAB56\uE7E0\uAB57"+
	"\uAB58\uE7E1\uAB59\uAB5A\uAB5B\uAB5C\uBEC1\uAB5D"+
	"\uAB5E\uAB5F\uAB60\uC2EA\uAB61\uAB62\uAB63\uE7E4"+
	"\uAB64\uAB65\uE7E3\uAB66\uAB67\uAB68\uAB69\uAB6A"+
	"\uAB6B\uCDE6\uAB6C\uC3B5\uAB6D\uAB6E\uE7E2\uBBB7"+
	"\uCFD6\uAB6F\uC1E1\uE7E9\uAB70\uAB71\uAB72\uE7E8"+
	"\uAB73\uAB74\uE7F4\uB2A3\uAB75\uAB76\uAB77\uAB78"+
	"\uE7EA\uAB79\uE7E6\uAB7A\uAB7B\uAB7C\uAB7D\uAB7E"+
	"\uE7EC\uE7EB\uC9BA\uAB80\uAB81\uD5E4\uAB82\uE7E5"+
	"\uB7A9\uE7E7\uAB83\uAB84\uAB85\uAB86\uAB87\uAB88"+
	"\uAB89\uE7EE\uAB8A\uAB8B\uAB8C\uAB8D\uE7F3\uAB8E"+
	"\uD6E9\uAB8F\uAB90\uAB91\uAB92\uE7ED\uAB93\uE7F2"+
	"\uAB94\uE7F1\uAB95\uAB96\uAB97\uB0E0\uAB98\uAB99"+
	"\uAB9A\uAB9B\uE7F5\uAB9C\uAB9D\uAB9E\uAB9F\uABA0"+
	"\uAC40\uAC41\uAC42\uAC43\uAC44\uAC45\uAC46\uAC47"+
	"\uAC48\uAC49\uAC4A\uC7F2\uAC4B\uC0C5\uC0ED\uAC4C"+
	"\uAC4D\uC1F0\uE7F0\uAC4E\uAC4F\uAC50\uAC51\uE7F6"+
	"\uCBF6\uAC52\uAC53\uAC54\uAC55\uAC56\uAC57\uAC58"+
	"\uAC59\uAC5A\uE8A2\uE8A1\uAC5B\uAC5C\uAC5D\uAC5E"+
	"\uAC5F\uAC60\uD7C1\uAC61\uAC62\uE7FA\uE7F9\uAC63"+
	"\uE7FB\uAC64\uE7F7\uAC65\uE7FE\uAC66\uE7FD\uAC67"+
	"\uE7FC\uAC68\uAC69\uC1D5\uC7D9\uC5FD\uC5C3\uAC6A"+
	"\uAC6B\uAC6C\uAC6D\uAC6E\uC7ED\uAC6F\uAC70\uAC71"+
	"\uAC72\uE8A3\uAC73\uAC74\uAC75\uAC76\uAC77\uAC78"+
	"\uAC79\uAC7A\uAC7B\uAC7C\uAC7D\uAC7E\uAC80\uAC81"+
	"\uAC82\uAC83\uAC84\uAC85\uAC86\uE8A6\uAC87\uE8A5"+
	"\uAC88\uE8A7\uBAF7\uE7F8\uE8A4\uAC89\uC8F0\uC9AA"+
	"\uAC8A\uAC8B\uAC8C\uAC8D\uAC8E\uAC8F\uAC90\uAC91"+
	"\uAC92\uAC93\uAC94\uAC95\uAC96\uE8A9\uAC97\uAC98"+
	"\uB9E5\uAC99\uAC9A\uAC9B\uAC9C\uAC9D\uD1FE\uE8A8"+
	"\uAC9E\uAC9F\uACA0\uAD40\uAD41\uAD42\uE8AA\uAD43"+
	"\uE8AD\uE8AE\uAD44\uC1A7\uAD45\uAD46\uAD47\uE8AF"+
	"\uAD48\uAD49\uAD4A\uE8B0\uAD4B\uAD4C\uE8AC\uAD4D"+
	"\uE8B4\uAD4E\uAD4F\uAD50\uAD51\uAD52\uAD53\uAD54"+
	"\uAD55\uAD56\uAD57\uAD58\uE8AB\uAD59\uE8B1\uAD5A"+
	"\uAD5B\uAD5C\uAD5D\uAD5E\uAD5F\uAD60\uAD61\uE8B5"+
	"\uE8B2\uE8B3\uAD62\uAD63\uAD64\uAD65\uAD66\uAD67"+
	"\uAD68\uAD69\uAD6A\uAD6B\uAD6C\uAD6D\uAD6E\uAD6F"+
	"\uAD70\uAD71\uE8B7\uAD72\uAD73\uAD74\uAD75\uAD76"+
	"\uAD77\uAD78\uAD79\uAD7A\uAD7B\uAD7C\uAD7D\uAD7E"+
	"\uAD80\uAD81\uAD82\uAD83\uAD84\uAD85\uAD86\uAD87"+
	"\uAD88\uAD89\uE8B6\uAD8A\uAD8B\uAD8C\uAD8D\uAD8E"+
	"\uAD8F\uAD90\uAD91\uAD92\uB9CF\uAD93\uF0AC\uAD94"+
	"\uF0AD\uAD95\uC6B0\uB0EA\uC8BF\uAD96\uCDDF\uAD97"+
	"\uAD98\uAD99\uAD9A\uAD9B\uAD9C\uAD9D\uCECD\uEAB1"+
	"\uAD9E\uAD9F\uADA0\uAE40\uEAB2\uAE41\uC6BF\uB4C9"+
	"\uAE42\uAE43\uAE44\uAE45\uAE46\uAE47\uAE48\uEAB3"+
	"\uAE49\uAE4A\uAE4B\uAE4C\uD5E7\uAE4D\uAE4E\uAE4F"+
	"\uAE50\uAE51\uAE52\uAE53\uAE54\uDDF9\uAE55\uEAB4"+
	"\uAE56\uEAB5\uAE57\uEAB6\uAE58\uAE59\uAE5A\uAE5B"+
	"\uB8CA\uDFB0\uC9F5\uAE5C\uCCF0\uAE5D\uAE5E\uC9FA"+
	"\uAE5F\uAE60\uAE61\uAE62\uAE63\uC9FB\uAE64\uAE65"+
	"\uD3C3\uCBA6\uAE66\uB8A6\uF0AE\uB1C2\uAE67\uE5B8"+
	"\uCCEF\uD3C9\uBCD7\uC9EA\uAE68\uB5E7\uAE69\uC4D0"+
	"\uB5E9\uAE6A\uEEAE\uBBAD\uAE6B\uAE6C\uE7DE\uAE6D"+
	"\uEEAF\uAE6E\uAE6F\uAE70\uAE71\uB3A9\uAE72\uAE73"+
	"\uEEB2\uAE74\uAE75\uEEB1\uBDE7\uAE76\uEEB0\uCEB7"+
	"\uAE77\uAE78\uAE79\uAE7A\uC5CF\uAE7B\uAE7C\uAE7D"+
	"\uAE7E\uC1F4\uDBCE\uEEB3\uD0F3\uAE80\uAE81\uAE82"+
	"\uAE83\uAE84\uAE85\uAE86\uAE87\uC2D4\uC6E8\uAE88"+
	"\uAE89\uAE8A\uB7AC\uAE8B\uAE8C\uAE8D\uAE8E\uAE8F"+
	"\uAE90\uAE91\uEEB4\uAE92\uB3EB\uAE93\uAE94\uAE95"+
	"\uBBFB\uEEB5\uAE96\uAE97\uAE98\uAE99\uAE9A\uE7DC"+
	"\uAE9B\uAE9C\uAE9D\uEEB6\uAE9E\uAE9F\uBDAE\uAEA0"+
	"\uAF40\uAF41\uAF42\uF1E2\uAF43\uAF44\uAF45\uCAE8"+
	"\uAF46\uD2C9\uF0DA\uAF47\uF0DB\uAF48\uF0DC\uC1C6"+
	"\uAF49\uB8ED\uBECE\uAF4A\uAF4B\uF0DE\uAF4C\uC5B1"+
	"\uF0DD\uD1F1\uAF4D\uF0E0\uB0CC\uBDEA\uAF4E\uAF4F"+
	"\uAF50\uAF51\uAF52\uD2DF\uF0DF\uAF53\uB4AF\uB7E8"+
	"\uF0E6\uF0E5\uC6A3\uF0E1\uF0E2\uB4C3\uAF54\uAF55"+
	"\uF0E3\uD5EE\uAF56\uAF57\uCCDB\uBED2\uBCB2\uAF58"+
	"\uAF59\uAF5A\uF0E8\uF0E7\uF0E4\uB2A1\uAF5B\uD6A2"+
	"\uD3B8\uBEB7\uC8AC\uAF5C\uAF5D\uF0EA\uAF5E\uAF5F"+
	"\uAF60\uAF61\uD1F7\uAF62\uD6CC\uBADB\uF0E9\uAF63"+
	"\uB6BB\uAF64\uAF65\uCDB4\uAF66\uAF67\uC6A6\uAF68"+
	"\uAF69\uAF6A\uC1A1\uF0EB\uF0EE\uAF6B\uF0ED\uF0F0"+
	"\uF0EC\uAF6C\uBBBE\uF0EF\uAF6D\uAF6E\uAF6F\uAF70"+
	"\uCCB5\uF0F2\uAF71\uAF72\uB3D5\uAF73\uAF74\uAF75"+
	"\uAF76\uB1D4\uAF77\uAF78\uF0F3\uAF79\uAF7A\uF0F4"+
	"\uF0F6\uB4E1\uAF7B\uF0F1\uAF7C\uF0F7\uAF7D\uAF7E"+
	"\uAF80\uAF81\uF0FA\uAF82\uF0F8\uAF83\uAF84\uAF85"+
	"\uF0F5\uAF86\uAF87\uAF88\uAF89\uF0FD\uAF8A\uF0F9"+
	"\uF0FC\uF0FE\uAF8B\uF1A1\uAF8C\uAF8D\uAF8E\uCEC1"+
	"\uF1A4\uAF8F\uF1A3\uAF90\uC1F6\uF0FB\uCADD\uAF91"+
	"\uAF92\uB4F1\uB1F1\uCCB1\uAF93\uF1A6\uAF94\uAF95"+
	"\uF1A7\uAF96\uAF97\uF1AC\uD5CE\uF1A9\uAF98\uAF99"+
	"\uC8B3\uAF9A\uAF9B\uAF9C\uF1A2\uAF9D\uF1AB\uF1A8"+
	"\uF1A5\uAF9E\uAF9F\uF1AA\uAFA0\uB040\uB041\uB042"+
	"\uB043\uB044\uB045\uB046\uB0A9\uF1AD\uB047\uB048"+
	"\uB049\uB04A\uB04B\uB04C\uF1AF\uB04D\uF1B1\uB04E"+
	"\uB04F\uB050\uB051\uB052\uF1B0\uB053\uF1AE\uB054"+
	"\uB055\uB056\uB057\uD1A2\uB058\uB059\uB05A\uB05B"+
	"\uB05C\uB05D\uB05E\uF1B2\uB05F\uB060\uB061\uF1B3"+
	"\uB062\uB063\uB064\uB065\uB066\uB067\uB068\uB069"+
	"\uB9EF\uB06A\uB06B\uB5C7\uB06C\uB0D7\uB0D9\uB06D"+
	"\uB06E\uB06F\uD4ED\uB070\uB5C4\uB071\uBDD4\uBBCA"+
	"\uF0A7\uB072\uB073\uB8DE\uB074\uB075\uF0A8\uB076"+
	"\uB077\uB0A8\uB078\uF0A9\uB079\uB07A\uCDEE\uB07B"+
	"\uB07C\uF0AA\uB07D\uB07E\uB080\uB081\uB082\uB083"+
	"\uB084\uB085\uB086\uB087\uF0AB\uB088\uB089\uB08A"+
	"\uB08B\uB08C\uB08D\uB08E\uB08F\uB090\uC6A4\uB091"+
	"\uB092\uD6E5\uF1E4\uB093\uF1E5\uB094\uB095\uB096"+
	"\uB097\uB098\uB099\uB09A\uB09B\uB09C\uB09D\uC3F3"+
	"\uB09E\uB09F\uD3DB\uB0A0\uB140\uD6D1\uC5E8\uB141"+
	"\uD3AF\uB142\uD2E6\uB143\uB144\uEEC1\uB0BB\uD5B5"+
	"\uD1CE\uBCE0\uBAD0\uB145\uBFF8\uB146\uB8C7\uB5C1"+
	"\uC5CC\uB147\uB148\uCAA2\uB149\uB14A\uB14B\uC3CB"+
	"\uB14C\uB14D\uB14E\uB14F\uB150\uEEC2\uB151\uB152"+
	"\uB153\uB154\uB155\uB156\uB157\uB158\uC4BF\uB6A2"+
	"\uB159\uEDEC\uC3A4\uB15A\uD6B1\uB15B\uB15C\uB15D"+
	"\uCFE0\uEDEF\uB15E\uB15F\uC5CE\uB160\uB6DC\uB161"+
	"\uB162\uCAA1\uB163\uB164\uEDED\uB165\uB166\uEDF0"+
	"\uEDF1\uC3BC\uB167\uBFB4\uB168\uEDEE\uB169\uB16A"+
	"\uB16B\uB16C\uB16D\uB16E\uB16F\uB170\uB171\uB172"+
	"\uB173\uEDF4\uEDF2\uB174\uB175\uB176\uB177\uD5E6"+
	"\uC3DF\uB178\uEDF3\uB179\uB17A\uB17B\uEDF6\uB17C"+
	"\uD5A3\uD1A3\uB17D\uB17E\uB180\uEDF5\uB181\uC3D0"+
	"\uB182\uB183\uB184\uB185\uB186\uEDF7\uBFF4\uBEEC"+
	"\uEDF8\uB187\uCCF7\uB188\uD1DB\uB189\uB18A\uB18B"+
	"\uD7C5\uD5F6\uB18C\uEDFC\uB18D\uB18E\uB18F\uEDFB"+
	"\uB190\uB191\uB192\uB193\uB194\uB195\uB196\uB197"+
	"\uEDF9\uEDFA\uB198\uB199\uB19A\uB19B\uB19C\uB19D"+
	"\uB19E\uB19F\uEDFD\uBEA6\uB1A0\uB240\uB241\uB242"+
	"\uB243\uCBAF\uEEA1\uB6BD\uB244\uEEA2\uC4C0\uB245"+
	"\uEDFE\uB246\uB247\uBDDE\uB2C7\uB248\uB249\uB24A"+
	"\uB24B\uB24C\uB24D\uB24E\uB24F\uB250\uB251\uB252"+
	"\uB253\uB6C3\uB254\uB255\uB256\uEEA5\uD8BA\uEEA3"+
	"\uEEA6\uB257\uB258\uB259\uC3E9\uB3F2\uB25A\uB25B"+
	"\uB25C\uB25D\uB25E\uB25F\uEEA7\uEEA4\uCFB9\uB260"+
	"\uB261\uEEA8\uC2F7\uB262\uB263\uB264\uB265\uB266"+
	"\uB267\uB268\uB269\uB26A\uB26B\uB26C\uB26D\uEEA9"+
	"\uEEAA\uB26E\uDEAB\uB26F\uB270\uC6B3\uB271\uC7C6"+
	"\uB272\uD6F5\uB5C9\uB273\uCBB2\uB274\uB275\uB276"+
	"\uEEAB\uB277\uB278\uCDAB\uB279\uEEAC\uB27A\uB27B"+
	"\uB27C\uB27D\uB27E\uD5B0\uB280\uEEAD\uB281\uF6C4"+
	"\uB282\uB283\uB284\uB285\uB286\uB287\uB288\uB289"+
	"\uB28A\uB28B\uB28C\uB28D\uB28E\uDBC7\uB28F\uB290"+
	"\uB291\uB292\uB293\uB294\uB295\uB296\uB297\uB4A3"+
	"\uB298\uB299\uB29A\uC3AC\uF1E6\uB29B\uB29C\uB29D"+
	"\uB29E\uB29F\uCAB8\uD2D3\uB2A0\uD6AA\uB340\uEFF2"+
	"\uB341\uBED8\uB342\uBDC3\uEFF3\uB6CC\uB0AB\uB343"+
	"\uB344\uB345\uB346\uCAAF\uB347\uB348\uEDB6\uB349"+
	"\uEDB7\uB34A\uB34B\uB34C\uB34D\uCEF9\uB7AF\uBFF3"+
	"\uEDB8\uC2EB\uC9B0\uB34E\uB34F\uB350\uB351\uB352"+
	"\uB353\uEDB9\uB354\uB355\uC6F6\uBFB3\uB356\uB357"+
	"\uB358\uEDBC\uC5F8\uB359\uD1D0\uB35A\uD7A9\uEDBA"+
	"\uEDBB\uB35B\uD1E2\uB35C\uEDBF\uEDC0\uB35D\uEDC4"+
	"\uB35E\uB35F\uB360\uEDC8\uB361\uEDC6\uEDCE\uD5E8"+
	"\uB362\uEDC9\uB363\uB364\uEDC7\uEDBE\uB365\uB366"+
	"\uC5E9\uB367\uB368\uB369\uC6C6\uB36A\uB36B\uC9E9"+
	"\uD4D2\uEDC1\uEDC2\uEDC3\uEDC5\uB36C\uC0F9\uB36D"+
	"\uB4A1\uB36E\uB36F\uB370\uB371\uB9E8\uB372\uEDD0"+
	"\uB373\uB374\uB375\uB376\uEDD1\uB377\uEDCA\uB378"+
	"\uEDCF\uB379\uCEF8\uB37A\uB37B\uCBB6\uEDCC\uEDCD"+
	"\uB37C\uB37D\uB37E\uB380\uB381\uCFF5\uB382\uB383"+
	"\uB384\uB385\uB386\uB387\uB388\uB389\uB38A\uB38B"+
	"\uB38C\uB38D\uEDD2\uC1F2\uD3B2\uEDCB\uC8B7\uB38E"+
	"\uB38F\uB390\uB391\uB392\uB393\uB394\uB395\uBCEF"+
	"\uB396\uB397\uB398\uB399\uC5F0\uB39A\uB39B\uB39C"+
	"\uB39D\uB39E\uB39F\uB3A0\uB440\uB441\uB442\uEDD6"+
	"\uB443\uB5EF\uB444\uB445\uC2B5\uB0AD\uCBE9\uB446"+
	"\uB447\uB1AE\uB448\uEDD4\uB449\uB44A\uB44B\uCDEB"+
	"\uB5E2\uB44C\uEDD5\uEDD3\uEDD7\uB44D\uB44E\uB5FA"+
	"\uB44F\uEDD8\uB450\uEDD9\uB451\uEDDC\uB452\uB1CC"+
	"\uB453\uB454\uB455\uB456\uB457\uB458\uB459\uB45A"+
	"\uC5F6\uBCEE\uEDDA\uCCBC\uB2EA\uB45B\uB45C\uB45D"+
	"\uB45E\uEDDB\uB45F\uB460\uB461\uB462\uC4EB\uB463"+
	"\uB464\uB4C5\uB465\uB466\uB467\uB0F5\uB468\uB469"+
	"\uB46A\uEDDF\uC0DA\uB4E8\uB46B\uB46C\uB46D\uB46E"+
	"\uC5CD\uB46F\uB470\uB471\uEDDD\uBFC4\uB472\uB473"+
	"\uB474\uEDDE\uB475\uB476\uB477\uB478\uB479\uB47A"+
	"\uB47B\uB47C\uB47D\uB47E\uB480\uB481\uB482\uB483"+
	"\uC4A5\uB484\uB485\uB486\uEDE0\uB487\uB488\uB489"+
	"\uB48A\uB48B\uEDE1\uB48C\uEDE3\uB48D\uB48E\uC1D7"+
	"\uB48F\uB490\uBBC7\uB491\uB492\uB493\uB494\uB495"+
	"\uB496\uBDB8\uB497\uB498\uB499\uEDE2\uB49A\uB49B"+
	"\uB49C\uB49D\uB49E\uB49F\uB4A0\uB540\uB541\uB542"+
	"\uB543\uB544\uB545\uEDE4\uB546\uB547\uB548\uB549"+
	"\uB54A\uB54B\uB54C\uB54D\uB54E\uB54F\uEDE6\uB550"+
	"\uB551\uB552\uB553\uB554\uEDE5\uB555\uB556\uB557"+
	"\uB558\uB559\uB55A\uB55B\uB55C\uB55D\uB55E\uB55F"+
	"\uB560\uB561\uB562\uB563\uEDE7\uB564\uB565\uB566"+
	"\uB567\uB568\uCABE\uECEA\uC0F1\uB569\uC9E7\uB56A"+
	"\uECEB\uC6EE\uB56B\uB56C\uB56D\uB56E\uECEC\uB56F"+
	"\uC6ED\uECED\uB570\uB571\uB572\uB573\uB574\uB575"+
	"\uB576\uB577\uB578\uECF0\uB579\uB57A\uD7E6\uECF3"+
	"\uB57B\uB57C\uECF1\uECEE\uECEF\uD7A3\uC9F1\uCBEE"+
	"\uECF4\uB57D\uECF2\uB57E\uB580\uCFE9\uB581\uECF6"+
	"\uC6B1\uB582\uB583\uB584\uB585\uBCC0\uB586\uECF5"+
	"\uB587\uB588\uB589\uB58A\uB58B\uB58C\uB58D\uB5BB"+
	"\uBBF6\uB58E\uECF7\uB58F\uB590\uB591\uB592\uB593"+
	"\uD9F7\uBDFB\uB594\uB595\uC2BB\uECF8\uB596\uB597"+
	"\uB598\uB599\uECF9\uB59A\uB59B\uB59C\uB59D\uB8A3"+
	"\uB59E\uB59F\uB5A0\uB640\uB641\uB642\uB643\uB644"+
	"\uB645\uB646\uECFA\uB647\uB648\uB649\uB64A\uB64B"+
	"\uB64C\uB64D\uB64E\uB64F\uB650\uB651\uB652\uECFB"+
	"\uB653\uB654\uB655\uB656\uB657\uB658\uB659\uB65A"+
	"\uB65B\uB65C\uB65D\uECFC\uB65E\uB65F\uB660\uB661"+
	"\uB662\uD3ED\uD8AE\uC0EB\uB663\uC7DD\uBACC\uB664"+
	"\uD0E3\uCBBD\uB665\uCDBA\uB666\uB667\uB8D1\uB668"+
	"\uB669\uB1FC\uB66A\uC7EF\uB66B\uD6D6\uB66C\uB66D"+
	"\uB66E\uBFC6\uC3EB\uB66F\uB670\uEFF5\uB671\uB672"+
	"\uC3D8\uB673\uB674\uB675\uB676\uB677\uB678\uD7E2"+
	"\uB679\uB67A\uB67B\uEFF7\uB3D3\uB67C\uC7D8\uD1ED"+
	"\uB67D\uD6C8\uB67E\uEFF8\uB680\uEFF6\uB681\uBBFD"+
	"\uB3C6\uB682\uB683\uB684\uB685\uB686\uB687\uB688"+
	"\uBDD5\uB689\uB68A\uD2C6\uB68B\uBBE0\uB68C\uB68D"+
	"\uCFA1\uB68E\uEFFC\uEFFB\uB68F\uB690\uEFF9\uB691"+
	"\uB692\uB693\uB694\uB3CC\uB695\uC9D4\uCBB0\uB696"+
	"\uB697\uB698\uB699\uB69A\uEFFE\uB69B\uB69C\uB0DE"+
	"\uB69D\uB69E\uD6C9\uB69F\uB6A0\uB740\uEFFD\uB741"+
	"\uB3ED\uB742\uB743\uF6D5\uB744\uB745\uB746\uB747"+
	"\uB748\uB749\uB74A\uB74B\uB74C\uB74D\uB74E\uB74F"+
	"\uB750\uB751\uB752\uCEC8\uB753\uB754\uB755\uF0A2"+
	"\uB756\uF0A1\uB757\uB5BE\uBCDA\uBBFC\uB758\uB8E5"+
	"\uB759\uB75A\uB75B\uB75C\uB75D\uB75E\uC4C2\uB75F"+
	"\uB760\uB761\uB762\uB763\uB764\uB765\uB766\uB767"+
	"\uB768\uF0A3\uB769\uB76A\uB76B\uB76C\uB76D\uCBEB"+
	"\uB76E\uB76F\uB770\uB771\uB772\uB773\uB774\uB775"+
	"\uB776\uB777\uB778\uB779\uB77A\uB77B\uB77C\uB77D"+
	"\uB77E\uB780\uB781\uB782\uB783\uB784\uB785\uB786"+
	"\uF0A6\uB787\uB788\uB789\uD1A8\uB78A\uBEBF\uC7EE"+
	"\uF1B6\uF1B7\uBFD5\uB78B\uB78C\uB78D\uB78E\uB4A9"+
	"\uF1B8\uCDBB\uB78F\uC7D4\uD5AD\uB790\uF1B9\uB791"+
	"\uF1BA\uB792\uB793\uB794\uB795\uC7CF\uB796\uB797"+
	"\uB798\uD2A4\uD6CF\uB799\uB79A\uF1BB\uBDD1\uB4B0"+
	"\uBEBD\uB79B\uB79C\uB79D\uB4DC\uCED1\uB79E\uBFDF"+
	"\uF1BD\uB79F\uB7A0\uB840\uB841\uBFFA\uF1BC\uB842"+
	"\uF1BF\uB843\uB844\uB845\uF1BE\uF1C0\uB846\uB847"+
	"\uB848\uB849\uB84A\uF1C1\uB84B\uB84C\uB84D\uB84E"+
	"\uB84F\uB850\uB851\uB852\uB853\uB854\uB855\uC1FE"+
	"\uB856\uB857\uB858\uB859\uB85A\uB85B\uB85C\uB85D"+
	"\uB85E\uB85F\uB860\uC1A2\uB861\uB862\uB863\uB864"+
	"\uB865\uB866\uB867\uB868\uB869\uB86A\uCAFA\uB86B"+
	"\uB86C\uD5BE\uB86D\uB86E\uB86F\uB870\uBEBA\uBEB9"+
	"\uD5C2\uB871\uB872\uBFA2\uB873\uCDAF\uF1B5\uB874"+
	"\uB875\uB876\uB877\uB878\uB879\uBDDF\uB87A\uB6CB"+
	"\uB87B\uB87C\uB87D\uB87E\uB880\uB881\uB882\uB883"+
	"\uB884\uD6F1\uF3C3\uB885\uB886\uF3C4\uB887\uB8CD"+
	"\uB888\uB889\uB88A\uF3C6\uF3C7\uB88B\uB0CA\uB88C"+
	"\uF3C5\uB88D\uF3C9\uCBF1\uB88E\uB88F\uB890\uF3CB"+
	"\uB891\uD0A6\uB892\uB893\uB1CA\uF3C8\uB894\uB895"+
	"\uB896\uF3CF\uB897\uB5D1\uB898\uB899\uF3D7\uB89A"+
	"\uF3D2\uB89B\uB89C\uB89D\uF3D4\uF3D3\uB7FB\uB89E"+
	"\uB1BF\uB89F\uF3CE\uF3CA\uB5DA\uB8A0\uF3D0\uB940"+
	"\uB941\uF3D1\uB942\uF3D5\uB943\uB944\uB945\uB946"+
	"\uF3CD\uB947\uBCE3\uB948\uC1FD\uB949\uF3D6\uB94A"+
	"\uB94B\uB94C\uB94D\uB94E\uB94F\uF3DA\uB950\uF3CC"+
	"\uB951\uB5C8\uB952\uBDEE\uF3DC\uB953\uB954\uB7A4"+
	"\uBFF0\uD6FE\uCDB2\uB955\uB4F0\uB956\uB2DF\uB957"+
	"\uF3D8\uB958\uF3D9\uC9B8\uB959\uF3DD\uB95A\uB95B"+
	"\uF3DE\uB95C\uF3E1\uB95D\uB95E\uB95F\uB960\uB961"+
	"\uB962\uB963\uB964\uB965\uB966\uB967\uF3DF\uB968"+
	"\uB969\uF3E3\uF3E2\uB96A\uB96B\uF3DB\uB96C\uBFEA"+
	"\uB96D\uB3EF\uB96E\uF3E0\uB96F\uB970\uC7A9\uB971"+
	"\uBCF2\uB972\uB973\uB974\uB975\uF3EB\uB976\uB977"+
	"\uB978\uB979\uB97A\uB97B\uB97C\uB9BF\uB97D\uB97E"+
	"\uF3E4\uB980\uB981\uB982\uB2AD\uBBFE\uB983\uCBE3"+
	"\uB984\uB985\uB986\uB987\uF3ED\uF3E9\uB988\uB989"+
	"\uB98A\uB9DC\uF3EE\uB98B\uB98C\uB98D\uF3E5\uF3E6"+
	"\uF3EA\uC2E1\uF3EC\uF3EF\uF3E8\uBCFD\uB98E\uB98F"+
	"\uB990\uCFE4\uB991\uB992\uF3F0\uB993\uB994\uB995"+
	"\uF3E7\uB996\uB997\uB998\uB999\uB99A\uB99B\uB99C"+
	"\uB99D\uF3F2\uB99E\uB99F\uB9A0\uBA40\uD7AD\uC6AA"+
	"\uBA41\uBA42\uBA43\uBA44\uF3F3\uBA45\uBA46\uBA47"+
	"\uBA48\uF3F1\uBA49\uC2A8\uBA4A\uBA4B\uBA4C\uBA4D"+
	"\uBA4E\uB8DD\uF3F5\uBA4F\uBA50\uF3F4\uBA51\uBA52"+
	"\uBA53\uB4DB\uBA54\uBA55\uBA56\uF3F6\uF3F7\uBA57"+
	"\uBA58\uBA59\uF3F8\uBA5A\uBA5B\uBA5C\uC0BA\uBA5D"+
	"\uBA5E\uC0E9\uBA5F\uBA60\uBA61\uBA62\uBA63\uC5F1"+
	"\uBA64\uBA65\uBA66\uBA67\uF3FB\uBA68\uF3FA\uBA69"+
	"\uBA6A\uBA6B\uBA6C\uBA6D\uBA6E\uBA6F\uBA70\uB4D8"+
	"\uBA71\uBA72\uBA73\uF3FE\uF3F9\uBA74\uBA75\uF3FC"+
	"\uBA76\uBA77\uBA78\uBA79\uBA7A\uBA7B\uF3FD\uBA7C"+
	"\uBA7D\uBA7E\uBA80\uBA81\uBA82\uBA83\uBA84\uF4A1"+
	"\uBA85\uBA86\uBA87\uBA88\uBA89\uBA8A\uF4A3\uBBC9"+
	"\uBA8B\uBA8C\uF4A2\uBA8D\uBA8E\uBA8F\uBA90\uBA91"+
	"\uBA92\uBA93\uBA94\uBA95\uBA96\uBA97\uBA98\uBA99"+
	"\uF4A4\uBA9A\uBA9B\uBA9C\uBA9D\uBA9E\uBA9F\uB2BE"+
	"\uF4A6\uF4A5\uBAA0\uBB40\uBB41\uBB42\uBB43\uBB44"+
	"\uBB45\uBB46\uBB47\uBB48\uBB49\uBCAE\uBB4A\uBB4B"+
	"\uBB4C\uBB4D\uBB4E\uBB4F\uBB50\uBB51\uBB52\uBB53"+
	"\uBB54\uBB55\uBB56\uBB57\uBB58\uBB59\uBB5A\uBB5B"+
	"\uBB5C\uBB5D\uBB5E\uBB5F\uBB60\uBB61\uBB62\uBB63"+
	"\uBB64\uBB65\uBB66\uBB67\uBB68\uBB69\uBB6A\uBB6B"+
	"\uBB6C\uBB6D\uBB6E\uC3D7\uD9E1\uBB6F\uBB70\uBB71"+
	"\uBB72\uBB73\uBB74\uC0E0\uF4CC\uD7D1\uBB75\uBB76"+
	"\uBB77\uBB78\uBB79\uBB7A\uBB7B\uBB7C\uBB7D\uBB7E"+
	"\uBB80\uB7DB\uBB81\uBB82\uBB83\uBB84\uBB85\uBB86"+
	"\uBB87\uF4CE\uC1A3\uBB88\uBB89\uC6C9\uBB8A\uB4D6"+
	"\uD5B3\uBB8B\uBB8C\uBB8D\uF4D0\uF4CF\uF4D1\uCBDA"+
	"\uBB8E\uBB8F\uF4D2\uBB90\uD4C1\uD6E0\uBB91\uBB92"+
	"\uBB93\uBB94\uB7E0\uBB95\uBB96\uBB97\uC1B8\uBB98"+
	"\uBB99\uC1BB\uF4D3\uBEAC\uBB9A\uBB9B\uBB9C\uBB9D"+
	"\uBB9E\uB4E2\uBB9F\uBBA0\uF4D4\uF4D5\uBEAB\uBC40"+
	"\uBC41\uF4D6\uBC42\uBC43\uBC44\uF4DB\uBC45\uF4D7"+
	"\uF4DA\uBC46\uBAFD\uBC47\uF4D8\uF4D9\uBC48\uBC49"+
	"\uBC4A\uBC4B\uBC4C\uBC4D\uBC4E\uB8E2\uCCC7\uF4DC"+
	"\uBC4F\uB2DA\uBC50\uBC51\uC3D3\uBC52\uBC53\uD4E3"+
	"\uBFB7\uBC54\uBC55\uBC56\uBC57\uBC58\uBC59\uBC5A"+
	"\uF4DD\uBC5B\uBC5C\uBC5D\uBC5E\uBC5F\uBC60\uC5B4"+
	"\uBC61\uBC62\uBC63\uBC64\uBC65\uBC66\uBC67\uBC68"+
	"\uF4E9\uBC69\uBC6A\uCFB5\uBC6B\uBC6C\uBC6D\uBC6E"+
	"\uBC6F\uBC70\uBC71\uBC72\uBC73\uBC74\uBC75\uBC76"+
	"\uBC77\uBC78\uCEC9\uBC79\uBC7A\uBC7B\uBC7C\uBC7D"+
	"\uBC7E\uBC80\uBC81\uBC82\uBC83\uBC84\uBC85\uBC86"+
	"\uBC87\uBC88\uBC89\uBC8A\uBC8B\uBC8C\uBC8D\uBC8E"+
	"\uCBD8\uBC8F\uCBF7\uBC90\uBC91\uBC92\uBC93\uBDF4"+
	"\uBC94\uBC95\uBC96\uD7CF\uBC97\uBC98\uBC99\uC0DB"+
	"\uBC9A\uBC9B\uBC9C\uBC9D\uBC9E\uBC9F\uBCA0\uBD40"+
	"\uBD41\uBD42\uBD43\uBD44\uBD45\uBD46\uBD47\uBD48"+
	"\uBD49\uBD4A\uBD4B\uBD4C\uBD4D\uBD4E\uBD4F\uBD50"+
	"\uBD51\uBD52\uBD53\uBD54\uBD55\uBD56\uBD57\uBD58"+
	"\uBD59\uBD5A\uBD5B\uBD5C\uBD5D\uBD5E\uBD5F\uBD60"+
	"\uBD61\uBD62\uBD63\uBD64\uBD65\uBD66\uBD67\uBD68"+
	"\uBD69\uBD6A\uBD6B\uBD6C\uBD6D\uBD6E\uBD6F\uBD70"+
	"\uBD71\uBD72\uBD73\uBD74\uBD75\uBD76\uD0F5\uBD77"+
	"\uBD78\uBD79\uBD7A\uBD7B\uBD7C\uBD7D\uBD7E\uF4EA"+
	"\uBD80\uBD81\uBD82\uBD83\uBD84\uBD85\uBD86\uBD87"+
	"\uBD88\uBD89\uBD8A\uBD8B\uBD8C\uBD8D\uBD8E\uBD8F"+
	"\uBD90\uBD91\uBD92\uBD93\uBD94\uBD95\uBD96\uBD97"+
	"\uBD98\uBD99\uBD9A\uBD9B\uBD9C\uBD9D\uBD9E\uBD9F"+
	"\uBDA0\uBE40\uBE41\uBE42\uBE43\uBE44\uBE45\uBE46"+
	"\uBE47\uBE48\uBE49\uBE4A\uBE4B\uBE4C\uF4EB\uBE4D"+
	"\uBE4E\uBE4F\uBE50\uBE51\uBE52\uBE53\uF4EC\uBE54"+
	"\uBE55\uBE56\uBE57\uBE58\uBE59\uBE5A\uBE5B\uBE5C"+
	"\uBE5D\uBE5E\uBE5F\uBE60\uBE61\uBE62\uBE63\uBE64"+
	"\uBE65\uBE66\uBE67\uBE68\uBE69\uBE6A\uBE6B\uBE6C"+
	"\uBE6D\uBE6E\uBE6F\uBE70\uBE71\uBE72\uBE73\uBE74"+
	"\uBE75\uBE76\uBE77\uBE78\uBE79\uBE7A\uBE7B\uBE7C"+
	"\uBE7D\uBE7E\uBE80\uBE81\uBE82\uBE83\uBE84\uBE85"+
	"\uBE86\uBE87\uBE88\uBE89\uBE8A\uBE8B\uBE8C\uBE8D"+
	"\uBE8E\uBE8F\uBE90\uBE91\uBE92\uBE93\uBE94\uBE95"+
	"\uBE96\uBE97\uBE98\uBE99\uBE9A\uBE9B\uBE9C\uBE9D"+
	"\uBE9E\uBE9F\uBEA0\uBF40\uBF41\uBF42\uBF43\uBF44"+
	"\uBF45\uBF46\uBF47\uBF48\uBF49\uBF4A\uBF4B\uBF4C"+
	"\uBF4D\uBF4E\uBF4F\uBF50\uBF51\uBF52\uBF53\uBF54"+
	"\uBF55\uBF56\uBF57\uBF58\uBF59\uBF5A\uBF5B\uBF5C"+
	"\uBF5D\uBF5E\uBF5F\uBF60\uBF61\uBF62\uBF63\uBF64"+
	"\uBF65\uBF66\uBF67\uBF68\uBF69\uBF6A\uBF6B\uBF6C"+
	"\uBF6D\uBF6E\uBF6F\uBF70\uBF71\uBF72\uBF73\uBF74"+
	"\uBF75\uBF76\uBF77\uBF78\uBF79\uBF7A\uBF7B\uBF7C"+
	"\uBF7D\uBF7E\uBF80\uF7E3\uBF81\uBF82\uBF83\uBF84"+
	"\uBF85\uB7B1\uBF86\uBF87\uBF88\uBF89\uBF8A\uF4ED"+
	"\uBF8B\uBF8C\uBF8D\uBF8E\uBF8F\uBF90\uBF91\uBF92"+
	"\uBF93\uBF94\uBF95\uBF96\uBF97\uBF98\uBF99\uBF9A"+
	"\uBF9B\uBF9C\uBF9D\uBF9E\uBF9F\uBFA0\uC040\uC041"+
	"\uC042\uC043\uC044\uC045\uC046\uC047\uC048\uC049"+
	"\uC04A\uC04B\uC04C\uC04D\uC04E\uC04F\uC050\uC051"+
	"\uC052\uC053\uC054\uC055\uC056\uC057\uC058\uC059"+
	"\uC05A\uC05B\uC05C\uC05D\uC05E\uC05F\uC060\uC061"+
	"\uC062\uC063\uD7EB\uC064\uC065\uC066\uC067\uC068"+
	"\uC069\uC06A\uC06B\uC06C\uC06D\uC06E\uC06F\uC070"+
	"\uC071\uC072\uC073\uC074\uC075\uC076\uC077\uC078"+
	"\uC079\uC07A\uC07B\uF4EE\uC07C\uC07D\uC07E\uE6F9"+
	"\uBEC0\uE6FA\uBAEC\uE6FB\uCFCB\uE6FC\uD4BC\uBCB6"+
	"\uE6FD\uE6FE\uBCCD\uC8D2\uCEB3\uE7A1\uC080\uB4BF"+
	"\uE7A2\uC9B4\uB8D9\uC4C9\uC081\uD7DD\uC2DA\uB7D7"+
	"\uD6BD\uCEC6\uB7C4\uC082\uC083\uC5A6\uE7A3\uCFDF"+
	"\uE7A4\uE7A5\uE7A6\uC1B7\uD7E9\uC9F0\uCFB8\uD6AF"+
	"\uD6D5\uE7A7\uB0ED\uE7A8\uE7A9\uC9DC\uD2EF\uBEAD"+
	"\uE7AA\uB0F3\uC8DE\uBDE1\uE7AB\uC8C6\uC084\uE7AC"+
	"\uBBE6\uB8F8\uD1A4\uE7AD\uC2E7\uBEF8\uBDCA\uCDB3"+
	"\uE7AE\uE7AF\uBEEE\uD0E5\uC085\uCBE7\uCCD0\uBCCC"+
	"\uE7B0\uBCA8\uD0F7\uE7B1\uC086\uD0F8\uE7B2\uE7B3"+
	"\uB4C2\uE7B4\uE7B5\uC9FE\uCEAC\uC3E0\uE7B7\uB1C1"+
	"\uB3F1\uC087\uE7B8\uE7B9\uD7DB\uD5C0\uE7BA\uC2CC"+
	"\uD7BA\uE7BB\uE7BC\uE7BD\uBCEA\uC3E5\uC0C2\uE7BE"+
	"\uE7BF\uBCA9\uC088\uE7C0\uE7C1\uE7B6\uB6D0\uE7C2"+
	"\uC089\uE7C3\uE7C4\uBBBA\uB5DE\uC2C6\uB1E0\uE7C5"+
	"\uD4B5\uE7C6\uB8BF\uE7C8\uE7C7\uB7EC\uC08A\uE7C9"+
	"\uB2F8\uE7CA\uE7CB\uE7CC\uE7CD\uE7CE\uE7CF\uE7D0"+
	"\uD3A7\uCBF5\uE7D1\uE7D2\uE7D3\uE7D4\uC9C9\uE7D5"+
	"\uE7D6\uE7D7\uE7D8\uE7D9\uBDC9\uE7DA\uF3BE\uC08B"+
	"\uB8D7\uC08C\uC8B1\uC08D\uC08E\uC08F\uC090\uC091"+
	"\uC092\uC093\uF3BF\uC094\uF3C0\uF3C1\uC095\uC096"+
	"\uC097\uC098\uC099\uC09A\uC09B\uC09C\uC09D\uC09E"+
	"\uB9DE\uCDF8\uC09F\uC0A0\uD8E8\uBAB1\uC140\uC2DE"+
	"\uEEB7\uC141\uB7A3\uC142\uC143\uC144\uC145\uEEB9"+
	"\uC146\uEEB8\uB0D5\uC147\uC148\uC149\uC14A\uC14B"+
	"\uEEBB\uD5D6\uD7EF\uC14C\uC14D\uC14E\uD6C3\uC14F"+
	"\uC150\uEEBD\uCAF0\uC151\uEEBC\uC152\uC153\uC154"+
	"\uC155\uEEBE\uC156\uC157\uC158\uC159\uEEC0\uC15A"+
	"\uC15B\uEEBF\uC15C\uC15D\uC15E\uC15F\uC160\uC161"+
	"\uC162\uC163\uD1F2\uC164\uC7BC\uC165\uC3C0\uC166"+
	"\uC167\uC168\uC169\uC16A\uB8E1\uC16B\uC16C\uC16D"+
	"\uC16E\uC16F\uC1E7\uC170\uC171\uF4C6\uD0DF\uF4C7"+
	"\uC172\uCFDB\uC173\uC174\uC8BA\uC175\uC176\uF4C8"+
	"\uC177\uC178\uC179\uC17A\uC17B\uC17C\uC17D\uF4C9"+
	"\uF4CA\uC17E\uF4CB\uC180\uC181\uC182\uC183\uC184"+
	"\uD9FA\uB8FE\uC185\uC186\uE5F1\uD3F0\uC187\uF4E0"+
	"\uC188\uCECC\uC189\uC18A\uC18B\uB3E1\uC18C\uC18D"+
	"\uC18E\uC18F\uF1B4\uC190\uD2EE\uC191\uF4E1\uC192"+
	"\uC193\uC194\uC195\uC196\uCFE8\uF4E2\uC197\uC198"+
	"\uC7CC\uC199\uC19A\uC19B\uC19C\uC19D\uC19E\uB5D4"+
	"\uB4E4\uF4E4\uC19F\uC1A0\uC240\uF4E3\uF4E5\uC241"+
	"\uC242\uF4E6\uC243\uC244\uC245\uC246\uF4E7\uC247"+
	"\uBAB2\uB0BF\uC248\uF4E8\uC249\uC24A\uC24B\uC24C"+
	"\uC24D\uC24E\uC24F\uB7AD\uD2ED\uC250\uC251\uC252";

    private final static String innerIndex8= 
	"\uD2AB\uC0CF\uC253\uBFBC\uEBA3\uD5DF\uEAC8\uC254"+
	"\uC255\uC256\uC257\uF1F3\uB6F8\uCBA3\uC258\uC259"+
	"\uC4CD\uC25A\uF1E7\uC25B\uF1E8\uB8FB\uF1E9\uBAC4"+
	"\uD4C5\uB0D2\uC25C\uC25D\uF1EA\uC25E\uC25F\uC260"+
	"\uF1EB\uC261\uF1EC\uC262\uC263\uF1ED\uF1EE\uF1EF"+
	"\uF1F1\uF1F0\uC5D5\uC264\uC265\uC266\uC267\uC268"+
	"\uC269\uF1F2\uC26A\uB6FA\uC26B\uF1F4\uD2AE\uDEC7"+
	"\uCBCA\uC26C\uC26D\uB3DC\uC26E\uB5A2\uC26F\uB9A2"+
	"\uC270\uC271\uC4F4\uF1F5\uC272\uC273\uF1F6\uC274"+
	"\uC275\uC276\uC1C4\uC1FB\uD6B0\uF1F7\uC277\uC278"+
	"\uC279\uC27A\uF1F8\uC27B\uC1AA\uC27C\uC27D\uC27E"+
	"\uC6B8\uC280\uBEDB\uC281\uC282\uC283\uC284\uC285"+
	"\uC286\uC287\uC288\uC289\uC28A\uC28B\uC28C\uC28D"+
	"\uC28E\uF1F9\uB4CF\uC28F\uC290\uC291\uC292\uC293"+
	"\uC294\uF1FA\uC295\uC296\uC297\uC298\uC299\uC29A"+
	"\uC29B\uC29C\uC29D\uC29E\uC29F\uC2A0\uC340\uEDB2"+
	"\uEDB1\uC341\uC342\uCBE0\uD2DE\uC343\uCBC1\uD5D8"+
	"\uC344\uC8E2\uC345\uC0DF\uBCA1\uC346\uC347\uC348"+
	"\uC349\uC34A\uC34B\uEBC1\uC34C\uC34D\uD0A4\uC34E"+
	"\uD6E2\uC34F\uB6C7\uB8D8\uEBC0\uB8CE\uC350\uEBBF"+
	"\uB3A6\uB9C9\uD6AB\uC351\uB7F4\uB7CA\uC352\uC353"+
	"\uC354\uBCE7\uB7BE\uEBC6\uC355\uEBC7\uB0B9\uBFCF"+
	"\uC356\uEBC5\uD3FD\uC357\uEBC8\uC358\uC359\uEBC9"+
	"\uC35A\uC35B\uB7CE\uC35C\uEBC2\uEBC4\uC9F6\uD6D7"+
	"\uD5CD\uD0B2\uEBCF\uCEB8\uEBD0\uC35D\uB5A8\uC35E"+
	"\uC35F\uC360\uC361\uC362\uB1B3\uEBD2\uCCA5\uC363"+
	"\uC364\uC365\uC366\uC367\uC368\uC369\uC5D6\uEBD3"+
	"\uC36A\uEBD1\uC5DF\uEBCE\uCAA4\uEBD5\uB0FB\uC36B"+
	"\uC36C\uBAFA\uC36D\uC36E\uD8B7\uF1E3\uC36F\uEBCA"+
	"\uEBCB\uEBCC\uEBCD\uEBD6\uE6C0\uEBD9\uC370\uBFE8"+
	"\uD2C8\uEBD7\uEBDC\uB8EC\uEBD8\uC371\uBDBA\uC372"+
	"\uD0D8\uC373\uB0B7\uC374\uEBDD\uC4DC\uC375\uC376"+
	"\uC377\uC378\uD6AC\uC379\uC37A\uC37B\uB4E0\uC37C"+
	"\uC37D\uC2F6\uBCB9\uC37E\uC380\uEBDA\uEBDB\uD4E0"+
	"\uC6EA\uC4D4\uEBDF\uC5A7\uD9F5\uC381\uB2B1\uC382"+
	"\uEBE4\uC383\uBDC5\uC384\uC385\uC386\uEBE2\uC387"+
	"\uC388\uC389\uC38A\uC38B\uC38C\uC38D\uC38E\uC38F"+
	"\uC390\uC391\uC392\uC393\uEBE3\uC394\uC395\uB8AC"+
	"\uC396\uCDD1\uEBE5\uC397\uC398\uC399\uEBE1\uC39A"+
	"\uC1B3\uC39B\uC39C\uC39D\uC39E\uC39F\uC6A2\uC3A0"+
	"\uC440\uC441\uC442\uC443\uC444\uC445\uCCF3\uC446"+
	"\uEBE6\uC447\uC0B0\uD2B8\uEBE7\uC448\uC449\uC44A"+
	"\uB8AF\uB8AD\uC44B\uEBE8\uC7BB\uCDF3\uC44C\uC44D"+
	"\uC44E\uEBEA\uEBEB\uC44F\uC450\uC451\uC452\uC453"+
	"\uEBED\uC454\uC455\uC456\uC457\uD0C8\uC458\uEBF2"+
	"\uC459\uEBEE\uC45A\uC45B\uC45C\uEBF1\uC8F9\uC45D"+
	"\uD1FC\uEBEC\uC45E\uC45F\uEBE9\uC460\uC461\uC462"+
	"\uC463\uB8B9\uCFD9\uC4E5\uEBEF\uEBF0\uCCDA\uCDC8"+
	"\uB0F2\uC464\uEBF6\uC465\uC466\uC467\uC468\uC469"+
	"\uEBF5\uC46A\uB2B2\uC46B\uC46C\uC46D\uC46E\uB8E0"+
	"\uC46F\uEBF7\uC470\uC471\uC472\uC473\uC474\uC475"+
	"\uB1EC\uC476\uC477\uCCC5\uC4A4\uCFA5\uC478\uC479"+
	"\uC47A\uC47B\uC47C\uEBF9\uC47D\uC47E\uECA2\uC480"+
	"\uC5F2\uC481\uEBFA\uC482\uC483\uC484\uC485\uC486"+
	"\uC487\uC488\uC489\uC9C5\uC48A\uC48B\uC48C\uC48D"+
	"\uC48E\uC48F\uE2DF\uEBFE\uC490\uC491\uC492\uC493"+
	"\uCDCE\uECA1\uB1DB\uD3B7\uC494\uC495\uD2DC\uC496"+
	"\uC497\uC498\uEBFD\uC499\uEBFB\uC49A\uC49B\uC49C"+
	"\uC49D\uC49E\uC49F\uC4A0\uC540\uC541\uC542\uC543"+
	"\uC544\uC545\uC546\uC547\uC548\uC549\uC54A\uC54B"+
	"\uC54C\uC54D\uC54E\uB3BC\uC54F\uC550\uC551\uEAB0"+
	"\uC552\uC553\uD7D4\uC554\uF4AB\uB3F4\uC555\uC556"+
	"\uC557\uC558\uC559\uD6C1\uD6C2\uC55A\uC55B\uC55C"+
	"\uC55D\uC55E\uC55F\uD5E9\uBECA\uC560\uF4A7\uC561"+
	"\uD2A8\uF4A8\uF4A9\uC562\uF4AA\uBECB\uD3DF\uC563"+
	"\uC564\uC565\uC566\uC567\uC9E0\uC9E1\uC568\uC569"+
	"\uF3C2\uC56A\uCAE6\uC56B\uCCF2\uC56C\uC56D\uC56E"+
	"\uC56F\uC570\uC571\uE2B6\uCBB4\uC572\uCEE8\uD6DB"+
	"\uC573\uF4AD\uF4AE\uF4AF\uC574\uC575\uC576\uC577"+
	"\uF4B2\uC578\uBABD\uF4B3\uB0E3\uF4B0\uC579\uF4B1"+
	"\uBDA2\uB2D5\uC57A\uF4B6\uF4B7\uB6E6\uB2B0\uCFCF"+
	"\uF4B4\uB4AC\uC57B\uF4B5\uC57C\uC57D\uF4B8\uC57E"+
	"\uC580\uC581\uC582\uC583\uF4B9\uC584\uC585\uCDA7"+
	"\uC586\uF4BA\uC587\uF4BB\uC588\uC589\uC58A\uF4BC"+
	"\uC58B\uC58C\uC58D\uC58E\uC58F\uC590\uC591\uC592"+
	"\uCBD2\uC593\uF4BD\uC594\uC595\uC596\uC597\uF4BE"+
	"\uC598\uC599\uC59A\uC59B\uC59C\uC59D\uC59E\uC59F"+
	"\uF4BF\uC5A0\uC640\uC641\uC642\uC643\uF4DE\uC1BC"+
	"\uBCE8\uC644\uC9AB\uD1DE\uE5F5\uC645\uC646\uC647"+
	"\uC648\uDCB3\uD2D5\uC649\uC64A\uDCB4\uB0AC\uDCB5"+
	"\uC64B\uC64C\uBDDA\uC64D\uDCB9\uC64E\uC64F\uC650"+
	"\uD8C2\uC651\uDCB7\uD3F3\uC652\uC9D6\uDCBA\uDCB6"+
	"\uC653\uDCBB\uC3A2\uC654\uC655\uC656\uC657\uDCBC"+
	"\uDCC5\uDCBD\uC658\uC659\uCEDF\uD6A5\uC65A\uDCCF"+
	"\uC65B\uDCCD\uC65C\uC65D\uDCD2\uBDE6\uC2AB\uC65E"+
	"\uDCB8\uDCCB\uDCCE\uDCBE\uB7D2\uB0C5\uDCC7\uD0BE"+
	"\uDCC1\uBBA8\uC65F\uB7BC\uDCCC\uC660\uC661\uDCC6"+
	"\uDCBF\uC7DB\uC662\uC663\uC664\uD1BF\uDCC0\uC665"+
	"\uC666\uDCCA\uC667\uC668\uDCD0\uC669\uC66A\uCEAD"+
	"\uDCC2\uC66B\uDCC3\uDCC8\uDCC9\uB2D4\uDCD1\uCBD5"+
	"\uC66C\uD4B7\uDCDB\uDCDF\uCCA6\uDCE6\uC66D\uC3E7"+
	"\uDCDC\uC66E\uC66F\uBFC1\uDCD9\uC670\uB0FA\uB9B6"+
	"\uDCE5\uDCD3\uC671\uDCC4\uDCD6\uC8F4\uBFE0\uC672"+
	"\uC673\uC674\uC675\uC9BB\uC676\uC677\uC678\uB1BD"+
	"\uC679\uD3A2\uC67A\uC67B\uDCDA\uC67C\uC67D\uDCD5"+
	"\uC67E\uC6BB\uC680\uDCDE\uC681\uC682\uC683\uC684"+
	"\uC685\uD7C2\uC3AF\uB7B6\uC7D1\uC3A9\uDCE2\uDCD8"+
	"\uDCEB\uDCD4\uC686\uC687\uDCDD\uC688\uBEA5\uDCD7"+
	"\uC689\uDCE0\uC68A\uC68B\uDCE3\uDCE4\uC68C\uDCF8"+
	"\uC68D\uC68E\uDCE1\uDDA2\uDCE7\uC68F\uC690\uC691"+
	"\uC692\uC693\uC694\uC695\uC696\uC697\uC698\uBCEB"+
	"\uB4C4\uC699\uC69A\uC3A3\uB2E7\uDCFA\uC69B\uDCF2"+
	"\uC69C\uDCEF\uC69D\uDCFC\uDCEE\uD2F0\uB2E8\uC69E"+
	"\uC8D7\uC8E3\uDCFB\uC69F\uDCED\uC6A0\uC740\uC741"+
	"\uDCF7\uC742\uC743\uDCF5\uC744\uC745\uBEA3\uDCF4"+
	"\uC746\uB2DD\uC747\uC748\uC749\uC74A\uC74B\uDCF3"+
	"\uBCF6\uDCE8\uBBC4\uC74C\uC0F3\uC74D\uC74E\uC74F"+
	"\uC750\uC751\uBCD4\uDCE9\uDCEA\uC752\uDCF1\uDCF6"+
	"\uDCF9\uB5B4\uC753\uC8D9\uBBE7\uDCFE\uDCFD\uD3AB"+
	"\uDDA1\uDDA3\uDDA5\uD2F1\uDDA4\uDDA6\uDDA7\uD2A9"+
	"\uC754\uC755\uC756\uC757\uC758\uC759\uC75A\uBAC9"+
	"\uDDA9\uC75B\uC75C\uDDB6\uDDB1\uDDB4\uC75D\uC75E"+
	"\uC75F\uC760\uC761\uC762\uC763\uDDB0\uC6CE\uC764"+
	"\uC765\uC0F2\uC766\uC767\uC768\uC769\uC9AF\uC76A"+
	"\uC76B\uC76C\uDCEC\uDDAE\uC76D\uC76E\uC76F\uC770"+
	"\uDDB7\uC771\uC772\uDCF0\uDDAF\uC773\uDDB8\uC774"+
	"\uDDAC\uC775\uC776\uC777\uC778\uC779\uC77A\uC77B"+
	"\uDDB9\uDDB3\uDDAD\uC4AA\uC77C\uC77D\uC77E\uC780"+
	"\uDDA8\uC0B3\uC1AB\uDDAA\uDDAB\uC781\uDDB2\uBBF1"+
	"\uDDB5\uD3A8\uDDBA\uC782\uDDBB\uC3A7\uC783\uC784"+
	"\uDDD2\uDDBC\uC785\uC786\uC787\uDDD1\uC788\uB9BD"+
	"\uC789\uC78A\uBED5\uC78B\uBEFA\uC78C\uC78D\uBACA"+
	"\uC78E\uC78F\uC790\uC791\uDDCA\uC792\uDDC5\uC793"+
	"\uDDBF\uC794\uC795\uC796\uB2CB\uDDC3\uC797\uDDCB"+
	"\uB2A4\uDDD5\uC798\uC799\uC79A\uDDBE\uC79B\uC79C"+
	"\uC79D\uC6D0\uDDD0\uC79E\uC79F\uC7A0\uC840\uC841"+
	"\uDDD4\uC1E2\uB7C6\uC842\uC843\uC844\uC845\uC846"+
	"\uDDCE\uDDCF\uC847\uC848\uC849\uDDC4\uC84A\uC84B"+
	"\uC84C\uDDBD\uC84D\uDDCD\uCCD1\uC84E\uDDC9\uC84F"+
	"\uC850\uC851\uC852\uDDC2\uC3C8\uC6BC\uCEAE\uDDCC"+
	"\uC853\uDDC8\uC854\uC855\uC856\uC857\uC858\uC859"+
	"\uDDC1\uC85A\uC85B\uC85C\uDDC6\uC2DC\uC85D\uC85E"+
	"\uC85F\uC860\uC861\uC862\uD3A9\uD3AA\uDDD3\uCFF4"+
	"\uC8F8\uC863\uC864\uC865\uC866\uC867\uC868\uC869"+
	"\uC86A\uDDE6\uC86B\uC86C\uC86D\uC86E\uC86F\uC870"+
	"\uDDC7\uC871\uC872\uC873\uDDE0\uC2E4\uC874\uC875"+
	"\uC876\uC877\uC878\uC879\uC87A\uC87B\uDDE1\uC87C"+
	"\uC87D\uC87E\uC880\uC881\uC882\uC883\uC884\uC885"+
	"\uC886\uDDD7\uC887\uC888\uC889\uC88A\uC88B\uD6F8"+
	"\uC88C\uDDD9\uDDD8\uB8F0\uDDD6\uC88D\uC88E\uC88F"+
	"\uC890\uC6CF\uC891\uB6AD\uC892\uC893\uC894\uC895"+
	"\uC896\uDDE2\uC897\uBAF9\uD4E1\uDDE7\uC898\uC899"+
	"\uC89A\uB4D0\uC89B\uDDDA\uC89C\uBFFB\uDDE3\uC89D"+
	"\uDDDF\uC89E\uDDDD\uC89F\uC8A0\uC940\uC941\uC942"+
	"\uC943\uC944\uB5D9\uC945\uC946\uC947\uC948\uDDDB"+
	"\uDDDC\uDDDE\uC949\uBDAF\uDDE4\uC94A\uDDE5\uC94B"+
	"\uC94C\uC94D\uC94E\uC94F\uC950\uC951\uC952\uDDF5"+
	"\uC953\uC3C9\uC954\uC955\uCBE2\uC956\uC957\uC958"+
	"\uC959\uDDF2\uC95A\uC95B\uC95C\uC95D\uC95E\uC95F"+
	"\uC960\uC961\uC962\uC963\uC964\uC965\uC966\uD8E1"+
	"\uC967\uC968\uC6D1\uC969\uDDF4\uC96A\uC96B\uC96C"+
	"\uD5F4\uDDF3\uDDF0\uC96D\uC96E\uDDEC\uC96F\uDDEF"+
	"\uC970\uDDE8\uC971\uC972\uD0EE\uC973\uC974\uC975"+
	"\uC976\uC8D8\uDDEE\uC977\uC978\uDDE9\uC979\uC97A"+
	"\uDDEA\uCBF2\uC97B\uDDED\uC97C\uC97D\uB1CD\uC97E"+
	"\uC980\uC981\uC982\uC983\uC984\uC0B6\uC985\uBCBB"+
	"\uDDF1\uC986\uC987\uDDF7\uC988\uDDF6\uDDEB\uC989"+
	"\uC98A\uC98B\uC98C\uC98D\uC5EE\uC98E\uC98F\uC990"+
	"\uDDFB\uC991\uC992\uC993\uC994\uC995\uC996\uC997"+
	"\uC998\uC999\uC99A\uC99B\uDEA4\uC99C\uC99D\uDEA3"+
	"\uC99E\uC99F\uC9A0\uCA40\uCA41\uCA42\uCA43\uCA44"+
	"\uCA45\uCA46\uCA47\uCA48\uDDF8\uCA49\uCA4A\uCA4B"+
	"\uCA4C\uC3EF\uCA4D\uC2FB\uCA4E\uCA4F\uCA50\uD5E1"+
	"\uCA51\uCA52\uCEB5\uCA53\uCA54\uCA55\uCA56\uDDFD"+
	"\uCA57\uB2CC\uCA58\uCA59\uCA5A\uCA5B\uCA5C\uCA5D"+
	"\uCA5E\uCA5F\uCA60\uC4E8\uCADF\uCA61\uCA62\uCA63"+
	"\uCA64\uCA65\uCA66\uCA67\uCA68\uCA69\uCA6A\uC7BE"+
	"\uDDFA\uDDFC\uDDFE\uDEA2\uB0AA\uB1CE\uCA6B\uCA6C"+
	"\uCA6D\uCA6E\uCA6F\uDEAC\uCA70\uCA71\uCA72\uCA73"+
	"\uDEA6\uBDB6\uC8EF\uCA74\uCA75\uCA76\uCA77\uCA78"+
	"\uCA79\uCA7A\uCA7B\uCA7C\uCA7D\uCA7E\uDEA1\uCA80"+
	"\uCA81\uDEA5\uCA82\uCA83\uCA84\uCA85\uDEA9\uCA86"+
	"\uCA87\uCA88\uCA89\uCA8A\uDEA8\uCA8B\uCA8C\uCA8D"+
	"\uDEA7\uCA8E\uCA8F\uCA90\uCA91\uCA92\uCA93\uCA94"+
	"\uCA95\uCA96\uDEAD\uCA97\uD4CC\uCA98\uCA99\uCA9A"+
	"\uCA9B\uDEB3\uDEAA\uDEAE\uCA9C\uCA9D\uC0D9\uCA9E"+
	"\uCA9F\uCAA0\uCB40\uCB41\uB1A1\uDEB6\uCB42\uDEB1"+
	"\uCB43\uCB44\uCB45\uCB46\uCB47\uCB48\uCB49\uDEB2"+
	"\uCB4A\uCB4B\uCB4C\uCB4D\uCB4E\uCB4F\uCB50\uCB51"+
	"\uCB52\uCB53\uCB54\uD1A6\uDEB5\uCB55\uCB56\uCB57"+
	"\uCB58\uCB59\uCB5A\uCB5B\uDEAF\uCB5C\uCB5D\uCB5E"+
	"\uDEB0\uCB5F\uD0BD\uCB60\uCB61\uCB62\uDEB4\uCAED"+
	"\uDEB9\uCB63\uCB64\uCB65\uCB66\uCB67\uCB68\uDEB8"+
	"\uCB69\uDEB7\uCB6A\uCB6B\uCB6C\uCB6D\uCB6E\uCB6F"+
	"\uCB70\uDEBB\uCB71\uCB72\uCB73\uCB74\uCB75\uCB76"+
	"\uCB77\uBDE5\uCB78\uCB79\uCB7A\uCB7B\uCB7C\uB2D8"+
	"\uC3EA\uCB7D\uCB7E\uDEBA\uCB80\uC5BA\uCB81\uCB82"+
	"\uCB83\uCB84\uCB85\uCB86\uDEBC\uCB87\uCB88\uCB89"+
	"\uCB8A\uCB8B\uCB8C\uCB8D\uCCD9\uCB8E\uCB8F\uCB90"+
	"\uCB91\uB7AA\uCB92\uCB93\uCB94\uCB95\uCB96\uCB97"+
	"\uCB98\uCB99\uCB9A\uCB9B\uCB9C\uCB9D\uCB9E\uCB9F"+
	"\uCBA0\uCC40\uCC41\uD4E5\uCC42\uCC43\uCC44\uDEBD"+
	"\uCC45\uCC46\uCC47\uCC48\uCC49\uDEBF\uCC4A\uCC4B"+
	"\uCC4C\uCC4D\uCC4E\uCC4F\uCC50\uCC51\uCC52\uCC53"+
	"\uCC54\uC4A2\uCC55\uCC56\uCC57\uCC58\uDEC1\uCC59"+
	"\uCC5A\uCC5B\uCC5C\uCC5D\uCC5E\uCC5F\uCC60\uCC61"+
	"\uCC62\uCC63\uCC64\uCC65\uCC66\uCC67\uCC68\uDEBE"+
	"\uCC69\uDEC0\uCC6A\uCC6B\uCC6C\uCC6D\uCC6E\uCC6F"+
	"\uCC70\uCC71\uCC72\uCC73\uCC74\uCC75\uCC76\uCC77"+
	"\uD5BA\uCC78\uCC79\uCC7A\uDEC2\uCC7B\uCC7C\uCC7D"+
	"\uCC7E\uCC80\uCC81\uCC82\uCC83\uCC84\uCC85\uCC86"+
	"\uCC87\uCC88\uCC89\uCC8A\uCC8B\uF2AE\uBBA2\uC2B2"+
	"\uC5B0\uC2C7\uCC8C\uCC8D\uF2AF\uCC8E\uCC8F\uCC90"+
	"\uCC91\uCC92\uD0E9\uCC93\uCC94\uCC95\uD3DD\uCC96"+
	"\uCC97\uCC98\uEBBD\uCC99\uCC9A\uCC9B\uCC9C\uCC9D"+
	"\uCC9E\uCC9F\uCCA0\uB3E6\uF2B0\uCD40\uF2B1\uCD41"+
	"\uCD42\uCAAD\uCD43\uCD44\uCD45\uCD46\uCD47\uCD48"+
	"\uCD49\uBAE7\uF2B3\uF2B5\uF2B4\uCBE4\uCFBA\uF2B2"+
	"\uCAB4\uD2CF\uC2EC\uCD4A\uCD4B\uCD4C\uCD4D\uCD4E"+
	"\uCD4F\uCD50\uCEC3\uF2B8\uB0F6\uF2B7\uCD51\uCD52"+
	"\uCD53\uCD54\uCD55\uF2BE\uCD56\uB2CF\uCD57\uCD58"+
	"\uCD59\uCD5A\uCD5B\uCD5C\uD1C1\uF2BA\uCD5D\uCD5E"+
	"\uCD5F\uCD60\uCD61\uF2BC\uD4E9\uCD62\uCD63\uF2BB"+
	"\uF2B6\uF2BF\uF2BD\uCD64\uF2B9\uCD65\uCD66\uF2C7"+
	"\uF2C4\uF2C6\uCD67\uCD68\uF2CA\uF2C2\uF2C0\uCD69"+
	"\uCD6A\uCD6B\uF2C5\uCD6C\uCD6D\uCD6E\uCD6F\uCD70"+
	"\uD6FB\uCD71\uCD72\uCD73\uF2C1\uCD74\uC7F9\uC9DF"+
	"\uCD75\uF2C8\uB9C6\uB5B0\uCD76\uCD77\uF2C3\uF2C9"+
	"\uF2D0\uF2D6\uCD78\uCD79\uBBD7\uCD7A\uCD7B\uCD7C"+
	"\uF2D5\uCDDC\uCD7D\uD6EB\uCD7E\uCD80\uF2D2\uF2D4"+
	"\uCD81\uCD82\uCD83\uCD84\uB8F2\uCD85\uCD86\uCD87"+
	"\uCD88\uF2CB\uCD89\uCD8A\uCD8B\uF2CE\uC2F9\uCD8C"+
	"\uD5DD\uF2CC\uF2CD\uF2CF\uF2D3\uCD8D\uCD8E\uCD8F"+
	"\uF2D9\uD3BC\uCD90\uCD91\uCD92\uCD93\uB6EA\uCD94"+
	"\uCAF1\uCD95\uB7E4\uF2D7\uCD96\uCD97\uCD98\uF2D8"+
	"\uF2DA\uF2DD\uF2DB\uCD99\uCD9A\uF2DC\uCD9B\uCD9C"+
	"\uCD9D\uCD9E\uD1D1\uF2D1\uCD9F\uCDC9\uCDA0\uCECF"+
	"\uD6A9\uCE40\uF2E3\uCE41\uC3DB\uCE42\uF2E0\uCE43"+
	"\uCE44\uC0AF\uF2EC\uF2DE\uCE45\uF2E1\uCE46\uCE47"+
	"\uCE48\uF2E8\uCE49\uCE4A\uCE4B\uCE4C\uF2E2\uCE4D"+
	"\uCE4E\uF2E7\uCE4F\uCE50\uF2E6\uCE51\uCE52\uF2E9"+
	"\uCE53\uCE54\uCE55\uF2DF\uCE56\uCE57\uF2E4\uF2EA"+
	"\uCE58\uCE59\uCE5A\uCE5B\uCE5C\uCE5D\uCE5E\uD3AC"+
	"\uF2E5\uB2F5\uCE5F\uCE60\uF2F2\uCE61\uD0AB\uCE62"+
	"\uCE63\uCE64\uCE65\uF2F5\uCE66\uCE67\uCE68\uBBC8"+
	"\uCE69\uF2F9\uCE6A\uCE6B\uCE6C\uCE6D\uCE6E\uCE6F"+
	"\uF2F0\uCE70\uCE71\uF2F6\uF2F8\uF2FA\uCE72\uCE73"+
	"\uCE74\uCE75\uCE76\uCE77\uCE78\uCE79\uF2F3\uCE7A"+
	"\uF2F1\uCE7B\uCE7C\uCE7D\uBAFB\uCE7E\uB5FB\uCE80"+
	"\uCE81\uCE82\uCE83\uF2EF\uF2F7\uF2ED\uF2EE\uCE84"+
	"\uCE85\uCE86\uF2EB\uF3A6\uCE87\uF3A3\uCE88\uCE89"+
	"\uF3A2\uCE8A\uCE8B\uF2F4\uCE8C\uC8DA\uCE8D\uCE8E"+
	"\uCE8F\uCE90\uCE91\uF2FB\uCE92\uCE93\uCE94\uF3A5"+
	"\uCE95\uCE96\uCE97\uCE98\uCE99\uCE9A\uCE9B\uC3F8"+
	"\uCE9C\uCE9D\uCE9E\uCE9F\uCEA0\uCF40\uCF41\uCF42"+
	"\uF2FD\uCF43\uCF44\uF3A7\uF3A9\uF3A4\uCF45\uF2FC"+
	"\uCF46\uCF47\uCF48\uF3AB\uCF49\uF3AA\uCF4A\uCF4B"+
	"\uCF4C\uCF4D\uC2DD\uCF4E\uCF4F\uF3AE\uCF50\uCF51"+
	"\uF3B0\uCF52\uCF53\uCF54\uCF55\uCF56\uF3A1\uCF57"+
	"\uCF58\uCF59\uF3B1\uF3AC\uCF5A\uCF5B\uCF5C\uCF5D"+
	"\uCF5E\uF3AF\uF2FE\uF3AD\uCF5F\uCF60\uCF61\uCF62"+
	"\uCF63\uCF64\uCF65\uF3B2\uCF66\uCF67\uCF68\uCF69"+
	"\uF3B4\uCF6A\uCF6B\uCF6C\uCF6D\uF3A8\uCF6E\uCF6F"+
	"\uCF70\uCF71\uF3B3\uCF72\uCF73\uCF74\uF3B5\uCF75"+
	"\uCF76\uCF77\uCF78\uCF79\uCF7A\uCF7B\uCF7C\uCF7D"+
	"\uCF7E\uD0B7\uCF80\uCF81\uCF82\uCF83\uF3B8\uCF84"+
	"\uCF85\uCF86\uCF87\uD9F9\uCF88\uCF89\uCF8A\uCF8B"+
	"\uCF8C\uCF8D\uF3B9\uCF8E\uCF8F\uCF90\uCF91\uCF92"+
	"\uCF93\uCF94\uCF95\uF3B7\uCF96\uC8E4\uF3B6\uCF97"+
	"\uCF98\uCF99\uCF9A\uF3BA\uCF9B\uCF9C\uCF9D\uCF9E"+
	"\uCF9F\uF3BB\uB4C0\uCFA0\uD040\uD041\uD042\uD043"+
	"\uD044\uD045\uD046\uD047\uD048\uD049\uD04A\uD04B"+
	"\uD04C\uD04D\uEEC3\uD04E\uD04F\uD050\uD051\uD052"+
	"\uD053\uF3BC\uD054\uD055\uF3BD\uD056\uD057\uD058"+
	"\uD1AA\uD059\uD05A\uD05B\uF4AC\uD0C6\uD05C\uD05D"+
	"\uD05E\uD05F\uD060\uD061\uD0D0\uD1DC\uD062\uD063"+
	"\uD064\uD065\uD066\uD067\uCFCE\uD068\uD069\uBDD6"+
	"\uD06A\uD1C3\uD06B\uD06C\uD06D\uD06E\uD06F\uD070"+
	"\uD071\uBAE2\uE1E9\uD2C2\uF1C2\uB2B9\uD072\uD073"+
	"\uB1ED\uF1C3\uD074\uC9C0\uB3C4\uD075\uD9F2\uD076"+
	"\uCBA5\uD077\uF1C4\uD078\uD079\uD07A\uD07B\uD6D4"+
	"\uD07C\uD07D\uD07E\uD080\uD081\uF1C5\uF4C0\uF1C6"+
	"\uD082\uD4AC\uF1C7\uD083\uB0C0\uF4C1\uD084\uD085"+
	"\uF4C2\uD086\uD087\uB4FC\uD088\uC5DB\uD089\uD08A"+
	"\uD08B\uD08C\uCCBB\uD08D\uD08E\uD08F\uD0E4\uD090"+
	"\uD091\uD092\uD093\uD094\uCDE0\uD095\uD096\uD097"+
	"\uD098\uD099\uF1C8\uD09A\uD9F3\uD09B\uD09C\uD09D"+
	"\uD09E\uD09F\uD0A0\uB1BB\uD140\uCFAE\uD141\uD142"+
	"\uD143\uB8A4\uD144\uD145\uD146\uD147\uD148\uF1CA"+
	"\uD149\uD14A\uD14B\uD14C\uF1CB\uD14D\uD14E\uD14F"+
	"\uD150\uB2C3\uC1D1\uD151\uD152\uD7B0\uF1C9\uD153"+
	"\uD154\uF1CC\uD155\uD156\uD157\uD158\uF1CE\uD159"+
	"\uD15A\uD15B\uD9F6\uD15C\uD2E1\uD4A3\uD15D\uD15E"+
	"\uF4C3\uC8B9\uD15F\uD160\uD161\uD162\uD163\uF4C4"+
	"\uD164\uD165\uF1CD\uF1CF\uBFE3\uF1D0\uD166\uD167"+
	"\uF1D4\uD168\uD169\uD16A\uD16B\uD16C\uD16D\uD16E"+
	"\uF1D6\uF1D1\uD16F\uC9D1\uC5E1\uD170\uD171\uD172"+
	"\uC2E3\uB9FC\uD173\uD174\uF1D3\uD175\uF1D5\uD176"+
	"\uD177\uD178\uB9D3\uD179\uD17A\uD17B\uD17C\uD17D"+
	"\uD17E\uD180\uF1DB\uD181\uD182\uD183\uD184\uD185"+
	"\uBAD6\uD186\uB0FD\uF1D9\uD187\uD188\uD189\uD18A"+
	"\uD18B\uF1D8\uF1D2\uF1DA\uD18C\uD18D\uD18E\uD18F"+
	"\uD190\uF1D7\uD191\uD192\uD193\uC8EC\uD194\uD195"+
	"\uD196\uD197\uCDCA\uF1DD\uD198\uD199\uD19A\uD19B"+
	"\uE5BD\uD19C\uD19D\uD19E\uF1DC\uD19F\uF1DE\uD1A0"+
	"\uD240\uD241\uD242\uD243\uD244\uD245\uD246\uD247"+
	"\uD248\uF1DF\uD249\uD24A\uCFE5\uD24B\uD24C\uD24D"+
	"\uD24E\uD24F\uD250\uD251\uD252\uD253\uD254\uD255"+
	"\uD256\uD257\uD258\uD259\uD25A\uD25B\uD25C\uD25D"+
	"\uD25E\uD25F\uD260\uD261\uD262\uD263\uF4C5\uBDF3"+
	"\uD264\uD265\uD266\uD267\uD268\uD269\uF1E0\uD26A"+
	"\uD26B\uD26C\uD26D\uD26E\uD26F\uD270\uD271\uD272"+
	"\uD273\uD274\uD275\uD276\uD277\uD278\uD279\uD27A"+
	"\uD27B\uD27C\uD27D\uF1E1\uD27E\uD280\uD281\uCEF7"+
	"\uD282\uD2AA\uD283\uF1FB\uD284\uD285\uB8B2\uD286"+
	"\uD287\uD288\uD289\uD28A\uD28B\uD28C\uD28D\uD28E"+
	"\uD28F\uD290\uD291\uD292\uD293\uD294\uD295\uD296"+
	"\uD297\uD298\uD299\uD29A\uD29B\uD29C\uD29D\uD29E"+
	"\uD29F\uD2A0\uD340\uD341\uD342\uD343\uD344\uD345"+
	"\uD346\uD347\uD348\uD349\uD34A\uD34B\uD34C\uD34D"+
	"\uD34E\uD34F\uD350\uD351\uD352\uD353\uD354\uD355"+
	"\uD356\uD357\uD358\uD359\uD35A\uD35B\uD35C\uD35D"+
	"\uD35E\uBCFB\uB9DB\uD35F\uB9E6\uC3D9\uCAD3\uEAE8"+
	"\uC0C0\uBEF5\uEAE9\uEAEA\uEAEB\uD360\uEAEC\uEAED"+
	"\uEAEE\uEAEF\uBDC7\uD361\uD362\uD363\uF5FB\uD364"+
	"\uD365\uD366\uF5FD\uD367\uF5FE\uD368\uF5FC\uD369"+
	"\uD36A\uD36B\uD36C\uBDE2\uD36D\uF6A1\uB4A5\uD36E"+
	"\uD36F\uD370\uD371\uF6A2\uD372\uD373\uD374\uF6A3"+
	"\uD375\uD376\uD377\uECB2\uD378\uD379\uD37A\uD37B"+
	"\uD37C\uD37D\uD37E\uD380\uD381\uD382\uD383\uD384"+
	"\uD1D4\uD385\uD386\uD387\uD388\uD389\uD38A\uD9EA"+
	"\uD38B\uD38C\uD38D\uD38E\uD38F\uD390\uD391\uD392"+
	"\uD393\uD394\uD395\uD396\uD397\uD398\uD399\uD39A"+
	"\uD39B\uD39C\uD39D\uD39E\uD39F\uD3A0\uD440\uD441"+
	"\uD442\uD443\uD444\uD445\uD446\uD447\uD448\uD449"+
	"\uD44A\uD44B\uD44C\uD44D\uD44E\uD44F\uD450\uD451"+
	"\uD452\uD453\uD454\uD455\uD456\uD457\uD458\uD459"+
	"\uD45A\uD45B\uD45C\uD45D\uD45E\uD45F\uF6A4\uD460"+
	"\uD461\uD462\uD463\uD464\uD465\uD466\uD467\uD468"+
	"\uEEBA\uD469\uD46A\uD46B\uD46C\uD46D\uD46E\uD46F"+
	"\uD470\uD471\uD472\uD473\uD474\uD475\uD476\uD477"+
	"\uD478\uD479\uD47A\uD47B\uD47C\uD47D\uD47E\uD480"+
	"\uD481\uD482\uD483\uD484\uD485\uD486\uD487\uD488"+
	"\uD489\uD48A\uD48B\uD48C\uD48D\uD48E\uD48F\uD490"+
	"\uD491\uD492\uD493\uD494\uD495\uD496\uD497\uD498"+
	"\uD499\uD5B2\uD49A\uD49B\uD49C\uD49D\uD49E\uD49F"+
	"\uD4A0\uD540\uD541\uD542\uD543\uD544\uD545\uD546"+
	"\uD547\uD3FE\uCCDC\uD548\uD549\uD54A\uD54B\uD54C"+
	"\uD54D\uD54E\uD54F\uCAC4\uD550\uD551\uD552\uD553"+
	"\uD554\uD555\uD556\uD557\uD558\uD559\uD55A\uD55B"+
	"\uD55C\uD55D\uD55E\uD55F\uD560\uD561\uD562\uD563"+
	"\uD564\uD565\uD566\uD567\uD568\uD569\uD56A\uD56B"+
	"\uD56C\uD56D\uD56E\uD56F\uD570\uD571\uD572\uD573"+
	"\uD574\uD575\uD576\uD577\uD578\uD579\uD57A\uD57B"+
	"\uD57C\uD57D\uD57E\uD580\uD581\uD582\uD583\uD584"+
	"\uD585\uD586\uD587\uD588\uD589\uD58A\uD58B\uD58C"+
	"\uD58D\uD58E\uD58F\uD590\uD591\uD592\uD593\uD594"+
	"\uD595\uD596\uD597\uD598\uD599\uD59A\uD59B\uD59C"+
	"\uD59D\uD59E\uD59F\uD5A0\uD640\uD641\uD642\uD643"+
	"\uD644\uD645\uD646\uD647\uD648\uD649\uD64A\uD64B"+
	"\uD64C\uD64D\uD64E\uD64F\uD650\uD651\uD652\uD653"+
	"\uD654\uD655\uD656\uD657\uD658\uD659\uD65A\uD65B"+
	"\uD65C\uD65D\uD65E\uD65F\uD660\uD661\uD662\uE5C0"+
	"\uD663\uD664\uD665\uD666\uD667\uD668\uD669\uD66A"+
	"\uD66B\uD66C\uD66D\uD66E\uD66F\uD670\uD671\uD672"+
	"\uD673\uD674\uD675\uD676\uD677\uD678\uD679\uD67A"+
	"\uD67B\uD67C\uD67D\uD67E\uD680\uD681\uF6A5\uD682"+
	"\uD683\uD684\uD685\uD686\uD687\uD688\uD689\uD68A"+
	"\uD68B\uD68C\uD68D\uD68E\uD68F\uD690\uD691\uD692"+
	"\uD693\uD694\uD695\uD696\uD697\uD698\uD699\uD69A"+
	"\uD69B\uD69C\uD69D\uD69E\uD69F\uD6A0\uD740\uD741"+
	"\uD742\uD743\uD744\uD745\uD746\uD747\uD748\uD749"+
	"\uD74A\uD74B\uD74C\uD74D\uD74E\uD74F\uD750\uD751"+
	"\uD752\uD753\uD754\uD755\uD756\uD757\uD758\uD759"+
	"\uD75A\uD75B\uD75C\uD75D\uD75E\uD75F\uBEAF\uD760"+
	"\uD761\uD762\uD763\uD764\uC6A9\uD765\uD766\uD767"+
	"\uD768\uD769\uD76A\uD76B\uD76C\uD76D\uD76E\uD76F"+
	"\uD770\uD771\uD772\uD773\uD774\uD775\uD776\uD777"+
	"\uD778\uD779\uD77A\uD77B\uD77C\uD77D\uD77E\uD780"+
	"\uD781\uD782\uD783\uD784\uD785\uD786\uD787\uD788"+
	"\uD789\uD78A\uD78B\uD78C\uD78D\uD78E\uD78F\uD790"+
	"\uD791\uD792\uD793\uD794\uD795\uD796\uD797\uD798"+
	"\uDAA5\uBCC6\uB6A9\uB8BC\uC8CF\uBCA5\uDAA6\uDAA7"+
	"\uCCD6\uC8C3\uDAA8\uC6FD\uD799\uD1B5\uD2E9\uD1B6"+
	"\uBCC7\uD79A\uBDB2\uBBE4\uDAA9\uDAAA\uD1C8\uDAAB"+
	"\uD0ED\uB6EF\uC2DB\uD79B\uCBCF\uB7ED\uC9E8\uB7C3"+
	"\uBEF7\uD6A4\uDAAC\uDAAD\uC6C0\uD7E7\uCAB6\uD79C"+
	"\uD5A9\uCBDF\uD5EF\uDAAE\uD6DF\uB4CA\uDAB0\uDAAF"+
	"\uD79D\uD2EB\uDAB1\uDAB2\uDAB3\uCAD4\uDAB4\uCAAB"+
	"\uDAB5\uDAB6\uB3CF\uD6EF\uDAB7\uBBB0\uB5AE\uDAB8"+
	"\uDAB9\uB9EE\uD1AF\uD2E8\uDABA\uB8C3\uCFEA\uB2EF"+
	"\uDABB\uDABC\uD79E\uBDEB\uCEDC\uD3EF\uDABD\uCEF3"+
	"\uDABE\uD3D5\uBBE5\uDABF\uCBB5\uCBD0\uDAC0\uC7EB"+
	"\uD6EE\uDAC1\uC5B5\uB6C1\uDAC2\uB7CC\uBFCE\uDAC3"+
	"\uDAC4\uCBAD\uDAC5\uB5F7\uDAC6\uC1C2\uD7BB\uDAC7"+
	"\uCCB8\uD79F\uD2EA\uC4B1\uDAC8\uB5FD\uBBD1\uDAC9"+
	"\uD0B3\uDACA\uDACB\uCEBD\uDACC\uDACD\uDACE\uB2F7"+
	"\uDAD1\uDACF\uD1E8\uDAD0\uC3D5\uDAD2\uD7A0\uDAD3"+
	"\uDAD4\uDAD5\uD0BB\uD2A5\uB0F9\uDAD6\uC7AB\uDAD7"+
	"\uBDF7\uC3A1\uDAD8\uDAD9\uC3FD\uCCB7\uDADA\uDADB"+
	"\uC0BE\uC6D7\uDADC\uDADD\uC7B4\uDADE\uDADF\uB9C8"+
	"\uD840\uD841\uD842\uD843\uD844\uD845\uD846\uD847"+
	"\uD848\uBBED\uD849\uD84A\uD84B\uD84C\uB6B9\uF4F8"+
	"\uD84D\uF4F9\uD84E\uD84F\uCDE3\uD850\uD851\uD852"+
	"\uD853\uD854\uD855\uD856\uD857\uF5B9\uD858\uD859"+
	"\uD85A\uD85B\uEBE0\uD85C\uD85D\uD85E\uD85F\uD860"+
	"\uD861\uCFF3\uBBBF\uD862\uD863\uD864\uD865\uD866"+
	"\uD867\uD868\uBAC0\uD4A5\uD869\uD86A\uD86B\uD86C"+
	"\uD86D\uD86E\uD86F\uE1D9\uD870\uD871\uD872\uD873"+
	"\uF5F4\uB1AA\uB2F2\uD874\uD875\uD876\uD877\uD878"+
	"\uD879\uD87A\uF5F5\uD87B\uD87C\uF5F7\uD87D\uD87E"+
	"\uD880\uBAD1\uF5F6\uD881\uC3B2\uD882\uD883\uD884"+
	"\uD885\uD886\uD887\uD888\uF5F9\uD889\uD88A\uD88B"+
	"\uF5F8\uD88C\uD88D\uD88E\uD88F\uD890\uD891\uD892"+
	"\uD893\uD894\uD895\uD896\uD897\uD898\uD899\uD89A"+
	"\uD89B\uD89C\uD89D\uD89E\uD89F\uD8A0\uD940\uD941"+
	"\uD942\uD943\uD944\uD945\uD946\uD947\uD948\uD949"+
	"\uD94A\uD94B\uD94C\uD94D\uD94E\uD94F\uD950\uD951"+
	"\uD952\uD953\uD954\uD955\uD956\uD957\uD958\uD959"+
	"\uD95A\uD95B\uD95C\uD95D\uD95E\uD95F\uD960\uD961"+
	"\uD962\uD963\uD964\uD965\uD966\uD967\uD968\uD969"+
	"\uD96A\uD96B\uD96C\uD96D\uD96E\uD96F\uD970\uD971"+
	"\uD972\uD973\uD974\uD975\uD976\uD977\uD978\uD979"+
	"\uD97A\uD97B\uD97C\uD97D\uD97E\uD980\uD981\uD982"+
	"\uD983\uD984\uD985\uD986\uD987\uD988\uD989\uD98A"+
	"\uD98B\uD98C\uD98D\uD98E\uD98F\uD990\uD991\uD992"+
	"\uD993\uD994\uD995\uD996\uD997\uD998\uD999\uD99A"+
	"\uD99B\uD99C\uD99D\uD99E\uD99F\uD9A0\uDA40\uDA41"+
	"\uDA42\uDA43\uDA44\uDA45\uDA46\uDA47\uDA48\uDA49"+
	"\uDA4A\uDA4B\uDA4C\uDA4D\uDA4E\uB1B4\uD5EA\uB8BA"+
	"\uDA4F\uB9B1\uB2C6\uD4F0\uCFCD\uB0DC\uD5CB\uBBF5"+
	"\uD6CA\uB7B7\uCCB0\uC6B6\uB1E1\uB9BA\uD6FC\uB9E1"+
	"\uB7A1\uBCFA\uEADA\uEADB\uCCF9\uB9F3\uEADC\uB4FB"+
	"\uC3B3\uB7D1\uBAD8\uEADD\uD4F4\uEADE\uBCD6\uBBDF"+
	"\uEADF\uC1DE\uC2B8\uD4DF\uD7CA\uEAE0\uEAE1\uEAE4"+
	"\uEAE2\uEAE3\uC9DE\uB8B3\uB6C4\uEAE5\uCAEA\uC9CD"+
	"\uB4CD\uDA50\uDA51\uE2D9\uC5E2\uEAE6\uC0B5\uDA52"+
	"\uD7B8\uEAE7\uD7AC\uC8FC\uD8D3\uD8CD\uD4DE\uDA53"+
	"\uD4F9\uC9C4\uD3AE\uB8D3\uB3E0\uDA54\uC9E2\uF4F6"+
	"\uDA55\uDA56\uDA57\uBAD5\uDA58\uF4F7\uDA59\uDA5A"+
	"\uD7DF\uDA5B\uDA5C\uF4F1\uB8B0\uD5D4\uB8CF\uC6F0"+
	"\uDA5D\uDA5E\uDA5F\uDA60\uDA61\uDA62\uDA63\uDA64"+
	"\uDA65\uB3C3\uDA66\uDA67\uF4F2\uB3AC\uDA68\uDA69"+
	"\uDA6A\uDA6B\uD4BD\uC7F7\uDA6C\uDA6D\uDA6E\uDA6F"+
	"\uDA70\uF4F4\uDA71\uDA72\uF4F3\uDA73\uDA74\uDA75"+
	"\uDA76\uDA77\uDA78\uDA79\uDA7A\uDA7B\uDA7C\uCCCB"+
	"\uDA7D\uDA7E\uDA80\uC8A4\uDA81\uDA82\uDA83\uDA84"+
	"\uDA85\uDA86\uDA87\uDA88\uDA89\uDA8A\uDA8B\uDA8C"+
	"\uDA8D\uF4F5\uDA8E\uD7E3\uC5BF\uF5C0\uDA8F\uDA90"+
	"\uF5BB\uDA91\uF5C3\uDA92\uF5C2\uDA93\uD6BA\uF5C1"+
	"\uDA94\uDA95\uDA96\uD4BE\uF5C4\uDA97\uF5CC\uDA98"+
	"\uDA99\uDA9A\uDA9B\uB0CF\uB5F8\uDA9C\uF5C9\uF5CA"+
	"\uDA9D\uC5DC\uDA9E\uDA9F\uDAA0\uDB40\uF5C5\uF5C6"+
	"\uDB41\uDB42\uF5C7\uF5CB\uDB43\uBEE0\uF5C8\uB8FA"+
	"\uDB44\uDB45\uDB46\uF5D0\uF5D3\uDB47\uDB48\uDB49"+
	"\uBFE7\uDB4A\uB9F2\uF5BC\uF5CD\uDB4B\uDB4C\uC2B7"+
	"\uDB4D\uDB4E\uDB4F\uCCF8\uDB50\uBCF9\uDB51\uF5CE"+
	"\uF5CF\uF5D1\uB6E5\uF5D2\uDB52\uF5D5\uDB53\uDB54"+
	"\uDB55\uDB56\uDB57\uDB58\uDB59\uF5BD\uDB5A\uDB5B"+
	"\uDB5C\uF5D4\uD3BB\uDB5D\uB3EC\uDB5E\uDB5F\uCCA4"+
	"\uDB60\uDB61\uDB62\uDB63\uF5D6\uDB64\uDB65\uDB66"+
	"\uDB67\uDB68\uDB69\uDB6A\uDB6B\uF5D7\uBEE1\uF5D8"+
	"\uDB6C\uDB6D\uCCDF\uF5DB\uDB6E\uDB6F\uDB70\uDB71"+
	"\uDB72\uB2C8\uD7D9\uDB73\uF5D9\uDB74\uF5DA\uF5DC"+
	"\uDB75\uF5E2\uDB76\uDB77\uDB78\uF5E0\uDB79\uDB7A"+
	"\uDB7B\uF5DF\uF5DD\uDB7C\uDB7D\uF5E1\uDB7E\uDB80"+
	"\uF5DE\uF5E4\uF5E5\uDB81\uCCE3\uDB82\uDB83\uE5BF"+
	"\uB5B8\uF5E3\uF5E8\uCCA3\uDB84\uDB85\uDB86\uDB87"+
	"\uDB88\uF5E6\uF5E7\uDB89\uDB8A\uDB8B\uDB8C\uDB8D"+
	"\uDB8E\uF5BE\uDB8F\uDB90\uDB91\uDB92\uDB93\uDB94"+
	"\uDB95\uDB96\uDB97\uDB98\uDB99\uDB9A\uB1C4\uDB9B"+
	"\uDB9C\uF5BF\uDB9D\uDB9E\uB5C5\uB2E4\uDB9F\uF5EC"+
	"\uF5E9\uDBA0\uB6D7\uDC40\uF5ED\uDC41\uF5EA\uDC42"+
	"\uDC43\uDC44\uDC45\uDC46\uF5EB\uDC47\uDC48\uB4DA"+
	"\uDC49\uD4EA\uDC4A\uDC4B\uDC4C\uF5EE\uDC4D\uB3F9"+
	"\uDC4E\uDC4F\uDC50\uDC51\uDC52\uDC53\uDC54\uF5EF"+
	"\uF5F1\uDC55\uDC56\uDC57\uF5F0\uDC58\uDC59\uDC5A"+
	"\uDC5B\uDC5C\uDC5D\uDC5E\uF5F2\uDC5F\uF5F3\uDC60"+
	"\uDC61\uDC62\uDC63\uDC64\uDC65\uDC66\uDC67\uDC68"+
	"\uDC69\uDC6A\uDC6B\uC9ED\uB9AA\uDC6C\uDC6D\uC7FB"+
	"\uDC6E\uDC6F\uB6E3\uDC70\uDC71\uDC72\uDC73\uDC74"+
	"\uDC75\uDC76\uCCC9\uDC77\uDC78\uDC79\uDC7A\uDC7B"+
	"\uDC7C\uDC7D\uDC7E\uDC80\uDC81\uDC82\uDC83\uDC84"+
	"\uDC85\uDC86\uDC87\uDC88\uDC89\uDC8A\uEAA6\uDC8B"+
	"\uDC8C\uDC8D\uDC8E\uDC8F\uDC90\uDC91\uDC92\uDC93"+
	"\uDC94\uDC95\uDC96\uDC97\uDC98\uDC99\uDC9A\uDC9B"+
	"\uDC9C\uDC9D\uDC9E\uDC9F\uDCA0\uDD40\uDD41\uDD42"+
	"\uDD43\uDD44\uDD45\uDD46\uDD47\uDD48\uDD49\uDD4A"+
	"\uDD4B\uDD4C\uDD4D\uDD4E\uDD4F\uDD50\uDD51\uDD52"+
	"\uDD53\uDD54\uDD55\uDD56\uDD57\uDD58\uDD59\uDD5A"+
	"\uDD5B\uDD5C\uDD5D\uDD5E\uDD5F\uDD60\uDD61\uDD62"+
	"\uDD63\uDD64\uDD65\uDD66\uDD67\uDD68\uDD69\uDD6A"+
	"\uDD6B\uDD6C\uDD6D\uDD6E\uDD6F\uDD70\uDD71\uDD72"+
	"\uDD73\uDD74\uDD75\uDD76\uDD77\uDD78\uDD79\uDD7A"+
	"\uDD7B\uDD7C\uDD7D\uDD7E\uDD80\uDD81\uDD82\uDD83"+
	"\uDD84\uDD85\uDD86\uDD87\uDD88\uDD89\uDD8A\uDD8B"+
	"\uDD8C\uDD8D\uDD8E\uDD8F\uDD90\uDD91\uDD92\uDD93"+
	"\uDD94\uDD95\uDD96\uDD97\uDD98\uDD99\uDD9A\uDD9B"+
	"\uDD9C\uDD9D\uDD9E\uDD9F\uDDA0\uDE40\uDE41\uDE42"+
	"\uDE43\uDE44\uDE45\uDE46\uDE47\uDE48\uDE49\uDE4A"+
	"\uDE4B\uDE4C\uDE4D\uDE4E\uDE4F\uDE50\uDE51\uDE52"+
	"\uDE53\uDE54\uDE55\uDE56\uDE57\uDE58\uDE59\uDE5A"+
	"\uDE5B\uDE5C\uDE5D\uDE5E\uDE5F\uDE60\uB3B5\uD4FE"+
	"\uB9EC\uD0F9\uDE61\uE9ED\uD7AA\uE9EE\uC2D6\uC8ED"+
	"\uBAE4\uE9EF\uE9F0\uE9F1\uD6E1\uE9F2\uE9F3\uE9F5"+
	"\uE9F4\uE9F6\uE9F7\uC7E1\uE9F8\uD4D8\uE9F9\uBDCE"+
	"\uDE62\uE9FA\uE9FB\uBDCF\uE9FC\uB8A8\uC1BE\uE9FD"+
	"\uB1B2\uBBD4\uB9F5\uE9FE\uDE63\uEAA1\uEAA2\uEAA3"+
	"\uB7F8\uBCAD\uDE64\uCAE4\uE0CE\uD4AF\uCFBD\uD5B7"+
	"\uEAA4\uD5DE\uEAA5\uD0C1\uB9BC\uDE65\uB4C7\uB1D9"+
	"\uDE66\uDE67\uDE68\uC0B1\uDE69\uDE6A\uDE6B\uDE6C"+
	"\uB1E6\uB1E7\uDE6D\uB1E8\uDE6E\uDE6F\uDE70\uDE71"+
	"\uB3BD\uC8E8\uDE72\uDE73\uDE74\uDE75\uE5C1\uDE76"+
	"\uDE77\uB1DF\uDE78\uDE79\uDE7A\uC1C9\uB4EF\uDE7B"+
	"\uDE7C\uC7A8\uD3D8\uDE7D\uC6F9\uD1B8\uDE7E\uB9FD"+
	"\uC2F5\uDE80\uDE81\uDE82\uDE83\uDE84\uD3AD\uDE85"+
	"\uD4CB\uBDFC\uDE86\uE5C2\uB7B5\uE5C3\uDE87\uDE88"+
	"\uBBB9\uD5E2\uDE89\uBDF8\uD4B6\uCEA5\uC1AC\uB3D9"+
	"\uDE8A\uDE8B\uCCF6\uDE8C\uE5C6\uE5C4\uE5C8\uDE8D"+
	"\uE5CA\uE5C7\uB5CF\uC6C8\uDE8E\uB5FC\uE5C5\uDE8F"+
	"\uCAF6\uDE90\uDE91\uE5C9\uDE92\uDE93\uDE94\uC3D4"+
	"\uB1C5\uBCA3\uDE95\uDE96\uDE97\uD7B7\uDE98\uDE99";

    private final static String innerIndex9= 
	"\uCDCB\uCBCD\uCACA\uCCD3\uE5CC\uE5CB\uC4E6\uDE9A"+
	"\uDE9B\uD1A1\uD1B7\uE5CD\uDE9C\uE5D0\uDE9D\uCDB8"+
	"\uD6F0\uE5CF\uB5DD\uDE9E\uCDBE\uDE9F\uE5D1\uB6BA"+
	"\uDEA0\uDF40\uCDA8\uB9E4\uDF41\uCAC5\uB3D1\uCBD9"+
	"\uD4EC\uE5D2\uB7EA\uDF42\uDF43\uDF44\uE5CE\uDF45"+
	"\uDF46\uDF47\uDF48\uDF49\uDF4A\uE5D5\uB4FE\uE5D6"+
	"\uDF4B\uDF4C\uDF4D\uDF4E\uDF4F\uE5D3\uE5D4\uDF50"+
	"\uD2DD\uDF51\uDF52\uC2DF\uB1C6\uDF53\uD3E2\uDF54"+
	"\uDF55\uB6DD\uCBEC\uDF56\uE5D7\uDF57\uDF58\uD3F6"+
	"\uDF59\uDF5A\uDF5B\uDF5C\uDF5D\uB1E9\uDF5E\uB6F4"+
	"\uE5DA\uE5D8\uE5D9\uB5C0\uDF5F\uDF60\uDF61\uD2C5"+
	"\uE5DC\uDF62\uDF63\uE5DE\uDF64\uDF65\uDF66\uDF67"+
	"\uDF68\uDF69\uE5DD\uC7B2\uDF6A\uD2A3\uDF6B\uDF6C"+
	"\uE5DB\uDF6D\uDF6E\uDF6F\uDF70\uD4E2\uD5DA\uDF71"+
	"\uDF72\uDF73\uDF74\uDF75\uE5E0\uD7F1\uDF76\uDF77"+
	"\uDF78\uDF79\uDF7A\uDF7B\uDF7C\uE5E1\uDF7D\uB1DC"+
	"\uD1FB\uDF7E\uE5E2\uE5E4\uDF80\uDF81\uDF82\uDF83"+
	"\uE5E3\uDF84\uDF85\uE5E5\uDF86\uDF87\uDF88\uDF89"+
	"\uDF8A\uD2D8\uDF8B\uB5CB\uDF8C\uE7DF\uDF8D\uDAF5"+
	"\uDF8E\uDAF8\uDF8F\uDAF6\uDF90\uDAF7\uDF91\uDF92"+
	"\uDF93\uDAFA\uD0CF\uC4C7\uDF94\uDF95\uB0EE\uDF96"+
	"\uDF97\uDF98\uD0B0\uDF99\uDAF9\uDF9A\uD3CA\uBAAA"+
	"\uDBA2\uC7F1\uDF9B\uDAFC\uDAFB\uC9DB\uDAFD\uDF9C"+
	"\uDBA1\uD7DE\uDAFE\uC1DA\uDF9D\uDF9E\uDBA5\uDF9F"+
	"\uDFA0\uD3F4\uE040\uE041\uDBA7\uDBA4\uE042\uDBA8"+
	"\uE043\uE044\uBDBC\uE045\uE046\uE047\uC0C9\uDBA3"+
	"\uDBA6\uD6A3\uE048\uDBA9\uE049\uE04A\uE04B\uDBAD"+
	"\uE04C\uE04D\uE04E\uDBAE\uDBAC\uBAC2\uE04F\uE050"+
	"\uE051\uBFA4\uDBAB\uE052\uE053\uE054\uDBAA\uD4C7"+
	"\uB2BF\uE055\uE056\uDBAF\uE057\uB9F9\uE058\uDBB0"+
	"\uE059\uE05A\uE05B\uE05C\uB3BB\uE05D\uE05E\uE05F"+
	"\uB5A6\uE060\uE061\uE062\uE063\uB6BC\uDBB1\uE064"+
	"\uE065\uE066\uB6F5\uE067\uDBB2\uE068\uE069\uE06A"+
	"\uE06B\uE06C\uE06D\uE06E\uE06F\uE070\uE071\uE072"+
	"\uE073\uE074\uE075\uE076\uE077\uE078\uE079\uE07A"+
	"\uE07B\uB1C9\uE07C\uE07D\uE07E\uE080\uDBB4\uE081"+
	"\uE082\uE083\uDBB3\uDBB5\uE084\uE085\uE086\uE087"+
	"\uE088\uE089\uE08A\uE08B\uE08C\uE08D\uE08E\uDBB7"+
	"\uE08F\uDBB6\uE090\uE091\uE092\uE093\uE094\uE095"+
	"\uE096\uDBB8\uE097\uE098\uE099\uE09A\uE09B\uE09C"+
	"\uE09D\uE09E\uE09F\uDBB9\uE0A0\uE140\uDBBA\uE141"+
	"\uE142\uD3CF\uF4FA\uC7F5\uD7C3\uC5E4\uF4FC\uF4FD"+
	"\uF4FB\uE143\uBEC6\uE144\uE145\uE146\uE147\uD0EF"+
	"\uE148\uE149\uB7D3\uE14A\uE14B\uD4CD\uCCAA\uE14C"+
	"\uE14D\uF5A2\uF5A1\uBAA8\uF4FE\uCBD6\uE14E\uE14F"+
	"\uE150\uF5A4\uC0D2\uE151\uB3EA\uE152\uCDAA\uF5A5"+
	"\uF5A3\uBDB4\uF5A8\uE153\uF5A9\uBDCD\uC3B8\uBFE1"+
	"\uCBE1\uF5AA\uE154\uE155\uE156\uF5A6\uF5A7\uC4F0"+
	"\uE157\uE158\uE159\uE15A\uE15B\uF5AC\uE15C\uB4BC"+
	"\uE15D\uD7ED\uE15E\uB4D7\uF5AB\uF5AE\uE15F\uE160"+
	"\uF5AD\uF5AF\uD0D1\uE161\uE162\uE163\uE164\uE165"+
	"\uE166\uE167\uC3D1\uC8A9\uE168\uE169\uE16A\uE16B"+
	"\uE16C\uE16D\uF5B0\uF5B1\uE16E\uE16F\uE170\uE171"+
	"\uE172\uE173\uF5B2\uE174\uE175\uF5B3\uF5B4\uF5B5"+
	"\uE176\uE177\uE178\uE179\uF5B7\uF5B6\uE17A\uE17B"+
	"\uE17C\uE17D\uF5B8\uE17E\uE180\uE181\uE182\uE183"+
	"\uE184\uE185\uE186\uE187\uE188\uE189\uE18A\uB2C9"+
	"\uE18B\uD3D4\uCACD\uE18C\uC0EF\uD6D8\uD2B0\uC1BF"+
	"\uE18D\uBDF0\uE18E\uE18F\uE190\uE191\uE192\uE193"+
	"\uE194\uE195\uE196\uE197\uB8AA\uE198\uE199\uE19A"+
	"\uE19B\uE19C\uE19D\uE19E\uE19F\uE1A0\uE240\uE241"+
	"\uE242\uE243\uE244\uE245\uE246\uE247\uE248\uE249"+
	"\uE24A\uE24B\uE24C\uE24D\uE24E\uE24F\uE250\uE251"+
	"\uE252\uE253\uE254\uE255\uE256\uE257\uE258\uE259"+
	"\uE25A\uE25B\uE25C\uE25D\uE25E\uE25F\uE260\uE261"+
	"\uE262\uE263\uE264\uE265\uE266\uE267\uE268\uE269"+
	"\uE26A\uE26B\uE26C\uE26D\uE26E\uE26F\uE270\uE271"+
	"\uE272\uE273\uE274\uE275\uE276\uE277\uE278\uE279"+
	"\uE27A\uE27B\uE27C\uE27D\uE27E\uE280\uE281\uE282"+
	"\uE283\uE284\uE285\uE286\uE287\uE288\uE289\uE28A"+
	"\uE28B\uE28C\uE28D\uE28E\uE28F\uE290\uE291\uE292"+
	"\uE293\uE294\uE295\uE296\uE297\uE298\uE299\uE29A"+
	"\uE29B\uE29C\uE29D\uE29E\uE29F\uE2A0\uE340\uE341"+
	"\uE342\uE343\uE344\uE345\uE346\uE347\uE348\uE349"+
	"\uE34A\uE34B\uE34C\uE34D\uE34E\uE34F\uE350\uE351"+
	"\uE352\uE353\uE354\uE355\uE356\uE357\uE358\uE359"+
	"\uE35A\uE35B\uE35C\uE35D\uE35E\uE35F\uE360\uE361"+
	"\uE362\uE363\uE364\uE365\uE366\uE367\uE368\uE369"+
	"\uE36A\uE36B\uE36C\uE36D\uBCF8\uE36E\uE36F\uE370"+
	"\uE371\uE372\uE373\uE374\uE375\uE376\uE377\uE378"+
	"\uE379\uE37A\uE37B\uE37C\uE37D\uE37E\uE380\uE381"+
	"\uE382\uE383\uE384\uE385\uE386\uE387\uF6C6\uE388"+
	"\uE389\uE38A\uE38B\uE38C\uE38D\uE38E\uE38F\uE390"+
	"\uE391\uE392\uE393\uE394\uE395\uE396\uE397\uE398"+
	"\uE399\uE39A\uE39B\uE39C\uE39D\uE39E\uE39F\uE3A0"+
	"\uE440\uE441\uE442\uE443\uE444\uE445\uF6C7\uE446"+
	"\uE447\uE448\uE449\uE44A\uE44B\uE44C\uE44D\uE44E"+
	"\uE44F\uE450\uE451\uE452\uE453\uE454\uE455\uE456"+
	"\uE457\uE458\uE459\uE45A\uE45B\uE45C\uE45D\uE45E"+
	"\uF6C8\uE45F\uE460\uE461\uE462\uE463\uE464\uE465"+
	"\uE466\uE467\uE468\uE469\uE46A\uE46B\uE46C\uE46D"+
	"\uE46E\uE46F\uE470\uE471\uE472\uE473\uE474\uE475"+
	"\uE476\uE477\uE478\uE479\uE47A\uE47B\uE47C\uE47D"+
	"\uE47E\uE480\uE481\uE482\uE483\uE484\uE485\uE486"+
	"\uE487\uE488\uE489\uE48A\uE48B\uE48C\uE48D\uE48E"+
	"\uE48F\uE490\uE491\uE492\uE493\uE494\uE495\uE496"+
	"\uE497\uE498\uE499\uE49A\uE49B\uE49C\uE49D\uE49E"+
	"\uE49F\uE4A0\uE540\uE541\uE542\uE543\uE544\uE545"+
	"\uE546\uE547\uE548\uE549\uE54A\uE54B\uE54C\uE54D"+
	"\uE54E\uE54F\uE550\uE551\uE552\uE553\uE554\uE555"+
	"\uE556\uE557\uE558\uE559\uE55A\uE55B\uE55C\uE55D"+
	"\uE55E\uE55F\uE560\uE561\uE562\uE563\uE564\uE565"+
	"\uE566\uE567\uE568\uE569\uE56A\uE56B\uE56C\uE56D"+
	"\uE56E\uE56F\uE570\uE571\uE572\uE573\uF6C9\uE574"+
	"\uE575\uE576\uE577\uE578\uE579\uE57A\uE57B\uE57C"+
	"\uE57D\uE57E\uE580\uE581\uE582\uE583\uE584\uE585"+
	"\uE586\uE587\uE588\uE589\uE58A\uE58B\uE58C\uE58D"+
	"\uE58E\uE58F\uE590\uE591\uE592\uE593\uE594\uE595"+
	"\uE596\uE597\uE598\uE599\uE59A\uE59B\uE59C\uE59D"+
	"\uE59E\uE59F\uF6CA\uE5A0\uE640\uE641\uE642\uE643"+
	"\uE644\uE645\uE646\uE647\uE648\uE649\uE64A\uE64B"+
	"\uE64C\uE64D\uE64E\uE64F\uE650\uE651\uE652\uE653"+
	"\uE654\uE655\uE656\uE657\uE658\uE659\uE65A\uE65B"+
	"\uE65C\uE65D\uE65E\uE65F\uE660\uE661\uE662\uF6CC"+
	"\uE663\uE664\uE665\uE666\uE667\uE668\uE669\uE66A"+
	"\uE66B\uE66C\uE66D\uE66E\uE66F\uE670\uE671\uE672"+
	"\uE673\uE674\uE675\uE676\uE677\uE678\uE679\uE67A"+
	"\uE67B\uE67C\uE67D\uE67E\uE680\uE681\uE682\uE683"+
	"\uE684\uE685\uE686\uE687\uE688\uE689\uE68A\uE68B"+
	"\uE68C\uE68D\uE68E\uE68F\uE690\uE691\uE692\uE693"+
	"\uE694\uE695\uE696\uE697\uE698\uE699\uE69A\uE69B"+
	"\uE69C\uE69D\uF6CB\uE69E\uE69F\uE6A0\uE740\uE741"+
	"\uE742\uE743\uE744\uE745\uE746\uE747\uF7E9\uE748"+
	"\uE749\uE74A\uE74B\uE74C\uE74D\uE74E\uE74F\uE750"+
	"\uE751\uE752\uE753\uE754\uE755\uE756\uE757\uE758"+
	"\uE759\uE75A\uE75B\uE75C\uE75D\uE75E\uE75F\uE760"+
	"\uE761\uE762\uE763\uE764\uE765\uE766\uE767\uE768"+
	"\uE769\uE76A\uE76B\uE76C\uE76D\uE76E\uE76F\uE770"+
	"\uE771\uE772\uE773\uE774\uE775\uE776\uE777\uE778"+
	"\uE779\uE77A\uE77B\uE77C\uE77D\uE77E\uE780\uE781"+
	"\uE782\uE783\uE784\uE785\uE786\uE787\uE788\uE789"+
	"\uE78A\uE78B\uE78C\uE78D\uE78E\uE78F\uE790\uE791"+
	"\uE792\uE793\uE794\uE795\uE796\uE797\uE798\uE799"+
	"\uE79A\uE79B\uE79C\uE79D\uE79E\uE79F\uE7A0\uE840"+
	"\uE841\uE842\uE843\uE844\uE845\uE846\uE847\uE848"+
	"\uE849\uE84A\uE84B\uE84C\uE84D\uE84E\uF6CD\uE84F"+
	"\uE850\uE851\uE852\uE853\uE854\uE855\uE856\uE857"+
	"\uE858\uE859\uE85A\uE85B\uE85C\uE85D\uE85E\uE85F"+
	"\uE860\uE861\uE862\uE863\uE864\uE865\uE866\uE867"+
	"\uE868\uE869\uE86A\uE86B\uE86C\uE86D\uE86E\uE86F"+
	"\uE870\uE871\uE872\uE873\uE874\uE875\uE876\uE877"+
	"\uE878\uE879\uE87A\uF6CE\uE87B\uE87C\uE87D\uE87E"+
	"\uE880\uE881\uE882\uE883\uE884\uE885\uE886\uE887"+
	"\uE888\uE889\uE88A\uE88B\uE88C\uE88D\uE88E\uE88F"+
	"\uE890\uE891\uE892\uE893\uE894\uEEC4\uEEC5\uEEC6"+
	"\uD5EB\uB6A4\uEEC8\uEEC7\uEEC9\uEECA\uC7A5\uEECB"+
	"\uEECC\uE895\uB7B0\uB5F6\uEECD\uEECF\uE896\uEECE"+
	"\uE897\uB8C6\uEED0\uEED1\uEED2\uB6DB\uB3AE\uD6D3"+
	"\uC4C6\uB1B5\uB8D6\uEED3\uEED4\uD4BF\uC7D5\uBEFB"+
	"\uCED9\uB9B3\uEED6\uEED5\uEED8\uEED7\uC5A5\uEED9"+
	"\uEEDA\uC7AE\uEEDB\uC7AF\uEEDC\uB2A7\uEEDD\uEEDE"+
	"\uEEDF\uEEE0\uEEE1\uD7EA\uEEE2\uEEE3\uBCD8\uEEE4"+
	"\uD3CB\uCCFA\uB2AC\uC1E5\uEEE5\uC7A6\uC3AD\uE898"+
	"\uEEE6\uEEE7\uEEE8\uEEE9\uEEEA\uEEEB\uEEEC\uE899"+
	"\uEEED\uEEEE\uEEEF\uE89A\uE89B\uEEF0\uEEF1\uEEF2"+
	"\uEEF4\uEEF3\uE89C\uEEF5\uCDAD\uC2C1\uEEF6\uEEF7"+
	"\uEEF8\uD5A1\uEEF9\uCFB3\uEEFA\uEEFB\uE89D\uEEFC"+
	"\uEEFD\uEFA1\uEEFE\uEFA2\uB8F5\uC3FA\uEFA3\uEFA4"+
	"\uBDC2\uD2BF\uB2F9\uEFA5\uEFA6\uEFA7\uD2F8\uEFA8"+
	"\uD6FD\uEFA9\uC6CC\uE89E\uEFAA\uEFAB\uC1B4\uEFAC"+
	"\uCFFA\uCBF8\uEFAE\uEFAD\uB3FA\uB9F8\uEFAF\uEFB0"+
	"\uD0E2\uEFB1\uEFB2\uB7E6\uD0BF\uEFB3\uEFB4\uEFB5"+
	"\uC8F1\uCCE0\uEFB6\uEFB7\uEFB8\uEFB9\uEFBA\uD5E0"+
	"\uEFBB\uB4ED\uC3AA\uEFBC\uE89F\uEFBD\uEFBE\uEFBF"+
	"\uE8A0\uCEFD\uEFC0\uC2E0\uB4B8\uD7B6\uBDF5\uE940"+
	"\uCFC7\uEFC3\uEFC1\uEFC2\uEFC4\uB6A7\uBCFC\uBEE2"+
	"\uC3CC\uEFC5\uEFC6\uE941\uEFC7\uEFCF\uEFC8\uEFC9"+
	"\uEFCA\uC7C2\uEFF1\uB6CD\uEFCB\uE942\uEFCC\uEFCD"+
	"\uB6C6\uC3BE\uEFCE\uE943\uEFD0\uEFD1\uEFD2\uD5F2"+
	"\uE944\uEFD3\uC4F7\uE945\uEFD4\uC4F8\uEFD5\uEFD6"+
	"\uB8E4\uB0F7\uEFD7\uEFD8\uEFD9\uE946\uEFDA\uEFDB"+
	"\uEFDC\uEFDD\uE947\uEFDE\uBEB5\uEFE1\uEFDF\uEFE0"+
	"\uE948\uEFE2\uEFE3\uC1CD\uEFE4\uEFE5\uEFE6\uEFE7"+
	"\uEFE8\uEFE9\uEFEA\uEFEB\uEFEC\uC0D8\uE949\uEFED"+
	"\uC1AD\uEFEE\uEFEF\uEFF0\uE94A\uE94B\uCFE2\uE94C"+
	"\uE94D\uE94E\uE94F\uE950\uE951\uE952\uE953\uB3A4"+
	"\uE954\uE955\uE956\uE957\uE958\uE959\uE95A\uE95B"+
	"\uE95C\uE95D\uE95E\uE95F\uE960\uE961\uE962\uE963"+
	"\uE964\uE965\uE966\uE967\uE968\uE969\uE96A\uE96B"+
	"\uE96C\uE96D\uE96E\uE96F\uE970\uE971\uE972\uE973"+
	"\uE974\uE975\uE976\uE977\uE978\uE979\uE97A\uE97B"+
	"\uE97C\uE97D\uE97E\uE980\uE981\uE982\uE983\uE984"+
	"\uE985\uE986\uE987\uE988\uE989\uE98A\uE98B\uE98C"+
	"\uE98D\uE98E\uE98F\uE990\uE991\uE992\uE993\uE994"+
	"\uE995\uE996\uE997\uE998\uE999\uE99A\uE99B\uE99C"+
	"\uE99D\uE99E\uE99F\uE9A0\uEA40\uEA41\uEA42\uEA43"+
	"\uEA44\uEA45\uEA46\uEA47\uEA48\uEA49\uEA4A\uEA4B"+
	"\uEA4C\uEA4D\uEA4E\uEA4F\uEA50\uEA51\uEA52\uEA53"+
	"\uEA54\uEA55\uEA56\uEA57\uEA58\uEA59\uEA5A\uEA5B"+
	"\uC3C5\uE3C5\uC9C1\uE3C6\uEA5C\uB1D5\uCECA\uB4B3"+
	"\uC8F2\uE3C7\uCFD0\uE3C8\uBCE4\uE3C9\uE3CA\uC3C6"+
	"\uD5A2\uC4D6\uB9EB\uCEC5\uE3CB\uC3F6\uE3CC\uEA5D"+
	"\uB7A7\uB8F3\uBAD2\uE3CD\uE3CE\uD4C4\uE3CF\uEA5E"+
	"\uE3D0\uD1CB\uE3D1\uE3D2\uE3D3\uE3D4\uD1D6\uE3D5"+
	"\uB2FB\uC0BB\uE3D6\uEA5F\uC0AB\uE3D7\uE3D8\uE3D9"+
	"\uEA60\uE3DA\uE3DB\uEA61\uB8B7\uDAE2\uEA62\uB6D3"+
	"\uEA63\uDAE4\uDAE3\uEA64\uEA65\uEA66\uEA67\uEA68"+
	"\uEA69\uEA6A\uDAE6\uEA6B\uEA6C\uEA6D\uC8EE\uEA6E"+
	"\uEA6F\uDAE5\uB7C0\uD1F4\uD2F5\uD5F3\uBDD7\uEA70"+
	"\uEA71\uEA72\uEA73\uD7E8\uDAE8\uDAE7\uEA74\uB0A2"+
	"\uCDD3\uEA75\uDAE9\uEA76\uB8BD\uBCCA\uC2BD\uC2A4"+
	"\uB3C2\uDAEA\uEA77\uC2AA\uC4B0\uBDB5\uEA78\uEA79"+
	"\uCFDE\uEA7A\uEA7B\uEA7C\uDAEB\uC9C2\uEA7D\uEA7E"+
	"\uEA80\uEA81\uEA82\uB1DD\uEA83\uEA84\uEA85\uDAEC"+
	"\uEA86\uB6B8\uD4BA\uEA87\uB3FD\uEA88\uEA89\uDAED"+
	"\uD4C9\uCFD5\uC5E3\uEA8A\uDAEE\uEA8B\uEA8C\uEA8D"+
	"\uEA8E\uEA8F\uDAEF\uEA90\uDAF0\uC1EA\uCCD5\uCFDD"+
	"\uEA91\uEA92\uEA93\uEA94\uEA95\uEA96\uEA97\uEA98"+
	"\uEA99\uEA9A\uEA9B\uEA9C\uEA9D\uD3E7\uC2A1\uEA9E"+
	"\uDAF1\uEA9F\uEAA0\uCBE5\uEB40\uDAF2\uEB41\uCBE6"+
	"\uD2FE\uEB42\uEB43\uEB44\uB8F4\uEB45\uEB46\uDAF3"+
	"\uB0AF\uCFB6\uEB47\uEB48\uD5CF\uEB49\uEB4A\uEB4B"+
	"\uEB4C\uEB4D\uEB4E\uEB4F\uEB50\uEB51\uEB52\uCBED"+
	"\uEB53\uEB54\uEB55\uEB56\uEB57\uEB58\uEB59\uEB5A"+
	"\uDAF4\uEB5B\uEB5C\uE3C4\uEB5D\uEB5E\uC1A5\uEB5F"+
	"\uEB60\uF6BF\uEB61\uEB62\uF6C0\uF6C1\uC4D1\uEB63"+
	"\uC8B8\uD1E3\uEB64\uEB65\uD0DB\uD1C5\uBCAF\uB9CD"+
	"\uEB66\uEFF4\uEB67\uEB68\uB4C6\uD3BA\uF6C2\uB3FB"+
	"\uEB69\uEB6A\uF6C3\uEB6B\uEB6C\uB5F1\uEB6D\uEB6E"+
	"\uEB6F\uEB70\uEB71\uEB72\uEB73\uEB74\uEB75\uEB76"+
	"\uF6C5\uEB77\uEB78\uEB79\uEB7A\uEB7B\uEB7C\uEB7D"+
	"\uD3EA\uF6A7\uD1A9\uEB7E\uEB80\uEB81\uEB82\uF6A9"+
	"\uEB83\uEB84\uEB85\uF6A8\uEB86\uEB87\uC1E3\uC0D7"+
	"\uEB88\uB1A2\uEB89\uEB8A\uEB8B\uEB8C\uCEED\uEB8D"+
	"\uD0E8\uF6AB\uEB8E\uEB8F\uCFF6\uEB90\uF6AA\uD5F0"+
	"\uF6AC\uC3B9\uEB91\uEB92\uEB93\uBBF4\uF6AE\uF6AD"+
	"\uEB94\uEB95\uEB96\uC4DE\uEB97\uEB98\uC1D8\uEB99"+
	"\uEB9A\uEB9B\uEB9C\uEB9D\uCBAA\uEB9E\uCFBC\uEB9F"+
	"\uEBA0\uEC40\uEC41\uEC42\uEC43\uEC44\uEC45\uEC46"+
	"\uEC47\uEC48\uF6AF\uEC49\uEC4A\uF6B0\uEC4B\uEC4C"+
	"\uF6B1\uEC4D\uC2B6\uEC4E\uEC4F\uEC50\uEC51\uEC52"+
	"\uB0D4\uC5F9\uEC53\uEC54\uEC55\uEC56\uF6B2\uEC57"+
	"\uEC58\uEC59\uEC5A\uEC5B\uEC5C\uEC5D\uEC5E\uEC5F"+
	"\uEC60\uEC61\uEC62\uEC63\uEC64\uEC65\uEC66\uEC67"+
	"\uEC68\uEC69\uC7E0\uF6A6\uEC6A\uEC6B\uBEB8\uEC6C"+
	"\uEC6D\uBEB2\uEC6E\uB5E5\uEC6F\uEC70\uB7C7\uEC71"+
	"\uBFBF\uC3D2\uC3E6\uEC72\uEC73\uD8CC\uEC74\uEC75"+
	"\uEC76\uB8EF\uEC77\uEC78\uEC79\uEC7A\uEC7B\uEC7C"+
	"\uEC7D\uEC7E\uEC80\uBDF9\uD1A5\uEC81\uB0D0\uEC82"+
	"\uEC83\uEC84\uEC85\uEC86\uF7B0\uEC87\uEC88\uEC89"+
	"\uEC8A\uEC8B\uEC8C\uEC8D\uEC8E\uF7B1\uEC8F\uEC90"+
	"\uEC91\uEC92\uEC93\uD0AC\uEC94\uB0B0\uEC95\uEC96"+
	"\uEC97\uF7B2\uF7B3\uEC98\uF7B4\uEC99\uEC9A\uEC9B"+
	"\uC7CA\uEC9C\uEC9D\uEC9E\uEC9F\uECA0\uED40\uED41"+
	"\uBECF\uED42\uED43\uF7B7\uED44\uED45\uED46\uED47"+
	"\uED48\uED49\uED4A\uF7B6\uED4B\uB1DE\uED4C\uF7B5"+
	"\uED4D\uED4E\uF7B8\uED4F\uF7B9\uED50\uED51\uED52"+
	"\uED53\uED54\uED55\uED56\uED57\uED58\uED59\uED5A"+
	"\uED5B\uED5C\uED5D\uED5E\uED5F\uED60\uED61\uED62"+
	"\uED63\uED64\uED65\uED66\uED67\uED68\uED69\uED6A"+
	"\uED6B\uED6C\uED6D\uED6E\uED6F\uED70\uED71\uED72"+
	"\uED73\uED74\uED75\uED76\uED77\uED78\uED79\uED7A"+
	"\uED7B\uED7C\uED7D\uED7E\uED80\uED81\uCEA4\uC8CD"+
	"\uED82\uBAAB\uE8B8\uE8B9\uE8BA\uBEC2\uED83\uED84"+
	"\uED85\uED86\uED87\uD2F4\uED88\uD4CF\uC9D8\uED89"+
	"\uED8A\uED8B\uED8C\uED8D\uED8E\uED8F\uED90\uED91"+
	"\uED92\uED93\uED94\uED95\uED96\uED97\uED98\uED99"+
	"\uED9A\uED9B\uED9C\uED9D\uED9E\uED9F\uEDA0\uEE40"+
	"\uEE41\uEE42\uEE43\uEE44\uEE45\uEE46\uEE47\uEE48"+
	"\uEE49\uEE4A\uEE4B\uEE4C\uEE4D\uEE4E\uEE4F\uEE50"+
	"\uEE51\uEE52\uEE53\uEE54\uEE55\uEE56\uEE57\uEE58"+
	"\uEE59\uEE5A\uEE5B\uEE5C\uEE5D\uEE5E\uEE5F\uEE60"+
	"\uEE61\uEE62\uEE63\uEE64\uEE65\uEE66\uEE67\uEE68"+
	"\uEE69\uEE6A\uEE6B\uEE6C\uEE6D\uEE6E\uEE6F\uEE70"+
	"\uEE71\uEE72\uEE73\uEE74\uEE75\uEE76\uEE77\uEE78"+
	"\uEE79\uEE7A\uEE7B\uEE7C\uEE7D\uEE7E\uEE80\uEE81"+
	"\uEE82\uEE83\uEE84\uEE85\uEE86\uEE87\uEE88\uEE89"+
	"\uEE8A\uEE8B\uEE8C\uEE8D\uEE8E\uEE8F\uEE90\uEE91"+
	"\uEE92\uEE93\uEE94\uEE95\uEE96\uEE97\uEE98\uEE99"+
	"\uEE9A\uEE9B\uEE9C\uEE9D\uEE9E\uEE9F\uEEA0\uEF40"+
	"\uEF41\uEF42\uEF43\uEF44\uEF45\uD2B3\uB6A5\uC7EA"+
	"\uF1FC\uCFEE\uCBB3\uD0EB\uE7EF\uCDE7\uB9CB\uB6D9"+
	"\uF1FD\uB0E4\uCBCC\uF1FE\uD4A4\uC2AD\uC1EC\uC6C4"+
	"\uBEB1\uF2A1\uBCD5\uEF46\uF2A2\uF2A3\uEF47\uF2A4"+
	"\uD2C3\uC6B5\uEF48\uCDC7\uF2A5\uEF49\uD3B1\uBFC5"+
	"\uCCE2\uEF4A\uF2A6\uF2A7\uD1D5\uB6EE\uF2A8\uF2A9"+
	"\uB5DF\uF2AA\uF2AB\uEF4B\uB2FC\uF2AC\uF2AD\uC8A7"+
	"\uEF4C\uEF4D\uEF4E\uEF4F\uEF50\uEF51\uEF52\uEF53"+
	"\uEF54\uEF55\uEF56\uEF57\uEF58\uEF59\uEF5A\uEF5B"+
	"\uEF5C\uEF5D\uEF5E\uEF5F\uEF60\uEF61\uEF62\uEF63"+
	"\uEF64\uEF65\uEF66\uEF67\uEF68\uEF69\uEF6A\uEF6B"+
	"\uEF6C\uEF6D\uEF6E\uEF6F\uEF70\uEF71\uB7E7\uEF72"+
	"\uEF73\uECA9\uECAA\uECAB\uEF74\uECAC\uEF75\uEF76"+
	"\uC6AE\uECAD\uECAE\uEF77\uEF78\uEF79\uB7C9\uCAB3"+
	"\uEF7A\uEF7B\uEF7C\uEF7D\uEF7E\uEF80\uEF81\uE2B8"+
	"\uF7CF\uEF82\uEF83\uEF84\uEF85\uEF86\uEF87\uEF88"+
	"\uEF89\uEF8A\uEF8B\uEF8C\uEF8D\uEF8E\uEF8F\uEF90"+
	"\uEF91\uEF92\uEF93\uEF94\uEF95\uEF96\uEF97\uEF98"+
	"\uEF99\uEF9A\uEF9B\uEF9C\uEF9D\uEF9E\uEF9F\uEFA0"+
	"\uF040\uF041\uF042\uF043\uF044\uF7D0\uF045\uF046"+
	"\uB2CD\uF047\uF048\uF049\uF04A\uF04B\uF04C\uF04D"+
	"\uF04E\uF04F\uF050\uF051\uF052\uF053\uF054\uF055"+
	"\uF056\uF057\uF058\uF059\uF05A\uF05B\uF05C\uF05D"+
	"\uF05E\uF05F\uF060\uF061\uF062\uF063\uF7D1\uF064"+
	"\uF065\uF066\uF067\uF068\uF069\uF06A\uF06B\uF06C"+
	"\uF06D\uF06E\uF06F\uF070\uF071\uF072\uF073\uF074"+
	"\uF075\uF076\uF077\uF078\uF079\uF07A\uF07B\uF07C"+
	"\uF07D\uF07E\uF080\uF081\uF082\uF083\uF084\uF085"+
	"\uF086\uF087\uF088\uF089\uF7D3\uF7D2\uF08A\uF08B"+
	"\uF08C\uF08D\uF08E\uF08F\uF090\uF091\uF092\uF093"+
	"\uF094\uF095\uF096\uE2BB\uF097\uBCA2\uF098\uE2BC"+
	"\uE2BD\uE2BE\uE2BF\uE2C0\uE2C1\uB7B9\uD2FB\uBDA4"+
	"\uCACE\uB1A5\uCBC7\uF099\uE2C2\uB6FC\uC8C4\uE2C3"+
	"\uF09A\uF09B\uBDC8\uF09C\uB1FD\uE2C4\uF09D\uB6F6"+
	"\uE2C5\uC4D9\uF09E\uF09F\uE2C6\uCFDA\uB9DD\uE2C7"+
	"\uC0A1\uF0A0\uE2C8\uB2F6\uF140\uE2C9\uF141\uC1F3"+
	"\uE2CA\uE2CB\uC2F8\uE2CC\uE2CD\uE2CE\uCAD7\uD8B8"+
	"\uD9E5\uCFE3\uF142\uF143\uF144\uF145\uF146\uF147"+
	"\uF148\uF149\uF14A\uF14B\uF14C\uF0A5\uF14D\uF14E"+
	"\uDCB0\uF14F\uF150\uF151\uF152\uF153\uF154\uF155"+
	"\uF156\uF157\uF158\uF159\uF15A\uF15B\uF15C\uF15D"+
	"\uF15E\uF15F\uF160\uF161\uF162\uF163\uF164\uF165"+
	"\uF166\uF167\uF168\uF169\uF16A\uF16B\uF16C\uF16D"+
	"\uF16E\uF16F\uF170\uF171\uF172\uF173\uF174\uF175"+
	"\uF176\uF177\uF178\uF179\uF17A\uF17B\uF17C\uF17D"+
	"\uF17E\uF180\uF181\uF182\uF183\uF184\uF185\uF186"+
	"\uF187\uF188\uF189\uF18A\uF18B\uF18C\uF18D\uF18E"+
	"\uF18F\uF190\uF191\uF192\uF193\uF194\uF195\uF196"+
	"\uF197\uF198\uF199\uF19A\uF19B\uF19C\uF19D\uF19E"+
	"\uF19F\uF1A0\uF240\uF241\uF242\uF243\uF244\uF245"+
	"\uF246\uF247\uF248\uF249\uF24A\uF24B\uF24C\uF24D"+
	"\uF24E\uF24F\uF250\uF251\uF252\uF253\uF254\uF255"+
	"\uF256\uF257\uF258\uF259\uF25A\uF25B\uF25C\uF25D"+
	"\uF25E\uF25F\uF260\uF261\uF262\uF263\uF264\uF265"+
	"\uF266\uF267\uF268\uF269\uF26A\uF26B\uF26C\uF26D"+
	"\uF26E\uF26F\uF270\uF271\uF272\uF273\uF274\uF275"+
	"\uF276\uF277\uF278\uF279\uF27A\uF27B\uF27C\uF27D"+
	"\uF27E\uF280\uF281\uF282\uF283\uF284\uF285\uF286"+
	"\uF287\uF288\uF289\uF28A\uF28B\uF28C\uF28D\uF28E"+
	"\uF28F\uF290\uF291\uF292\uF293\uF294\uF295\uF296"+
	"\uF297\uF298\uF299\uF29A\uF29B\uF29C\uF29D\uF29E"+
	"\uF29F\uF2A0\uF340\uF341\uF342\uF343\uF344\uF345"+
	"\uF346\uF347\uF348\uF349\uF34A\uF34B\uF34C\uF34D"+
	"\uF34E\uF34F\uF350\uF351\uC2ED\uD4A6\uCDD4\uD1B1"+
	"\uB3DB\uC7FD\uF352\uB2B5\uC2BF\uE6E0\uCABB\uE6E1"+
	"\uE6E2\uBED4\uE6E3\uD7A4\uCDD5\uE6E5\uBCDD\uE6E4"+
	"\uE6E6\uE6E7\uC2EE\uF353\uBDBE\uE6E8\uC2E6\uBAA7"+
	"\uE6E9\uF354\uE6EA\uB3D2\uD1E9\uF355\uF356\uBFA5"+
	"\uE6EB\uC6EF\uE6EC\uE6ED\uF357\uF358\uE6EE\uC6AD"+
	"\uE6EF\uF359\uC9A7\uE6F0\uE6F1\uE6F2\uE5B9\uE6F3"+
	"\uE6F4\uC2E2\uE6F5\uE6F6\uD6E8\uE6F7\uF35A\uE6F8"+
	"\uB9C7\uF35B\uF35C\uF35D\uF35E\uF35F\uF360\uF361"+
	"\uF7BB\uF7BA\uF362\uF363\uF364\uF365\uF7BE\uF7BC"+
	"\uBAA1\uF366\uF7BF\uF367\uF7C0\uF368\uF369\uF36A"+
	"\uF7C2\uF7C1\uF7C4\uF36B\uF36C\uF7C3\uF36D\uF36E"+
	"\uF36F\uF370\uF371\uF7C5\uF7C6\uF372\uF373\uF374"+
	"\uF375\uF7C7\uF376\uCBE8\uF377\uF378\uF379\uF37A"+
	"\uB8DF\uF37B\uF37C\uF37D\uF37E\uF380\uF381\uF7D4"+
	"\uF382\uF7D5\uF383\uF384\uF385\uF386\uF7D6\uF387"+
	"\uF388\uF389\uF38A\uF7D8\uF38B\uF7DA\uF38C\uF7D7"+
	"\uF38D\uF38E\uF38F\uF390\uF391\uF392\uF393\uF394"+
	"\uF395\uF7DB\uF396\uF7D9\uF397\uF398\uF399\uF39A"+
	"\uF39B\uF39C\uF39D\uD7D7\uF39E\uF39F\uF3A0\uF440"+
	"\uF7DC\uF441\uF442\uF443\uF444\uF445\uF446\uF7DD"+
	"\uF447\uF448\uF449\uF7DE\uF44A\uF44B\uF44C\uF44D"+
	"\uF44E\uF44F\uF450\uF451\uF452\uF453\uF454\uF7DF"+
	"\uF455\uF456\uF457\uF7E0\uF458\uF459\uF45A\uF45B"+
	"\uF45C\uF45D\uF45E\uF45F\uF460\uF461\uF462\uDBCB"+
	"\uF463\uF464\uD8AA\uF465\uF466\uF467\uF468\uF469"+
	"\uF46A\uF46B\uF46C\uE5F7\uB9ED\uF46D\uF46E\uF46F"+
	"\uF470\uBFFD\uBBEA\uF7C9\uC6C7\uF7C8\uF471\uF7CA"+
	"\uF7CC\uF7CB\uF472\uF473\uF474\uF7CD\uF475\uCEBA"+
	"\uF476\uF7CE\uF477\uF478\uC4A7\uF479\uF47A\uF47B"+
	"\uF47C\uF47D\uF47E\uF480\uF481\uF482\uF483\uF484"+
	"\uF485\uF486\uF487\uF488\uF489\uF48A\uF48B\uF48C"+
	"\uF48D\uF48E\uF48F\uF490\uF491\uF492\uF493\uF494"+
	"\uF495\uF496\uF497\uF498\uF499\uF49A\uF49B\uF49C"+
	"\uF49D\uF49E\uF49F\uF4A0\uF540\uF541\uF542\uF543"+
	"\uF544\uF545\uF546\uF547\uF548\uF549\uF54A\uF54B"+
	"\uF54C\uF54D\uF54E\uF54F\uF550\uF551\uF552\uF553"+
	"\uF554\uF555\uF556\uF557\uF558\uF559\uF55A\uF55B"+
	"\uF55C\uF55D\uF55E\uF55F\uF560\uF561\uF562\uF563"+
	"\uF564\uF565\uF566\uF567\uF568\uF569\uF56A\uF56B"+
	"\uF56C\uF56D\uF56E\uF56F\uF570\uF571\uF572\uF573"+
	"\uF574\uF575\uF576\uF577\uF578\uF579\uF57A\uF57B"+
	"\uF57C\uF57D\uF57E\uF580\uF581\uF582\uF583\uF584"+
	"\uF585\uF586\uF587\uF588\uF589\uF58A\uF58B\uF58C"+
	"\uF58D\uF58E\uF58F\uF590\uF591\uF592\uF593\uF594"+
	"\uF595\uF596\uF597\uF598\uF599\uF59A\uF59B\uF59C"+
	"\uF59D\uF59E\uF59F\uF5A0\uF640\uF641\uF642\uF643"+
	"\uF644\uF645\uF646\uF647\uF648\uF649\uF64A\uF64B"+
	"\uF64C\uF64D\uF64E\uF64F\uF650\uF651\uF652\uF653"+
	"\uF654\uF655\uF656\uF657\uF658\uF659\uF65A\uF65B"+
	"\uF65C\uF65D\uF65E\uF65F\uF660\uF661\uF662\uF663"+
	"\uF664\uF665\uF666\uF667\uF668\uF669\uF66A\uF66B"+
	"\uF66C\uF66D\uF66E\uF66F\uF670\uF671\uF672\uF673"+
	"\uF674\uF675\uF676\uF677\uF678\uF679\uF67A\uF67B"+
	"\uF67C\uF67D\uF67E\uF680\uF681\uF682\uF683\uF684"+
	"\uF685\uF686\uF687\uF688\uF689\uF68A\uF68B\uF68C"+
	"\uF68D\uF68E\uF68F\uF690\uF691\uF692\uF693\uF694"+
	"\uF695\uF696\uF697\uF698\uF699\uF69A\uF69B\uF69C"+
	"\uF69D\uF69E\uF69F\uF6A0\uF740\uF741\uF742\uF743"+
	"\uF744\uF745\uF746\uF747\uF748\uF749\uF74A\uF74B"+
	"\uF74C\uF74D\uF74E\uF74F\uF750\uF751\uF752\uF753"+
	"\uF754\uF755\uF756\uF757\uF758\uF759\uF75A\uF75B"+
	"\uF75C\uF75D\uF75E\uF75F\uF760\uF761\uF762\uF763"+
	"\uF764\uF765\uF766\uF767\uF768\uF769\uF76A\uF76B"+
	"\uF76C\uF76D\uF76E\uF76F\uF770\uF771\uF772\uF773"+
	"\uF774\uF775\uF776\uF777\uF778\uF779\uF77A\uF77B"+
	"\uF77C\uF77D\uF77E\uF780\uD3E3\uF781\uF782\uF6CF"+
	"\uF783\uC2B3\uF6D0\uF784\uF785\uF6D1\uF6D2\uF6D3"+
	"\uF6D4\uF786\uF787\uF6D6\uF788\uB1AB\uF6D7\uF789"+
	"\uF6D8\uF6D9\uF6DA\uF78A\uF6DB\uF6DC\uF78B\uF78C"+
	"\uF78D\uF78E\uF6DD\uF6DE\uCFCA\uF78F\uF6DF\uF6E0"+
	"\uF6E1\uF6E2\uF6E3\uF6E4\uC0F0\uF6E5\uF6E6\uF6E7"+
	"\uF6E8\uF6E9\uF790\uF6EA\uF791\uF6EB\uF6EC\uF792"+
	"\uF6ED\uF6EE\uF6EF\uF6F0\uF6F1\uF6F2\uF6F3\uF6F4"+
	"\uBEA8\uF793\uF6F5\uF6F6\uF6F7\uF6F8\uF794\uF795"+
	"\uF796\uF797\uF798\uC8FA\uF6F9\uF6FA\uF6FB\uF6FC"+
	"\uF799\uF79A\uF6FD\uF6FE\uF7A1\uF7A2\uF7A3\uF7A4"+
	"\uF7A5\uF79B\uF79C\uF7A6\uF7A7\uF7A8\uB1EE\uF7A9"+
	"\uF7AA\uF7AB\uF79D\uF79E\uF7AC\uF7AD\uC1DB\uF7AE"+
	"\uF79F\uF7A0\uF7AF\uF840\uF841\uF842\uF843\uF844"+
	"\uF845\uF846\uF847\uF848\uF849\uF84A\uF84B\uF84C"+
	"\uF84D\uF84E\uF84F\uF850\uF851\uF852\uF853\uF854"+
	"\uF855\uF856\uF857\uF858\uF859\uF85A\uF85B\uF85C"+
	"\uF85D\uF85E\uF85F\uF860\uF861\uF862\uF863\uF864"+
	"\uF865\uF866\uF867\uF868\uF869\uF86A\uF86B\uF86C"+
	"\uF86D\uF86E\uF86F\uF870\uF871\uF872\uF873\uF874"+
	"\uF875\uF876\uF877\uF878\uF879\uF87A\uF87B\uF87C"+
	"\uF87D\uF87E\uF880\uF881\uF882\uF883\uF884\uF885"+
	"\uF886\uF887\uF888\uF889\uF88A\uF88B\uF88C\uF88D"+
	"\uF88E\uF88F\uF890\uF891\uF892\uF893\uF894\uF895"+
	"\uF896\uF897\uF898\uF899\uF89A\uF89B\uF89C\uF89D"+
	"\uF89E\uF89F\uF8A0\uF940\uF941\uF942\uF943\uF944"+
	"\uF945\uF946\uF947\uF948\uF949\uF94A\uF94B\uF94C"+
	"\uF94D\uF94E\uF94F\uF950\uF951\uF952\uF953\uF954"+
	"\uF955\uF956\uF957\uF958\uF959\uF95A\uF95B\uF95C"+
	"\uF95D\uF95E\uF95F\uF960\uF961\uF962\uF963\uF964"+
	"\uF965\uF966\uF967\uF968\uF969\uF96A\uF96B\uF96C"+
	"\uF96D\uF96E\uF96F\uF970\uF971\uF972\uF973\uF974"+
	"\uF975\uF976\uF977\uF978\uF979\uF97A\uF97B\uF97C"+
	"\uF97D\uF97E\uF980\uF981\uF982\uF983\uF984\uF985"+
	"\uF986\uF987\uF988\uF989\uF98A\uF98B\uF98C\uF98D"+
	"\uF98E\uF98F\uF990\uF991\uF992\uF993\uF994\uF995"+
	"\uF996\uF997\uF998\uF999\uF99A\uF99B\uF99C\uF99D"+
	"\uF99E\uF99F\uF9A0\uFA40\uFA41\uFA42\uFA43\uFA44"+
	"\uFA45\uFA46\uFA47\uFA48\uFA49\uFA4A\uFA4B\uFA4C"+
	"\uFA4D\uFA4E\uFA4F\uFA50\uFA51\uFA52\uFA53\uFA54"+
	"\uFA55\uFA56\uFA57\uFA58\uFA59\uFA5A\uFA5B\uFA5C"+
	"\uFA5D\uFA5E\uFA5F\uFA60\uFA61\uFA62\uFA63\uFA64"+
	"\uFA65\uFA66\uFA67\uFA68\uFA69\uFA6A\uFA6B\uFA6C"+
	"\uFA6D\uFA6E\uFA6F\uFA70\uFA71\uFA72\uFA73\uFA74"+
	"\uFA75\uFA76\uFA77\uFA78\uFA79\uFA7A\uFA7B\uFA7C"+
	"\uFA7D\uFA7E\uFA80\uFA81\uFA82\uFA83\uFA84\uFA85"+
	"\uFA86\uFA87\uFA88\uFA89\uFA8A\uFA8B\uFA8C\uFA8D"+
	"\uFA8E\uFA8F\uFA90\uFA91\uFA92\uFA93\uFA94\uFA95"+
	"\uFA96\uFA97\uFA98\uFA99\uFA9A\uFA9B\uFA9C\uFA9D"+
	"\uFA9E\uFA9F\uFAA0\uFB40\uFB41\uFB42\uFB43\uFB44"+
	"\uFB45\uFB46\uFB47\uFB48\uFB49\uFB4A\uFB4B\uFB4C"+
	"\uFB4D\uFB4E\uFB4F\uFB50\uFB51\uFB52\uFB53\uFB54"+
	"\uFB55\uFB56\uFB57\uFB58\uFB59\uFB5A\uFB5B\uC4F1"+
	"\uF0AF\uBCA6\uF0B0\uC3F9\uFB5C\uC5B8\uD1BB\uFB5D"+
	"\uF0B1\uF0B2\uF0B3\uF0B4\uF0B5\uD1BC\uFB5E\uD1EC"+
	"\uFB5F\uF0B7\uF0B6\uD4A7\uFB60\uCDD2\uF0B8\uF0BA"+
	"\uF0B9\uF0BB\uF0BC\uFB61\uFB62\uB8EB\uF0BD\uBAE8"+
	"\uFB63\uF0BE\uF0BF\uBEE9\uF0C0\uB6EC\uF0C1\uF0C2"+
	"\uF0C3\uF0C4\uC8B5\uF0C5\uF0C6\uFB64\uF0C7\uC5F4"+
	"\uFB65\uF0C8\uFB66\uFB67\uFB68\uF0C9\uFB69\uF0CA"+
	"\uF7BD\uFB6A\uF0CB\uF0CC\uF0CD\uFB6B\uF0CE\uFB6C"+
	"\uFB6D\uFB6E\uFB6F\uF0CF\uBAD7\uFB70\uF0D0\uF0D1"+
	"\uF0D2\uF0D3\uF0D4\uF0D5\uF0D6\uF0D8\uFB71\uFB72"+
	"\uD3A5\uF0D7\uFB73\uF0D9\uFB74\uFB75\uFB76\uFB77"+
	"\uFB78\uFB79\uFB7A\uFB7B\uFB7C\uFB7D\uF5BA\uC2B9"+
	"\uFB7E\uFB80\uF7E4\uFB81\uFB82\uFB83\uFB84\uF7E5"+
	"\uF7E6\uFB85\uFB86\uF7E7\uFB87\uFB88\uFB89\uFB8A"+
	"\uFB8B\uFB8C\uF7E8\uC2B4\uFB8D\uFB8E\uFB8F\uFB90"+
	"\uFB91\uFB92\uFB93\uFB94\uFB95\uF7EA\uFB96\uF7EB"+
	"\uFB97\uFB98\uFB99\uFB9A\uFB9B\uFB9C\uC2F3\uFB9D"+
	"\uFB9E\uFB9F\uFBA0\uFC40\uFC41\uFC42\uFC43\uFC44"+
	"\uFC45\uFC46\uFC47\uFC48\uF4F0\uFC49\uFC4A\uFC4B"+
	"\uF4EF\uFC4C\uFC4D\uC2E9\uFC4E\uF7E1\uF7E2\uFC4F"+
	"\uFC50\uFC51\uFC52\uFC53\uBBC6\uFC54\uFC55\uFC56"+
	"\uFC57\uD9E4\uFC58\uFC59\uFC5A\uCAF2\uC0E8\uF0A4"+
	"\uFC5B\uBADA\uFC5C\uFC5D\uC7AD\uFC5E\uFC5F\uFC60"+
	"\uC4AC\uFC61\uFC62\uF7EC\uF7ED\uF7EE\uFC63\uF7F0"+
	"\uF7EF\uFC64\uF7F1\uFC65\uFC66\uF7F4\uFC67\uF7F3"+
	"\uFC68\uF7F2\uF7F5\uFC69\uFC6A\uFC6B\uFC6C\uF7F6"+
	"\uFC6D\uFC6E\uFC6F\uFC70\uFC71\uFC72\uFC73\uFC74"+
	"\uFC75\uEDE9\uFC76\uEDEA\uEDEB\uFC77\uF6BC\uFC78"+
	"\uFC79\uFC7A\uFC7B\uFC7C\uFC7D\uFC7E\uFC80\uFC81"+
	"\uFC82\uFC83\uFC84\uF6BD\uFC85\uF6BE\uB6A6\uFC86"+
	"\uD8BE\uFC87\uFC88\uB9C4\uFC89\uFC8A\uFC8B\uD8BB"+
	"\uFC8C\uDCB1\uFC8D\uFC8E\uFC8F\uFC90\uFC91\uFC92"+
	"\uCAF3\uFC93\uF7F7\uFC94\uFC95\uFC96\uFC97\uFC98"+
	"\uFC99\uFC9A\uFC9B\uFC9C\uF7F8\uFC9D\uFC9E\uF7F9"+
	"\uFC9F\uFCA0\uFD40\uFD41\uFD42\uFD43\uFD44\uF7FB"+
	"\uFD45\uF7FA\uFD46\uB1C7\uFD47\uF7FC\uF7FD\uFD48"+
	"\uFD49\uFD4A\uFD4B\uFD4C\uF7FE\uFD4D\uFD4E\uFD4F"+
	"\uFD50\uFD51\uFD52\uFD53\uFD54\uFD55\uFD56\uFD57"+
	"\uC6EB\uECB4\uFD58\uFD59\uFD5A\uFD5B\uFD5C\uFD5D"+
	"\uFD5E\uFD5F\uFD60\uFD61\uFD62\uFD63\uFD64\uFD65"+
	"\uFD66\uFD67\uFD68\uFD69\uFD6A\uFD6B\uFD6C\uFD6D"+
	"\uFD6E\uFD6F\uFD70\uFD71\uFD72\uFD73\uFD74\uFD75"+
	"\uFD76\uFD77\uFD78\uFD79\uFD7A\uFD7B\uFD7C\uFD7D"+
	"\uFD7E\uFD80\uFD81\uFD82\uFD83\uFD84\uFD85\uB3DD"+
	"\uF6B3\uFD86\uFD87\uF6B4\uC1E4\uF6B5\uF6B6\uF6B7"+
	"\uF6B8\uF6B9\uF6BA\uC8A3\uF6BB\uFD88\uFD89\uFD8A"+
	"\uFD8B\uFD8C\uFD8D\uFD8E\uFD8F\uFD90\uFD91\uFD92"+
	"\uFD93\uC1FA\uB9A8\uEDE8\uFD94\uFD95\uFD96\uB9EA"+
	"\uD9DF\uFD97\uFD98\uFD99\uFD9A\uFD9B\u6A63\u6A64"+
	"\u6A65\u6A66\u6A67\u6A68\u6A69\u6A6A\u6A6B\u6A6C"+
	"\u6A6D\u6A6E\u6A6F\u6A70\u6A71\u6A72\u6A73\u6A74"+
	"\u6A75\u6A76\u6A77\u6A78\u6A79\u6A7A\u6A7B\u6A7C"+
	"\u6A7D\u6A7E\u6A7F\u6A80\u6A81\u6A82\u6A83\u6A84"+
	"\u6A85\u6A86\u6A87\u6A88\u6A89\u6A8A\u6A8B\u6A8C"+
	"\u6A8D\u6A8E\u6A8F\u6A90\u6A91\u6A92\u6A93\u6A94"+
	"\u6A95\u6A96\u6A97\u6A98\u6A99\u6A9A\u6A9B\u6A9C"+
	"\u6A9D\u6A9E\u6A9F\u6AA0\u6AA1\u6AA2\u6AA3\u6AA4"+
	"\u6AA5\u6AA6\u6AA7\u6AA8\u6AA9\u6AAA\u6AAB\u6AAC"+
	"\u6AAD\u6AAE\u6AAF\u6AB0\u6AB1\u6AB2\u6AB3\u6AB4"+
	"\u6AB5\u6AB6\u6AB7\u6AB8\u6AB9\u6ABA\u6ABB\u6ABC";

    private final static String innerIndex10= 
	"\u6ABD\u6ABE\u6ABF\u6AC0\u6AC1\u6AC2\u6AC3\u6AC4"+
	"\u6AC5\u6AC6\u6AC7\u6AC8\u6AC9\u6ACA\u6ACB\u6ACC"+
	"\u6ACD\u6ACE\u6ACF\u6AD0\u6AD1\u6AD2\u6AD3\u6AD4"+
	"\u6AD5\u6AD6\u6AD7\u6AD8\u6AD9\u6ADA\u6ADB\u6ADC"+
	"\u6ADD\u6ADE\u6ADF\u6AE0\u6AE1\u6AE2\u6AE3\u6AE4"+
	"\u6AE5\u6AE6\u6AE7\u6AE8\u6AE9\u6AEA\u6AEB\u6AEC"+
	"\u6AED\u6AEE\u6AEF\u6AF0\u6AF1\u6AF2\u6AF3\u6AF4"+
	"\u6AF5\u6AF6\u6AF7\u6AF8\u6AF9\u6AFA\u6AFB\u6AFC"+
	"\u6AFD\u6AFE\u6AFF\u6B00\u6B01\u6B02\u6B03\u6B04"+
	"\u6B05\u6B06\u6B07\u6B08\u6B09\u6B0A\u6B0B\u6B0C"+
	"\u6B0D\u6B0E\u6B0F\u6B10\u6B11\u6B12\u6B13\u6B14"+
	"\u6B15\u6B16\u6B17\u6B18\u6B19\u6B1A\u6B1B\u6B1C"+
	"\u6B1D\u6B1E\u6B1F\u6B20\u6B21\u6B22\u6B23\u6B24"+
	"\u6B25\u6B26\u6B27\u6B28\u6B29\u6B2A\u6B2B\u6B2C"+
	"\u6B2D\u6B2E\u6B2F\u6B30\u6B31\u6B32\u6B33\u6B34"+
	"\u6B35\u6B36\u6B37\u6B38\u6B39\u6B3A\u6B3B\u6B3C"+
	"\u6B3D\u6B3E\u6B3F\u6B40\u6B41\u6B42\u6B43\u6B44"+
	"\u6B45\u6B46\u6B47\u6B48\u6B49\u6B4A\u6B4B\u6B4C"+
	"\u6B4D\u6B4E\u6B4F\u6B50\u6B51\u6B52\u6B53\u6B54"+
	"\u6B55\u6B56\u6B57\u6B58\u6B59\u6B5A\u6B5B\u6B5C"+
	"\u6B5D\u6B5E\u6B5F\u6B60\u6B61\u6B62\u6B63\u6B64"+
	"\u6B65\u6B66\u6B67\u6B68\u6B69\u6B6A\u6B6B\u6B6C"+
	"\u6B6D\u6B6E\u6B6F\u6B70\u6B71\u6B72\u6B73\u6B74"+
	"\u6B75\u6B76\u6B77\u6B78\u6B79\u6B7A\u6B7B\u6B7C"+
	"\u6B7D\u6B7E\u6B7F\u6B80\u6B81\u6B82\u6B83\u6B84"+
	"\u6B85\u6B86\u6B87\u6B88\u6B89\u6B8A\u6B8B\u6B8C"+
	"\u6B8D\u6B8E\u6B8F\u6B90\u6B91\u6B92\u6B93\u6B94"+
	"\u6B95\u6B96\u6B97\u6B98\u6B99\u6B9A\u6B9B\u6B9C"+
	"\u6B9D\u6B9E\u6B9F\u6BA0\u6BA1\u6BA2\u6BA3\u6BA4"+
	"\u6BA5\u6BA6\u6BA7\u6BA8\u6BA9\u6BAA\u6BAB\u6BAC"+
	"\u6BAD\u6BAE\u6BAF\u6BB0\u6BB1\u6BB2\u6BB3\u6BB4"+
	"\u6BB5\u6BB6\u6BB7\u6BB8\u6BB9\u6BBA\u6BBB\u6BBC"+
	"\u6BBD\u6BBE\u6BBF\u6BC0\u6BC1\u6BC2\u6BC3\u6BC4"+
	"\u6BC5\u6BC6\u6BC7\u6BC8\u6BC9\u6BCA\u6BCB\u6BCC"+
	"\u6BCD\u6BCE\u6BCF\u6BD0\u6BD1\u6BD2\u6BD3\u6BD4"+
	"\u6BD5\u6BD6\u6BD7\u6BD8\u6BD9\u6BDA\u6BDB\u6BDC"+
	"\u6BDD\u6BDE\u6BDF\u6BE0\u6BE1\u6BE2\u6BE3\u6BE4"+
	"\u6BE5\u6BE6\u6BE7\u6BE8\u6BE9\u6BEA\u6BEB\u6BEC"+
	"\u6BED\u6BEE\u6BEF\u6BF0\u6BF1\u6BF2\u6BF3\u6BF4"+
	"\u6BF5\u6BF6\u6BF7\u6BF8\u6BF9\u6BFA\u6BFB\u6BFC"+
	"\u6BFD\u6BFE\u6BFF\u6C00\u6C01\u6C02\u6C03\u6C04"+
	"\u6C05\u6C06\u6C07\u6C08\u6C09\u6C0A\u6C0B\u6C0C"+
	"\u6C0D\u6C0E\u6C0F\u6C10\u6C11\u6C12\u6C13\u6C14"+
	"\u6C15\u6C16\u6C17\u6C18\u6C19\u6C1A\u6C1B\u6C1C"+
	"\u6C1D\u6C1E\u6C1F\u6C20\u6C21\u6C22\u6C23\u6C24"+
	"\u6C25\u6C26\u6C27\u6C28\u6C29\u6C2A\u6C2B\u6C2C"+
	"\u6C2D\u6C2E\u6C2F\u6C30\u6C31\u6C32\u6C33\u6C34"+
	"\u6C35\u6C36\u6C37\u6C38\u6C39\u6C3A\u6C3B\u6C3C"+
	"\u6C3D\u6C3E\u6C3F\u6C40\u6C41\u6C42\u6C43\u6C44"+
	"\u6C45\u6C46\u6C47\u6C48\u6C49\u6C4A\u6C4B\u6C4C"+
	"\u6C4D\u6C4E\u6C4F\u6C50\u6C51\u6C52\u6C53\u6C54"+
	"\u6C55\u6C56\u6C57\u6C58\u6C59\u6C5A\u6C5B\u6C5C"+
	"\u6C5D\u6C5E\u6C5F\u6C60\u6C61\u6C62\u6C63\u6C64"+
	"\u6C65\u6C66\u6C67\u6C68\u6C69\u6C6A\u6C6B\u6C6C"+
	"\u6C6D\u6C6E\u6C6F\u6C70\u6C71\u6C72\u6C73\u6C74"+
	"\u6C75\u6C76\u6C77\u6C78\u6C79\u6C7A\u6C7B\u6C7C"+
	"\u6C7D\u6C7E\u6C7F\u6C80\u6C81\u6C82\u6C83\u6C84"+
	"\u6C85\u6C86\u6C87\u6C88\u6C89\u6C8A\u6C8B\u6C8C"+
	"\u6C8D\u6C8E\u6C8F\u6C90\u6C91\u6C92\u6C93\u6C94"+
	"\u6C95\u6C96\u6C97\u6C98\u6C99\u6C9A\u6C9B\u6C9C"+
	"\u6C9D\u6C9E\u6C9F\u6CA0\u6CA1\u6CA2\u6CA3\u6CA4"+
	"\u6CA5\u6CA6\u6CA7\u6CA8\u6CA9\u6CAA\u6CAB\u6CAC"+
	"\u6CAD\u6CAE\u6CAF\u6CB0\u6CB1\u6CB2\u6CB3\u6CB4"+
	"\u6CB5\u6CB6\u6CB7\u6CB8\u6CB9\u6CBA\u6CBB\u6CBC"+
	"\u6CBD\u6CBE\u6CBF\u6CC0\u6CC1\u6CC2\u6CC3\u6CC4"+
	"\u6CC5\u6CC6\u6CC7\u6CC8\u6CC9\u6CCA\u6CCB\u6CCC"+
	"\u6CCD\u6CCE\u6CCF\u6CD0\u6CD1\u6CD2\u6CD3\u6CD4"+
	"\u6CD5\u6CD6\u6CD7\u6CD8\u6CD9\u6CDA\u6CDB\u6CDC"+
	"\u6CDD\u6CDE\u6CDF\u6CE0\u6CE1\u6CE2\u6CE3\u6CE4"+
	"\u6CE5\u6CE6\u6CE7\u6CE8\u6CE9\u6CEA\u6CEB\u6CEC"+
	"\u6CED\u6CEE\u6CEF\u6CF0\u6CF1\u6CF2\u6CF3\u6CF4"+
	"\u6CF5\u6CF6\u6CF7\u6CF8\u6CF9\u6CFA\u6CFB\u6CFC"+
	"\u6CFD\u6CFE\u6CFF\u6D00\u6D01\u6D02\u6D03\u6D04"+
	"\u6D05\u6D06\u6D07\u6D08\u6D09\u6D0A\u6D0B\u6D0C"+
	"\u6D0D\u6D0E\u6D0F\u6D10\u6D11\u6D12\u6D13\u6D14"+
	"\u6D15\u6D16\u6D17\u6D18\u6D19\u6D1A\u6D1B\u6D1C"+
	"\u6D1D\u6D1E\u6D1F\u6D20\u6D21\u6D22\u6D23\u6D24"+
	"\u6D25\u6D26\u6D27\u6D28\u6D29\u6D2A\u6D2B\u6D2C"+
	"\u6D2D\u6D2E\u6D2F\u6D30\u6D31\u6D32\u6D33\u6D34"+
	"\u6D35\u6D36\u6D37\u6D38\u6D39\u6D3A\u6D3B\u6D3C"+
	"\u6D3D\u6D3E\u6D3F\u6D40\u6D41\u6D42\u6D43\u6D44"+
	"\u6D45\u6D46\u6D47\u6D48\u6D49\u6D4A\u6D4B\u6D4C"+
	"\u6D4D\u6D4E\u6D4F\u6D50\u6D51\u6D52\u6D53\u6D54"+
	"\u6D55\u6D56\u6D57\u6D58\u6D59\u6D5A\u6D5B\u6D5C"+
	"\u6D5D\u6D5E\u6D5F\u6D60\u6D61\u6D62\u6D63\u6D64"+
	"\u6D65\u6D66\u6D67\u6D68\u6D69\u6D6A\u6D6B\u6D6C"+
	"\u6D6D\u6D6E\u6D6F\u6D70\u6D71\u6D72\u6D73\u6D74"+
	"\u6D75\u6D76\u6D77\u6D78\u6D79\u6D7A\u6D7B\u6D7C"+
	"\u6D7D\u6D7E\u6D7F\u6D80\u6D81\u6D82\u6D83\u6D84"+
	"\u6D85\u6D86\u6D87\u6D88\u6D89\u6D8A\u6D8B\u6D8C"+
	"\u6D8D\u6D8E\u6D8F\u6D90\u6D91\u6D92\u6D93\u6D94"+
	"\u6D95\u6D96\u6D97\u6D98\u6D99\u6D9A\u6D9B\u6D9C"+
	"\u6D9D\u6D9E\u6D9F\u6DA0\u6DA1\u6DA2\u6DA3\u6DA4"+
	"\u6DA5\u6DA6\u6DA7\u6DA8\u6DA9\u6DAA\u6DAB\u6DAC"+
	"\u6DAD\u6DAE\u6DAF\u6DB0\u6DB1\u6DB2\u6DB3\u6DB4"+
	"\u6DB5\u6DB6\u6DB7\u6DB8\u6DB9\u6DBA\u6DBB\u6DBC"+
	"\u6DBD\u6DBE\u6DBF\u6DC0\u6DC1\u6DC2\u6DC3\u6DC4"+
	"\u6DC5\u6DC6\u6DC7\u6DC8\u6DC9\u6DCA\u6DCB\u6DCC"+
	"\u6DCD\u6DCE\u6DCF\u6DD0\u6DD1\u6DD2\u6DD3\u6DD4"+
	"\u6DD5\u6DD6\u6DD7\u6DD8\u6DD9\u6DDA\u6DDB\u6DDC"+
	"\u6DDD\u6DDE\u6DDF\u6DE0\u6DE1\u6DE2\u6DE3\u6DE4"+
	"\u6DE5\u6DE6\u6DE7\u6DE8\u6DE9\u6DEA\u6DEB\u6DEC"+
	"\u6DED\u6DEE\u6DEF\u6DF0\u6DF1\u6DF2\u6DF3\u6DF4"+
	"\u6DF5\u6DF6\u6DF7\u6DF8\u6DF9\u6DFA\u6DFB\u6DFC"+
	"\u6DFD\u6DFE\u6DFF\u6E00\u6E01\u6E02\u6E03\u6E04"+
	"\u6E05\u6E06\u6E07\u6E08\u6E09\u6E0A\u6E0B\u6E0C"+
	"\u6E0D\u6E0E\u6E0F\u6E10\u6E11\u6E12\u6E13\u6E14"+
	"\u6E15\u6E16\u6E17\u6E18\u6E19\u6E1A\u6E1B\u6E1C"+
	"\u6E1D\u6E1E\u6E1F\u6E20\u6E21\u6E22\u6E23\u6E24"+
	"\u6E25\u6E26\u6E27\u6E28\u6E29\u6E2A\u6E2B\u6E2C"+
	"\u6E2D\u6E2E\u6E2F\u6E30\u6E31\u6E32\u6E33\u6E34"+
	"\u6E35\u6E36\u6E37\u6E38\u6E39\u6E3A\u6E3B\u6E3C"+
	"\u6E3D\u6E3E\u6E3F\u6E40\u6E41\u6E42\u6E43\u6E44"+
	"\u6E45\u6E46\u6E47\u6E48\u6E49\u6E4A\u6E4B\u6E4C"+
	"\u6E4D\u6E4E\u6E4F\u6E50\u6E51\u6E52\u6E53\u6E54"+
	"\u6E55\u6E56\u6E57\u6E58\u6E59\u6E5A\u6E5B\u6E5C"+
	"\u6E5D\u6E5E\u6E5F\u6E60\u6E61\u6E62\u6E63\u6E64"+
	"\u6E65\u6E66\u6E67\u6E68\u6E69\u6E6A\u6E6B\u6E6C"+
	"\u6E6D\u6E6E\u6E6F\u6E70\u6E71\u6E72\u6E73\u6E74"+
	"\u6E75\u6E76\u6E77\u6E78\u6E79\u6E7A\u6E7B\u6E7C"+
	"\u6E7D\u6E7E\u6E7F\u6E80\u6E81\u6E82\u6E83\u6E84"+
	"\u6E85\u6E86\u6E87\u6E88\u6E89\u6E8A\u6E8B\u6E8C"+
	"\u6E8D\u6E8E\u6E8F\u6E90\u6E91\u6E92\u6E93\u6E94"+
	"\u6E95\u6E96\u6E97\u6E98\u6E99\u6E9A\u6E9B\u6E9C"+
	"\u6E9D\u6E9E\u6E9F\u6EA0\u6EA1\u6EA2\u6EA3\u6EA4"+
	"\u6EA5\u6EA6\u6EA7\u6EA8\u6EA9\u6EAA\u6EAB\u6EAC"+
	"\u6EAD\u6EAE\u6EAF\u6EB0\u6EB1\u6EB2\u6EB3\u6EB4"+
	"\u6EB5\u6EB6\u6EB7\u6EB8\u6EB9\u6EBA\u6EBB\u6EBC"+
	"\u6EBD\u6EBE\u6EBF\u6EC0\u6EC1\u6EC2\u6EC3\u6EC4"+
	"\u6EC5\u6EC6\u6EC7\u6EC8\u6EC9\u6ECA\u6ECB\u6ECC"+
	"\u6ECD\u6ECE\u6ECF\u6ED0\u6ED1\u6ED2\u6ED3\u6ED4"+
	"\u6ED5\u6ED6\u6ED7\u6ED8\u6ED9\u6EDA\u6EDB\u6EDC"+
	"\u6EDD\u6EDE\u6EDF\u6EE0\u6EE1\u6EE2\u6EE3\u6EE4"+
	"\u6EE5\u6EE6\u6EE7\u6EE8\u6EE9\u6EEA\u6EEB\u6EEC"+
	"\u6EED\u6EEE\u6EEF\u6EF0\u6EF1\u6EF2\u6EF3\u6EF4"+
	"\u6EF5\u6EF6\u6EF7\u6EF8\u6EF9\u6EFA\u6EFB\u6EFC"+
	"\u6EFD\u6EFE\u6EFF\u6F00\u6F01\u6F02\u6F03\u6F04"+
	"\u6F05\u6F06\u6F07\u6F08\u6F09\u6F0A\u6F0B\u6F0C"+
	"\u6F0D\u6F0E\u6F0F\u6F10\u6F11\u6F12\u6F13\u6F14"+
	"\u6F15\u6F16\u6F17\u6F18\u6F19\u6F1A\u6F1B\u6F1C"+
	"\u6F1D\u6F1E\u6F1F\u6F20\u6F21\u6F22\u6F23\u6F24"+
	"\u6F25\u6F26\u6F27\u6F28\u6F29\u6F2A\u6F2B\u6F2C"+
	"\u6F2D\u6F2E\u6F2F\u6F30\u6F31\u6F32\u6F33\u6F34"+
	"\u6F35\u6F36\u6F37\u6F38\u6F39\u6F3A\u6F3B\u6F3C"+
	"\u6F3D\u6F3E\u6F3F\u6F40\u6F41\u6F42\u6F43\u6F44"+
	"\u6F45\u6F46\u6F47\u6F48\u6F49\u6F4A\u6F4B\u6F4C"+
	"\u6F4D\u6F4E\u6F4F\u6F50\u6F51\u6F52\u6F53\u6F54"+
	"\u6F55\u6F56\u6F57\u6F58\u6F59\u6F5A\u6F5B\u6F5C"+
	"\u6F5D\u6F5E\u6F5F\u6F60\u6F61\u6F62\u6F63\u6F64"+
	"\u6F65\u6F66\u6F67\u6F68\u6F69\u6F6A\u6F6B\u6F6C"+
	"\u6F6D\u6F6E\u6F6F\u6F70\u6F71\u6F72\u6F73\u6F74"+
	"\u6F75\u6F76\u6F77\u6F78\u6F79\u6F7A\u6F7B\u6F7C"+
	"\u6F7D\u6F7E\u6F7F\u6F80\u6F81\u6F82\u6F83\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD\uFFFD"+
	"\uAAA1\uAAA2\uAAA3\uAAA4\uAAA5\uAAA6\uAAA7\uAAA8"+
	"\uAAA9\uAAAA\uAAAB\uAAAC\uAAAD\uAAAE\uAAAF\uAAB0"+
	"\uAAB1\uAAB2\uAAB3\uAAB4\uAAB5\uAAB6\uAAB7\uAAB8"+
	"\uAAB9\uAABA\uAABB\uAABC\uAABD\uAABE\uAABF\uAAC0"+
	"\uAAC1\uAAC2\uAAC3\uAAC4\uAAC5\uAAC6\uAAC7\uAAC8"+
	"\uAAC9\uAACA\uAACB\uAACC\uAACD\uAACE\uAACF\uAAD0"+
	"\uAAD1\uAAD2\uAAD3\uAAD4\uAAD5\uAAD6\uAAD7\uAAD8"+
	"\uAAD9\uAADA\uAADB\uAADC\uAADD\uAADE\uAADF\uAAE0"+
	"\uAAE1\uAAE2\uAAE3\uAAE4\uAAE5\uAAE6\uAAE7\uAAE8"+
	"\uAAE9\uAAEA\uAAEB\uAAEC\uAAED\uAAEE\uAAEF\uAAF0"+
	"\uAAF1\uAAF2\uAAF3\uAAF4\uAAF5\uAAF6\uAAF7\uAAF8"+
	"\uAAF9\uAAFA\uAAFB\uAAFC\uAAFD\uAAFE\uABA1\uABA2"+
	"\uABA3\uABA4\uABA5\uABA6\uABA7\uABA8\uABA9\uABAA"+
	"\uABAB\uABAC\uABAD\uABAE\uABAF\uABB0\uABB1\uABB2"+
	"\uABB3\uABB4\uABB5\uABB6\uABB7\uABB8\uABB9\uABBA"+
	"\uABBB\uABBC\uABBD\uABBE\uABBF\uABC0\uABC1\uABC2"+
	"\uABC3\uABC4\uABC5\uABC6\uABC7\uABC8\uABC9\uABCA"+
	"\uABCB\uABCC\uABCD\uABCE\uABCF\uABD0\uABD1\uABD2"+
	"\uABD3\uABD4\uABD5\uABD6\uABD7\uABD8\uABD9\uABDA"+
	"\uABDB\uABDC\uABDD\uABDE\uABDF\uABE0\uABE1\uABE2"+
	"\uABE3\uABE4\uABE5\uABE6\uABE7\uABE8\uABE9\uABEA"+
	"\uABEB\uABEC\uABED\uABEE\uABEF\uABF0\uABF1\uABF2"+
	"\uABF3\uABF4\uABF5\uABF6\uABF7\uABF8\uABF9\uABFA"+
	"\uABFB\uABFC\uABFD\uABFE\uACA1\uACA2\uACA3\uACA4"+
	"\uACA5\uACA6\uACA7\uACA8\uACA9\uACAA\uACAB\uACAC"+
	"\uACAD\uACAE\uACAF\uACB0\uACB1\uACB2\uACB3\uACB4"+
	"\uACB5\uACB6\uACB7\uACB8\uACB9\uACBA\uACBB\uACBC"+
	"\uACBD\uACBE\uACBF\uACC0\uACC1\uACC2\uACC3\uACC4"+
	"\uACC5\uACC6\uACC7\uACC8\uACC9\uACCA\uACCB\uACCC"+
	"\uACCD\uACCE\uACCF\uACD0\uACD1\uACD2\uACD3\uACD4"+
	"\uACD5\uACD6\uACD7\uACD8\uACD9\uACDA\uACDB\uACDC"+
	"\uACDD\uACDE\uACDF\uACE0\uACE1\uACE2\uACE3\uACE4"+
	"\uACE5\uACE6\uACE7\uACE8\uACE9\uACEA\uACEB\uACEC"+
	"\uACED\uACEE\uACEF\uACF0\uACF1\uACF2\uACF3\uACF4"+
	"\uACF5\uACF6\uACF7\uACF8\uACF9\uACFA\uACFB\uACFC"+
	"\uACFD\uACFE\uADA1\uADA2\uADA3\uADA4\uADA5\uADA6"+
	"\uADA7\uADA8\uADA9\uADAA\uADAB\uADAC\uADAD\uADAE"+
	"\uADAF\uADB0\uADB1\uADB2\uADB3\uADB4\uADB5\uADB6"+
	"\uADB7\uADB8\uADB9\uADBA\uADBB\uADBC\uADBD\uADBE"+
	"\uADBF\uADC0\uADC1\uADC2\uADC3\uADC4\uADC5\uADC6"+
	"\uADC7\uADC8\uADC9\uADCA\uADCB\uADCC\uADCD\uADCE"+
	"\uADCF\uADD0\uADD1\uADD2\uADD3\uADD4\uADD5\uADD6"+
	"\uADD7\uADD8\uADD9\uADDA\uADDB\uADDC\uADDD\uADDE"+
	"\uADDF\uADE0\uADE1\uADE2\uADE3\uADE4\uADE5\uADE6"+
	"\uADE7\uADE8\uADE9\uADEA\uADEB\uADEC\uADED\uADEE"+
	"\uADEF\uADF0\uADF1\uADF2\uADF3\uADF4\uADF5\uADF6"+
	"\uADF7\uADF8\uADF9\uADFA\uADFB\uADFC\uADFD\uADFE"+
	"\uAEA1\uAEA2\uAEA3\uAEA4\uAEA5\uAEA6\uAEA7\uAEA8"+
	"\uAEA9\uAEAA\uAEAB\uAEAC\uAEAD\uAEAE\uAEAF\uAEB0"+
	"\uAEB1\uAEB2\uAEB3\uAEB4\uAEB5\uAEB6\uAEB7\uAEB8"+
	"\uAEB9\uAEBA\uAEBB\uAEBC\uAEBD\uAEBE\uAEBF\uAEC0"+
	"\uAEC1\uAEC2\uAEC3\uAEC4\uAEC5\uAEC6\uAEC7\uAEC8"+
	"\uAEC9\uAECA\uAECB\uAECC\uAECD\uAECE\uAECF\uAED0"+
	"\uAED1\uAED2\uAED3\uAED4\uAED5\uAED6\uAED7\uAED8"+
	"\uAED9\uAEDA\uAEDB\uAEDC\uAEDD\uAEDE\uAEDF\uAEE0"+
	"\uAEE1\uAEE2\uAEE3\uAEE4\uAEE5\uAEE6\uAEE7\uAEE8"+
	"\uAEE9\uAEEA\uAEEB\uAEEC\uAEED\uAEEE\uAEEF\uAEF0"+
	"\uAEF1\uAEF2\uAEF3\uAEF4\uAEF5\uAEF6\uAEF7\uAEF8"+
	"\uAEF9\uAEFA\uAEFB\uAEFC\uAEFD\uAEFE\uAFA1\uAFA2"+
	"\uAFA3\uAFA4\uAFA5\uAFA6\uAFA7\uAFA8\uAFA9\uAFAA"+
	"\uAFAB\uAFAC\uAFAD\uAFAE\uAFAF\uAFB0\uAFB1\uAFB2"+
	"\uAFB3\uAFB4\uAFB5\uAFB6\uAFB7\uAFB8\uAFB9\uAFBA"+
	"\uAFBB\uAFBC\uAFBD\uAFBE\uAFBF\uAFC0\uAFC1\uAFC2"+
	"\uAFC3\uAFC4\uAFC5\uAFC6\uAFC7\uAFC8\uAFC9\uAFCA"+
	"\uAFCB\uAFCC\uAFCD\uAFCE\uAFCF\uAFD0\uAFD1\uAFD2"+
	"\uAFD3\uAFD4\uAFD5\uAFD6\uAFD7\uAFD8\uAFD9\uAFDA"+
	"\uAFDB\uAFDC\uAFDD\uAFDE\uAFDF\uAFE0\uAFE1\uAFE2"+
	"\uAFE3\uAFE4\uAFE5\uAFE6\uAFE7\uAFE8\uAFE9\uAFEA"+
	"\uAFEB\uAFEC\uAFED\uAFEE\uAFEF\uAFF0\uAFF1\uAFF2"+
	"\uAFF3\uAFF4\uAFF5\uAFF6\uAFF7\uAFF8\uAFF9\uAFFA"+
	"\uAFFB\uAFFC\uAFFD\uAFFE\uF8A1\uF8A2\uF8A3\uF8A4"+
	"\uF8A5\uF8A6\uF8A7\uF8A8\uF8A9\uF8AA\uF8AB\uF8AC"+
	"\uF8AD\uF8AE\uF8AF\uF8B0\uF8B1\uF8B2\uF8B3\uF8B4"+
	"\uF8B5\uF8B6\uF8B7\uF8B8\uF8B9\uF8BA\uF8BB\uF8BC"+
	"\uF8BD\uF8BE\uF8BF\uF8C0\uF8C1\uF8C2\uF8C3\uF8C4"+
	"\uF8C5\uF8C6\uF8C7\uF8C8\uF8C9\uF8CA\uF8CB\uF8CC"+
	"\uF8CD\uF8CE\uF8CF\uF8D0\uF8D1\uF8D2\uF8D3\uF8D4"+
	"\uF8D5\uF8D6\uF8D7\uF8D8\uF8D9\uF8DA\uF8DB\uF8DC"+
	"\uF8DD\uF8DE\uF8DF\uF8E0\uF8E1\uF8E2\uF8E3\uF8E4"+
	"\uF8E5\uF8E6\uF8E7\uF8E8\uF8E9\uF8EA\uF8EB\uF8EC"+
	"\uF8ED\uF8EE\uF8EF\uF8F0\uF8F1\uF8F2\uF8F3\uF8F4"+
	"\uF8F5\uF8F6\uF8F7\uF8F8\uF8F9\uF8FA\uF8FB\uF8FC"+
	"\uF8FD\uF8FE\uF9A1\uF9A2\uF9A3\uF9A4\uF9A5\uF9A6"+
	"\uF9A7\uF9A8\uF9A9\uF9AA\uF9AB\uF9AC\uF9AD\uF9AE"+
	"\uF9AF\uF9B0\uF9B1\uF9B2\uF9B3\uF9B4\uF9B5\uF9B6"+
	"\uF9B7\uF9B8\uF9B9\uF9BA\uF9BB\uF9BC\uF9BD\uF9BE"+
	"\uF9BF\uF9C0\uF9C1\uF9C2\uF9C3\uF9C4\uF9C5\uF9C6"+
	"\uF9C7\uF9C8\uF9C9\uF9CA\uF9CB\uF9CC\uF9CD\uF9CE"+
	"\uF9CF\uF9D0\uF9D1\uF9D2\uF9D3\uF9D4\uF9D5\uF9D6"+
	"\uF9D7\uF9D8\uF9D9\uF9DA\uF9DB\uF9DC\uF9DD\uF9DE"+
	"\uF9DF\uF9E0\uF9E1\uF9E2\uF9E3\uF9E4\uF9E5\uF9E6"+
	"\uF9E7\uF9E8\uF9E9\uF9EA\uF9EB\uF9EC\uF9ED\uF9EE"+
	"\uF9EF\uF9F0\uF9F1\uF9F2\uF9F3\uF9F4\uF9F5\uF9F6"+
	"\uF9F7\uF9F8\uF9F9\uF9FA\uF9FB\uF9FC\uF9FD\uF9FE"+
	"\uFAA1\uFAA2\uFAA3\uFAA4\uFAA5\uFAA6\uFAA7\uFAA8"+
	"\uFAA9\uFAAA\uFAAB\uFAAC\uFAAD\uFAAE\uFAAF\uFAB0"+
	"\uFAB1\uFAB2\uFAB3\uFAB4\uFAB5\uFAB6\uFAB7\uFAB8"+
	"\uFAB9\uFABA\uFABB\uFABC\uFABD\uFABE\uFABF\uFAC0"+
	"\uFAC1\uFAC2\uFAC3\uFAC4\uFAC5\uFAC6\uFAC7\uFAC8"+
	"\uFAC9\uFACA\uFACB\uFACC\uFACD\uFACE\uFACF\uFAD0"+
	"\uFAD1\uFAD2\uFAD3\uFAD4\uFAD5\uFAD6\uFAD7\uFAD8"+
	"\uFAD9\uFADA\uFADB\uFADC\uFADD\uFADE\uFADF\uFAE0"+
	"\uFAE1\uFAE2\uFAE3\uFAE4\uFAE5\uFAE6\uFAE7\uFAE8"+
	"\uFAE9\uFAEA\uFAEB\uFAEC\uFAED\uFAEE\uFAEF\uFAF0"+
	"\uFAF1\uFAF2\uFAF3\uFAF4\uFAF5\uFAF6\uFAF7\uFAF8"+
	"\uFAF9\uFAFA\uFAFB\uFAFC\uFAFD\uFAFE\uFBA1\uFBA2"+
	"\uFBA3\uFBA4\uFBA5\uFBA6\uFBA7\uFBA8\uFBA9\uFBAA"+
	"\uFBAB\uFBAC\uFBAD\uFBAE\uFBAF\uFBB0\uFBB1\uFBB2"+
	"\uFBB3\uFBB4\uFBB5\uFBB6\uFBB7\uFBB8\uFBB9\uFBBA"+
	"\uFBBB\uFBBC\uFBBD\uFBBE\uFBBF\uFBC0\uFBC1\uFBC2"+
	"\uFBC3\uFBC4\uFBC5\uFBC6\uFBC7\uFBC8\uFBC9\uFBCA"+
	"\uFBCB\uFBCC\uFBCD\uFBCE\uFBCF\uFBD0\uFBD1\uFBD2"+
	"\uFBD3\uFBD4\uFBD5\uFBD6\uFBD7\uFBD8\uFBD9\uFBDA"+
	"\uFBDB\uFBDC\uFBDD\uFBDE\uFBDF\uFBE0\uFBE1\uFBE2"+
	"\uFBE3\uFBE4\uFBE5\uFBE6\uFBE7\uFBE8\uFBE9\uFBEA"+
	"\uFBEB\uFBEC\uFBED\uFBEE\uFBEF\uFBF0\uFBF1\uFBF2"+
	"\uFBF3\uFBF4\uFBF5\uFBF6\uFBF7\uFBF8\uFBF9\uFBFA"+
	"\uFBFB\uFBFC\uFBFD\uFBFE\uFCA1\uFCA2\uFCA3\uFCA4"+
	"\uFCA5\uFCA6\uFCA7\uFCA8\uFCA9\uFCAA\uFCAB\uFCAC"+
	"\uFCAD\uFCAE\uFCAF\uFCB0\uFCB1\uFCB2\uFCB3\uFCB4"+
	"\uFCB5\uFCB6\uFCB7\uFCB8\uFCB9\uFCBA\uFCBB\uFCBC"+
	"\uFCBD\uFCBE\uFCBF\uFCC0\uFCC1\uFCC2\uFCC3\uFCC4"+
	"\uFCC5\uFCC6\uFCC7\uFCC8\uFCC9\uFCCA\uFCCB\uFCCC"+
	"\uFCCD\uFCCE\uFCCF\uFCD0\uFCD1\uFCD2\uFCD3\uFCD4"+
	"\uFCD5\uFCD6\uFCD7\uFCD8\uFCD9\uFCDA\uFCDB\uFCDC"+
	"\uFCDD\uFCDE\uFCDF\uFCE0\uFCE1\uFCE2\uFCE3\uFCE4"+
	"\uFCE5\uFCE6\uFCE7\uFCE8\uFCE9\uFCEA\uFCEB\uFCEC"+
	"\uFCED\uFCEE\uFCEF\uFCF0\uFCF1\uFCF2\uFCF3\uFCF4"+
	"\uFCF5\uFCF6\uFCF7\uFCF8\uFCF9\uFCFA\uFCFB\uFCFC"+
	"\uFCFD\uFCFE\uFDA1\uFDA2\uFDA3\uFDA4\uFDA5\uFDA6"+
	"\uFDA7\uFDA8\uFDA9\uFDAA\uFDAB\uFDAC\uFDAD\uFDAE"+
	"\uFDAF\uFDB0\uFDB1\uFDB2\uFDB3\uFDB4\uFDB5\uFDB6"+
	"\uFDB7\uFDB8\uFDB9\uFDBA\uFDBB\uFDBC\uFDBD\uFDBE"+
	"\uFDBF\uFDC0\uFDC1\uFDC2\uFDC3\uFDC4\uFDC5\uFDC6"+
	"\uFDC7\uFDC8\uFDC9\uFDCA\uFDCB\uFDCC\uFDCD\uFDCE"+
	"\uFDCF\uFDD0\uFDD1\uFDD2\uFDD3\uFDD4\uFDD5\uFDD6"+
	"\uFDD7\uFDD8\uFDD9\uFDDA\uFDDB\uFDDC\uFDDD\uFDDE"+
	"\uFDDF\uFDE0\uFDE1\uFDE2\uFDE3\uFDE4\uFDE5\uFDE6"+
	"\uFDE7\uFDE8\uFDE9\uFDEA\uFDEB\uFDEC\uFDED\uFDEE"+
	"\uFDEF\uFDF0\uFDF1\uFDF2\uFDF3\uFDF4\uFDF5\uFDF6"+
	"\uFDF7\uFDF8\uFDF9\uFDFA\uFDFB\uFDFC\uFDFD\uFDFE"+
	"\uFEA1\uFEA2\uFEA3\uFEA4\uFEA5\uFEA6\uFEA7\uFEA8"+
	"\uFEA9\uFEAA\uFEAB\uFEAC\uFEAD\uFEAE\uFEAF\uFEB0"+
	"\uFEB1\uFEB2\uFEB3\uFEB4\uFEB5\uFEB6\uFEB7\uFEB8"+
	"\uFEB9\uFEBA\uFEBB\uFEBC\uFEBD\uFEBE\uFEBF\uFEC0"+
	"\uFEC1\uFEC2\uFEC3\uFEC4\uFEC5\uFEC6\uFEC7\uFEC8"+
	"\uFEC9\uFECA\uFECB\uFECC\uFECD\uFECE\uFECF\uFED0"+
	"\uFED1\uFED2\uFED3\uFED4\uFED5\uFED6\uFED7\uFED8"+
	"\uFED9\uFEDA\uFEDB\uFEDC\uFEDD\uFEDE\uFEDF\uFEE0"+
	"\uFEE1\uFEE2\uFEE3\uFEE4\uFEE5\uFEE6\uFEE7\uFEE8"+
	"\uFEE9\uFEEA\uFEEB\uFEEC\uFEED\uFEEE\uFEEF\uFEF0"+
	"\uFEF1\uFEF2\uFEF3\uFEF4\uFEF5\uFEF6\uFEF7\uFEF8"+
	"\uFEF9\uFEFA\uFEFB\uFEFC\uFEFD\uFEFE\uA140\uA141"+
	"\uA142\uA143\uA144\uA145\uA146\uA147\uA148\uA149"+
	"\uA14A\uA14B\uA14C\uA14D\uA14E\uA14F\uA150\uA151"+
	"\uA152\uA153\uA154\uA155\uA156\uA157\uA158\uA159"+
	"\uA15A\uA15B\uA15C\uA15D\uA15E\uA15F\uA160\uA161"+
	"\uA162\uA163\uA164\uA165\uA166\uA167\uA168\uA169"+
	"\uA16A\uA16B\uA16C\uA16D\uA16E\uA16F\uA170\uA171"+
	"\uA172\uA173\uA174\uA175\uA176\uA177\uA178\uA179"+
	"\uA17A\uA17B\uA17C\uA17D\uA17E\uA180\uA181\uA182"+
	"\uA183\uA184\uA185\uA186\uA187\uA188\uA189\uA18A"+
	"\uA18B\uA18C\uA18D\uA18E\uA18F\uA190\uA191\uA192"+
	"\uA193\uA194\uA195\uA196\uA197\uA198\uA199\uA19A"+
	"\uA19B\uA19C\uA19D\uA19E\uA19F\uA1A0\uA240\uA241"+
	"\uA242\uA243\uA244\uA245\uA246\uA247\uA248\uA249"+
	"\uA24A\uA24B\uA24C\uA24D\uA24E\uA24F\uA250\uA251"+
	"\uA252\uA253\uA254\uA255\uA256\uA257\uA258\uA259"+
	"\uA25A\uA25B\uA25C\uA25D\uA25E\uA25F\uA260\uA261"+
	"\uA262\uA263\uA264\uA265\uA266\uA267\uA268\uA269"+
	"\uA26A\uA26B\uA26C\uA26D\uA26E\uA26F\uA270\uA271"+
	"\uA272\uA273\uA274\uA275\uA276\uA277\uA278\uA279"+
	"\uA27A\uA27B\uA27C\uA27D\uA27E\uA280\uA281\uA282"+
	"\uA283\uA284\uA285\uA286\uA287\uA288\uA289\uA28A"+
	"\uA28B\uA28C\uA28D\uA28E\uA28F\uA290\uA291\uA292"+
	"\uA293\uA294\uA295\uA296\uA297\uA298\uA299\uA29A"+
	"\uA29B\uA29C\uA29D\uA29E\uA29F\uA2A0\uA340\uA341"+
	"\uA342\uA343\uA344\uA345\uA346\uA347\uA348\uA349"+
	"\uA34A\uA34B\uA34C\uA34D\uA34E\uA34F\uA350\uA351"+
	"\uA352\uA353\uA354\uA355\uA356\uA357\uA358\uA359"+
	"\uA35A\uA35B\uA35C\uA35D\uA35E\uA35F\uA360\uA361"+
	"\uA362\uA363\uA364\uA365\uA366\uA367\uA368\uA369"+
	"\uA36A\uA36B\uA36C\uA36D\uA36E\uA36F\uA370\uA371"+
	"\uA372\uA373\uA374\uA375\uA376\uA377\uA378\uA379"+
	"\uA37A\uA37B\uA37C\uA37D\uA37E\uA380\uA381\uA382"+
	"\uA383\uA384\uA385\uA386\uA387\uA388\uA389\uA38A"+
	"\uA38B\uA38C\uA38D\uA38E\uA38F\uA390\uA391\uA392"+
	"\uA393\uA394\uA395\uA396\uA397\uA398\uA399\uA39A"+
	"\uA39B\uA39C\uA39D\uA39E\uA39F\uA3A0\uA440\uA441"+
	"\uA442\uA443\uA444\uA445\uA446\uA447\uA448\uA449"+
	"\uA44A\uA44B\uA44C\uA44D\uA44E\uA44F\uA450\uA451"+
	"\uA452\uA453\uA454\uA455\uA456\uA457\uA458\uA459"+
	"\uA45A\uA45B\uA45C\uA45D\uA45E\uA45F\uA460\uA461"+
	"\uA462\uA463\uA464\uA465\uA466\uA467\uA468\uA469"+
	"\uA46A\uA46B\uA46C\uA46D\uA46E\uA46F\uA470\uA471"+
	"\uA472\uA473\uA474\uA475\uA476\uA477\uA478\uA479"+
	"\uA47A\uA47B\uA47C\uA47D\uA47E\uA480\uA481\uA482"+
	"\uA483\uA484\uA485\uA486\uA487\uA488\uA489\uA48A"+
	"\uA48B\uA48C\uA48D\uA48E\uA48F\uA490\uA491\uA492"+
	"\uA493\uA494\uA495\uA496\uA497\uA498\uA499\uA49A"+
	"\uA49B\uA49C\uA49D\uA49E\uA49F\uA4A0\uA540\uA541"+
	"\uA542\uA543\uA544\uA545\uA546\uA547\uA548\uA549"+
	"\uA54A\uA54B\uA54C\uA54D\uA54E\uA54F\uA550\uA551"+
	"\uA552\uA553\uA554\uA555\uA556\uA557\uA558\uA559"+
	"\uA55A\uA55B\uA55C\uA55D\uA55E\uA55F\uA560\uA561"+
	"\uA562\uA563\uA564\uA565\uA566\uA567\uA568\uA569"+
	"\uA56A\uA56B\uA56C\uA56D\uA56E\uA56F\uA570\uA571"+
	"\uA572\uA573\uA574\uA575\uA576\uA577\uA578\uA579"+
	"\uA57A\uA57B\uA57C\uA57D\uA57E\uA580\uA581\uA582"+
	"\uA583\uA584\uA585\uA586\uA587\uA588\uA589\uA58A"+
	"\uA58B\uA58C\uA58D\uA58E\uA58F\uA590\uA591\uA592"+
	"\uA593\uA594\uA595\uA596\uA597\uA598\uA599\uA59A"+
	"\uA59B\uA59C\uA59D\uA59E\uA59F\uA5A0\uA640\uA641"+
	"\uA642\uA643\uA644\uA645\uA646\uA647\uA648\uA649"+
	"\uA64A\uA64B\uA64C\uA64D\uA64E\uA64F\uA650\uA651"+
	"\uA652\uA653\uA654\uA655\uA656\uA657\uA658\uA659"+
	"\uA65A\uA65B\uA65C\uA65D\uA65E\uA65F\uA660\uA661"+
	"\uA662\uA663\uA664\uA665\uA666\uA667\uA668\uA669"+
	"\uA66A\uA66B\uA66C\uA66D\uA66E\uA66F\uA670\uA671"+
	"\uA672\uA673\uA674\uA675\uA676\uA677\uA678\uA679"+
	"\uA67A\uA67B\uA67C\uA67D\uA67E\uA680\uA681\uA682"+
	"\uA683\uA684\uA685\uA686\uA687\uA688\uA689\uA68A"+
	"\uA68B\uA68C\uA68D\uA68E\uA68F\uA690\uA691\uA692"+
	"\uA693\uA694\uA695\uA696\uA697\uA698\uA699\uA69A"+
	"\uA69B\uA69C\uA69D\uA69E\uA69F\uA6A0\uA740\uA741"+
	"\uA742\uA743\uA744\uA745\uA746\uA747\uA748\uA749"+
	"\uA74A\uA74B\uA74C\uA74D\uA74E\uA74F\uA750\uA751"+
	"\uA752\uA753\uA754\uA755\uA756\uA757\uA758\uA759"+
	"\uA75A\uA75B\uA75C\uA75D\uA75E\uA75F\uA760\uA761"+
	"\uA762\uA763\uA764\uA765\uA766\uA767\uA768\uA769"+
	"\uA76A\uA76B\uA76C\uA76D\uA76E\uA76F\uA770\uA771"+
	"\uA772\uA773\uA774\uA775\uA776\uA777\uA778\uA779"+
	"\uA77A\uA77B\uA77C\uA77D\uA77E\uA780\uA781\uA782"+
	"\uA783\uA784\uA785\uA786\uA787\uA788\uA789\uA78A"+
	"\uA78B\uA78C\uA78D\uA78E\uA78F\uA790\uA791\uA792"+
	"\uA793\uA794\uA795\uA796\uA797\uA798\uA799\uA79A"+
	"\uA79B\uA79C\uA79D\uA79E\uA79F\uA7A0\uA2AB\uA2AC"+
	"\uA2AD\uA2AE\uA2AF\uA2B0\u2000\uA2E4\uA2EF\uA2F0"+
	"\uA2FD\uA2FE\uA4F4\uA4F5\uA4F6\uA4F7\uA4F8\uA4F9"+
	"\uA4FA\uA4FB\uA4FC\uA4FD\uA4FE\uA5F7\uA5F8\uA5F9"+
	"\uA5FA\uA5FB\uA5FC\uA5FD\uA5FE\uA6B9\uA6BA\uA6BB"+
	"\uA6BC\uA6BD\uA6BE\uA6BF\uA6C0\uA6D9\uA6DA\uA6DB"+
	"\uA6DC\uA6DD\uA6DE\uA6DF\uA6EC\uA6ED\uA6F3\uA6F6"+
	"\uA6F7\uA6F8\uA6F9\uA6FA\uA6FB\uA6FC\uA6FD\uA6FE"+
	"\uA7C2\uA7C3\uA7C4\uA7C5\uA7C6\uA7C7\uA7C8\uA7C9"+
	"\uA7CA\uA7CB\uA7CC\uA7CD\uA7CE\uA7CF\uA7D0\uA7F2"+
	"\uA7F3\uA7F4\uA7F5\uA7F6\uA7F7\uA7F8\uA7F9\uA7FA"+
	"\uA7FB\uA7FC\uA7FD\uA7FE\uA896\uA897\uA898\uA899"+
	"\uA89A\uA89B\uA89C\uA89D\uA89E\uA89F\uA8A0\uA8BC"+
	"\u2001\uA8C1\uA8C2\uA8C3\uA8C4\uA8EA\uA8EB\uA8EC"+
	"\uA8ED\uA8EE\uA8EF\uA8F0\uA8F1\uA8F2\uA8F3\uA8F4"+
	"\uA8F5\uA8F6\uA8F7\uA8F8\uA8F9\uA8FA\uA8FB\uA8FC"+
	"\uA8FD\uA8FE\uA958\uA95B\uA95D\uA95E\uA95F\u2002"+
	"\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A"+
	"\u200B\u200C\u200D\u200E\uA997\uA998\uA999\uA99A"+
	"\uA99B\uA99C\uA99D\uA99E\uA99F\uA9A0\uA9A1\uA9A2"+
	"\uA9A3\uA9F0\uA9F1\uA9F2\uA9F3\uA9F4\uA9F5\uA9F6"+
	"\uA9F7\uA9F8\uA9F9\uA9FA\uA9FB\uA9FC\uA9FD\uA9FE"+
	"\uD7FA\uD7FB\uD7FC\uD7FD\uD7FE\u200F\uFE51\uFE52"+
	"\uFE53\u2010\u2011\u2012\u2013\u2014\uFE59\u2015"+
	"\u2016\u2017\u2018\u2019\u201A\u201B\uFE61\u201C"+
	"\u201D\u201E\u201F\uFE66\uFE67\u2020\u2021\u2022"+
	"\u2023\uFE6C\uFE6D\u2024\u2025\u2026\u2027\u2028"+
	"\u2029\u202A\u202B\uFE76\u202C\u202D\u202E\u202F"+
	"\u2030\u2031\u2032\uFE7E\u2033\u2034\u2035\u2036"+
	"\u2037\u2038\u2039\u203A\u203B\u203C\u203D\u203E"+
	"\u203F\u2040\u2041\u2042\uFE90\uFE91\u2043\u2044"+
	"\u2045\u2046\u2047\u2048\u2049\u204A\u204B\u204C"+
	"\u204D\u204E\u204F\u2050\uFEA0\u2051\u2052\u2053"+
	"\u2054\u2055\u2056\u2057\u2058\u2059\u205A\u205B"+
	"\u205C\u205D\u205E\u205F\u2060\u2061\u2062\u2063"+
	"\u2064\u2065\u2066\u2067\u2068\u2069\u206A\u206B"+
	"\u206C\u206D\u206E\u206F\u2070\u2071\u2072\u2073"+
	"\u2074\u2075\u2076\u2077\u2078\u2079\u207A\u207B"+
	"\u207C\u207D\u207E\u207F\u2080\u2081\u2082\u2083"+
	"\u2084\u2085\u2086\u2087\u2088\u2089\u208A\u208B"+
	"\u208C\u208D\u208E\u208F\u2090\u2091\u2092\u2093"+
	"\u2094\u2095\u2096\u2097\u2098\u2099\u209A\u209B"+
	"\u209C\u209D\u209E\u209F\u20A0\u20A1\u20A2\u20A3"+
	"\u20A4\u20A5\u20A6\u20A7\u20A8\u20A9\u20AA\u20AB"+
	"\u20AC\u20AD\u20AE\u20AF\u20B0\u20B1\u20B2\u20B3"+
	"\u20B4\u20B5\u20B6\u20B7\u20B8\u20B9\u20BA\u20BB"+
	"\u20BC\u20BD\u20BE\u20BF\u20C0\u20C1\u20C2\u20C3"+
	"\u20C4\u20C5\u20C6\u20C7\u20C8\u20C9\u20CA\u20CB"+
	"\u20CC\u20CD\u20CE\u20CF\u20D0\u20D1\u20D2\u20D3"+
	"\u20D4\u20D5\u20D6\u20D7\u20D8\u20D9\u20DA\u20DB"+
	"\u20DC\u20DD\u20DE\u20DF\u20E0\u20E1\u20E2\u20E3"+
	"\u20E4\u20E5\u20E6\u20E7\u20E8\u20E9\u20EA\u20EB"+
	"\u20EC\u20ED\u20EE\u20EF\u20F0\u20F1\u20F2\u20F3"+
	"\u20F4\u20F5\u20F6\u20F7\u20F8\u20F9\u20FA\u20FB"+
	"\u20FC\u20FD\u20FE\u20FF\u2100\u2101\u2102\u2103"+
	"\u2104\u2105\u2106\u2107\u2108\u2109\u210A\u210B"+
	"\u210C\u210D\u210E\u210F\u2110\u2111\u2112\u2113"+
	"\u2114\u2115\u2116\u2117\u2118\u2119\u211A\u211B"+
	"\u211C\u211D\u211E\u211F\u2120\u2121\u2122\u2123"+
	"\u2124\u2125\u2126\u2127\u2128\u2129\u212A\u212B"+
	"\u212C\u212D\u212E\u212F\u2130\u2131\u2132\u2133"+
	"\u2134\u2135\u2136\u2137\u2138\u2139\u213A\u213B"+
	"\u213C\u213D\u213E\u213F\u2140\u2141\u2142\u2143"+
	"\u2144\u2145\u2146\u2147\u2148\u2149\u214A\u214B"+
	"\u214C\u214D\u214E\u214F\u2150\u2151\u2152\u2153"+
	"\u2154\u2155\u2156\u2157\u2158\u2159\u215A\u215B"+
	"\u215C\u215D\u215E\u215F\u2160\u2161\u2162\u2163"+
	"\u2164\u2165\u2166\u2167\u2168\u2169\u216A\u216B"+
	"\u216C\u216D\u216E\u216F\u2170\u2171\u2172\u2173"+
	"\u2174\u2175\u2176\u2177\u2178\u2179\u217A\u217B"+
	"\u217C\u217D\u217E\u217F\u2180\u2181\u2182\u2183"+
	"\u2184\u2185\u2186\u2187\u2188\u2189\u218A\u218B"+
	"\u218C\u218D\u218E\u218F\u2190\u2191\u2192\u2193"+
	"\u2194\u2195\u2196\u2197\u2198\u2199\u219A\u219B"+
	"\u219C\u219D\u219E\u219F\u21A0\u21A1\u21A2\u21A3"+
	"\u21A4\u21A5\u21A6\u21A7\u21A8\u21A9\u21AA\u21AB"+
	"\u21AC\u21AD\u21AE\u21AF\u21B0\u21B1\u21B2\u21B3"+
	"\u21B4\u21B5\u21B6\u21B7\u21B8\u21B9\u21BA\u21BB"+
	"\u21BC\u21BD\u21BE\u21BF\u21C0\u21C1\u21C2\u21C3"+
	"\u21C4\u21C5\u21C6\u21C7\u21C8\u21C9\u21CA\u21CB"+
	"\u21CC\u21CD\u21CE\u21CF\u21D0\u21D1\u21D2\u21D3"+
	"\u21D4\u21D5\u21D6\u21D7\u21D8\u21D9\u21DA\u21DB"+
	"\u21DC\u21DD\u21DE\u21DF\u21E0\u21E1\u21E2\u21E3"+
	"\u21E4\u21E5\u21E6\u21E7\u21E8\u21E9\u21EA\u21EB"+
	"\u21EC\u21ED\u21EE\u21EF\u21F0\u21F1\u21F2\u21F3"+
	"\u21F4\u21F5\u21F6\u21F7\u21F8\u21F9\u21FA\u21FB"+
	"\u21FC\u21FD\u21FE\u21FF\u2200\u2201\u2202\u2203"+
	"\u2204\u2205\u2206\u2207\u2208\u2209\u220A\u220B"+
	"\u220C\u220D\u220E\u220F\u2210\u2211\u2212\u2213"+
	"\u2214\u2215\u2216\u2217\u2218\u2219\u221A\u221B"+
	"\u221C\u221D\u221E\u221F\u2220\u2221\u2222\u2223"+
	"\u2224\u2225\u2226\u2227\u2228\u2229\u222A\u222B"+
	"\u222C\u222D\u222E\u222F\u2230\u2231\u2232\u2233"+
	"\u2234\u2235\u2236\u2237\u2238\u2239\u223A\u223B"+
	"\u223C\u223D\u223E\u223F\u2240\u2241\u2242\u2243"+
	"\u2244\u2245\u2246\u2247\u2248\u2249\u224A\u224B"+
	"\u224C\u224D\u224E\u224F\u2250\u2251\u2252\u2253"+
	"\u2254\u2255\u2256\u2257\u2258\u2259\u225A\u225B"+
	"\u225C\u225D\u225E\u225F\u2260\u2261\u2262\u2263"+
	"\u2264\u2265\u2266\u2267\u2268\u2269\u226A\u226B"+
	"\u226C\u226D\u226E\u226F\u2270\u2271\u2272\u2273"+
	"\u2274\u2275\u2276\u2277\u2278\u2279\u227A\u227B"+
	"\u227C\u227D\u227E\u227F\u2280\u2281\u2282\u2283"+
	"\u2284\u2285\u2286\u2287\u2288\u2289\u228A\u228B"+
	"\u228C\u228D\u228E\u228F\u2290\u2291\u2292\u2293"+
	"\u2294\u2295\u2296\u2297\u2298\u2299\u229A\u229B"+
	"\u229C\u229D\u229E\u229F\u22A0\u22A1\u22A2\u22A3"+
	"\u22A4\u22A5\u22A6\u22A7\u22A8\u22A9\u22AA\u22AB"+
	"\u22AC\u22AD\u22AE\u22AF\u22B0\u22B1\u22B2\u22B3"+
	"\u22B4\u22B5\u22B6\u22B7\u22B8\u22B9\u22BA\u22BB"+
	"\u22BC\u22BD\u22BE\u22BF\u22C0\u22C1\u22C2\u22C3"+
	"\u22C4\u22C5\u22C6\u22C7\u22C8\u22C9\u22CA\u22CB"+
	"\u22CC\u22CD\u22CE\u22CF\u22D0\u22D1\u22D2\u22D3"+
	"\u22D4\u22D5\u22D6\u22D7\u22D8\u22D9\u22DA\u22DB"+
	"\u22DC\u22DD\u22DE\u22DF\u22E0\u22E1\u22E2\u22E3"+
	"\u22E4\u22E5\u22E6\u22E7\u22E8\u22E9\u22EA\u22EB";

    private final static String innerIndex11= 
	"\u22EC\u22ED\u22EE\u22EF\u22F0\u22F1\u22F2\u22F3"+
	"\u22F4\u22F5\u22F6\u22F7\u22F8\u22F9\u22FA\u22FB"+
	"\u22FC\u22FD\u22FE\u22FF\u2300\u2301\u2302\u2303"+
	"\u2304\u2305\u2306\u2307\u2308\u2309\u230A\u230B"+
	"\u230C\u230D\u230E\u230F\u2310\u2311\u2312\u2313"+
	"\u2314\u2315\u2316\u2317\u2318\u2319\u231A\u231B"+
	"\u231C\u231D\u231E\u231F\u2320\u2321\u2322\u2323"+
	"\u2324\u2325\u2326\u2327\u2328\u2329\u232A\u232B"+
	"\u232C\u232D\u232E\u232F\u2330\u2331\u2332\u2333"+
	"\u2334\u2335\u2336\u2337\u2338\u2339\u233A\u233B"+
	"\u233C\u233D\u233E\u233F\u2340\u2341\u2342\u2343"+
	"\u2344\u2345\u2346\u2347\u2348\u2349\u234A\u234B"+
	"\u234C\u234D\u234E\u234F\u2350\u2351\u2352\u2353"+
	"\u2354\u2355\u2356\u2357\u2358\u2359\u235A\u235B"+
	"\u235C\u235D\u235E\u235F\u2360\u2361\u2362\u2363"+
	"\u2364\u2365\u2366\u2367\u2368\u2369\u236A\u236B"+
	"\u236C\u236D\u236E\u236F\u2370\u2371\u2372\u2373"+
	"\u2374\u2375\u2376\u2377\u2378\u2379\u237A\u237B"+
	"\u237C\u237D\u237E\u237F\u2380\u2381\u2382\u2383"+
	"\u2384\u2385\u2386\u2387\u2388\u2389\u238A\u238B"+
	"\u238C\u238D\u238E\u238F\u2390\u2391\u2392\u2393"+
	"\u2394\u2395\u2396\u2397\u2398\u2399\u239A\u239B"+
	"\u239C\u239D\u239E\u239F\u23A0\u23A1\u23A2\u23A3"+
	"\u23A4\u23A5\u23A6\u23A7\u23A8\u23A9\u23AA\u23AB"+
	"\u23AC\u23AD\u23AE\u23AF\u23B0\u23B1\u23B2\u23B3"+
	"\u23B4\u23B5\u23B6\u23B7\u23B8\u23B9\u23BA\u23BB"+
	"\u23BC\u23BD\u23BE\u23BF\u23C0\u23C1\u23C2\u23C3"+
	"\u23C4\u23C5\u23C6\u23C7\u23C8\u23C9\u23CA\u23CB"+
	"\u23CC\u23CD\u23CE\u23CF\u23D0\u23D1\u23D2\u23D3"+
	"\u23D4\u23D5\u23D6\u23D7\u23D8\u23D9\u23DA\u23DB"+
	"\u23DC\u23DD\u23DE\u23DF\u23E0\u23E1\u23E2\u23E3"+
	"\u23E4\u23E5\u23E6\u23E7\u23E8\u23E9\u23EA\u23EB"+
	"\u23EC\u23ED\u23EE\u23EF\u23F0\u23F1\u23F2\u23F3"+
	"\u23F4\u23F5\u23F6\u23F7\u23F8\u23F9\u23FA\u23FB"+
	"\u23FC\u23FD\u23FE\u23FF\u2400\u2401\u2402\u2403"+
	"\u2404\u2405\u2406\u2407\u2408\u2409\u240A\u240B"+
	"\u240C\u240D\u240E\u240F\u2410\u2411\u2412\u2413"+
	"\u2414\u2415\u2416\u2417\u2418\u2419\u241A\u241B"+
	"\u241C\u241D\u241E\u241F\u2420\u2421\u2422\u2423"+
	"\u2424\u2425\u2426\u2427\u2428\u2429\u242A\u242B"+
	"\u242C\u242D\u242E\u242F\u2430\u2431\u2432\u2433"+
	"\u2434\u2435\u2436\u2437\u2438\u2439\u243A\u243B"+
	"\u243C\u243D\u243E\u243F\u2440\u2441\u2442\u2443"+
	"\u2444\u2445\u2446\u2447\u2448\u2449\u244A\u244B"+
	"\u244C\u244D\u244E\u244F\u2450\u2451\u2452\u2453"+
	"\u2454\u2455\u2456\u2457\u2458\u2459\u245A\u245B"+
	"\u245C\u245D\u245E\u245F\u2460\u2461\u2462\u2463"+
	"\u2464\u2465\u2466\u2467\u2468\u2469\u246A\u246B"+
	"\u246C\u246D\u246E\u246F\u2470\u2471\u2472\u2473"+
	"\u2474\u2475\u2476\u2477\u2478\u2479\u247A\u247B"+
	"\u247C\u247D\u247E\u247F\u2480\u2481\u2482\u2483"+
	"\u2484\u2485\u2486\u2487\u2488\u2489\u248A\u248B"+
	"\u248C\u248D\u248E\u248F\u2490\u2491\u2492\u2493"+
	"\u2494\u2495\u2496\u2497\u2498\u2499\u249A\u249B"+
	"\u249C\u249D\u249E\u249F\u24A0\u24A1\u24A2\u24A3"+
	"\u24A4\u24A5\u24A6\u24A7\u24A8\u24A9\u24AA\u24AB"+
	"\u24AC\u24AD\u24AE\u24AF\u24B0\u24B1\u24B2\u24B3"+
	"\u24B4\u24B5\u24B6\u24B7\u24B8\u24B9\u24BA\u24BB"+
	"\u24BC\u24BD\u24BE\u24BF\u24C0\u24C1\u24C2\u24C3"+
	"\u24C4\u24C5\u24C6\u24C7\u24C8\u24C9\u24CA\u24CB"+
	"\u24CC\u24CD\u24CE\u24CF\u24D0\u24D1\u24D2\u24D3"+
	"\u24D4\u24D5\u24D6\u24D7\u24D8\u24D9\u24DA\u24DB"+
	"\u24DC\u24DD\u24DE\u24DF\u24E0\u24E1\u24E2\u24E3"+
	"\u24E4\u24E5\u24E6\u24E7\u24E8\u24E9\u24EA\u24EB"+
	"\u24EC\u24ED\u24EE\u24EF\u24F0\u24F1\u24F2\u24F3"+
	"\u24F4\u24F5\u24F6\u24F7\u24F8\u24F9\u24FA\u24FB"+
	"\u24FC\u24FD\u24FE\u24FF\u2500\u2501\u2502\u2503"+
	"\u2504\u2505\u2506\u2507\u2508\u2509\u250A\u250B"+
	"\u250C\u250D\u250E\u250F\u2510\u2511\u2512\u2513"+
	"\u2514\u2515\u2516\u2517\u2518\u2519\u251A\u251B"+
	"\u251C\u251D\u251E\u251F\u2520\u2521\u2522\u2523"+
	"\u2524\u2525\u2526\u2527\u2528\u2529\u252A\u252B"+
	"\u252C\u252D\u252E\u252F\u2530\u2531\u2532\u2533"+
	"\u2534\u2535\u2536\u2537\u2538\u2539\u253A\u253B"+
	"\u253C\u253D\u253E\u253F\u2540\u2541\u2542\u2543"+
	"\u2544\u2545\u2546\u2547\u2548\u2549\u254A\u254B"+
	"\u254C\u254D\u254E\u254F\u2550\u2551\u2552\u2553"+
	"\u2554\u2555\u2556\u2557\u2558\u2559\u255A\u255B"+
	"\u255C\u255D\u255E\u255F\u2560\u2561\u2562\u2563"+
	"\u2564\u2565\u2566\u2567\u2568\u2569\u256A\u256B"+
	"\u256C\u256D\u256E\u256F\u2570\u2571\u2572\u2573"+
	"\u2574\u2575\u2576\u2577\u2578\u2579\u257A\u257B"+
	"\u257C\u257D\u257E\u257F\u2580\u2581\u2582\u2583"+
	"\u2584\u2585\u2586\u2587\u2588\u2589\u258A\u258B"+
	"\u258C\u258D\u258E\u258F\u2590\u2591\u2592\u2593"+
	"\u2594\u2595\u2596\u2597\u2598\u2599\u259A\u259B"+
	"\u259C\u259D\u259E\u259F\u25A0\u25A1\u25A2\u25A3"+
	"\u25A4\u25A5\u25A6\u25A7\u25A8\u25A9\u25AA\u25AB"+
	"\u25AC\u25AD\u25AE\u25AF\u25B0\u25B1\u25B2\u25B3"+
	"\u25B4\u25B5\u25B6\u25B7\u25B8\u25B9\u25BA\u25BB"+
	"\u25BC\u25BD\u25BE\u25BF\u25C0\u25C1\u25C2\u25C3"+
	"\u25C4\u25C5\u25C6\u25C7\u25C8\u25C9\u25CA\u25CB"+
	"\u25CC\u25CD\u25CE\u25CF\u25D0\u25D1\u25D2\u25D3"+
	"\u25D4\u25D5\u25D6\u25D7\u25D8\u25D9\u25DA\u25DB"+
	"\u25DC\u25DD\u25DE\u25DF\u25E0\u25E1\u25E2\u25E3"+
	"\u25E4\u25E5\u25E6\u25E7\u25E8\u25E9\u25EA\u25EB"+
	"\u25EC\u25ED\u25EE\u25EF\u25F0\u25F1\u25F2\u25F3"+
	"\u25F4\u25F5\u25F6\u25F7\u25F8\u25F9\u25FA\u25FB"+
	"\u25FC\u25FD\u25FE\u25FF\u2600\u2601\u2602\u2603"+
	"\u2604\u2605\u2606\u2607\u2608\u2609\u260A\u260B"+
	"\u260C\u260D\u260E\u260F\u2610\u2611\u2612\u2613"+
	"\u2614\u2615\u2616\u2617\u2618\u2619\u261A\u261B"+
	"\u261C\u261D\u261E\u261F\u2620\u2621\u2622\u2623"+
	"\u2624\u2625\u2626\u2627\u2628\u2629\u262A\u262B"+
	"\u262C\u262D\u262E\u262F\u2630\u2631\u2632\u2633"+
	"\u2634\u2635\u2636\u2637\u2638\u2639\u263A\u263B"+
	"\u263C\u263D\u263E\u263F\u2640\u2641\u2642\u2643"+
	"\u2644\u2645\u2646\u2647\u2648\u2649\u264A\u264B"+
	"\u264C\u264D\u264E\u264F\u2650\u2651\u2652\u2653"+
	"\u2654\u2655\u2656\u2657\u2658\u2659\u265A\u265B"+
	"\u265C\u265D\u265E\u265F\u2660\u2661\u2662\u2663"+
	"\u2664\u2665\u2666\u2667\u2668\u2669\u266A\u266B"+
	"\u266C\u266D\u266E\u266F\u2670\u2671\u2672\u2673"+
	"\u2674\u2675\u2676\u2677\u2678\u2679\u267A\u267B"+
	"\u267C\u267D\u267E\u267F\u2680\u2681\u2682\u2683"+
	"\u2684\u2685\u2686\u2687\u2688\u2689\u268A\u268B"+
	"\u268C\u268D\u268E\u268F\u2690\u2691\u2692\u2693"+
	"\u2694\u2695\u2696\u2697\u2698\u2699\u269A\u269B"+
	"\u269C\u269D\u269E\u269F\u26A0\u26A1\u26A2\u26A3"+
	"\u26A4\u26A5\u26A6\u26A7\u26A8\u26A9\u26AA\u26AB"+
	"\u26AC\u26AD\u26AE\u26AF\u26B0\u26B1\u26B2\u26B3"+
	"\u26B4\u26B5\u26B6\u26B7\u26B8\u26B9\u26BA\u26BB"+
	"\u26BC\u26BD\u26BE\u26BF\u26C0\u26C1\u26C2\u26C3"+
	"\u26C4\u26C5\u26C6\u26C7\u26C8\u26C9\u26CA\u26CB"+
	"\u26CC\u26CD\u26CE\u26CF\u26D0\u26D1\u26D2\u26D3"+
	"\u26D4\u26D5\u26D6\u26D7\u26D8\u26D9\u26DA\u26DB"+
	"\u26DC\u26DD\u26DE\u26DF\u26E0\u26E1\u26E2\u26E3"+
	"\u26E4\u26E5\u26E6\u26E7\u26E8\u26E9\u26EA\u26EB"+
	"\u26EC\u26ED\u26EE\u26EF\u26F0\u26F1\u26F2\u26F3"+
	"\u26F4\u26F5\u26F6\u26F7\u26F8\u26F9\u26FA\u26FB"+
	"\u26FC\u26FD\u26FE\u26FF\u2700\u2701\u2702\u2703"+
	"\u2704\u2705\u2706\u2707\u2708\u2709\u270A\u270B"+
	"\u270C\u270D\u270E\u270F\u2710\u2711\u2712\u2713"+
	"\u2714\u2715\u2716\u2717\u2718\u2719\u271A\u271B"+
	"\u271C\u271D\u271E\u271F\u2720\u2721\u2722\u2723"+
	"\u2724\u2725\u2726\u2727\u2728\u2729\u272A\u272B"+
	"\u272C\u272D\u272E\u272F\u2730\u2731\u2732\u2733"+
	"\u2734\u2735\u2736\u2737\u2738\u2739\u273A\u273B"+
	"\u273C\u273D\u273E\u273F\u2740\u2741\u2742\u2743"+
	"\u2744\u2745\u2746\u2747\u2748\u2749\u274A\u274B"+
	"\u274C\u274D\u274E\u274F\u2750\u2751\u2752\u2753"+
	"\u2754\u2755\u2756\u2757\u2758\u2759\u275A\u275B"+
	"\u275C\u275D\u275E\u275F\u2760\u2761\u2762\u2763"+
	"\u2764\u2765\u2766\u2767\u2768\u2769\u276A\u276B"+
	"\u276C\u276D\u276E\u276F\u2770\u2771\u2772\u2773"+
	"\u2774\u2775\u2776\u2777\u2778\u2779\u277A\u277B"+
	"\u277C\u277D\u277E\u277F\u2780\u2781\u2782\u2783"+
	"\u2784\u2785\u2786\u2787\u2788\u2789\u278A\u278B"+
	"\u278C\u278D\u278E\u278F\u2790\u2791\u2792\u2793"+
	"\u2794\u2795\u2796\u2797\u2798\u2799\u279A\u279B"+
	"\u279C\u279D\u279E\u279F\u27A0\u27A1\u27A2\u27A3"+
	"\u27A4\u27A5\u27A6\u27A7\u27A8\u27A9\u27AA\u27AB"+
	"\u27AC\u27AD\u27AE\u27AF\u27B0\u27B1\u27B2\u27B3"+
	"\u27B4\u27B5\u27B6\u27B7\u27B8\u27B9\u27BA\u27BB"+
	"\u27BC\u27BD\u27BE\u27BF\u27C0\u27C1\u27C2\u27C3"+
	"\u27C4\u27C5\u27C6\u27C7\u27C8\u27C9\u27CA\u27CB"+
	"\u27CC\u27CD\u27CE\u27CF\u27D0\u27D1\u27D2\u27D3"+
	"\u27D4\u27D5\u27D6\u27D7\u27D8\u27D9\u27DA\u27DB"+
	"\u27DC\u27DD\u27DE\u27DF\u27E0\u27E1\u27E2\u27E3"+
	"\u27E4\u27E5\u27E6\u27E7\u27E8\u27E9\u27EA\u27EB"+
	"\u27EC\u27ED\u27EE\u27EF\u27F0\u27F1\u27F2\u27F3"+
	"\u27F4\u27F5\u27F6\u27F7\u27F8\u27F9\u27FA\u27FB"+
	"\u27FC\u27FD\u27FE\u27FF\u2800\u2801\u2802\u2803"+
	"\u2804\u2805\u2806\u2807\u2808\u2809\u280A\u280B"+
	"\u280C\u280D\u280E\u280F\u2810\u2811\u2812\u2813"+
	"\u2814\u2815\u2816\u2817\u2818\u2819\u281A\u281B"+
	"\u281C\u281D\u281E\u281F\u2820\u2821\u2822\u2823"+
	"\u2824\u2825\u2826\u2827\u2828\u2829\u282A\u282B"+
	"\u282C\u282D\u282E\u282F\u2830\u2831\u2832\u2833"+
	"\u2834\u2835\u2836\u2837\u2838\u2839\u283A\u283B"+
	"\u283C\u283D\u283E\u283F\u2840\u2841\u2842\u2843"+
	"\u2844\u2845\u2846\u2847\u2848\u2849\u284A\u284B"+
	"\u284C\u284D\u284E\u284F\u2850\u2851\u2852\u2853"+
	"\u2854\u2855\u2856\u2857\u2858\u2859\u285A\u285B"+
	"\u285C\u285D\u285E\u285F\u2860\u2861\u2862\u2863"+
	"\u2864\u2865\u2866\u2867\u2868\u2869\u286A\u286B"+
	"\u286C\u286D\u286E\u286F\u2870\u2871\u2872\u2873"+
	"\u2874\u2875\u2876\u2877\u2878\u2879\u287A\u287B"+
	"\u287C\u287D\u287E\u287F\u2880\u2881\u2882\u2883"+
	"\u2884\u2885\u2886\u2887\u2888\u2889\u288A\u288B"+
	"\u288C\u288D\u288E\u288F\u2890\u2891\u2892\u2893"+
	"\u2894\u2895\u2896\u2897\u2898\u2899\u289A\u289B"+
	"\u289C\u289D\u289E\u289F\u28A0\u28A1\u28A2\u28A3"+
	"\u28A4\u28A5\u28A6\u28A7\u28A8\u28A9\u28AA\u28AB"+
	"\u28AC\u28AD\u28AE\u28AF\u28B0\u28B1\u28B2\u28B3"+
	"\u28B4\u28B5\u28B6\u28B7\u28B8\u28B9\u28BA\u28BB"+
	"\u28BC\u28BD\u28BE\u28BF\u28C0\u28C1\u28C2\u28C3"+
	"\u28C4\u28C5\u28C6\u28C7\u28C8\u28C9\u28CA\u28CB"+
	"\u28CC\u28CD\u28CE\u28CF\u28D0\u28D1\u28D2\u28D3"+
	"\u28D4\u28D5\u28D6\u28D7\u28D8\u28D9\u28DA\u28DB"+
	"\u28DC\u28DD\u28DE\u28DF\u28E0\u28E1\u28E2\u28E3"+
	"\u28E4\u28E5\u28E6\u28E7\u28E8\u28E9\u28EA\u28EB"+
	"\u28EC\u28ED\u28EE\u28EF\u28F0\u28F1\u28F2\u28F3"+
	"\u28F4\u28F5\u28F6\u28F7\u28F8\u28F9\u28FA\u28FB"+
	"\u28FC\u28FD\u28FE\u28FF\u2900\u2901\u2902\u2903"+
	"\u2904\u2905\u2906\u2907\u2908\u2909\u290A\u290B"+
	"\u290C\u290D\u290E\u290F\u2910\u2911\u2912\u2913"+
	"\u2914\u2915\u2916\u2917\u2918\u2919\u291A\u291B"+
	"\u291C\u291D\u291E\u291F\u2920\u2921\u2922\u2923"+
	"\u2924\u2925\u2926\u2927\u2928\u2929\u292A\u292B"+
	"\u292C\u292D\u292E\u292F\u2930\u2931\u2932\u2933"+
	"\u2934\u2935\u2936\u2937\u2938\u2939\u293A\u293B"+
	"\u293C\u293D\u293E\u293F\u2940\u2941\u2942\u2943"+
	"\u2944\u2945\u2946\u2947\u2948\u2949\u294A\u294B"+
	"\u294C\u294D\u294E\u294F\u2950\u2951\u2952\u2953"+
	"\u2954\u2955\u2956\u2957\u2958\u2959\u295A\u295B"+
	"\u295C\u295D\u295E\u295F\u2960\u2961\u2962\u2963"+
	"\u2964\u2965\u2966\u2967\u2968\u2969\u296A\u296B"+
	"\u296C\u296D\u296E\u296F\u2970\u2971\u2972\u2973"+
	"\u2974\u2975\u2976\u2977\u2978\u2979\u297A\u297B"+
	"\u297C\u297D\u297E\u297F\u2980\u2981\u2982\u2983"+
	"\u2984\u2985\u2986\u2987\u2988\u2989\u298A\u298B"+
	"\u298C\u298D\u298E\u298F\u2990\u2991\u2992\u2993"+
	"\u2994\u2995\u2996\u2997\u2998\u2999\u299A\u299B"+
	"\u299C\u299D\u299E\u299F\u29A0\u29A1\u29A2\u29A3"+
	"\u29A4\u29A5\u29A6\u29A7\u29A8\u29A9\u29AA\u29AB"+
	"\u29AC\u29AD\u29AE\u29AF\u29B0\u29B1\u29B2\u29B3"+
	"\u29B4\u29B5\u29B6\u29B7\u29B8\u29B9\u29BA\u29BB"+
	"\u29BC\u29BD\u29BE\u29BF\u29C0\u29C1\u29C2\u29C3"+
	"\u29C4\u29C5\u29C6\u29C7\u29C8\u29C9\u29CA\u29CB"+
	"\u29CC\u29CD\u29CE\u29CF\u29D0\u29D1\u29D2\u29D3"+
	"\u29D4\u29D5\u29D6\u29D7\u29D8\u29D9\u29DA\u29DB"+
	"\u29DC\u29DD\u29DE\u29DF\u29E0\u29E1\u29E2\u29E3"+
	"\u29E4\u29E5\u29E6\u29E7\u29E8\u29E9\u29EA\u29EB"+
	"\u29EC\u29ED\u29EE\u29EF\u29F0\u29F1\u29F2\u29F3"+
	"\u29F4\u29F5\u29F6\u29F7\u29F8\u29F9\u29FA\u29FB"+
	"\u29FC\u29FD\u29FE\u29FF\u2A00\u2A01\u2A02\u2A03"+
	"\u2A04\u2A05\u2A06\u2A07\u2A08\u2A09\u2A0A\u2A0B"+
	"\u2A0C\u2A0D\u2A0E\u2A0F\u2A10\u2A11\u2A12\u2A13"+
	"\u2A14\u2A15\u2A16\u2A17\u2A18\u2A19\u2A1A\u2A1B"+
	"\u2A1C\u2A1D\u2A1E\u2A1F\u2A20\u2A21\u2A22\u2A23"+
	"\u2A24\u2A25\u2A26\u2A27\u2A28\u2A29\u2A2A\u2A2B"+
	"\u2A2C\u2A2D\u2A2E\u2A2F\u2A30\u2A31\u2A32\u2A33"+
	"\u2A34\u2A35\u2A36\u2A37\u2A38\u2A39\u2A3A\u2A3B"+
	"\u2A3C\u2A3D\u2A3E\u2A3F\u2A40\u2A41\u2A42\u2A43"+
	"\u2A44\u2A45\u2A46\u2A47\u2A48\u2A49\u2A4A\u2A4B"+
	"\u2A4C\u2A4D\u2A4E\u2A4F\u2A50\u2A51\u2A52\u2A53"+
	"\u2A54\u2A55\u2A56\u2A57\u2A58\u2A59\u2A5A\u2A5B"+
	"\u2A5C\u2A5D\u2A5E\u2A5F\u2A60\u2A61\u2A62\u2A63"+
	"\u2A64\u2A65\u2A66\u2A67\u2A68\u2A69\u2A6A\u2A6B"+
	"\u2A6C\u2A6D\u2A6E\u2A6F\u2A70\u2A71\u2A72\u2A73"+
	"\u2A74\u2A75\u2A76\u2A77\u2A78\u2A79\u2A7A\u2A7B"+
	"\u2A7C\u2A7D\u2A7E\u2A7F\u2A80\u2A81\u2A82\u2A83"+
	"\u2A84\u2A85\u2A86\u2A87\u2A88\u2A89\u2A8A\u2A8B"+
	"\u2A8C\u2A8D\u2A8E\u2A8F\u2A90\u2A91\u2A92\u2A93"+
	"\u2A94\u2A95\u2A96\u2A97\u2A98\u2A99\u2A9A\u2A9B"+
	"\u2A9C\u2A9D\u2A9E\u2A9F\u2AA0\u2AA1\u2AA2\u2AA3"+
	"\u2AA4\u2AA5\u2AA6\u2AA7\u2AA8\u2AA9\u2AAA\u2AAB"+
	"\u2AAC\u2AAD\u2AAE\u2AAF\u2AB0\u2AB1\u2AB2\u2AB3"+
	"\u2AB4\u2AB5\u2AB6\u2AB7\u2AB8\u2AB9\u2ABA\u2ABB"+
	"\u2ABC\u2ABD\u2ABE\u2ABF\u2AC0\u2AC1\u2AC2\u2AC3"+
	"\u2AC4\u2AC5\u2AC6\u2AC7\u2AC8\u2AC9\u2ACA\u2ACB"+
	"\u2ACC\u2ACD\u2ACE\u2ACF\u2AD0\u2AD1\u2AD2\u2AD3"+
	"\u2AD4\u2AD5\u2AD6\u2AD7\u2AD8\u2AD9\u2ADA\u2ADB"+
	"\u2ADC\u2ADD\u2ADE\u2ADF\u2AE0\u2AE1\u2AE2\u2AE3"+
	"\u2AE4\u2AE5\u2AE6\u2AE7\u2AE8\u2AE9\u2AEA\u2AEB"+
	"\u2AEC\u2AED\u2AEE\u2AEF\u2AF0\u2AF1\u2AF2\u2AF3"+
	"\u2AF4\u2AF5\u2AF6\u2AF7\u2AF8\u2AF9\u2AFA\u2AFB"+
	"\u2AFC\u2AFD\u2AFE\u2AFF\u2B00\u2B01\u2B02\u2B03"+
	"\u2B04\u2B05\u2B06\u2B07\u2B08\u2B09\u2B0A\u2B0B"+
	"\u2B0C\u2B0D\u2B0E\u2B0F\u2B10\u2B11\u2B12\u2B13"+
	"\u2B14\u2B15\u2B16\u2B17\u2B18\u2B19\u2B1A\u2B1B"+
	"\u2B1C\u2B1D\u2B1E\u2B1F\u2B20\u2B21\u2B22\u2B23"+
	"\u2B24\u2B25\u2B26\u2B27\u2B28\u2B29\u2B2A\u2B2B"+
	"\u2B2C\u2B2D\u2B2E\u2B2F\u2B30\u2B31\u2B32\u2B33"+
	"\u2B34\u2B35\u2B36\u2B37\u2B38\u2B39\u2B3A\u2B3B"+
	"\u2B3C\u2B3D\u2B3E\u2B3F\u2B40\u2B41\u2B42\u2B43"+
	"\u2B44\u2B45\u2B46\u2B47\u2B48\u2B49\u2B4A\u2B4B"+
	"\u2B4C\u2B4D\u2B4E\u2B4F\u2B50\u2B51\u2B52\u2B53"+
	"\u2B54\u2B55\u2B56\u2B57\u2B58\u2B59\u2B5A\u2B5B"+
	"\u2B5C\u2B5D\u2B5E\u2B5F\u2B60\u2B61\u2B62\u2B63"+
	"\u2B64\u2B65\u2B66\u2B67\u2B68\u2B69\u2B6A\u2B6B"+
	"\u2B6C\u2B6D\u2B6E\u2B6F\u2B70\u2B71\u2B72\u2B73"+
	"\u2B74\u2B75\u2B76\u2B77\u2B78\u2B79\u2B7A\u2B7B"+
	"\u2B7C\u2B7D\u2B7E\u2B7F\u2B80\u2B81\u2B82\u2B83"+
	"\u2B84\u2B85\u2B86\u2B87\u2B88\u2B89\u2B8A\u2B8B"+
	"\u2B8C\u2B8D\u2B8E\u2B8F\u2B90\u2B91\u2B92\u2B93"+
	"\u2B94\u2B95\u2B96\u2B97\u2B98\u2B99\u2B9A\u2B9B"+
	"\u2B9C\u2B9D\u2B9E\u2B9F\u2BA0\u2BA1\u2BA2\u2BA3"+
	"\u2BA4\u2BA5\u2BA6\u2BA7\u2BA8\u2BA9\u2BAA\u2BAB"+
	"\u2BAC\u2BAD\u2BAE\u2BAF\u2BB0\u2BB1\u2BB2\u2BB3"+
	"\u2BB4\u2BB5\u2BB6\u2BB7\u2BB8\u2BB9\u2BBA\u2BBB"+
	"\u2BBC\u2BBD\u2BBE\u2BBF\u2BC0\u2BC1\u2BC2\u2BC3"+
	"\u2BC4\u2BC5\u2BC6\u2BC7\u2BC8\u2BC9\u2BCA\u2BCB"+
	"\u2BCC\u2BCD\u2BCE\u2BCF\u2BD0\u2BD1\u2BD2\u2BD3"+
	"\u2BD4\u2BD5\u2BD6\u2BD7\u2BD8\u2BD9\u2BDA\u2BDB"+
	"\u2BDC\u2BDD\u2BDE\u2BDF\u2BE0\u2BE1\u2BE2\u2BE3"+
	"\u2BE4\u2BE5\u2BE6\u2BE7\u2BE8\u2BE9\u2BEA\u2BEB"+
	"\u2BEC\u2BED\u2BEE\u2BEF\u2BF0\u2BF1\u2BF2\u2BF3"+
	"\u2BF4\u2BF5\u2BF6\u2BF7\u2BF8\u2BF9\u2BFA\u2BFB"+
	"\u2BFC\u2BFD\u2BFE\u2BFF\u2C00\u2C01\u2C02\u2C03"+
	"\u2C04\u2C05\u2C06\u2C07\u2C08\u2C09\u2C0A\u2C0B"+
	"\u2C0C\u2C0D\u2C0E\u2C0F\u2C10\u2C11\u2C12\u2C13"+
	"\u2C14\u2C15\u2C16\u2C17\u2C18\u2C19\u2C1A\u2C1B"+
	"\u2C1C\u2C1D\u2C1E\u2C1F\u2C20\u2C21\u2C22\u2C23"+
	"\u2C24\u2C25\u2C26\u2C27\u2C28\u2C29\u2C2A\u2C2B"+
	"\u2C2C\u2C2D\u2C2E\u2C2F\u2C30\u2C31\u2C32\u2C33"+
	"\u2C34\u2C35\u2C36\u2C37\u2C38\u2C39\u2C3A\u2C3B"+
	"\u2C3C\u2C3D\u2C3E\u2C3F\u2C40\u2C41\u2C42\u2C43"+
	"\u2C44\u2C45\u2C46\u2C47\u2C48\u2C49\u2C4A\u2C4B"+
	"\u2C4C\u2C4D\u2C4E\u2C4F\u2C50\u2C51\u2C52\u2C53"+
	"\u2C54\u2C55\u2C56\u2C57\u2C58\u2C59\u2C5A\u2C5B"+
	"\u2C5C\u2C5D\u2C5E\u2C5F\u2C60\u2C61\u2C62\u2C63"+
	"\u2C64\u2C65\u2C66\u2C67\u2C68\u2C69\u2C6A\u2C6B"+
	"\u2C6C\u2C6D\u2C6E\u2C6F\u2C70\u2C71\u2C72\u2C73"+
	"\u2C74\u2C75\u2C76\u2C77\u2C78\u2C79\u2C7A\u2C7B"+
	"\u2C7C\u2C7D\u2C7E\u2C7F\u2C80\u2C81\u2C82\u2C83"+
	"\u2C84\u2C85\u2C86\u2C87\u2C88\u2C89\u2C8A\u2C8B"+
	"\u2C8C\u2C8D\u2C8E\u2C8F\u2C90\u2C91\u2C92\u2C93"+
	"\u2C94\u2C95\u2C96\u2C97\u2C98\u2C99\u2C9A\u2C9B"+
	"\u2C9C\u2C9D\u2C9E\u2C9F\u2CA0\u2CA1\u2CA2\u2CA3"+
	"\u2CA4\u2CA5\u2CA6\u2CA7\u2CA8\u2CA9\u2CAA\u2CAB"+
	"\u2CAC\u2CAD\u2CAE\u2CAF\u2CB0\u2CB1\u2CB2\u2CB3"+
	"\u2CB4\u2CB5\u2CB6\u2CB7\u2CB8\u2CB9\u2CBA\u2CBB"+
	"\u2CBC\u2CBD\u2CBE\u2CBF\u2CC0\u2CC1\u2CC2\u2CC3"+
	"\u2CC4\u2CC5\u2CC6\u2CC7\u2CC8\u2CC9\u2CCA\u2CCB"+
	"\u2CCC\u2CCD\u2CCE\u2CCF\u2CD0\u2CD1\u2CD2\u2CD3"+
	"\u2CD4\u2CD5\u2CD6\u2CD7\u2CD8\u2CD9\u2CDA\u2CDB"+
	"\u2CDC\u2CDD\u2CDE\u2CDF\u2CE0\u2CE1\u2CE2\u2CE3"+
	"\u2CE4\u2CE5\u2CE6\u2CE7\u2CE8\u2CE9\u2CEA\u2CEB"+
	"\u2CEC\u2CED\u2CEE\u2CEF\u2CF0\u2CF1\u2CF2\u2CF3"+
	"\u2CF4\u2CF5\u2CF6\u2CF7\u2CF8\u2CF9\u2CFA\u2CFB"+
	"\u2CFC\u2CFD\u2CFE\u2CFF\u2D00\u2D01\u2D02\u2D03"+
	"\u2D04\u2D05\u2D06\u2D07\u2D08\u2D09\u2D0A\u2D0B"+
	"\u2D0C\u2D0D\u2D0E\u2D0F\u2D10\u2D11\u2D12\u2D13"+
	"\u2D14\u2D15\u2D16\u2D17\u2D18\u2D19\u2D1A\u2D1B"+
	"\u2D1C\u2D1D\u2D1E\u2D1F\u2D20\u2D21\u2D22\u2D23"+
	"\u2D24\u2D25\u2D26\u2D27\u2D28\u2D29\u2D2A\u2D2B"+
	"\u2D2C\u2D2D\u2D2E\u2D2F\u2D30\u2D31\u2D32\u2D33"+
	"\u2D34\u2D35\u2D36\u2D37\u2D38\u2D39\u2D3A\u2D3B"+
	"\u2D3C\u2D3D\u2D3E\u2D3F\u2D40\u2D41\u2D42\u2D43"+
	"\u2D44\u2D45\u2D46\u2D47\u2D48\u2D49\u2D4A\u2D4B"+
	"\u2D4C\u2D4D\u2D4E\u2D4F\u2D50\u2D51\u2D52\u2D53"+
	"\u2D54\u2D55\u2D56\u2D57\u2D58\u2D59\u2D5A\u2D5B"+
	"\u2D5C\u2D5D\u2D5E\u2D5F\u2D60\u2D61\u2D62\u2D63"+
	"\u2D64\u2D65\u2D66\u2D67\u2D68\u2D69\u2D6A\u2D6B"+
	"\u2D6C\u2D6D\u2D6E\u2D6F\u2D70\u2D71\u2D72\u2D73"+
	"\u2D74\u2D75\u2D76\u2D77\u2D78\u2D79\u2D7A\u2D7B"+
	"\u2D7C\u2D7D\u2D7E\u2D7F\u2D80\u2D81\u2D82\u2D83"+
	"\u2D84\u2D85\u2D86\u2D87\u2D88\u2D89\u2D8A\u2D8B"+
	"\u2D8C\u2D8D\u2D8E\u2D8F\u2D90\u2D91\u2D92\u2D93"+
	"\u2D94\u2D95\u2D96\u2D97\u2D98\u2D99\u2D9A\u2D9B"+
	"\u2D9C\u2D9D\u2D9E\u2D9F\u2DA0\u2DA1\u2DA2\u2DA3"+
	"\u2DA4\u2DA5\u2DA6\u2DA7\u2DA8\u2DA9\u2DAA\u2DAB"+
	"\u2DAC\u2DAD\u2DAE\u2DAF\u2DB0\u2DB1\u2DB2\u2DB3"+
	"\u2DB4\u2DB5\u2DB6\u2DB7\u2DB8\u2DB9\u2DBA\u2DBB"+
	"\u2DBC\u2DBD\u2DBE\u2DBF\u2DC0\u2DC1\u2DC2\u2DC3"+
	"\u2DC4\u2DC5\u2DC6\u2DC7\u2DC8\u2DC9\u2DCA\u2DCB"+
	"\u2DCC\u2DCD\u2DCE\u2DCF\u2DD0\u2DD1\u2DD2\u2DD3"+
	"\u2DD4\u2DD5\u2DD6\u2DD7\u2DD8\u2DD9\u2DDA\u2DDB"+
	"\u2DDC\u2DDD\u2DDE\u2DDF\u2DE0\u2DE1\u2DE2\u2DE3"+
	"\u2DE4\u2DE5\u2DE6\u2DE7\u2DE8\u2DE9\u2DEA\u2DEB"+
	"\u2DEC\u2DED\u2DEE\u2DEF\u2DF0\u2DF1\u2DF2\u2DF3"+
	"\u2DF4\u2DF5\u2DF6\u2DF7\u2DF8\u2DF9\u2DFA\u2DFB"+
	"\u2DFC\u2DFD\u2DFE\u2DFF\u2E00\u2E01\u2E02\u2E03"+
	"\u2E04\u2E05\u2E06\u2E07\u2E08\u2E09\u2E0A\u2E0B"+
	"\u2E0C\u2E0D\u2E0E\u2E0F\u2E10\u2E11\u2E12\u2E13"+
	"\u2E14\u2E15\u2E16\u2E17\u2E18\u2E19\u2E1A\u2E1B"+
	"\u2E1C\u2E1D\u2E1E\u2E1F\u2E20\u2E21\u2E22\u2E23"+
	"\u2E24\u2E25\u2E26\u2E27\u2E28\u2E29\u2E2A\u2E2B"+
	"\u2E2C\u2E2D\u2E2E\u2E2F\u2E30\u2E31\u2E32\u2E33"+
	"\u2E34\u2E35\u2E36\u2E37\u2E38\u2E39\u2E3A\u2E3B"+
	"\u2E3C\u2E3D\u2E3E\u2E3F\u2E40\u2E41\u2E42\u2E43"+
	"\u2E44\u2E45\u2E46\u2E47\u2E48\u2E49\u2E4A\u2E4B"+
	"\u2E4C\u2E4D\u2E4E\u2E4F\u2E50\u2E51\u2E52\u2E53"+
	"\u2E54\u2E55\u2E56\u2E57\u2E58\u2E59\u2E5A\u2E5B"+
	"\u2E5C\u2E5D\u2E5E\u2E5F\u2E60\u2E61\u2E62\u2E63"+
	"\u2E64\u2E65\u2E66\u2E67\u2E68\u2E69\u2E6A\u2E6B"+
	"\u2E6C\u2E6D\u2E6E\u2E6F\u2E70\u2E71\u2E72\u2E73"+
	"\u2E74\u2E75\u2E76\u2E77\u2E78\u2E79\u2E7A\u2E7B"+
	"\u2E7C\u2E7D\u2E7E\u2E7F\u2E80\u2E81\u2E82\u2E83"+
	"\u2E84\u2E85\u2E86\u2E87\u2E88\u2E89\u2E8A\u2E8B"+
	"\u2E8C\u2E8D\u2E8E\u2E8F\u2E90\u2E91\u2E92\u2E93"+
	"\u2E94\u2E95\u2E96\u2E97\u2E98\u2E99\u2E9A\u2E9B"+
	"\u2E9C\u2E9D\u2E9E\u2E9F\u2EA0\u2EA1\u2EA2\u2EA3"+
	"\u2EA4\u2EA5\u2EA6\u2EA7\u2EA8\u2EA9\u2EAA\u2EAB"+
	"\u2EAC\u2EAD\u2EAE\u2EAF\u2EB0\u2EB1\u2EB2\u2EB3"+
	"\u2EB4\u2EB5\u2EB6\u2EB7\u2EB8\u2EB9\u2EBA\u2EBB"+
	"\u2EBC\u2EBD\u2EBE\u2EBF\u2EC0\u2EC1\u2EC2\u2EC3"+
	"\u2EC4\u2EC5\u2EC6\u2EC7\u2EC8\u2EC9\u2ECA\u2ECB"+
	"\u2ECC\u2ECD\u2ECE\u2ECF\u2ED0\u2ED1\u2ED2\u2ED3"+
	"\u2ED4\u2ED5\u2ED6\u2ED7\u2ED8\u2ED9\u2EDA\u2EDB"+
	"\u2EDC\u2EDD\u2EDE\u2EDF\u2EE0\u2EE1\u2EE2\u2EE3"+
	"\u2EE4\u2EE5\u2EE6\u2EE7\u2EE8\u2EE9\u2EEA\u2EEB"+
	"\u2EEC\u2EED\u2EEE\u2EEF\u2EF0\u2EF1\u2EF2\u2EF3"+
	"\u2EF4\u2EF5\u2EF6\u2EF7\u2EF8\u2EF9\u2EFA\u2EFB"+
	"\u2EFC\u2EFD\u2EFE\u2EFF\u2F00\u2F01\u2F02\u2F03"+
	"\u2F04\u2F05\u2F06\u2F07\u2F08\u2F09\u2F0A\u2F0B"+
	"\u2F0C\u2F0D\u2F0E\u2F0F\u2F10\u2F11\u2F12\u2F13"+
	"\u2F14\u2F15\u2F16\u2F17\u2F18\u2F19\u2F1A\u2F1B"+
	"\u2F1C\u2F1D\u2F1E\u2F1F\u2F20\u2F21\u2F22\u2F23"+
	"\u2F24\u2F25\u2F26\u2F27\u2F28\u2F29\u2F2A\u2F2B"+
	"\u2F2C\u2F2D\u2F2E\u2F2F\u2F30\u2F31\u2F32\u2F33"+
	"\u2F34\u2F35\u2F36\u2F37\u2F38\u2F39\u2F3A\u2F3B"+
	"\u2F3C\u2F3D\u2F3E\u2F3F\u2F40\u2F41\u2F42\u2F43"+
	"\u2F44\u2F45\u2F46\u2F47\u2F48\u2F49\u2F4A\u2F4B"+
	"\u2F4C\u2F4D\u2F4E\u2F4F\u2F50\u2F51\u2F52\u2F53"+
	"\u2F54\u2F55\u2F56\u2F57\u2F58\u2F59\u2F5A\u2F5B"+
	"\u2F5C\u2F5D\u2F5E\u2F5F\u2F60\u2F61\u2F62\u2F63"+
	"\u2F64\u2F65\u2F66\u2F67\u2F68\u2F69\u2F6A\u2F6B"+
	"\u2F6C\u2F6D\u2F6E\u2F6F\u2F70\u2F71\u2F72\u2F73"+
	"\u2F74\u2F75\u2F76\u2F77\u2F78\u2F79\u2F7A\u2F7B"+
	"\u2F7C\u2F7D\u2F7E\u2F7F\u2F80\u2F81\u2F82\u2F83"+
	"\u2F84\u2F85\u2F86\u2F87\u2F88\u2F89\u2F8A\u2F8B"+
	"\u2F8C\u2F8D\u2F8E\u2F8F\u2F90\u2F91\u2F92\u2F93"+
	"\u2F94\u2F95\u2F96\u2F97\u2F98\u2F99\u2F9A\u2F9B"+
	"\u2F9C\u2F9D\u2F9E\u2F9F\u2FA0\u2FA1\u2FA2\u2FA3"+
	"\u2FA4\u2FA5\u2FA6\u2FA7\u2FA8\u2FA9\u2FAA\u2FAB"+
	"\u2FAC\u2FAD\u2FAE\u2FAF\u2FB0\u2FB1\u2FB2\u2FB3"+
	"\u2FB4\u2FB5\u2FB6\u2FB7\u2FB8\u2FB9\u2FBA\u2FBB"+
	"\u2FBC\u2FBD\u2FBE\u2FBF\u2FC0\u2FC1\u2FC2\u2FC3"+
	"\u2FC4\u2FC5\u2FC6\u2FC7\u2FC8\u2FC9\u2FCA\u2FCB"+
	"\u2FCC\u2FCD\u2FCE\u2FCF\u2FD0\u2FD1\u2FD2\u2FD3"+
	"\u2FD4\u2FD5\u2FD6\u2FD7\u2FD8\u2FD9\u2FDA\u2FDB"+
	"\u2FDC\u2FDD\u2FDE\u2FDF\u2FE0\u2FE1\u2FE2\u2FE3"+
	"\u2FE4\u2FE5\u2FE6\u2FE7\u2FE8\u2FE9\u2FEA\u2FEB"+
	"\u2FEC\u2FED\u2FEE\u2FEF\u2FF0\u2FF1\u2FF2\u2FF3"+
	"\u2FF4\u2FF5\u2FF6\u2FF7\u2FF8\u2FF9\u2FFA\u2FFB"+
	"\u2FFC\u2FFD\u2FFE\u2FFF\u3000\u3001\u3002\u3003"+
	"\u3004\u3005\u3006\u3007\u3008\u3009\u300A\u300B"+
	"\u300C\u300D\u300E\u300F\u3010\u3011\u3012\u3013"+
	"\u3014\u3015\u3016\u3017\u3018\u3019\u301A\u301B"+
	"\u301C\u301D\u301E\u301F\u3020\u3021\u3022\u3023"+
	"\u3024\u3025\u3026\u3027\u3028\u3029\u302A\u302B"+
	"\u302C\u302D\u302E\u302F\u3030\u3031\u3032\u3033"+
	"\u3034\u3035\u3036\u3037\u3038\u3039\u303A\u303B"+
	"\u303C\u303D\u303E\u303F\u3040\u3041\u3042\u3043"+
	"\u3044\u3045\u3046\u3047\u3048\u3049\u304A\u304B"+
	"\u304C\u304D\u304E\u304F\u3050\u3051\u3052\u3053"+
	"\u3054\u3055\u3056\u3057\u3058\u3059\u305A\u305B"+
	"\u305C\u305D\u305E\u305F\u3060\u3061\u3062\u3063"+
	"\u3064\u3065\u3066\u3067\u3068\u3069\u306A\u306B"+
	"\u306C\u306D\u306E\u306F\u3070\u3071\u3072\u3073"+
	"\u3074\u3075\u3076\u3077\u3078\u3079\u307A\u307B"+
	"\u307C\u307D\u307E\u307F\u3080\u3081\u3082\u3083"+
	"\u3084\u3085\u3086\u3087\u3088\u3089\u308A\u308B"+
	"\u308C\u308D\u308E\u308F\u3090\u3091\u3092\u3093"+
	"\u3094\u3095\u3096\u3097\u3098\u3099\u309A\u309B"+
	"\u309C\u309D\u309E\u309F\u30A0\u30A1\u30A2\u30A3"+
	"\u30A4\u30A5\u30A6\u30A7\u30A8\u30A9\u30AA\u30AB"+
	"\u30AC\u30AD\u30AE\u30AF\u30B0\u30B1\u30B2\u30B3"+
	"\u30B4\u30B5\u30B6\u30B7\u30B8\u30B9\u30BA\u30BB"+
	"\u30BC\u30BD\u30BE\u30BF\u30C0\u30C1\u30C2\u30C3"+
	"\u30C4\u30C5\u30C6\u30C7\u30C8\u30C9\u30CA\u30CB"+
	"\u30CC\u30CD\u30CE\u30CF\u30D0\u30D1\u30D2\u30D3"+
	"\u30D4\u30D5\u30D6\u30D7\u30D8\u30D9\u30DA\u30DB"+
	"\u30DC\u30DD\u30DE\u30DF\u30E0\u30E1\u30E2\u30E3"+
	"\u30E4\u30E5\u30E6\u30E7\u30E8\u30E9\u30EA\u30EB"+
	"\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007"+
	"\u2008\u2009\u200A\u200B\u200C\u200D\u200E\u200F"+
	"\u2010\u2011\u2012\u2013\u2014\u2015\u2016\u2017"+
	"\u2018\u2019\u201A\u201B\u201C\u201D\u201E\u201F"+
	"\u2020\u2021\u2022\u2023\u2024\u2025\u2026\u2027"+
	"\u2028\u2029\u202A\u202B\uFD9C\u202C\u202D\u202E"+
	"\u202F\u2030\u2031\u2032\u2033\u2034\u2035\u2036"+
	"\u2037\u2038\u2039\u203A\u203B\u203C\u203D\u203E"+
	"\u203F\u2040\u2041\u2042\u2043\u2044\u2045\u2046"+
	"\u2047\u2048\u2049\u204A\u204B\u204C\u204D\u204E"+
	"\u204F\u2050\u2051\u2052\u2053\u2054\u2055\u2056"+
	"\u2057\u2058\u2059\u205A\u205B\u205C\u205D\u205E"+
	"\u205F\u2060\u2061\u2062\u2063\u2064\u2065\u2066"+
	"\u2067\u2068\u2069\u206A\u206B\u206C\u206D\u206E"+
	"\u206F\u2070\u2071\u2072\u2073\u2074\u2075\u2076"+
	"\u2077\uFD9D\u2078\u2079\u207A\u207B\u207C\u207D"+
	"\u207E\u207F\u2080\u2081\u2082\u2083\u2084\u2085"+
	"\u2086\u2087\u2088\u2089\u208A\u208B\u208C\u208D"+
	"\u208E\u208F\u2090\u2091\u2092\uFD9E\u2093\u2094"+
	"\u2095\u2096\u2097\u2098\u2099\u209A\u209B\u209C"+
	"\u209D\u209E\u209F\u20A0\u20A1\u20A2\u20A3\u20A4"+
	"\u20A5\u20A6\u20A7\u20A8\u20A9\u20AA\u20AB\u20AC"+
	"\u20AD\u20AE\u20AF\u20B0\u20B1\u20B2\u20B3\u20B4"+
	"\u20B5\u20B6\u20B7\u20B8\u20B9\u20BA\u20BB\u20BC"+
	"\u20BD\u20BE\u20BF\u20C0\u20C1\u20C2\u20C3\u20C4"+
	"\u20C5\u20C6\u20C7\u20C8\u20C9\u20CA\u20CB\u20CC"+
	"\u20CD\u20CE\u20CF\u20D0\u20D1\u20D2\u20D3\u20D4"+
	"\u20D5\u20D6\u20D7\u20D8\u20D9\u20DA\u20DB\u20DC"+
	"\u20DD\u20DE\u20DF\u20E0\u20E1\u20E2\u20E3\uFD9F"+
	"\u20E4\u20E5\u20E6\u20E7\u20E8\u20E9\u20EA\u20EB"+
	"\u20EC\uFDA0\u20ED\u20EE\u20EF\u20F0\u20F1\u20F2"+
	"\u20F3\u20F4\u20F5\u20F6\u20F7\u20F8\u20F9\u20FA"+
	"\u20FB\u20FC\u20FD\u20FE\u20FF\u2100\u2101\u2102"+
	"\u2103\u2104\u2105\u2106\uFE40\uFE41\uFE42\uFE43"+
	"\u2107\uFE44\u2108\uFE45\uFE46\u2109\u210A\u210B"+
	"\uFE47\u210C\u210D\u210E\u210F\u2110\u2111\uFE48"+
	"\uFE49\uFE4A\u2112\uFE4B\uFE4C\u2113\u2114\uFE4D"+
	"\uFE4E\uFE4F\u2115\u2116\u2117\u2118\u2119\u211A"+
	"\u211B\u211C\u211D\u211E\u211F\u2120\u2121\u2122"+
	"\u2123\u2124\u2125\u2126\u2127\u2128\u2129\u212A"+
	"\u212B\u212C\u212D\u212E\u212F\u2130\u2131\u2132"+
	"\u2133\u2134\u2135\u2136\u2137\u2138\u2139\u213A"+
	"\u213B\u213C\u213D\u213E\u213F\u2140\u2141\u2142"+
	"\u2143\u2144\u2145\u2146\u2147\u2148\u2149\u214A"+
	"\u214B\u214C\u214D\u214E\u214F\u2150\u2151\u2152"+
	"\u2153\u2154\u2155\u2156\u2157\u2158\u2159\u215A"+
	"\u215B\u215C\u215D\u215E\u215F\u2160\u2161\u2162"+
	"\u2163\u2164\u2165\u2166\u2167\u2168\u2169\u216A"+
	"\u216B\u216C\u216D\u216E\u216F\u2170\u2171\u2172"+
	"\u2173\u2174\u2175\u2176\u2177\u2178\u2179\u217A"+
	"\u217B\u217C\u217D\u217E\u217F\u2180\u2181\u2182"+
	"\u2183\u2184\u2185\u2186\u2187\u2188\u2189\u218A"+
	"\u218B\u218C\u218D\u218E\u218F\u2190\u2191\u2192"+
	"\u2193\u2194\u2195\u2196\u2197\u2198\u2199\u219A"+
	"\u219B\u219C\u219D\u219E\u219F\u21A0\u21A1\u21A2"+
	"\u21A3\u21A4\u21A5\u21A6\u21A7\u21A8\u21A9\u21AA"+
	"\u21AB\u21AC\u21AD\u21AE\u21AF\u21B0\u21B1\u21B2"+
	"\u21B3\u21B4\u21B5\u21B6\u21B7\u21B8\u21B9\u21BA"+
	"\u21BB\u21BC\u21BD\u21BE\u21BF\u21C0\u21C1\u21C2"+
	"\u21C3\u21C4\u21C5\u21C6\u21C7\u21C8\u21C9\u21CA"+
	"\u21CB\u21CC\u21CD\u21CE\u21CF\u21D0\u21D1\u21D2"+
	"\u21D3\u21D4\u21D5\u21D6\u21D7\u21D8\u21D9\u21DA"+
	"\u21DB\u21DC\u21DD\u21DE\u21DF\u21E0\u21E1\u21E2"+
	"\u21E3\u21E4\u21E5\u21E6\u21E7\u21E8\u21E9\u21EA";

    private final static String innerIndex12= 
	"\u21EB\u21EC\u21ED\u21EE\u21EF\u21F0\u21F1\u21F2"+
	"\u21F3\u21F4\u21F5\u21F6\u21F7\u21F8\u21F9\u21FA"+
	"\u21FB\u21FC\u21FD\u21FE\u21FF\u2200\u2201\u2202"+
	"\u2203\u2204\u2205\u2206\u2207\u2208\u2209\u220A"+
	"\u220B\u220C\u220D\u220E\u220F\u2210\u2211\u2212"+
	"\u2213\u2214\u2215\u2216\u2217\u2218\u2219\u221A"+
	"\u221B\u221C\u221D\u221E\u221F\u2220\u2221\u2222"+
	"\u2223\u2224\u2225\u2226\u2227\u2228\u2229\u222A"+
	"\u222B\u222C\u222D\u222E\u222F\u2230\u2231\u2232"+
	"\u2233\u2234\u2235\u2236\u2237\u2238\u2239\u223A"+
	"\u223B\u223C\u223D\u223E\u223F\u2240\u2241\u2242"+
	"\u2243\u2244\u2245\u2246\u2247\u2248\u2249\u224A"+
	"\u224B\u224C\u224D\u224E\u224F\u2250\u2251\u2252"+
	"\u2253\u2254\u2255\u2256\u2257\u2258\u2259\u225A"+
	"\u225B\u225C\u225D\u225E\u225F\u2260\u2261\u2262"+
	"\u2263\u2264\u2265\u2266\u2267\u2268\u2269\u226A"+
	"\u226B\u226C\u226D\u226E\u226F\u2270\u2271\u2272"+
	"\u2273\u2274\u2275\u2276\u2277\u2278\u2279\u227A"+
	"\u227B\u227C\u227D\u227E\u227F\u2280\u2281\u2282"+
	"\u2283\u2284\u2285\u2286\u2287\u2288\u2289\u228A"+
	"\u228B\u228C\u228D\u228E\u228F\u2290\u2291\u2292"+
	"\u2293\u2294\u2295\u2296\u2297\u2298\u2299\u229A"+
	"\u229B\u229C\u229D\u229E\u229F\u22A0\u22A1\u22A2"+
	"\u22A3\u22A4\u22A5\u22A6\u22A7\u22A8\u22A9\u22AA"+
	"\u22AB\u22AC\u22AD\u22AE\u22AF\u22B0\u22B1\u22B2"+
	"\u22B3\u22B4\u22B5\u22B6\u22B7\u22B8\u22B9\u22BA"+
	"\u22BB\u22BC\u22BD\u22BE\u22BF\u22C0\u22C1\u22C2"+
	"\u22C3\u22C4\u22C5\u22C6\u22C7\u22C8\u22C9\u22CA"+
	"\u22CB\u22CC\u22CD\u22CE\u22CF\u22D0\u22D1\u22D2"+
	"\u22D3\u22D4\u22D5\u22D6\u22D7\u22D8\u22D9\u22DA"+
	"\u22DB\u22DC\u22DD\u22DE\u22DF\u22E0\u22E1\u22E2"+
	"\u22E3\u22E4\u22E5\u22E6\u22E7\u22E8\u22E9\u22EA"+
	"\u22EB\u22EC\u22ED\u22EE\u22EF\u22F0\u22F1\u22F2"+
	"\u22F3\u22F4\u22F5\u22F6\u22F7\u22F8\u22F9\u22FA"+
	"\u22FB\u22FC\u22FD\u22FE\u22FF\u2300\u2301\u2302"+
	"\u2303\u2304\u2305\u2306\u2307\u2308\u2309\u230A"+
	"\u230B\u230C\u230D\u230E\u230F\u2310\u2311\u2312"+
	"\u2313\u2314\u2315\u2316\u2317\u2318\u2319\u231A"+
	"\u231B\u231C\u231D\u231E\u231F\u2320\u2321\u2322"+
	"\u2323\u2324\u2325\u2326\u2327\u2328\u2329\u232A"+
	"\u232B\u232C\u232D\u232E\u232F\u2330\u2331\u2332"+
	"\u2333\u2334\u2335\u2336\u2337\u2338\u2339\u233A"+
	"\u233B\u233C\u233D\u233E\u233F\u2340\u2341\u2342"+
	"\u2343\u2344\u2345\u2346\u2347\u2348\u2349\u234A"+
	"\u234B\u234C\u234D\u234E\u234F\u2350\u2351\u2352"+
	"\u2353\u2354\u2355\u2356\u2357\u2358\u2359\u235A"+
	"\u235B\u235C\u235D\u235E\u235F\u2360\u2361\u2362"+
	"\u2363\u2364\u2365\u2366\u2367\u2368\u2369\u236A"+
	"\u236B\u236C\u236D\u236E\u236F\u2370\u2371\u2372"+
	"\u2373\u2374\u2375\u2376\u2377\u2378\u2379\u237A"+
	"\u237B\u237C\u237D\u237E\u237F\u2380\u2381\u2382"+
	"\u2383\u2384\u2385\u2386\u2387\u2388\u2389\u238A"+
	"\u238B\u238C\u238D\u238E\u238F\u2390\u2391\u2392"+
	"\u2393\u2394\u2395\u2396\u2397\u2398\u2399\u239A"+
	"\u239B\u239C\u239D\u239E\u239F\u23A0\u23A1\u23A2"+
	"\u23A3\u23A4\u23A5\u23A6\u23A7\u23A8\u23A9\u23AA"+
	"\u23AB\u23AC\u23AD\u23AE\u23AF\u23B0\u23B1\u23B2"+
	"\u23B3\u23B4\u23B5\u23B6\u23B7\u23B8\u23B9\u23BA"+
	"\u23BB\u23BC\u23BD\u23BE\u23BF\u23C0\u23C1\u23C2"+
	"\u23C3\u23C4\u23C5\u23C6\u23C7\u23C8\u23C9\u23CA"+
	"\u23CB\u23CC\u23CD\u23CE\u23CF\u23D0\u23D1\u23D2"+
	"\u23D3\u23D4\u23D5\u23D6\u23D7\u23D8\u23D9\u23DA"+
	"\u23DB\u23DC\u23DD\u23DE\u23DF\u23E0\u23E1\u23E2"+
	"\u23E3\u23E4\u23E5\u23E6\u23E7\u23E8\u23E9\u23EA"+
	"\u23EB\u23EC\u23ED\u23EE\u23EF\u23F0\u23F1\u23F2"+
	"\u23F3\u23F4\u23F5\u23F6\u23F7\u23F8\u23F9\u23FA"+
	"\u23FB\u23FC\u23FD\u23FE\u23FF\u2400\u2401\u2402"+
	"\u2403\u2404\u2405\u2406\u2407\u2408\u2409\u240A"+
	"\u240B\u240C\u240D\u240E\u240F\u2410\u2411\u2412"+
	"\u2413\u2414\u2415\u2416\u2417\u2418\u2419\u241A"+
	"\u241B\u241C\u241D\u241E\u241F\u2420\u2421\u2422"+
	"\u2423\u2424\u2425\u2426\u2427\u2428\u2429\u242A"+
	"\u242B\u242C\u242D\u242E\u242F\u2430\u2431\u2432"+
	"\u2433\u2434\u2435\u2436\u2437\u2438\u2439\u243A"+
	"\u243B\u243C\u243D\u243E\u243F\u2440\u2441\u2442"+
	"\u2443\u2444\u2445\u2446\u2447\u2448\u2449\u244A"+
	"\u244B\u244C\u244D\u244E\u244F\u2450\u2451\u2452"+
	"\u2453\u2454\u2455\u2456\u2457\u2458\u2459\u245A"+
	"\u245B\u245C\u245D\u245E\u245F\u2460\u2461\u2462"+
	"\u2463\u2464\u2465\u2466\u2467\u2468\u2469\u246A"+
	"\u246B\u246C\u246D\u246E\u246F\u2470\u2471\u2472"+
	"\u2473\u2474\u2475\u2476\u2477\u2478\u2479\u247A"+
	"\u247B\u247C\u247D\u247E\u247F\u2480\u2481\u2482"+
	"\u2483\u2484\u2485\u2486\u2487\u2488\u2489\u248A"+
	"\u248B\u248C\u248D\u248E\u248F\u2490\u2491\u2492"+
	"\u2493\u2494\u2495\u2496\u2497\u2498\u2499\u249A"+
	"\u249B\u249C\u249D\u249E\u249F\u24A0\u24A1\u24A2"+
	"\u24A3\u24A4\u24A5\u24A6\u24A7\u24A8\u24A9\u24AA"+
	"\u24AB\u24AC\u24AD\u24AE\u24AF\u24B0\u24B1\u24B2"+
	"\u24B3\u24B4\u24B5\u24B6\u24B7\u24B8\u24B9\u24BA"+
	"\u24BB\u24BC\u24BD\u24BE\u24BF\u24C0\u24C1\u24C2"+
	"\u24C3\u24C4\u24C5\u24C6\u24C7\u24C8\u24C9\u24CA"+
	"\u24CB\u24CC\u24CD\u24CE\u24CF\u24D0\u24D1\u24D2"+
	"\u24D3\u24D4\u24D5\u24D6\u24D7\u24D8\u24D9\u24DA"+
	"\u24DB\u24DC\u24DD\u24DE\u24DF\u24E0\u24E1\u24E2"+
	"\u24E3\u24E4\u24E5\u24E6\u24E7\u24E8\u24E9\u24EA"+
	"\u24EB\u24EC\u24ED\u24EE\u24EF\u24F0\u24F1\u24F2"+
	"\u24F3\u24F4\u24F5\u24F6\u24F7\u24F8\u24F9\u24FA"+
	"\u24FB\u24FC\u24FD\u24FE\u24FF\u2500\u2501\u2502"+
	"\u2503\u2504\u2505\u2506\u2507\u2508\u2509\u250A"+
	"\u250B\u250C\u250D\u250E\u250F\u2510\u2511\u2512"+
	"\u2513\u2514\u2515\u2516\u2517\u2518\u2519\u251A"+
	"\uA955\uA6F2\u251B\uA6F4\uA6F5\uA6E0\uA6E1\uA6F0"+
	"\uA6F1\uA6E2\uA6E3\uA6EE\uA6EF\uA6E6\uA6E7\uA6E4"+
	"\uA6E5\uA6E8\uA6E9\uA6EA\uA6EB\u251C\u251D\u251E"+
	"\u251F\uA968\uA969\uA96A\uA96B\uA96C\uA96D\uA96E"+
	"\uA96F\uA970\uA971\u2520\uA972\uA973\uA974\uA975"+
	"\u2521\uA976\uA977\uA978\uA979\uA97A\uA97B\uA97C"+
	"\uA97D\uA97E\uA980\uA981\uA982\uA983\uA984\u2522"+
	"\uA985\uA986\uA987\uA988\u2523\u2524\u2525\u2526"+
	"\u2527\u2528\u2529\u252A\u252B\u252C\u252D\u252E"+
	"\u252F\u2530\u2531\u2532\u2533\u2534\u2535\u2536"+
	"\u2537\u2538\u2539\u253A\u253B\u253C\u253D\u253E"+
	"\u253F\u2540\u2541\u2542\u2543\u2544\u2545\u2546"+
	"\u2547\u2548\u2549\u254A\u254B\u254C\u254D\u254E"+
	"\u254F\u2550\u2551\u2552\u2553\u2554\u2555\u2556"+
	"\u2557\u2558\u2559\u255A\u255B\u255C\u255D\u255E"+
	"\u255F\u2560\u2561\u2562\u2563\u2564\u2565\u2566"+
	"\u2567\u2568\u2569\u256A\u256B\u256C\u256D\u256E"+
	"\u256F\u2570\u2571\u2572\u2573\u2574\u2575\u2576"+
	"\u2577\u2578\u2579\u257A\u257B\u257C\u257D\u257E"+
	"\u257F\u2580\u2581\u2582\u2583\u2584\u2585\u2586"+
	"\u2587\u2588\u2589\u258A\u258B\u258C\u258D\u258E"+
	"\u258F\u2590\u2591\u2592\u2593\u2594\u2595\u2596"+
	"\u2597\u2598\u2599\u259A\u259B\u259C\u259D\u259E"+
	"\u259F\u25A0\u25A1\u25A2\u25A3\u25A4\u25A5\u25A6"+
	"\u25A7\u25A8\u25A9\u25AA\u25AB\u25AC\u25AD\u25AE"+
	"\u25AF\u25B0\u25B1\u25B2\u25B3\u25B4\u25B5\u25B6"+
	"\u25B7\uA3A1\uA3A2\uA3A3\uA1E7\uA3A5\uA3A6\uA3A7"+
	"\uA3A8\uA3A9\uA3AA\uA3AB\uA3AC\uA3AD\uA3AE\uA3AF"+
	"\uA3B0\uA3B1\uA3B2\uA3B3\uA3B4\uA3B5\uA3B6\uA3B7"+
	"\uA3B8\uA3B9\uA3BA\uA3BB\uA3BC\uA3BD\uA3BE\uA3BF"+
	"\uA3C0\uA3C1\uA3C2\uA3C3\uA3C4\uA3C5\uA3C6\uA3C7"+
	"\uA3C8\uA3C9\uA3CA\uA3CB\uA3CC\uA3CD\uA3CE\uA3CF"+
	"\uA3D0\uA3D1\uA3D2\uA3D3\uA3D4\uA3D5\uA3D6\uA3D7"+
	"\uA3D8\uA3D9\uA3DA\uA3DB\uA3DC\uA3DD\uA3DE\uA3DF"+
	"\uA3E0\uA3E1\uA3E2\uA3E3\uA3E4\uA3E5\uA3E6\uA3E7"+
	"\uA3E8\uA3E9\uA3EA\uA3EB\uA3EC\uA3ED\uA3EE\uA3EF"+
	"\uA3F0\uA3F1\uA3F2\uA3F3\uA3F4\uA3F5\uA3F6\uA3F7"+
	"\uA3F8\uA3F9\uA3FA\uA3FB\uA3FC\uA3FD\uA1AB\u25B8"+
	"\u25B9\u25BA\u25BB\u25BC\u25BD\u25BE\u25BF\u25C0"+
	"\u25C1\u25C2\u25C3\u25C4\u25C5\u25C6\u25C7\u25C8"+
	"\u25C9\u25CA\u25CB\u25CC\u25CD\u25CE\u25CF\u25D0"+
	"\u25D1\u25D2\u25D3\u25D4\u25D5\u25D6\u25D7\u25D8"+
	"\u25D9\u25DA\u25DB\u25DC\u25DD\u25DE\u25DF\u25E0"+
	"\u25E1\u25E2\u25E3\u25E4\u25E5\u25E6\u25E7\u25E8"+
	"\u25E9\u25EA\u25EB\u25EC\u25ED\u25EE\u25EF\u25F0"+
	"\u25F1\u25F2\u25F3\u25F4\u25F5\u25F6\u25F7\u25F8"+
	"\u25F9\u25FA\u25FB\u25FC\u25FD\u25FE\u25FF\u2600"+
	"\u2601\u2602\u2603\u2604\u2605\u2606\u2607\u2608"+
	"\u2609\u260A\u260B\u260C\u260D\u260E\u260F\u2610"+
	"\u2611\u2612\u2613\u2614\u2615\u2616\u2617\u2618"+
	"\u2619\u261A\u261B\u261C\u261D\u261E\u261F\u2620"+
	"\u2621\u2622\u2623\u2624\u2625\u2626\u2627\u2628"+
	"\u2629\u262A\u262B\u262C\u262D\u262E\u262F\u2630"+
	"\u2631\u2632\u2633\u2634\u2635\u2636\u2637\u2638"+
	"\uA1E9\uA1EA\uA956\uA3FE\uA957\uA3A4\u2639\u263A"+
	"\u263B\u263C\u263D\u263E\u263F\u2640\u2641\u2642"+
	"\u2643\u2644\u2645\u2646\u2647\u2648\u2649\u264A"+
	"\u264B\u264C\u264D\u264E\u264F\u2650\u2651\u2652";

    private final static short index1[] = {
        0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 
        16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 
        32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 
        48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 
        64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 
        80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 
        96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 
        112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 
        128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 
        144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 
        160, 161, 162, 163, 164, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 
        181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196
    };

    String index2[] = {
        innerIndex0,
        innerIndex1,
        innerIndex2,
        innerIndex3,
        innerIndex4,
        innerIndex5,
        innerIndex6,
        innerIndex7,
        innerIndex8,
        innerIndex9,
        innerIndex10,
        innerIndex11,
        innerIndex12
    };

    private int getGB18030(short[] outerIndex, String[] innerIndex, char ch) {
	int offset = outerIndex[((ch & 0xff00) >> 8 )] << 8;

	return innerIndex[offset >> 12].charAt((offset & 0xfff) + (ch & 0xff));
    }

}
